package com.bingce.controlnetwork;

import static com.bingce.AppChannel.customChannel;

import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;
import android.widget.ImageView;

import androidx.appcompat.app.AppCompatDelegate;
import androidx.preference.PreferenceManager;

import com.bingce.AppChannel;
import com.bingce.RepositoryBaseApp;
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase;
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorConstants;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlnetwork.activity.NotificationActivity;
import com.bingce.utils.DES;
import com.bingce.utils.ThreadPoolUtil;
import com.bumptech.glide.Glide;
import com.mikepenz.materialdrawer.util.AbstractDrawerImageLoader;
import com.mikepenz.materialdrawer.util.DrawerImageLoader;
import com.umeng.commonsdk.UMConfigure;

import net.tatans.tensorflowtts.tts.TtsManager;

import org.polaric.colorful.Colorful;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.Utils;
import cn.leancloud.LCInstallation;
import cn.leancloud.LCLogger;
import cn.leancloud.LeanCloud;
import cn.leancloud.push.PushService;
import cn.liuyanbing.surveyor.AppConfig;
import io.reactivex.functions.Consumer;
import io.reactivex.plugins.RxJavaPlugins;

public class App extends RepositoryBaseApp {

    /**
     * 试用次数
     */
    public int tryCount = 0; //非setting
    /**
     * 上次手动点击跳过首屏广告的时间
     */
    public long lastSplashShowTime = 0;//非setting
    /**
     * 磁偏角，用于校正指南针
     */
    public float magneticDeclination = 0;
    /**
     * 蓝牙是否连接
     */
    public boolean isBluetoothConnected = false;
    /**
     * 蓝牙是否重连
     */
    public boolean isBluetoothReconnect = false;
    /**
     * 退出软件后是否关闭设备
     */
    public boolean isCloseInstrumentAfterExit = false;
    /**
     * 用户状态发生变化，例如登录
     */
    public boolean isUserStatusChanged = false;
    /**
     * 注册码第二版
     */
    public String registerCodeV2 = "";

    public String wifiChatClientIp = "";
    public String lastPhoneLocation = "";

    public String levelSurveyorTime;

    private static App app;

    @Override
    public void onCreate() {
        customChannel = AppChannel.CUSTOM_CONTROL_PUBLIC;//AppChannel.CUSTOM_PUBLIC;
        super.onCreate();
        app = this;

        initThemeDark();

        RxJavaPlugins.setErrorHandler(new Consumer<Throwable>() {
            @Override
            public void accept(Throwable throwable) {
                Log.e("rxThrowable", String.valueOf(throwable));
            }
        });

        UMConfigure.preInit(this, "62c2c24705844627b5d6f943", "public");

        // 在 LeanCloud.initialize() 之前调用
        LeanCloud.setLogLevel(LCLogger.Level.DEBUG);

        LeanCloud.initialize(this
                , DES.decryptIt("//HYY/P0uNjW2bvmcfWVsRqUZcBGI1mjKeLw17mtlc4poF3nDQzo0Q==")//id,BvjcpX3kNB69xP8N0fHxDQav-gzGzoHsz
                , DES.decryptIt("LBk6WfZhiDVIlf/vpesX6s8j4ik+d23atgE0QGuwyw8=")
                , "https://leancloud.bingce.com");

        PushService.setDefaultPushCallback(this, NotificationActivity.class);
        PushService.setAutoWakeUp(true);
        PushService.setDefaultChannelId(this, "normal");
        LCInstallation.getCurrentInstallation().saveInBackground().subscribe();

        DrawerImageLoader.init(new AbstractDrawerImageLoader() {
            @Override
            public void set(ImageView imageView, Uri uri, Drawable placeholder) {
//                Picasso.get().load(uri).placeholder(placeholder).into(imageView);
                Glide.with(App.this).load(uri).into(imageView);
            }

            @Override
            public void cancel(ImageView imageView) {
                Glide.with(App.this).clear(imageView);
//                Picasso.get().cancelRequest(imageView);
            }
        });

        restorePrefs();

        if (customChannel == AppChannel.CUSTOM_CONTROL_PRO) {
            Colorful.config(this).primaryColor(Colorful.ThemeColor.BROWN).apply();
            Colorful.config(this).accentColor(Colorful.ThemeColor.BROWN).apply();
        }


        initControlApp();
    }

    private void initThemeDark() {
        Colorful.defaults()
                .primaryColor(Colorful.ThemeColor.BLUE)
                .accentColor(Colorful.ThemeColor.BLUE)
                .translucent(false)
                .dark(SettingValueUtil.isThemeDark());
        Colorful.init(this);

        if (SettingValueUtil.isThemeDark()) {
            AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_YES);
        } else {
            AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_NO);
        }
    }

    private void restorePrefs() {
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        registerCodeV2 = prefs.getString("registerCodeV2", "");
        lastSplashShowTime = prefs.getLong("lastSplashShowTime", 0);
        wifiChatClientIp = prefs.getString("wifiChatClientIp", "");
        isCloseInstrumentAfterExit = prefs.getBoolean("isCloseInstrumentAfterExit", false);
        magneticDeclination = prefs.getFloat("magneticDeclination", 0);
        lastPhoneLocation = prefs.getString("lastPhoneLocation", "");
        levelSurveyorTime = prefs.getString("levelSurveyorTime", "1");
        if (TextUtils.isEmpty(levelSurveyorTime)) {
            levelSurveyorTime = "1";
        }
    }

    @Override
    public void onTrimMemory(int level) {
        super.onTrimMemory(level);
        if (level == TRIM_MEMORY_MODERATE) {
            //开始自杀，清场掉所有的activity
            ActivityUtils.finishAllActivities();
        }
    }

    @Override
    public String applicationId() {
        return BuildConfig.APPLICATION_ID;
    }

    @Override
    public boolean isThemeDark() {
        return false;
    }

    @Override
    public int angleNotNumber() {
        return AppConfig.instance.angleDotNum();
    }

    public static App getApp() {
        return app;
    }

    @Override
    protected void initTtsManager() {
        TtsManager.getInstance().init(this);
    }

    private void initControlApp() {
        userAgreementAgree = true;
        ISurveyorConstants.isOpenAtr = SettingValueUtil.isOpenAtr();
        Utils.init(this);
        ThreadPoolUtil.execute(ToleranceLoadUtil::init);
        //数据库
        SurveyorDataBase.getInstance();
    }
}
