package com.bingce.controlnetwork.activity;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.CompoundButton;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.lifecycle.ViewModelProvider;
import androidx.preference.PreferenceManager;
import androidx.viewpager.widget.ViewPager;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.AppChannel;
import com.bingce.BaseApp;
import com.bingce.activity.BaseMainActivity;
import com.bingce.controlapphelper.event.DatabaseReadyEvent;
import com.bingce.controlapphelper.event.SwitchPageEvent;
import com.bingce.controlapphelper.util.InterfaceUtil;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlnetwork.App;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.backup.launch.BackupImportHelper;
import com.bingce.controlnetwork.activity.user.RegisterActivity;
import com.bingce.controlnetwork.fragment.MainSurveyFragment;
import com.bingce.controlnetwork.fragment.ProjectListFragment;
import com.bingce.controlnetwork.newui.configfragment.ConfigFragment;
import com.bingce.controlnetwork.newui.mefragment.MeFragment;
import com.bingce.controlnetwork.viewmodel.MainActivityViewModel;
import com.bingce.data.sync.GlobalFlowCenter;
import com.bingce.device.Device;
import com.bingce.device.ui.manager.DeviceConnectManager;
import com.bingce.device.ui.manager.DeviceRemindManager;
import com.bingce.device.ui.manager.DeviceRequestManager;
import com.bingce.event.CountDownEvent;
import com.bingce.event.OpenShortUrlEvent;
import com.bingce.surveyor.agentweb.AgentWebActivity;
import com.bingce.totalstation.TotalStation;
import com.bingce.ui.TabEntity;
import com.bingce.utils.ColorUtil;
import com.bingce.utils.DateUtils;
import com.bingce.utils.SdkUtils;
import com.bingce.utils.SoundPoolUtil;
import com.bingce.utils.VipManager;
import com.daimajia.numberprogressbar.NumberProgressBar;
import com.flyco.tablayout.CommonTabLayout;
import com.flyco.tablayout.listener.CustomTabEntity;
import com.flyco.tablayout.listener.OnTabSelectListener;
import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.material_design_iconic_typeface_library.MaterialDesignIconic;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;
import com.umeng.commonsdk.UMConfigure;

import net.tatans.tensorflowtts.tts.TtsManager;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.polaric.colorful.Colorful;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.ServiceUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.leancloud.LCObject;
import cn.leancloud.LCQuery;
import cn.leancloud.LCUser;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

public class MainActivity extends BaseMainActivity implements Device.IDeviceConnectListener, Device.IDeviceRequestListener, Device.IDeviceRemindListener {
    public MainActivityUtils mainActivityUtils = new MainActivityUtils(this, () -> this);
    private final DeviceConnectManager deviceConnectManager = new DeviceConnectManager(this, this);
    public final DeviceRequestManager deviceRequestManager = new DeviceRequestManager(this, this);
    public final DeviceRemindManager deviceRemindManager = new DeviceRemindManager(this, this);
    public final static String MAIN_IN_ACTION_NAME = "cn.liuyanbing.surveyor.main_activity_local_receiver";
    private Toolbar mToolbar;
    //    private AccountHeader accountHeader = null;
//    private Drawer drawerLeft = null;
    private NumberProgressBar cutDownProgressBar;
    private CheckBox powerOffCheckbox;
    private final List<Fragment> mFragments = new ArrayList<>();
    private String[] mTitles;
    private final int[] mIconUnSelectIds = new int[]{R.drawable.ic_tab_project_unselect, R.drawable.ic_ruler_unselected, R.mipmap.tab_home_unselect, R.mipmap.tab_me_unselect};
    private final int[] mIconSelectIds = new int[]{R.drawable.ic_tab_project_selected, R.drawable.ic_ruler_selected, R.mipmap.tab_home_select, R.mipmap.tab_me_select};
    private final ArrayList<CustomTabEntity> mTabEntities = new ArrayList<>();
    private ViewPager mViewPager;
    private CommonTabLayout mTabLayout;
    private BackupImportHelper backupImportUtil = new BackupImportHelper(this);
    private MaterialDialog langDialog, unbindDeviceDialog;

    private MainActivityViewModel mainActivityViewModel;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mainActivityViewModel = new ViewModelProvider(this).get(MainActivityViewModel.class);
        EventBus.getDefault().register(this);

        if (isThemeDark()) {
            setTheme(R.style.CustomThemeDark);
        }
        Colorful.applyTheme(this);
        setContentView(R.layout.activity_main);
        initWidgetListener(savedInstanceState);//初始化组件和监听器

        ((App) Utils.getApp()).tryCount = 0;
        IntentFilter filter = new IntentFilter();
        filter.addAction(MAIN_IN_ACTION_NAME);

//        if (LCUser.getCurrentUser() != null && LCUser.getCurrentUser().isAuthenticated()) {
//
//        }
        VipManager.getInstance().controlSdkRefresh(this, ((App) Utils.getApp()).registerCodeV2, new VipManager.OnSdkRefreshListener() {
            @Override
            public void onRefreshSuccess() {

            }

            @Override
            public void onRefreshFailed(int errorCode) {

            }
        });
    }

    /**
     * 初始化组件和监听器，屏幕方向发生改变时执行一次
     */
    private void initWidgetListener(Bundle savedInstanceState) {
        mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        setTitle(R.string.app_name);
        //set the back arrow in the toolbar
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(false);
            getSupportActionBar().setDisplayShowTitleEnabled(true);
        }
        cutDownProgressBar = findViewById(R.id.cut_down_progressbar);
        mViewPager = findViewById(R.id.activity_main_viewpager);
        mTabLayout = findViewById(R.id.activity_main_tab_layout);
        if (isThemeDark()) {
            if (mTabLayout != null) {
                mTabLayout.setBackgroundColor(Color.DKGRAY);
                mTabLayout.setTextUnselectColor(Color.LTGRAY);
                mTabLayout.setUnderlineColor(Color.BLACK);
            }
        }
        mTitles = new String[]{getString(R.string.project), getString(R.string.survey), getString(R.string.label_manager), getString(R.string.me)};
        mFragments.add(new ProjectListFragment());
        mFragments.add(new MainSurveyFragment());
        mFragments.add(new ConfigFragment());
//        mFragments.add(new ServiceFragment());
        mFragments.add(new MeFragment());
        for (int i = 0; i < mTitles.length; i++) {
            mTabEntities.add(new TabEntity(mTitles[i], mIconSelectIds[i], mIconUnSelectIds[i]));
        }

        mTabLayout.setTabData(mTabEntities);
        mTabLayout.setOnTabSelectListener(new OnTabSelectListener() {
            @Override
            public void onTabSelect(int position) {
                mViewPager.setCurrentItem(position);
            }

            @Override
            public void onTabReselect(int position) {
                if (position == 0) {
//                    mTabLayout.showMsg(0, mRandom.nextInt(100) + 1);
//                    UnreadMsgUtils.show(mTabLayout_2.getMsgView(0), mRandom.nextInt(100) + 1);
                }
            }
        });

        mViewPager.setAdapter(new MyPagerAdapter(getSupportFragmentManager()));
        mViewPager.setCurrentItem(0);
        mViewPager.setOffscreenPageLimit(4);

        mViewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

            }

            @Override
            public void onPageSelected(int position) {
                setCurrentTab(position);
            }

            @Override
            public void onPageScrollStateChanged(int state) {

            }
        });
    }

    public void setCurrentTab(int position) {
        mTabLayout.setCurrentTab(position);
    }


    private void updateSplashLog(final boolean autoDismiss, final boolean skipDismiss, final boolean clickImageDismiss) {
        if (AppChannel.customChannel != AppChannel.CUSTOM_CONTROL_PUBLIC)
            return;//只统计公版
        final Date today = DateUtils.reverse2Date(DateUtils.getTodayStartTime());
        new LCQuery<>("SplashLog")
                .whereEqualTo("date", today)
                .findInBackground().subscribe(new Observer<List<LCObject>>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(List<LCObject> avObjects) {
                        if (avObjects != null && !avObjects.isEmpty()) {
                            avObjects.get(0).increment("showTimes");
                            if (autoDismiss)
                                avObjects.get(0).increment("autoDismissTimes");
                            if (skipDismiss)
                                avObjects.get(0).increment("skipTimes");
                            if (clickImageDismiss)
                                avObjects.get(0).increment("clickTimes");
                            avObjects.get(0).saveInBackground().subscribe();
                        } else {
                            LCObject splashLog = new LCObject("SplashLog");
                            splashLog.put("date", today);
                            splashLog.saveInBackground().subscribe();
                        }
                    }

                    @Override
                    public void onError(Throwable e) {

                    }

                    @Override
                    public void onComplete() {

                    }
                });
    }

    /**
     * 读取项目文件
     */
    @Override
    public void initialize() {
        //友盟统计
        UMConfigure.init(this, "62c2c24705844627b5d6f943", "public", UMConfigure.DEVICE_TYPE_PHONE, "");
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.Q) {//安卓10以下，请求读取硬件信息
            mainActivityUtils.requestPhonePermission();
        }
        try {//微信插件实例化
            if (Utils.getApp() instanceof BaseApp) {
                IWXAPI wxApi = WXAPIFactory.createWXAPI(this, "wx8b41ad369eba83a9");
                wxApi.registerApp("wx8b41ad369eba83a9");
                ((BaseApp) Utils.getApp()).setWxApi(wxApi);
            }
        } catch (Exception ignored) {
        }
//        CheckControlUpdateUtil.checkUpdate(this, false);
        EventBus.getDefault().post(new DatabaseReadyEvent());

        if (LCUser.getCurrentUser() == null) {
            VipManager.getInstance().controlSdkRefresh(this, ((App) Utils.getApp()).registerCodeV2, new VipManager.OnSdkRefreshListener() {
                @Override
                public void onRefreshSuccess() {
                    Log.d("controlSdkRefresh", "onRefreshSuccess");
                }

                @Override
                public void onRefreshFailed(int errorCode) {
                    Log.d("controlSdkRefresh", "onRefreshFailed--errorCode--" + errorCode);
                }
            });
        }

        GlobalFlowCenter.Companion.getInstance().onMainActivityInitial();
    }

    // this piece of code is show how to press twice back button to exit App
    private long exitTime = 0;

    public void ExitApp() {
        if ((System.currentTimeMillis() - exitTime) > 2000) {
            ToastUtils.showShort(getString(R.string.press_again_exit));
            exitTime = System.currentTimeMillis();
        } else {
            if (((App) Utils.getApp()).isBluetoothConnected) {
                if (TotalStation.getInstance().isSupportPowerOff()) {
                    showPowerOffDialog();
                } else {
                    finish();
                }
            } else {
                finish();
            }
        }
    }

    public void showPowerOffDialog() {
        MaterialDialog dialog = new MaterialDialog.Builder(this)
                .title(R.string.hint)
                .customView(R.layout.dialog_power_off, false)
                .onPositive(new MaterialDialog.SingleButtonCallback() {
                    @Override
                    public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                        if (((App) Utils.getApp()).isCloseInstrumentAfterExit) {
                            TotalStation.getInstance().powerOff();
                        }
                        finish();
                    }
                })
                .positiveText(getString(R.string.confirm))
                .negativeText(getString(R.string.cancel)).build();

        powerOffCheckbox = dialog.getCustomView().findViewById(R.id.dialog_power_off_checkbox);
        powerOffCheckbox.setChecked(((App) Utils.getApp()).isCloseInstrumentAfterExit);
        powerOffCheckbox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                ((App) Utils.getApp()).isCloseInstrumentAfterExit = isChecked;
                SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(Utils.getApp());
                SharedPreferences.Editor editor = prefs.edit();
                editor.putBoolean("isCloseInstrumentAfterExit", ((App) Utils.getApp()).isCloseInstrumentAfterExit);
                editor.apply();
            }
        });
        dialog.show();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
//            if (drawerLeft != null && drawerLeft.isDrawerOpen()) {
//                drawerLeft.closeDrawer();
//                return true;
//            }

            if (mViewPager != null) {
                Fragment viewPagerFragment = getViewPagerFragment(this, mViewPager.getId(), mViewPager.getCurrentItem());
                if (viewPagerFragment instanceof InterfaceUtil.ICustomGroup) {
                    if (((InterfaceUtil.ICustomGroup) viewPagerFragment).isEditModel()) {
                        ((InterfaceUtil.ICustomGroup) viewPagerFragment).cancelEditModel();
                        return true;
                    }
                }
            }
            ExitApp();
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    public static Fragment getViewPagerFragment(AppCompatActivity activity, int viewId, int index) {
        String tag = "android:switcher:" + viewId + ":" + index;
        return activity.getSupportFragmentManager().findFragmentByTag(tag);
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
//        //add the values which need to be saved from the drawer to the bundle
//        if (drawerLeft != null)
//            outState = drawerLeft.saveInstanceState(outState);
//        //add the values which need to be saved from the accountHeader to the bundle
//        if (accountHeader != null)
//            outState = accountHeader.saveInstanceState(outState);
        super.onSaveInstanceState(outState);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_activity_main, menu);
        menu.findItem(R.id.menu_activity_main_bluetooth).setIcon(
                new IconicsDrawable(Utils.getApp()).icon(MaterialDesignIconic.Icon.gmi_bluetooth).color(Color.WHITE).sizeDp(20));
        menu.findItem(R.id.menu_activity_main_bound_device_activate).setTitle(R.string.bound_device_activate);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
            case R.id.menu_activity_main_bluetooth:
                mainActivityUtils.requestBluetoothPermission();
                break;
            case R.id.menu_activity_main_bound_device_activate:
                showActivateType();
                break;
            case R.id.menu_activity_main_settings:
                Intent intent4 = new Intent(Utils.getApp(), SettingsActivity.class);
                startActivityForResult(intent4, 331);
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    /**
     * 选择激活方式
     */
    private void showActivateType() {
//        if (AppChannel.customChannel == AppChannel.CUSTOM_CONTROL_PUBLIC) {
//            if (LCUser.getCurrentUser() == null) {
//                Intent intent = new Intent(Utils.getApp(), LoginActivity.class);
//                intent.putExtra("finishTo", UserInfoActivity.class.getName());
//                startActivity(intent);
//            } else {
//                startActivity(new Intent(MainActivity.this, UserInfoActivity.class));
//            }
//        } else {
//            startActivity(new Intent(MainActivity.this, RegisterActivity.class));
//        }
        startActivity(new Intent(MainActivity.this, RegisterActivity.class));
    }

    @Override
    public void onResume() {
        super.onResume();
        //如果是从其他Activity返回到MainActivity时，不显示广告，如果是从后台切换出来，检查并显示广告；
        //一定放到super.onResume前，否则TopActivity始终是MainActivity，因为App的onActivityResumed在super.onResume内执行

        //按照设置选项，保持屏幕常亮，放在onResume而不放在onCreate里是因为，用户更改设置后返回主页就要开始常亮了
        boolean isWakeLock = PreferenceManager.getDefaultSharedPreferences(this).getBoolean("key_is_wake_lock", false);
        if (isWakeLock) {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        } else {
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }

        SdkUtils.tick();
        EventBus.getDefault().post(new CountDownEvent());
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EventBus.getDefault().unregister(this);

        TtsManager.getInstance().stopTts();
        VipManager.getInstance().destroy();

        SoundPoolUtil.getInstance().onDestroy();
        ((App) Utils.getApp()).isBluetoothReconnect = false;
        TotalStation.getInstance().stopAll();
        Device.getInstance().requestDisconnect();
        for (String serviceName : ServiceUtils.getAllRunningServices()) {
            ServiceUtils.stopService(serviceName);
        }
        ActivityUtils.finishAllActivities();

        GlobalFlowCenter.Companion.getInstance().onMainActivityDestroy();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(SwitchPageEvent event) {
        int type = event.getType();
        try {
            int index = 1;
            if (type == SwitchPageEvent.TYPE_SWITCH_TO_MEASURE_PAGE_AND_DATA) {
                mViewPager.setCurrentItem(index);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(OpenShortUrlEvent event) {
        if (event.getShortUrl() != null) {
//            Intent intent = new Intent();
//            intent.setClass(Utils.getApp(), AgentWebActivity.class);
//            intent.putExtra("short_url", event.getShortUrl());
//            startActivity(intent);
            AgentWebActivity.navigation2(MainActivity.this, event.getShortUrl());
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(CountDownEvent event) {
        if (cutDownProgressBar != null) {
            cutDownProgressBar.setVisibility(View.VISIBLE);
            if (!SdkUtils.isRegCodeValidate() && SdkUtils.isVipValidate()) {
                int maxProgress = (int) (SdkUtils.availableTimeMax() / 60 / 1000L);
                cutDownProgressBar.setMax(maxProgress);
                int progress = Math.max((int) (SdkUtils.availableTime() / 60 / 1000L), 0);//可用时间可能小于0
                cutDownProgressBar.setProgress(progress);
                if (progress < 60) {
                    cutDownProgressBar.setPrefix(getString(R.string.remaining_offline_time) + progress + getString(R.string.minute) + " ");
                } else {
                    cutDownProgressBar.setPrefix(getString(R.string.remaining_offline_time) + progress / 60 + getString(R.string.hour) + progress % 60 + getString(R.string.minute) + " ");
                }
                if (event.isNeedRefresh()) {
                    VipManager.getInstance().controlSdkRefresh(this, ((App) Utils.getApp()).registerCodeV2, null);
                }
                if (progress == 240 || progress == 180 || progress == 120 || progress == 60 || (progress > 0 && progress < 60 && progress % 10 == 0)) {
                    new AlertDialog.Builder(this)
                            .setTitle(R.string.warning)
                            .setMessage(getString(R.string.remaining_offline_time) + progress + getString(R.string.minute) + "," + getString(R.string.the_network_reboot_software_can_reset_the_remaining_time_offline))
                            .setPositiveButton(R.string.i_know, null)
                            .show();
                }
                if (progress > 240) {//离线时间大于4小时，主题色显示
                    cutDownProgressBar.setProgressTextColor(ColorUtil.getColorPrimary(this));
                    cutDownProgressBar.setReachedBarColor(ColorUtil.getColorPrimary(this));
                    cutDownProgressBar.setUnreachedBarColor(ColorUtil.getColorPrimary(this));
                } else if (progress > 120) {//离线时间2-5小时间显示橙色警告
                    cutDownProgressBar.setProgressTextColor(ContextCompat.getColor(this, R.color.md_orange_800));
                    cutDownProgressBar.setReachedBarColor(ContextCompat.getColor(this, R.color.md_orange_800));
                    cutDownProgressBar.setUnreachedBarColor(ContextCompat.getColor(this, R.color.md_orange_800));
                } else {//离线时间小于2小时红色警告
                    cutDownProgressBar.setProgressTextColor(Color.RED);
                    cutDownProgressBar.setReachedBarColor(Color.RED);
                    cutDownProgressBar.setUnreachedBarColor(Color.RED);
                }
                Log.d("cutdownProgressBar", "setProgress" + progress);
            } else {
                cutDownProgressBar.setVisibility(View.GONE);
            }
        }
    }

    @Override
    public void onDeviceConnectLost() {
        mainActivityUtils.onDeviceConnectLost();
    }

    @Override
    public void onDeviceConnectFailed() {
        mainActivityUtils.onDeviceConnectFailed();
    }

    @Override
    public void onDeviceConnectSucceed() {
        mainActivityUtils.onDeviceConnectSucceed();
    }

    @Override
    public void onDeviceConnectNeedOpenBT() {
    }

    @Override
    public void onDeviceRequestConnect() {
        mainActivityUtils.onRequestConnect();
    }

    @Override
    public void onDeviceRequestDisconnect() {

    }

    @Override
    public void onDeviceRequestSetupChat() {

    }

    @Override
    public void onRemindConnectDevice() {
        mainActivityUtils.showRemindConnectDeviceDialog();
    }

    private class MyPagerAdapter extends FragmentPagerAdapter {
        public MyPagerAdapter(@NonNull FragmentManager fm) {
            super(fm);
        }

        public MyPagerAdapter(@NonNull FragmentManager fm, int behavior) {
            super(fm, behavior);
        }

        @NonNull
        @Override
        public Fragment getItem(int position) {
            return mFragments.get(position);
        }

        @Override
        public int getCount() {
            return mFragments.size();
        }
    }

    private void restartApplication() {
        android.os.Process.killProcess(android.os.Process.myPid());
    }

    private boolean isThemeDark() {
        return SettingValueUtil.isThemeDark();
    }

}
