package com.bingce.controlnetwork.activity;

import android.Manifest;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.provider.Settings;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleEventObserver;
import androidx.lifecycle.LifecycleOwner;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.AppChannel;
import com.bingce.BaseApp;
import com.bingce.controlnetwork.R;
import com.bingce.device.Device;
import com.bingce.device.enums.ConnectTypeEnum;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.device.service.comm.ble.BleService;
import com.bingce.device.service.comm.emulator.EmulatorService;
import com.bingce.device.service.comm.localgps.LocationService;
import com.bingce.device.service.comm.localgps.NativePositioningService;
import com.bingce.device.service.comm.otg.OTGCommService;
import com.bingce.device.service.comm.socket.TsSocketService;
import com.bingce.device.service.comm.spp.BTService;
import com.bingce.device.ui.ConnectLevelActivity;
import com.bingce.device.ui.ConnectRtkActivity;
import com.bingce.device.ui.ConnectTSActivity;
import com.bingce.device.ui.utils.DeviceUtil;
import com.bingce.device.ui.widget.BCProgressDialog;
import com.bingce.level.Level;
import com.bingce.rtk.event.RtkSpeechEvent;
import com.bingce.totalstation.TotalStation;
import com.bingce.totalstation.TsConfig;
import com.bingce.utils.AppCompatActivityProviderUtils;
import com.bingce.utils.IProvider;
import com.bingce.utils.RomUtil;
import com.bingce.utils.SoundPoolUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.bingce.utils.ValidateUtil;
import com.hjq.permissions.OnPermissionCallback;
import com.hjq.permissions.OnPermissionPageCallback;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;

import net.tatans.tensorflowtts.tts.TtsManager;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.List;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.ServiceUtils;
import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

public class MainActivityUtils extends AppCompatActivityProviderUtils {
    private BCProgressDialog deviceConnectingProgressDialog;

    public MainActivityUtils(AppCompatActivity activity, IProvider<AppCompatActivity> activityIProvider) {
        super(activityIProvider);
        activity.getLifecycle().addObserver(new LifecycleEventObserver() {
            @Override
            public void onStateChanged(@NonNull LifecycleOwner source, @NonNull Lifecycle.Event event) {
                switch (event) {
                    case ON_CREATE:
                        EventBus.getDefault().register(MainActivityUtils.this);
                        SoundPoolUtil.getInstance().initHintSing(activity, SoundPoolUtil.TEST, R.raw.savedone);
                        break;
                    case ON_DESTROY:
                        EventBus.getDefault().unregister(MainActivityUtils.this);
                        SoundPoolUtil.getInstance().onDestroy();
                        break;
                }
            }
        });
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(RtkSpeechEvent event) {
        if (!StringUtils.isEmpty(event.getText())) TtsManager.getInstance().speak(event.getText());
    }

    /**
     * 连接设备成功后，启动相关服务，因为Ntrip与SN无关，可以在此执行，与SN相关的服务在RtkReceivedSerialNumberEvent事件后执行
     */
    public void onDeviceConnectSucceed() {
        if (deviceConnectingProgressDialog != null && deviceConnectingProgressDialog.isShowing()) {
            deviceConnectingProgressDialog.dismiss();
        }

        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
            TotalStation.getInstance().runnableStart();
            TotalStation.getInstance().init();
        } else if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_LEVEL) {
            Level.getInstance().init();
        }
        //记录用户联机时手机定位的大概位置
        if (ActivityCompat.checkSelfPermission(/*this*/context(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            LocationManager mLocationManager = ((LocationManager) context().getSystemService(Context.LOCATION_SERVICE));
            if (mLocationManager != null) {
                List<String> providers = mLocationManager.getProviders(true);
                if (providers.contains(LocationManager.GPS_PROVIDER)) {
                    Location location = mLocationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
                    if (location != null) {
                        ((BaseApp) Utils.getApp()).lastPhoneLocation = location.getLatitude() + "," + location.getLongitude();
                        Util.putPreference("lastPhoneLocation", ((BaseApp) Utils.getApp()).lastPhoneLocation);
                    }
                }
            }
        }
    }


    public void onDeviceConnectFailed() {
        if (deviceConnectingProgressDialog != null && deviceConnectingProgressDialog.isShowing()) {
            deviceConnectingProgressDialog.dismiss();
            ToastUtils.showShort(R.string.device_connect_failed);
        }
    }

    public void onDeviceConnectLost() {
        ToastUtils.showShort(R.string.device_connect_lost);
    }

    public void onRequestConnect() {
        if (deviceConnectingProgressDialog == null || !deviceConnectingProgressDialog.isShowing()) {
            deviceConnectingProgressDialog = new BCProgressDialog.Builder(ActivityUtils.getTopActivity()).title(R.string.connect_device).build();
            deviceConnectingProgressDialog.show();
        }
    }

    /**
     * 连接蓝牙先请求打开定位权限
     */
    public void requestBluetoothPermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S && XXPermissions.isGranted(ActivityUtils.getTopActivity(), Permission.BLUETOOTH_CONNECT)) {//S及以后不需要位置权限
            checkConnectLast();
        } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {//S及以后不需要位置权限,此时只需请求connect权限
            if (XXPermissions.isGranted(ActivityUtils.getTopActivity(), Permission.BLUETOOTH_CONNECT)) {
                checkConnectLast();
            } else {
                new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                        .title(R.string.location_permission)
                        .content(R.string.bluetooth_need_location_permission)
                        .canceledOnTouchOutside(false)
                        .positiveText(R.string.to_allow)
                        .negativeText(R.string.refuse)
                        .onPositive((dialog, which) -> {
                            XXPermissions.with(ActivityUtils.getTopActivity())
                                    .permission(Permission.BLUETOOTH_CONNECT)
                                    .request(new OnPermissionCallback() {
                                        @Override
                                        public void onGranted(List<String> permissions, boolean all) {
                                            checkConnectLast();
                                        }

                                        @Override
                                        public void onDenied(List<String> permissions, boolean never) {
                                            if (never) {
                                                ToastUtils.showLong(R.string.bluetooth_connect_need_permission);
                                                XXPermissions.startPermissionActivity(ActivityUtils.getTopActivity(), permissions, new OnPermissionPageCallback() {
                                                    @Override
                                                    public void onGranted() {
                                                        requestBluetoothPermission();
                                                    }

                                                    @Override
                                                    public void onDenied() {
                                                        ToastUtils.showLong(R.string.bluetooth_connect_need_permission);
                                                    }
                                                });
                                            } else {
                                                requestBluetoothPermission();
                                            }
                                        }
                                    });
                        })
                        .onNegative((dialog, which) -> {

                        }).show();
            }
        } else {//S以下，需要请求两个权限，位置和Connect权限
            if (XXPermissions.isGranted(ActivityUtils.getTopActivity(), Permission.ACCESS_FINE_LOCATION, Permission.BLUETOOTH_CONNECT)) {
                checkConnectLast();
            } else {
                new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                        .title(R.string.location_permission)
                        .content(R.string.bluetooth_need_location_permission)
                        .canceledOnTouchOutside(false)
                        .positiveText(R.string.to_allow)
                        .negativeText(R.string.refuse)
                        .onPositive((dialog, which) -> {
                            XXPermissions.with(ActivityUtils.getTopActivity())
                                    .permission(Permission.ACCESS_FINE_LOCATION, Permission.BLUETOOTH_CONNECT)
                                    .request(new OnPermissionCallback() {
                                        @Override
                                        public void onGranted(List<String> permissions, boolean all) {
                                            if (all) {
                                                checkConnectLast();
                                            } else {
                                                requestBluetoothPermission();
                                            }
                                        }

                                        @Override
                                        public void onDenied(List<String> permissions, boolean never) {
                                            if (never) {
                                                ToastUtils.showLong(R.string.bluetooth_connection_requires_positioning_permission_please_turn_it_on_manually);
                                                XXPermissions.startPermissionActivity(ActivityUtils.getTopActivity(), permissions, new OnPermissionPageCallback() {
                                                    @Override
                                                    public void onGranted() {
                                                        requestBluetoothPermission();
                                                    }

                                                    @Override
                                                    public void onDenied() {
                                                        ToastUtils.showLong(R.string.bluetooth_connect_need_permission);
                                                    }
                                                });
                                            } else {
                                                requestBluetoothPermission();
                                            }
                                        }
                                    });
                        })
                        .onNegative((dialog, which) -> {

                        }).show();
            }
        }
    }

    /**
     * 检查权限
     */
    public void requestPhonePermission() {
        if (!XXPermissions.isGranted(ActivityUtils.getTopActivity(), Permission.READ_PHONE_STATE)) {
            long lastReadPhoneStateDeniedTime = Util.getLongPreference("lastReadPhoneStateDeniedTime", 0L);
            if (System.currentTimeMillis() - lastReadPhoneStateDeniedTime > 48 * 60 * 60 * 1000L) {//每48小时提示一次
                new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                        .title(R.string.mobile_phone_permissions)
                        .content(R.string.verify_imei_imsi_code_to_prevent_account_theft)
                        .canceledOnTouchOutside(false)
                        .positiveText(R.string.to_allow)
                        .negativeText(R.string.refuse)
                        .onPositive(new MaterialDialog.SingleButtonCallback() {
                            @Override
                            public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                                XXPermissions.with(ActivityUtils.getTopActivity())
                                        .permission(Permission.READ_PHONE_STATE)
                                        .request(new OnPermissionCallback() {
                                            @Override
                                            public void onGranted(List<String> permissions, boolean all) {
                                            }

                                            @Override
                                            public void onDenied(List<String> permissions, boolean never) {
                                                if (never) {
                                                    ToastUtils.showShort(R.string.manually_enable_the_phone_state_permission_to_bind_hardware_devices);
                                                }
                                                Util.putPreference("lastReadPhoneStateDeniedTime", System.currentTimeMillis());
                                            }
                                        });
                            }
                        })
                        .onNegative(new MaterialDialog.SingleButtonCallback() {
                            @Override
                            public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                                Util.putPreference("lastReadPhoneStateDeniedTime", System.currentTimeMillis());
                            }
                        }).show();
            }
        }
    }

    public void showRemindConnectDeviceDialog() {
        new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                .title(R.string.hint)
                .content(R.string.no_instrument_connected)
                .positiveText(R.string.connect)
                .onPositive(new MaterialDialog.SingleButtonCallback() {
                    @Override
                    public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                        checkConnectLast();
                    }
                })
                .negativeText(R.string.cancel)
                .show();
    }


    public void checkConnectLast() {
        if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_SPP) {
            if (!ServiceUtils.isServiceRunning(BTService.class.getName())) {
                ServiceUtils.startService(BTService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_BLE) {
            if (!ServiceUtils.isServiceRunning(BleService.class.getName())) {
                ServiceUtils.startService(BleService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_QXBT) {
            //千寻RTK需要GnssServer连接蓝牙
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_HOTSPOT) {
            if (!ServiceUtils.isServiceRunning(TsSocketService.class.getName())) {
                ServiceUtils.startService(TsSocketService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_OTG) {
            if (!ServiceUtils.isServiceRunning(OTGCommService.class.getName())) {
                ServiceUtils.startService(OTGCommService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_TPCOM) {
            if (!ServiceUtils.isServiceRunning(TsSocketService.class.getName())) {
                ServiceUtils.startService(TsSocketService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_PHONE_LOCATION) {
            if (!ServiceUtils.isServiceRunning(NativePositioningService.class.getName())) {
                ServiceUtils.startService(NativePositioningService.class);
            }
        } else if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_EMULATOR) {
            if (!ServiceUtils.isServiceRunning(EmulatorService.class.getName())) {
                ServiceUtils.startService(EmulatorService.class);
            }
        }
        if (!Device.getInstance().isDeviceConnected()) {
            if (!"".equals(Device.getInstance().deviceBlueName)) {
                String instrumentInfo = ActivityUtils.getTopActivity().getString(R.string.instrument_name) + ":" + Device.getInstance().deviceBlueName + "\n" + ActivityUtils.getTopActivity().getString(R.string.instrument_type_name) + ":" + Device.getInstance().deviceBrand;
                new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                        .title(R.string.quick_connect)
                        .content(instrumentInfo)
                        .positiveText(R.string.connect)
                        .negativeText(R.string.config)
                        .neutralText(R.string.switch_instrument)
                        .onPositive(new MaterialDialog.SingleButtonCallback() {
                            @Override
                            public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                                if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_HOTSPOT) {
                                    startHotspotComm();
                                } else {
                                    Device.getInstance().requestConnect();
                                    ToastUtils.showShort(R.string.connecting_please_wait);
                                }
                            }
                        }).onNegative(new MaterialDialog.SingleButtonCallback() {
                            @Override
                            public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                                Intent intent;
                                if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectTSActivity.class);
                                } else if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_LEVEL) {
                                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectLevelActivity.class);
                                } else {
                                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectRtkActivity.class);
                                }
                                intent.putExtra("customChannel", AppChannel.customChannel);
                                ActivityUtils.getTopActivity().startActivity(intent);
                            }
                        }).onNeutral(new MaterialDialog.SingleButtonCallback() {
                            @Override
                            public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                                chooseInstrumentType();
                            }
                        }).show();
            } else {
                chooseInstrumentType();
            }
        } else {
            if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_PHONE_LOCATION) {
                ToastUtils.showShort(R.string.native_rtk_is_running);
            } else {
                Intent intent;
                if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectTSActivity.class);
                } else if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_LEVEL) {
                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectLevelActivity.class);
                } else {
                    intent = new Intent(ActivityUtils.getTopActivity(), ConnectRtkActivity.class);
                }
                intent.putExtra("customChannel", AppChannel.customChannel);
                ActivityUtils.getTopActivity().startActivity(intent);
            }
        }
    }

    private void chooseInstrumentType() {
        new MaterialDialog.Builder(ActivityUtils.getTopActivity())
                .title(R.string.choose_instrument_type)
                .items(new String[]{ActivityUtils.getTopActivity().getString(R.string.total_station), ActivityUtils.getTopActivity().getString(R.string.device_type_level)})
                .itemsCallback(new MaterialDialog.ListCallback() {
                    @Override
                    public void onSelection(MaterialDialog dialog, View itemView, int which, CharSequence text) {
                        switch (which) {
                            case 0:
                                Intent intent = new Intent(ActivityUtils.getTopActivity(), ConnectTSActivity.class);
                                intent.putExtra("customChannel", AppChannel.customChannel);
                                ActivityUtils.getTopActivity().startActivity(intent);
                                break;
                            case 1:
                                Intent intent2 = new Intent(ActivityUtils.getTopActivity(), ConnectLevelActivity.class);
                                intent2.putExtra("customChannel", AppChannel.customChannel);
                                ActivityUtils.getTopActivity().startActivity(intent2);
                                break;
                        }
                    }
                }).show();
    }


    /**
     * 开启wifi通信
     */
    private void startHotspotComm() {
        if (!ValidateUtil.isIPAddress(TsConfig.getInstance().getIp())) {
            ToastUtils.showShort(R.string.please_input_correct_ip);
            return;
        }
        if (TsConfig.getInstance().getPort() < 1024 || TsConfig.getInstance().getPort() > 65535) {
            ToastUtils.showShort(R.string.please_input_correct_port);
            return;
        }
        if (DeviceUtil.isWifiApOpen()) {
            ServiceUtils.stopService(EmulatorService.class);
            ServiceUtils.stopService(LocationService.class);
            ServiceUtils.stopService(NativePositioningService.class);
            ServiceUtils.stopService(BTService.class);
            ServiceUtils.stopService(BleService.class);
            ServiceUtils.startService(TsSocketService.class);
            ThreadPoolUtil.execute(new Runnable() {
                @Override
                public void run() {
                    Device.getInstance().requestConnect();
                }
            }, 200);
        } else {
            ToastUtils.showShort(R.string.please_open_hotspot_first);
            //打开网络共享与热点设置页面
            if (RomUtil.isOppo() || RomUtil.isLenove() || RomUtil.isGionee()) {
                //OPPO系统系统特殊，会让在Manifest.xml声明Settings$TetherSettingsActivity，现在让用户自己找热点界面，并开启
                ToastUtils.showShort(R.string.find_tether_setting_and_open);
                ActivityUtils.getTopActivity().startActivity(new Intent(Settings.ACTION_SETTINGS));
            } else {
                Intent intent = new Intent();
                intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                ComponentName comp = new ComponentName("com.android.settings", "com.android.settings.Settings$TetherSettingsActivity");
                intent.setComponent(comp);
                ActivityUtils.getTopActivity().startActivity(intent);
            }
        }
    }
}
