package com.bingce.controlnetwork.activity;


import android.annotation.SuppressLint;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.WindowManager;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.preference.ListPreference;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.SwitchPreferenceCompat;

import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorConstants;
import com.bingce.controlnetwork.App;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.controller.KeyEventAction;

import org.polaric.colorful.Colorful;
import org.polaric.colorful.ColorfulActivity;

/**
 * 设置页面
 */
public class SettingsActivity extends ColorfulActivity {
    private Toolbar mToolbar;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        App app = App.getApp();
        if (app.isLandscape()) {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        } else {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
        }
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);
        mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        if (savedInstanceState == null) {
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.activity_settings_content, createSettingFragment())
                    .commit();
        }
    }

    private Fragment createSettingFragment() {
        return new SettingsFragment();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }

    public static class SettingsFragment extends PreferenceFragmentCompat {
        private SwitchPreferenceCompat nightModePrefs;
        private SwitchPreferenceCompat isOpenAtr;
//        private ColorPickerPreference themePrimaryColor, themeAccentColor;

        private ListPreference volume_plus_prefs, volume_minus_prefs;

        @Override
        public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
            setPreferencesFromResource(R.xml.prefs, rootKey);

            nightModePrefs = findPreference("key_is_night_mode");
//            themePrimaryColor = findPreference("key_theme_primary_color");
//            themeAccentColor = findPreference("key_theme_accent_color");


            volume_plus_prefs = findPreference("key_volume_plus_function");
            volume_minus_prefs = findPreference("key_volume_minus_function");

            volume_plus_prefs.setSummary(volume_plus_prefs.getEntry());
            volume_minus_prefs.setSummary(volume_minus_prefs.getEntry());

            isOpenAtr = findPreference("key_is_open_atr");

            nightModePrefs.setOnPreferenceChangeListener((preference, newValue) -> {
                Colorful.config(App.getApp()).dark((Boolean) newValue).apply();
                new AlertDialog.Builder(getActivity())
                        .setMessage(R.string.please_restart_to_apply_new_theme)
                        .setPositiveButton(R.string.confirm, null)
                        .setOnDismissListener(dialogInterface -> {
                            Intent intent = new Intent();
                            intent.putExtra("isThemeChanged", true);
                            getActivity().setResult(RESULT_OK, intent);
                            getActivity().finish();
                        }).show();
                return true;
            });


//            //主色
//            themePrimaryColor.setOnPreferenceChangeListener((preference, newValue) -> {
//                new AlertDialog.Builder(getActivity())
//                        .setMessage(R.string.please_restart_to_apply_new_theme)
//                        .setPositiveButton(R.string.confirm, null)
//                        .setOnDismissListener(new DialogInterface.OnDismissListener() {
//                            @Override
//                            public void onDismiss(DialogInterface dialogInterface) {
//                                Intent intent = new Intent();
//                                intent.putExtra("isThemeChanged", true);
//                                getActivity().setResult(RESULT_OK, intent);
//                                getActivity().finish();
//                            }
//                        }).show();
//                return true;
//            });
//            //配色
//            themeAccentColor.setOnPreferenceChangeListener((preference, newValue) -> {
//                new AlertDialog.Builder(getActivity())
//                        .setMessage(R.string.please_restart_to_apply_new_theme)
//                        .setPositiveButton(R.string.confirm, null)
//                        .setOnDismissListener(new DialogInterface.OnDismissListener() {
//                            @Override
//                            public void onDismiss(DialogInterface dialogInterface) {
//                                Intent intent = new Intent();
//                                intent.putExtra("isThemeChanged", true);
//                                getActivity().setResult(RESULT_OK, intent);
//                                getActivity().finish();
//                            }
//                        }).show();
//                return true;
//            });

            volume_plus_prefs.setOnPreferenceChangeListener((preference, newValue) -> {
                setKeyEvent(newValue, volume_plus_prefs);
                return true;
            });
            volume_minus_prefs.setOnPreferenceChangeListener((preference, newValue) -> {
                setKeyEvent(newValue, volume_minus_prefs);
                return true;
            });

            isOpenAtr.setOnPreferenceChangeListener((preference, newValue) -> {
                ISurveyorConstants.isOpenAtr = (boolean) newValue;
                return true;
            });

//            if (AppChannel.customChannel != AppChannel.CUSTOM_CONTROL_PUBLIC) {
//                //主配色隐藏
//                themePrimaryColor.setVisible(false);
//                themeAccentColor.setVisible(false);
//            }

        }

        private void setKeyEvent(Object newValue, ListPreference volume_minus_prefs) {
            switch (String.valueOf(newValue)) {
                case KeyEventAction.NONE:
                    volume_minus_prefs.setSummary("无");
                    break;
                case KeyEventAction.CAL:
                    volume_minus_prefs.setSummary("计算");
                    break;
                case KeyEventAction.SURVEY:
                    volume_minus_prefs.setSummary("观测");
                    break;
                case KeyEventAction.RECORD:
                    volume_minus_prefs.setSummary("记录");
                    break;
                case KeyEventAction.LAST:
                    volume_minus_prefs.setSummary("上一点");
                    break;
                case KeyEventAction.NEXT:
                    volume_minus_prefs.setSummary("下一点");
                    break;
                case KeyEventAction.SURVEY_NO_SEND:
                    volume_minus_prefs.setSummary("观测后不发送结果");
                    break;
                case KeyEventAction.SURVEY_AND_RECORD:
                    volume_minus_prefs.setSummary("观测并记录");
                    break;
            }
        }

    }


}
