package com.bingce.controlnetwork.activity;

import static com.king.app.updater.util.AppUtils.getPackageInfo;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.Nullable;

import com.bingce.controlnetwork.databinding.ActivitySplashBinding;
import com.bingce.file.FileOperator;
import com.bingce.path.PathManager;
import com.bingce.utils.NetworkUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.zip.ZipUtil;

import org.polaric.colorful.ColorfulActivity;

import java.io.File;
import java.io.FileNotFoundException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import cn.leancloud.LCFile;
import cn.leancloud.LCObject;
import cn.leancloud.LCQuery;

/**
 * 欢迎页
 */
@SuppressLint("CustomSplashScreen")
public class SplashActivity extends ColorfulActivity {

    private boolean isEnterMainActivity = false;

    private static final String CODE_NO_UPLOAD = "0";
    private static final String CODE_UPLOAD_COMPARE_SMALL_WITH_FIRST = "1";
    private static final String CODE_UPLOAD_COMPARE_SMALL_WITHOUT_FIRST = "2";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
//        getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        ActivitySplashBinding binding = ActivitySplashBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        startCopyDbFileToCache();
    }

    private void startCopyDbFileToCache() {
        ThreadPoolUtil.execute(() -> {
            try {
                File copyFolder = PathManager.appDocumentFolder();
                File[] files = copyFolder.listFiles();
                List<File> collectFileList = null;
                if (files != null) {
                    collectFileList = new ArrayList<>(Arrays.asList(files));
                }

                if (collectFileList != null && !collectFileList.isEmpty()) {

                    SimpleDateFormat simple = new SimpleDateFormat("yyyyMMddHHmmss", Locale.getDefault());
                    String fileName = "控制测量电子手薄数据备份_" + "_" + getVersionName() + "_" + simple.format(new Date());
                    File zipTartFile = new File(copyFolder, fileName + ".zip");
                    boolean zip = ZipUtil.zip(zipTartFile, collectFileList);

                    if (zip) {
                        //获取上一次备份的文件
                        File lastFile = getBeforeTargetFile(targetFolder());
                        //备份文件
                        File targetFile = new File(targetFolder().getAbsolutePath(), fileName);
                        boolean isCopySuccess = FileOperator.copyFile(zipTartFile.getAbsolutePath(), targetFile.getAbsolutePath());
                        //无论备份成功或者失败都删除压缩后的文件
                        zipTartFile.delete();
                        if (isCopySuccess) {
                            //delete多余文件
                            deleteRemainCacheFile(lastFile, targetFile);

                            if (NetworkUtil.isNetworkAvailable(SplashActivity.this)) {
                                String code = queryUploadConfig();
                                checkUpload(code, lastFile, targetFile);
                            }
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                goMainActivity();
            }
        });
    }

    /**
     * 文件个数大于3就删除
     *
     * @param lastFile
     * @param targetFile
     */
    private void deleteRemainCacheFile(File lastFile, File targetFile) {
        File file = targetFolder();
        if (file != null) {
            File[] files = file.listFiles();
            if (files != null && files.length > 3) {
                File deleteFile = null;
                for (File fileEach : files) {
                    String eachName = fileEach.getName();
                    if (eachName.equals(getFileName(lastFile)) || eachName.equals(getFileName(targetFile))) {
                        continue;
                    }
                    if (deleteFile != null && deleteFile.lastModified() > fileEach.lastModified()) {
                        deleteFile = fileEach;
                    }
                    if (deleteFile == null) {
                        deleteFile = fileEach;
                    }
                }
                if (deleteFile != null) {
                    deleteFile.delete();
                }
            }
        }
    }

    private String getFileName(File file) {
        if (file != null) {
            return file.getName();
        }
        return null;
    }

    /**
     * 控制测量电子手薄是否自动上传可能错误的数据库
     * 0.不上传
     * 1.比上次小就上传上次备份数据；但是如果是首次备份，也将该备份上传
     * 2.比上次小就上传上次备份数据；如果是首次备份，不会将该备份上传
     */
    private void checkUpload(String code, File lastFile, File targetFile) throws FileNotFoundException {
        if (CODE_NO_UPLOAD.equals(code)) {
            //不上传
        } else if (CODE_UPLOAD_COMPARE_SMALL_WITH_FIRST.equals(code)) {
            if (compareSmall(lastFile, targetFile)) {
                uploadFile(lastFile);
            } else if (lastFile == null) {
                uploadFile(targetFile);
            }
        } else if (CODE_UPLOAD_COMPARE_SMALL_WITHOUT_FIRST.equals(code) && compareSmall(lastFile, targetFile)) {
            uploadFile(lastFile);
        }
    }

    /**
     * 是否比上次小
     *
     * @param lastFile
     * @param targetFile
     * @return
     */
    private boolean compareSmall(File lastFile, File targetFile) {
        return lastFile != null && targetFile.length() < lastFile.length() / 2;
    }

    /**
     * 获取上次备份的文件
     *
     * @param fileFolder
     * @return
     */
    private File getBeforeTargetFile(File fileFolder) {
        File targetFile = null;
        if (fileFolder != null) {
            File[] files = fileFolder.listFiles();
            if (files != null) {
                for (File eachFile : files) {
                    if (targetFile == null || targetFile.lastModified() < eachFile.lastModified()) {
                        targetFile = eachFile;
                    }
                }
            }
        }
        return targetFile;
    }

    /**
     * 上传leanCloud文件
     */
    private void uploadFile(File zipTartFile) throws FileNotFoundException {
        Log.d("hwhw", "数据库备份文件上传LeanCloud");
        LCFile lcFile = LCFile.withAbsoluteLocalPath(zipTartFile.getName(), zipTartFile.getAbsolutePath());
        lcFile.save();
    }

    private void goMainActivity() {
        if (!isEnterMainActivity) {
            isEnterMainActivity = true;
            startActivity(new Intent(this, MainActivity.class));
            finish();
        }
    }


    public File targetFolder() {
        File cacheFolder = getCacheDir();
        File targetFolder = new File(cacheFolder, "dbBackUp");
        if (!targetFolder.exists()) {
            targetFolder.mkdirs();
        }
        return targetFolder;
    }

    private String getVersionName() {
        try {
            return getPackageInfo(this).versionName;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    private String queryUploadConfig() {
        List<LCObject> avObjects = new LCQuery<>("Configuration").whereEqualTo("name", "ControlAppAutoUploadErrorDatabase")
                .limit(1).find();
        if (avObjects != null && !avObjects.isEmpty() && avObjects.get(0).getString("value") != null) {
            return avObjects.get(0).getString("value");
        }
        return CODE_NO_UPLOAD;
    }

}
