package com.bingce.controlnetwork.activity.backup;

import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;

import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.appcompat.widget.AppCompatTextView;
import androidx.appcompat.widget.Toolbar;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase;
import com.bingce.controlapphelper.util.CheckStorageUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityExportLayoutBinding;
import com.bingce.file.DownloadFileWriter;
import com.bingce.file.FileOperator;
import com.bingce.path.PathManager;
import com.bingce.utils.AsynchronousHelper;
import com.bingce.utils.FileUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.zip.ZipUtil;
import com.github.ybq.android.spinkit.SpinKitView;

import org.polaric.colorful.ColorfulActivity;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.ToastUtils;

public abstract class AbstractBackupExportActivity extends ColorfulActivity implements AsynchronousHelper.IDelegate {

    public static final String BACKUP_NAME = "控制测量电子手薄数据";
    public static final String FOLDER_DADA_DATABASES = "dada_databases";
    public static final String FOLDER_DATA_FILE_PROJECT = "data_file_project";
    public static final String FOLDER_ANDROID = "android";
    public static final String FOLDER_SDCARD = "sdcard";
    public static final String BACKUP_TXT = "backup.txt";


    private final AsynchronousHelper asynchronousHelper = new AsynchronousHelper(this);
    private File zipFile = null;

    protected abstract String tipsString();

    private ActivityExportLayoutBinding binding;

    @Override
    final protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityExportLayoutBinding.inflate(getLayoutInflater());
//        getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        binding.shareButton.setOnClickListener(v -> doCollect());
        binding.progressText.setText(tipsString());
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }

    protected abstract String applicationId();

    protected abstract String zipFileName(String dateString, String projectName);

    private void doCollect() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) { //如果是>=Q版本，不需要申请权限
            doExport();
        } else { //如果是低于Q版本，则先申请存储权限
            //检测是否有写权限
            requestStoragePermission();
        }
    }

    public void requestStoragePermission() {
        CheckStorageUtil.check(this, this::doExport);
    }

    private void doExport() {
        if (zipFile == null) {
            startCollect();
        } else {
            FileOperator.shareFile(this, zipFile, /*BuildConfig.APPLICATION_ID*/applicationId());
        }
    }

    private void startCollect() {
//
        binding.progressText.setText("");
        asynchronousHelper.start();
        binding.shareButton.setEnabled(false);
        binding.spinKit.setVisibility(View.VISIBLE);
        binding.tvCompleted.setVisibility(View.GONE);
        ThreadPoolUtil.execute(() -> {

            File packageFile = getFilesDir().getParentFile();

            //1
            File database1 = new File(packageFile, "databases");
            File[] files = database1.listFiles();
            List<File> dataDatabasesFiles = getDbListFiles(files);
            //2
            File file2 = new File(PathManager.appFolder(), "NOPROJECT");
            File[] files2 = file2.listFiles();
            List<File> dataProjectFiles = getDbListFiles(files2);
            //3
            File file3 = PathManager.appDocumentFolder();
            File[] files3 = file3.listFiles();
            List<File> androidFiles = getDbListFiles(files3);
            //4
            File file4 = new File(FileUtil.getSDPath(), "控制测量电子手薄");
            File[] files4 = file4.listFiles();
            List<File> sdFiles = getDbListFiles(files4);

            File fileTargetP = new File(PathManager.appCacheFolder(this), BACKUP_NAME);
            FileUtil.deleteDir(fileTargetP);

            File fileTxt = new File(fileTargetP, "backup.txt");
            FileUtil.writeString(FOLDER_ANDROID, fileTxt);

            copyFile(dataDatabasesFiles, new File(fileTargetP, FOLDER_DADA_DATABASES));
            copyFile(dataProjectFiles, new File(fileTargetP, FOLDER_DATA_FILE_PROJECT));
            copyFile(androidFiles, new File(fileTargetP, FOLDER_ANDROID));
            copyFile(sdFiles, new File(fileTargetP, FOLDER_SDCARD));

            SimpleDateFormat simple = new SimpleDateFormat("yyyyMMddHHmmss", Locale.getDefault());
            String fileName = zipFileName(simple.format(new Date()), "");
            File target = new File(fileTargetP, fileName + ".zip");
            List<File> list = new ArrayList<>();
            list.add(fileTargetP);
            boolean zip = ZipUtil.zip(target, list);
            Log.d("hwhw", "备份是否ok" + zip);
            if (zip) {
                zipFile = target;
            }
            asynchronousHelper.completed();
        });
    }

    @Override
    public void onBackPressed() {
        if (asynchronousHelper.isWorking()) {
            ToastUtils.showShort("数据收集中，请稍候。");
            return;
        }
        super.onBackPressed();
    }

    public static void start(Context context, Class<? extends AbstractBackupExportActivity> clz) {
        start(context, clz, null);
    }

    private static final String KEY_PROJECT_NAME = "key_project_name";

    public static void start(Context context, Class<? extends AbstractBackupExportActivity> clz, String projectName) {
        Intent intent = new Intent(context, clz);
        if (!StringUtils.isEmpty(projectName)) {
            intent.putExtra(KEY_PROJECT_NAME, projectName);
        }
        context.startActivity(intent);
    }

    @Override
    public void onUpdateProgress(String msg) {
        String string = binding.progressText.getText().toString();
        binding.progressText.setText(string + "\n" + msg);
        binding.scrollView.post(() -> binding.scrollView.fullScroll(View.FOCUS_DOWN));
    }

    @Override
    public void onCompleted() {
        if (zipFile == null) {
            return;
        }
        if (isDestroyed() || isFinishing()) {
            return;
        }
        Button shareButton = findViewById(R.id.share_button);
        if (shareButton != null) {
            shareButton.setEnabled(true);
            shareButton.setText(R.string.share);
        }
        SpinKitView spinKitView = findViewById(R.id.spin_kit);
        if (spinKitView != null) {
            spinKitView.setVisibility(View.INVISIBLE);
        }
        AppCompatTextView completeTv = findViewById(R.id.tv_completed);
        if (completeTv != null) {
            completeTv.setText(Tools.getString(R.string.collection_completed));
            completeTv.setVisibility(View.VISIBLE);
        }
        //将文件存储到download中
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            String tipsString;
            if (DownloadFileWriter.writeHigherQ(this, zipFile)) {
                tipsString = "数据备份完毕，存储在根目录「Download\\" + zipFile.getName() + "」压缩包中.";
            } else {
                tipsString = "数据备份完毕，请存放到安全位置以便日后恢复.";
            }
            //弾框提示
            showDialog(this, applicationId(), tipsString, zipFile);
        } else {
            File resultFile = DownloadFileWriter.writeLowerQ(zipFile);
            String tipsString;
            if (resultFile != null) {
                tipsString = "数据备份完毕，存储在「" + resultFile.getAbsolutePath() + "」压缩包中.";
            } else {
                tipsString = "数据备份完毕，请存放到安全位置以便日后恢复.";
            }
            showDialog(this, applicationId(), tipsString, zipFile);
        }
    }

    private static void showDialog(Context context, String applicationId, String title, File zipFile) {
        //弾框提示
        new MaterialDialog.Builder(context).title(Tools.getString(R.string.notice))
                .content(title)
                .positiveText(R.string.send_to)
                .negativeText(R.string.cancel)
                .cancelable(false)
                .onPositive((dialog, which) -> FileOperator.shareFile(context, zipFile, /*BuildConfig.APPLICATION_ID*/applicationId))
                .show();
    }


    /**
     * 筛选出需要的db文件
     *
     * @param files
     * @return
     */
    @WorkerThread
    private List<File> getDbListFiles(File[] files) {
        List<File> fileList = new ArrayList<>();
        if (files != null) {
            for (File file : files) {
                if (file.getName().contains(SurveyorDataBase.DB_NAME)) {
                    fileList.add(file);
                }
            }
        }
        return fileList;
    }

    @WorkerThread
    private void copyFile(List<File> files, File targetFile) {
        if (files != null && !files.isEmpty()) {
            if (!targetFile.exists()) {
                targetFile.mkdirs();
            }

            for (File file : files) {
                FileOperator.copyFile(file.getAbsolutePath(), targetFile.getAbsolutePath() + File.separator + file.getName());
            }
        }
    }

}
