package com.bingce.controlnetwork.activity.backup.launch;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityMigration2PrivateFolderLayoutBinding;
import com.bingce.file.DownloadFilePicker;
import com.bingce.utils.AsynchronousHelper;
import com.bingce.utils.IntentUtil;

import org.polaric.colorful.ColorfulActivity;

public class MigrationFolder2PrivateRoomActivity extends ColorfulActivity implements AsynchronousHelper.IDelegate {
    private final DownloadFilePicker downloadFilePicker = new DownloadFilePicker(this);
    private final AsynchronousHelper migrationHelper = new AsynchronousHelper(this);
    private ActivityMigration2PrivateFolderLayoutBinding binding;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
//        getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        binding = ActivityMigration2PrivateFolderLayoutBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);


        //从备份文件导入
        binding.backupFromZip.setOnClickListener(v -> {
            binding.layout2.setVisibility(View.INVISIBLE);
            ZipMigrationUtils.importFromZip(MigrationFolder2PrivateRoomActivity.this,
                    "控制测量电子手薄数据+日期",
                    migrationHelper,
                    downloadFilePicker,
                    new ZipMigrationUtils.IMigrationListener() {
                        @Override
                        public void onMigrationStart() {
                            showWorkLayout();
                        }

                        @Override
                        public void onMigrationInterrupt() {
                            back2Layout2();
                        }
                    });
        });
        boolean fromMainActivity = IntentUtil.boolExtra(this, KEY_FROM_MAIN_ACTIVITY);
        if (fromMainActivity) {
            binding.enterAppDirect.setVisibility(View.GONE);
        } else {
            binding.enterAppDirect.setOnClickListener(v -> {
                binding.backupFromZip.setEnabled(false);
                binding.enterAppDirect.setEnabled(false);
                enterApp(MigrationFolder2PrivateRoomActivity.this);
            });
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }


    private void showWorkLayout() {
        binding.workLayout.setVisibility(View.VISIBLE);
        binding.back.setEnabled(false);
        binding.back.setOnClickListener(v -> back2Layout2());
        binding.next.setEnabled(false);
        binding.next.setOnClickListener(v -> {
            binding.back.setEnabled(false);
            binding.next.setEnabled(false);
            enterApp(MigrationFolder2PrivateRoomActivity.this);
        });
        binding.tvCompleted.setVisibility(View.INVISIBLE);
        binding.spinKit.setVisibility(View.VISIBLE);
        binding.progressText.setText("");
    }

    private void back2Layout2() {
        binding.workLayout.setVisibility(View.INVISIBLE);
        binding.layout2.setVisibility(View.VISIBLE);
    }

    private static void enterApp(Activity activity) {
        boolean fromMainActivity = IntentUtil.boolExtra(activity, KEY_FROM_MAIN_ACTIVITY);
        if (fromMainActivity) {
            activity.setResult(Activity.RESULT_OK);
            activity.finish();
        }
    }

    @Override
    public void onBackPressed() {
        boolean fromMainActivity = IntentUtil.boolExtra(this, KEY_FROM_MAIN_ACTIVITY);
        if (fromMainActivity) {
            super.onBackPressed();
        }
    }

    @Override
    public void onUpdateProgress(String msg) {
        binding.progressText.setText(binding.progressText.getText() + "\n" + msg);
        binding.scrollView.post(() -> binding.scrollView.fullScroll(View.FOCUS_DOWN));
    }

    @Override
    public void onCompleted() {
        binding.next.setEnabled(true);
        binding.back.setEnabled(true);
        binding.tvCompleted.setVisibility(View.VISIBLE);
        binding.spinKit.setVisibility(View.INVISIBLE);
    }

    private static final String KEY_FROM_MAIN_ACTIVITY = "from_main_activity";

    public static void start(Context context) {
        context.startActivity(intent(context, false));
    }

    public static Intent intent(Context context, boolean fromMainActivity) {
        Intent intent = new Intent(context, MigrationFolder2PrivateRoomActivity.class);
        intent.putExtra(KEY_FROM_MAIN_ACTIVITY, fromMainActivity);
        return intent;
    }

}
