package com.bingce.controlnetwork.activity.backup.launch;

import android.content.Context;
import android.text.TextUtils;

import com.bingce.controlnetwork.activity.backup.AbstractBackupExportActivity;
import com.bingce.file.DownloadFilePicker;
import com.bingce.file.FileOperator;
import com.bingce.path.PathManager;
import com.bingce.utils.AsynchronousHelper;
import com.bingce.utils.FileUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.zip.ZipHelper;

import java.io.File;

import blankj.utilcode.util.FileUtils;
import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.activity.backup.AbstractBackupExportActivity;

public class ZipMigrationUtils {

    public static void importFromZip(Context context, String backupName, AsynchronousHelper asynchronousHelper, DownloadFilePicker downloadFilePicker, IMigrationListener listener) {
        downloadFilePicker.doPick(duplicate -> {
            String errorString = "请选择正确的备份数据，位于根目录Download文件夹内，名称样式为「" + backupName + ".zip」";
            if (duplicate == null || !duplicate.exists()) {
                ToastUtils.showShort(errorString);
                if (listener != null) {
                    listener.onMigrationInterrupt();
                }
                return;
            }
            if (!duplicate.getName().endsWith(".zip")) {
                ToastUtils.showShort(errorString);
                if (listener != null) {
                    listener.onMigrationInterrupt();
                }
                return;
            }
            asynchronousHelper.start();
            if (listener != null) {
                listener.onMigrationStart();
            }
            //切换线路执行数据解压
            ThreadPoolUtil.execute(() -> {
                //解压缩

                File unzipFolder = new File(context.getExternalCacheDir(), duplicate.getName().replace(".zip", ""));
                File[] prjFolders = unzipBackupFile(duplicate, unzipFolder, errorString);
                if (prjFolders == null) {
                    return;
                }

                for (File file : prjFolders) {
                    File[] folder = file.listFiles();
                    String backUpType = null;
                    for (File fileBackup : folder) {
                        if (!fileBackup.isDirectory()) {
                            if (fileBackup.getName().equals(AbstractBackupExportActivity.BACKUP_TXT)) {
                                FileUtil.ReadFileResult readFileResult = FileUtil.readStringFrom(fileBackup);
                                if (readFileResult.success) {
                                    backUpType = readFileResult.string;
                                }
                                break;
                            }
                        }
                    }

                    for (File fileBackup : folder) {
                        if (fileBackup.isDirectory()) {
                            String name = fileBackup.getName();
                            if (TextUtils.equals(name, backUpType)) {
                                File[] dbFileList = fileBackup.listFiles();
                                for (File dbFile : dbFileList) {
                                    FileOperator.copy(dbFile, PathManager.appDocumentFolder());
                                }
                            }
                        }
                    }
                }


                //完成数据导入
                asynchronousHelper.completed();
            });
        });
    }

    private static File[] unzipBackupFile(File duplicate, File unzipFolder, String errorString) {
        if (!unzipFolder.exists()) {
            if (!unzipFolder.mkdirs()) {
                ToastUtils.showShort("解压备份文件失败");
                return null;
            }
        } else {
            //先删除，再创建
            try {
                FileUtils.delete(unzipFolder);
            } catch (Exception e) {
                e.printStackTrace();
                ToastUtils.showShort("解压备份文件失败");
                return null;
            }
            if (!unzipFolder.mkdirs()) {
                ToastUtils.showShort("解压备份文件失败");
                return null;
            }
        }
        if (!ZipHelper.unZipFile(unzipFolder.getAbsolutePath(), duplicate.getAbsolutePath())) {
            ToastUtils.showShort(errorString);
            return null;
        }
        //解析文件内容
        return unzipFolder.listFiles();
    }


    public interface IMigrationListener {
        void onMigrationStart();

        void onMigrationInterrupt();
    }
}
