package com.bingce.controlnetwork.activity.fragmentcontainer;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.MenuItem;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;

import org.polaric.colorful.ColorfulActivity;

import java.util.List;

import com.bingce.controlnetwork.R;

public class FragmentContainerActivity extends ColorfulActivity {
    private static final String KEY_TAG = "_TAG_";
    private static final String KEY_TITLE_STRING_ID = "__title_res";
    private static final String KEY_TITLE_STRING = "__title__";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //设置view
        setContentView(R.layout.activity_fragment_container_layout);
        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //设置fragment
        if (savedInstanceState == null) {
            Intent intent = getIntent();
            if (intent == null) {
                finish();
                return;
            }
            int title = intent.getIntExtra(KEY_TITLE_STRING_ID, -1);
            if (title > 0) {
                setTitle(title);
            } else {
                String string = intent.getStringExtra(KEY_TITLE_STRING);
                setTitle(string);
            }
            String tag = intent.getStringExtra(KEY_TAG);
            Bundle bundle = intent.getExtras();
            Fragment fragment = FragmentContainer.getFragment(tag, bundle);
            if (fragment == null) {
                finish();
                return;
            }
            getSupportFragmentManager().beginTransaction()
                    .replace(R.id.container, fragment)
                    .commitAllowingStateLoss();
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            if (!isCancelBack()) {
                finish();
            }
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        if (isCancelBack()) {
            return;
        }
        super.onBackPressed();
    }

    private boolean isCancelBack() {
        List<Fragment> fragmentList = getSupportFragmentManager()
                .getFragments();
        boolean cancelBack = false;
        for (Fragment fragment : fragmentList) {
            if (fragment instanceof IOnBackFilter) {
                if (((IOnBackFilter) fragment).cancelBack()) {
                    cancelBack = true;
                    break;
                }
            }
        }
        return cancelBack;
    }

    public static void start(Context context, String tag, int titleRes, Bundle bundle) {
        Intent intent = new Intent(context, FragmentContainerActivity.class);
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        intent.putExtra(KEY_TAG, tag);
        intent.putExtra(KEY_TITLE_STRING_ID, titleRes);
        context.startActivity(intent);
    }

    public static void start(Context context, String tag, String title, Bundle bundle) {
        Intent intent = new Intent(context, FragmentContainerActivity.class);
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        intent.putExtra(KEY_TAG, tag);
        intent.putExtra(KEY_TITLE_STRING, title);
        context.startActivity(intent);
    }

    public static Intent startIntent(Context context,
                                     String tag,
                                     int titleRes,
                                     Bundle bundle) {
        Intent intent = new Intent(context, FragmentContainerActivity.class);
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        intent.putExtra(KEY_TAG, tag);
        intent.putExtra(KEY_TITLE_STRING_ID, titleRes);
        return intent;
    }

    public static Intent startIntent(Context context,
                                     String tag,
                                     String title,
                                     Bundle bundle) {
        Intent intent = new Intent(context, FragmentContainerActivity.class);
        if (bundle != null) {
            intent.putExtras(bundle);
        }
        intent.putExtra(KEY_TAG, tag);
        intent.putExtra(KEY_TITLE_STRING, title);
        return intent;
    }
}
