package com.bingce.controlnetwork.activity.user;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.view.MenuItem;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;

import org.polaric.colorful.ColorfulActivity;

import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.Utils;
import cn.leancloud.LCUser;

import com.bingce.controlnetwork.App;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityEditUserNicknamePasswordBinding;

public class EditNicknamePasswordActivity extends ColorfulActivity {

    private ActivityEditUserNicknamePasswordBinding binding;
    private boolean isShowPassword01, isShowPassword02;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityEditUserNicknamePasswordBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        //隐藏显示按钮01（输入密码）
        binding.llPassword01.setOnClickListener(v -> {
            if (isShowPassword01) {
                // 明文显示密码
                binding.etPassword01.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_NORMAL);
                binding.ivPassword01.setImageDrawable(getDrawable(R.drawable.icon_password_open_eyes));
            } else {
                // 隐藏密码
                binding.etPassword01.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
                binding.ivPassword01.setImageDrawable(getDrawable(R.drawable.icon_password_close_eyes));
            }
            isShowPassword01 = !isShowPassword01;
        });

        //隐藏显示按钮02（确认密码）
        binding.llPassword02.setOnClickListener(v -> {
            if (isShowPassword02) {
                // 明文显示密码
                binding.etPassword02.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_NORMAL);
                binding.ivPassword02.setImageDrawable(getDrawable(R.drawable.icon_password_open_eyes));
            } else {
                // 隐藏密码
                binding.etPassword02.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
                binding.ivPassword02.setImageDrawable(getDrawable(R.drawable.icon_password_close_eyes));
            }
            isShowPassword02 = !isShowPassword02;
        });

        binding.etPassword01.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                checkPw();
            }
        });
        binding.etPassword02.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                checkPw();
            }
        });

        //完成
        binding.tvComplete.setOnClickListener(v -> {
            String nickName = binding.etNickName.getText().toString();
            String password = binding.etPassword01.getText().toString();
            if (LCUser.getCurrentUser() != null) {
                LCUser.getCurrentUser().setPassword(password);
                LCUser.getCurrentUser().put("nickname", nickName);
                LCUser.getCurrentUser().saveInBackground().subscribe();
                ((App) Utils.getApp()).isUserStatusChanged = true;
            }
            finish();
        });
    }

    private void checkPw() {
        String nickName = binding.etNickName.getText().toString();
        String pw1 = binding.etPassword01.getText().toString();
        String pw2 = binding.etPassword02.getText().toString();
        boolean isEmptyNickName = StringUtils.isEmpty(nickName);
        boolean isEmptyPw01 = StringUtils.isEmpty(pw1);
        boolean isEmptyPw02 = StringUtils.isEmpty(pw2);
        if (!isEmptyNickName && !isEmptyPw01 && !isEmptyPw02 && pw1.equals(pw2)) {
            binding.tvComplete.setEnabled(true);
            binding.tvComplete.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
        } else {
            binding.tvComplete.setEnabled(false);
            binding.tvComplete.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
        }
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, EditNicknamePasswordActivity.class));
    }
}
