package com.bingce.controlnetwork.activity.user;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;

import com.bingce.utils.IntentUtil;
import com.bingce.utils.ValidateUtil;

import org.polaric.colorful.ColorfulActivity;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityInputEmailBinding;

public class InputEmailActivity extends ColorfulActivity {
    public static final String KEY_EMAIL = "email";
    public static final String KEY_TODO = "todo";
    private ActivityInputEmailBinding binding;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityInputEmailBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null) getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        getSupportActionBar().setTitle(R.string.email_verify);
        if (getEmail() != null) {
            binding.etEmail.setText(getEmail());
            binding.etEmail.setEnabled(false);
            if (checkEmail()) {
                binding.tvNextPage.setEnabled(true);
                binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
            } else {
                binding.tvNextPage.setEnabled(false);
                binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
            }
        }

        binding.etEmail.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                if (checkEmail()) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                }
            }
        });

        ActivityResultLauncher<Intent> verifyCodeLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
            Intent data = result.getData();
            if (data != null) {
                setResult(RESULT_OK, data);
                finish();
            }
        });
        //下一步
        binding.tvNextPage.setOnClickListener(v -> {
            String email = binding.etEmail.getText().toString();
            VerifyEmailCodeActivity.start(this, email, getToDo(), verifyCodeLauncher);
        });
    }

    private boolean checkEmail() {
        return ValidateUtil.isEmail(binding.etEmail.getText().toString());
    }

    public static void start(Context context, String email, int todo, ActivityResultLauncher<Intent> launcher) {
        Intent intent = new Intent(context, InputEmailActivity.class);
        intent.putExtra(KEY_EMAIL, email);
        intent.putExtra(KEY_TODO, todo);
        launcher.launch(intent);
    }


    private String getEmail() {
        return IntentUtil.stringExtra(getIntent(), KEY_EMAIL);
    }


    private int getToDo() {
        return IntentUtil.intExtra(getIntent(), KEY_TODO);
    }
}
