package com.bingce.controlnetwork.activity.user;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;

import com.bingce.utils.IntentUtil;

import org.polaric.colorful.ColorfulActivity;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityInputPhoneNumberBinding;

public class InputPhoneNumberActivity extends ColorfulActivity {
    public static final String KEY_PHONE = "phone_number";
    public static final String KEY_EMAIL = "email";
    public static final String KEY_UNION_ID = "unionid";
    public static final String KEY_TODO = "todo";
    private ActivityInputPhoneNumberBinding binding;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityInputPhoneNumberBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null) getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        getSupportActionBar().setTitle(R.string.mobile_phone_verify);

        binding.ccp.registerCarrierNumberEditText(binding.etPhoneNumber);
        binding.ccp.setPhoneNumberValidityChangeListener(isValidNumber -> {
            if (isValidNumber) {
                binding.tvNextPage.setEnabled(true);
                binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
            } else {
                binding.tvNextPage.setEnabled(false);
                binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
            }
        });
        if (getPhoneNumber() != null) {
            binding.etPhoneNumber.setText(getPhoneNumber().replaceFirst("\\+86", ""));
            binding.etPhoneNumber.setEnabled(false);
            binding.ccp.setCcpClickable(false);
        }

        ActivityResultLauncher<Intent> verifyCodeLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
            Intent data = result.getData();
            if (data != null && data.hasExtra("isMail") && data.hasExtra("phoneNumberOrEmail")) {
                setResult(RESULT_OK, data);
                finish();
            }
        });
        //下一步
        binding.tvNextPage.setOnClickListener(v -> {
            String phoneNumber = binding.ccp.getFullNumberWithPlus();
            VerifySMSCodeActivity.start(this, phoneNumber, getEmail(), getUnionId(), getToDo(), verifyCodeLauncher);
        });
    }

    public static void start(Context context, String phoneNumber, String email, String unionid, int todo, ActivityResultLauncher<Intent> launcher) {
        Intent intent = new Intent(context, InputPhoneNumberActivity.class);
        intent.putExtra(KEY_PHONE, phoneNumber);
        intent.putExtra(KEY_EMAIL, email);
        intent.putExtra(KEY_UNION_ID, unionid);
        intent.putExtra(KEY_TODO, todo);
        launcher.launch(intent);
    }

    private String getPhoneNumber() {
        return IntentUtil.stringExtra(getIntent(), KEY_PHONE);
    }


    private String getEmail() {
        return IntentUtil.stringExtra(getIntent(), KEY_EMAIL);
    }

    private String getUnionId() {
        return IntentUtil.stringExtra(getIntent(), KEY_UNION_ID);
    }

    private int getToDo() {
        return IntentUtil.intExtra(getIntent(), KEY_TODO);
    }
}
