package com.bingce.controlnetwork.activity.user;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.Nullable;

import com.bingce.AppChannel;
import com.bingce.BaseApp;
import com.bingce.activity.UserCloudFunction;
import com.bingce.controlnetwork.App;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityLoginBinding;
import com.bingce.controlnetwork.wxapi.MD5;
import com.bingce.data.sync.utils.LeanCloudUserLoginStateUtils;
import com.bingce.event.WxRespCodeEvent;
import com.bingce.ui.TabEntity;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.ValidateUtil;
import com.bingce.utils.VipManager;
import com.flyco.tablayout.listener.CustomTabEntity;
import com.flyco.tablayout.listener.OnTabSelectListener;
import com.hbb20.CountryCodePicker;
import com.tencent.mm.opensdk.modelmsg.SendAuth;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.polaric.colorful.ColorfulActivity;

import java.util.ArrayList;
import java.util.Random;

import alibaba.fastjson.JSON;
import alibaba.fastjson.JSONObject;
import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.leancloud.LCException;
import cn.leancloud.LCUser;
import io.reactivex.Observer;
import io.reactivex.disposables.Disposable;

public class LoginActivity extends ColorfulActivity {

    private ActivityLoginBinding binding;
    private final String[] mTitles = new String[]{Utils.getApp().getString(R.string.phone_number), Utils.getApp().getString(R.string.e_mail)};
    private final ArrayList<CustomTabEntity> tabEntityList = new ArrayList<>();
    private boolean isLoginPassword = true, isLoginByEmail = false;

    private ProgressDialog progressDialog;
    private String wxRespCode;
    private String finishTo;
    private ActivityResultLauncher<Intent> signUpOrLoginLauncher;


    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityLoginBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setSupportActionBar(binding.getRoot().findViewById(R.id.toolbar));
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        EventBus.getDefault().register(this);

        finishTo = getIntent().getStringExtra("finishTo");

        //绑定Tab数据
        for (String mTitle : mTitles) {
            tabEntityList.add(new TabEntity(mTitle));
        }
        binding.tabLoginMode.setTabData(tabEntityList);

        //切换手机号、邮箱
        binding.tabLoginMode.setOnTabSelectListener(new OnTabSelectListener() {
            @Override
            public void onTabSelect(int position) {
                isLoginByEmail = position != 0;
                onSelectTabPhoneEmail();
            }

            @Override
            public void onTabReselect(int position) {

            }
        });

        //切换验证码、密码
        binding.tvLoginPasswordCode.setOnClickListener(v -> {
            onClickBtnPassword();
        });

        binding.ccp.registerCarrierNumberEditText(binding.etPhoneNumber);
        binding.ccp.setPhoneNumberValidityChangeListener(new CountryCodePicker.PhoneNumberValidityChangeListener() {
            @Override
            public void onValidityChanged(boolean isValidNumber) {
                if (isValidNumber) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                }
            }
        });
        //下一步
        binding.tvNextPage.setOnClickListener(v -> {
            if (checkValid()) {
                if (isLoginByEmail) {
                    String email = binding.etEmail.getText().toString().trim();
                    if (!isLoginPassword) {//验证码登录
                        VerifyEmailCodeActivity.start(this, email, VerifyEmailCodeActivity.TO_SIGNUP_OR_LOGIN_BY_EMAIL, signUpOrLoginLauncher);
                    } else {
                        String password = binding.etPassword.getText().toString();
                        loginByEmailPassword(email, password);
                    }
                } else {
                    String phoneNumber = binding.ccp.getFullNumberWithPlus();
                    if (!isLoginPassword) {//验证码登录
                        VerifySMSCodeActivity.start(this, phoneNumber, null, null, VerifySMSCodeActivity.TO_SIGNUP_OR_LOGIN_BY_PHONE_NUMBER, signUpOrLoginLauncher);
                    } else {
                        String password = binding.etPassword.getText().toString();
                        if (phoneNumber.startsWith("+86")) {
                            phoneNumber = phoneNumber.replaceFirst("\\+86", "");
                        }
                        loginByPhonePassword(phoneNumber, password);
                    }
                }
            }
        });

        binding.ivWechatLogin.setOnClickListener(v -> {
            wxLogin();
        });
        signUpOrLoginLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
            if (result.getResultCode() == Activity.RESULT_OK) {
                Intent data = result.getData();
                if (data != null) {
                    if (data.hasExtra("sessionToken") && data.hasExtra("newUser")) {
                        String sessionToken = data.getStringExtra("sessionToken");
                        boolean newUser = data.getBooleanExtra("newUser", false);
                        progressDialog = new ProgressDialog(LoginActivity.this);
                        progressDialog.setMessage(getString(R.string.logging));
                        progressDialog.setCanceledOnTouchOutside(false);
                        progressDialog.show();
                        LCUser.becomeWithSessionTokenInBackground(sessionToken, true).subscribe(new Observer<LCUser>() {
                            @Override
                            public void onSubscribe(Disposable d) {

                            }

                            @Override
                            public void onNext(LCUser lcUser) {
                                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                loginSuccess();
                                if (newUser) {
                                    EditNicknamePasswordActivity.start(LoginActivity.this);
                                }
                            }

                            @Override
                            public void onError(Throwable e) {
                                ToastUtils.showShort(getString(R.string.error_code) + new LCException(e).getCode());
                            }

                            @Override
                            public void onComplete() {

                            }
                        });
                    }
                }
            }
        });

        binding.etPassword.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                checkValid();
            }
        });
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, LoginActivity.class));
    }

    public void onSelectTabPhoneEmail() {
        binding.etPassword.setText("");
        if (!isLoginByEmail) {//手机号登录
            binding.llPhoneNumber.setVisibility(View.VISIBLE);
            binding.llEmail.setVisibility(View.GONE);
            binding.etPhoneNumber.setText("");
            if (isLoginPassword) {
                binding.signUpHint.setText(R.string.if_you_do_not_have_an_account_you_can_use_a_verification_code_to_sign_up_and_log_in);
            } else {
                binding.signUpHint.setText(R.string.unsigned_up_mobile_phone_number_will_be_automatically_signed_up_and_logged_in);
            }
        } else {//邮箱登录
            binding.llPhoneNumber.setVisibility(View.GONE);
            binding.llEmail.setVisibility(View.VISIBLE);
            binding.etEmail.setText("");
            if (isLoginPassword) {
                binding.signUpHint.setText(R.string.if_you_do_not_have_an_account_you_can_use_a_verification_code_to_sign_up_and_log_in);
            } else {
                binding.signUpHint.setText(R.string.unsigned_up_email_will_be_automatically_signed_up_and_logged_in);
            }
        }
        checkValid();
    }

    public void onClickBtnPassword() {
        isLoginPassword = !isLoginPassword;
        if (!isLoginPassword) {//现在改为验证码登录
            binding.llPassword.setVisibility(View.GONE);
            binding.tvNextPage.setText(R.string.next);
            binding.tvLoginPasswordCode.setText(R.string.log_in_with_password);
            if (!isLoginByEmail) {//手机号登录
                binding.signUpHint.setText(R.string.unsigned_up_mobile_phone_number_will_be_automatically_signed_up_and_logged_in);
            } else {
                binding.signUpHint.setText(R.string.unsigned_up_email_will_be_automatically_signed_up_and_logged_in);
            }
        } else {//现在改为密码登录
            binding.llPassword.setVisibility(View.VISIBLE);
            binding.tvNextPage.setText(R.string.log_in);
            binding.tvLoginPasswordCode.setText(R.string.log_in_using_verify_code);
            binding.signUpHint.setText(R.string.if_you_do_not_have_an_account_you_can_use_a_verification_code_to_sign_up_and_log_in);
        }
        checkValid();
    }

    public boolean checkValid() {
        if (!isLoginPassword) {//现在改为验证码登录
            if (isLoginByEmail) {
                if (ValidateUtil.isEmail(binding.etEmail.getText().toString().trim())) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                    return false;
                }
            } else {
                if (binding.ccp.isValidFullNumber()) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                    return false;
                }
            }
        } else {//现在改为密码登录
            if (isLoginByEmail) {
                if (ValidateUtil.isEmail(binding.etEmail.getText().toString().trim()) && !StringUtils.isEmpty(binding.etPassword.getText().toString().trim())) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                    return false;
                }
            } else {
                if (binding.ccp.isValidFullNumber() && !StringUtils.isEmpty(binding.etPassword.getText().toString().trim())) {
                    binding.tvNextPage.setEnabled(true);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_theme_green_full));
                } else {
                    binding.tvNextPage.setEnabled(false);
                    binding.tvNextPage.setBackground(getDrawable(R.drawable.rectangle_radius_8_gray_full));
                    return false;
                }
            }
        }
        return true;
    }

    public void wxLogin() {
        wxRespCode = null;
        if (Utils.getApp() instanceof BaseApp) {
            if (((BaseApp) Utils.getApp()).getWxApi() != null) {
                if (!((BaseApp) Utils.getApp()).getWxApi().isWXAppInstalled()) {
                    ToastUtils.showShort(R.string.you_do_not_have_the_wechat_app_installed);
                } else {
                    final SendAuth.Req req = new SendAuth.Req();
                    //应用授权作用域，如获取用户个人信息则填写 snsapi_userinfo
                    req.scope = "snsapi_userinfo";
                    //用于保持请求和回调的状态，授权请求后原样带回给第三方。
                    //该参数可用于防止 csrf 攻击（跨站请求伪造攻击），
                    //建议第三方带上该参数，可设置为简单的随机数加 session 进行校验
                    req.state = getNonceStr();
                    if (((BaseApp) Utils.getApp()).getWxApi() != null)
                        ((BaseApp) Utils.getApp()).getWxApi().sendReq(req);
                }
            }
        }
    }

    //生成随机号，防重发
    private String getNonceStr() {
        Random random = new Random();
        return MD5.getMessageDigest((System.currentTimeMillis() + String.valueOf(random.nextInt(10000))).getBytes());
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(WxRespCodeEvent event) {
        wxRespCode = event.getCode();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (wxRespCode != null) {
            loginByWxCode();
            wxRespCode = null;
        }
    }

    private void loginByWxCode() {
        if (progressDialog != null)
            progressDialog.dismiss();
        progressDialog = new ProgressDialog(LoginActivity.this);
        progressDialog.setMessage(getString(R.string.logging));
        progressDialog.setCanceledOnTouchOutside(false);
        progressDialog.show();
        UserCloudFunction.loginByWxCode(wxRespCode, AppChannel.customChannel, new UserCloudFunction.OnGetSessionTokenListener() {
            @Override
            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                LCUser.becomeWithSessionTokenInBackground(sessionToken, true).subscribe(new Observer<LCUser>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(LCUser lcUser) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        if (newUser) {
                            EditNicknamePasswordActivity.start(LoginActivity.this);
                        }
                        ((App) Utils.getApp()).isUserStatusChanged = true;
                        setResult(RESULT_OK);
                        finish();
                    }

                    @Override
                    public void onError(Throwable e) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        ToastUtils.showShort(getString(R.string.error_code) + new LCException(e).getCode());
                    }

                    @Override
                    public void onComplete() {

                    }
                });
            }

            @Override
            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                if (code == 321 || code == 322) {//321此微信未绑定用户，请使用其他方式登录,322缺少手机号或手机号未验证
                    JSONObject jsonObject = JSON.parseObject(resp);
                    if (jsonObject.containsKey("data")) {
                        JSONObject data = jsonObject.getJSONObject("data");
                        if (data.containsKey("unionid")) {
                            String unionid = data.getString("unionid");
                            InputPhoneNumberActivity.start(LoginActivity.this, null, null, unionid, VerifySMSCodeActivity.TO_SIGNUP_OR_LOGIN_BY_WEIXIN_BIND_PHONE, signUpOrLoginLauncher);
                        }
                    }
                } else {
                    ToastUtils.showLong(msg);
                }
            }

            @Override
            public void onSessionTokenGetException(String msg) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                ToastUtils.showLong(msg);
            }
        });
    }


    private void loginSuccess() {
        if (progressDialog != null)
            progressDialog.dismiss();
        progressDialog = new ProgressDialog(LoginActivity.this);
        progressDialog.setMessage(getString(R.string.logging));
        progressDialog.setCanceledOnTouchOutside(false);
        progressDialog.show();
        VipManager.getInstance().controlSdkRefresh(LoginActivity.this, ((App) Utils.getApp()).registerCodeV2, new VipManager.OnSdkRefreshListener() {
            @Override
            public void onRefreshSuccess() {
                if (!StringUtils.isEmpty(finishTo)) {
                    try {
                        Intent intent = new Intent(LoginActivity.this, Class.forName(finishTo));
                        intent.putExtras(getIntent());
                        startActivity(intent);
                    } catch (ClassNotFoundException | NullPointerException ex) {
                        ex.printStackTrace();
                    }
                }

                ((App) Utils.getApp()).isUserStatusChanged = true;
                LeanCloudUserLoginStateUtils.onLogIn();
                setResult(RESULT_OK);
                finish();
            }

            @Override
            public void onRefreshFailed(int errorCode) {
                ToastUtils.showShort(getString(R.string.error_code) + ":" + errorCode);
            }
        });
    }

    private void loginByEmailPassword(String email, String password) {
        if (progressDialog != null)
            progressDialog.dismiss();
        progressDialog = new ProgressDialog(LoginActivity.this);
        progressDialog.setMessage(getString(R.string.logging));
        progressDialog.setCanceledOnTouchOutside(false);
        progressDialog.show();
        UserCloudFunction.loginByEmailPassword(email, password, AppChannel.customChannel, new UserCloudFunction.OnGetSessionTokenListener() {
            @Override
            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                LCUser.becomeWithSessionTokenInBackground(sessionToken, true).subscribe(new Observer<LCUser>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(LCUser lcUser) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        loginSuccess();
                        if (newUser) {
                            EditNicknamePasswordActivity.start(LoginActivity.this);
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        ToastUtils.showShort(getString(R.string.error_code) + new LCException(e).getCode());
                    }

                    @Override
                    public void onComplete() {

                    }
                });
            }

            @Override
            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                if (code == 303) {//缺少手机号或未验证
                    InputPhoneNumberActivity.start(LoginActivity.this, null, email, null, VerifySMSCodeActivity.TO_SIGNUP_OR_LOGIN_BY_EMAIL_BIND_PHONE, signUpOrLoginLauncher);
                } else if (code == 304) {//邮箱未验证
                    InputEmailActivity.start(LoginActivity.this, email, VerifyEmailCodeActivity.TO_SIGNUP_OR_LOGIN_BY_EMAIL, signUpOrLoginLauncher);
                } else {
                    if (msg != null)
                        ToastUtils.showLong(msg);
                }
            }

            @Override
            public void onSessionTokenGetException(String msg) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                ToastUtils.showLong(msg);
            }
        });
    }

    private void loginByPhonePassword(String phoneNumber, String password) {
        if (progressDialog != null)
            progressDialog.dismiss();
        progressDialog = new ProgressDialog(LoginActivity.this);
        progressDialog.setMessage(getString(R.string.logging));
        progressDialog.setCanceledOnTouchOutside(false);
        progressDialog.show();
        UserCloudFunction.loginByPhonePassword(phoneNumber, password, AppChannel.customChannel, new UserCloudFunction.OnGetSessionTokenListener() {
            @Override
            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                LCUser.becomeWithSessionTokenInBackground(sessionToken, true).subscribe(new Observer<LCUser>() {
                    @Override
                    public void onSubscribe(Disposable d) {

                    }

                    @Override
                    public void onNext(LCUser lcUser) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        loginSuccess();
                        if (newUser) {
                            EditNicknamePasswordActivity.start(LoginActivity.this);
                        }
                    }

                    @Override
                    public void onError(Throwable e) {
                        if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        ToastUtils.showShort(getString(R.string.error_code) + new LCException(e).getCode());
                    }

                    @Override
                    public void onComplete() {

                    }
                });
            }

            @Override
            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                if (code == 303) {//手机号未验证
                    InputPhoneNumberActivity.start(LoginActivity.this, phoneNumber, null, null, VerifySMSCodeActivity.TO_SIGNUP_OR_LOGIN_BY_PHONE_NUMBER, signUpOrLoginLauncher);
                } else {
                    if (msg != null)
                        ToastUtils.showLong(msg);
                }
            }

            @Override
            public void onSessionTokenGetException(String msg) {
                if (!ActivityUtils.isLiving(LoginActivity.this)) return;
                if (progressDialog != null)
                    progressDialog.dismiss();
                ToastUtils.showLong(msg);
            }
        });
    }
}
