package com.bingce.controlnetwork.activity.user;

import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;
import android.widget.RadioGroup;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.controlnetwork.App;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityOpeningRenewalBinding;
import com.bingce.data.sync.utils.LeanCloudConfigLoader;
import com.bingce.event.PayOverEvent;
import com.bingce.pay.OrderType;
import com.bingce.pay.WxPayV3;
import com.bingce.surveyor.agentweb.AgentWebActivity;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.DateUtils;
import com.bingce.utils.StringUtil;
import com.bingce.utils.Util;
import com.bingce.utils.VipManager;
import com.bumptech.glide.Glide;
import com.huawei.hms.hmsscankit.ScanUtil;
import com.huawei.hms.hmsscankit.WriterException;
import com.huawei.hms.ml.scan.HmsBuildBitmapOption;
import com.huawei.hms.ml.scan.HmsScan;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.polaric.colorful.ColorfulActivity;

import java.util.Date;

import blankj.utilcode.util.AppUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.leancloud.LCUser;

public class OpeningRenewalVipActivity extends ColorfulActivity {
    private int total_fee = -1;
    private ImageView qrImageView;
    private int queryCount = 0;
    private int oneMonthPrice = -1, threeMonthsPrice = -1, oneYearPrice = -1;
    private MaterialDialog payingDialog;
    private ActivityOpeningRenewalBinding binding;
    private String orderType = OrderType.ORDER_TYPE_SURVEYOR_365;
    private boolean hasPayResult = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityOpeningRenewalBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        setSupportActionBar(binding.getRoot().findViewById(R.id.toolbar));
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        EventBus.getDefault().register(this);

        binding.activityOpeningRenewalTvVersion.setText("V" + AppUtils.getAppVersionName());

        if (VipManager.getInstance().expireDate != null) {
            binding.activityOpeningRenewalTvExpirationDate.setText(getString(R.string.current_expiration_date) + ":" + DateUtils.toFull(VipManager.getInstance().expireDate));
        } else {
            binding.activityOpeningRenewalTvExpirationDate.setText(getString(R.string.current_expiration_date) + ":无");
        }

        binding.activityOpeningRenewalRadiogroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                if (oneMonthPrice <= 0 || threeMonthsPrice <= 0 || oneYearPrice <= 0) {
                    ToastUtils.showShort("请先联网获取套餐价格");
                    return;
                }
                if (checkedId == binding.activityOpeningRenewalRadiobuttonOneMonth.getId()) {
                    total_fee = oneMonthPrice;
                    orderType = OrderType.ORDER_TYPE_SURVEYOR_30;
                } else if (checkedId == binding.activityOpeningRenewalRadiobuttonThreeMonths.getId()) {
                    total_fee = threeMonthsPrice;
                    orderType = OrderType.ORDER_TYPE_SURVEYOR_90;
                } else if (checkedId == binding.activityOpeningRenewalRadiobuttonOneYear.getId()) {
                    total_fee = oneYearPrice;
                    orderType = OrderType.ORDER_TYPE_SURVEYOR_365;
                }
                if (total_fee <= 0) {
                    binding.activityOpeningRenewalTvTotalFee.setText("联网获取价格");
                } else {
                    binding.activityOpeningRenewalTvTotalFee.setText(Util.formatDouble2StringDotAuto(total_fee / 100.0) + "元");
                }
            }
        });

        if (VipManager.getInstance().checkReg()) {
            new AlertDialog.Builder(this)
                    .setTitle(R.string.hint)
                    .setMessage("此手机已经绑定激活，请确定是否需要购买会员，绑定激活和购买会员可以二选其一")
                    .setPositiveButton(R.string.i_know, null)
                    .show();
        }

        LeanCloudConfigLoader.findInBackground("opening_renewal_fee", instance -> {
//        new LCQuery<>("Configuration")
//                .whereEqualTo("name", "opening_renewal_fee")
//                .limit(1)
//                .findInBackground().subscribe(new Observer<List<LCObject>>() {
//                    @Override
//                    public void onSubscribe(Disposable d) {
//
//                    }
//
//                    @Override
//                    public void onNext(List<LCObject> avObjects) {
            if (!ActivityUtils.isLiving(OpeningRenewalVipActivity.this)) return;
            if (/*avObjects != null && !avObjects.isEmpty() && avObjects.get(0).getString("value") != null*/instance != null && StringUtil.isNotEmpty(instance.value)) {
                String value = /*avObjects.get(0).getString("value")*/instance.value;
                String value2 = /*avObjects.get(0).getString("value2")*/instance.value2;
                String[] valueArr = value.split(",");
                String[] value2Arr = value2.split(",");
                if (valueArr.length == 3 && value2Arr.length == 3) {
                    try {
                        oneMonthPrice = Integer.parseInt(valueArr[0]);
                        threeMonthsPrice = Integer.parseInt(valueArr[1]);
                        oneYearPrice = Integer.parseInt(valueArr[2]);
                        binding.activityOpeningRenewalRadiobuttonOneMonth.setText(value2Arr[0] + "  " + oneMonthPrice / 100.0 + "元");
                        binding.activityOpeningRenewalRadiobuttonThreeMonths.setText(value2Arr[1] + "  " + threeMonthsPrice / 100.0 + "元");
                        binding.activityOpeningRenewalRadiobuttonOneYear.setText(value2Arr[2] + "  " + oneYearPrice / 100.0 + "元");
                        if (binding.activityOpeningRenewalRadiobuttonOneMonth.isChecked()) {
                            total_fee = oneMonthPrice;
                        } else if (binding.activityOpeningRenewalRadiobuttonThreeMonths.isChecked()) {
                            total_fee = threeMonthsPrice;
                        } else if (binding.activityOpeningRenewalRadiobuttonOneYear.isChecked()) {
                            total_fee = oneYearPrice;
                        }
                        if (total_fee <= 0) {
                            binding.activityOpeningRenewalTvTotalFee.setText("联网获取价格");
                        } else {
                            binding.activityOpeningRenewalTvTotalFee.setText(Util.formatDouble2StringDotAuto(total_fee / 100.0) + "元");
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }

            } else {
                binding.activityOpeningRenewalTvTotalFee.setText("联网获取价格");
            }
//                    }
//
//                    @Override
//                    public void onError(Throwable e) {
//
//                    }
//
//                    @Override
//                    public void onComplete() {
//
//                    }
        });
        LeanCloudConfigLoader.findInBackground("opening_renewal_activity", instance -> {
//        new LCQuery<>("Configuration")
//                .whereEqualTo("name", "opening_renewal_activity")
//                .limit(1)
//                .findInBackground().subscribe(new Observer<List<LCObject>>() {
//                    @Override
//                    public void onSubscribe(Disposable d) {
//
//                    }
//
//                    @Override
//                    public void onNext(List<LCObject> avObjects) {
            if (!ActivityUtils.isLiving(OpeningRenewalVipActivity.this)) return;
            if (/*avObjects != null && !avObjects.isEmpty() && avObjects.get(0).getString("value") != null*/instance != null && StringUtil.isNotEmpty(instance.value)) {
                String activity = /*avObjects.get(0).getString("value")*/instance.value;
                binding.activityOpeningRenewalTvActivity.setText(activity);
            }
//                    }
//
//                    @Override
//                    public void onError(Throwable e) {
//
//                    }
//
//                    @Override
//                    public void onComplete() {
//
//                    }
        });

        String adsQueryUrl = "android_opening_renewal_vip_img_ads";
        LeanCloudConfigLoader.findInBackground(adsQueryUrl, instance -> {
//        new LCQuery<>("Configuration")
//                .whereEqualTo("name", adsQueryUrl)
//                .limit(1)
//                .findInBackground()
//                .subscribe(new Observer<List<LCObject>>() {
//                    @Override
//                    public void onSubscribe(Disposable d) {
//
//                    }
//
//                    @Override
//                    public void onNext(List<LCObject> avObjects) {
            if (!ActivityUtils.isLiving(OpeningRenewalVipActivity.this)) return;
            if (/*avObjects != null && !avObjects.isEmpty()*/instance != null) {
                String adsUrl = /*avObjects.get(0).getString("value")*/instance.value;
                if (adsUrl != null) {
                    binding.activityOpeningRenewalImgAds.setVisibility(View.VISIBLE);
                    Glide.with(OpeningRenewalVipActivity.this).load(adsUrl).into(binding.activityOpeningRenewalImgAds);

                    String actionUrl = /*avObjects.get(0).getString("value2")*/instance.value2;
                    if (actionUrl != null) {
                        binding.activityOpeningRenewalImgAds.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
//                                            Intent intent = new Intent();
//                                            intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                                            intent.setData(Uri.parse(actionUrl));
//                                            startActivity(intent);
                                AgentWebActivity.navigation2(
                                        OpeningRenewalVipActivity.this,
                                        Uri.parse(actionUrl));
                            }
                        });
                    }
                }
            }
//                    }
//
//                    @Override
//                    public void onError(Throwable e) {
//
//                    }
//
//                    @Override
//                    public void onComplete() {
//
//                    }
        });

        binding.activityOpeningRenewalBtnTaobaoLink.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
//                Intent intent = new Intent();
//                intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                intent.putExtra("short_url", "taobaoUrl");
//                startActivity(intent);
                AgentWebActivity.navigation2(
                        OpeningRenewalVipActivity.this,
                        "taobaoUrl");
            }
        });
        binding.activityOpeningRenewalBtnContactCustomerService.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
//                Intent intent = new Intent();
//                intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                intent.putExtra("short_url", "android_url_contact_author");
//                startActivity(intent);
                AgentWebActivity.navigation2(
                        OpeningRenewalVipActivity.this,
                        "android_url_contact_author"
                );
            }
        });

        binding.activityOpeningRenewalBtnWepay.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                if (total_fee <= 0) {
                    ToastUtils.showShort("请联网获取价格");
                    return;
                }
                binding.activityOpeningRenewalRadiobuttonOneYear.setEnabled(false);
                binding.activityOpeningRenewalRadiobuttonOneMonth.setEnabled(false);
                binding.activityOpeningRenewalRadiobuttonThreeMonths.setEnabled(false);
                showPayListDialog();
            }
        });

        binding.activityOpeningRenewalBtnOpeningRenewalVipTutorial.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
//                Intent intent = new Intent();
//                intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                intent.putExtra("short_url", "android_url_opening_renewal_tutorial");
//                startActivity(intent);
                AgentWebActivity.navigation2(
                        OpeningRenewalVipActivity.this,
                        "android_url_opening_renewal_tutorial"
                );
            }
        });
    }

    public void showPayListDialog() {
        if (LCUser.getCurrentUser() == null) {
            ToastUtils.showShort("请重新登录");
            return;
        }
        if (LCUser.getCurrentUser().getMobilePhoneNumber() == null) {
            ToastUtils.showShort("账号绑定的手机号为空，请绑定后再试");
            return;
        }
        int fee = total_fee <= 0 ? 50000 : total_fee;
        nativeOrder(fee);//由于云引擎appid是测量员的，所以无法调用app下单，只能二维码下单
//        new MaterialDialog.Builder(this)
//                .title(R.string.choose_wechat_pay_type)
//                .items(new String[]{getString(R.string.wechat_direct_pay), getString(R.string.wechat_qr_pay)})
//                .itemsCallback(new MaterialDialog.ListCallback() {
//                    @Override
//                    public void onSelection(MaterialDialog dialog, View itemView, int which, CharSequence text) {
//                        switch (which) {
//                            case 0:
//                                appOrder(fee);
//                                break;
//                            case 1:
//                                nativeOrder(fee);
//                                break;
//                        }
//                        binding.activityOpeningRenewalBtnWepay.setEnabled(false);
//                    }
//                }).show();
    }

    /**
     * 二维码下单
     *
     * @param fee 金额，单位分
     */
    private void nativeOrder(int fee) {
        WxPayV3.getInstance().generateNativeVipOrder(this,
                LCUser.getCurrentUser().getSessionToken(),
                "导线助手会员",
                fee,
                orderType,
                new WxPayV3.nativeOrderGenerateListener() {
                    @Override
                    public void onOrderGenerateSuccess(String code_url) {
                        showQrCode(code_url);
                    }

                    @Override
                    public void onOrderGenerateFailed(String errorMsg) {
                        ToastUtils.showLong(errorMsg);
                    }
                });
    }

    /**
     * App跳转微信支付，下单
     *
     * @param fee 金额，单位分
     */
    private void appOrder(int fee) {
        //查询订单是否支付成功
        payingDialog = new MaterialDialog.Builder(OpeningRenewalVipActivity.this)
                .title(R.string.hint)
                .content("正在支付……稍后请点击\"支付完成\"")
                .positiveText(R.string.pay_over)
                .onPositive(new MaterialDialog.SingleButtonCallback() {
                    @Override
                    public void onClick(@NonNull MaterialDialog dialog, @NonNull DialogAction which) {
                        //查询订单是否支付成功
                        queryOrderStatus();
                    }
                })
                .show();

        WxPayV3.getInstance().generateAppVipOrder(this,
                LCUser.getCurrentUser().getSessionToken(),
                "导线助手会员",
                fee,
                orderType,
                new WxPayV3.appOrderGenerateListener() {
                    @Override
                    public void onOrderGenerateSuccess() {
                        hasPayResult = false;
                        WxPayV3.getInstance().pay();
                    }

                    @Override
                    public void onOrderGenerateFailed(String errorMsg) {
                        ToastUtils.showLong(errorMsg);
                    }
                });
    }

    /**
     * 查询订单支付状态
     */
    private void queryOrderStatus() {
        binding.activityOpeningRenewalBtnWepay.setEnabled(true);
        if (payingDialog != null)
            payingDialog.dismiss();
        WxPayV3.getInstance().queryVipOrder(this, new WxPayV3.VipOrderQueryListener() {
            @Override
            public void onOrderPaySuccess(String orderType, Date paySuccessDate) {
                VipManager.getInstance().controlSdkRefresh(OpeningRenewalVipActivity.this, ((App) Utils.getApp()).registerCodeV2, new VipManager.OnSdkRefreshListener() {
                    @Override
                    public void onRefreshSuccess() {
                        binding.activityOpeningRenewalTvExpirationDate.setText(String.format("%s:%s", getString(R.string.current_expiration_date), DateUtils.toFull(VipManager.getInstance().expireDate)));
                        new AlertDialog.Builder(OpeningRenewalVipActivity.this)
                                .setTitle(R.string.success)
                                .setMessage("续费成功，当前到期时间为" + DateUtils.toFull(VipManager.getInstance().expireDate))
                                .setPositiveButton(R.string.confirm, null)
                                .setOnDismissListener(new DialogInterface.OnDismissListener() {
                                    @Override
                                    public void onDismiss(DialogInterface dialog) {
                                        setResult(RESULT_OK);
                                        finish();
                                    }
                                })
                                .show();
                    }

                    @Override
                    public void onRefreshFailed(int errorCode) {
                        ToastUtils.showShort(getString(R.string.error_code) + ":" + errorCode);
                    }
                });
            }

            @Override
            public void onOrderQueryFailed(String errorMsg) {
                tryQueryPayResultAgain(getString(R.string.pay_failed), errorMsg);
            }
        });
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            setResult(RESULT_OK);
            finish();
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (hasPayResult) {
            queryOrderStatus();
            hasPayResult = false;
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(PayOverEvent event) {
        hasPayResult = true;
    }

    /**
     * 查询失败后再次显示查询查询
     */
    private void tryQueryPayResultAgain(String title, String message) {
        if (!ActivityUtils.isLiving(this)) {
            return;
        }
        new AlertDialog.Builder(OpeningRenewalVipActivity.this)
                .setTitle(title)
                .setMessage(message)
                .setPositiveButton(R.string.query_again, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        if (queryCount >= 5) {
                            ToastUtils.showLong(R.string.query_times_cannot_bigger_than_five);
                            return;
                        }
                        queryCount++;
                        //查询订单是否支付成功
                        queryOrderStatus();
                    }
                })
                .setNegativeButton(R.string.cancel, null)
                .show();
    }

    /**
     * 生成二维码
     */
    public void showQrCode(String code_url) {
        MaterialDialog dialog = new MaterialDialog.Builder(this)
                .title(R.string.wechat_qr_pay)
                .customView(R.layout.dialog_register_pay_qr_code, false)
                .positiveText(this.getString(R.string.pay_over))
                .dismissListener(new DialogInterface.OnDismissListener() {
                    @Override
                    public void onDismiss(DialogInterface dialogInterface) {
                        //查询订单是否支付成功
                        queryOrderStatus();
                    }
                })
                .build();

        qrImageView = dialog.getCustomView().findViewById(R.id.dialog_register_pay_qr_code_imageview);
        try {
            HmsBuildBitmapOption options = new HmsBuildBitmapOption.Creator()
                    .setBitmapMargin(1)
                    .setBitmapColor(Color.BLACK)
                    .setBitmapBackgroundColor(Color.WHITE)
                    .create();
            Bitmap resultImage = ScanUtil.buildBitmap(code_url, HmsScan.QRCODE_SCAN_TYPE, 700, 700, options);
            qrImageView.setImageBitmap(resultImage);
        } catch (WriterException e) {
            ToastUtils.showShort("二维码参数错误!");
        }
        if (!ActivityUtils.isLiving(this)) {
            return;
        }
        dialog.show();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EventBus.getDefault().unregister(this);
        if (payingDialog != null) {
            payingDialog.dismiss();
            payingDialog = null;
        }
    }
}
