package com.bingce.controlnetwork.activity.user;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;

import androidx.appcompat.app.AlertDialog;

import com.bingce.AppChannel;
import com.bingce.activity.VipCloudFunction;
import com.bingce.controlapphelper.util.HideKeyUtil;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityRedeemBinding;
import com.bingce.utils.DateUtils;
import com.bingce.utils.VipManager;

import org.polaric.colorful.ColorfulActivity;

import java.util.Date;

import blankj.utilcode.util.AppUtils;
import blankj.utilcode.util.ToastUtils;
import cn.leancloud.LCObject;
import cn.leancloud.LCUser;

public class RedeemVipActivity extends ColorfulActivity {
    private ProgressDialog progressDialog;
    private LCObject cashVoucherLCObject;
    private ActivityRedeemBinding binding;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityRedeemBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        setSupportActionBar(binding.getRoot().findViewById(R.id.toolbar));
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        initMaterialEditTextColor();

        binding.activityRedeemTvVersion.setText("V" + AppUtils.getAppVersionName());

        if (LCUser.getCurrentUser() != null) {
            if (VipManager.getInstance().expireDate != null) {
                binding.activityRedeemTvExpirationDate.setText(getString(R.string.current_expiration_date) + ":" + DateUtils.toFull(VipManager.getInstance().expireDate));
            } else {
                binding.activityRedeemTvExpirationDate.setText(getString(R.string.current_expiration_date) + ":无");
            }
        } else {
            ToastUtils.showShort(R.string.not_log_in);
            finish();
        }

        if (VipManager.getInstance().checkReg()) {
            new AlertDialog.Builder(this)
                    .setTitle(R.string.hint)
                    .setMessage("此手机已经绑定激活，请确定是否需要购买会员，绑定激活和购买会员可以二选其一")
                    .setPositiveButton(R.string.i_know, null)
                    .show();
        }

        binding.activityRedeemMetRedeemCode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                binding.activityRedeemBtnRedeem.setEnabled(!"".equals(binding.activityRedeemMetRedeemCode.getText().toString().trim()));
            }
        });

        binding.activityRedeemBtnRedeem.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                if (LCUser.getCurrentUser() == null) {
                    ToastUtils.showShort(R.string.not_log_in);
                    return;
                }
                HideKeyUtil.hideSoftInput();
                if (binding.activityRedeemMetRedeemCode.getText().toString().trim().length() != 24) {
                    ToastUtils.showShort(R.string.cash_voucher_code_length_must_be_24);
                    return;
                }

                progressDialog = new ProgressDialog(RedeemVipActivity.this);
                progressDialog.setMessage(getString(R.string.please_wait));
                progressDialog.setCanceledOnTouchOutside(false);
                progressDialog.show();

                String cashVoucherCode = binding.activityRedeemMetRedeemCode.getText().toString().trim().toLowerCase();
                VipCloudFunction.redeemVip(LCUser.getCurrentUser().getSessionToken(), cashVoucherCode, AppChannel.customChannel, new VipCloudFunction.OnRedeemVipListener() {
                    @Override
                    public void onRedeemSuccess(Date newExpireDate) {
                        if (progressDialog != null)
                            progressDialog.dismiss();

                        savePayLog(cashVoucherCode, newExpireDate);

                        new AlertDialog.Builder(RedeemVipActivity.this)
                                .setTitle(R.string.success)
                                .setMessage("续费成功，当前到期时间为" + DateUtils.toFull(newExpireDate))
                                .setPositiveButton(R.string.confirm, null)
                                .setOnDismissListener(new DialogInterface.OnDismissListener() {
                                    @Override
                                    public void onDismiss(DialogInterface dialog) {
                                        setResult(RESULT_OK);
                                        finish();
                                    }
                                })
                                .show();
                    }

                    @Override
                    public void onRedeemFailed(String errorMsg) {
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        ToastUtils.showShort(errorMsg);
                    }
                });
            }
        });
    }

    private void savePayLog(String cashVoucher, Date newExpireDate) {
        final LCObject payLog = new LCObject("PayLog");
        payLog.put("payFor", "兑换会员");
        payLog.put("user", LCUser.getCurrentUser());
        payLog.put("currentExpireDate", newExpireDate);
        payLog.put("client", "android");
        payLog.put("channel", "cashVoucher");
        payLog.put("total_fee", "0");
        payLog.put("cash_fee", "0");
        payLog.put("versionCode", AppUtils.getAppVersionCode());
        payLog.put("cashVoucher", cashVoucher);
        payLog.saveInBackground().subscribe();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            finish();
        }
        return super.onOptionsItemSelected(item);
    }

    private void initMaterialEditTextColor() {
        if (SettingValueUtil.isThemeDark()) {
            binding.activityRedeemMetRedeemCode.setMetTextColor(Color.WHITE);

            binding.activityRedeemMetRedeemCode.setPrimaryColor(Color.LTGRAY);

            binding.activityRedeemMetRedeemCode.setFocusFraction(1.0f);

            binding.activityRedeemMetRedeemCode.setMetHintTextColor(Color.GRAY);

            binding.activityRedeemMetRedeemCode.setUnderlineColor(Color.GRAY);
        }
    }
}
