package com.bingce.controlnetwork.activity.user;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.SpannableString;
import android.text.Spanned;
import android.text.style.TextAppearanceSpan;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;

import com.bingce.AppChannel;
import com.bingce.activity.UserCloudFunction;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ActivityVerifyCodeBinding;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.SendCodeCountDownManager;

import org.polaric.colorful.ColorfulActivity;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import blankj.utilcode.util.ToastUtils;

public class VerifySMSCodeActivity extends ColorfulActivity {
    public static final String KEY_PHONE = "phone_number";
    public static final String KEY_EMAIL = "email";
    public static final String KEY_UNION_ID = "unionid";
    public static final String KEY_TODO = "todo";

    public static final int TO_VERIFY_PHONE_NUMBER = 0;
    public static final int TO_SIGNUP_OR_LOGIN_BY_PHONE_NUMBER = 1;
    public static final int TO_SIGNUP_OR_LOGIN_BY_EMAIL_BIND_PHONE = 2;
    public static final int TO_SIGNUP_OR_LOGIN_BY_WEIXIN_BIND_PHONE = 3;
    private ActivityVerifyCodeBinding binding;
    private ProgressDialog progressDialog;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityVerifyCodeBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        Toolbar mToolbar = findViewById(R.id.toolbar);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null) getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        getSupportActionBar().setTitle(R.string.verify);

        binding.tvInputHintTitle.setText(R.string.enter_sms_verification_code);

        String phoneOrEmail = getPhoneNumber();
        String trip = String.format(getString(R.string.format_enter_verification_code), phoneOrEmail);
        binding.tvInputTrip.setText(matcherChangeText(trip, phoneOrEmail.startsWith("+") ? ("\\" + phoneOrEmail) : phoneOrEmail));//+前需要转义

        SendCodeCountDownManager.getInstance().start(this, 60000, 1000, binding.tvSendAuthCode, new SendCodeCountDownManager.OnCountDownListener() {
            @Override
            public void onSendValid() {
                sendCode();
            }

            @Override
            public void onSendInvalid() {

            }
        });
        binding.tvSendAuthCode.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                SendCodeCountDownManager.getInstance().start(VerifySMSCodeActivity.this, 60000, 1000, binding.tvSendAuthCode, new SendCodeCountDownManager.OnCountDownListener() {
                    @Override
                    public void onSendValid() {
                        sendCode();
                    }

                    @Override
                    public void onSendInvalid() {

                    }
                });
            }
        });

        binding.etInputAuthCode.setOnInputListener(code -> {
            verifyCode(code);
        });
    }

    private void sendCode() {
        if (getPhoneNumber() == null) {
            ToastUtils.showShort(R.string.phone_number_invalid);
            return;
        }
        UserCloudFunction.sendSMSCode(getPhoneNumber(), new UserCloudFunction.OnSendCodeListener() {
            @Override
            public void onCodeSendSuccess() {
                ToastUtils.showShort(R.string.send_success);
            }

            @Override
            public void onCodeSendFailed(String errorMsg) {
                ToastUtils.showLong(errorMsg);
            }
        });
    }

    private void verifyCode(String code) {
        if (progressDialog != null)
            progressDialog.dismiss();
        progressDialog = new ProgressDialog(VerifySMSCodeActivity.this);
        progressDialog.setMessage(getString(R.string.logging));
        progressDialog.setCanceledOnTouchOutside(false);
        progressDialog.show();

        switch (getTodo()) {
            case TO_VERIFY_PHONE_NUMBER:
                if (getPhoneNumber() == null) {
                    ToastUtils.showShort(R.string.phone_number_invalid);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                UserCloudFunction.verifySMSCode(getPhoneNumber(), code, new UserCloudFunction.OnVerifyCodeListener() {
                    @Override
                    public void onCodeVerifySuccess() {
                        if (progressDialog != null)
                            progressDialog.dismiss();

                        SendCodeCountDownManager.getInstance().resetRemainingTime();//验证成功，重置倒计时，否则跳转到下一个验证页面不会自动发送验证码

                        Intent intent = new Intent();
                        intent.putExtra("isMail", false);
                        intent.putExtra("phoneNumberOrEmail", getPhoneNumber());
                        setResult(RESULT_OK, intent);
                        finish();
                    }

                    @Override
                    public void onCodeVerifyFailed(String errorMsg) {
                        if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                        if (progressDialog != null)
                            progressDialog.dismiss();
                        ToastUtils.showLong(errorMsg);
                    }
                });
                break;
            case TO_SIGNUP_OR_LOGIN_BY_PHONE_NUMBER:
                if (getPhoneNumber() == null) {
                    ToastUtils.showShort(R.string.phone_number_invalid);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                UserCloudFunction.signUpOrLoginByMobilePhone(getPhoneNumber(), code, AppChannel.customChannel,
                        new UserCloudFunction.OnGetSessionTokenListener() {
                            @Override
                            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();

                                SendCodeCountDownManager.getInstance().resetRemainingTime();//验证成功，重置倒计时，否则跳转到下一个验证页面不会自动发送验证码

                                Intent intent = new Intent();
                                intent.putExtra("sessionToken", sessionToken);
                                intent.putExtra("newUser", newUser);
                                setResult(RESULT_OK, intent);
                                finish();
                            }

                            @Override
                            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }

                            @Override
                            public void onSessionTokenGetException(String msg) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }
                        });
                break;
            case TO_SIGNUP_OR_LOGIN_BY_EMAIL_BIND_PHONE:
                if (getEmail() == null) {
                    ToastUtils.showShort(R.string.email_address_illegal);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                if (getPhoneNumber() == null) {
                    ToastUtils.showShort(R.string.phone_number_invalid);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                UserCloudFunction.loginByEmailBindPhone(getEmail(), getPhoneNumber(), code, AppChannel.customChannel,
                        new UserCloudFunction.OnGetSessionTokenListener() {
                            @Override
                            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();

                                SendCodeCountDownManager.getInstance().resetRemainingTime();//验证成功，重置倒计时，否则跳转到下一个验证页面不会自动发送验证码

                                Intent intent = new Intent();
                                intent.putExtra("sessionToken", sessionToken);
                                intent.putExtra("newUser", newUser);
                                setResult(RESULT_OK, intent);
                                finish();
                            }

                            @Override
                            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }

                            @Override
                            public void onSessionTokenGetException(String msg) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }
                        });
                break;
            case TO_SIGNUP_OR_LOGIN_BY_WEIXIN_BIND_PHONE:
                if (getUnionId() == null) {
                    ToastUtils.showShort(R.string.wechat_unionid_invalid);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                if (getPhoneNumber() == null) {
                    ToastUtils.showShort(R.string.phone_number_invalid);
                    if (progressDialog != null)
                        progressDialog.dismiss();
                    return;
                }
                UserCloudFunction.loginByWxUnionIdBindPhone(getUnionId(), getPhoneNumber(), AppChannel.customChannel,
                        new UserCloudFunction.OnGetSessionTokenListener() {
                            @Override
                            public void onSessionTokenGetSuccess(String sessionToken, boolean newUser) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();

                                SendCodeCountDownManager.getInstance().resetRemainingTime();//验证成功，重置倒计时，否则跳转到下一个验证页面不会自动发送验证码

                                Intent intent = new Intent();
                                intent.putExtra("sessionToken", sessionToken);
                                intent.putExtra("newUser", newUser);
                                setResult(RESULT_OK, intent);
                                finish();
                            }

                            @Override
                            public void onSessionTokenGetFailed(int code, String msg, String resp) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }

                            @Override
                            public void onSessionTokenGetException(String msg) {
                                if (!ActivityUtils.isLiving(VerifySMSCodeActivity.this)) return;
                                if (progressDialog != null)
                                    progressDialog.dismiss();
                                ToastUtils.showLong(msg);
                            }
                        });
                break;
        }

    }

    private String getPhoneNumber() {
        return IntentUtil.stringExtra(getIntent(), KEY_PHONE);
    }

    private String getEmail() {
        return IntentUtil.stringExtra(getIntent(), KEY_EMAIL);
    }

    private String getUnionId() {
        return IntentUtil.stringExtra(getIntent(), KEY_UNION_ID);
    }


    private int getTodo() {
        return IntentUtil.intExtra(getIntent(), KEY_TODO);
    }

    public static void start(Context context, String phoneNumber, String email, String unionid, int todo, ActivityResultLauncher<Intent> launcher) {
        Intent intent = new Intent(context, VerifySMSCodeActivity.class);
        intent.putExtra(KEY_PHONE, phoneNumber);
        intent.putExtra(KEY_EMAIL, email);
        intent.putExtra(KEY_UNION_ID, unionid);
        intent.putExtra(KEY_TODO, todo);
        launcher.launch(intent);
    }

    private SpannableString matcherChangeText(String str, String keyword) {
        SpannableString ss = new SpannableString(str);
        Pattern pattern = Pattern.compile(keyword);
        Matcher matcher = pattern.matcher(ss);
        while (matcher.find()) {
            int start = matcher.start();
            int end = matcher.end();
            ss.setSpan(new TextAppearanceSpan(this, R.style.style_color_black), start, end, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);//new ForegroundColorSpan(color)
        }
        return ss;
    }
}
