package com.bingce.controlnetwork.adapter;

import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.ui.vh.GroupParentViewHolder;
import com.bingce.controlnetwork.ui.vh.TextSelectableViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/*
 * 限差详情adapter
 * */
public abstract class AbstractPointSelectAdapter extends BaseExpandableListAdapter {
    private final ArrayList<String> selectedId = new ArrayList<>();
    private final ArrayList<String> selectedName = new ArrayList<>();
    private final int maxCount;
    private final Context context;

    public AbstractPointSelectAdapter(PickPointResult defaultValue,
                                      int maxCount,
                                      Context context) {
        this.maxCount = maxCount;
        if (defaultValue != null) {
            List<String> ids = defaultValue.getSelectedIds();
            if (ids != null) {
                selectedId.addAll(ids);
            }

            List<String> names = defaultValue.getSelectedNames();
            if (names != null) {
                selectedName.addAll(names);
            }
        }
        this.context = context;
    }

    protected Context context() {
        return context;
    }

    public ArrayList<String> getSelectedId() {
        return selectedId;
    }

    public ArrayList<String> getSelectedName() {
        return selectedName;
    }

    public abstract int getGroupCount();

    public abstract int getChildrenCount(int groupPosition);

    public abstract String getGroup(int groupPosition);

    public abstract PointRecord getChild(int groupPosition, int childPosition);

    @Override
    final public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    final public long getChildId(int groupPosition, int childPosition) {
        return groupPosition * 10000L + childPosition;
    }

    @Override
    public boolean hasStableIds() {
        return true;
    }

    protected boolean showGroupArrow() {
        return true;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        GroupParentViewHolder viewHolder = null;
        if (convertView == null) {
            convertView = ViewFactory.generatorView(parent.getContext(), parent, ViewItemType.groupParent);
            viewHolder = new GroupParentViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (GroupParentViewHolder) convertView.getTag();
        }
        if (viewHolder != null) {
            String groupName = getGroup(groupPosition);
            if (viewHolder.name != null) {
                viewHolder.name.setText(groupName);
            }
            if (viewHolder.expandIcon != null) {
                if (showGroupArrow()) {
                    viewHolder.expandIcon.setImageResource(isExpanded ? R.drawable.ic_arrow_up : R.drawable.ic_arrow_down);
                    viewHolder.expandIcon.setVisibility(View.VISIBLE);
                } else {
                    viewHolder.expandIcon.setVisibility(View.GONE);
                }
            }
        }
        return convertView;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        PointRecord record = getChild(groupPosition, childPosition);
        RecyclerView.ViewHolder viewHolder = null;
        if (convertView == null) {
            viewHolder = ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.textSelectable);
            if (viewHolder == null) {
                return null;
            }
            convertView = viewHolder.itemView;
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (RecyclerView.ViewHolder) convertView.getTag();
        }
        //更新item内容
        if (viewHolder instanceof TextSelectableViewHolder) {
            TextSelectableViewHolder titleWithSubtitleViewHolder = (TextSelectableViewHolder) viewHolder;
            titleWithSubtitleViewHolder.name.setText(record.getName());
            if (selectedId.contains(record.getId())) {
                titleWithSubtitleViewHolder.status.setVisibility(View.VISIBLE);
            } else {
                titleWithSubtitleViewHolder.status.setVisibility(View.GONE);
            }
        }
        convertView.setOnClickListener(v -> {
            if (!selectedId.contains(record.getId())) {
//                if (singlePick) {
//                    selectedId.clear();
//                    selectedName.clear();
//                }
//                selectedId.add(record.getId());
//                selectedName.add(record.getName());
                if (maxCount == 1) {
                    selectedId.clear();
                    selectedName.clear();
                    selectedId.add(record.getId());
                    selectedName.add(record.getName());
                } else if (maxCount < 0) {
                    selectedId.add(record.getId());
                    selectedName.add(record.getName());
                } else if (selectedId.size() >= maxCount) {
                    ToastUtils.showShort(Tools.getString(R.string.max_select_hint, maxCount));
                } else {
                    selectedId.add(record.getId());
                    selectedName.add(record.getName());
                }
            } else {
                selectedId.remove(record.getId());
                selectedName.remove(record.getName());
            }
            notifyDataSetChanged();
        });
        return convertView;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }
}
