package com.bingce.controlnetwork.adapter;

import android.view.ViewGroup;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IUniversalListItem;

import org.jetbrains.annotations.NotNull;

import java.util.List;

public abstract class BaseAdapter<T, VH extends RecyclerView.ViewHolder> extends RecyclerView.Adapter<VH> {
    private final List<T> mValues;

    public BaseAdapter(List<T> items) {
        mValues = items;
    }

    public void updateData(List<T> list) {
        mValues.clear();
        if (list != null && !list.isEmpty()) {
            mValues.addAll(list);
        }
        notifyDataSetChanged();
    }

    protected abstract VH createVH(@NotNull ViewGroup parent, int viewType);

    @Override
    public @NotNull
    VH onCreateViewHolder(@NotNull ViewGroup parent, int viewType) {
        VH viewHolder = createVH(parent, viewType);
        assert viewHolder != null;
        return viewHolder;
    }

    protected T getItem(int position) {
        return mValues.get(position);
    }

    protected boolean clickEnable(int position) {
        return true;
    }

    @Override
    public int getItemCount() {
        if (mValues == null) {
            return 0;
        }
        return mValues.size();
    }

    public interface IOnItemClick<T extends IUniversalListItem<?>> {
        void onItemClick(T item);
    }
}
