package com.bingce.controlnetwork.adapter;

import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.View;

import com.bingce.controlapphelper.model.IUniversalListItem;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import com.bingce.controlnetwork.adapter.contextmenu.IAdapterContextMenu;
import com.bingce.controlnetwork.ui.vh.IndexTextViewHolder;

public class BaseIndexTextWithContextMenuAdapter<T extends IUniversalListItem<?>>
        extends IndexWithTextAdapter<T>
        implements View.OnCreateContextMenuListener {
    private int currentLongPressIndex = -1;
    private final IAdapterContextMenu<T> adapterContextMenu;

    public BaseIndexTextWithContextMenuAdapter(List<T> items,
                                               IOnItemClick<T> callback,
                                               IAdapterContextMenu<T> adapterContextMenu) {
        super(items, callback);
        this.adapterContextMenu = adapterContextMenu;
    }

    @Override
    final public void onBindViewHolder(@NotNull IndexTextViewHolder holder, int position) {
        super.onBindViewHolder(holder, position);
        holder.itemView.setOnLongClickListener(v -> {
            currentLongPressIndex = position;
            return false;
        });
        holder.itemView.setOnCreateContextMenuListener(this);
    }

    @Override
    final public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        if (adapterContextMenu == null) {
            return;
        }
        menu.clear();
        for (int index = 0; index < adapterContextMenu.menuItemSize(); index++) {
            menu.add(0, index, 0, adapterContextMenu.menuItemName(index));
            MenuItem menuItem = menu.getItem(index);
            if (menuItem != null) {
                menuItem.setOnMenuItemClickListener(item ->
                        adapterContextMenu.onMenuItemClick(getItem(currentLongPressIndex), item));
            }
        }
    }
}
