package com.bingce.controlnetwork.adapter;

import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import com.bingce.controlnetwork.ui.vh.IndexTextViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

public class IndexWithTextAdapter<T extends IUniversalListItem<?>> extends BaseAdapter<T, IndexTextViewHolder> {
    private IOnItemClick<T> callback;
    private boolean showIndex = true;

    public IndexWithTextAdapter(List<T> items, IOnItemClick<T> callback) {
        super(items);
        this.callback = callback;
    }

    public IndexWithTextAdapter(List<T> items) {
        super(items);
        this.callback = null;
    }

    public void setCallback(IOnItemClick<T> callback) {
        this.callback = callback;
    }

    protected ViewItemType itemType() {
        return ViewItemType.indexWithName;
    }

    @Override
    final protected IndexTextViewHolder createVH(@NotNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder viewHolder = ViewFactory.generatorViewHolder(parent.getContext(), parent, itemType());
        if (viewHolder instanceof IndexTextViewHolder) {
            return (IndexTextViewHolder) viewHolder;
        }
        return null;
    }

    protected void hideIndex() {
        showIndex = false;
        notifyDataSetChanged();
    }

    protected void showIndex() {
        showIndex = true;
        notifyDataSetChanged();
    }

    protected boolean clickEnable(int position) {
        return true;
    }

    @Override
    public void onBindViewHolder(final @NotNull IndexTextViewHolder holder, int position) {
        T item = getItem(position);
        if (showIndex) {
            holder.index.setVisibility(View.VISIBLE);
            holder.index.setText(String.valueOf(position + 1));
        } else {
            holder.index.setVisibility(View.GONE);
        }
        onSetName(holder.name, item);
        holder.itemView.setOnClickListener(v -> {
            if (callback != null && clickEnable(position)) {
                callback.onItemClick(item);
            }
        });
    }

    protected void onSetName(TextView nameTextView, T item) {
        nameTextView.setText(item.getName());
    }

    public interface IOnItemClick<T extends IUniversalListItem<?>> {
        void onItemClick(T item);
    }
}
