package com.bingce.controlnetwork.adapter;

import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IAdapter;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.IDistanceTypeChangeListener;
import com.bingce.controlapphelper.surveyor.data.IMeasureCache;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;

import com.bingce.controlnetwork.ui.vh.SurveyorDataAloneDistanceViewHolder;
import com.bingce.controlnetwork.ui.vh.SurveyorDataViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

/**
 * 测量记录adapter
 * 井定向-距离
 */
public class MeasureAngleDistanceAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder>
        implements IAdapter, IDistanceTypeChangeListener {

    public static final String TYPE_ANGLE = "TYPE_ANGLE";
    public static final String TYPE_DISTANCE = "TYPE_DISTANCE";

    private final IMeasureCache cachedData;
    private final ISurveyorData surveyorData;
    private final ICycleRecorder cycleRecorder;
    private boolean showSlopDistance = true;
    private final String stationId;
    private final String type;//区分水平还是距离


    public MeasureAngleDistanceAdapter(
            String stationId,
            IMeasureCache cachedData,
            ICycleRecorder cycleRecorder,
            ISurveyorData surveyorData,
            String type) {
        this.stationId = stationId;
        this.cachedData = cachedData;
        this.surveyorData = surveyorData;
        this.cycleRecorder = cycleRecorder;
        this.type = type;
        //注册数据监听
        cachedData.register(this);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh = null;
        switch (type) {
            case TYPE_ANGLE:
                vh = ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.surveyorStationPointItem);
                break;
            case TYPE_DISTANCE:
                vh = ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.aloneDistanceMeasure);
                break;
        }
        assert vh != null;
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        if (holder instanceof SurveyorDataAloneDistanceViewHolder) {
            SurveyorDataAloneDistanceViewHolder holderDistance = (SurveyorDataAloneDistanceViewHolder) holder;
            holderDistance.setup(position,
                    stationId,
                    showSlopDistance,
                    (CachedDataAloneDistance) cachedData,
                    cycleRecorder,
                    surveyorData);
        } else if (holder instanceof SurveyorDataViewHolder) {
            SurveyorDataViewHolder holderHorizontalAngle = (SurveyorDataViewHolder) holder;
            holderHorizontalAngle.setup(position,
                    stationId,
                    showSlopDistance,
                    (ICachedData) cachedData,
                    cycleRecorder,
                    surveyorData);
        }

        holder.itemView.setOnClickListener(v -> {
            if (iItemClickListener != null) {
                iItemClickListener.click(position);
            }
        });
    }

    @Override
    public int getItemCount() {
        return cachedData.pointCount();
    }

    @Override
    public void onDataChange() {
        notifyDataSetChanged();
    }

    @Override
    public void onShowSlopDistance(boolean value) {
        this.showSlopDistance = value;
    }


    private IItemClickListener iItemClickListener;

    public void setOnItemClickListener(IItemClickListener iItemClickListener) {
        this.iItemClickListener = iItemClickListener;
    }

    public interface IItemClickListener {
        void click(int position);
    }
}
