package com.bingce.controlnetwork.adapter;

import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IAdapter;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.IDistanceTypeChangeListener;
import com.bingce.controlapphelper.surveyor.data.IMeasureCache;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;

import java.util.Objects;

import com.bingce.controlnetwork.ui.vh.SurveyorDataViewHolderHeightTraverse;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

/**
 * 高程导线数据适配器
 */
public class MeasureHeightTraverseAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder>
        implements IAdapter, IDistanceTypeChangeListener {

    private final IMeasureCache cachedData;
    private final ISurveyorData surveyorData;
    private final ICycleRecorder cycleRecorder;
    private boolean showSlopDistance = true;
    private final String stationId;
    private final int typeBackOrFront;//区分后视还是前视


    public MeasureHeightTraverseAdapter(
            String stationId,
            IMeasureCache cachedData,
            ICycleRecorder cycleRecorder,
            ISurveyorData surveyorData,
            int typeBackOrFront) {
        this.stationId = stationId;
        this.cachedData = cachedData;
        this.surveyorData = surveyorData;
        this.cycleRecorder = cycleRecorder;
        this.typeBackOrFront = typeBackOrFront;
        //注册数据监听
        cachedData.register(this);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return Objects.requireNonNull(ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.heightTraverseSurveyor));
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        SurveyorDataViewHolderHeightTraverse viewHolder = (SurveyorDataViewHolderHeightTraverse) holder;
        viewHolder.setup(position,
                stationId,
                showSlopDistance,
                (ICachedData) cachedData,
                cycleRecorder,
                surveyorData);

        holder.itemView.setOnClickListener(v -> {
            if (iItemClickListener != null) {
                iItemClickListener.click(position);
            }
        });
    }

    @Override
    public int getItemCount() {
        return cachedData.pointCount();
    }

    @Override
    public void onDataChange() {
        notifyDataSetChanged();
    }

    @Override
    public void onShowSlopDistance(boolean value) {
        this.showSlopDistance = value;
    }


    private IItemClickListener iItemClickListener;

    public void setOnItemClickListener(IItemClickListener iItemClickListener) {
        this.iItemClickListener = iItemClickListener;
    }

    public interface IItemClickListener {
        void click(int position);
    }
}
