package com.bingce.controlnetwork.adapter;

import android.view.View;
import android.widget.ImageView;

import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.ui.vh.IndexTextSelectableViewHolder;
import com.bingce.controlnetwork.ui.vh.IndexTextViewHolder;

public class MultipleSelectableIndexWithTextAdapter<T extends IUniversalListItem<?>> extends IndexWithTextAdapter<T> {
    private final List<String> selectedIds = new ArrayList<>();

    public MultipleSelectableIndexWithTextAdapter(List<String> selectedIds,
                                                  List<T> items,
                                                  IOnItemClick<T> callback) {
        super(items);
        setCallback(item -> {
            onItemClicked(item.getId());
            if (callback == null) {
                return;
            }
            callback.onItemClick(item);
        });
        this.selectedIds.addAll(selectedIds);
    }

    protected void onItemClicked(String id) {
        if (selectedIds.contains(id)) {
            selectedIds.remove(id);
        } else {
            selectedIds.add(id);
        }
        notifyDataSetChanged();
    }

    @Override
    protected ViewItemType itemType() {
        return ViewItemType.indexWithNameSelectable;
    }

    @Override
    public void onBindViewHolder(@NotNull IndexTextViewHolder holder, int position) {
        super.onBindViewHolder(holder, position);
        //设置状态
        T t = getItem(position);
        if (t == null) {
            return;
        }
        if (holder instanceof IndexTextSelectableViewHolder) {
            ImageView iv = ((IndexTextSelectableViewHolder) holder).status;
            if (iv != null) {
                if (selectedIds.contains(t.getId())) {
                    iv.setVisibility(View.VISIBLE);
                } else {
                    iv.setVisibility(View.GONE);
                }
            }
        }
    }

    public List<String> selectedIds() {
        return selectedIds;
    }
}
