package com.bingce.controlnetwork.adapter;

import android.content.Context;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentActivity;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDeleteEdit;
import com.bingce.controlnetwork.fragment.PointFragment;
import com.bingce.controlnetwork.ui.widget.dialog.LoadingUtil;

public class PointRecordAdapter extends BaseIndexTextWithContextMenuAdapter<PointRecord> {
    public PointRecordAdapter(List<PointRecord> items,
                              @NonNull FragmentActivity activity) {
        super(items,
                item -> editRecord(activity, item),
                new ContextMenuDeleteEdit<PointRecord>() {
                    @Override
                    protected void delete(PointRecord item) {
                        deletePoint(activity, item);
                    }

                    @Override
                    protected void edit(PointRecord item) {
                        editRecord(activity, item);
                    }

                    /**
                     * 删除点
                     * 项目里这个点只要有数据就不能删除
                     *
                     * @param item
                     */
                    private void deletePoint(Context context, PointRecord item) {
                        LoadingUtil.show(context, null);

                        ThreadPoolUtil.execute(() -> {
                            String msg = PointUtil.getUsedMsg(item.getId());
                            if (msg != null) {
                                ToastUtils.showShort(msg);
                                dismissLoading();
                                return;
                            }
                            SurveyorDatabaseFactory
                                    .instance
                                    .getPointDataSource()
                                    .delete(item);

                            dismissLoading();
                        });
                    }

                    private void dismissLoading() {
                        ThreadPoolUtil.executeInMain(LoadingUtil::dismiss);
                    }

                });
    }


    private static void editRecord(FragmentActivity activity, PointRecord record) {
        PointFragment.edit(activity.getSupportFragmentManager(),
                record.projectId,
                record.getName(), record.getId(), record.getPointType(), record.getX(), record.getY(), record.getZ());
    }
}
