package com.bingce.controlnetwork.adapter;

import android.content.Context;

import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.model.KnownAndUnknownPoints;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlnetwork.R;

import java.util.List;

public class PointSelectAdapter extends AbstractPointSelectAdapter {
    private final KnownAndUnknownPoints data;
    private final List<PointRecord> globalPoints;

    public PointSelectAdapter(KnownAndUnknownPoints points,
                              List<PointRecord> globalPoints,
                              PickPointResult defaultValue,
                              int maxCount,
                              Context context) {
        super(defaultValue, maxCount, context);
        this.data = points;
        this.globalPoints = globalPoints;
    }

    @Override
    public int getGroupCount() {
//        return data.groupCount() + 1;
        return data.groupCount() ;
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        if (groupPosition >= data.groupCount()) {
            if (globalPoints == null) {
                return 0;
            }
            return globalPoints.size();
        }
        return data.childrenCount(groupPosition);
    }

    @Override
    public String getGroup(int groupPosition) {
        if (groupPosition >= data.groupCount()) {
            return context().getString(R.string.global_control_point);
        }
        return data.groupName(groupPosition, context());
    }

    @Override
    public PointRecord getChild(int groupPosition, int childPosition) {
        if (groupPosition >= data.groupCount()) {
            if (globalPoints == null) {
                return null;
            }
            return globalPoints.get(childPosition);
        }
        return data.child(groupPosition, childPosition);
    }
}