package com.bingce.controlnetwork.adapter;

import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.bingce.controlapphelper.datasource.database.prism.PrismRecord;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.contextmenu.IAdapterContextMenu;
import com.bingce.controlnetwork.ui.vh.PrismManageViewHolder;

import org.jetbrains.annotations.NotNull;

import java.util.List;

public class PrismMagegeAdapter extends BaseAdapter<PrismRecord, PrismManageViewHolder>
        implements View.OnCreateContextMenuListener {
    private IOnItemClick<PrismRecord> callback;

    private int currentLongPressIndex = -1;

    private final IAdapterContextMenu<PrismRecord> adapterContextMenu;

    public PrismMagegeAdapter(List<PrismRecord> items, IOnItemClick<PrismRecord> callback, IAdapterContextMenu<PrismRecord> adapterContextMenu) {
        super(items);
        this.callback = callback;
        this.adapterContextMenu = adapterContextMenu;
    }

    public void setCallback(IOnItemClick<PrismRecord> callback) {
        this.callback = callback;
    }

    @Override
    final protected PrismManageViewHolder createVH(@NotNull ViewGroup parent, int viewType) {
        View itemView = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_prism_manager, parent, false);
        PrismManageViewHolder prismManageViewHolder = new PrismManageViewHolder(itemView);
        return prismManageViewHolder;
    }

    protected boolean clickEnable(int position) {
        return true;
    }

    @Override
    public void onBindViewHolder(final @NotNull PrismManageViewHolder holder, int position) {
        //长按出菜单
        holder.itemView.setOnLongClickListener(v -> {
            currentLongPressIndex = position;
            return false;
        });
        holder.itemView.setOnCreateContextMenuListener(this);

        PrismRecord item = getItem(position);

        holder.tvName.setText(item.getShowPrismTypeName());
        holder.tvType.setText(item.prismTypeLarge);

        //绝对常数
        holder.tvConstant.setText(item.prismTypeConstant);

        holder.itemView.setOnClickListener(v -> {
            if (callback != null && clickEnable(position)) {
                callback.onItemClick(item);
            }
        });
    }

    public interface IOnItemClick<T> {
        void onItemClick(T item);
    }

    @Override
    final public void onCreateContextMenu(ContextMenu menu, View v, ContextMenu.ContextMenuInfo menuInfo) {
        if (adapterContextMenu == null) {
            return;
        }
        PrismRecord currentItem = getItem(currentLongPressIndex);
        if (currentItem.modify == PrismRecord.MODIFY_DEFAULT) {
            return;
        }
        menu.clear();
        for (int index = 0; index < adapterContextMenu.menuItemSize(); index++) {
            menu.add(0, index, 0, adapterContextMenu.menuItemName(index));
            MenuItem menuItem = menu.getItem(index);
            if (menuItem != null) {
                menuItem.setOnMenuItemClickListener(menuItemBack ->
                        adapterContextMenu.onMenuItemClick(currentItem, menuItemBack));
            }
        }
    }
}
