package com.bingce.controlnetwork.adapter;

import android.view.View;
import android.widget.ImageView;

import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.ui.vh.IndexTextSelectableViewHolder;
import com.bingce.controlnetwork.ui.vh.IndexTextViewHolder;

public class SelectableIndexWithTextAdapter<T extends IUniversalListItem<?>> extends IndexWithTextAdapter<T> {
    private String selectedId;
    private final List<String> enableList = new ArrayList<>();

    public SelectableIndexWithTextAdapter(String selectedId, List<T> items, List<String> enableIndex, IOnItemClick<T> callback) {
        super(items);
        setCallback(item -> {
            updateCurrentSelectedId(item.getId());
            if (callback == null) {
                return;
            }
            callback.onItemClick(item);
        });
        if (enableIndex != null) {
            enableList.addAll(enableIndex);
        }
        this.selectedId = selectedId;
    }

    public SelectableIndexWithTextAdapter(String selectedId, List<T> items, IOnItemClick<T> callback) {
        this(selectedId, items, null, callback);
    }

    protected void updateCurrentSelectedId(String id) {
        selectedId = id;
        notifyDataSetChanged();
    }

    //    @Override
//    protected RecyclerView.ViewHolder createVH(@NotNull ViewGroup parent, int viewType) {
//        return ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewFactory.ItemType.indexWithNameSelectable);
//    }
    @Override
    protected ViewItemType itemType() {
        return ViewItemType.indexWithNameSelectable;
    }

    @Override
    protected boolean clickEnable(int position) {
        if (enableList.isEmpty() || enableList.contains(String.valueOf(position))) {
            return super.clickEnable(position);
        }
        return false;
    }

    @Override
    public void onBindViewHolder(@NotNull IndexTextViewHolder holder, int position) {
        super.onBindViewHolder(holder, position);
        //设置状态
        T t = getItem(position);
        if (t == null) {
            return;
        }
        if (holder instanceof IndexTextSelectableViewHolder) {
            ImageView iv = ((IndexTextSelectableViewHolder) holder).status;
            if (iv != null) {
                if (t.getId().equals(selectedId)) {
                    iv.setVisibility(View.VISIBLE);
                } else {
                    iv.setVisibility(View.GONE);
                }
            }

            if (!enableList.isEmpty()) {
                if (enableList.contains(String.valueOf(position))) {
                    ((IndexTextSelectableViewHolder) holder).mask.setVisibility(View.GONE);
                    ((IndexTextSelectableViewHolder) holder).item.setAlpha(1f);
                } else {
                    ((IndexTextSelectableViewHolder) holder).mask.setVisibility(View.VISIBLE);
                    ((IndexTextSelectableViewHolder) holder).item.setAlpha(.5f);
                }
            }
        }
    }

    protected String currentSelectedItemId() {
        return selectedId;
    }

    protected boolean isItemSelected(int index) {
        return currentSelectedItemId().equals(getItem(index).getId());
    }
}
