package com.bingce.controlnetwork.adapter;


import static com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord.isLineStation;

import android.content.Context;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.LiveData;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.surveyorrecord.ISurveyorRecordDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuEdit;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.fragment.stationdetail.SingleStationDetailAndEditFragment;
import com.bingce.controlnetwork.util.SurveyorStationUtil;
import com.bingce.utils.ThreadPoolUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SurveyorStationListAdapter extends BaseIndexPrefixTextWithContextMenuAdapter<SurveyorStationRecord> {
    private final LifecycleOwner lifecycleOwner;

    public SurveyorStationListAdapter(List<SurveyorStationRecord> items,
                                      @NonNull FragmentManager fragmentManager,
                                      @NonNull LifecycleOwner lifecycleOwner,
                                      @NonNull IUIRunner iuiRunner,
                                      Context context,
                                      IOnItemClick<SurveyorStationRecord> itemClick,
                                      boolean editAble) {
        super(items, itemClick,
                contextMenu(fragmentManager, lifecycleOwner, iuiRunner, context, editAble));
        this.lifecycleOwner = lifecycleOwner;
    }

    @Override
    protected void onSetName(TextView nameTextView, SurveyorStationRecord item) {
        String pointId = item.getItems().get(0).getPointId();
        nameTextView.setText(cachedId2Name.get(pointId));

        if (item.getItems() == null || item.getItems().isEmpty()) {
            return;
        }
        //停止原livedata订阅
        Object tag = nameTextView.getTag();
        LiveData<PointRecord> pointRecordLiveData;
        if (tag instanceof LiveData) {
            pointRecordLiveData = (LiveData<PointRecord>) tag;
            pointRecordLiveData.removeObservers(lifecycleOwner);
        }
        //创建新订阅
        pointRecordLiveData = SurveyorDatabaseFactory
                .instance
                .getPointDataSource()
                .findById(pointId);
        nameTextView.setTag(pointRecordLiveData);

        //监听
        pointRecordLiveData.observe(lifecycleOwner, pointRecord -> {
            if (pointRecord == null) {
                return;
            }
            nameTextView.setText(pointRecord.getName());
            cachedId2Name.put(pointRecord.id, pointRecord.getName());
        });
    }

    @Override
    protected void onSetPrefix(TextView prefixTextView, SurveyorStationRecord item) {
        String lineId = item.getLineId();
        if (cachedId2LineName.containsKey(lineId)) {
            prefixTextView.setText(cachedId2LineName.get(lineId));
        } else {
            prefixTextView.setText(StationUtil.mulCycleAngleStationPrefix());
        }

        if (lineId == null || lineId.isEmpty()) {
            return;
        }
        //停止原livedata订阅
        Object tag = prefixTextView.getTag();
        LiveData<LineRecord> lineRecordLiveData;
        if (tag instanceof LiveData) {
            lineRecordLiveData = (LiveData<LineRecord>) tag;
            lineRecordLiveData.removeObservers(lifecycleOwner);
        }
        //创建新订阅
        lineRecordLiveData = SurveyorDatabaseFactory
                .instance
                .getLineRecordDatabase()
                .liveData(lineId);
        prefixTextView.setTag(lineRecordLiveData);

        //监听
        lineRecordLiveData.observe(lifecycleOwner, lineRecord -> {
            if (lineRecord == null) {
                return;
            }
            prefixTextView.setText(lineRecord.name + "-");
            cachedId2LineName.put(lineRecord.id, lineRecord.name);
            cachedId2LineType.put(lineRecord.id, lineRecord.type);
        });
    }

    private static final Map<String, String> cachedId2Name = new HashMap<>();
    private static final Map<String, String> cachedId2LineName = new HashMap<>();
    private static final Map<String, String> cachedId2LineType = new HashMap<>();

    private static ContextMenuEdit<SurveyorStationRecord> contextMenu(
            @NonNull FragmentManager fragmentManager,
            @NonNull LifecycleOwner lifecycleOwner,
            @NonNull IUIRunner iuiRunner,
            Context context,
            boolean editAble) {
        if (!editAble) {
            return null;
        }
        return new ContextMenuEdit<SurveyorStationRecord>() {
            @Override
            protected void edit(SurveyorStationRecord item) {
                if (SurveyorStationRecord.isLineWell(cachedId2LineType.get(item.getLineId()))) {
                    TipsWith2OptionFragment.tips(fragmentManager, lifecycleOwner,
                            context.getString(R.string.button_edit_station),
                            context.getString(R.string.button_delete_station),
                            context.getString(R.string.tips_select_default_station_operator_well),
                            new TipsWith2OptionFragment.SimpleCallback() {
                                @Override
                                public void onConfirm() {
                                    SurveyorStationUtil.editLine(context, item.getLineId());
                                }

                                @Override
                                public void onCancel() {
                                    try2DeleteLine(item.getLineId(), context, fragmentManager, lifecycleOwner, iuiRunner);
                                }
                            });
                    return;
                }


                //检测是否属于导线测量
                if (isLineStation(item.getLineId())) {
                    String lineId = item.getLineId();
                    TipsWith2OptionFragment.tips(fragmentManager, lifecycleOwner,
                            context.getString(R.string.button_edit_line),
                            context.getString(R.string.button_delete_line),
                            context.getString(R.string.tips_forbid_delete_station_because_batch),
                            new TipsWith2OptionFragment.SimpleCallback() {
                                @Override
                                public void onConfirm() {
                                    try2EditLine(lineId, context, fragmentManager, lifecycleOwner, iuiRunner);
                                }

                                @Override
                                public void onCancel() {
                                    try2DeleteLine(lineId, context, fragmentManager, lifecycleOwner, iuiRunner);
                                }
                            });
                } else {
                    TipsWith2OptionFragment.tips(fragmentManager, lifecycleOwner,
                            context.getString(R.string.button_edit_station),
                            context.getString(R.string.button_delete_station),
                            context.getString(R.string.tips_select_default_station_operator),
                            new TipsWith2OptionFragment.SimpleCallback() {
                                @Override
                                public void onConfirm() {
                                    ThreadPoolUtil.execute(() -> {
                                        String stationName = StationUtil.getStationName(item);
                                        String lineType = StationUtil.getLineTypeFromStation(item);
                                        SingleStationDetailAndEditFragment.start(context, item.getId(), stationName, item.getProjectId(), lineType);
                                    });
                                }

                                @Override
                                public void onCancel() {
                                    String stationId = item.getId();
                                    try2DeleteStation(stationId, context, fragmentManager, lifecycleOwner, iuiRunner);
                                }
                            });
                }
            }
        };
    }

    @WorkerThread
    private static boolean isStationHasData(String stationId) {
        ISurveyorRecordDataSource surveyorRecordDataSource =
                SurveyorDatabaseFactory.instance.getSurveyorRecordDataSource();
        return surveyorRecordDataSource.recordCount(stationId) > 0;
    }

    @WorkerThread
    private static boolean isLineHasData(String lineId) {
        boolean lineHasData = Tools.isLineHasData(lineId);
//        ISurveyorStationDataSource stationDataSource =
//                SurveyorDatabaseFactory.instance.getSurveyorStation();
//        List<SurveyorStationRecord> stationRecordList = stationDataSource.getDataListByLine(lineId);
//        if (stationRecordList == null || stationRecordList.isEmpty()) {
//            return false;
//        }
//        ISurveyorRecordDataSource surveyorRecordDataSource =
//                SurveyorDatabaseFactory.instance.getSurveyorRecordDataSource();
//        boolean hasData = false;
//        for (SurveyorStationRecord stationRecord : stationRecordList) {
//            if (stationRecord == null) {
//                continue;
//            }
//            if (surveyorRecordDataSource.recordCount(stationRecord.getId()) <= 0) {
//                continue;
//            }
//            hasData = true;
//            break;
//        }
        return lineHasData;
    }

    public static void try2DeleteLine(String lineId, Context context,
                                      FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                      @NonNull IUIRunner iuiRunner) {
        ThreadPoolUtil.execute(() -> {
            if (isLineHasData(lineId)) {
                iuiRunner.runOnUI(() -> TipsFragment.cancelAbleTips(fragmentManager, lifecycleOwner, context,
                        context.getString(R.string.tips_is_sure_to_delete_line_with_record),
                        4,
                        () -> SurveyorDatabaseFactory
                                .instance
                                .getSurveyorStation()
                                .deleteByLineId(lineId)));
            } else {
                SurveyorDatabaseFactory
                        .instance
                        .getSurveyorStation()
                        .deleteByLineId(lineId);
            }
        });
    }

    public static void try2EditLine(String lineId, Context context,
                                    FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                    @NonNull IUIRunner iuiRunner) {
        ThreadPoolUtil.execute(() -> {
            if (isLineHasData(lineId)) {
                iuiRunner.runOnUI(() -> TipsFragment.cancelAbleTips(fragmentManager, lifecycleOwner, context,
                        context.getString(R.string.tips_is_sure_to_edit_line_with_record),
                        4,
                        () -> SurveyorStationUtil.editLine(context, lineId)));
            } else {
                SurveyorStationUtil.editLine(context, lineId);
            }
        });
    }

    public static void try2DeleteStation(String stationId, Context context,
                                         FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                         @NonNull IUIRunner iuiRunner) {
        ThreadPoolUtil.execute(() -> {
            if (isStationHasData(stationId)) {
                iuiRunner.runOnUI(() -> TipsFragment.cancelAbleTips(fragmentManager, lifecycleOwner, context,
                        context.getString(R.string.tips_is_sure_to_delete_station_with_record),
                        4,
                        () -> SurveyorDatabaseFactory
                                .instance
                                .getSurveyorStation()
                                .deleteById(stationId)));
            } else {
                SurveyorDatabaseFactory
                        .instance
                        .getSurveyorStation()
                        .deleteById(stationId);
            }
        });
    }
}
