package com.bingce.controlnetwork.adapter;

import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IAdapter;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.IDistanceTypeChangeListener;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;

import org.jetbrains.annotations.NotNull;

import com.bingce.controlnetwork.ui.vh.SurveyorDataViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

/*
 * 测量记录adapter
 * */
public class SurveyorStationRecordsAdapter extends RecyclerView.Adapter<SurveyorDataViewHolder>
        implements IAdapter, IDistanceTypeChangeListener {
    private final ICachedData cachedData;
    private final ISurveyorData surveyorData;
    private final ICycleRecorder cycleRecorder;
    private boolean showSlopDistance = true;
    private final String stationId;

    public SurveyorStationRecordsAdapter(
            String stationId,
            ICachedData cachedData,
            ICycleRecorder cycleRecorder,
            ISurveyorData surveyorData) {
        this.stationId = stationId;
        this.cachedData = cachedData;
        this.surveyorData = surveyorData;
        this.cycleRecorder = cycleRecorder;
        //注册数据监听
        cachedData.register(this);
    }

    @NonNull
    @NotNull
    @Override
    public SurveyorDataViewHolder onCreateViewHolder(@NonNull @NotNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh = ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.surveyorStationPointItem);
        assert vh != null;
        return (SurveyorDataViewHolder) vh;
    }

    @Override
    public void onBindViewHolder(@NonNull @NotNull SurveyorDataViewHolder holder, int position) {
        holder.setup(position,
                stationId,
                showSlopDistance,
                cachedData,
                cycleRecorder,
                surveyorData);
    }

    @Override
    public int getItemCount() {
        return cachedData.pointCount();
    }

    @Override
    public void onDataChange() {
        notifyItemRangeChanged(0, getItemCount());
    }

    @Override
    public void onShowSlopDistance(boolean value) {
        this.showSlopDistance = value;
    }
}
