package com.bingce.controlnetwork.adapter;

import android.view.ViewGroup;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IUniversalListItemWithTitleAndSubtitle;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.TitleWithSubtitleViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

import org.jetbrains.annotations.NotNull;

import java.util.List;

public class TitleWithSubtitleAdapter<T extends IUniversalListItemWithTitleAndSubtitle<?>>
        extends RecyclerView.Adapter<TitleWithSubtitleViewHolder> {

    private final List<T> mValues;
    private final TitleWithSubtitleAdapter.IOnItemClick<T> callback;

    public TitleWithSubtitleAdapter(List<T> items, TitleWithSubtitleAdapter.IOnItemClick<T> callback) {
        mValues = items;
        this.callback = callback;
    }

    public void updateData(List<T> list) {
        if (mValues != null) {
            mValues.clear();
        }
        if (list != null) {
            mValues.addAll(list);
        }
        notifyDataSetChanged();
    }

    protected RecyclerView.ViewHolder createVH(@NotNull ViewGroup parent, int viewType) {
        return ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.titleWithSubtitle);
    }

    @Override
    public @NotNull
    TitleWithSubtitleViewHolder onCreateViewHolder(@NotNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder viewHolder = createVH(parent, viewType);
        assert viewHolder != null;
        return (TitleWithSubtitleViewHolder) viewHolder;
    }

    protected T getItem(int position) {
        return mValues.get(position);
    }

    @Override
    public void onBindViewHolder(final TitleWithSubtitleViewHolder holder, int position) {
        T item = getItem(position);
        holder.title.setText(item.getTitle());
        holder.subTitle.setText(item.getSubtitle());
        holder.itemView.setOnClickListener(v -> {
            if (callback != null) {
                callback.onItemClick(item);
            }
        });
    }

    @Override
    public int getItemCount() {
        if (mValues == null) {
            return 0;
        }
        return mValues.size();
    }

    public interface IOnItemClick<T extends IUniversalListItemWithTitleAndSubtitle<?>> {
        void onItemClick(T item);
    }
}