package com.bingce.controlnetwork.adapter;

import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.interfacee.IProjectTolerance;
import com.bingce.controlapphelper.model.UniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.util.NumUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.ui.vh.GroupParentViewHolder;
import com.bingce.controlnetwork.ui.vh.KeyBoolViewHolder;
import com.bingce.controlnetwork.ui.vh.KeyValueViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

import java.util.Arrays;
import java.util.List;

/*
 * 限差详情adapter
 * */
public class ToleranceDetailAdapter extends BaseExpandableListAdapter {
    private IProjectTolerance data;
    private static final List<ViewItemType> childTypes =
            Arrays.asList(
                    ViewItemType.nameWithBool,
                    ViewItemType.nameWithText);

    public ToleranceDetailAdapter(IProjectTolerance limit) {
        data = limit;
    }

    public void updateData(IProjectTolerance limit) {
        this.data = limit;
        notifyDataSetChanged();
    }

    @Override
    public int getGroupCount() {
        return data.groupCount();
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        return data.childrenCount(groupPosition);
    }

    @Override
    public String getGroup(int groupPosition) {
        return data.groupName(groupPosition);
    }

    @Override
    public UniversalListItem<?> getChild(int groupPosition, int childPosition) {
        return data.child(groupPosition, childPosition);
    }

    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return groupPosition * 10L + childPosition;
    }

    @Override
    public boolean hasStableIds() {
        return true;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        GroupParentViewHolder viewHolder = null;
        if (convertView == null) {
            convertView = ViewFactory.generatorView(parent.getContext(), parent, ViewItemType.groupParent);
            viewHolder = new GroupParentViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (GroupParentViewHolder) convertView.getTag();
        }
        if (viewHolder != null) {
            String groupName = getGroup(groupPosition);
            if (viewHolder.name != null) {
                viewHolder.name.setText(groupName);
            }
            if (viewHolder.expandIcon != null) {
                viewHolder.expandIcon.setImageResource(isExpanded ? R.drawable.ic_arrow_up : R.drawable.ic_arrow_down);
            }
        }
        return convertView;
    }

    @Override
    public int getChildTypeCount() {
        return childTypes.size();
    }

    @Override
    public int getChildType(int groupPosition, int childPosition) {
        return childTypes.indexOf(getChild(groupPosition, childPosition).getType());
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        UniversalListItem<?> item = getChild(groupPosition, childPosition);
        RecyclerView.ViewHolder viewHolder = null;
        if (convertView == null) {
            viewHolder = ViewFactory.generatorViewHolder(parent.getContext(), parent, item.getType());
            convertView = viewHolder.itemView;
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (RecyclerView.ViewHolder) convertView.getTag();
        }
        if (viewHolder instanceof KeyValueViewHolder) {
            KeyValueViewHolder kvHolder = (KeyValueViewHolder) viewHolder;
            kvHolder.key.setText(item.getName());
            kvHolder.value.setText(NumUtil.formatScientificNotation(item.getValue().toString()));
        } else if (viewHolder instanceof KeyBoolViewHolder) {
            KeyBoolViewHolder kvHolder = (KeyBoolViewHolder) viewHolder;
            kvHolder.key.setText(item.getName());
            kvHolder.checkBox.setChecked((Boolean) item.getValue());
        }
        return convertView;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }

}
