package com.bingce.controlnetwork.adapter.contextmenu;

import android.content.Context;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.fragment.StringInputFragment;
import com.bingce.utils.ThreadPoolUtil;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.StringInputFragment;

public class SurveyorPeriodAdapterContextMenu extends ContextMenuDeleteEdit<SurveyorPeriodRecord> {
    private @NonNull
    final LifecycleOwner lifecycleOwner;
    private @NonNull
    final FragmentManager fragmentManager;
    private final String selectedId;
    private final Context context;

    public SurveyorPeriodAdapterContextMenu(Context context, String selectedId,
                                            @NonNull LifecycleOwner lifecycleOwner,
                                            @NonNull FragmentManager fragmentManager) {
        this.context = context;
        this.selectedId = selectedId;
        this.lifecycleOwner = lifecycleOwner;
        this.fragmentManager = fragmentManager;
    }

    @Override
    protected void delete(SurveyorPeriodRecord item) {
        String id = item.getId();
        if (selectedId.equals(id)) {
            ToastUtils.showShort(Tools.getString(R.string.current_period_selected_not_delete));
        } else {
            SurveyorDatabaseFactory
                    .instance
                    .getSurveyorPeriod()
                    .deleteById(id);
            //检测是否需要修改默认测期
            ThreadPoolUtil.execute(new Runnable() {
                @Override
                public void run() {
                    IConfigDataSource configDataSource =
                            SurveyorDatabaseFactory
                                    .instance.getConfigDataSource();
                    ConfigRecord configRecord = configDataSource.getByKeySync(ConfigConstants.KEY_DEFAULT_PERIOD_ID);
                    if (configRecord != null && id.equals(configRecord.getConfigValue())) {
                        configDataSource.set(ConfigConstants.KEY_DEFAULT_PERIOD_ID, selectedId);
                    }
                }
            });
        }
    }

    @Override
    protected void edit(SurveyorPeriodRecord item) {
        StringInputFragment
                .input(
                        fragmentManager,
                        Tools.getString(R.string.modify_period_name),
                        item.getName(),
                        lifecycleOwner,
                        string -> {
                            if (string != null && !string.isEmpty()) {
                                item.setName(string);
                                SurveyorDatabaseFactory
                                        .instance
                                        .getSurveyorPeriod()
                                        .saveRecord(item);
                            }
                        });
    }
}
