package com.bingce.controlnetwork.adapter.stationdetail;

import android.text.TextUtils;
import android.view.View;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.DraggableTextViewHolder;
import com.bingce.controlnetwork.ui.vh.StationGroupItemViewHolder;
import com.bingce.controlnetwork.ui.widget.recyclerview.AbstractDraggableRecycleViewAdapter;

import java.util.ArrayList;
import java.util.List;

public abstract class BaseStationDetailAdapter extends AbstractDraggableRecycleViewAdapter<StationDetailItem> {
    private final boolean adjustable;
    private final IOnSelectPointDelegate selectPointCallback;

    protected BaseStationDetailAdapter(IOnSelectPointDelegate selectPointCallback,
                                       List<StationDetailItem> items,
                                       boolean adjustable) {
        super(items);
        this.adjustable = adjustable;
        this.selectPointCallback = selectPointCallback;
    }

    public void setStationPoint(String id, String name) {
        StationDetailItem item = getItem(1);
        if (item != null) {
            if (StationDetailItem.ITEM_TYPE_STATION_ITEM.equals(item.getType())) {
                setItem(1, new StationDetailItem(
                        name, StationDetailItem.ITEM_TYPE_STATION_ITEM, id));
                notifyItemChanged(1);
            } else {
                StationDetailItem newInstance = new StationDetailItem(
                        name, StationDetailItem.ITEM_TYPE_STATION_ITEM, id);
                insert(1, newInstance);
                notifyItemInserted(1);
            }
        }
    }

    public void setSurveyorPoint(List<String> ids, List<String> names) {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 0; index < ids.size(); index++) {
            StationDetailItem item = new StationDetailItem(
                    names.get(index), StationDetailItem.ITEM_TYPE_POINT_ITEM, ids.get(index));
            items.add(item);
        }
        if (items == null || items.isEmpty()) {
            return;
        }
        for (int i = getItemCount() - 1; i >= 0; i--) {
            StationDetailItem item = getItem(i);
            if (TextUtils.equals(item.getType(), StationDetailItem.ITEM_TYPE_POINT_ITEM)) {
                remove(item);
            }
        }
        insertAll(items);
        notifyDataSetChanged();
    }

    private StationDetailItem getStationItem() {
        StationDetailItem o = getItem(1);
        if (o != null && StationDetailItem.ITEM_TYPE_STATION_ITEM.equals(o.getType())) {
            return o;
        }
        return null;
    }

    @Override
    protected void bindItem(StationDetailItem item, RecyclerView.ViewHolder viewHolder) {
        if (viewHolder instanceof DraggableTextViewHolder) {
            DraggableTextViewHolder draggableTextViewHolder = (DraggableTextViewHolder) viewHolder;
            DraggableTextViewHolder.update(draggableTextViewHolder, item, adjustable);
        } else if (viewHolder instanceof StationGroupItemViewHolder) {
            StationGroupItemViewHolder surveyorStationSetupGroupParentViewHolder = ((StationGroupItemViewHolder) viewHolder);
            StationGroupItemViewHolder.update(
                    surveyorStationSetupGroupParentViewHolder,
                    item,
                    adjustable,
                    (itemType, surveyorStationSetupGroupParentViewHolder1) -> {
                        if (StationDetailItem.ITEM_TYPE_STATION_GROUP.equals(item.getType())) {
                            View.OnClickListener listener = v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    StationDetailItem stationItem = getStationItem();
                                    if (stationItem != null) {
                                        pickPointResult.getSelectedIds().add(stationItem.getOriginalPointId());
                                        pickPointResult.getSelectedNames().add(stationItem.getName());
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (int index = 2; index < getItemCount(); index++) {
                                        StationDetailItem obj = getItem(index);
                                        if (obj == null) {
                                            continue;
                                        }
                                        if (StationDetailItem.ITEM_TYPE_POINT_ITEM.equals(obj.getType())) {
                                            filters.add(obj.getOriginalPointId());
                                        }
                                    }
                                    selectPointCallback.onSelectStationPoint(pickPointResult, filters);
                                }
                            };
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(listener);
                            surveyorStationSetupGroupParentViewHolder1.switchView.setOnClickListener(listener);
                            StationDetailItem stationItem = getStationItem();
                            if (stationItem != null) {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.INVISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.VISIBLE);
                            } else {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.VISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.INVISIBLE);
                            }
                        } else if (StationDetailItem.ITEM_TYPE_POINT_GROUP.equals(item.getType())) {
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (int index = 2; index < getItemCount(); index++) {
                                        StationDetailItem o = getItem(index);
                                        if (o == null) {
                                            continue;
                                        }
                                        if (StationDetailItem.ITEM_TYPE_POINT_ITEM.equals(o.getType())) {
                                            pickPointResult.getSelectedIds().add(o.getOriginalPointId());
                                            pickPointResult.getSelectedNames().add(o.getName());
                                        }
                                    }
                                    String filterId = "";
                                    StationDetailItem filter = getStationItem();
                                    if (filter != null) {
                                        filterId = filter.getOriginalPointId();
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    if (!filterId.isEmpty()) {
                                        filters.add(filterId);
                                    }
                                    selectPointCallback.onSelectSurveyorPoint(pickPointResult, filters);
                                }
                            });
                        }
                    });
        }
    }

    @Override
    public int getItemViewType(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return -1;
        }
        if (StationDetailItem.ITEM_TYPE_POINT_GROUP.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_STATION_GROUP.equals(item.getType())) {
            return ViewItemType.surveyorStationCreateStationGroupItem.ordinal();
        }
        return super.getItemViewType(position);
    }

    @Override
    public boolean isItemSwipeAble(int position) {
        StationDetailItem item = getItem(position);
        if (!adjustable || item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_STATION_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_POINT_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_STATION_ITEM.equals(item.getType());
    }

    @Override
    public boolean isItemDragAble(int position) {
        StationDetailItem item = getItem(position);
        if (!adjustable || item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_STATION_ITEM.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_STATION_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_POINT_GROUP.equals(item.getType());
    }

    public List<SurveyorPoint> collectData() {
        List<SurveyorPoint> stationItems = new ArrayList<>();
        for (int index = 0; index < getItemCount(); index++) {
            StationDetailItem item = getItem(index);
            if (item == null) {
                continue;
            }
            if (StationDetailItem.ITEM_TYPE_STATION_ITEM.equals(item.getType())) {
                SurveyorPoint stationItem = new SurveyorPoint();
                stationItem.setOriginalPointId(item.getOriginalPointId());
                stationItem.setType(SurveyorPoint.TYPE_STATION);
                stationItems.add(stationItem);
            } else if (StationDetailItem.ITEM_TYPE_POINT_ITEM.equals(item.getType())) {
                SurveyorPoint stationItem = new SurveyorPoint();
                stationItem.setOriginalPointId(item.getOriginalPointId());
                stationItem.setType(SurveyorPoint.TYPE_POINT);
                stationItems.add(stationItem);
            }
        }
        return stationItems;
    }

    public int surveyorPointSize() {
        int size = getItemCount();
        int count = 0;
        for (int index = 0; index < size; index++) {
            StationDetailItem item = getItem(index);
            if (item == null || !StationDetailItem.ITEM_TYPE_POINT_ITEM.equals(item.getType())) {
                continue;
            }
            count++;
        }
        return count;
    }

    public interface IOnSelectPointDelegate {
        void onSelectStationPoint(PickPointResult pickPointResult, ArrayList<String> filters);

        void onSelectSurveyorPoint(PickPointResult pickPointResult, ArrayList<String> filters);
    }
}
