package com.bingce.controlnetwork.adapter.stationdetail;

import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.DraggableTextViewHolder;
import com.bingce.controlnetwork.ui.vh.StationGroupItemViewHolder;
import com.bingce.controlnetwork.ui.widget.recyclerview.AbstractDraggableRecycleViewAdapter;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * (复合、无定向)导线创建  known1-unknown-known2格式
 */
public class StationCreateLineKnownUnknownKnownAdapter extends AbstractDraggableRecycleViewAdapter<StationDetailItem> {
    private final IOnSelectPointDelegate selectPointCallback;

    public StationCreateLineKnownUnknownKnownAdapter(IOnSelectPointDelegate selectPointCallback,
                                                     List<StationDetailItem> items) {
        super(items);
        this.selectPointCallback = selectPointCallback;
    }

    public void setKnownPoints1(List<String> ids, List<String> names) {
        if (ids == null || names == null || ids.size() != names.size()) {
            return;
        }
        //先移除之前known point
        List<StationDetailItem> knownItems = knownItems1();
        for (StationDetailItem item : knownItems) {
            remove(item);
        }
        //添加新的known point
        for (int index = 0; index < ids.size(); index++) {
            insert(
                    1 + index,
                    new StationDetailItem(
                            names.get(index),
                            StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM1,
                            ids.get(index)));
        }
        //通知变化
        if (knownItems.size() == ids.size()) {
            notifyItemRangeChanged(1, ids.size());
        } else {
            notifyItemRangeChanged(1, getItemCount() - 1);
        }
    }

    public void setUnknownPoints(List<String> ids, List<String> names) {
        if (ids == null || names == null || ids.size() != names.size()) {
            return;
        }
        //先移除之前unknown point
        List<StationDetailItem> unknowns = unknownItems();
        for (StationDetailItem item : unknowns) {
            remove(item);
        }
        //搜索unknown-group索引
        int targetIndex = -1;
        for (int index = 0; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj != null && StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(obj.getType())) {
                targetIndex = index;
                break;
            }
        }
        if (targetIndex >= 0) {
            //添加新的unknown point
            for (int index = 0; index < ids.size(); index++) {
                insert(
                        targetIndex + 1 + index,
                        new StationDetailItem(
                                names.get(index),
                                StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM,
                                ids.get(index)));
            }
        }
        //通知变化
        if (unknowns.size() == ids.size()) {
            notifyItemRangeChanged(targetIndex + 1, ids.size());
        } else {
            notifyItemRangeChanged(targetIndex + 1, getItemCount() - 1);
        }
    }

    public void setKnownPoints2(List<String> ids, List<String> names) {
        if (ids == null || names == null || ids.size() != names.size()) {
            return;
        }
        //搜索known-group2索引
        int targetIndex = -1;
        for (int index = 0; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj != null && StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(obj.getType())) {
                targetIndex = index;
                break;
            }
        }
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 0; index < ids.size(); index++) {
            StationDetailItem item = new StationDetailItem(
                    names.get(index), StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM2, ids.get(index));
            items.add(item);
        }
        if (targetIndex > 0) {
            replace(targetIndex + 1, items);
            notifyItemRangeChanged(targetIndex + 1, items.size());
        }
    }

    public List<StationDetailItem> knownItems1() {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM1.equals(obj.getType())) {
                break;
            }
            items.add(obj);
        }
        return items;
    }

    public List<StationDetailItem> unknownItems() {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(obj.getType())) {
                continue;
            }
            items.add(obj);
        }
        return items;
    }

    public List<StationDetailItem> knownItems2() {
        List<StationDetailItem> items = new ArrayList<>();
        int targetIndex = -1;
        for (int index = getItemCount() - 1; index >= 0; index--) {
            StationDetailItem obj = getItem(index);
            if (obj != null && StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(obj.getType())) {
                targetIndex = index;
                break;
            }
        }
        if (targetIndex < 0) {
            return items;
        }
        for (int index = targetIndex + 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM2.equals(obj.getType())) {
                break;
            }
            items.add(obj);
        }
        return items;
    }

    @Override
    protected void bindItem(StationDetailItem item, RecyclerView.ViewHolder viewHolder) {
        if (viewHolder instanceof DraggableTextViewHolder) {
            DraggableTextViewHolder draggableTextViewHolder = (DraggableTextViewHolder) viewHolder;
            DraggableTextViewHolder.update(draggableTextViewHolder, item, true);
        } else if (viewHolder instanceof StationGroupItemViewHolder) {
            StationGroupItemViewHolder surveyorStationSetupGroupParentViewHolder = ((StationGroupItemViewHolder) viewHolder);
            StationGroupItemViewHolder.update(
                    surveyorStationSetupGroupParentViewHolder,
                    item,
                    true,
                    (itemType, surveyorStationSetupGroupParentViewHolder1) -> {
                        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1.equals(item.getType())) {
                            View.OnClickListener listener = v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (StationDetailItem obj : knownItems1()) {
                                        pickPointResult.getSelectedIds().add(obj.getOriginalPointId());
                                        pickPointResult.getSelectedNames().add(obj.getName());
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (StationDetailItem obj : unknownItems()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    for (StationDetailItem obj : knownItems2()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    selectPointCallback.onSelectKnownPoint1(pickPointResult, filters);
                                }
                            };
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(listener);
                            surveyorStationSetupGroupParentViewHolder1.switchView.setOnClickListener(listener);

                            List<StationDetailItem> knownItems = knownItems1();
                            if (knownItems.isEmpty()) {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.VISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.INVISIBLE);
                            } else {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.INVISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.VISIBLE);
                            }
                        } else if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType())) {
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (int index = 2; index < getItemCount(); index++) {
                                        StationDetailItem o = getItem(index);
                                        if (o == null) {
                                            continue;
                                        }
                                        if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(o.getType())) {
                                            pickPointResult.getSelectedIds().add(o.getOriginalPointId());
                                            pickPointResult.getSelectedNames().add(o.getName());
                                        }
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (StationDetailItem obj : knownItems1()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    for (StationDetailItem obj : knownItems2()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    selectPointCallback.onSelectUnknownPoint(pickPointResult, filters);
                                }
                            });
                        } else if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(item.getType())) {
                            View.OnClickListener listener = v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (StationDetailItem obj : knownItems2()) {
                                        pickPointResult.getSelectedIds().add(obj.getOriginalPointId());
                                        pickPointResult.getSelectedNames().add(obj.getName());
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (StationDetailItem obj : knownItems1()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    for (StationDetailItem obj : unknownItems()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    selectPointCallback.onSelectKnownPoint2(pickPointResult, filters);
                                }
                            };
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(listener);
                            surveyorStationSetupGroupParentViewHolder1.switchView.setOnClickListener(listener);

                            List<StationDetailItem> knownItems = knownItems2();
                            if (knownItems.isEmpty()) {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.VISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.INVISIBLE);
                            } else {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.INVISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.VISIBLE);
                            }
                        }
                    });
        }
    }

    private static final int ITEM_TYPE_KNOWN_POINT1 = 11;
    private static final int ITEM_TYPE_UNKNOWN_POINT = ITEM_TYPE_KNOWN_POINT1 + 1;
    private static final int ITEM_TYPE_KNOWN_POINT2 = ITEM_TYPE_UNKNOWN_POINT + 1;

    @Override
    public int getItemViewType(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return -1;
        }
        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(item.getType())) {
            return ViewItemType.surveyorStationCreateStationGroupItem.ordinal();
        }
        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM1.equals(item.getType())) {
            return ITEM_TYPE_KNOWN_POINT1;
        }
        if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(item.getType())) {
            return ITEM_TYPE_UNKNOWN_POINT;
        }
        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM2.equals(item.getType())) {
            return ITEM_TYPE_KNOWN_POINT2;
        }
        return super.getItemViewType(position);
    }

    @NonNull
    @NotNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull @NotNull ViewGroup parent, int viewType) {
        if (ITEM_TYPE_KNOWN_POINT1 == viewType || ITEM_TYPE_UNKNOWN_POINT == viewType || ITEM_TYPE_KNOWN_POINT2 == viewType) {
            return super.onCreateViewHolder(parent, ViewItemType.draggableItem.ordinal());
        }
        return super.onCreateViewHolder(parent, viewType);
    }

    @Override
    public boolean isItemSwipeAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(item.getType());
    }

    @Override
    public boolean isItemDragAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2.equals(item.getType());
    }

    public interface IOnSelectPointDelegate {
        void onSelectKnownPoint1(PickPointResult pickPointResult, ArrayList<String> filters);

        void onSelectUnknownPoint(PickPointResult pickPointResult, ArrayList<String> filters);

        void onSelectKnownPoint2(PickPointResult pickPointResult, ArrayList<String> filters);
    }
}
