package com.bingce.controlnetwork.adapter.stationdetail;

import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.DraggableTextViewHolder;
import com.bingce.controlnetwork.ui.vh.StationGroupItemViewHolder;
import com.bingce.controlnetwork.ui.widget.recyclerview.AbstractDraggableRecycleViewAdapter;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * 支导线创建
 */
public class StationCreateLineSingleAdapter extends AbstractDraggableRecycleViewAdapter<StationDetailItem> {
    private final IOnSelectPointDelegate selectPointCallback;

    public StationCreateLineSingleAdapter(IOnSelectPointDelegate selectPointCallback,
                                          List<StationDetailItem> items) {
        super(items);
        this.selectPointCallback = selectPointCallback;
    }

    public void setKnownPoints(List<String> ids, List<String> names) {
        if (ids == null || names == null || ids.size() != names.size()) {
            return;
        }
        //先移除之前known point
        List<StationDetailItem> knownItems = knownItems();
        for (StationDetailItem item : knownItems) {
            remove(item);
        }
        //添加新的known point
        for (int index = 0; index < ids.size(); index++) {
            insert(
                    1 + index,
                    new StationDetailItem(
                            names.get(index),
                            StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM,
                            ids.get(index)));
        }
        //通知变化
        if (knownItems.size() == ids.size()) {
            notifyItemRangeChanged(1, ids.size());
        } else {
            notifyItemRangeChanged(1, getItemCount() - 1);
        }
    }

    public void setUnknownPoints(List<String> ids, List<String> names) {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 0; index < ids.size(); index++) {
            StationDetailItem item = new StationDetailItem(
                    names.get(index), StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM, ids.get(index));
            items.add(item);
        }

        int index = -1;
        for (int i = 0; i < getItemCount(); i++) {
            StationDetailItem item = getItem(i);
            if (item == null || !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType())) {
                continue;
            }
            index = i;
            break;
        }
        if (index > 0) {
            replace(index + 1, items);
            notifyItemRangeChanged(index + 1, items.size());
        }
    }

    public List<StationDetailItem> knownItems() {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM.equals(obj.getType())) {
                break;
            }
            items.add(obj);
        }
        return items;
    }

    public List<StationDetailItem> unknownItems() {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(obj.getType())) {
                continue;
            }
            items.add(obj);
        }
        return items;
    }

    @Override
    protected void bindItem(StationDetailItem item, RecyclerView.ViewHolder viewHolder) {
        if (viewHolder instanceof DraggableTextViewHolder) {
            DraggableTextViewHolder draggableTextViewHolder = (DraggableTextViewHolder) viewHolder;
            DraggableTextViewHolder.update(draggableTextViewHolder, item, true);
        } else if (viewHolder instanceof StationGroupItemViewHolder) {
            StationGroupItemViewHolder surveyorStationSetupGroupParentViewHolder = ((StationGroupItemViewHolder) viewHolder);
            StationGroupItemViewHolder.update(
                    surveyorStationSetupGroupParentViewHolder,
                    item,
                    true,
                    (itemType, surveyorStationSetupGroupParentViewHolder1) -> {
                        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP.equals(item.getType())) {
                            View.OnClickListener listener = v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (StationDetailItem obj : knownItems()) {
                                        pickPointResult.getSelectedIds().add(obj.getOriginalPointId());
                                        pickPointResult.getSelectedNames().add(obj.getName());
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (int index = 2; index < getItemCount(); index++) {
                                        StationDetailItem obj = getItem(index);
                                        if (obj == null) {
                                            continue;
                                        }
                                        if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(obj.getType())) {
                                            filters.add(obj.getOriginalPointId());
                                        }
                                    }
                                    selectPointCallback.onSelectKnownPoint(pickPointResult, filters);
                                }
                            };
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(listener);
                            surveyorStationSetupGroupParentViewHolder1.switchView.setOnClickListener(listener);

                            List<StationDetailItem> knownItems = knownItems();
                            if (knownItems.isEmpty()) {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.VISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.INVISIBLE);
                            } else {
                                surveyorStationSetupGroupParentViewHolder1.pick.setVisibility(View.INVISIBLE);
                                surveyorStationSetupGroupParentViewHolder1.switchView.setVisibility(View.VISIBLE);
                            }
                        } else if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType())) {
                            surveyorStationSetupGroupParentViewHolder1.pick.setOnClickListener(v -> {
                                if (selectPointCallback != null) {
                                    //收集信息
                                    PickPointResult pickPointResult = new PickPointResult();
                                    for (int index = 2; index < getItemCount(); index++) {
                                        StationDetailItem o = getItem(index);
                                        if (o == null) {
                                            continue;
                                        }
                                        if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(o.getType())) {
                                            pickPointResult.getSelectedIds().add(o.getOriginalPointId());
                                            pickPointResult.getSelectedNames().add(o.getName());
                                        }
                                    }
                                    ArrayList<String> filters = new ArrayList<>();
                                    for (StationDetailItem obj : knownItems()) {
                                        if (obj == null) {
                                            continue;
                                        }
                                        filters.add(obj.getOriginalPointId());
                                    }
                                    selectPointCallback.onSelectUnknownPoint(pickPointResult, filters);
                                }
                            });
                        }
                    });
        }
    }

    private static final int ITEM_TYPE_KNOWN_POINT = 11;
    private static final int ITEM_TYPE_UNKNOWN_POINT = ITEM_TYPE_KNOWN_POINT + 1;

    @Override
    public int getItemViewType(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return -1;
        }
        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType())) {
            return ViewItemType.surveyorStationCreateStationGroupItem.ordinal();
        }
        if (StationDetailItem.ITEM_TYPE_KNOWN_POINT_ITEM.equals(item.getType())) {
            return ITEM_TYPE_KNOWN_POINT;
        }
        if (StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_ITEM.equals(item.getType())) {
            return ITEM_TYPE_UNKNOWN_POINT;
        }
        return super.getItemViewType(position);
    }

    @NonNull
    @NotNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull @NotNull ViewGroup parent, int viewType) {
        if (ITEM_TYPE_KNOWN_POINT == viewType || ITEM_TYPE_UNKNOWN_POINT == viewType) {
            return super.onCreateViewHolder(parent, ViewItemType.draggableItem.ordinal());
        }
        return super.onCreateViewHolder(parent, viewType);
    }

    @Override
    public boolean isItemSwipeAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType());
    }

    @Override
    public boolean isItemDragAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP.equals(item.getType());
    }

    public interface IOnSelectPointDelegate {
        void onSelectKnownPoint(PickPointResult pickPointResult, ArrayList<String> filters);

        void onSelectUnknownPoint(PickPointResult pickPointResult, ArrayList<String> filters);
    }
}
