package com.bingce.controlnetwork.adapter.stationdetail;

import android.text.TextUtils;
import android.view.View;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.ItemWellViewHolder;
import com.bingce.controlnetwork.ui.vh.StationGroupItemViewHolder;
import com.bingce.controlnetwork.ui.widget.recyclerview.AbstractDraggableRecycleViewAdapter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 一井定向-创建站选点的适配器
 */
public class StationCreateLineWellAdapter extends AbstractDraggableRecycleViewAdapter<StationDetailItem> {
    private final IOnSelectPointDelegate selectPointCallback;

    private final Map<String, Integer> groupPositionMap = new HashMap<>();
    private final Map<String, PickPointResult> lastSelectMap = new HashMap<>();

    public StationCreateLineWellAdapter(IOnSelectPointDelegate selectPointCallback,
                                        List<StationDetailItem> items) {
        super(items);
        for (int i = 0; i < items.size(); i++) {
            groupPositionMap.put(items.get(i).getType(), i);
        }
        this.selectPointCallback = selectPointCallback;
    }

    /**
     * 设置地面连接方向点数据
     */
    public void setGroundLinkPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK,
                StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK + StationDetailItem._DATA);
    }

    private void setPointsData(PickPointResult pickPointResult, String groupType, String dataType) {
        if (pickPointResult == null || pickPointResult.getSelectedIds() == null
                || pickPointResult.getSelectedNames() == null
                || pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedNames().size()) {
            return;
        }
        //记录选择的
        lastSelectMap.put(groupType, pickPointResult);

        List<StationDetailItem> pointsOld = getPointsFromType(dataType);
        removeAll(pointsOld);

        List<StationDetailItem> newList = new ArrayList<>();
        for (int index = 0; index < pickPointResult.getSelectedIds().size(); index++) {
            newList.add(new StationDetailItem(
                    pickPointResult.getSelectedNames().get(index),
                    dataType,
                    pickPointResult.getSelectedIds().get(index)));
        }

        int insertPosition = groupPositionMap.get(groupType);
        insertForWell(insertPosition + 1, newList);

        updateGroupPositionMap();

        notifyItemRangeChanged(0, getItemCount());

    }

    /**
     * 更新group的基准位置
     */
    private void updateGroupPositionMap() {
        for (int i = 0; i < getItemCount(); i++) {
            String type = getItem(i).getType();
            if (type.equals(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK)
                    || type.equals(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE)
                    || type.equals(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE)
                    || type.equals(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE)
                    || type.equals(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK)) {
                groupPositionMap.put(type, i);
            }
        }
    }

    /**
     * 设置地面近井点数据
     */
    public void setGroundSidePoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE,
                StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE + StationDetailItem._DATA);
    }

    /**
     * 设置钢丝点数据
     */
    public void setSteelWirePoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE,
                StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE + StationDetailItem._DATA);
    }

    /**
     * 设置地下近井点数据
     */
    public void setUndergroudSidePoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE,
                StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE + StationDetailItem._DATA);
    }

    /**
     * 设置地下连接方向点数据
     */
    public void setUndergroudLinkPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK,
                StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK + StationDetailItem._DATA);
    }


    public List<StationDetailItem> getPointsFromType(String dataType) {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !dataType.equals(obj.getType())) {
                continue;
            }

            if (dataType.contains(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE)) {
                obj.setWellSteel(true);
            }

            items.add(obj);
        }
        return items;
    }

    @Override
    protected void bindItem(StationDetailItem item, RecyclerView.ViewHolder viewHolder) {
        if (viewHolder instanceof ItemWellViewHolder) {
            ItemWellViewHolder itemWellViewHolder = (ItemWellViewHolder) viewHolder;
            ItemWellViewHolder.update(itemWellViewHolder, item, true, (itemType, itemWellViewHolderBack) -> {
//                itemWellViewHolder.switchCompat.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
//                    @Override
//                    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
//                        item.setWellSteel(isChecked);
//                        if (selectPointCallback != null) {
//                            selectPointCallback.onSelectOnlyMeasureHorAngle(isChecked);
//                        }
//                    }
//                });
            });
        } else if (viewHolder instanceof StationGroupItemViewHolder) {
            StationGroupItemViewHolder stationGroupItemViewHolder = ((StationGroupItemViewHolder) viewHolder);

            StationGroupItemViewHolder.update(
                    stationGroupItemViewHolder,
                    item,
                    true,
                    (itemType, stationGroupItemViewHolderBack) -> {
                        stationGroupItemViewHolder.pick.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                backToData(itemType, stationGroupItemViewHolderBack);
                            }
                        });
                    });
        }
    }

    @Override
    public int getItemViewType(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return -1;
        }
        if (StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE.equals(item.getType()) ||
                StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK.equals(item.getType())) {
            return ViewItemType.surveyorStationCreateStationGroupItem.ordinal();
        } else {
            return ViewItemType.wellItem.ordinal();
        }
    }

    @Override
    public boolean isItemSwipeAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE.equals(item.getType())
                && !StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE.equals(item.getType())
                && !StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK.equals(item.getType());
    }

    @Override
    public boolean isItemDragAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE.equals(item.getType()) &&
                !StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE.equals(item.getType())
                && !StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE.equals(item.getType())
                && !StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK.equals(item.getType());
    }


    public void removeItemForLastSelect(int adapterPosition) {
        StationDetailItem item = getItem(adapterPosition);
        assert item != null;
        String groupType = item.getType().replaceAll(StationDetailItem._DATA, "");
        PickPointResult pickPointResult = lastSelectMap.get(groupType);
        if (pickPointResult != null && !pickPointResult.getSelectedIds().isEmpty()) {
            int index = pickPointResult.getSelectedIds().indexOf(item.getOriginalPointId());
            pickPointResult.getSelectedIds().remove(index);
            pickPointResult.getSelectedNames().remove(index);
        } else {
            lastSelectMap.remove(groupType);
        }
        updateGroupPositionMap();
// 原先只有这一句       lastSelectMap.put(item.getType().replaceAll(StationDetailItem._DATA, ""), null);
    }

    public interface IOnSelectPointDelegate {
        void onSelectGroundLink(PickPointResult pickPointResult, List<String> filters);

        void onSelectGroundSide(PickPointResult pickPointResult, List<String> filters);

        void onSelectSteelWire(PickPointResult pickPointResult, List<String> filters);

        void onSelectUndergroundSide(PickPointResult pickPointResult, List<String> filters);

        void onSelectUndergroundLink(PickPointResult pickPointResult, List<String> filters);

//        void onSelectOnlyMeasureHorAngle(boolean measureHorAngle);
    }

    /**
     * 返回数据给上一级
     *
     * @param itemType
     * @param holder
     */
    private void backToData(String itemType, StationGroupItemViewHolder holder) {
        if (selectPointCallback == null) {
            return;
        }

        PickPointResult pickPointResult = lastSelectMap.get(itemType);

        List<String> filterIds = getFilterIds(pickPointResult);

        switch (itemType) {
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK:
                selectPointCallback.onSelectGroundLink(pickPointResult, filterIds);
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE:
                selectPointCallback.onSelectGroundSide(pickPointResult, filterIds);
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE:
                selectPointCallback.onSelectSteelWire(pickPointResult, filterIds);
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE:
                selectPointCallback.onSelectUndergroundSide(pickPointResult, filterIds);
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK:
                selectPointCallback.onSelectUndergroundLink(pickPointResult, filterIds);
                break;
        }
    }

    private List<String> getFilterIds(PickPointResult pickPointResult) {
        List<String> filterIds = new ArrayList<>();
        for (int i = 0; i < getItemCount(); i++) {
            StationDetailItem item = getItem(i);
            String pointId = item.getOriginalPointId();

            if (!TextUtils.isEmpty(pointId)) {
                if (pickPointResult == null) {
                    filterIds.add(pointId);
                } else {
                    if (!pickPointResult.getSelectedIds().contains(pointId)) {
                        filterIds.add(pointId);
                    }
                }
            }
        }
        return filterIds;
    }

}
