package com.bingce.controlnetwork.adapter.stationdetail;

import android.os.Handler;
import android.text.TextUtils;
import android.view.View;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.ui.vh.ItemWellViewHolder;
import com.bingce.controlnetwork.ui.vh.StationGroupItemViewHolder;
import com.bingce.controlnetwork.ui.widget.recyclerview.AbstractDraggableRecycleViewAdapter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 二井定向-地下-创建站选点的适配器
 */
public class StationCreateLineWellDoubleUnderGroundAdapter extends AbstractDraggableRecycleViewAdapter<StationDetailItem> {
    private final IOnSelectPointDelegate selectPointCallback;

    private final Map<String, Integer> groupPositionMap = new HashMap<>();
    private final Map<String, PickPointResult> lastSelectMap = new HashMap<>();

    public StationCreateLineWellDoubleUnderGroundAdapter(IOnSelectPointDelegate selectPointCallback,
                                                         List<StationDetailItem> items) {
        super(items);
        for (int i = 0; i < items.size(); i++) {
            groupPositionMap.put(items.get(i).getType(), i);
        }
        this.selectPointCallback = selectPointCallback;
    }

    private void setPointsData(PickPointResult pickPointResult, String groupType, String dataType) {
        if (pickPointResult == null || pickPointResult.getSelectedIds() == null
                || pickPointResult.getSelectedNames() == null
                || pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedNames().size()) {
            return;
        }
        //记录选择的
        lastSelectMap.put(groupType, pickPointResult);

        List<StationDetailItem> pointsOld = getPointsFromType(dataType);
        removeAll(pointsOld);

        List<StationDetailItem> newList = new ArrayList<>();
        for (int index = 0; index < pickPointResult.getSelectedIds().size(); index++) {
            newList.add(new StationDetailItem(
                    pickPointResult.getSelectedNames().get(index),
                    dataType,
                    pickPointResult.getSelectedIds().get(index)));
        }

        int insertPosition = groupPositionMap.get(groupType);
        insertForWell(insertPosition + 1, newList);
        updateGroupPositionMap();
        notifyItemRangeChanged(0, getItemCount());
    }

    /**
     * 更新group的基准位置
     */
    private void updateGroupPositionMap() {
        for (int i = 0; i < getItemCount(); i++) {
            String type = getItem(i).getType();
            if (type.equals(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A)
                    || type.equals(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B)) {
                groupPositionMap.put(type, i);
            }
        }
    }

    /**
     * 设置地面连接点 左
     */
    public void setGroundLinkLeftPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT,
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT + StationDetailItem._DATA);
    }

    /**
     * 设置地面连接点 右
     */
    public void setGroundLinkRightPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT,
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT + StationDetailItem._DATA);
    }

    /**
     * 设置地面近井点 A
     */
    public void setGroundSideAPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A,
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A + StationDetailItem._DATA);
    }


    /**
     * 设置地面近井点 B
     */
    public void setGroundSideBPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B,
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B + StationDetailItem._DATA);
    }

    /**
     * 设置钢丝A
     */
    public void setSteelAPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A,
                StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A + StationDetailItem._DATA);
    }

    /**
     * 设置钢丝B
     */
    public void setSteelBPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B,
                StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B + StationDetailItem._DATA);
    }

    /**
     * 设置地下连接方向
     */
    public void setUnderGroundLinkPoints(PickPointResult pickPointResult) {
        setPointsData(pickPointResult, StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK,
                StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK + StationDetailItem._DATA);
    }


    public List<StationDetailItem> getPointsFromType(String dataType) {
        List<StationDetailItem> items = new ArrayList<>();
        for (int index = 1; index < getItemCount(); index++) {
            StationDetailItem obj = getItem(index);
            if (obj == null || !dataType.equals(obj.getType())) {
                continue;
            }

            if (dataType.contains(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE)) {
                //注意是contains
                obj.setWellSteel(true);
            }

            items.add(obj);
        }
        return items;
    }

    @Override
    protected void bindItem(StationDetailItem item, RecyclerView.ViewHolder viewHolder) {
        if (viewHolder instanceof ItemWellViewHolder) {
            ItemWellViewHolder itemWellViewHolder = (ItemWellViewHolder) viewHolder;
            ItemWellViewHolder.update(itemWellViewHolder, item, true, (itemType, itemWellViewHolderBack) -> {
//                itemWellViewHolder.switchCompat.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
//                    @Override
//                    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
//                        item.setWellSteel(isChecked);
//                        if (selectPointCallback != null) {
//                            selectPointCallback.onSelectOnlyMeasureHorAngle(isChecked);
//                        }
//                    }
//                });
            });
        } else if (viewHolder instanceof StationGroupItemViewHolder) {
            StationGroupItemViewHolder stationGroupItemViewHolder = ((StationGroupItemViewHolder) viewHolder);

            StationGroupItemViewHolder.update(
                    stationGroupItemViewHolder,
                    item,
                    true,
                    (itemType, stationGroupItemViewHolderBack) -> {
                        stationGroupItemViewHolder.pick.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                backToData(itemType, stationGroupItemViewHolderBack);
                            }
                        });
                    });
        }
    }

    @Override
    public int getItemViewType(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return -1;
        }
        if (StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B.equals(item.getType()) ||
                StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK.equals(item.getType())
        ) {
            return ViewItemType.surveyorStationCreateStationGroupItem.ordinal();
        } else {
            return ViewItemType.wellItem.ordinal();
        }
    }

    @Override
    public boolean isItemSwipeAble(int position) {
        return isItemSwipeAbleOrItemDragAble(position);
    }

    @Override
    public boolean isItemDragAble(int position) {
        return isItemSwipeAbleOrItemDragAble(position);
    }

    private boolean isItemSwipeAbleOrItemDragAble(int position) {
        StationDetailItem item = getItem(position);
        if (item == null) {
            return false;
        }
        return !StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B.equals(item.getType())
                && !StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK.equals(item.getType());
    }

    public void removeItemForLastSelect(int adapterPosition) {
        StationDetailItem item = getItem(adapterPosition);
        assert item != null;
        String groupType = item.getType().replaceAll(StationDetailItem._DATA, "");
        PickPointResult pickPointResult = lastSelectMap.get(groupType);
        if (pickPointResult != null && !pickPointResult.getSelectedIds().isEmpty()) {
            int index = pickPointResult.getSelectedIds().indexOf(item.getOriginalPointId());
            pickPointResult.getSelectedIds().remove(index);
            pickPointResult.getSelectedNames().remove(index);
        } else {
            lastSelectMap.remove(groupType);
        }
        //延迟更新groupType的位置
        new Handler().postDelayed(() -> updateGroupPositionMap(), 300);
    }

    public interface IOnSelectPointDelegate {
        void onSelectGroundLinkLeft(PickPointResult pickPointResult, List<String> filters);

        void onSelectGroundLinkRight(PickPointResult pickPointResult, List<String> filters);

        void onSelectGroundSideA(PickPointResult pickPointResult, List<String> filters);

        void onSelectGroundSideB(PickPointResult pickPointResult, List<String> filters);

        void onSelectSteelA(PickPointResult pickPointResult, List<String> filters);

        void onSelectSteelB(PickPointResult pickPointResult, List<String> filters);

        void onSelectUnderGroundLink(PickPointResult pickPointResult, List<String> filters);

        void onSelectOnlyMeasureHorAngle(boolean measureHorAngle);

    }

    /**
     * 返回数据给上一级
     *
     * @param itemType
     * @param holder
     */
    private void backToData(String itemType, StationGroupItemViewHolder holder) {
        if (selectPointCallback == null) {
            return;
        }

        PickPointResult pickPointResult = lastSelectMap.get(itemType);

        List<String> filterIds = getFilterIds(pickPointResult);

        switch (itemType) {
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT:
                selectPointCallback.onSelectGroundLinkLeft(pickPointResult, filterIds);
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT:
                selectPointCallback.onSelectGroundLinkRight(pickPointResult, filterIds);
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A:
                selectPointCallback.onSelectGroundSideA(pickPointResult, filterIds);
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B:
                selectPointCallback.onSelectGroundSideB(pickPointResult, filterIds);
                break;

            case StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A:
                selectPointCallback.onSelectSteelA(pickPointResult, filterIds);
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B:
                selectPointCallback.onSelectSteelB(pickPointResult, filterIds);
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK:
                selectPointCallback.onSelectUnderGroundLink(pickPointResult, filterIds);
                break;
        }
    }

    private List<String> getFilterIds(PickPointResult pickPointResult) {
        List<String> filterIds = new ArrayList<>();
        for (int i = 0; i < getItemCount(); i++) {
            StationDetailItem item = getItem(i);
            String pointId = item.getOriginalPointId();

            if (!TextUtils.isEmpty(pointId)) {
                if (pickPointResult == null) {
                    filterIds.add(pointId);
                } else {
                    if (!pickPointResult.getSelectedIds().contains(pointId)) {
                        filterIds.add(pointId);
                    }
                }
            }
        }
        return filterIds;
    }

}
