package com.bingce.controlnetwork.fragment;

import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;

import com.bingce.utils.SoftKeyUtils;

import com.bingce.controlnetwork.R;

public abstract class AbstractFullScreenDialogFragment extends AppCompatDialogFragment {

    private View viewSpace;
    private View confirm;

    @Nullable
    @Override
    final public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        getDialog().getWindow().requestFeature(Window.FEATURE_NO_TITLE);
        View contentView = inflater.inflate(R.layout.fragment_fullscreen_dialog_layout, container, false);
        FrameLayout contentContainer = contentView.findViewById(R.id.content_container);
        contentView.setOnClickListener(v -> {
            if (quitWhenTouchBg()) {
                doDismiss();
            } else {
                hideSoft();
            }
        });

        if (getArguments() != null) {
            TextView title = contentView.findViewById(R.id.title);
            if (title != null) {
                String string = getArguments().getString(KEY_TITLE_NAME);
                if (string == null || string.isEmpty()) {
                    title.setVisibility(View.GONE);
                } else {
                    title.setText(string);
                }
            }
            TextView subTitle = contentView.findViewById(R.id.sub_title);
            if (subTitle != null) {
                String string = getArguments().getString(KEY_SUB_TITLE_NAME);
                if (string == null || string.isEmpty()) {
                    subTitle.setVisibility(View.GONE);
                } else {
                    subTitle.setText(string);
                }
            }
        }

        View cancel = contentView.findViewById(R.id.cancel_button);
        if (cancel != null) {
            cancel.setOnClickListener(v -> onCancel());
        }
        viewSpace = contentView.findViewById(R.id.viewSpace);
        confirm = contentView.findViewById(R.id.confirm_button);
        if (confirm != null) {
            confirm.setOnClickListener(v -> {
                hideSoft();
                onConfirm();
            });
        }

        contentContainer.addView(createContentView(inflater, contentContainer, savedInstanceState));
        return contentView;
    }

    protected void onCancel() {
        doDismiss();
    }

    protected void onConfirm() {
        dismiss();
    }

    protected abstract View createContentView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState);

    private void doDismiss() {
        hideSoft();
        dismiss();
    }

    private void hideSoft() {
        SoftKeyUtils.hideSoftKey();
    }

    protected boolean quitWhenTouchBg() {
        return true;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(0x33000000));
        getDialog().getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.MATCH_PARENT);
    }

    protected AbstractFullScreenDialogFragment() {
    }

    private static final String KEY_SUB_TITLE_NAME = "__ATTRIBUTE__NAME__";
    private static final String KEY_TITLE_NAME = "__GROUP__NAME__";

    protected static void setArgs(Bundle args, String title, String subTitle) {
        args.putString(KEY_SUB_TITLE_NAME, subTitle);
        args.putString(KEY_TITLE_NAME, title);
    }

    protected void setHideConfirm() {
        viewSpace.setVisibility(View.GONE);
        confirm.setVisibility(View.GONE);
    }
}
