package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.IndexWithTextAdapter;

import org.jetbrains.annotations.NotNull;

public abstract class AbstractHierarchicalListFragment extends Fragment {
    private View layout1;
    private View layout2;
    private View back1;
    private View back2;
    private TextView title1;
    private TextView title2;
    private RecyclerView recyclerView1;
    private RecyclerView recyclerView2;
    private boolean is1Layout = true;//通知当前显示的是layout1还是layout2
    private int currentListIndex = 0;//当前显示的list在所有集合中的index
    private final IndexWithTextAdapter.IOnItemClick<IUniversalListItem<?>> callback = item -> onItemClicked(currentListIndex, item);

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View root = inflater.inflate(R.layout.fragment_hierarchical_list_layout, container, false);

        layout1 = root.findViewById(R.id.layout1);
        if (layout1 != null) {
            recyclerView1 = layout1.findViewById(R.id.list1);
        }
        back1 = root.findViewById(R.id.back1);
        if (back1 != null) {
            back1.setOnClickListener(v -> onBackClicked(currentListIndex));
        }
        title1 = root.findViewById(R.id.title1);

        layout2 = root.findViewById(R.id.layout2);
        if (layout2 != null) {
            recyclerView2 = layout2.findViewById(R.id.list2);
        }
        back2 = root.findViewById(R.id.back2);
        if (back2 != null) {
            back2.setOnClickListener(v -> onBackClicked(currentListIndex));
        }
        title2 = root.findViewById(R.id.title2);

        //初始化
        if (layout2 != null) {
            layout2.setVisibility(View.GONE);
        }
        //加载list1
        setupList(0, title1, recyclerView1, callback);
        return root;
    }

    //执行back
    final protected void back2(int listIndex) {
        currentListIndex = listIndex;
        is1Layout = !is1Layout;

        updateButtonVisibility();

        doSwitch(layout1, layout2, title1, title2, recyclerView1, recyclerView2, this,
                callback,
                is1Layout, listIndex, false);
    }

    //执行forward
    final protected void forward2(int listIndex) {
        currentListIndex = listIndex;
        is1Layout = !is1Layout;

        updateButtonVisibility();

        doSwitch(layout1, layout2, title1, title2, recyclerView1, recyclerView2, this,
                callback,
                is1Layout, listIndex, true);
    }

    private void updateButtonVisibility() {
        if (is1Layout && back1 != null) {
            back1.setVisibility(!isFirstIndex(currentListIndex) ? View.VISIBLE : View.GONE);
        } else if (!is1Layout && back2 != null) {
            back2.setVisibility(!isFirstIndex(currentListIndex) ? View.VISIBLE : View.GONE);
        }
    }

    protected boolean isFirstIndex(int index) {
        return index == 0;
    }

    protected int preIndexOf(int index) {
        return index - 1;
    }

    final boolean doBack() {
        if (currentListIndex > 0) {
            back2(preIndexOf(currentListIndex));
            return true;
        }
        return false;
    }

    private static void doSwitch(View layout1, View layout2, TextView title1, TextView title2,
                                 RecyclerView recyclerView1,
                                 RecyclerView recyclerView2,
                                 AbstractHierarchicalListFragment fragment,
                                 final IndexWithTextAdapter.IOnItemClick<IUniversalListItem<?>> callback,
                                 boolean targetLayoutIsLayout1,
                                 int listIndex,
                                 boolean moveForward) {
        //先更新界面，再执行动画
        View currentLayout = layout1;
        View targetLayout = layout1;
        TextView targetTitle = title1;
        RecyclerView targetRecycleView = recyclerView1;

        if (targetLayoutIsLayout1) {
            currentLayout = layout2;
        } else {
            targetLayout = layout2;
            targetTitle = title2;
            targetRecycleView = recyclerView2;
        }

        //先设置数据
        fragment.setupList(listIndex, targetTitle, targetRecycleView, callback);

        //执行动画
        Animation outAnimation = AnimationUtils.loadAnimation(fragment.getContext(),
                moveForward ? R.anim.slide_out_left : R.anim.slide_out_right);
        outAnimation.setFillAfter(true);
        View finalCurrentLayout = currentLayout;
        outAnimation.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {
            }

            @Override
            public void onAnimationEnd(Animation animation) {
                finalCurrentLayout.clearAnimation();
                finalCurrentLayout.setVisibility(View.GONE);
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });

        Animation inAnimation = AnimationUtils.loadAnimation(fragment.getContext(),
                moveForward ? R.anim.slide_in_right : R.anim.slide_in_left);
        inAnimation.setFillAfter(true);
        targetLayout.setVisibility(View.VISIBLE);

        currentLayout.startAnimation(outAnimation);
        targetLayout.startAnimation(inAnimation);
    }

    //通知child，通知list的item被点击，下一步执行 forward或结束加载
    protected abstract void onItemClicked(int currentListIndex, IUniversalListItem<?> item);

    //通知child,准备back，用于加载数据
    protected abstract void onBackClicked(int currentListIndex);

    //通知child，加载相应list的数据
    protected abstract void setupList(int listIndex, TextView title, RecyclerView recyclerView,
                                      final IndexWithTextAdapter.IOnItemClick<IUniversalListItem<?>> callback);

    protected static class Item implements IUniversalListItem<String> {
        private final String name;
        private final String id;

        Item(String name, String id) {
            this.name = name;
            this.id = id;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getId() {
            return id;
        }

        @Override
        public String getValue() {
            return id;
        }
    }

    protected AbstractHierarchicalListFragment() {
    }
}
