package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.level.ToleranceLevelRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.ToleranceUtil;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.ui.ViewUtil;
import com.bingce.controlnetwork.ui.widget.TextItemWithMore;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.utils.IOnSingleGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import com.bingce.controlnetwork.R;

/*
 * 项目限差详情，包含'LimitDetailFragment'+选择模板
 * */
public abstract class AbstractProjectToleranceFragment extends BaseFragment {
    private String currentToleranceDetailRecordId;

    protected String projectType;

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            projectType = getArguments().getString(BundleConstants.KEY_PROJECT_TYPE);
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_project_limit_detail_layout, container, false);
        //设置限差模板选择事件
        registerSelectTemplateCallback(view, () -> ThreadPoolUtil.execute(this::onSelectToleranceTemplateClick));
        //设置详情
        onCreateView(view);
        return view;
    }

    /**
     * 获取新建项目时的默认限差
     *
     * @param projectType
     */
    protected IToleranceDetail getNewDefaultTolerance(String projectType) {
        return ToleranceLoadUtil.getToleranceLoad(projectType).getNewDefaultTolerance(projectType);
    }

    protected abstract void onCreateView(View view);

    protected void setCurrentToleranceName(View view, IToleranceDetail toleranceRecord) {
        View tv = view.findViewById(R.id.setting_index);
        ThreadPoolUtil.execute(() -> {
            if (toleranceRecord == null) return;
            IToleranceOperate operate = ToleranceOperateUtil.getOperate(toleranceRecord.getId());
            if (operate == null) return;
            ToleranceGroupRecord group = operate.getGroup(toleranceRecord);
            ToleranceLevelRecord level = operate.getLevel(toleranceRecord);
            String name = ToleranceUtil.completedToleranceName(group, level, toleranceRecord);
            runOnUI(() -> {
                if (tv instanceof TextItemWithMore) {
                    ((TextItemWithMore) tv).setDescribeText(name);
                }
            });
        });
    }

    protected void registerSelectTemplateCallback(View view, Runnable runnable) {
        ViewUtil.setOnClick(view.findViewById(R.id.setting_index), v -> runnable.run());
    }

    /**
     * 限差模板的点击后逻辑
     */
    @WorkerThread
    protected void onSelectToleranceTemplateClick() {
        IToleranceOperate operate = ToleranceOperateUtil.getOperate(currentToleranceDetailRecordId);
        if (operate == null) return;
        IToleranceDetail toleranceDetailRecordTj = operate.getIToleranceDetail(currentToleranceDetailRecordId);
        if (toleranceDetailRecordTj == null) {
            return;
        }

        ThreadPoolUtil.executeInMain(() ->
                SelectToleranceBottomSheetFragment.select(
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        projectType,
                        toleranceDetailRecordTj.getGroupId(),
                        toleranceDetailRecordTj.getLevelId(),
                        toleranceDetailRecordTj.getId(),
                        toleranceId -> ThreadPoolUtil.execute(() -> {
                            //根据toleranceId查找缓存数据
                            IToleranceDetail instance = operate.getIToleranceDetail(toleranceId);
                            //更新页面数据
                            updateToleranceDetail(getView(), instance);
                            //新增
                            //这里增加存入数据库 已备杀死app后能获取到限差数据
                            operate.saveToleranceDetail(instance);
                        })
                ));
    }

    protected void onCurrentToleranceChanged(String toleranceId) {
        currentToleranceDetailRecordId = toleranceId;
    }

    protected void newToleranceDetailFragment(Bundle args, IOnSingleGetCallback<ToleranceDetailFragment> callback) {
        if (callback != null) {
            callback.onGet(ToleranceDetailFragment.newInstance(args, false));
        }
    }

    @WorkerThread
    protected void updateToleranceDetail(View view, IToleranceDetail tolerance) {
        if (tolerance == null) {
            return;
        }
        onCurrentToleranceChanged(tolerance.getId());
        runOnUI(() -> {
            //设置限差名称
            setCurrentToleranceName(view, tolerance);
            //设置限差详情
            Fragment detailFragment =
                    getParentFragmentManager().findFragmentByTag(AbstractProjectToleranceFragment.class.getName());
            if (detailFragment == null) {
                Bundle args = new Bundle();
                if (getArguments() != null) {
                    args.putString(BundleConstants.KEY_TOLERANCE_ID, tolerance.getId());
                }
                newToleranceDetailFragment(args, detailFragmentInstance -> getParentFragmentManager()
                        .beginTransaction()
                        .replace(R.id.fragment_container, detailFragmentInstance, AbstractProjectToleranceFragment.class.getName())
                        .commit());
            } else {
                if (detailFragment instanceof ToleranceDetailFragment) {
                    ((ToleranceDetailFragment) detailFragment).switchTolerance(tolerance.getId());
                }
            }
        });
    }

    protected String currentTolerance() {
        return currentToleranceDetailRecordId;
    }
}
