package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.UniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlnetwork.adapter.SelectableIndexWithTextAdapter;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.SelectableIndexWithTextAdapter;

/**
 * 弹框，用于选择限差模板
 */
public class BottomSheetSelectListFragment extends BottomSheetDialogFragment {
    public static final String TAG = BottomSheetSelectListFragment.class.getName();

    private BottomSheetSelectListFragment() {
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_bottom_popup_list_select_layout,
                container, false);
        // Set the adapter
        Context context = view.getContext();
        RecyclerView recyclerView = view.findViewById(R.id.list);
        recyclerView.setLayoutManager(new LinearLayoutManager(context));

        String targetId = null;
        List<Item> items = new ArrayList<>();
        List<String> indexes = null;
        if (getArguments() != null) {
            List<String> strings = getArguments().getStringArrayList(KEY_ITEMS);
            indexes = getArguments().getStringArrayList(KEY_ENABLE_INDEX);
            int defaultIndex = getArguments().getInt(KEY_DEFAULT_ID, -1);
            if (strings != null) {
                for (int index = 0; index < strings.size(); index++) {
                    String string = strings.get(index);
                    Item item = new Item();
                    item.setId(String.valueOf(index));
                    item.setName(string);
                    items.add(item);
                }
                if (defaultIndex < 0 || defaultIndex >= strings.size()) {
                    if (!items.isEmpty()) {
                        targetId = items.get(0).getId();
                    }
                } else {
                    targetId = items.get(defaultIndex).getId();
                }
            }
        }

        SelectableIndexWithTextAdapter<Item> adapter =
                new SelectableIndexWithTextAdapter<>(
                        targetId,
                        items,
                        indexes,
                        item -> {
                            Bundle args = new Bundle();
                            String id = item.getId();
                            int index = Integer.parseInt(id);
                            args.putInt(KEY_INDEX, index);
                            args.putString(KEY_STRING, item.getName());
                            getParentFragmentManager().setFragmentResult(KEY_REQUEST_SELECT, args);
                            dismiss();
                        });
        recyclerView.setAdapter(adapter);
        return view;
    }

    private static final String KEY_ITEMS = "__ITEMS__";
    private static final String KEY_INDEX = "__INDEX__";
    private static final String KEY_ENABLE_INDEX = "__ENABLE_INDEX__";
    private static final String KEY_DEFAULT_ID = "__defaultId__";
    private static final String KEY_STRING = "__STRING__";
    private static final String KEY_REQUEST_SELECT = "__SELECT__";

    public static void showSelect(FragmentManager fragmentManager,
                                  @NonNull LifecycleOwner lifecycleOwner,
                                  int defaultIndex,
                                  ArrayList<String> items,
                                  ArrayList<String> enableIndexes,
                                  IOnSelectCallback callback) {
        if (callback == null) {
            return;
        }
        Bundle args = new Bundle();
        args.putStringArrayList(KEY_ITEMS, items);
        if (enableIndexes != null && !enableIndexes.isEmpty()) {
            args.putStringArrayList(KEY_ENABLE_INDEX, enableIndexes);
        }
        args.putInt(KEY_DEFAULT_ID, defaultIndex);
        BottomSheetSelectListFragment fragment = new BottomSheetSelectListFragment();
        fragment.setArguments(args);
        fragmentManager
                .setFragmentResultListener(KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onSelected(result.getInt(KEY_INDEX), result.getString(KEY_STRING));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

//    public static void showSelect(FragmentManager fragmentManager,
//                                  @NonNull LifecycleOwner lifecycleOwner,
//                                  int defaultIndex,
//                                  ArrayList<String> items,
//                                  IOnSelectCallback callback) {
//        showSelect(fragmentManager, lifecycleOwner, defaultIndex, items, null, callback);
//    }

    public interface IOnSelectCallback {
        void onSelected(int index, String string);
    }

    private static class Item extends UniversalListItem<String> {
        @Override
        public ViewItemType getType() {
            return null;
        }
    }
}