package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigOperate;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.ISurveyorPeriodDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.SurveyorPeriodAdapter;
import com.bingce.controlnetwork.ui.ViewUtil;
import com.bingce.controlnetwork.util.DefaultNameUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.ToastUtils;


/**
 * 弹框，用于选择测期
 */
public class BottomSheetSurveyorPeriodFragment extends BottomSheetDialogFragment implements IUIRunner {
    private static final String KEY_TIME_ID = "__TIME_ID";
    public static final String KEY_REQUEST_TIME = "KEY_REQUEST_TIME_ID";
    public static final String KEY_RESULT_NAME = "KEY_RESULT_NAME";
    public static final String KEY_RESULT_ID = "___result__id";
    private SurveyorPeriodAdapter adapter = null;

    private BottomSheetSurveyorPeriodFragment() {
    }

    public static void pick(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            String projectId, String currentCycleId, ICallback callback) {
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_TIME,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_TIME.equals(requestKey)) {
                                callback.onSelect(result.getString(KEY_RESULT_ID), result.getString(KEY_RESULT_NAME));
                            }
                        });
        BottomSheetDialogFragment fragment = new BottomSheetSurveyorPeriodFragment();
        Bundle args = new Bundle();
        args.putString(KEY_TIME_ID, currentCycleId);
        args.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        fragment.setArguments(args);
        fragment.show(fragmentManager, "time_index");
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_surveryor_station_number_list, container, false);
        RecyclerView recyclerView = view.findViewById(R.id.list);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        String projectId = "";
        String currentCycleId = "";
        if (getArguments() != null) {
            projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            currentCycleId = getArguments().getString(KEY_TIME_ID);
        }

        adapter = new SurveyorPeriodAdapter(currentCycleId, this);
        recyclerView.setAdapter(adapter);

        Helper helper = new Helper(adapter, recyclerView,
                view.findViewById(R.id.loading), view.findViewById(R.id.empty_tips), getViewLifecycleOwner());

        helper.refresh(projectId);

        String finalProjectId = projectId;
        ViewUtil.setOnClick(
                view.findViewById(R.id.new_item),
                v -> createNewCycle(getContext(), getParentFragmentManager(), getChildFragmentManager(), getViewLifecycleOwner(),
                        adapter, this, finalProjectId));
        return view;
    }

    private static void createNewCycle(@Nullable Context context, FragmentManager parentManager, FragmentManager manager, LifecycleOwner lifecycleOwner,
                                       SurveyorPeriodAdapter adapter, IUIRunner runner, String finalProjectId) {
        if (context == null) {
            return;
        }
        StringInputFragment.input(manager,
                context.getString(R.string.tips_input_new_cycle_name),
                DefaultNameUtil.getDefaultPeriodName(),
                lifecycleOwner,
                string -> {
                    if (string == null || string.isEmpty()) {
                        ToastUtils.showShort(R.string.tips_cycle_name_cannt_empty);
                    } else {
                        SurveyorPeriodRecord record =
                                new SurveyorPeriodRecord(finalProjectId, string);
                        SurveyorDatabaseFactory.instance
                                .getSurveyorPeriod()
                                .saveRecord(record);
                        if (adapter != null) {
                            adapter.changeCurrentSelect(record.id);
                        }
                        //自动切换到新测回
                        Bundle args = new Bundle();
                        args.putString(BottomSheetSurveyorPeriodFragment.KEY_RESULT_ID, record.getId());
                        args.putString(BottomSheetSurveyorPeriodFragment.KEY_RESULT_NAME, record.getName());
                        parentManager.setFragmentResult(BottomSheetSurveyorPeriodFragment.KEY_REQUEST_TIME, args);
                        //询问是否从其他已有测期复制测站列表
                        tryCopyStationFromOther(context, manager, lifecycleOwner, record, runner, finalProjectId);
                        //更新默认测回和测期
                        ConfigOperate.changeDefaultProjectAndCycle(finalProjectId, record.id);

                    }
                });
    }

    private static void tryCopyStationFromOther(
            @Nullable Context context, FragmentManager manager, LifecycleOwner lifecycleOwner,
            SurveyorPeriodRecord record, IUIRunner runner, String finalProjectId) {
        //检测其他测期是否有测站
        ThreadPoolUtil.execute(() -> {
            ISurveyorPeriodDataSource surveyorCycleDataSource =
                    SurveyorDatabaseFactory.instance
                            .getSurveyorPeriod();
            List<SurveyorPeriodRecord> cycleRecords = surveyorCycleDataSource.getDataListSyncByProjectId(finalProjectId);
            if (cycleRecords == null || cycleRecords.isEmpty()) {
                return;
            }
            List<SurveyorPeriodRecord> cycleRecordWithStation = new ArrayList<>();
            ISurveyorStationDataSource stationDataSource =
                    SurveyorDatabaseFactory.instance.getSurveyorStation();
            for (SurveyorPeriodRecord cycleRecord : cycleRecords) {
                if (cycleRecord == null) {
                    continue;
                }

                List<SurveyorStationRecord> stationRecordList = stationDataSource.getDataList(finalProjectId, cycleRecord.getId());
                if (stationRecordList == null || stationRecordList.isEmpty()) {
                    continue;
                }

                cycleRecordWithStation.add(cycleRecord);
            }

            if (cycleRecordWithStation.isEmpty()) {
                return;
            }

            ArrayList<String> cycleNameList = new ArrayList<>();
            List<String> cycleIdList = new ArrayList<>();
            for (SurveyorPeriodRecord cycleRecord : cycleRecordWithStation) {
                if (cycleRecord == null) {
                    continue;
                }
                cycleNameList.add(cycleRecord.getName());
                cycleIdList.add(cycleRecord.getId());
            }
            //当前其他测期已有测站，则提示是否拷贝
            runner.runOnUI(() -> {
                if (context == null) {
                    return;
                }
                TipsWith2OptionFragment.tips(manager,
                        lifecycleOwner,
                        context.getString(R.string.confirm_button_copy_station_list),
                        context.getString(R.string.cancel_button_create_empty_cycle),
                        context.getString(R.string.tips_should_copy_station_list_from_other_cycle),
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                copyStationListFromOther(context, manager, lifecycleOwner, cycleNameList, cycleIdList, record);
                            }
                        });
            });
        });
    }

    private static void copyStationListFromOther(
            Context context, FragmentManager manager, LifecycleOwner lifecycleOwner,
            ArrayList<String> cycleNameList, List<String> surveyorPeriodIdList,
            SurveyorPeriodRecord surveyorPeriodRecord) {
        if (surveyorPeriodRecord == null) {
            return;
        }
        ListDialogFragment.showList(
                manager, context.getString(R.string.tips_select_cycle_to_copy),
                cycleNameList, lifecycleOwner, (index, value) -> {
                    String surveyorPeriodId = surveyorPeriodIdList.get(index);
                    String surveyorPeriodRecordId = surveyorPeriodRecord.getId();
                    String projectId = surveyorPeriodRecord.getProjectId();
                    ISurveyorStationDataSource stationDataSource =
                            SurveyorDatabaseFactory.instance.getSurveyorStation();
                    ILineRecordDataSource lineRecordDataSource =
                            SurveyorDatabaseFactory.instance.getLineRecordDatabase();
                    ThreadPoolUtil.execute(() -> {
                        List<SurveyorStationRecord> stationRecordList =
                                stationDataSource.getDataList(projectId, surveyorPeriodId);
                        if (stationRecordList == null || stationRecordList.isEmpty()) {
                            return;
                        }
                        String lastLineId = "";//当前遍历到的要拷贝的导线id
                        String newLineId = "";//新建的导线id
                        List<SurveyorStationRecord> newStations = new ArrayList<>();
                        List<LineRecord> newLines = new ArrayList<>();
                        long date = (new Date()).getTime();
                        for (SurveyorStationRecord surveyorStationRecord : stationRecordList) {
                            if (surveyorStationRecord == null) {
                                continue;
                            }
                            if (surveyorStationRecord.isMulAngleStation()) {
                                lastLineId = LineRecord.TYPE_MUL_ANGLE;
                                newLineId = LineRecord.TYPE_MUL_ANGLE;
                            } else if (SurveyorStationRecord.isLineStation(surveyorStationRecord.getLineId())) {
                                //拷贝导线
                                if (!lastLineId.equals(surveyorStationRecord.getLineId())) {
                                    lastLineId = surveyorStationRecord.getLineId();
                                    //执行创建导线以及相应测站的逻辑
                                    LineRecord lineRecord = lineRecordDataSource.findById(lastLineId);
                                    if (lineRecord == null) {
                                        continue;
                                    }
                                    LineRecord newLine = LineRecord.copy(lineRecord, surveyorPeriodRecordId, date++);
                                    newLines.add(newLine);
                                    newLineId = newLine.id;
                                }
                            }
                            //拷贝测站
                            newStations.add(
                                    SurveyorStationRecord.copy(
                                            surveyorStationRecord, surveyorPeriodRecordId, newLineId, date++));
                        }
                        //存储测站
                        stationDataSource.save(newStations);
                        //存储导线
                        lineRecordDataSource.save(newLines);
                    });
                });
    }

    public void runOnUI(Runnable runnable) {
        if (getActivity() == null) {
            return;
        }
        getActivity().runOnUiThread(runnable);
    }

    public interface ICallback {
        void onSelect(String id, String name);
    }

    public static class Helper {
        private final SurveyorPeriodAdapter adapter;
        private final View loadingView;
        private final View emptyView;
        private final RecyclerView recyclerView;
        private final LifecycleOwner lifecycleOwner;

        Helper(SurveyorPeriodAdapter adapter,
               RecyclerView recyclerView,
               View loadingView,
               View emptyView,
               LifecycleOwner lifecycleOwner) {
            this.loadingView = loadingView;
            this.emptyView = emptyView;
            this.adapter = adapter;
            this.recyclerView = recyclerView;
            this.lifecycleOwner = lifecycleOwner;
        }

        private void update(List<SurveyorPeriodRecord> projectRecords) {
            adapter.updateData(projectRecords);
        }

        public void refresh(String projectId) {
            ISurveyorPeriodDataSource surveyorCycleDataSource =
                    SurveyorDatabaseFactory.instance.getSurveyorPeriod();
            surveyorCycleDataSource.getLiveDataListByProjectId(projectId)
                    .observe(lifecycleOwner,
                            records -> {
                                //隐藏loading
                                if (loadingView != null) {
                                    loadingView.setVisibility(View.GONE);
                                }
                                if (records == null || records.isEmpty()) {
                                    if (emptyView != null) {
                                        emptyView.setVisibility(View.VISIBLE);
                                    }
                                    recyclerView.setVisibility(View.GONE);
                                } else {
                                    //刷新adapter
                                    if (emptyView != null) {
                                        emptyView.setVisibility(View.GONE);
                                    }
                                    recyclerView.setVisibility(View.VISIBLE);
                                    update(records);
                                }
                            });
        }
    }
}