package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.SurveyorStationListAdapter;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.stationdetail.SingleStationDetailAndEditFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.StationSurveyorFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureFragment;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

/**
 * 测站列表(项目和测量共用)
 */
public class ComponentSurveyorStationListFragment extends BaseFragment {
    public static final String TAG = "cn.liuyanbing.surveyor.fragment.dummy__SurveyorStationListFragment___";
    private SurveyorStationListAdapter adapter;
    private RecyclerView recyclerView;

    private LiveData<List<SurveyorStationRecord>> stationLiveData;

    /**
     * 去测量或者详情页面
     */
    private void goStationMeasureOrDetail(SurveyorStationRecord item, boolean detailOrSurveyorFinal) {
        String stationId = item.getId();
        ThreadPoolUtil.execute(() -> {
            SurveyorStationRecord stationRecord = SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getRecordSync(stationId);
            if (stationRecord == null || stationRecord.getItems() == null || stationRecord.getItems().isEmpty()) {
                return;
            }

            String lineTypeFromStation = StationUtil.getLineTypeFromStation(stationRecord);

            //站的名称 导线+点名
            String stationName = StationUtil.getStationName(stationRecord);

            if (detailOrSurveyorFinal) {
                SingleStationDetailAndEditFragment.start(getContext(), stationId, stationName, item.getProjectId(), null);
            } else {
                //StationSurveyorFragment原始测量页
                if (ProjectType.isConnection(item.projectType)) {
                    MeasureFragment.start(getContext(), stationId, stationName);
                } else {
                    StationSurveyorFragment.start(getContext(), stationId, stationName);
                }
            }
        });
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_list_layout, container, false);
        if (view instanceof RecyclerView) {
            recyclerView = (RecyclerView) view;
            if (getArguments() != null) {
                update(getArguments().getString(BundleConstants.KEY_PROJECT_ID),
                        getArguments().getString(KEY_TIME_INDEX));
            }
        }
        return view;
    }

    /**
     * 根据项目id和测期id获取测站记录
     *
     * @param projectId
     * @param periodId
     */
    public void update(String projectId, String periodId) {
        if (stationLiveData != null) {
            stationLiveData.removeObservers(this);
        }

        stationLiveData = SurveyorDatabaseFactory
                .instance
                .getSurveyorStation().getLiveDataList(projectId, periodId);

        stationLiveData.observe(getViewLifecycleOwner(), this::updateUi);
    }

    private void updateUi(List<SurveyorStationRecord> dataList) {
        runOnUI(() -> {
            if (adapter == null) {
                boolean detailOrSurveyor = false;
                if (getArguments() != null) {
                    detailOrSurveyor = getArguments().getBoolean(KEY_DETAIL_OR_SURVEYOR, false);
                }
                boolean detailOrSurveyorFinal = detailOrSurveyor;
                adapter = new SurveyorStationListAdapter(dataList,
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        runnable -> {
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(runnable);
                            }
                        },
                        getContext(),
                        item -> goStationMeasureOrDetail(item, detailOrSurveyorFinal),
                        detailOrSurveyor);
                recyclerView.setAdapter(adapter);
            } else {
                adapter.updateData(dataList);
            }
        });
    }

    private static final String KEY_TIME_INDEX = "___time_index__";
    private static final String KEY_DETAIL_OR_SURVEYOR = "___editable___";//标识点击item进入测量界面还是测站详情

    public static ComponentSurveyorStationListFragment newInstance(String projectId, String surveyorPeriodId, boolean isDetail) {
        ComponentSurveyorStationListFragment fragment = new ComponentSurveyorStationListFragment();
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(KEY_TIME_INDEX, surveyorPeriodId);
        bundle.putBoolean(KEY_DETAIL_OR_SURVEYOR, isDetail);
        fragment.setArguments(bundle);
        return fragment;
    }

    public static ComponentSurveyorStationListFragment newInstance(String projectId, String surveyorPeriodId) {
        return newInstance(projectId, surveyorPeriodId, true);
    }
}
