package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord;
import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.IndexWithTextAdapter;
import com.bingce.controlnetwork.adapter.SelectableIndexWithTextAdapter;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 选择项目和测期的组件
 */
public class ComponentSwitchStationListFragment extends AbstractHierarchicalListFragment {
    private static final int INDEX_PROJECT_LIST = 0;
    private static final int INDEX_CYCLE_LIST = 1;
    private String projectId = "";
    private String projectName = "";
    private SelectableIndexWithTextAdapter<IUniversalListItem<?>> projectAdapter;
    private SelectableIndexWithTextAdapter<IUniversalListItem<?>> periodAdapter;

    @Override
    protected void onItemClicked(int currentListIndex, IUniversalListItem<?> item) {
        if (INDEX_PROJECT_LIST == currentListIndex) {
            projectId = item.getId();
            projectName = item.getName();
            forward2(INDEX_CYCLE_LIST);
        } else {
            SelectProjectAndSurveyorCycleFragment.SelectedResult result =
                    new SelectProjectAndSurveyorCycleFragment.SelectedResult(projectId, projectName, item.getId(), item.getName());
            EventBus.getDefault().post(result);
        }
    }

    @Override
    protected void onBackClicked(int currentListIndex) {
        if (INDEX_CYCLE_LIST == currentListIndex) {
            back2(INDEX_PROJECT_LIST);
        } else {
            //结束
            ToastUtils.showShort(R.string.cannot_go_back);
        }
    }

    @Override
    protected void setupList(int listIndex, TextView title, RecyclerView recyclerView,
                             IndexWithTextAdapter.IOnItemClick<IUniversalListItem<?>> callback) {
        if (INDEX_PROJECT_LIST == listIndex) {
            //加载
            if (title != null) {
                title.setText(Tools.getString(R.string.please_select_an_project));
            }
            ThreadPoolUtil.execute(() -> {
                List<ProjectRecord> projectRecords = SurveyorDatabaseFactory
                        .instance
                        .getProjectDataSource()
                        .getDataList();
                String selectedProject = "";
                if (getArguments() != null) {
                    selectedProject = getArguments().getString(KEY_PROJECT_ID);
                }
                List<IUniversalListItem<?>> items = new ArrayList<>();
                int targetIndex = 0;
                for (int index = 0; index < projectRecords.size(); index++) {
                    ProjectRecord projectRecord = projectRecords.get(index);
                    Item item = new Item(projectRecord.getName(), projectRecord.getId());
                    items.add(item);
                    if (selectedProject.equals(item.getId())) {
                        targetIndex = index;
                    }
                }
                int finalTargetIndex = targetIndex;
                String finalSelectedProject = selectedProject;
                ThreadPoolUtil.executeInMain(() -> {
                    if (projectAdapter == null) {
                        projectAdapter = new SelectableIndexWithTextAdapter<>(finalSelectedProject, items, callback);
                        recyclerView.setAdapter(projectAdapter);
                    } else {
                        projectAdapter.updateData(items);
                    }
                    recyclerView.scrollToPosition(finalTargetIndex);
                });
            });
        } else if (INDEX_CYCLE_LIST == listIndex) {
            //加载
            if (title != null) {
                title.setText(Tools.getString(R.string.please_select_an_period));
            }
            ThreadPoolUtil.execute(() -> {
                List<SurveyorPeriodRecord> periodRecords = SurveyorDatabaseFactory.instance
                        .getSurveyorPeriod().getDataListSyncByProjectId(projectId);
                List<IUniversalListItem<?>> items = new ArrayList<>();
                String selectedPeriodId = "";
                if (getArguments() != null) {
                    selectedPeriodId = getArguments().getString(KEY_CYCLE_ID);
                }
                int targetIndex = 0;
                for (int index = 0; index < periodRecords.size(); index++) {
                    SurveyorPeriodRecord surveyorPeriodRecord = periodRecords.get(index);
                    Item item = new Item(surveyorPeriodRecord.getName(), surveyorPeriodRecord.getId());
                    items.add(item);
                    if (selectedPeriodId.equals(item.getId())) {
                        targetIndex = index;
                    }
                }
                int finalTargetIndex = targetIndex;
                if (periodAdapter == null) {
                    periodAdapter = new SelectableIndexWithTextAdapter<>(selectedPeriodId, items, callback);
                    ThreadPoolUtil.executeInMain(() -> {
                        recyclerView.setAdapter(periodAdapter);
                        recyclerView.scrollToPosition(finalTargetIndex);
                    });
                } else {
                    ThreadPoolUtil.executeInMain(() -> {
                        periodAdapter.updateData(items);
                        recyclerView.scrollToPosition(finalTargetIndex);
                    });
                }
            });
        }
    }

    private static final String KEY_PROJECT_ID = "___project_id";
    private static final String KEY_CYCLE_ID = "__cycle_id";

    public static ComponentSwitchStationListFragment newInstance(String projectId, String cycleId) {
        ComponentSwitchStationListFragment fragment = new ComponentSwitchStationListFragment();
        Bundle args = new Bundle();
        args.putString(KEY_PROJECT_ID, projectId);
        args.putString(KEY_CYCLE_ID, cycleId);
        fragment.setArguments(args);
        return fragment;
    }
}
