package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetailDataSource;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupConstants;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.level.ToleranceLevelRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil;
import com.bingce.controlapphelper.model.IUniversalListItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.IndexWithTextAdapter;
import com.bingce.controlnetwork.adapter.SelectableIndexWithTextAdapter;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;


/**
 * 选择限差模板的内容
 */
public class ComponentSwitchToleranceFragment extends AbstractHierarchicalListFragment {
    private static final int INDEX_GROUP = 0;
    private static final int INDEX_LEVEL = 1;
    private static final int INDEX_TOLERANCE = 2;
    private String projectType = "";
    private String groupId = "";
    private String levelId = "";
    private String toleranceId = "";
    private SelectableIndexWithTextAdapter<IUniversalListItem<?>> adapterGroup = null;
    private SelectableIndexWithTextAdapter<IUniversalListItem<?>> adapterLevel = null;
    private SelectableIndexWithTextAdapter<IUniversalListItem<?>> adapterDetail = null;

    @Override
    protected void onItemClicked(int currentListIndex, IUniversalListItem<?> item) {
        if (INDEX_GROUP == currentListIndex) {
            groupId = item.getId();
            //检测目标group是否有level，如果没有直接进入detail
            List<ToleranceLevelRecord> levelRecordList = ToleranceLoadUtil.getToleranceLoad(projectType).levelListOf(projectType, groupId);
            if (levelRecordList.isEmpty()) {
                levelId = "";
                forward2(INDEX_TOLERANCE);
            } else {
                forward2(INDEX_LEVEL);
            }
        } else if (INDEX_LEVEL == currentListIndex) {
            levelId = item.getId();
            forward2(INDEX_TOLERANCE);
        } else {
            callBackData(item.getId());
        }
    }

    private void callBackData(String id) {
        SelectedToleranceEvent result = new SelectedToleranceEvent(id);
        EventBus.getDefault().post(result);
    }

    @Override
    protected int preIndexOf(int index) {
        if (INDEX_TOLERANCE == index) {
            if (levelId == null || levelId.isEmpty()) {
                return INDEX_GROUP;
            } else {
                return INDEX_LEVEL;
            }
        }
        return super.preIndexOf(index);
    }

    @Override
    protected void onBackClicked(int currentListIndex) {
        if (INDEX_TOLERANCE == currentListIndex) {
            if (levelId == null || levelId.isEmpty()) {
                back2(INDEX_GROUP);
            } else {
                back2(INDEX_LEVEL);
            }
        } else if (INDEX_LEVEL == currentListIndex) {
            back2(INDEX_GROUP);
        } else {
            ToastUtils.showShort(R.string.cannot_go_back);
        }
    }

    @Override
    protected void setupList(int listIndex, TextView title, RecyclerView recyclerView,
                             final IndexWithTextAdapter.IOnItemClick<IUniversalListItem<?>> callback) {
        if (INDEX_GROUP == listIndex) {
            if (title != null) {
                title.setText(R.string.tips_please_select_tolerance_group);
            }
            List<ToleranceGroupRecord> toleranceGroupRecords = ToleranceLoadUtil.getToleranceLoad(projectType).getGroupFromType(projectType);
            List<IUniversalListItem<?>> items = new ArrayList<>();
            int targetIndex = 0;
            IUniversalListItem<?> customGroup = null;
            int index = -1;
            for (ToleranceGroupRecord toleranceGroupRecord : toleranceGroupRecords) {
                if (toleranceGroupRecord.isDeleted()) {
                    continue;
                }
                if (ToleranceGroupConstants.VALUE_CUSTOM_GROUP.equals(toleranceGroupRecord.getId())) {
                    customGroup = toleranceGroupRecord;
                    continue;
                }
                index++;
                items.add(toleranceGroupRecord);
                if (groupId.equals(toleranceGroupRecord.getId())) {
                    targetIndex = index;
                }
            }
            //添加自定义分组
            if (customGroup != null) {
                items.add(customGroup);
                if (groupId.equals(customGroup.getId())) {
                    targetIndex = items.size() - 1;
                }
            }
            if (adapterGroup == null) {
                adapterGroup = new SelectableIndexWithTextAdapter<>(groupId, items, callback);
            } else {
                adapterGroup.updateData(items);
            }
            recyclerView.setAdapter(adapterGroup);
            recyclerView.scrollToPosition(targetIndex);
        } else if (INDEX_LEVEL == listIndex) {
            if (title != null) {
                title.setText(R.string.tips_please_select_tolerance_level);
            }
            List<ToleranceLevelRecord> toleranceLevelRecords = ToleranceLoadUtil.getToleranceLoad(projectType).levelListOf(projectType, groupId);
            List<IUniversalListItem<?>> items = new ArrayList<>();
            int targetIndex = 0;
            for (int index = 0; index < toleranceLevelRecords.size(); index++) {
                ToleranceLevelRecord toleranceLevelRecord = toleranceLevelRecords.get(index);
                items.add(toleranceLevelRecord);
                if (levelId != null && levelId.equals(toleranceLevelRecord.getId())) {
                    targetIndex = index;
                }
            }
            if (adapterLevel == null) {
                adapterLevel = new SelectableIndexWithTextAdapter<>(levelId, items, callback);
            } else {
                adapterLevel.updateData(items);
            }
            recyclerView.setAdapter(adapterLevel);
            recyclerView.scrollToPosition(targetIndex);
        } else if (INDEX_TOLERANCE == listIndex) {
            ThreadPoolUtil.execute(() -> {
                IToleranceDetailDataSource toleranceDetailDataSource =
                        SurveyorDatabaseFactory.instance.getToleranceDetailDataSource();
                List<IToleranceDetail> toleranceDetailList;
                int titleRes = R.string.tips_please_select_instrument_precision;
                if (levelId == null || levelId.isEmpty()) {//自定义模板
                    toleranceDetailList = ToleranceLoadUtil.getToleranceLoad(projectType).getDetailListByGroup(groupId);
                    //选择模板
                    titleRes = R.string.tips_please_select_custom_tolerance;
                } else {
                    toleranceDetailList = ToleranceLoadUtil.getToleranceLoad(projectType).detailListOfLevel(projectType, levelId);
                }

                if (!ToleranceGroupConstants.VALUE_CUSTOM_GROUP.equals(groupId) &&
                        toleranceDetailList != null && toleranceDetailList.size() == 1) {
                    callBackData(toleranceDetailList.get(0).getId());
                    return;
                }


                int finalTitleRes = titleRes;
                runOnUI(() -> {
                    if (title != null) {
                        title.setText(finalTitleRes);
                    }
                });
                List<IUniversalListItem<?>> items = new ArrayList<>();
                int targetIndex = 0;
                for (int index = 0; index < toleranceDetailList.size(); index++) {
                    IToleranceDetail toleranceDetailRecordTj = toleranceDetailList.get(index);
                    items.add((IUniversalListItem<?>) toleranceDetailRecordTj);
                    if (toleranceId.equals(toleranceDetailRecordTj.getId())) {
                        targetIndex = index;
                    }
                }
                int finalTargetIndex = targetIndex;
                runOnUI(() -> {
                    if (adapterDetail == null) {
                        adapterDetail = new SelectableIndexWithTextAdapter<>(toleranceId, items, callback);
                    } else {
                        adapterDetail.updateData(items);
                    }
                    recyclerView.setAdapter(adapterDetail);
                    recyclerView.scrollToPosition(finalTargetIndex);
                });
            });
        }
    }

    private void runOnUI(Runnable runnable) {
        if (getActivity() == null) {
            return;
        }
        getActivity().runOnUiThread(runnable);
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            projectType = getArguments().getString(KEY_PROJECT_TYPE);
            groupId = getArguments().getString(KEY_GROUP);
            levelId = getArguments().getString(KEY_LEVEL);
            toleranceId = getArguments().getString(KEY_TOLERANCE);
        }
    }

    private static final String KEY_PROJECT_TYPE = "___PROJECT_TYPE___";
    private static final String KEY_GROUP = "___GROUP___";
    private static final String KEY_LEVEL = "___LEVEL___";
    private static final String KEY_TOLERANCE = "___PRECISION__";

    public static ComponentSwitchToleranceFragment newInstance(String projectType,
                                                               String groupId, String levelId, String toleranceId) {
        ComponentSwitchToleranceFragment fragment = new ComponentSwitchToleranceFragment();
        Bundle args = new Bundle();
        args.putString(KEY_PROJECT_TYPE, projectType);
        args.putString(KEY_GROUP, groupId);
        args.putString(KEY_LEVEL, levelId);
        args.putString(KEY_TOLERANCE, toleranceId);
        fragment.setArguments(args);
        return fragment;
    }

    public static class SelectedToleranceEvent {
        public final String toleranceId;

        public SelectedToleranceEvent(String toleranceId) {
            this.toleranceId = toleranceId;
        }
    }
}
