package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;

import org.jetbrains.annotations.NotNull;

import com.bingce.controlnetwork.R;

public final class FullLoadingFragment extends AppCompatDialogFragment {
    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_full_loading_layout, container, false);
        if (getArguments() != null) {
            String msg = getArguments().getString(KEY_MSG);
            if (msg != null && !msg.isEmpty()) {
                TextView tips = rootView.findViewById(R.id.tips);
                if (tips != null) {
                    tips.setText(msg);
                }
            }
        }
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() != null) {
            final Window window = getDialog().getWindow();

            window.setWindowAnimations(R.style.animate_full_dialog);
            window.setBackgroundDrawable(null);

            WindowManager.LayoutParams params = window.getAttributes();
            params.width = ViewGroup.LayoutParams.MATCH_PARENT;
            params.height = ViewGroup.LayoutParams.MATCH_PARENT;
            window.setAttributes(params);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        if (getDialog() == null) {
            return;
        }
        getDialog().setOnKeyListener((dialog, keyCode, event) -> {
            // pass on to be processed as normal
            return keyCode == android.view.KeyEvent.KEYCODE_BACK; // pretend we've processed it
        });
    }

    private static final String KEY_MSG = "__KEY_MSG__";

    public static DialogFragment showLoading(FragmentManager fragmentManager) {
        return showLoading(fragmentManager, null);
    }

    public static DialogFragment showLoading(FragmentManager fragmentManager, String msg) {
        DialogFragment fragment = new FullLoadingFragment();
        if (msg != null && !msg.isEmpty()) {
            Bundle args = new Bundle();
            args.putString(KEY_MSG, msg);
            fragment.setArguments(args);
        }
        fragment.show(fragmentManager, FullLoadingFragment.class.getName());
        return fragment;
    }
}
