package com.bingce.controlnetwork.fragment;

import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.RecyclerView;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.R;

public class ListDialogFragment extends DialogFragment {
    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() == null) {
            return;
        }
        Window window = getDialog().getWindow();
        window.setBackgroundDrawable(new ColorDrawable(0x00000000));
        window.setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_station_select_point_datasource_layout, container, false);
        RecyclerView recyclerView = rootView.findViewById(R.id.list);
        if (recyclerView != null && getArguments() != null) {
            List<String> list = getArguments().getStringArrayList(KEY_LIST);
            _Adapter adapter = new _Adapter(list, (index, value) -> {
                Bundle args = new Bundle();
                args.putInt(KEY_INDEX, index);
                args.putString(KEY_RESULT, value);
                getParentFragmentManager().setFragmentResult(TAG, args);
                dismiss();
            });
            recyclerView.setAdapter(adapter);
        }
        TextView title = rootView.findViewById(R.id.title);
        if (title != null && getArguments() != null) {
            String string = getArguments().getString(KEY_TITLE);
            if (string != null && string.isEmpty()) {
                title.setText(string);
            }
        }
        return rootView;
    }

    private static class _VH extends RecyclerView.ViewHolder {
        final TextView textView;

        public _VH(@NonNull @NotNull View itemView) {
            super(itemView);
            if (itemView instanceof TextView) {
                textView = (TextView) itemView;
            } else {
                textView = null;
            }
        }
    }

    private static class _Adapter extends RecyclerView.Adapter<_VH> {
        private final List<String> list;
        private final IResultCallback resultCallback;

        _Adapter(List<String> list, IResultCallback resultCallback) {
            this.list = list;
            this.resultCallback = resultCallback;
        }

        @NonNull
        @NotNull
        @Override
        public _VH onCreateViewHolder(@NonNull @NotNull ViewGroup parent, int viewType) {
            View rootView = LayoutInflater.from(parent.getContext())
                    .inflate(android.R.layout.simple_list_item_1, parent, false);
            return new _VH(rootView);
        }

        @Override
        public void onBindViewHolder(@NonNull @NotNull ListDialogFragment._VH holder, int position) {
            String value = list.get(position);
            if (holder.textView != null) {
                holder.textView.setText(value);
                holder.textView.setOnClickListener(v -> {
                    if (resultCallback != null) {
                        resultCallback.onResult(position, value);
                    }
                });
            }
        }

        @Override
        public int getItemCount() {
            return list == null ? 0 : list.size();
        }
    }

    public interface IResultCallback {
        void onResult(int index, String value);
    }

    private static final String TAG = ListDialogFragment.class.getName();
    private static final String KEY_INDEX = "key_index";
    private static final String KEY_RESULT = "key value";
    private static final String KEY_LIST = "key list";
    private static final String KEY_TITLE = "KEY TITLE";

    public static void showList(FragmentManager fragmentManager,
                                ArrayList<String> list,
                                @NonNull LifecycleOwner lifecycleOwner,
                                IResultCallback callback) {
//        DialogFragment fragment = new ListDialogFragment();
//
//        Bundle args = new Bundle();
//        args.putStringArrayList(KEY_LIST, list);
//        fragment.setArguments(args);
//
//        fragmentManager
//                .setFragmentResultListener(
//                        TAG,
//                        lifecycleOwner,
//                        (requestKey, result) -> {
//                            if (TAG.equals(requestKey)) {
//                                callback.onResult(result.getInt(KEY_INDEX, -1), result.getString(KEY_RESULT));
//                            }
//                        });
//        fragment.show(fragmentManager, TAG);
        showList(fragmentManager, null, list, lifecycleOwner, callback);
    }

    public static void showList(FragmentManager fragmentManager,
                                String title,
                                ArrayList<String> list,
                                @NonNull LifecycleOwner lifecycleOwner,
                                IResultCallback callback) {
        DialogFragment fragment = new ListDialogFragment();

        Bundle args = new Bundle();
        args.putStringArrayList(KEY_LIST, list);
        args.putString(KEY_TITLE, title);
        fragment.setArguments(args);

        fragmentManager
                .setFragmentResultListener(
                        TAG,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (TAG.equals(requestKey)) {
                                callback.onResult(result.getInt(KEY_INDEX, -1), result.getString(KEY_RESULT));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }
}
