package com.bingce.controlnetwork.fragment

import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.bingce.controlapphelper.datasource.database.ProjectType
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.route.LevelRouteRecord
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord
import com.bingce.controlapphelper.datasource.database.route.RouteRecord
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentProjectSurveyorStationListLayoutBinding
import com.bingce.controlnetwork.fragment.base.BaseFragment
import com.bingce.controlnetwork.fragment.bottompage.BottomBean
import com.bingce.controlnetwork.fragment.bottompage.BottomPageFragment
import com.bingce.controlnetwork.fragment.bottompage.IData
import com.bingce.controlnetwork.func.level.fragment.LevelSurveyorHomeFragment
import com.bingce.controlnetwork.newui.stationlist.StationListFragment
import com.bingce.controlnetwork.newui.stationlist.StationListVm
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 测量首页
 */
class MainSurveyFragment : BaseFragment() {

    private val mBinding get() = _binding!!

    private var _binding: FragmentProjectSurveyorStationListLayoutBinding? = null

    private val scope = MainScope()

    //筛选项目的项目类型
    private var selectProjectType: String? = null

    private val stationListVm by viewModels<StationListVm>()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding =
            FragmentProjectSurveyorStationListLayoutBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
        scope.cancel()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        SurveyorDatabaseFactory.instance
            .configDataSource
            .getLiveDataByKey(ConfigConstants.KEY_DEFAULT_PROJECT_ID)
            .observe(viewLifecycleOwner) { configRecord ->
                refreshUi(configRecord?.configValue)
            }

        initListener()
    }

    private fun initListener() {
        mBinding.title.setOnClickListener {
            BottomPageFragment.showBottom(
                childFragmentManager, viewLifecycleOwner,
                selectData,
                object : BottomPageFragment.CallBack {
                    override fun back(selectData: List<BottomBean>) {
                        scope.launch {
                            val title = getFilterTitle(
                                selectData[0].name,
                                selectData[1].name
                            )
                            if (ProjectType.isHeightTraverse(selectProjectType)) {
                                switchContent(
                                    title,
                                    SurveyorRouteStationFragment.newInstance(
                                        selectData[0].id,
                                        selectData[1].id
                                    )
                                )
                            } else if (ProjectType.isLevel(selectProjectType)) {
                                switchContent(
                                    title,
                                    LevelSurveyorHomeFragment.newInstance(
                                        selectData[0].id,
                                        selectData[1].id,
                                        false
                                    )
                                )
                            } else {
                                stationListVm.setArgument(selectData[0].id, selectData[1].id)
                                switchContent(
                                    title,
                                    StationListFragment()
                                )
                            }
                        }
                    }
                })
        }
    }

    private val selectData = object : IData {
        override suspend fun getData(bottomBean: BottomBean?): List<BottomBean> {
            val dataList: MutableList<BottomBean> = ArrayList()

            if (bottomBean == null) {
                //一级数据
                val projectRecords = SurveyorDatabaseFactory.instance
                    .projectDataSource
                    .dataList
                for (projectRecord in projectRecords) {
                    dataList.add(
                        BottomBean(
                            projectRecord.getId(), projectRecord.getProjectName(),
                            0
                        )
                    )
                }
            } else {
                if (bottomBean.pageIndex == 0) {
                    //二级数据
                    val projectRecord =
                        SurveyorDatabaseFactory.instance.projectDataSource.getRecord(bottomBean.id)
                    selectProjectType = projectRecord.getProjectType()
                    if (ProjectType.isHeightTraverse(selectProjectType)) {
                        val routeList = SurveyorDatabaseFactory.instance.getRouteDataSource()
                            .getByProjectId(bottomBean.id)
                        for (routeRecord in routeList) {
                            dataList.add(
                                BottomBean(
                                    routeRecord.getId(), routeRecord.name,
                                    1, tag = selectProjectType
                                )
                            )
                        }
                    } else if (ProjectType.isLevel(selectProjectType)) {
                        val routeList = LevelingDbFactory.instance.levelDataSource
                            .getByProjectId(bottomBean.id)
                        for (routeRecord in routeList) {
                            dataList.add(
                                BottomBean(
                                    routeRecord.getId(), routeRecord.name,
                                    1, tag = selectProjectType
                                )
                            )
                        }
                    } else {
                        val periodRecords = SurveyorDatabaseFactory.instance
                            .surveyorPeriod.getDataListSyncByProjectId(bottomBean.id)
                        for (periodItem in periodRecords) {
                            dataList.add(
                                BottomBean(
                                    periodItem.getId(), periodItem.getName(),
                                    1, tag = selectProjectType
                                )
                            )
                        }
                    }
                }
            }

            return dataList
        }

        override fun getTitle(pageIndex: Int): String {
            return if (pageIndex == 0) {
                Tools.getString(R.string.please_select_an_project)
            } else if (selectProjectType != null && pageIndex == 1) {
                if (ProjectType.isHeightTraverse(selectProjectType)
                    || ProjectType.isLevel(selectProjectType)
                ) {
                    Tools.getString(R.string.please_select_an_route)
                } else {
                    Tools.getString(R.string.please_select_an_period)
                }
            } else {
                ""
            }
        }
    }

    /**
     * 刷新页面
     */
    private fun refreshUi(projectId: String?) {
        scope.launch {
            val projectRecord = getProjectRecord(projectId)
            if (projectRecord == null) {
                switchContent(null, null)
                return@launch
            }
            val projectType = projectRecord.getProjectType()
            if (ProjectType.isHeightTraverse(projectType)) {
                loadHeightTraverseData(projectRecord)
            } else if (ProjectType.isLevel(projectType)) {
                loadLevelData(projectRecord)
            } else {
                loadNotHeightTraverseData(projectRecord)
            }
        }
    }

    /**
     * 加载三角导线和联系测量数据
     */
    private suspend fun loadNotHeightTraverseData(projectRecord: ProjectRecord) {
        val cycleRecord = getPeriodRecord(projectRecord.getId())

        val fragment =
            if (cycleRecord == null)
                EmptyFragment()
            else {
                stationListVm.setArgument(projectRecord.getId(), cycleRecord.getId())
                StationListFragment()

//                ComponentSurveyorStationListFragment.newInstance(
//                    projectRecord.getId(),
//                    cycleRecord.getId(),
//                    false
//                )
            }
        switchContent(
            getFilterTitle(projectRecord.getProjectName(), cycleRecord?.getName()),
            fragment
        )
    }


    /**
     * 加载高程导线数据
     */
    private suspend fun loadHeightTraverseData(projectRecord: ProjectRecord) {
        val routeRecord = getRouteRecord(projectRecord.getId())

        val fragment =
            if (routeRecord == null)
                EmptyFragment()
            else SurveyorRouteStationFragment.newInstance(
                projectRecord.getId(),
                routeRecord.getId()
            )

        switchContent(
            getFilterTitle(projectRecord.getProjectName(), routeRecord?.routeName),
            fragment
        )
    }

    private suspend fun loadLevelData(projectRecord: ProjectRecord) {
        val routeRecord = getLevelRouteRecord(projectRecord.getId())
        val fragment =
            if (routeRecord == null)
                EmptyFragment()
            else LevelSurveyorHomeFragment.newInstance(
                projectRecord.getId(),
                routeRecord.getId(),
                false
            )

        switchContent(
            getFilterTitle(projectRecord.getProjectName(), routeRecord?.routeName),
            fragment
        )
    }

    private fun getFilterTitle(projectName: String, secondName: String?): String {
        val suffix = if (secondName == null) "" else " > $secondName"
        return projectName + suffix
    }

    private suspend fun switchContent(title: String?, fragment: Fragment?) {
        withContext(Dispatchers.Main) {
            setFilterTitle(title)
            val targetFragment = fragment ?: EmptyFragment()
            childFragmentManager
                .beginTransaction()
                .replace(R.id.fragment_container, targetFragment, null)
                .commit()
        }
    }

    private fun setFilterTitle(title: String?) {
        if (TextUtils.isEmpty(title)) {
            mBinding.title.visibility = View.INVISIBLE
        } else {
            mBinding.title.visibility = View.VISIBLE
            mBinding.titleView.text = title
        }
    }

    private suspend fun getProjectRecord(projectId: String?): ProjectRecord? {
        return withContext(Dispatchers.IO) {
            val projectRecord =
                SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId)
            if (projectRecord != null) {
                return@withContext projectRecord
            }
            SurveyorDatabaseFactory.instance.projectDataSource.defaultRecordSync
        }
    }

    private suspend fun getPeriodRecord(
        projectId: String,
    ): SurveyorPeriodRecord? {
        return withContext(Dispatchers.IO) {
            val defaultPeriodId = SurveyorDatabaseFactory.instance
                .configDataSource.getByKeySync(ConfigConstants.KEY_DEFAULT_PERIOD_ID)?.configValue
            val periodRecord =
                SurveyorDatabaseFactory.instance.surveyorPeriod.getByIdAndProjectId(
                    defaultPeriodId,
                    projectId
                )
            if (periodRecord != null) {
                return@withContext periodRecord
            }
            SurveyorDatabaseFactory.instance.surveyorPeriod.getDefaultSync(projectId)
        }
    }

    private suspend fun getRouteRecord(
        projectId: String,
    ): RouteRecord? {
        return withContext(Dispatchers.IO) {
            val defaultRouteId = SurveyorDatabaseFactory.instance
                .configDataSource.getByKeySync(ConfigConstants.KEY_DEFAULT_ROUTE_ID)?.configValue
            val routeRecord =
                SurveyorDatabaseFactory.instance.getRouteDataSource()
                    .getByIdAndProjectId(defaultRouteId, projectId)
            if (routeRecord != null) {
                return@withContext routeRecord
            }
            SurveyorDatabaseFactory.instance.getRouteDataSource()
                .getDefault(projectId)
        }
    }

    private suspend fun getLevelRouteRecord(
        projectId: String,
    ): LevelRouteRecord? {
        return withContext(Dispatchers.IO) {
            val defaultRouteId = SurveyorDatabaseFactory.instance
                .configDataSource.getByKeySync(ConfigConstants.KEY_DEFAULT_ROUTE_ID)?.configValue
            val routeRecord =
                LevelingDbFactory.instance.levelDataSource
                    .getByIdSync(defaultRouteId)
            if (routeRecord != null) {
                return@withContext routeRecord
            }
            LevelingDbFactory.instance.levelDataSource
                .getDefault(projectId)
        }
    }

}