package com.bingce.controlnetwork.fragment;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ExpandableListView;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.IPointDataSource;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.KnownAndUnknownPoints;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.AbstractPointSelectAdapter;
import com.bingce.controlnetwork.adapter.PointSelectAdapter;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;


public class PickPointFragment extends BaseFragment {
    public static final String TAG = PickPointFragment.class.getName();
    AbstractPointSelectAdapter adapter;

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        //读取数据类型
        int maxCount = -1;
        if (getArguments() != null) {
            maxCount = getArguments().getInt(KEY_MAX_COUNT, -1);
        }
        View view = inflater.inflate(R.layout.fragment_pick_point_bottom_sheet_layout, container, false);
        if (view instanceof ExpandableListView) {
            ExpandableListView expandableListView = (ExpandableListView) view;
            String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            List<String> filterId = getArguments().getStringArrayList(KEY_FILTER);
            PickPointResult pickPointResult = getArguments().getParcelable(BundleConstants.KEY_RESULT);

            int finalMaxCount1 = maxCount;
            ThreadPoolUtil.execute(() -> {
                IPointDataSource pointDataSource =
                        SurveyorDatabaseFactory.instance.getPointDataSource();
                //去掉全局控制点
//                List<PointRecordTj> globalControlPoints = pointDataSource
//                        .getListByProjectId(PointRecordTj.GLOBAL_CONTROL_POINT_PROJECT_ID, PointRecordTj.TYPE_GLOBAL_POINT);

                KnownAndUnknownPoints knownAndUnknownPoints =
                        KnownAndUnknownPoints.loadData(false, filterId, projectId);

                List<String> filterName = new ArrayList<>();
                if (filterId != null) {
                    for (String id : filterId) {
                        PointRecord record = pointDataSource.findByIdSync(id);
                        if (record == null) {
                            continue;
                        }
                        filterName.add(record.getName());
                    }
                }

                //过滤全局控制点
                List<PointRecord> others = new ArrayList<>();
                //去掉全局控制点
//                for (PointRecordTj pointRecord : globalControlPoints) {
//                    if ((filterId != null && filterId.contains(pointRecord.getId())) ||
//                            filterName.contains(pointRecord.getName()) ||
//                            knownAndUnknownPoints.containsKnownPoint(pointRecord)) {
//                        continue;
//                    }
//                    others.add(pointRecord);
//                }
                adapter = new PointSelectAdapter(knownAndUnknownPoints, others,
                        pickPointResult,
                        finalMaxCount1, getContext());
                runOnUI(() -> {
                    expandableListView.setAdapter(adapter);
                    int groupCount = adapter.getGroupCount();
                    for (int index = 0; index < groupCount; index++) {
                        expandableListView.expandGroup(index);
                    }
                });
            });
//            String pointType = getArguments().getString(KEY_POINT_TYPE);
//            if (PointRecord.TYPE_KNOWN_POINT.equals(pointType)) {
//                KnownAndUnknownPoints.loadKnownData(filterId, projectId, instance -> runOnUI(() -> {
//                    adapter = new KnownAndUnknownPointSelectAdapter(instance, pickPointResult,
//                            finalMaxCount1, getContext());
//                    expandableListView.setAdapter(adapter);
//                    int groupCount = adapter.getGroupCount();
//                    for (int index = 0; index < groupCount; index++) {
//                        expandableListView.expandGroup(index);
//                    }
//                }));
//            } else {
//                KnownAndUnknownPoints.loadKnownAndUnknownData(filterId, projectId, instance -> runOnUI(() -> {
//                    adapter = new KnownAndUnknownPointSelectAdapter(instance, pickPointResult,
//                            finalMaxCount1, getContext());
//                    expandableListView.setAdapter(adapter);
//                    int groupCount = adapter.getGroupCount();
//                    for (int index = 0; index < groupCount; index++) {
//                        expandableListView.expandGroup(index);
//                    }
//                }));
//            }
        }
        setHasOptionsMenu(true);
        return view;
    }

    @Override
    public void onCreateOptionsMenu(@NonNull @NotNull Menu menu, @NonNull @NotNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        if (item.getItemId() == R.id.menu_confirm) {
            Intent intent = new Intent();
            PickPointResult pickPointResult = new PickPointResult();
            pickPointResult.setSelectedIds(adapter.getSelectedId());
            pickPointResult.setSelectedNames(adapter.getSelectedName());
            intent.putExtra(BundleConstants.KEY_RESULT, pickPointResult);
            if (getArguments() != null) {
                intent.putExtra(KEY_TYPE, getArguments().getString(KEY_TYPE));
            }
            Activity activity = getActivity();
            if (activity != null) {
                activity.setResult(Activity.RESULT_OK, intent);
                activity.finish();
            }
        }
        return super.onOptionsItemSelected(item);
    }

    public static final String KEY_TYPE = "__TYPE";
    private static final String KEY_FILTER = "__filter__id";
    private static final String KEY_MAX_COUNT = "__key__max_count";
    private static final String KEY_POINT_TYPE = "__point_type";

    public static PickPointFragment newInstance(Bundle args) {
        PickPointFragment pickPointFragment = new PickPointFragment();
        pickPointFragment.setArguments(args);
        return pickPointFragment;
    }

    public static ActivityResultLauncher<Intent> createActivityLauncher(Fragment fragment, IOnSelectCallback selectCallback) {
        return fragment.registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
            if (result != null && Activity.RESULT_OK == result.getResultCode()) {
                Intent intent = result.getData();
                if (intent != null) {
                    PickPointResult pickPointResult = intent.getParcelableExtra(BundleConstants.KEY_RESULT);
                    String type = intent.getStringExtra(KEY_TYPE);
                    selectCallback.onSelect(type, pickPointResult);
                }
            }
        });
    }

    public static void pickKnownPoint( Context context,
                                      String type,
                                      int maxCount,
                                      String title,
                                      ArrayList<String> filterPointId, String projectId,
                                      PickPointResult result, ActivityResultLauncher<Intent> launcher) {
        pickPoint( context,
                type, maxCount, title, filterPointId, projectId, result, launcher);
    }

    public static void pickKnownAndUnknownPoint(Context context,
                                                String type,
                                                int maxCount,
                                                String title,
                                                List<String> filterPointId, String projectId,
                                                PickPointResult result, ActivityResultLauncher<Intent> launcher) {
        pickPoint(context,
                type, maxCount, title, filterPointId, projectId, result, launcher);
    }

    public static void pickPoint(Context context,
                                  String type,
                                  int maxCount,
                                  String title,
                                  List<String> filterPointId, String projectId,
                                  PickPointResult result, ActivityResultLauncher<Intent> launcher) {
//        ArrayList<String> list = new ArrayList<>();
//        list.add(context.getString(R.string.global_control_point));
//        if (knownOnly) {
//            list.add(context.getString(R.string.project_known_point));
//        } else {
//            list.add(context.getString(R.string.project_known_unknown_point));
//        }
//        ListDialogFragment.showList(fragmentManager, list, lifecycleOwner, (index, value) -> {
        if (launcher == null) {
            return;
        }
        Bundle args = new Bundle();
//            if (index == 0) {
//                args.putString(BundleConstants.KEY_PROJECT_ID, PointRecord.GLOBAL_CONTROL_POINT_PROJECT_ID);
//            } else {
        args.putString(BundleConstants.KEY_PROJECT_ID, projectId);
//            }
//            if (knownOnly) {
//                args.putString(KEY_POINT_TYPE, PointRecord.TYPE_KNOWN_POINT);
//            }
        args.putString(KEY_TYPE, type);
        args.putInt(KEY_MAX_COUNT, maxCount);
        args.putStringArrayList(KEY_FILTER, (ArrayList<String>) filterPointId);
        args.putParcelable(BundleConstants.KEY_RESULT, result);
        launcher.launch(FragmentContainerActivity.startIntent(context, TAG, title, args));
//        });
    }

    public interface IOnSelectCallback {
        void onSelect(String type, PickPointResult result);
    }
}