package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlapphelper.widget.swiperecyclerview.widget.DefaultItemDecoration;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.databinding.NewFragmentPointListBinding;
import com.bingce.controlnetwork.newui.adapter.PointListAdapter;
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding;
import com.bingce.controlnetwork.newui.createeditpoint.CreateEditPointActvity;
import com.bingce.controlnetwork.newui.widget.swipemenu.SwipeMenuEditDelete;
import com.bingce.controlnetwork.util.DialogUtil;
import com.bingce.controlnetwork.util.ImportPointsUtilPla;
import com.bingce.utils.ThreadPoolUtil;

import java.util.Collections;
import java.util.List;


/**
 * 点列表
 */
public class PointListFragment extends BaseFragmentBinding<NewFragmentPointListBinding> {
    public static final String TAG = PointListFragment.class.getName();
    private final static String KEY_TYPE = "___KEY___";

    //    private PointRecordAdapter adapter;
    private PointListAdapter adapter;
    @NonNull
    private final ImportPointsUtilPla importPointsUtil;

    /**
     * 去未知点列表
     *
     * @param projectId
     * @param context
     */
    public static void go2UnknownPointList(String projectId, String projectType, Context context) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_TYPE, PointRecord.TYPE_UNKNOWN_POINT);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        FragmentContainerActivity.start(context, TAG, R.string.project_unknown_points, bundle);
    }

    /**
     * 去已知点列表
     *
     * @param projectId
     * @param context
     */
    public static void go2KnownPointList(String projectId, String projectType, Context context) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_TYPE, PointRecord.TYPE_KNOWN_POINT);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        FragmentContainerActivity.start(context, TAG, R.string.project_known_points, bundle);
    }

    /**
     * 去全局控制点
     *
     * @param context
     */
    public static void go2GlobalPointList(Context context) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_TYPE, PointRecord.TYPE_GLOBAL_POINT);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, PointRecord.GLOBAL_CONTROL_POINT_PROJECT_ID);
        FragmentContainerActivity.start(context, TAG, R.string.global_control_point, bundle);
    }

    public static PointListFragment newInstance(Bundle bundle) {
        PointListFragment fragment = new PointListFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    public PointListFragment() {
        importPointsUtil = new ImportPointsUtilPla(this);
    }

    /**
     * 更新点列表
     *
     * @param pointRecords
     */
    private void updateData(List<PointRecord> pointRecords) {
        //排序
        Collections.sort(pointRecords, (o1, o2) -> {
            if (o1.getCreateDate().before(o2.getCreateDate())) {
                return -1;
            } else if (o1.getCreateDate().after(o2.getCreateDate())) {
                return 1;
            }
            if (o1.getOrderIndex() < o2.getOrderIndex()) {
                return -1;
            } else if (o1.getOrderIndex() > o2.getOrderIndex()) {
                return 1;
            }
            return 0;
        });

        adapter.refreshData(pointRecords);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_import_and_clear, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        int id = item.getItemId();
        if (getArguments() == null) {
            return super.onOptionsItemSelected(item);
        }
        String pointType = getPonitType();
        String projectId = getProjectId();
        if (id == R.id.menu_item_import) {
            //导入点
            importPoints(pointType, projectId);
        } else if (id == R.id.menu_item_clear) {
            //清空点
            clearPoints(pointType, projectId);
        }
        return super.onOptionsItemSelected(item);
    }

    private String getPonitType() {
        return requireArguments().getString(KEY_TYPE, PointRecord.TYPE_GLOBAL_POINT);
    }

    private String getProjectId() {
        return requireArguments().getString(BundleConstants.KEY_PROJECT_ID);
    }

    private String getProjectType() {
        return requireArguments().getString(BundleConstants.KEY_PROJECT_TYPE);
    }

    /**
     * 创建点
     */
    private void createPoints(String pointType, String projectId, String projectType) {
        goCreateEditPointActvity(projectId, projectType, pointType, null);
    }

    private void goCreateEditPointActvity(String projectId, String projectType, String pointType, PointRecord bean) {
        CreateEditPointActvity.launch(requireContext(), projectId, projectType,
                pointType, bean);
    }

    /**
     * 清空点
     */
    private void clearPoints(String pointType, String projectId) {
        String hint = "";
        if (PointRecord.TYPE_KNOWN_POINT.equals(pointType)) {
            hint = getString(R.string.tips_ready_2_clear_all_known_point);
        } else if (PointRecord.TYPE_UNKNOWN_POINT.equals(pointType)) {
            hint = getString(R.string.tips_ready_2_clear_all_unknown_point);
        }
        DialogUtil.showConfirmCountDownDialog(getChildFragmentManager(), getViewLifecycleOwner(),
                hint,
                Tools.getString(R.string.confirm), () -> SurveyorDatabaseFactory
                        .instance
                        .getPointDataSource()
                        .clearByProjectId(projectId, pointType));
    }

    /**
     * 导入点
     */
    private void importPoints(String pointType, String projectId) {
        importPointsUtil.showImportDialog(projectId, pointType);
    }

    @Override
    protected NewFragmentPointListBinding getViewBinding(@NonNull LayoutInflater inflater, @Nullable ViewGroup container) {
        return NewFragmentPointListBinding.inflate(inflater, container, false);
    }

    @Override
    protected void initView() {
        setHasOptionsMenu(true);
        initAapter();
        initListener();
    }

    private void initListener() {
        mBinding.tvNewPoint.setOnClickListener((v) -> {
            //新建点
            if (getArguments() == null) {
                return;
            }
            createPoints(getPonitType(), getProjectId(), getProjectType());
        });
    }

    private void initAapter() {
        mBinding.ilSwipeList.swipeRecyclerView.addItemDecoration(new DefaultItemDecoration(Tools.getColor(R.color.transparent), 1, 1));
        mBinding.ilSwipeList.swipeRecyclerView.setSwipeMenuCreator(new SwipeMenuEditDelete(requireContext()));
        mBinding.ilSwipeList.swipeRecyclerView.setOnItemMenuClickListener((menuBridge, adapterPosition) -> {
            menuBridge.closeMenu();
            dealMenuClick(menuBridge.getPosition(), adapter.getBean(adapterPosition));
        });
        adapter = new PointListAdapter(requireContext());
        adapter.setProjectType(getProjectType());
        mBinding.ilSwipeList.swipeRecyclerView.setAdapter(adapter);
    }

    /**
     * @param menuPostion 编辑或删除的position
     * @param bean        数据模型
     */
    private void dealMenuClick(int menuPostion, PointRecord bean) {
        if (SwipeMenuEditDelete.isEditMenu(menuPostion)) {
            startEdit(bean);
        } else if (SwipeMenuEditDelete.isDeleteMenu(menuPostion)) {
            startDelete(bean);
        }
    }

    private void startDelete(PointRecord bean) {
        SurveyorDatabaseFactory
                .instance
                .getPointDataSource()
                .delete(bean);
    }

    private void startEdit(PointRecord bean) {
        goCreateEditPointActvity(getProjectId(), getProjectType(), getPonitType(), bean);
    }


    @Override
    protected void initData() {
        if (getArguments() == null) {
            return;
        }
        String pointType = getArguments().getString(KEY_TYPE, PointRecord.TYPE_GLOBAL_POINT);
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);

        String projectType = getProjectType();
        ThreadPoolUtil.execute(() -> {
            //是否是平面坐标
            boolean isPlatCoordinate = true;
            if (ProjectType.isHeightTraverse(projectType)) {
                isPlatCoordinate = false;
            }
            adapter.setIsPlatCoordinate(isPlatCoordinate);
            getData(projectId, pointType);
        });
    }

    private void getData(String projectId, String pointType) {
        ThreadPoolUtil.executeInMain(() -> SurveyorDatabaseFactory
                .instance
                .getPointDataSource()
                .getLiveDataListByProject(projectId, pointType)
                .observe(getViewLifecycleOwner(), this::updateData));
    }
}
