package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.prism.IPrismManagerDataSource;
import com.bingce.controlapphelper.datasource.database.prism.PrismConstants;
import com.bingce.controlapphelper.datasource.database.prism.PrismRecord;
import com.bingce.controlnetwork.adapter.PrismMagegeAdapter;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.PrismMagegeAdapter;
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDeleteEdit;
import com.bingce.controlnetwork.fragment.base.BaseFragment;

/**
 * 棱镜管理
 */
public class PrismManageFragment extends BaseFragment {

    public static final String TAG = PrismManageFragment.class.getName();
    private PrismMagegeAdapter adapter = null;
    private RecyclerView recyclerView;


    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);

    }

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_prism_manager, container, false);
        recyclerView = rootView.findViewById(R.id.list);
        View createButton = rootView.findViewById(R.id.create_new);
        if (createButton != null) {
            createButton.setOnClickListener(this::createCustom);
        }
        return rootView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initData();
    }

    private void initData() {
        IPrismManagerDataSource prismManagerDataSource = SurveyorDatabaseFactory.instance.getPrismManagerDataSource();
        prismManagerDataSource.getAllRecordLiveData().observe(getViewLifecycleOwner(), prismRecords -> {
            if (prismRecords == null) return;
            if (adapter == null) {
                adapter = new _Adapter(
                        prismRecords,
                        item -> {

                        });
                recyclerView.setAdapter(adapter);
            } else {
                adapter.updateData(prismRecords);
            }


        });


    }

    /**
     * 创建自定义
     *
     * @param view
     */
    private void createCustom(View view) {
        PrismManagerCreateCustomFragment.create(getChildFragmentManager(), getViewLifecycleOwner(), (nameBefore, name, typeLarge, constant) -> {
            //选择完的结果
            ThreadPoolUtil.execute(() -> {
                PrismRecord record = PrismRecord.createRecord(name, PrismConstants.TYPE_USER, constant, PrismRecord.MODIFY_ALLOW, typeLarge);
                SurveyorDatabaseFactory.instance.getPrismManagerDataSource().saveRecord(record);
            });
        });
    }

    private void editCustom(PrismRecord item) {
        PrismManagerCreateCustomFragment.edit(getChildFragmentManager(),
                getViewLifecycleOwner(),
                item.prismTypeName, item.prismTypeLarge, item.prismTypeConstant,
                (nameBefore, name, typeLarge, constant) -> ThreadPoolUtil.execute(() -> {
                    IPrismManagerDataSource prismManagerDataSource = SurveyorDatabaseFactory.instance.getPrismManagerDataSource();
                    PrismRecord record = prismManagerDataSource.getPrismTypeNameSync(nameBefore);
                    if (record != null) {
                        record.prismTypeName = name;
                        record.prismType = PrismConstants.TYPE_USER;
                        record.prismTypeConstant = constant;
                        record.prismTypeLarge = typeLarge;
                        prismManagerDataSource.update(record);
                    }
                }));
    }

    private class _Adapter extends PrismMagegeAdapter {
        public _Adapter(
                List<PrismRecord> records,
                IOnItemClick<PrismRecord> callback) {
            super(records == null ? new ArrayList<>() : records,
                    callback,
                    new ContextMenuDeleteEdit<PrismRecord>() {
                        @Override
                        protected void delete(PrismRecord item) {
                            ThreadPoolUtil.execute(() -> {
                                IPrismManagerDataSource prismManagerDataSource = SurveyorDatabaseFactory.instance.getPrismManagerDataSource();
                                prismManagerDataSource.delete(item);
                            });
                        }

                        @Override
                        protected void edit(PrismRecord item) {
                            editCustom(item);
                        }
                    });
        }


    }

    /**
     * 启动方法
     *
     * @param context
     */
    public static void start(Context context) {
        Bundle bundle = new Bundle();
        FragmentContainerActivity.start(context, TAG, R.string.prism_manager, bundle);
    }

    public static PrismManageFragment newInstance(Bundle bundle) {
        PrismManageFragment fragment = new PrismManageFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
