package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.prism.IPrismManagerDataSource;
import com.bingce.controlapphelper.datasource.database.prism.PrismConstants;
import com.bingce.controlapphelper.datasource.database.prism.PrismRecord;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.SoftKeyUtils;
import com.bingce.utils.ThreadPoolUtil;
import com.rengwuxian.materialedittext.MaterialEditText;

import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 新建自定义棱镜类型
 */
public class PrismManagerCreateCustomFragment extends DialogFragment {

    private final String[] spinnerData = {PrismConstants.TYPE_PRISM_LARGE, PrismConstants.TYPE_TAPE_LARGE};

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setStyle(STYLE_NORMAL, R.style.Dialog_FullScreen);
    }

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View contentView = inflater.inflate(R.layout.fragment_prism_manager_create_custom, container, false);
        if (contentView != null) {
            contentView.setOnClickListener(v -> {
                SoftKeyUtils.hideSoftKey();
                dismiss();
            });
            String type = VALUE_NEW;
            String prismTypeName = "";
            String prismTypeType = "";
            String prismTypeConstant = "";
            if (getArguments() != null) {
                type = getArguments().getString(KEY_TYPE, VALUE_NEW);
                prismTypeName = getArguments().getString(KEY_PRISM_TYPE_NAME);
                prismTypeType = getArguments().getString(KEY_PRISM_TYPE_LARGE);
                prismTypeConstant = getArguments().getString(KEY_PRISM_TYPE_CONSTANT);
            }
            //设置标题
            TextView title = contentView.findViewById(R.id.title);
            if (title != null) {
                if (VALUE_NEW.equals(type)) {
                    title.setText(Tools.getString(R.string.text_create_new_surveyor_station));
                } else if (VALUE_EDIT.equals(type)) {
                    title.setText(Tools.getString(R.string.menu_edit));
                }
            }

            MaterialEditText etName = contentView.findViewById(R.id.etName);
            Spinner spinner = contentView.findViewById(R.id.spinner);
            MaterialEditText etContant = contentView.findViewById(R.id.etContant);

            List<String> spinnerDataList = Arrays.asList(spinnerData);
            spinner.setAdapter(new ArrayAdapter<>(getContext(), android.R.layout.simple_list_item_1, spinnerDataList));

            //编辑的时候重新赋值
            if (VALUE_EDIT.equals(type)) {
                //设置值
                etName.setText(prismTypeName);
                if (!TextUtils.isEmpty(prismTypeName)) {
                    etName.setSelection(prismTypeName.length());
                }
                etContant.setText(prismTypeConstant);
                spinner.setSelection(spinnerDataList.indexOf(prismTypeType));
            }

            IPrismManagerDataSource prismManagerDataSource = SurveyorDatabaseFactory.instance.getPrismManagerDataSource();

            Button confirm = contentView.findViewById(R.id.confirm);
            if (confirm != null) {
                String finalPrismTypeNameBefore = prismTypeName;
                String finalType = type;
                confirm.setOnClickListener(v -> {
                    String name = string(etName);
                    if (name.isEmpty()) {
                        ToastUtils.showShort(R.string.name_is_required);
                        return;
                    }
                    String constant = string(etContant);
                    if (constant.isEmpty()) {
                        ToastUtils.showShort(R.string.absolute_constants_cannot_be_empty);
                        return;
                    }
                    double constantD = Double.parseDouble(constant);
                    if (constantD > 0) {
                        ToastUtils.showShort(R.string.absolute_constants_must_be_negative);
                        return;
                    }

                    String selectedItem = (String) spinner.getSelectedItem();

                    //检测名称是否重复
                    ThreadPoolUtil.execute(() -> {
                        PrismRecord prismRecord = prismManagerDataSource.getPrismTypeNameSync(name);

                        if (VALUE_NEW.equals(finalType)) {
                            if (prismRecord != null) {
                                //点名重复，提示
                                TipsFragment.tipsOnly(
                                        getChildFragmentManager(),
                                        getViewLifecycleOwner(),
                                        getContext(),
                                        Tools.getString(R.string.custom_name_cannot_be_repeated));
                                return;
                            }
                        }

                        //1.新建没有重名
                        //2.编辑
                        runOnUI(() -> {
                            onCallback(finalPrismTypeNameBefore, name, selectedItem, constant, getParentFragmentManager());
                            SoftKeyUtils.hideSoftKey();
                            dismiss();
                        });


                    });
                });
            }
            Button cancel = contentView.findViewById(R.id.cancel);
            if (cancel != null) {
                cancel.setOnClickListener(v -> {
                    SoftKeyUtils.hideSoftKey();
                    dismiss();
                });
            }

        }
        return contentView;
    }

    private void runOnUI(Runnable runnable) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(runnable);
        }
    }

    private PrismManagerCreateCustomFragment() {
    }

    private static void onCallback(
            final String nameBefore,
            final String name,
            final String type,
            final String constant,
            final FragmentManager fragmentManager) {
        Bundle args = new Bundle();

        args.putString(KEY_PRISM_TYPE_NAME_BEFORE, nameBefore);
        args.putString(KEY_PRISM_TYPE_NAME, name);
        args.putString(KEY_PRISM_TYPE_LARGE, type);
        args.putString(KEY_PRISM_TYPE_CONSTANT, constant);

        fragmentManager.setFragmentResult(KEY_REQUEST_INPUT, args);
    }

    private static String string(MaterialEditText editText) {
        if (editText == null || editText.getText() == null) {
            return "";
        }
        return editText.getText().toString().trim();
    }

    private static double doubleValue(MaterialEditText editText) {
        String string = string(editText);
        if (string.isEmpty()) {
            return 0;
        }
        return Double.parseDouble(string);
    }

    private static void setString(MaterialEditText editText, Bundle args, String key) {
        if (editText == null || args == null) {
            return;
        }
        editText.setText(args.getString(key));
    }


    private static final String KEY_PRISM_TYPE_NAME = "KEY_PRISM_TYPE_NAME";
    private static final String KEY_PRISM_TYPE_NAME_BEFORE = "KEY_PRISM_TYPE_NAME_BEFORE";
    private static final String KEY_PRISM_TYPE_LARGE = "KEY_PRISM_TYPE_LARGE";
    private static final String KEY_PRISM_TYPE_CONSTANT = "KEY_PRISM_TYPE_CONSTANT";

    private static final String KEY_TYPE = "KEY_TYPE";
    private static final String VALUE_NEW = "___new___";
    private static final String VALUE_EDIT = "___edit___";

    private static final String KEY_REQUEST_INPUT = "___request__code___";

    public static void create(FragmentManager fragmentManager,
                              LifecycleOwner lifecycleOwner,
                              @Nullable ICallback callback) {
        Bundle args = new Bundle();
        show(fragmentManager, lifecycleOwner, args, callback);
    }


    public static void edit(FragmentManager fragmentManager,
                            LifecycleOwner lifecycleOwner,
                            String prismTypeName,
                            String prismTypeLarge,
                            String prismTypeConstant,
                            @Nullable ICallback callback) {
        Bundle args = new Bundle();
        args.putString(KEY_PRISM_TYPE_NAME, prismTypeName);
        args.putString(KEY_PRISM_TYPE_LARGE, prismTypeLarge);
        args.putString(KEY_PRISM_TYPE_CONSTANT, prismTypeConstant);
        args.putString(KEY_TYPE, VALUE_EDIT);

        show(fragmentManager, lifecycleOwner, args, callback);
    }


    private static void show(FragmentManager fragmentManager,
                             LifecycleOwner lifecycleOwner,
                             @Nullable Bundle args,
                             @Nullable ICallback callback) {
        PrismManagerCreateCustomFragment fragment = new PrismManagerCreateCustomFragment();
        if (args != null) {
            fragment.setArguments(args);
        }
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_INPUT,
                        lifecycleOwner,
                        (key, result) -> {
                            if (KEY_REQUEST_INPUT.equals(key)) {
                                if (callback == null) {
                                    return;
                                }
                                callback.onGet(
                                        result.getString(KEY_PRISM_TYPE_NAME_BEFORE, ""),
                                        result.getString(KEY_PRISM_TYPE_NAME, ""),
                                        result.getString(KEY_PRISM_TYPE_LARGE, ""),
                                        result.getString(KEY_PRISM_TYPE_CONSTANT, "")
                                );
                            }
                        });
        fragment.show(fragmentManager, PrismManagerCreateCustomFragment.class.getName());
    }

    public interface ICallback {
        void onGet(String nameBefore, String name, String typeLarge, String constant);
    }

}
