package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.project.IProjectDataSource;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.ISurveyorPeriodDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.level.ToleranceLevelRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.util.SurveyorRecordUtil;
import com.bingce.controlapphelper.util.ToleranceUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDeleteEdit;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.projectdetail.ProjectDetailFragment;
import com.bingce.controlnetwork.fragment.projectedit.AbstractProjectEditFragment;
import com.bingce.controlnetwork.func.level.fragment.ProjectDetailLevelFragment;
import com.bingce.controlnetwork.newui.adapter.ProjectListAdapter;
import com.bingce.controlnetwork.newui.projectedit.ProjectCreateEditActivity;
import com.bingce.controlnetwork.newui.widget.selectitem.BaseSelectItemFragment;
import com.bingce.controlnetwork.newui.widget.selectitem.HorDividerDecoration;
import com.bingce.controlnetwork.newui.widget.selectitem.SelectProjectTypeFragment;
import com.bingce.controlnetwork.util.DialogUtil;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.surveyor.agentweb.AgentWebActivity;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import jp.wasabeef.recyclerview.animators.FadeInLeftAnimator;

/**
 * 主页面-项目
 */
public class ProjectListFragment extends BaseFragment {
    private ProjectListAdapter projectListAdapter;
    private RecyclerView recyclerView;
    private View createNewProject;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_project_list, container, false);
        recyclerView = view.findViewById(R.id.recycler_view_project_list);
        createNewProject = view.findViewById(R.id.create_new);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView();
        initListener();
        initObserver();
    }

    private void initAdapter() {
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setItemAnimator(new FadeInLeftAnimator());
        recyclerView.addItemDecoration(new HorDividerDecoration(requireContext()));

        projectListAdapter = new ProjectListAdapter(requireContext());
        projectListAdapter.setAdapterContextMenu(new ContextMenuDeleteEdit<ProjectRecord>() {
            @Override
            protected void delete(ProjectRecord item) {
                startDeleteProject(item);
            }

            @Override
            protected void edit(ProjectRecord item) {
                startEditProject(item);
            }
        });
        projectListAdapter.setOnItemClickListener(this::goProjectDetail);
        recyclerView.setAdapter(projectListAdapter);
    }

    private void initListener() {
        //新建项目
        createNewProject.setOnClickListener(v -> {
            //选择项目类型
            BaseSelectItemFragment.select(new SelectProjectTypeFragment(),
                    getChildFragmentManager(),
                    getViewLifecycleOwner(),
                    value -> goCreateOrEditProject(value, null));
        });
    }

    /**
     * 去新建项目或编辑项目
     */
    private void goCreateOrEditProject(String projctType, ProjectRecord projectRecord) {
        if (ProjectType.isLevel(projctType)) {
            String projectId = null;
            if (projectRecord != null) {
                projectId = projectRecord.getId();
            }
            ProjectCreateEditActivity.launch(requireContext(), projctType, projectId);
        } else {
            if (projectRecord == null) {
                AbstractProjectEditFragment.createProject(getChildFragmentManager(), requireContext(), projctType);
            } else {
                AbstractProjectEditFragment.editProject(getChildFragmentManager(), requireContext(),
                        projectRecord.getId(), projctType, projectRecord.getToleranceId(), projectRecord.getProjectName());
            }
        }
    }

    private void initObserver() {
        SurveyorDatabaseFactory
                .instance
                .getProjectDataSource()
                .getLiveDataList()
                .observe(getViewLifecycleOwner(), this::addToleranceNameToProject);
    }


    private void addToleranceNameToProject(List<ProjectRecord> projectRecords) {
        ThreadPoolUtil.execute(() -> {
            for (ProjectRecord record : projectRecords) {
                String toleName = null;
                IToleranceOperate operate = ToleranceOperateUtil.getOperate(record.getToleranceId());
                if (operate != null) {
                    IToleranceDetail iToleranceDetail = operate.getIToleranceDetail(record.getToleranceId());
                    if (iToleranceDetail != null) {
                        ToleranceGroupRecord group = operate.getGroup(iToleranceDetail);
                        ToleranceLevelRecord level1 = operate.getLevel(iToleranceDetail);
                        toleName = ToleranceUtil.completedToleranceName(group, level1, iToleranceDetail);
                    }
                }
                record.toleranceName = toleName;
            }
            refreshAdapter(projectRecords);
        });
    }

    private void refreshAdapter(List<ProjectRecord> projectRecords) {
        ThreadPoolUtil.executeInMain(() -> {
            if (projectListAdapter != null) {
                projectListAdapter.refreshData(projectRecords);
            }
        });
    }

    private void initView() {
        initAdapter();
    }

    /**
     * 进入项目详情
     */
    private void goProjectDetail(ProjectRecord item) {
        if (ProjectType.isLevel(item.projectType)) {
            ProjectDetailLevelFragment.start(requireContext(), item.projectType, item.getId(), item.getName());
        } else {
            ProjectDetailFragment.start(requireContext(),
                    item.projectType,
                    item.getId(),
                    item.getName());
        }
    }

    @Override
    public void onCreateOptionsMenu(@NotNull Menu menu, @NotNull MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_activity_project_list, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.menu_activity_project_list_help) {
            AgentWebActivity.navigation2(getContext(), "android_url_activity_project_list_help");
        }
        return super.onOptionsItemSelected(item);
    }

    private void startEditProject(ProjectRecord item) {
        goCreateOrEditProject(item.projectType, item);
    }

    private void startDeleteProject(ProjectRecord item) {
        //删除的时候判断是否需要更新默认项目
        ThreadPoolUtil.execute(() -> {
            //检测是否有测量数据
            if (SurveyorRecordUtil.isProjectSurveyor(item.getId())) {
                //弹框提示
                runOnUI(() -> TipsFragment.cancelAbleTips(
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        requireContext(),
                        "要删除的项目存在测量数据，是否确认删除？",
                        4,
                        () -> ThreadPoolUtil.execute(() -> doDelete(item))));
            } else {
                ThreadPoolUtil.executeInMain(() -> {
                    DialogUtil.showConfirmDialog(getChildFragmentManager(), getViewLifecycleOwner(), "您确定要删除该项目吗？",
                            Tools.getString(R.string.confirm), () -> {
                                ThreadPoolUtil.execute(() -> {
                                    doDelete(item);
                                });
                            });
                });
            }
        });
    }

    @WorkerThread
    private void doDelete(ProjectRecord item) {
        IProjectDataSource projectDataSource = SurveyorDatabaseFactory.instance.getProjectDataSource();
        IConfigDataSource configDataSource = SurveyorDatabaseFactory.instance.getConfigDataSource();

        ConfigRecord defaultProject = configDataSource.getByKeySync(ConfigConstants.KEY_DEFAULT_PROJECT_ID);

        boolean needChangeDefaultProjectId = false;
        if (defaultProject == null || defaultProject.getConfigValue().equals(item.getId())) {
            needChangeDefaultProjectId = true;
        }

        //先删除项目及关联
        projectDataSource.delete(item);

        if (needChangeDefaultProjectId) {
            String projectId = "";
            String periodId = "";
            List<ProjectRecord> projectRecords = projectDataSource.getDataList();
            if (projectRecords != null) {
                ISurveyorPeriodDataSource surveyorPeriodDataSource = SurveyorDatabaseFactory.instance.getSurveyorPeriod();
                for (ProjectRecord projectRecord : projectRecords) {
                    if (projectRecord == null) {
                        continue;
                    }
                    List<SurveyorPeriodRecord> surveyorPeriodRecordList = surveyorPeriodDataSource.getDataListSyncByProjectId(projectRecord.getId());
                    for (SurveyorPeriodRecord surveyorPeriodRecord : surveyorPeriodRecordList) {
                        if (surveyorPeriodRecord == null) {
                            continue;
                        }
                        periodId = surveyorPeriodRecord.getId();
                        break;
                    }
                    if (!periodId.isEmpty()) {
                        projectId = projectRecord.getId();
                        break;
                    }
                }
            }
            configDataSource.set(ConfigConstants.KEY_DEFAULT_PROJECT_ID, projectId);
            configDataSource.set(ConfigConstants.KEY_DEFAULT_PERIOD_ID, periodId);
        }
    }

}
