package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.View;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.IProjectDataSource;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.SurveyorRecordUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.utils.ThreadPoolUtil;

import blankj.utilcode.util.ToastUtils;

/*
 * 项目限差详情，包含'LimitDetailFragment'+选择模板
 * */
public class ProjectToleranceSettingFragment extends AbstractProjectToleranceFragment {
    public static final int title = R.string.project_limite_detail;
    public static final String TAG = ProjectToleranceSettingFragment.class.getName();

    @Override
    protected void onCurrentToleranceChanged(String toleranceId) {
        super.onCurrentToleranceChanged(toleranceId);
        if (getArguments() == null) {
            return;
        }
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        ThreadPoolUtil.execute(() -> {
            IProjectDataSource projectDataSource =
                    SurveyorDatabaseFactory.instance.getProjectDataSource();
            ProjectRecord projectRecord = projectDataSource.getRecord(projectId);
            projectRecord.setToleranceId(toleranceId);
            projectDataSource.saveRecordSync(projectRecord);
        });
    }

    @Override
    protected void onCreateView(View view) {
        //读取当前项目配置的限差方案，并设置限差界面
        if (getArguments() == null) {
            return;
        }
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        ThreadPoolUtil.execute(() -> {
            IProjectDataSource projectDataSource = SurveyorDatabaseFactory
                    .instance
                    .getProjectDataSource();
            ProjectRecord projectRecord = projectDataSource.getRecord(projectId);
            if (projectRecord == null) {
                return;
            }
            IToleranceOperate operate = ToleranceOperateUtil.getOperate(projectRecord.getToleranceId());

            if (operate == null) {
                IToleranceDetail defaultRecord = ToleranceLoadUtil.getToleranceLoad(projectType).getNewDefaultTolerance(projectType);
                updateToleranceDetail(view, defaultRecord);
            } else {
                IToleranceDetail lastToleranceDetail = operate.getIToleranceDetail(projectRecord.getToleranceId());
                updateToleranceDetail(view, lastToleranceDetail);
            } 
        });
    }

    @Override
    protected void onSelectToleranceTemplateClick() {
        //读取当前项目配置的限差方案，并设置限差界面
        if (getArguments() == null) {
            return;
        }
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        //判断当前project是否存在已经测量的station
        ThreadPoolUtil.execute(() -> {
            if (SurveyorRecordUtil.isProjectSurveyor(projectId)) {
                ToastUtils.showShort(R.string.tips_tolerance_cannt_modify);
            } else {
                ProjectToleranceSettingFragment.super.onSelectToleranceTemplateClick();
            }
        });
    }

    public static void start(Context context, String projectId, String projectType) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        FragmentContainerActivity.start(context, TAG, title, bundle);
    }

    public static ProjectToleranceSettingFragment newInstance(Bundle bundle) {
        ProjectToleranceSettingFragment fragment = new ProjectToleranceSettingFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
