package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.os.Environment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.MainThread;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.DialogFragment;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.project.IProjectDataSource;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.schedule.ISurveyorScheduleDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.ISurveyorPeriodDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.surveyor.method.ExportUtils;
import com.bingce.controlapphelper.util.CheckStorageUtil;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.BuildConfig;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.surveyor.method.ExportUtilsPla;
import com.bingce.controlnetwork.ui.widget.treeview.TreeNode;
import com.bingce.controlnetwork.ui.widget.treeview.TreeView;
import com.bingce.controlnetwork.ui.widget.treeview.custom.MyNodeViewFactory;
import com.bingce.file.FileOperator;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.zip.ZipUtil;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import blankj.utilcode.util.FileUtils;
import blankj.utilcode.util.ToastUtils;

/**
 * 导线文件导出（基于选择的导线数据导出）
 */
public class SelectExportSurveyorRecordWireFileFragment extends BaseFragment {
    public static final String TAG = SelectExportSurveyorRecordWireFileFragment.class.getName();
    private TreeNode root;
    private TreeView treeView;
    private boolean isOpen = false;
    private Date currentDate, expireDate;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        root = TreeNode.root();
        if (getContext() != null) {
            treeView = new TreeView(root, getContext(), new MyNodeViewFactory());
        }
        View rootView = inflater.inflate(R.layout.fragment_select_export_surveyor_layout, container, false);
        View exportBtn = rootView.findViewById(R.id.export);
        exportBtn.setEnabled(true);
        exportBtn.setOnClickListener(v -> {
            if (ExportUtilsPla.isCanExport()) {
                requestStoragePermission();
            }
        });

        ThreadPoolUtil.execute(() -> {
            buildNodeTree(root);
            ThreadPoolUtil.executeInMain(() -> {
                RecyclerView list = rootView.findViewById(R.id.list);
                treeView.setup(list);
                list.setVisibility(View.VISIBLE);
                View loading = rootView.findViewById(R.id.loading);
                loading.setVisibility(View.GONE);
            });
        });
        return rootView;
    }

    @MainThread
    private static void doExport(TreeNode root,
                                 Context context,
                                 ICallback callback) {
        ThreadPoolUtil.execute(() -> {
            //层遍历node树，找到所有被选中的station
            List<ExportUtils.ExportItem> exportItem = new ArrayList<>();
            List<TreeNode> nodeList = new ArrayList<>();
            nodeList.add(root);
            while (!nodeList.isEmpty()) {
                TreeNode first = nodeList.remove(0);
                if (first.isEnable() && first.isSelected() && first.getValue().getValue() instanceof ExportUtils.ExportItem) {
                    exportItem.add((ExportUtils.ExportItem) first.getValue().getValue());
                }
                nodeList.addAll(first.getChildren());
            }
            if (exportItem.isEmpty()) {
                if (callback != null) {
                    ThreadPoolUtil.executeInMain(() -> callback.onFailureInMainThread(context.getString(R.string.error_tips_please_select_exported_station)));
                }
                return;
            }

            //获取测站测期的完整数据列表
//            List<ExportUtils.ExportItem> exportCompleteLoopList = getExportProjectPeridList(root);

            //执行导出
            String exportFolder = "控制测量电子手薄导出文件" + File.separator + "导线文件导出";
            List<File> files = ExportUtils.doExportWire(context, exportItem, exportFolder);

            //将文件存储到一个zip中
            String absolutePath = Environment.getExternalStorageDirectory().getAbsolutePath();
//            File appSharedFolder = PathManager.appDocumentFolder();
            Date date = new Date();
            SimpleDateFormat format = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss", Locale.getDefault());
            String dateString = format.format(date);
            File zip = new File(absolutePath, exportFolder + File.separator + dateString + ".zip");
            if (!ZipUtil.zip(zip, files)) {
                zip = null;
            }
            //存储成功，删除zip文件
            for (File file : files) {
                if (file == null) {
                    continue;
                }
                FileUtils.delete(file);
            }

            //提示导出成功
            if (callback != null) {
                File finalZip = zip;
                ThreadPoolUtil.executeInMain(() -> {
                    if (finalZip == null) {
                        callback.onFailureInMainThread(Tools.getString(R.string.export_failure));
                    } else {
                        callback.onSuccessInMainThread(finalZip);
                    }
                });
            }
        });
    }

    private interface ICallback {
        void onFailureInMainThread(String msg);

        void onSuccessInMainThread(File zip);
    }

    /**
     * 根据
     * <p>
     * （项目->测期->测站）
     * <p>
     * 三级结构，构建node树
     * <p>
     * 检测每个测站数据是否正常记录，如果没有则将其设置为不可选中状态
     *
     * @param root node树的根节点
     */
    @WorkerThread
    private static void buildNodeTree(TreeNode root) {
        IProjectDataSource projectDataSource = SurveyorDatabaseFactory.instance.getProjectDataSource();
        List<ProjectRecord> projectRecordList = projectDataSource.getDataList();
        //遍历项目
        for (ProjectRecord projectRecord : projectRecordList) {
            //创建项目节点
            TreeNode projectNode =
                    new TreeNode(new TreeNode.Value(projectRecord.projectName, projectRecord.id), 0);
            ISurveyorPeriodDataSource surveyorPeriodDataSource = SurveyorDatabaseFactory.instance.getSurveyorPeriod();
            List<SurveyorPeriodRecord> surveyorPeriodRecordList = surveyorPeriodDataSource.getDataListSyncByProjectId(projectRecord.getId());
            boolean projectHasData = false;
            //遍历测期
            for (SurveyorPeriodRecord periodRecord : surveyorPeriodRecordList) {
                //创建测期节点
                TreeNode periodNode = new TreeNode(new TreeNode.Value(periodRecord.getName(), periodRecord.id), 1);
                ISurveyorStationDataSource surveyorStationDataSource = SurveyorDatabaseFactory.instance.getSurveyorStation();
                List<SurveyorStationRecord> stationRecordList = surveyorStationDataSource.getDataList(projectRecord.getId(), periodRecord.getId());
                ISurveyorScheduleDataSource surveyorScheduleDataSource = SurveyorDatabaseFactory.instance.getScheduleDataSource();

                boolean periodHasData = false;
                //遍历测站
                if (stationRecordList == null || stationRecordList.isEmpty()) {
                    continue;
                }

                //初始化导线数据map
                Map<String, LineRecord> lineMap = new HashMap<>();
                for (SurveyorStationRecord stationRecord : stationRecordList) {
                    if (stationRecord == null) {
                        continue;
                    }
                    String lineId = stationRecord.getLineId();
                    LineRecord lineRecord = SurveyorDatabaseFactory.instance.getLineRecordDatabase().findById(lineId);
                    if (lineRecord == null) {
                        continue;
                    }
                    if (LineRecord.TYPE_MUL_ANGLE.equals(StationUtil.getLineTypeFromStation(stationRecord))) {
                        //排除多测回测角
                        continue;
                    }
                    if (lineMap.containsKey(lineId)) {
                        LineRecord lineRecordAlready = lineMap.get(lineId);
                        if (lineRecordAlready != null && lineRecordAlready.isCanSelected) {
                            lineRecordAlready.isCanSelected = StationUtil.isStationCanExport(stationRecord);
                        }
                    } else {
                        lineRecord.isCanSelected = StationUtil.isStationCanExport(stationRecord);
                        lineMap.put(lineId, lineRecord);
                    }
                }

                //添加第三节点 导线
                for (LineRecord lineRecord : lineMap.values()) {
                    ExportUtils.ExportItem exportItem =
                            new ExportUtils.ExportItem(projectRecord.getProjectName(), projectRecord.getId(),
                                    projectRecord.projectType,
                                    periodRecord.getId(), periodRecord.getName(),
                                    lineRecord.id,
                                    lineRecord.name,
                                    "");
                    TreeNode lineNode = new TreeNode(new TreeNode.Value(exportItem, lineRecord.id),
                            2);

                    lineNode.setEnable(lineRecord.isCanSelected);
                    periodNode.addChild(lineNode);

                    if (lineRecord.isCanSelected) {
                        projectHasData = true;
                        periodHasData = true;
                    }
                }

                periodNode.setEnable(periodHasData);
                projectNode.addChild(periodNode);
            }

            projectNode.setEnable(projectHasData);
            root.addChild(projectNode);
        }
    }

    public static SelectExportSurveyorRecordWireFileFragment newInstance(Bundle args) {
        SelectExportSurveyorRecordWireFileFragment fragment = new SelectExportSurveyorRecordWireFileFragment();
        fragment.setArguments(args);
        return fragment;
    }

    public static void start(Context context) {
        FragmentContainerActivity.start(context, TAG, R.string.export_wire_file, null);
    }

    public void requestStoragePermission() {
        CheckStorageUtil.check(getContext(), this::exportHasPermission);
    }

    private void exportHasPermission() {
        DialogFragment dialogFragment = FullLoadingFragment.showLoading(getChildFragmentManager());
        doExport(root, getContext(), new SelectExportSurveyorRecordWireFileFragment.ICallback() {
            @Override
            public void onFailureInMainThread(String msg) {
                dialogFragment.dismiss();
                ToastUtils.showShort(R.string.tips_please_select_exported_data);
            }

            @Override
            public void onSuccessInMainThread(File zip) {
                if (dialogFragment.isVisible()) {
                    dialogFragment.dismiss();
                }
                TipsWith2OptionFragment.tipsCannotCancel(getChildFragmentManager(), getViewLifecycleOwner(),
                        getString(R.string.share),
                        getString(R.string.cancel),
                        getString(R.string.export_successful),
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                FileOperator.shareFile(getContext(), zip, BuildConfig.APPLICATION_ID);
                            }
                        });
            }
        });
    }
}