package com.bingce.controlnetwork.fragment;

import android.content.DialogInterface;
import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.R;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.jetbrains.annotations.NotNull;

/**
 * 弹框，先选项目，再选测期
 */
public class SelectProjectAndSurveyorCycleFragment extends DialogFragment {
    public static final String TAG = SelectProjectAndSurveyorCycleFragment.class.getName();
    private ComponentSwitchStationListFragment componentSwitchStationListFragment;

    private SelectProjectAndSurveyorCycleFragment() {
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater
                .inflate(R.layout.fragment_single_framelayout_layout,
                        container,
                        false);
        String currentProjectId = "";
        String cycleId = "";
        if (getArguments() != null) {
            currentProjectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            cycleId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
        }

        componentSwitchStationListFragment =
                ComponentSwitchStationListFragment.newInstance(currentProjectId, cycleId);

        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.container, componentSwitchStationListFragment)
                .commit();

        view.setOnClickListener(v -> dismiss());
        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (getDialog() != null) {
            final Window window = getDialog().getWindow();

            window.setWindowAnimations(R.style.animate_dialog);
            window.setBackgroundDrawable(null);

            WindowManager.LayoutParams params = window.getAttributes();
            params.gravity = Gravity.BOTTOM;
            params.width = ViewGroup.LayoutParams.MATCH_PARENT;
            params.height = ViewGroup.LayoutParams.WRAP_CONTENT;
            window.setAttributes(params);
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);
        getDialog().setOnKeyListener(new DialogInterface.OnKeyListener() {
            @Override
            public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
                if ((keyCode == android.view.KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP)) {
                    if (!componentSwitchStationListFragment.doBack()) {
                        dismiss();
                    }
                    return true;
                }
                return false;
            }
        });
    }

    @Override
    public void onStop() {
        super.onStop();
        EventBus.getDefault().unregister(this);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(SelectedResult event) {
        Bundle args = new Bundle();
        args.putString(BundleConstants.KEY_PROJECT_ID, event.projectId);
        args.putString(BundleConstants.KEY_PROJECT_NAME, event.projectName);
        args.putString(BundleConstants.KEY_PERIOD_ID, event.cycleId);
        args.putString(BundleConstants.KEY_PERIOD_NAME, event.cycleName);
        getParentFragmentManager()
                .setFragmentResult(KEY_REQUEST_SELECT, args);
        dismiss();
    }

    private static final String KEY_REQUEST_SELECT = "__SELECT__";

    public static void select(FragmentManager fragmentManager,
                              @NonNull LifecycleOwner lifecycleOwner,
                              String defaultProjectId,
                              String defaultToleranceId,
                              IOnSelectCallback callback) {
        Bundle args = new Bundle();
        args.putString(BundleConstants.KEY_PROJECT_ID, defaultProjectId);
        args.putString(BundleConstants.KEY_PERIOD_ID, defaultToleranceId);
        SelectProjectAndSurveyorCycleFragment fragment = new SelectProjectAndSurveyorCycleFragment();
        fragment.setArguments(args);
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onSurveyorCycleSelected(
                                        result.getString(BundleConstants.KEY_PROJECT_ID),
                                        result.getString(BundleConstants.KEY_PROJECT_NAME),
                                        result.getString(BundleConstants.KEY_PERIOD_ID),
                                        result.getString(BundleConstants.KEY_PERIOD_NAME));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

    public interface IOnSelectCallback {
        void onSurveyorCycleSelected(String projectId, String projectName, String cycleId, String cycleName);
    }

    public static class SelectedResult {
        public final String projectId;
        public final String projectName;
        public final String cycleId;
        public final String cycleName;

        public SelectedResult(String projectId, String projectName,
                              String cycleId, String cycleName) {
            this.projectId = projectId;
            this.projectName = projectName;
            this.cycleId = cycleId;
            this.cycleName = cycleName;
        }
    }
}