package com.bingce.controlnetwork.fragment;

import android.app.Dialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.model.UniversalListItem;
import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.ReSurveyorCycleAdapter;
import com.bingce.controlnetwork.databinding.FragmentResurveryorLayoutCycleBinding;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.adapter.ReSurveyorCycleAdapter;

/**
 * 高程导线
 * 选择要重测的测回
 */
public class SelectReSurveyorCycleFragment extends BaseBottomSheetDialogFragment {
    private FragmentResurveryorLayoutCycleBinding mBinding;

    private int resultCycleIndex = -1;

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater,
                             @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        mBinding = FragmentResurveryorLayoutCycleBinding.inflate(inflater, container, false);

        List<UniversalListItem<String>> items = new ArrayList<>();

        if (getArguments() != null) {
            int cycleCount = getArguments().getInt(KEY_CYCLE_COUNT, 0);
            for (int i = 0; i < cycleCount; i++) {
                UniversalListItem<String> item = new UniversalListItem<String>() {
                    @Override
                    public ViewItemType getType() {
                        return ViewItemType.indexWithName;
                    }
                };
                item.setName(Tools.getString(R.string.test_n_cycle, (i + 1)));
                items.add(item);
            }
        }

        ReSurveyorCycleAdapter adapter = new ReSurveyorCycleAdapter(items, cycleIndex -> {
            resultCycleIndex = cycleIndex;
        });
        mBinding.list.setAdapter(adapter);

        mBinding.confirm.setOnClickListener(v -> {
            if (resultCycleIndex == -1) {
                ToastUtils.showShort(R.string.please_select_retest_cycle);
                return;
            }
            Bundle args = new Bundle();
            args.putInt(KEY_RESULT_CYCLE, resultCycleIndex);
            getParentFragmentManager().setFragmentResult(KEY_REQUEST_TIME, args);
            dismiss();
        });

        return mBinding.getRoot();
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
    }

    public static final String KEY_REQUEST_TIME = "KEY_REQUEST_TIME_ID";


    private static final String KEY_CYCLE_COUNT = "__cycle_count__ ";
    private static final String KEY_RESULT_CYCLE = "__result_cycle__ ";

    private SelectReSurveyorCycleFragment() {
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Dialog dialog = getDialog();
        if (dialog == null) {
            return;
        }
        dialog.setOnDismissListener(dialogInterface -> {
            Bundle args = new Bundle();
            getParentFragmentManager().setFragmentResult(KEY_REQUEST_TIME, args);
            dismiss();
        });
    }

    public static void start(@NonNull FragmentManager fragmentManager,
                             @NonNull LifecycleOwner lifecycleOwner,
                             int cycleCount,
                             ISelectedCallback callback) {
        if (callback == null) {
            return;
        }
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_TIME,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_TIME.equals(requestKey)) {
                                int targetCycleIndex = result.getInt(KEY_RESULT_CYCLE, -1);
                                callback.onSelected(targetCycleIndex);
                            }
                        });
        BottomSheetDialogFragment fragment = new SelectReSurveyorCycleFragment();
        Bundle args = new Bundle();
        args.putInt(KEY_CYCLE_COUNT, cycleCount);
        fragment.setArguments(args);
        fragment.show(fragmentManager, SelectReSurveyorCycleFragment.class.getName());
    }

    public interface ISelectedCallback {
        void onSelected(int cycleIndex);
    }
}