package com.bingce.controlnetwork.fragment;

import android.app.Dialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlnetwork.adapter.ResurveyorPointsAdapter;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.ResurveyorPointsAdapter;

/**
 * 选择要补测的方向
 */
public class SelectReSurveyorPointFragment extends BaseBottomSheetDialogFragment {
    private int cycleIndex = -1;

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater,
                             @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_resurveryor_layout, container, false);

        RecyclerView recyclerView = rootView.findViewById(R.id.list);
        if (recyclerView != null && getArguments() != null) {
            int cycleCount = getArguments().getInt(KEY_CYCLE_COUNT);
            boolean showLeftOrRight = getArguments().getBoolean(KEY_SHOW_LEFT_RIGHT, false);
            if (cycleCount > 0) {
                cycleIndex = 0;
                View layoutCycle = rootView.findViewById(R.id.layout_cycle);
                if (layoutCycle != null) {
                    layoutCycle.setVisibility(View.VISIBLE);
                    TextView cycleText = rootView.findViewById(R.id.cycle_text);
                    layoutCycle.setOnClickListener(v -> {
                        ArrayList<String> cycleList = SurveyorCycleUtil.cycleStrings(cycleCount, showLeftOrRight);
                        runOnUI(() -> BottomSheetSelectListFragment.showSelect(
                                getChildFragmentManager(),
                                getViewLifecycleOwner(),
                                cycleIndex,
                                cycleList,
                                null,//不再限制测期
                                (index, string) -> {
                                    cycleIndex = index;
                                    if (cycleText != null) {
                                        cycleText.setText(string);
                                    }
                                }));
                    });
                }
            }

            //默认添加0方向
            List<String> allPointIds = getArguments().getStringArrayList(KEY_ALL_POINT_IDS);
            List<String> allPointNames = getArguments().getStringArrayList(KEY_ALL_POINT_NAMES);

            List<ResurveyorPointsAdapter.Item> items = new ArrayList<>();
            for (int index = 0; index < allPointIds.size(); index++) {
                ResurveyorPointsAdapter.Item item =
                        new ResurveyorPointsAdapter.Item(allPointNames.get(index), allPointIds.get(index));
                items.add(item);
            }
            ResurveyorPointsAdapter adapter = new ResurveyorPointsAdapter(
                    Collections.singletonList(allPointIds.get(0)),
                    items,
                    getContext(),
                    getChildFragmentManager(),
                    getViewLifecycleOwner());
            recyclerView.setAdapter(adapter);

            Button confirm = rootView.findViewById(R.id.confirm);
            if (confirm != null) {
                confirm.setOnClickListener(v -> {
                    Bundle args = new Bundle();
                    List<String> ids = adapter.selectedIds();
                    ArrayList<String> results = new ArrayList<>(ids);
                    args.putStringArrayList(KEY_SELECTED_POINT_IDS, results);
                    args.putInt(KEY_CYCLE_INDEX, cycleIndex);
                    getParentFragmentManager().setFragmentResult(KEY_REQUEST_TIME, args);
                    dismiss();
                });
            }
        }

        return rootView;
    }

    public static final String KEY_REQUEST_TIME = "KEY_REQUEST_TIME_ID";
    private static final String KEY_ALL_POINT_IDS = "__all_point_ids ";
    private static final String KEY_ALL_POINT_NAMES = "__all_point_names ";
    private static final String KEY_SELECTED_POINT_IDS = "__selected_point_ids ";

    private static final String KEY_CYCLE_COUNT = "__cycle_count__ ";
    private static final String KEY_SHOW_LEFT_RIGHT = "__show_left_or_right__";
    private static final String KEY_CYCLE_INDEX = "__cycle_INDEX__ ";

    private SelectReSurveyorPointFragment() {
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Dialog dialog = getDialog();
        if (dialog == null) {
            return;
        }
        dialog.setOnDismissListener(dialogInterface -> {
            Bundle args = new Bundle();
            getParentFragmentManager().setFragmentResult(KEY_REQUEST_TIME, args);
            dismiss();
        });
    }

    public static void pick(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            boolean showLeftOrRight,
                            ArrayList<String> allPointIds,
                            ArrayList<String> allPointNames,
                            ISelectedCallback callback) {
        pick(fragmentManager, lifecycleOwner, -1, showLeftOrRight,
                allPointIds, allPointNames, callback);
    }

    public static void pick(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            int cycleCount,
                            boolean showLeftOrRight,
                            ArrayList<String> allPointIds,
                            ArrayList<String> allPointNames,
                            ISelectedCallback callback) {
        if (callback == null) {
            return;
        }
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_TIME,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_TIME.equals(requestKey)) {
                                List<String> selectedIds = result.getStringArrayList(KEY_SELECTED_POINT_IDS);
                                int targetCycleIndex = result.getInt(KEY_CYCLE_INDEX, -1);
                                callback.onSelected(targetCycleIndex, selectedIds);
                            }
                        });
        BottomSheetDialogFragment fragment = new SelectReSurveyorPointFragment();
        Bundle args = new Bundle();
        args.putStringArrayList(KEY_ALL_POINT_IDS, allPointIds);
        args.putStringArrayList(KEY_ALL_POINT_NAMES, allPointNames);
        args.putBoolean(KEY_SHOW_LEFT_RIGHT, showLeftOrRight);
        args.putInt(KEY_CYCLE_COUNT, cycleCount);
        fragment.setArguments(args);
        fragment.show(fragmentManager, SelectReSurveyorPointFragment.class.getName());
    }

    public interface ISelectedCallback {
        void onSelected(int cycleIndex, List<String> pointIds);
    }
}