package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import com.bingce.controlnetwork.R;

/**
 * 选择站的类型（闭合导线，直导线等）
 */
public class SelectSurveyorStationTypeFragment extends BottomSheetDialogFragment {
    public static final String TAG = SelectProjectAndSurveyorCycleFragment.class.getName();
    //上个页面传过来的
    private String projectType;

    private final int[] needHideIdsTriangle = {
            R.id.wire_well_double, R.id.wire_well
    };
    private final int[] needHideIdsConnection = {
            R.id.default_type, R.id.wire_zhi, R.id.wire_fuhe, R.id.wire_close,
            R.id.wire_random
    };
    private final int[] needHideIdsHeightTraverse = {
            R.id.default_type, R.id.wire_zhi, R.id.wire_fuhe, R.id.wire_close,
            R.id.wire_random, R.id.wire_well_double, R.id.wire_well
    };

    private SelectSurveyorStationTypeFragment() {
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            projectType = getArguments().getString(KEY_PROJECT_TYPE);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater
                .inflate(
                        R.layout.fragment_bottom_popup_surveyor_station_type_select_layout,
                        container,
                        false);
        initListener(view);
        initShowOrHideView(view);
        return view;
    }

    private void initShowOrHideView(View view) {
        ProjectType.getTarget(projectType, (projectTypeBack) -> {
            if (projectTypeBack == ProjectType.TRIANGLE) {
                hideViews(view, needHideIdsTriangle);
            } else if (projectTypeBack == ProjectType.CONNECTION) {
                hideViews(view, needHideIdsConnection);
            } else if (projectTypeBack == ProjectType.HEIGHT_TRAVERSE) {
                hideViews(view, needHideIdsHeightTraverse);
            }
            return null;
        });
    }

    private void hideViews(View view, int[] ids) {
        for (int i = 0; i < ids.length; i++) {
            View currentView = view.findViewById(ids[i]);
            currentView.setVisibility(View.GONE);
        }
    }

    private void initListener(View view) {
        View.OnClickListener onClickListener = v -> {
            if (v == null) {
                dismiss();
                return;
            }
            int id = v.getId();
            String type = TYPE_DEFAULT;
            if (id == R.id.wire_zhi) {
                type = TYPE_LINE_SINGLE;
            } else if (id == R.id.wire_fuhe) {
                type = TYPE_LINE_ATTACH;
            } else if (id == R.id.wire_close) {
                type = TYPE_LINE_CLOSE;
            } else if (id == R.id.wire_random) {
                type = TYPE_LINE_RANDOM;
            } else if (id == R.id.wire_well) {
                type = TYPE_WELL_WIRE;
            } else if (id == R.id.wire_well_double) {
                type = TYPE_WELL_WIRE_DOUBLE;
            }
            Bundle args = new Bundle();
            args.putString(KEY_RESULT_TYPE, type);
            getParentFragmentManager().setFragmentResult(KEY_REQUEST_SELECT, args);
            dismiss();
        };

        setOnClickListener(view, R.id.default_type, onClickListener);
        setOnClickListener(view, R.id.wire_zhi, onClickListener);
        setOnClickListener(view, R.id.wire_fuhe, onClickListener);
        setOnClickListener(view, R.id.wire_close, onClickListener);
        setOnClickListener(view, R.id.wire_random, onClickListener);
        setOnClickListener(view, R.id.wire_well, onClickListener);
        setOnClickListener(view, R.id.wire_well_double, onClickListener);
        setOnClickListener(view, R.id.cancel, v -> dismiss());
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() == null) {
            return;
        }
        getDialog().getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    private static void setOnClickListener(View root, int id, View.OnClickListener onClickListener) {
        View view = root.findViewById(id);
        if (view != null) {
            view.setOnClickListener(onClickListener);
        }
    }

    private static final String KEY_PROJECT_TYPE = "__KEY_PROJECT_TYPE__";

    private static final String KEY_REQUEST_SELECT = "__SELECT__";
    private static final String KEY_RESULT_TYPE = "__KEY_RESULT_TYPE__";

    public static final String TYPE_DEFAULT = "__default__";//多测回测角
    public static final String TYPE_LINE_SINGLE = "__line_single";//支导线
    public static final String TYPE_LINE_ATTACH = "__line_attach";//附合导线
    public static final String TYPE_LINE_CLOSE = "__line_close";//闭合导线
    public static final String TYPE_LINE_RANDOM = "__line_random";//无定向导线
    public static final String TYPE_WELL_WIRE = "__well_wire";//井定向
    public static final String TYPE_WELL_WIRE_DOUBLE = "__well_wire_double";//二井定向

    public static void select(FragmentManager fragmentManager,
                              @NonNull LifecycleOwner lifecycleOwner,
                              String projectType,
                              IOnSelectCallback callback) {
        SelectSurveyorStationTypeFragment fragment = new SelectSurveyorStationTypeFragment();
        Bundle bundle = new Bundle();
        bundle.putString(KEY_PROJECT_TYPE, projectType);
        fragment.setArguments(bundle);
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onSurveyorCycleSelected(result.getString(KEY_RESULT_TYPE));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }


    public interface IOnSelectCallback {
        void onSurveyorCycleSelected(String type);
    }
}