package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.jetbrains.annotations.NotNull;

import com.bingce.controlnetwork.R;

/**
 * 限差选择框（限差设置-限差模板点击后的操作）
 */
public class SelectToleranceBottomSheetFragment extends DialogFragment {
    public static final String TAG = SelectToleranceBottomSheetFragment.class.getName();
    private ComponentSwitchToleranceFragment componentSwitchToleranceFragment;

    private SelectToleranceBottomSheetFragment() {
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater
                .inflate(R.layout.fragment_tolerance_select_framelayout_layout,
                        container,
                        false);
        String projectType = "";
        String groupId = "";
        String levelId = "";
        String toleranceId = "";
        if (getArguments() != null) {
            projectType = getArguments().getString(KEY_PROJECT_TYPE);
            groupId = getArguments().getString(KEY_GROUP_ID);
            levelId = getArguments().getString(KEY_LEVEL_ID);
            toleranceId = getArguments().getString(KEY_TOLERANCE_ID);
        }

        componentSwitchToleranceFragment =
                ComponentSwitchToleranceFragment.newInstance(projectType, groupId, levelId, toleranceId);

        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.container, componentSwitchToleranceFragment)
                .commit();

        View toleranceManager = view.findViewById(R.id.btn_tolerance_manager);
        if (toleranceManager != null) {
            String finalProjectType = projectType;
            toleranceManager.setOnClickListener(v -> ToleranceManageFragment.start(getContext(), finalProjectType));
        }

        view.setOnClickListener(v -> dismiss());
        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (getDialog() != null) {
            final Window window = getDialog().getWindow();

            window.setWindowAnimations(R.style.animate_dialog);
            window.setBackgroundDrawable(null);

            WindowManager.LayoutParams params = window.getAttributes();
            params.gravity = Gravity.BOTTOM;
            params.width = ViewGroup.LayoutParams.MATCH_PARENT;
            params.height = ViewGroup.LayoutParams.WRAP_CONTENT;
            window.setAttributes(params);
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);
        getDialog().setOnKeyListener((dialog, keyCode, event) -> {
            if ((keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP)) {
                if (!componentSwitchToleranceFragment.doBack()) {
                    dismiss();
                }
                return true;
            }
            return false;
        });
    }

    @Override
    public void onStop() {
        super.onStop();
        EventBus.getDefault().unregister(this);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(ComponentSwitchToleranceFragment.SelectedToleranceEvent event) {
        Bundle args = new Bundle();
        args.putString(KEY_TOLERANCE_ID, event.toleranceId);
        getParentFragmentManager().setFragmentResult(KEY_REQUEST_SELECT, args);
        dismiss();
    }

    private static final String KEY_REQUEST_SELECT = "__SELECT__";
    private static final String KEY_PROJECT_TYPE = "___project__type";
    private static final String KEY_GROUP_ID = "___group__id";
    private static final String KEY_LEVEL_ID = "____level__id";
    private static final String KEY_TOLERANCE_ID = "____tolerance__id";

    public static void select(FragmentManager fragmentManager,
                              @NonNull LifecycleOwner lifecycleOwner,
                              String projectType,
                              String defaultGroupId,
                              String defaultLevelId,
                              String defaultToleranceId,
                              IOnSelectCallback callback) {
        Bundle args = new Bundle();
        args.putString(KEY_PROJECT_TYPE, projectType);
        args.putString(KEY_GROUP_ID, defaultGroupId);
        args.putString(KEY_LEVEL_ID, defaultLevelId);
        args.putString(KEY_TOLERANCE_ID, defaultToleranceId);

        SelectToleranceBottomSheetFragment fragment = new SelectToleranceBottomSheetFragment();
        fragment.setArguments(args);

        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onToleranceSelected(
                                        result.getString(KEY_TOLERANCE_ID));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

    public interface IOnSelectCallback {
        void onToleranceSelected(String toleranceId);
    }
}