package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlnetwork.R;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.jetbrains.annotations.NotNull;

/**
 * 选择限差参考模板（自定义限差管理点击+后的操作）
 */
public class SelectToleranceTemplateBottomSheetFragment extends DialogFragment {
    public static final String TAG = SelectToleranceTemplateBottomSheetFragment.class.getName();
    private ComponentSwitchToleranceFragment componentSwitchToleranceFragment;

    private String projectType;

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            projectType = getArguments().getString(KEY_PROJECT_TYPE, "");
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater
                .inflate(R.layout.fragment_tolerance_template_select_layout,
                        container,
                        false);

        componentSwitchToleranceFragment =
                ComponentSwitchToleranceFragment.newInstance(projectType, "", "", "");

        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.container, componentSwitchToleranceFragment)
                .commit();

        view.setOnClickListener(v -> dismiss());
        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        if (getDialog() != null) {
            final Window window = getDialog().getWindow();

            window.setWindowAnimations(R.style.animate_dialog);
            window.setBackgroundDrawable(null);

            WindowManager.LayoutParams params = window.getAttributes();
            params.gravity = Gravity.BOTTOM;
            params.width = ViewGroup.LayoutParams.MATCH_PARENT;
            params.height = ViewGroup.LayoutParams.WRAP_CONTENT;
            window.setAttributes(params);
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);
        getDialog().setOnKeyListener((dialog, keyCode, event) -> {
            if ((keyCode == KeyEvent.KEYCODE_BACK && event.getAction() == KeyEvent.ACTION_UP)) {
                if (!componentSwitchToleranceFragment.doBack()) {
                    dismiss();
                }
                return true;
            }
            return false;
        });
    }

    @Override
    public void onStop() {
        super.onStop();
        EventBus.getDefault().unregister(this);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(ComponentSwitchToleranceFragment.SelectedToleranceEvent event) {
        Bundle args = new Bundle();
        args.putString(KEY_TOLERANCE_ID, event.toleranceId);
        getParentFragmentManager().setFragmentResult(KEY_REQUEST_SELECT, args);
        dismiss();
    }

    private static final String KEY_REQUEST_SELECT = "__SELECT__";
    private static final String KEY_TOLERANCE_ID = "___key__tolerance__id";
    private static final String KEY_PROJECT_TYPE = "KEY_PROJECT_TYPE";

    public static void select(FragmentManager fragmentManager,
                              @NonNull LifecycleOwner lifecycleOwner,
                              String projectType,
                              IOnSelectCallback callback) {
        SelectToleranceTemplateBottomSheetFragment fragment =
                new SelectToleranceTemplateBottomSheetFragment();

        Bundle bundle = new Bundle();
        bundle.putString(KEY_PROJECT_TYPE, projectType);
        fragment.setArguments(bundle);

        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onToleranceSelected(
                                        result.getString(KEY_TOLERANCE_ID));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

    public interface IOnSelectCallback {
        void onToleranceSelected(String toleranceId);
    }
}