package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.ViewItemType;
import com.bingce.controlapphelper.surveyor.method.RadianMethod;
import com.bingce.controlapphelper.surveyor.method.SurveyorAdjustMethod;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlnetwork.adapter.BaseAdapter;
import com.bingce.utils.Util;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.ui.vh.StationAdjustItemViewHolder;
import com.bingce.controlnetwork.ui.widget.ViewFactory;

public class StationDataAdjustDialogFragment extends DialogFragment {
    public static final String KEY_SHOW_TYPE = "__key_show_type";
    private static final String KEY_LIST = "__key_list";
    private static final String KEY_IS_SHOW_SLOP_DISTANCE = "__key_show_slop";

    //key_type对应的常量
    public static final String TYPE_ANGLE_DISTANCE = "TYPE_ANGLE_DISTANCE";//水平角，垂直角，距离
    public static final String TYPE_ANGLE_VER_AND_DISTANCE = "TYPE_ANGLE_VER_AND_DISTANCE";//垂直角，距离
    public static final String TYPE_ALONE_DISTANCE = "TYPE_ALONE_DISTANCE";//距离


    private String showType;

    private StationDataAdjustDialogFragment() {
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        Bundle arguments = getArguments();
        if (arguments != null) {
            showType = arguments.getString(KEY_SHOW_TYPE);
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() == null) {
            return;
        }
        Window window = getDialog().getWindow();
        window.setBackgroundDrawable(new ColorDrawable(0x00000000));
        window.setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_station_data_adjust_dialog_layout, container, false);
        if (getArguments() != null) {
            ArrayList<SurveyorAdjustMethod.Item> items = getArguments().getParcelableArrayList(KEY_LIST);
            boolean isShowSlop = getArguments().getBoolean(KEY_IS_SHOW_SLOP_DISTANCE, true);
            RecyclerView recyclerView = rootView.findViewById(R.id.list);
            TextView distanceTitle = rootView.findViewById(R.id.distance_angle_title);
            if (recyclerView != null) {
                _Adapter adapter = new _Adapter(items);
                recyclerView.setAdapter(adapter);
                if (distanceTitle != null) {
                    distanceTitle.setText(isShowSlop ? R.string.distance_slope : R.string.distance_horizontal);
                }
            }

            initShowView(rootView.findViewById(R.id.hor_angle_title),
                    rootView.findViewById(R.id.ver_angle_title),
                    rootView.findViewById(R.id.distance_angle_title));

        }
        return rootView;
    }


    private void initShowView(View horView, View verView, View disView) {
        if (showType == null) return;
        switch (showType) {
            case TYPE_ALONE_DISTANCE:
                horView.setVisibility(View.GONE);
                verView.setVisibility(View.GONE);
                disView.setVisibility(View.VISIBLE);
                break;
            case TYPE_ANGLE_VER_AND_DISTANCE:
                horView.setVisibility(View.GONE);
                verView.setVisibility(View.VISIBLE);
                disView.setVisibility(View.VISIBLE);
                break;
            case TYPE_ANGLE_DISTANCE:
            default:
                horView.setVisibility(View.VISIBLE);
                verView.setVisibility(View.VISIBLE);
                disView.setVisibility(View.VISIBLE);
                break;
        }
    }


    private class _Adapter extends BaseAdapter<SurveyorAdjustMethod.Item, StationAdjustItemViewHolder> {

        public _Adapter(List<SurveyorAdjustMethod.Item> items) {
            super(items);
        }

        @Override
        protected StationAdjustItemViewHolder createVH(@NotNull ViewGroup parent, int viewType) {
            RecyclerView.ViewHolder viewHolder = ViewFactory.generatorViewHolder(parent.getContext(), parent, ViewItemType.stationAdjust);
            if (viewHolder instanceof StationAdjustItemViewHolder) {
                return (StationAdjustItemViewHolder) viewHolder;
            }
            return null;
        }

        @Override
        public void onBindViewHolder(@NonNull @NotNull StationAdjustItemViewHolder holder, int position) {
            SurveyorAdjustMethod.Item item = getItem(position);

            initShowView(holder.horAngle, holder.verAngle, holder.distance);

            setText(holder.pointName, item.getPointName());
            setText(holder.horAngle, formatAngle(item.getHorAngle()));
            setText(holder.verAngle, formatAngle(item.getVerAngle()));
            setText(holder.distance, formatDistance(item.getDistance()));
        }


        private void setText(TextView textView, String string) {
            if (textView != null) {
                textView.setText(string);
            }
        }

        private String formatAngle(double angle) {
            if (angle < 0) {
                return "";
            }
            return RadianMethod.dmsStringOf(angle);
        }

        private String formatDistance(double distance) {
            if (distance < 0) {
                return "";
            }
            return Util.formatDouble2String(distance, SettingValueUtil.getLengthDecimalPoint());
        }
    }

    public static void showDialog(String type, ArrayList<SurveyorAdjustMethod.Item> finalItemList,
                                  Boolean isShowSlopDistance,
                                  FragmentManager fragmentManager) {
        StationDataAdjustDialogFragment dialogFragment = new StationDataAdjustDialogFragment();
        Bundle args = new Bundle();
        args.putString(KEY_SHOW_TYPE, type);
        args.putParcelableArrayList(KEY_LIST, finalItemList);
        args.putBoolean(KEY_IS_SHOW_SLOP_DISTANCE, isShowSlopDistance);
        dialogFragment.setArguments(args);
        dialogFragment.show(fragmentManager, StationDataAdjustDialogFragment.class.getName());
    }
}
