package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.rengwuxian.materialedittext.MaterialEditText;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.R;

public class StringInputFragment extends AbstractFullScreenDialogFragment {
    @Override
    protected View createContentView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.content_string_input_layout, container, false);
        MaterialEditText editText = view.findViewById(R.id.editor);
        if (editText != null && getArguments() != null) {
            editText.setText(getArguments().getString(KEY_DEFAULT_STRING));
        }
        return view;
    }

    @Override
    protected void onConfirm() {
        Bundle args = new Bundle();
        if (getView() != null) {
            MaterialEditText editText = getView().findViewById(R.id.editor);
            if (editText.getText() != null) {
                String string = editText.getText().toString();
                if (string.isEmpty()) {
                    ToastUtils.showShort(R.string.string_no_empty);
                    return;
                }
                args.putString(KEY_STRING_RESULT, string);
            }
        }
        getParentFragmentManager()
                .setFragmentResult(KEY_REQUEST_INPUT, args);
        super.onConfirm();
    }

    private static final String TAG = StringInputFragment.class.getName();
    private static final String KEY_REQUEST_INPUT = StringInputFragment.class.getName() + "_REQUEST";
    private static final String KEY_STRING_RESULT = StringInputFragment.class.getName() + "_result";
    private static final String KEY_DEFAULT_STRING = StringInputFragment.class.getName() + "__DEFAULT_STRING";

    public static void input(FragmentManager fragmentManager,
                             String title,
                             @NonNull LifecycleOwner lifecycleOwner,
                             IInputCallback callback) {
        input(fragmentManager, title, null, lifecycleOwner, callback);
    }

    public static void input(FragmentManager fragmentManager,
                             String title,
                             String defaultString,
                             @NonNull LifecycleOwner lifecycleOwner,
                             IInputCallback callback) {
        Bundle args = new Bundle();
        StringInputFragment fragment = new StringInputFragment();
        fragment.setArguments(args);
        if (defaultString != null) {
            args.putString(KEY_DEFAULT_STRING, defaultString);
        }
        setArgs(args, title, null);
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_INPUT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_INPUT.equals(requestKey)) {
                                callback.onGetInput(result.getString(KEY_STRING_RESULT));
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

    public interface IInputCallback {
        void onGetInput(String string);
    }
}
