package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.route.RouteRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.ScheduleUtil;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.SurveyorStationListAdapter;
import com.bingce.controlnetwork.databinding.FragmentSurveyorRouteStationBinding;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.station.heighttraverse.CreateStationForHeightTraverseFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.heighttraverse.MeasureHeightTraverseFragment;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 高程导线 测量页-项目路线下的测站列表和新增测站
 */
public class SurveyorRouteStationFragment extends BaseFragment {
    public static final String TAG = SurveyorRouteStationFragment.class.getName();
    private SurveyorStationListAdapter adapter;
    private FragmentSurveyorRouteStationBinding mBinding;

    private static final String KEY_DETAIL_OR_SURVEYOR = "___editable___";//标识点击item进入测量界面还是测站详情

    public static SurveyorRouteStationFragment newInstance(String projectId, String routeId) {
        return newInstance(projectId, routeId, true);
    }

    public static SurveyorRouteStationFragment newInstance(String projectId, String routeId, boolean detailOrSurveyor) {
        SurveyorRouteStationFragment fragment = new SurveyorRouteStationFragment();
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_ROUTE_ID, routeId);
        bundle.putBoolean(KEY_DETAIL_OR_SURVEYOR, detailOrSurveyor);
        fragment.setArguments(bundle);
        return fragment;
    }

    /**
     * 去测量或者详情页面
     */
    private void goStationMeasureOrDetail(SurveyorStationRecord item) {
        String stationId = item.getId();
        ThreadPoolUtil.execute(() -> {
            SurveyorStationRecord stationRecord = SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getRecordSync(stationId);
            if (stationRecord == null || stationRecord.getItems() == null || stationRecord.getItems().isEmpty()) {
                return;
            }

            //站的名称 导线+点名
            String stationName = StationUtil.getStationName(stationRecord);

            MeasureHeightTraverseFragment.start(requireContext(), stationId, stationName);
        });
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        mBinding = FragmentSurveyorRouteStationBinding.inflate(inflater, container, false);
        return mBinding.getRoot();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView();
        initListener();
        initData();
    }

    private void initView() {
        mBinding.btnCreateStation.setVisibility(getRouteId() == null ? View.GONE : View.VISIBLE);
    }

    private void initListener() {
        mBinding.btnCreateStation.setOnClickListener(v -> checkCreateStation(checkSuccess -> {
            if (!checkSuccess) return;
            CreateStationForHeightTraverseFragment.start(requireContext(), getProjectId(), getRouteId());
        }));
    }

    private void checkCreateStation(ICallBackCheckStation callBackCheckStation) {
        ThreadPoolUtil.execute(() -> {
            String projectId = getProjectId();
            String routeId = getRouteId();
            if (projectId == null || routeId == null) {
                ToastUtils.showShort(R.string.parameter_exception);
                callBackCheckStation.result(false);
                return;
            }
            RouteRecord routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getByIdSync(routeId);
            if (routeRecord == null) {
                ToastUtils.showShort(R.string.route_record_exception);
                callBackCheckStation.result(false);
                return;
            }

            //判断上一个测站是否通过检测
            SurveyorStationRecord stationLatest = SurveyorDatabaseFactory.instance.getSurveyorStation().getByProjectRouteLatest(projectId, routeId);
            if (stationLatest != null && stationLatest.items != null && stationLatest.items.size() > 0) {
                String stationNameNotFinish = ScheduleUtil.INSTANCE.getStationNameNotFinishForHeightTraverse(stationLatest);
                if (!TextUtils.isEmpty(stationNameNotFinish)) {
                    ToastUtils.showShort(Tools.getString(R.string.station_failed_test, stationNameNotFinish));
                    callBackCheckStation.result(false);
                    return;
                }
            }

            SurveyorStationRecord stationRecord = SurveyorDatabaseFactory.instance.getSurveyorStation().getByProjectRouteLatest(projectId, routeId);
            if (stationRecord != null && stationRecord.items != null) {
                for (SurveyorPoint surveyorPoint : stationRecord.items) {
                    if (surveyorPoint.getOriginalPointId().equals(routeRecord.getEndPointId())) {
                        ToastUtils.showShort(R.string.route_begin_end_been_created);
                        callBackCheckStation.result(false);
                        return;
                    }
                }
            }

            callBackCheckStation.result(true);
        });
    }

    private void initData() {
        if (getArguments() != null) {
            update(getProjectId(), getRouteId());
        }
    }

    private String getProjectId() {
        if (getArguments() != null) {
            return getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        }
        return null;
    }

    private String getRouteId() {
        if (getArguments() != null) {
            return getArguments().getString(BundleConstants.KEY_ROUTE_ID);
        }
        return null;
    }

    /**
     * 根据项目id和路线id获取测站记录
     */
    public void update(String projectId, String routeId) {
        LiveData<List<SurveyorStationRecord>> liveData = SurveyorDatabaseFactory.instance.getSurveyorStation().getByProjectRouteLiveData(projectId, routeId);
        liveData.observe(getViewLifecycleOwner(), this::updateUi);
    }

    private void updateUi(List<SurveyorStationRecord> stationList) {
        if (adapter == null) {
            adapter = new SurveyorStationListAdapter(stationList,
                    getChildFragmentManager(),
                    getViewLifecycleOwner(),
                    this::runOnUI,
                    getContext(),
                    this::goStationMeasureOrDetail,
                    true);
            mBinding.recyclerView.setAdapter(adapter);
        } else {
            adapter.updateData(stationList);
        }
    }

    private interface ICallBackCheckStation {
        void result(boolean checkSuccess);
    }


}
