package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.SwitchPreferenceCompat;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

public class SurveyorSettingsFragment extends Fragment {
    public static final String TAG = SurveyorSettingsFragment.class.getName();

    @Nullable
    @org.jetbrains.annotations.Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_fragment_container_layout, container, false);
        _SettingsFragment fragment = new _SettingsFragment();
        fragment.setArguments(getArguments());
        getChildFragmentManager()
                .beginTransaction()
                .add(R.id.container, fragment)
                .commit();
        return rootView;
    }

    public static class _SettingsFragment extends PreferenceFragmentCompat {
        @Override
        public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
            addPreferencesFromResource(R.xml.surveyor_settings_prefs);

            SwitchPreferenceCompat distanceType = findPreference("surveyor_settings_key_distance_type");
            if (getArguments() != null) {
                String stationId = getArguments().getString(BundleConstants.KEY_SURVEYOR_STATION_ID);
                ThreadPoolUtil.execute(() -> {
                    ISurveyorStationDataSource stationDataSource =
                            SurveyorDatabaseFactory.instance.getSurveyorStation();
                    SurveyorStationRecord stationRecord = stationDataSource.getRecordSync(stationId);
                    if (stationRecord == null) {
                        return;
                    }
                    runOnUI(() -> {
                        if (distanceType != null) {
                            distanceType.setChecked(stationRecord.isShowSlopDistance());
                            distanceType.setOnPreferenceChangeListener((preference, newValue) -> {
                                if (newValue instanceof Boolean) {
//                                            Log.d("___distance_type", newValue.toString());
                                    stationRecord.setShowSlopDistance((boolean) newValue);
                                    stationDataSource.save(stationRecord);
                                }
                                return true;
                            });
                        }
                    });
                });
            }
        }

        private void runOnUI(Runnable action) {
            if (getActivity() != null) {
                getActivity().runOnUiThread(action);
            }
        }
    }

//    public static void show(FragmentManager fragmentManager) {
//        DialogFragment dialogFragment = new SurveyorSettingDialogFragment();
//        dialogFragment.show(fragmentManager, SurveyorSettingDialogFragment.class.getName());
//    }

    public static void start(Context context, String surveyorStationId, String surveyorStationName) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_ID, surveyorStationId);
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_NAME, surveyorStationName);
        FragmentContainerActivity.start(context, TAG, surveyorStationName, bundle);
    }

    public static SurveyorSettingsFragment newInstance(Bundle bundle) {
        SurveyorSettingsFragment fragment = new SurveyorSettingsFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
