package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlnetwork.R;

public class TipsFragment extends AppCompatDialogFragment {
    private final Handler mainHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(@NonNull Message msg) {
            View contentView = getView();
            if (contentView == null) {
                return;
            }
            Button confirm = contentView.findViewById(R.id.confirm_button);
            int countDown = (Integer) msg.obj - 1;
            if (countDown <= 0) {
                confirm.setEnabled(true);
                confirm.setText(R.string.confirm);
            } else {
                confirm.setEnabled(false);
                confirm.setText(String.valueOf(countDown));
                Message message = Message.obtain();
                message.obj = countDown;
                mainHandler.sendMessageDelayed(message, 1000);
            }
        }
    };

    private TipsFragment() {
        super();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View contentView = inflater.inflate(R.layout.fragment_tips_layout, container, false);
        Button confirm = contentView.findViewById(R.id.confirm_button);
        if (getArguments() != null) {
            TextView tvTitle = contentView.findViewById(R.id.title);
            if (tvTitle != null) {
                tvTitle.setText(getArguments().getString(KEY_TITLE));
            }
            TextView tvTips = contentView.findViewById(R.id.content);
            if (tvTips != null) {
                tvTips.setText(getArguments().getString(KEY_CONTENT));
            }
            int countDown = getArguments().getInt(KEY_COUNT_DOWN, -1);
            if (countDown > 0) {
                confirm.setEnabled(false);
                confirm.setText(String.valueOf(countDown));
                Message message = Message.obtain();
                message.obj = countDown;
                mainHandler.sendMessageDelayed(message, 1000);
            }
        }

        if (confirm != null) {
            confirm.setOnClickListener(v -> {
                //这里主动dismiss不会收到回调
                Bundle args = new Bundle();
                args.putBoolean(KEY_QUIT_CANCEL, false);
                getParentFragmentManager().setFragmentResult(getTag(), args);
                dismiss();
            });
        }
        return contentView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() == null) {
            return;
        }
        if (getArguments() != null) {
            boolean cancelAble = getArguments().getBoolean(KEY_CANCEL_ABLE, true);
            if (!cancelAble) {
                getDialog().setCancelable(false);
            }
        }
        getDialog().getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    private static final String KEY_TITLE = "__TITLE";
    private static final String KEY_CONTENT = "___CONTENT";
    private static final String KEY_COUNT_DOWN = "__COUNT_DOWN";
    private static final String KEY_CANCEL_ABLE = "__cancel__able";
    private static final String KEY_QUIT_CANCEL = "__key_quit_cancel__";

    public static void noCancelAbleTips(@NonNull FragmentManager fragmentManager,
                                        @NonNull LifecycleOwner lifecycleOwner,
                                        Context context,
                                        String content,
                                        int countdown,
                                        IConfirmCallback callback) {
        baseTips(fragmentManager, lifecycleOwner, context.getString(R.string.notice), content, countdown, false, false, callback);
    }

    public static void cancelAbleTips(@NonNull FragmentManager fragmentManager,
                                      @NonNull LifecycleOwner lifecycleOwner,
                                      Context context,
                                      String content,
                                      int countdown,
                                      IConfirmCallback callback) {
        baseTips(fragmentManager, lifecycleOwner, context.getString(R.string.notice), content, countdown, true, true, callback);
    }

    public static void tipsOnly(@NonNull FragmentManager fragmentManager,
                                @NonNull LifecycleOwner lifecycleOwner,
                                Context context,
                                String content) {
        baseTips(fragmentManager, lifecycleOwner, context.getString(R.string.notice), content,
                -1, true, false, null);
    }

    private static void baseTips(
            @NonNull FragmentManager fragmentManager,
            @NonNull LifecycleOwner lifecycleOwner,
            String title,
            String content,
            int countdown,
            boolean cancelAble,
            boolean filterCancel,
            IConfirmCallback callback) {
        Bundle args = new Bundle();
        TipsFragment fragment = new TipsFragment();
        fragment.setArguments(args);
        args.putString(KEY_TITLE, title);
        args.putString(KEY_CONTENT, content);
        args.putInt(KEY_COUNT_DOWN, countdown);
        args.putBoolean(KEY_CANCEL_ABLE, cancelAble);

        String tag = String.valueOf(System.currentTimeMillis());

        fragmentManager
                .setFragmentResultListener(
                        tag,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (callback != null) {
                                if (tag.equals(requestKey)) {
                                    boolean isCancel = result.getBoolean(KEY_QUIT_CANCEL, true);
                                    if (!isCancel || !filterCancel) {
                                        callback.onConfirm();
                                    }
                                }
                            }
                        });

        fragment.show(fragmentManager, tag);


    }

    public interface IConfirmCallback {
        void onConfirm();
    }
}
