package com.bingce.controlnetwork.fragment;

import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.FragmentTipsWith2OptionLayoutBinding;

public class TipsWith2OptionFragment extends AppCompatDialogFragment {

    private static TipsWith2OptionFragment currentFragmet;//为了全局消失使用

    private static final int COUNT_DOWN_TIME = 5;//如果有倒计时，默认是5秒

    //key 返回的是确实还是取消...
    private static final String KEY_CALL_BACK_CODE = "KEY_CALL_BACK_CODE";
    private static final int CALL_BACK_CODE_CONFIRM = 1;
    private static final int CALL_BACK_CODE_CANCEL = 2;
    private static final int CALL_BACK_CODE_OPTION = 3;
    private static final int CALL_BACK_CODE_IGNORE = 4;

    private static final String KEY_TITLE = "__TITLE";
    private static final String KEY_CONTENT = "___CONTENT";
    private static final String KEY_CONFIRM_BUTTON = "_confirm_button";
    private static final String KEY_CANCEL_BUTTON = "_cancel_button";
    private static final String KEY_OPTION_BUTTON = "_option_button";
    private static final String KEY_CANCELABLE = "__cancelable";
    private static final String KEY_AUTO_DISMISS = "__auto_dismiss";//点击按钮后是否可以自动关闭
    private static final String KEY_COUNT_DOWN = "__COUNT_DOWN";

    private int callBackCode;


    private final Handler mainHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(@NonNull Message msg) {
            View contentView = getView();
            if (contentView == null) {
                return;
            }
            Button confirm = contentView.findViewById(R.id.confirm_button);
            int countDown = (Integer) msg.obj - 1;
            if (countDown <= 0) {
                confirm.setEnabled(true);
                confirm.setText(R.string.confirm);
                confirm.setTextColor(Color.BLACK);
            } else {
                confirm.setEnabled(false);
                confirm.setText(String.valueOf(countDown));
                Message message = Message.obtain();
                message.obj = countDown;
                mainHandler.sendMessageDelayed(message, 1000);
            }
        }
    };
    private FragmentTipsWith2OptionLayoutBinding mBinding;

    public static void hardTips(@NonNull FragmentManager fragmentManager,
                                @NonNull LifecycleOwner lifecycleOwner,
                                String content,
                                IOperatorCallback callback) {
        Bundle build = new Argument().content(content).cancelable(false).build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }


    public static void tips(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            Argument argument,
                            IOperatorCallback callback) {
        Bundle build = argument.build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }

    public static void dismissNowMe() {
        if (currentFragmet != null && currentFragmet.getLifecycle().getCurrentState() == Lifecycle.State.RESUMED) {
            currentFragmet.dismissAllowingStateLoss();
            currentFragmet = null;
        }
    }

    public static void tips(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            String confirmButton,
                            String cancelButton,
                            String optionButton,
                            String content,
                            IOperatorCallback callback) {
        Bundle build = new Argument().confirmText(confirmButton).cancelText(cancelButton).optionText(optionButton).content(content).build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }

    public static void tips(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            String confirmButton,
                            String cancelButton,
                            String content,
                            IOperatorCallback callback) {
        Bundle build = new Argument().confirmText(confirmButton).cancelText(cancelButton)
                .content(content).build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }

    public static void tipsCountDownTime(@NonNull FragmentManager fragmentManager,
                                         @NonNull LifecycleOwner lifecycleOwner,
                                         String confirmButton,
                                         String cancelButton,
                                         String optionButton,
                                         String content,
                                         IOperatorCallback callback) {
        Bundle build = new Argument().confirmText(confirmButton).cancelText(cancelButton).optionText(optionButton)
                .content(content).countDownTime(COUNT_DOWN_TIME).build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }


    public static void tipsCannotCancel(@NonNull FragmentManager fragmentManager,
                                        @NonNull LifecycleOwner lifecycleOwner,
                                        String confirmButton,
                                        String cancelButton,
                                        String content,
                                        IOperatorCallback callback) {
        Bundle build = new Argument().confirmText(confirmButton).cancelText(cancelButton)
                .content(content).cancelable(false).build();
        init(fragmentManager, lifecycleOwner, callback, build);
    }

    @Override
    public void onStart() {
        super.onStart();
        Dialog dialog = getDialog();
        if (dialog != null) {
            dialog.setCancelable(isCancelableOwn());

            Window window = dialog.getWindow();
            if (window != null) {
                window.setLayout(
                        WindowManager.LayoutParams.MATCH_PARENT,
                        WindowManager.LayoutParams.WRAP_CONTENT);
            }
        }
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        currentFragmet = this;

        mBinding = FragmentTipsWith2OptionLayoutBinding.inflate(inflater, container, false);
        initData();
        initClickListener();
        initCountDownTime();

        return mBinding.getRoot();
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
        currentFragmet = null;
    }


    private void initData() {
        mBinding.title.setText(getTitle());
        mBinding.content.setText(getContent());
        mBinding.cancelButton.setText(getCancelText());
        String optionText = getOptionText();
        if (TextUtils.isEmpty(optionText)) {
            mBinding.optionButton.setVisibility(View.GONE);
        } else {
            mBinding.optionButton.setVisibility(View.VISIBLE);
            mBinding.optionButton.setText(optionText);
        }
        mBinding.confirmButton.setText(getConfirmText());
    }

    private void initCountDownTime() {
        int countDownTime = getCountDownTime();
        if (countDownTime > 0) {
            mBinding.confirmButton.setEnabled(false);
            mBinding.confirmButton.setText(String.valueOf(countDownTime));
            mBinding.confirmButton.setTextColor(Color.RED);
            Message message = Message.obtain();
            message.obj = countDownTime;
            mainHandler.sendMessageDelayed(message, 1000);
        }
    }

    private void initClickListener() {
        mBinding.confirmButton.setOnClickListener(view -> {
            callBackCode = CALL_BACK_CODE_CONFIRM;
            setResultForBtnClick();
        });
        mBinding.cancelButton.setOnClickListener(view -> {
            callBackCode = CALL_BACK_CODE_CANCEL;
            setResultForBtnClick();
        });
        mBinding.optionButton.setOnClickListener(view -> {
            callBackCode = CALL_BACK_CODE_OPTION;
            setResultForBtnClick();
        });
    }

    @Override
    public void onDismiss(@NonNull DialogInterface dialog) {
        setResultForIgnore();
        super.onDismiss(dialog);
    }

    private void setResultForIgnore() {
        if (isCallBackCodeConfirm() || isCallBackCodeCancel()
                || isCallBackCodeOption()) {
            return;
        }
        callBackCode = CALL_BACK_CODE_IGNORE;
        setResult();
    }

    private void setResultForBtnClick() {
        setResult();

        if (isAutoDismiss() || !isCallBackCodeConfirm()) {
            dismissAllowingStateLoss();
        }
    }

    private boolean isCallBackCodeCancel() {
        return callBackCode == CALL_BACK_CODE_CANCEL;
    }

    private boolean isCallBackCodeConfirm() {
        return callBackCode == CALL_BACK_CODE_CONFIRM;
    }

    private boolean isCallBackCodeOption() {
        return callBackCode == CALL_BACK_CODE_OPTION;
    }

    private void setResult() {
        Bundle bundle = new Bundle();
        bundle.putInt(KEY_CALL_BACK_CODE, callBackCode);
        getParentFragmentManager().setFragmentResult(getTag(), bundle);
    }

    private static void init(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                             IOperatorCallback callback, Bundle bundle) {
        TipsWith2OptionFragment fragment = new TipsWith2OptionFragment();
        fragment.setArguments(bundle);
        fragment.setCancelable(true);

        String tag = String.valueOf(System.currentTimeMillis());

        fragmentManager
                .setFragmentResultListener(
                        tag,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (callback != null) {
                                if (fragment.getTag().equals(requestKey)) {
                                    int type = result.getInt(KEY_CALL_BACK_CODE, -1);
                                    if (CALL_BACK_CODE_CONFIRM == type) {
                                        callback.onConfirm();
                                    } else if (CALL_BACK_CODE_OPTION == type) {
                                        callback.onOption();
                                    } else if (CALL_BACK_CODE_CANCEL == type) {
                                        callback.onCancel();
                                    } else if (CALL_BACK_CODE_IGNORE == type) {
                                        callback.onIgnore();
                                    }
                                }
                            }
                        });
        fragment.show(fragmentManager, tag);
    }

    public boolean isAutoDismiss() {
        assert getArguments() != null;
        return getArguments().getBoolean(KEY_AUTO_DISMISS, true);
    }

    public boolean isCancelableOwn() {
        assert getArguments() != null;
        return getArguments().getBoolean(KEY_CANCELABLE, true);
    }

    public String getTitle() {
        assert getArguments() != null;
        return getArguments().getString(KEY_TITLE, Tools.getString(R.string.notice));
    }

    public String getConfirmText() {
        assert getArguments() != null;
        return getArguments().getString(KEY_CONFIRM_BUTTON, Tools.getString(R.string.confirm));
    }

    public String getCancelText() {
        assert getArguments() != null;
        return getArguments().getString(KEY_CANCEL_BUTTON, Tools.getString(R.string.cancel));
    }

    public String getOptionText() {
        assert getArguments() != null;
        return getArguments().getString(KEY_OPTION_BUTTON, "");
    }

    public String getContent() {
        assert getArguments() != null;
        return getArguments().getString(KEY_CONTENT, "");
    }

    public int getCountDownTime() {
        assert getArguments() != null;
        return getArguments().getInt(KEY_COUNT_DOWN, 0);
    }

    public static class Argument {
        private Boolean cancelable;//是否可以取消
        private Boolean autoDismiss;
        private String title;
        private String confirmText;
        private String cancelText;
        private String optionText;
        private String content;
        private Integer countDownTime;

        public Argument() {
        }

        public Argument autoDismiss(boolean autoDismiss) {
            this.autoDismiss = autoDismiss;
            return this;
        }

        public Argument cancelable(boolean cancelable) {
            this.cancelable = cancelable;
            return this;
        }

        public Argument title(String title) {
            this.title = title;
            return this;
        }

        public Argument confirmText(String confirmText) {
            this.confirmText = confirmText;
            return this;
        }

        public Argument cancelText(String cancelText) {
            this.cancelText = cancelText;
            return this;
        }

        public Argument optionText(String optionText) {
            this.optionText = optionText;
            return this;
        }

        public Argument content(String content) {
            this.content = content;
            return this;
        }

        public Argument countDownTime(int countDownTime) {
            this.countDownTime = countDownTime;
            return this;
        }

        public Bundle build() {
            Bundle bundle = new Bundle();
            putValue(bundle, KEY_TITLE, title);
            putValue(bundle, KEY_CONTENT, content);
            putValue(bundle, KEY_CONFIRM_BUTTON, confirmText);
            putValue(bundle, KEY_CANCEL_BUTTON, cancelText);
            putValue(bundle, KEY_OPTION_BUTTON, optionText);
            putValue(bundle, KEY_CANCELABLE, cancelable);
            putValue(bundle, KEY_AUTO_DISMISS, autoDismiss);
            putValue(bundle, KEY_COUNT_DOWN, countDownTime);
            return bundle;
        }

        private void putValue(Bundle bundle, String key, Object value) {
            if (value != null) {
                if (value instanceof Boolean) {
                    bundle.putBoolean(key, (Boolean) value);
                } else if (value instanceof String) {
                    bundle.putString(key, (String) value);
                } else if (value instanceof Integer) {
                    bundle.putInt(key, (Integer) value);
                }
            }
        }

    }

    public interface IOperatorCallback {
        void onConfirm();

        void onOption();

        void onCancel();

        void onIgnore();
    }

    public static class SimpleCallback implements IOperatorCallback {

        @Override
        public void onConfirm() {

        }

        @Override
        public void onOption() {

        }

        @Override
        public void onCancel() {

        }

        @Override
        public void onIgnore() {

        }
    }
}
