package com.bingce.controlnetwork.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.SwitchCompat;
import androidx.fragment.app.FragmentManager;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.ProjectTolerance;
import com.bingce.controlapphelper.model.ToleranceAttributesType;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.event.RefreshToleranceTotailEvent;
import com.bingce.utils.ThreadPoolUtil;
import com.rengwuxian.materialedittext.MaterialEditText;

import org.greenrobot.eventbus.EventBus;

public class ToleranceAttributeEditFragment extends AbstractFullScreenDialogFragment {

    @Override
    protected View createContentView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View contentView = inflater.inflate(R.layout.content_tolerance_attribute_editor_layout, container, false);
        if (getArguments() != null) {
            ThreadPoolUtil.execute(() -> {
                String toleranceId = getArguments().getString(BundleConstants.KEY_TOLERANCE_ID);
                ToleranceDetailRecord record = SurveyorDatabaseFactory
                        .instance
                        .getToleranceDetailDataSource()
                        .getRecordSync(toleranceId);
                if (record == null) {
                    return;
                }
                ProjectTolerance projectTolerance = new ProjectTolerance(record);
                int attributeIndex = getArguments().getInt(KEY_ATTRIBUTE_INDEX, -1);
                if (attributeIndex < 0) {
                    return;
                }
                ToleranceAttributesType type = ToleranceAttributesType.values()[attributeIndex];
                MaterialEditText editor = contentView.findViewById(R.id.editor);
                SwitchCompat switchCompat = contentView.findViewById(R.id.switch_control);
                View integerView = contentView.findViewById(R.id.integerValue);
                if (projectTolerance.isBoolValue(type)) {
                    if (editor != null) {
                        integerView.setVisibility(View.GONE);
                    }
                    if (integerView != null) {
                        integerView.setVisibility(View.GONE);
                    }
                    if (switchCompat != null) {
                        switchCompat.setVisibility(View.VISIBLE);
                        switchCompat.setChecked(projectTolerance.getBoolAttribute(type));
                    }
                } else if (projectTolerance.isDoubleValue(type)) {
                    if (editor != null) {
                        editor.setVisibility(View.VISIBLE);
                        String value = String.valueOf(projectTolerance.getDoubleAttribute(type));
                        editor.setText(value);
                        editor.setHint(value);
                    }
                    if (switchCompat != null) {
                        switchCompat.setVisibility(View.GONE);
                    }
                    if (integerView != null) {
                        integerView.setVisibility(View.GONE);
                    }
                } else if (projectTolerance.isIntegerValue(type)) {
                    if (editor != null) {
                        editor.setVisibility(View.GONE);
                    }
                    if (switchCompat != null) {
                        switchCompat.setVisibility(View.GONE);
                    }
                    if (integerView != null) {
                        integerView.setVisibility(View.VISIBLE);
                        String value = String.valueOf(projectTolerance.getIntegerAttribute(type));
                        TextView countView = integerView.findViewById(R.id.count);
                        countView.setText(value);
                        View left = integerView.findViewById(R.id.left);
                        View right = integerView.findViewById(R.id.right);
                        if (left != null) {
                            if (Integer.parseInt(value) <= 1) {
                                left.setEnabled(false);
                            }
                            left.setOnClickListener(v -> {
                                int count = Integer.parseInt(countView.getText().toString());
                                countView.setText(String.valueOf(count - 1));
                                if (count - 1 <= 1) {//禁止减小
                                    left.setEnabled(false);
                                }
                                //减小后增大变为可用
                                if (right != null) {
                                    right.setEnabled(true);
                                }
                            });
                        }
                        if (right != null) {
                            right.setOnClickListener(v -> {
                                int count = Integer.parseInt(countView.getText().toString());
                                countView.setText(String.valueOf(count + 1));
                                //增大后减小变为可用
                                if (left != null) {
                                    left.setEnabled(true);
                                }
                                if (type == ToleranceAttributesType.singleSurveyor && count + 1 >= 2) {
                                    right.setEnabled(false);
                                }
                            });
                        }
                    }
                }
            });
        }
        return contentView;
    }

    @Override
    protected void onConfirm() {
        ThreadPoolUtil.execute(() -> {
            if (getArguments() == null || getView() == null) {
                return;
            }
            String toleranceId = getArguments().getString(BundleConstants.KEY_TOLERANCE_ID);
            ToleranceDetailRecord record = SurveyorDatabaseFactory
                    .instance
                    .getToleranceDetailDataSource()
                    .getRecordSync(toleranceId);
            if (record == null) {
                return;
            }
            ProjectTolerance projectTolerance = new ProjectTolerance(record);
            int attributeIndex = getArguments().getInt(KEY_ATTRIBUTE_INDEX, -1);
            if (attributeIndex < 0) {
                return;
            }
            MaterialEditText editor = getView().findViewById(R.id.editor);
            SwitchCompat switchCompat = getView().findViewById(R.id.switch_control);
            TextView count = getView().findViewById(R.id.count);
            ToleranceAttributesType type = ToleranceAttributesType.values()[attributeIndex];
            if (projectTolerance.isBoolValue(type)) {
                projectTolerance.setBoolAttribute(type, switchCompat.isChecked());
            } else if (projectTolerance.isDoubleValue(type)) {
                String string = editor.getText().toString();
                if (string.isEmpty()) {
                    dismiss();
                    return;
                }
                projectTolerance.setDoubleAttribute(type, Double.parseDouble(string));
            } else if (projectTolerance.isIntegerValue(type)) {
                String string = count.getText().toString();
                if (string.isEmpty()) {
                    dismiss();
                    return;
                }
                projectTolerance.setIntegerAttribute(type, Integer.parseInt(string));
            }
            //存储数据
            projectTolerance.save2DataSource(() -> {
                EventBus.getDefault().post(new RefreshToleranceTotailEvent());
                dismissAllowingStateLoss();
            });

        });
    }

    private static final String KEY_ATTRIBUTE_INDEX = "__attribute_index___";

    public static void editAndSave(String toleranceId,
                                   ToleranceAttributesType attributesType,
                                   String groupTitle,
                                   String title,
                                   FragmentManager fragmentManager) {
        ToleranceAttributeEditFragment fragment = new ToleranceAttributeEditFragment();
        Bundle args = new Bundle();
        args.putInt(KEY_ATTRIBUTE_INDEX, attributesType.ordinal());
        args.putString(BundleConstants.KEY_TOLERANCE_ID, toleranceId);
        setArgs(args, groupTitle, title);
        fragment.setArguments(args);
        fragment.show(fragmentManager, ToleranceAttributeEditFragment.class.getName());
    }
}
