package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ExpandableListView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.leveling.tolerance.LevelingToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.interfacee.IProjectTolerance;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.ProjectTolerance;
import com.bingce.controlapphelper.model.tolerance.ProjectToleranceLeveling;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.ToleranceDetailAdapter;
import com.bingce.controlnetwork.event.RefreshToleranceTotailEvent;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;


/**
 * 限差详情页
 */
public class ToleranceDetailFragment extends Fragment {
    public static final String TAG = ToleranceDetailFragment.class.getName();
    private String currentToleranceId;
    private ToleranceDetailAdapter adapter;

    private ExpandableListView expandableListView;

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void refresh(RefreshToleranceTotailEvent event) {
        refreshUi();
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            currentToleranceId = getArguments().getString(BundleConstants.KEY_TOLERANCE_ID);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        EventBus.getDefault().register(this);
        View view = inflater.inflate(R.layout.fragment_limit_detail_list, container, false);
        expandableListView = view.findViewById(R.id.list);
        refreshUi();
        return view;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        EventBus.getDefault().unregister(this);
    }

    public void switchTolerance(String id) {
        currentToleranceId = id;
        refreshUi();
    }

    private void refreshUi() {
        IToleranceOperate operate = ToleranceOperateUtil.getOperate(currentToleranceId);
        if (operate == null) {
            return;
        }
        ThreadPoolUtil.execute(() -> {
            IToleranceDetail iToleranceDetail = operate.getIToleranceDetail(currentToleranceId);
            ThreadPoolUtil.executeInMain(() -> {
                if (iToleranceDetail != null) {
                    if (iToleranceDetail instanceof LevelingToleranceDetail) {
                        updateTolerance(expandableListView, new ProjectToleranceLeveling((LevelingToleranceDetail) iToleranceDetail));
                    } else if (iToleranceDetail instanceof ToleranceDetailRecord) {
                        updateTolerance(expandableListView, new ProjectTolerance((ToleranceDetailRecord) iToleranceDetail));
                    }
                }
            });
        });

    }

    /**
     * 填充数据
     *
     * @param expandableListView
     * @param projectTolerance
     */
    private void updateTolerance(ExpandableListView expandableListView, IProjectTolerance projectTolerance) {
        if (!currentToleranceId.equals(projectTolerance.id())) {
            return;
        }
        boolean editable = projectTolerance.isWritable();
        //填充adapter
        if (adapter == null) {
            adapter = new ToleranceDetailAdapter(projectTolerance);
            expandableListView.setAdapter(adapter);
            expandAllItemView(expandableListView, projectTolerance);
        } else {
            adapter.updateData(projectTolerance);
            expandAllItemView(expandableListView, projectTolerance);
        }
        //设置点击事件
        expandableListView.setOnChildClickListener((parent, v, groupPosition, childPosition, viewId) -> {
            boolean forbidModify = false;
            if (getArguments() != null) {
                forbidModify = getArguments().getBoolean(KEY_FORBID_MODIFY, false);
            }
            if (editable && !forbidModify) {//执行编辑
                ToleranceAttributeEditFragment
                        .editAndSave(currentToleranceId,
                                projectTolerance.attributesType(groupPosition, childPosition),
                                projectTolerance.attributesName(groupPosition, childPosition),
                                projectTolerance.groupName(groupPosition),
                                getChildFragmentManager());
            } else if (forbidModify) {
                TipsFragment.tipsOnly(getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        getContext(),
                        getString(R.string.tips_tolerance_cannt_modify_because_has_surveyed));
            } else {
                TipsFragment.tipsOnly(getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        getContext(),
                        getString(R.string.tips_tolerance_template_cannt_delete));
            }
            return true;
        });
    }

    /**
     * 全部展开
     *
     * @param expandableListView
     * @param projectTolerance
     */
    private void expandAllItemView(ExpandableListView expandableListView, IProjectTolerance projectTolerance) {
        int groupCount = projectTolerance.groupCount();
        for (int index = 0; index < groupCount; index++) {
            expandableListView.expandGroup(index);
        }
    }

    private static final String KEY_FORBID_MODIFY = "__forbid_modify";

    public static ToleranceDetailFragment newInstance(Bundle args, boolean forbidModify) {
        ToleranceDetailFragment fragment = new ToleranceDetailFragment();
        args.putBoolean(KEY_FORBID_MODIFY, forbidModify);
        fragment.setArguments(args);
        return fragment;
    }

    public static ToleranceDetailFragment newInstance(Bundle args) {
        ToleranceDetailFragment fragment = new ToleranceDetailFragment();
        fragment.setArguments(args);
        return fragment;
    }

    public static void start(Context context, String toleranceId, boolean forbidModify) {
        Bundle args = new Bundle();
        args.putBoolean(KEY_FORBID_MODIFY, forbidModify);
        args.putString(BundleConstants.KEY_TOLERANCE_ID, toleranceId);
        FragmentContainerActivity.start(context, TAG, "限差设置", args);
    }
}