package com.bingce.controlnetwork.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.IProjectDataSource;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetailDataSource;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.level.ToleranceLevelRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.surveyor.method.ToleranceMethodKt;
import com.bingce.controlapphelper.util.SurveyorRecordUtil;
import com.bingce.controlapphelper.util.ToleranceUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.BaseIndexTextWithContextMenuAdapter;
import com.bingce.controlnetwork.adapter.IUpdateListAdapter;
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDeleteEdit;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 自定义限差管理
 */
public class ToleranceManageFragment extends BaseFragment {
    public static final String TAG = ToleranceManageFragment.class.getName();
    private BaseIndexTextWithContextMenuAdapter<ToleranceDetailRecord> adapter = null;

    private String projectType;

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (getArguments() != null) {
            projectType = getArguments().getString(BundleConstants.KEY_PROJECT_TYPE);
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable @org.jetbrains.annotations.Nullable ViewGroup container, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_tolerance_manager_layout, container, false);

        RecyclerView recyclerView = rootView.findViewById(R.id.list);
        IToleranceDetailDataSource toleranceDetailDataSource =
                SurveyorDatabaseFactory.instance.getToleranceDetailDataSource();
        toleranceDetailDataSource
                .getCustomLiveDataListByGroup()
                .observe(getViewLifecycleOwner(),
                        toleranceDetailRecords -> {
                            if (adapter == null) {
                                adapter = new _Adapter(
                                        getChildFragmentManager(),
                                        getViewLifecycleOwner(),
                                        getContext(),
                                        toleranceDetailRecords,
                                        this::clickToleranceItem);
                                recyclerView.setAdapter(adapter);
                            } else {
                                adapter.updateData(toleranceDetailRecords);
                            }
                        });

        View createButton = rootView.findViewById(R.id.create_new);
        if (createButton != null) {
            createButton.setOnClickListener(this::showSelectedToleranceDlg);
        }
        return rootView;
    }

    /**
     * 点击限差进行编辑
     *
     * @param item
     */
    private void clickToleranceItem(ToleranceDetailRecord item) {
        ThreadPoolUtil.execute(() -> {
            IProjectDataSource projectDataSource = SurveyorDatabaseFactory
                    .instance.getProjectDataSource();
            List<ProjectRecord> projectRecordList = projectDataSource
                    .getDataListByToleranceId(item.getId());
            boolean hasSurveyed = false;
            for (ProjectRecord projectRecord : projectRecordList) {
                if (SurveyorRecordUtil.isProjectSurveyor(projectRecord.getId())) {
                    hasSurveyed = true;
                    break;
                }
            }
            boolean finalHasSurveyed = hasSurveyed;
            runOnUI(() -> ToleranceDetailFragment.start(getContext(), item.getId(), finalHasSurveyed));
        });
    }

    /**
     * 新建选择限差参考模版
     */
    private void showSelectedToleranceDlg(View view) {
        SelectToleranceTemplateBottomSheetFragment.select(
                getChildFragmentManager(),
                getViewLifecycleOwner(),
                projectType,
                toleranceId -> onSelectedTolerance(
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        getContext(),
                        toleranceId));
    }

    private static void onSelectedTolerance(FragmentManager fragmentManager,
                                            LifecycleOwner lifecycleOwner,
                                            Context context,
                                            String toleranceId) {
        //查询 限差 名称
        IToleranceDetailDataSource toleranceDetailDataSource =
                SurveyorDatabaseFactory.instance.getToleranceDetailDataSource();
        ThreadPoolUtil.execute(() -> {
            //先检测预置模板
            ToleranceDetailRecord toleranceDetailRecord = (ToleranceDetailRecord) ToleranceLoadUtil.getTriangleToleranceLoad().detailOf(toleranceId);
            if (toleranceDetailRecord == null) {
                toleranceDetailRecord = toleranceDetailDataSource.getRecordSync(toleranceId);
            }
            if (toleranceDetailRecord == null) {
                return;
            }

            //查询限差完整名称
            ToleranceGroupRecord toleranceGroupRecord = ToleranceLoadUtil.getTriangleToleranceLoad().groupOf(toleranceDetailRecord.getGroupId());
            ToleranceLevelRecord toleranceLevelRecord = ToleranceLoadUtil.getTriangleToleranceLoad().levelOf(toleranceDetailRecord.getLevelId());
            if (toleranceGroupRecord == null) {
                ToastUtils.showShort(R.string.error_tips_tolerance_template_read_failure);
                return;
            }
            ToleranceDetailRecord finalToleranceDetailRecord = toleranceDetailRecord;
            ThreadPoolUtil.executeInMain(() -> StringInputFragment
                    .input(
                            fragmentManager,
                            context.getString(R.string.tips_please_input_new_tolerance_name),
                            ToleranceUtil.completedToleranceName(toleranceGroupRecord, toleranceLevelRecord, finalToleranceDetailRecord),
                            lifecycleOwner,
                            string -> {
                                if (string == null || string.isEmpty()) {
                                    ToastUtils.showShort(R.string.string_no_empty);
                                    return;
                                }
                                gotoEditNewTolerance(context, finalToleranceDetailRecord, string);
                            }
                    ));
        });
    }

    private static void gotoEditNewTolerance(Context context, ToleranceDetailRecord template, String name) {
        ToleranceDetailRecord newInstance = template.convertCustomRecord(template);
        newInstance.singleSurveyor = template.singleSurveyor;
        newInstance.setName(name);
        SurveyorDatabaseFactory
                .instance
                .getToleranceDetailDataSource()
                .saveWithCallback(newInstance, () -> ToleranceDetailFragment.start(context, newInstance.getId(), false));
    }

    private static class _Adapter extends BaseIndexTextWithContextMenuAdapter<ToleranceDetailRecord>
            implements IUpdateListAdapter<ToleranceDetailRecord> {
        public _Adapter(
                FragmentManager fragmentManager,
                LifecycleOwner lifecycleOwner,
                Context context,
                List<ToleranceDetailRecord> records,
                IOnItemClick<ToleranceDetailRecord> callback) {
            super(records == null ? new ArrayList<>() : records, callback, new ContextMenuDeleteEdit<ToleranceDetailRecord>() {
                @Override
                protected void delete(ToleranceDetailRecord item) {
                    //检测是否有project使用了当前限差方案
                    ThreadPoolUtil.execute(() -> {
                        long count = SurveyorDatabaseFactory
                                .instance
                                .getProjectDataSource()
                                .recordCountWithTolerance(item.getId());
                        if (count <= 0) {
                            IToleranceDetailDataSource toleranceDetailDataSource =
                                    SurveyorDatabaseFactory.instance.getToleranceDetailDataSource();
                            toleranceDetailDataSource.delete(item);
                        } else {
                            ThreadPoolUtil.executeInMain(() -> Toast
                                    .makeText(context, R.string.tips_tolerance_cannt_delete_because_of_used, Toast.LENGTH_SHORT)
                                    .show());
                        }
                    });
                }

                @Override
                protected void edit(ToleranceDetailRecord item) {
                    ToleranceMethodKt.changeToleranceName(
                            fragmentManager, lifecycleOwner, item);
                }
            });
        }
    }

    public static void start(Context context, String projectType) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        FragmentContainerActivity.start(context, TAG, R.string.name_custom_telorance_manager, bundle);
    }

    public static ToleranceManageFragment newInstance(Bundle bundle) {
        ToleranceManageFragment fragment = new ToleranceManageFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

}
