package com.bingce.controlnetwork.fragment;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.utils.ThreadPoolUtil;

public class ToleranceSelectFragment extends AbstractProjectToleranceFragment {
    public static final String TAG = ToleranceSelectFragment.class.getName();

    @Override
    protected void onCreateView(View view) {
        String toleranceId = null;
        if (getArguments() != null) {
            toleranceId = getArguments().getString(BundleConstants.KEY_TOLERANCE_ID);
        }
        initData(view, toleranceId);
        setHasOptionsMenu(true);
    }

    private void initData(View view, String toleranceId) {
        if (TextUtils.isEmpty(toleranceId)) {
            IToleranceDetail firstDefaultTolerance = getNewDefaultTolerance(projectType);
            updateToleranceDetail(view, firstDefaultTolerance);
        } else {
            ThreadPoolUtil.execute(() -> {
                IToleranceOperate operate = ToleranceOperateUtil.getOperate(toleranceId);
                if (operate != null) {
                    IToleranceDetail iToleranceDetail = operate.getIToleranceDetail(toleranceId);
                    updateToleranceDetail(view, iToleranceDetail);
                }
            });
        }
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.tolerance_select_menu, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (item.getItemId() == R.id.confirm_button) {
            Intent intent = new Intent();
            intent.putExtra(BundleConstants.KEY_TOLERANCE_ID, currentTolerance());
            Activity activity = getActivity();
            if (activity != null) {
                activity.setResult(Activity.RESULT_OK, intent);
                activity.finish();
            }
        }
        return super.onOptionsItemSelected(item);
    }

    public static void start(Context context, String toleranceId, String projectType,
                             ActivityResultLauncher<Intent> launcher) {
        if (launcher == null) {
            return;
        }
        Bundle args = new Bundle();
        args.putString(BundleConstants.KEY_TOLERANCE_ID, toleranceId);
        args.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        launcher.launch(FragmentContainerActivity.startIntent(context, TAG, R.string.select_tolerance, args));
    }

    public static ActivityResultLauncher<Intent> createActivityLauncher(Fragment fragment, ISelectCallback selectCallback) {
        return fragment.registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
            if (result != null && Activity.RESULT_OK == result.getResultCode()) {
                Intent intent = result.getData();
                if (intent != null) {
                    String id = intent.getStringExtra(BundleConstants.KEY_TOLERANCE_ID);
                    selectCallback.onSelectTolerance(id);
                    return;
                }
            }
//            selectCallback.onSelectTolerance(null);
        });
    }

    public static ToleranceSelectFragment newInstance(Bundle bundle) {
        ToleranceSelectFragment fragment = new ToleranceSelectFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    public interface ISelectCallback {
        void onSelectTolerance(String id);
    }

}