package com.bingce.controlnetwork.fragment.base;

import android.os.Bundle;
import android.view.View;

import androidx.annotation.IdRes;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Lifecycle;

public abstract class BaseLazyFragment extends Fragment {

    /**
     * 是否执行懒加载
     */
    private boolean isLoaded = false;

    /**
     * 当前Fragment是否对用户可见
     */
    private boolean isVisibleToUser = false;

    /**
     * view是否已经创建
     */
    private boolean onViewCreated = false;

    final protected void runOnUI(Runnable runnable) {
        if (getActivity() == null) {
            return;
        }
        getActivity().runOnUiThread(runnable);
    }

    final protected <T extends View> T findViewById(@IdRes int id) {
        if (getActivity() == null) {
            return null;
        }
        return getActivity().findViewById(id);
    }

    final protected void setCreatedData(Runnable runnable) {
        if (getLifecycle().getCurrentState().isAtLeast(Lifecycle.State.CREATED)) {
            runnable.run();
        }
    }


    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        this.isVisibleToUser = isVisibleToUser;
        judgeLoadData();
    }

    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        isVisibleToUser = !hidden;
    }


    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        onViewCreated = true;
        judgeLoadData();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        isLoaded = false;
        isVisibleToUser = false;
        onViewCreated = false;
    }

    private void judgeLoadData() {
        if (!isLoaded && isVisibleToUser && onViewCreated) {
            loadData();
            isLoaded = true;
        }
    }

    protected abstract void loadData();
}
