package com.bingce.controlnetwork.fragment.bottompage

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.bingce.controlnetwork.databinding.FragmentBottomPageContentBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class BottomPageContentFragment(private val dataList: List<BottomBean>) : Fragment() {
    private var _binding: FragmentBottomPageContentBinding? = null
    val mBinding get() = _binding!!

    private val viewModel: BottomSwitchVm by viewModels({ requireParentFragment() })

    private val mainScope = MainScope()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentBottomPageContentBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mainScope.cancel()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        initView()
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        if (!hidden) {
            refreshTitleVisiable()
        }
    }

    private fun initView() {
        mBinding.back.setOnClickListener {
            viewModel.selectList.removeLast()
            getBottomPageFragment()?.removeFragment()
        }

        val adapter =
            BottomPageAdapter(requireContext(), dataList, object : BottomPageAdapter.CallBack {
                override fun back(bottomBean: BottomBean) {
                    itemClick(bottomBean)
                }
            })
        mBinding.recyclerView.adapter = adapter

        mBinding.title.text = getBottomPageFragment()?.getTitle(this)
        refreshTitleVisiable()
    }

    private fun itemClick(bottomBean: BottomBean) {
        mainScope.launch {
            val dataNext = withContext(Dispatchers.IO) {
                getBottomPageFragment()?.getDataNext(bottomBean)
            }
            viewModel.selectList.add(bottomBean)
            if (dataNext.isNullOrEmpty()) {
                getBottomPageFragment()?.setResultData()
            } else {
                getBottomPageFragment()?.addFragment(dataNext, true)
            }
        }
    }

    private fun getBottomPageFragment(): BottomPageFragment? {
        return parentFragment as? BottomPageFragment
    }

    private fun refreshTitleVisiable() {
        parentFragmentManager.fragments.let {
            if (it.size <= 1) {
                mBinding.back.visibility = View.GONE
            } else {
                mBinding.back.visibility = View.VISIBLE
            }
        }
    }


}