package com.bingce.controlnetwork.fragment.bottompage

import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentTransaction
import androidx.fragment.app.setFragmentResult
import androidx.fragment.app.viewModels
import androidx.lifecycle.LifecycleOwner
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.GsonUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentSingleFramelayoutLayoutBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class BottomPageFragment(val data: IData) : DialogFragment() {

    private val mainScope = MainScope()

    private var _binding: FragmentSingleFramelayoutLayoutBinding? = null
    val mBinding get() = _binding!!

    private val viewModel: BottomSwitchVm by viewModels()

    companion object {
        @JvmStatic
        fun showBottom(
                fragmentManager: FragmentManager,
                lifecycleOwner: LifecycleOwner,
                data: IData,
                callBack: CallBack
        ) {
            val bottomPageFragment = BottomPageFragment(data)
            val tag = System.currentTimeMillis().toString()

            fragmentManager.setFragmentResultListener(
                    tag,
                    lifecycleOwner
            ) { requestKey: String, bundle: Bundle ->
                bottomPageFragment.dismiss()
                if (requestKey == tag) {
                    bundle.getString(BundleConstants.KEY_SELECT_LIST)?.let { it ->
                        GsonUtil.toAny<List<BottomBean>>(it)?.let juadge@{
                            if (it.size < 2) {
                                ToastUtils.showShort(R.string.no_data)
                                return@juadge
                            }
                            callBack.back(it)
                        }
                    }
                }
            }
            bottomPageFragment.show(fragmentManager, tag)
        }
    }


    override fun onStart() {
        super.onStart()
        dialog?.window?.run {
            setWindowAnimations(R.style.animate_dialog)
            setBackgroundDrawable(null)
            val params = attributes
            params.gravity = Gravity.BOTTOM
            params.width = ViewGroup.LayoutParams.MATCH_PARENT
            params.height = ViewGroup.LayoutParams.WRAP_CONTENT
            attributes = params
        }

    }


    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        _binding = FragmentSingleFramelayoutLayoutBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mainScope.cancel()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        mainScope.launch {
            val firstPageData = withContext(Dispatchers.IO) {
                data.getData(null)
            }
            firstPageData?.let {
                addFragment(it, false)
            }
        }
    }

    fun addFragment(dataNext: List<BottomBean>, isNeedAnim: Boolean) {
        addOrDeleteFragment(isNeedAnim) {
            val contentFragment = BottomPageContentFragment(dataNext)
            add(R.id.container, contentFragment)
        }
    }

    fun removeFragment() {
        childFragmentManager.fragments.last()?.let {
            addOrDeleteFragment {
                remove(it)
            }
        }
    }

    fun setResultData() {
        val toJson = GsonUtil.toJson(viewModel.selectList)
        val bundle = Bundle()
        bundle.putString(BundleConstants.KEY_SELECT_LIST, toJson)
        setFragmentResult(tag!!, bundle)
    }

    suspend fun getDataNext(currentData: BottomBean): List<BottomBean>? {
        return data.getData(currentData)
    }

    fun getTitle(bottomPageContentFragment: BottomPageContentFragment): String {
        return data.getTitle(childFragmentManager.fragments.indexOf(bottomPageContentFragment))
    }

    private fun addOrDeleteFragment(
            isNeedAnim: Boolean = true,
            block: FragmentTransaction.() -> Unit
    ) {
        val transaction = childFragmentManager
                .beginTransaction()
        if (isNeedAnim) {
            transaction.setCustomAnimations(R.anim.slide_in_right, R.anim.slide_out_right)
        }
        block(transaction)
        transaction.commit()
    }

    interface CallBack {
        fun back(selectData: List<BottomBean>)
    }


}