package com.bingce.controlnetwork.fragment.heighttraverse;

import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.IdRes;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.route.RouteRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.util.CheckLocationUtil;
import com.bingce.controlapphelper.util.GetLocationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.DialogCreateRouteBinding;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.utils.ThreadPoolUtil;
import com.rengwuxian.materialedittext.MaterialEditText;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

public class CreateOrEditRouteDialog extends DialogFragment {
    private PickPointResult pickPointResultStart;
    private PickPointResult pickPointResultEnd;

    //选择起点还是终点的标志
    private static final String SELECT_START_TAG = "SELECT_START_TAG";
    private static final String SELECT_END_TAG = "SELECT_END_TAG";

    private static final String KEY_PROJECT_ID = "KEY_PROJECT_ID";
    private static final String KEY_ROUTE_ID = "KEY_ROUTE_ID";

    private static final String KEY_ROUTE_NAME = "KEY_ROUTE_NAME";
    private static final String KEY_START_POINT_ID = "KEY_START_POINT_ID";
    private static final String KEY_END_POINT_ID = "KEY_END_POINT_ID";

    private DialogCreateRouteBinding mBinding;
    private GetLocationUtil getLocationUtil;

    public static void start(String projectId, String routeId, FragmentManager fragmentManager,
                             @NonNull LifecycleOwner lifecycleOwner,
                             CallBack callback) {
        Bundle args = new Bundle();
        args.putString(KEY_PROJECT_ID, projectId);
        args.putString(KEY_ROUTE_ID, routeId);
        showFragment(args, fragmentManager, lifecycleOwner, callback);
    }


    private static void showFragment(Bundle args, FragmentManager fragmentManager, LifecycleOwner lifecycleOwner, CallBack callback) {
        String tag = String.valueOf(System.currentTimeMillis());
        CreateOrEditRouteDialog fragment = new CreateOrEditRouteDialog();
        fragment.setArguments(args);
        fragmentManager
                .setFragmentResultListener(
                        tag,
                        lifecycleOwner,
                        (requestKey, resultBundle) -> {
                            if (tag.equals(requestKey)) {
                                callback.result(
                                        resultBundle.getString(KEY_ROUTE_NAME),
                                        resultBundle.getString(KEY_START_POINT_ID),
                                        resultBundle.getString(KEY_END_POINT_ID)
                                );
                            }
                        });
        fragment.show(fragmentManager, tag);
    }

    private final ActivityResultLauncher<Intent> defaultStationLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (SELECT_START_TAG.equals(type)) {
            pickPointResultStart = pickPointResult;
        } else if (SELECT_END_TAG.equals(type)) {
            pickPointResultEnd = pickPointResult;
        }
        if (pickPointResult == null || isListEmpty(pickPointResult.getSelectedIds()) || isListEmpty(pickPointResult.getSelectedNames())
                || pickPointResult.getSelectedNames().size() != pickPointResult.getSelectedIds().size()) {
            ToastUtils.showShort(R.string.abnormal_point_selection);
            return;
        }
        setEtData(type);
    });

    private void setEtData(String type) {
        if (SELECT_START_TAG.equals(type)) {
            setEtDataSingle(mBinding.ilStart.etPoint, mBinding.ilStart.etHeight,
                    mBinding.ilStart.etB, mBinding.ilStart.etL, pickPointResultStart.getSelectedIds().get(0));
        } else if (SELECT_END_TAG.equals(type)) {
            setEtDataSingle(mBinding.ilEnd.etPoint, mBinding.ilEnd.etHeight,
                    mBinding.ilEnd.etB, mBinding.ilEnd.etL,
                    pickPointResultEnd.getSelectedIds().get(0));
        }
    }

    private void setEtDataSingle(MaterialEditText etPointName, MaterialEditText etPointHeight,
                                 MaterialEditText etB, MaterialEditText etL,
                                 String pointId) {
        ThreadPoolUtil.execute(() -> {
            PointRecord pointRecord = SurveyorDatabaseFactory.instance.getPointDataSource().findByIdSync(pointId);
            ThreadPoolUtil.executeInMain(() -> {
                etPointName.setText(pointRecord.getName());
                etPointHeight.setText(doubleToString(pointRecord.getZ()));

                if (pointRecord.getLatitude() != 0 && pointRecord.getLongitude() != 0) {
                    etB.setText(doubleToString(pointRecord.getLatitude()));
                    etL.setText(doubleToString(pointRecord.getLongitude()));
                } else {
                    etB.setText("");
                    etL.setText("");
                }
            });
        });
    }

    private <T> boolean isListEmpty(List<T> list) {
        return list == null || list.isEmpty();
    }

    private MaterialEditText getEtPoint(@IdRes int id) {
        View view = getView();
        assert view != null;
        return view.findViewById(id);
    }

    @Override
    public void onStart() {
        super.onStart();
//        getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(0x33000000));
        getDialog().getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.MATCH_PARENT);
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getLocationUtil = new GetLocationUtil();
        getLifecycle().addObserver(getLocationUtil);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        mBinding = DialogCreateRouteBinding.inflate(inflater, container, false);
        initView();
        initListener();
        initData();
        return mBinding.getRoot();
    }

    private void initView() {
        mBinding.ilStart.llCoord.setVisibility(View.VISIBLE);
        mBinding.ilEnd.llCoord.setVisibility(View.VISIBLE);
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
    }

    private void initData() {
        String routeId = getRouteId();
        if (routeId != null) {
            ThreadPoolUtil.execute(() -> {
                RouteRecord routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getByIdSync(routeId);
                if (routeRecord != null) {
                    PointRecord startPointRecord = SurveyorDatabaseFactory.instance.getPointDataSource().findByIdSync(routeRecord.getStartPointId());
                    PointRecord endPointRecord = SurveyorDatabaseFactory.instance.getPointDataSource().findByIdSync(routeRecord.getEndPointId());
                    setPickPointResultForEdit(SELECT_START_TAG, startPointRecord.getId(), startPointRecord.getName());
                    setPickPointResultForEdit(SELECT_END_TAG, endPointRecord.getId(), endPointRecord.getName());

                    ThreadPoolUtil.executeInMain(() -> {
                        setEtData(SELECT_START_TAG);
                        setEtData(SELECT_END_TAG);
                    });
                }
            });
        }
    }

    private void setPickPointResultForEdit(String type, String id, String name) {
        if (type.equals(SELECT_START_TAG)) {
            pickPointResultStart = getPickPointResultFromIdName(id, name);
        } else {
            pickPointResultEnd = getPickPointResultFromIdName(id, name);
        }
    }

    private PickPointResult getPickPointResultFromIdName(String id, String name) {
        List<String> ids = new ArrayList<>();
        List<String> names = new ArrayList<>();
        ids.add(id);
        names.add(name);
        PickPointResult pickPointResult = new PickPointResult();
        pickPointResult.setSelectedIds(ids);
        pickPointResult.setSelectedNames(names);
        return pickPointResult;
    }

    private void initListener() {
        mBinding.tvSelectStartPoint.setOnClickListener(v -> selectPoint(SELECT_START_TAG));
        mBinding.tvSelectEndPoint.setOnClickListener(v -> selectPoint(SELECT_END_TAG));

        mBinding.btnCancel.setOnClickListener((v) -> dismissAllowingStateLoss());

        mBinding.btnConfirm.setOnClickListener((v) -> {
            checkConfirm();
        });

        mBinding.ivGetLocationStartPoint.setOnClickListener(v -> {
            getLocation(result -> {
                setLocation(mBinding.ilStart.etB, mBinding.ilStart.etL, result);
            });
        });
        mBinding.ivGetLocationEndPoint.setOnClickListener(v -> {
            getLocation(result -> {
                setLocation(mBinding.ilEnd.etB, mBinding.ilEnd.etL, result);
            });
        });
    }

    private void getLocation(GetLocationUtil.IResultListener resultListener) {
        CheckLocationUtil.check(requireContext(), () -> {
            getLocationUtil.getResult(resultListener);
        });
    }

    private void setLocation(MaterialEditText etB, MaterialEditText etL, GetLocationUtil.Result result) {
        if (result != null) {
            etB.setText(result.getLatitude());
            etL.setText(result.getLongitude());
        }
    }

    private void checkConfirm() {
        ThreadPoolUtil.execute(() -> {

            String startPointName = mBinding.ilStart.etPoint.getText().toString().trim();
            if (TextUtils.isEmpty(startPointName)) {
                ToastUtils.showShort(R.string.please_enter_or_select_a_starting_point);
                return;
            }

            String endPointName = mBinding.ilEnd.etPoint.getText().toString().trim();
            if (TextUtils.isEmpty(endPointName)) {
                ToastUtils.showShort(R.string.please_enter_or_select_a_destination);
                return;
            }

            //取消起点终点重复的限制
//            if (startPointName.equals(endPointName)) {
//                ToastUtils.showShort(R.string.duplicate_start_and_end_names);
//                return;
//            }

            boolean canEdit = checkCanEdit();
            if (!canEdit) {
                ToastUtils.showShort(R.string.route_has_data_not_edit_hint);
                return;
            }

            String startHeight = mBinding.ilStart.etHeight.getText().toString().trim();
            String startB = mBinding.ilStart.etB.getText().toString().trim();
            String startL = mBinding.ilStart.etL.getText().toString().trim();
            String endHeight = mBinding.ilEnd.etHeight.getText().toString().trim();
            String endB = mBinding.ilEnd.etB.getText().toString().trim();
            String endL = mBinding.ilEnd.etL.getText().toString().trim();

            PointRecord startPoint;
            PointRecord endPoint;

            String projectId = getProjectId();
            startPoint = SurveyorDatabaseFactory.instance.getPointDataSource().findByNameSync(startPointName, projectId);
            endPoint = SurveyorDatabaseFactory.instance.getPointDataSource().findByNameSync(endPointName, projectId);

            if (startPoint == null) {
                startPoint = createPoint(startPointName, startHeight, startB, startL);
            } else {
                editPoint(startPoint, startPointName, startHeight, startB, startL);
            }

            if (endPoint == null) {
                endPoint = createPoint(endPointName, endHeight, endB, endL);
            } else {
                editPoint(endPoint, endPointName, endHeight, endB, endL);
            }

            //存数据库
            SurveyorDatabaseFactory.instance.getPointDataSource().saveSync(startPoint);
            SurveyorDatabaseFactory.instance.getPointDataSource().saveSync(endPoint);

            setResultToPage(startPoint.getId(), endPoint.getId(),
                    startPoint.getName() + "-" + endPoint.getName());
        });
    }

    @WorkerThread
    private boolean checkCanEdit() {
        //路线有数据时候不能编辑
        String routeId = getRouteId();
        if (routeId == null) {
            return true;
        }
        RouteRecord routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getByIdSync(getRouteId());
        if (routeRecord != null) {
            List<SurveyorStationRecord> stationList = SurveyorDatabaseFactory.instance.getSurveyorStation().getByProjectRoute(getProjectId(), getRouteId());
            if (stationList != null && !stationList.isEmpty()) {
                for (SurveyorStationRecord stationRecord : stationList) {
                    long recordCount = SurveyorDatabaseFactory.instance.getSurveyorRecordDataSource().recordCount(stationRecord.getId());
                    if (recordCount > 0) {
                        return false;
                    }
                }
            }
        }
        return true;
    }

    @WorkerThread
    private PointRecord createPoint(String pointName, String height, String b, String l) {
        return createPointRecord(pointName, height, b, l);
    }

    @WorkerThread
    private void editPoint(PointRecord pointRecord, String pointNameEdit, String height, String b, String l) {
        pointRecord.setName(pointNameEdit);
        pointRecord.setZ(stringToDouble(height));
        pointRecord.setLatitude(stringToDouble(b));
        pointRecord.setLongitude(stringToDouble(l));
    }

    private void setResultToPage(String startPointId, String endPointId, String routeName) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> {
                Bundle resultBundle = new Bundle();
                resultBundle.putString(KEY_START_POINT_ID, startPointId);
                resultBundle.putString(KEY_END_POINT_ID, endPointId);
                resultBundle.putString(KEY_ROUTE_NAME, routeName);
                getParentFragmentManager()
                        .setFragmentResult(getTag(), resultBundle);
                dismissAllowingStateLoss();
            });
        }
    }

    private PointRecord createPointRecord(String pointText, String height, String b, String l) {
        String pointType;
        if (isKnownPoint(height)) {
            pointType = PointRecord.TYPE_KNOWN_POINT;
        } else {
            pointType = PointRecord.TYPE_UNKNOWN_POINT;
        }
        double h = stringToDouble(height);
        PointRecord point = PointRecord.point(getProjectId(), pointType, pointText, "", 0, 0, h, "", -1);
        point.setLatitude(stringToDouble(b));
        point.setLongitude(stringToDouble(l));
        return point;
    }

    private boolean isKnownPoint(String height) {
        double heightD = stringToDouble(height);
        return heightD != 0;
    }

    private double stringToDouble(String value) {
        if (TextUtils.isEmpty(value)) {
            return 0;
        }
        return Double.parseDouble(value);
    }

    private String doubleToString(double value) {
        return String.valueOf(value);
    }

    private void tipsOnly(String msg) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(() -> TipsFragment.tipsOnly(
                    getChildFragmentManager(),
                    getViewLifecycleOwner(),
                    requireContext(),
                    msg));
        }
    }

    private void selectPoint(String type) {
        String title = null;
        PickPointResult pickPointResultSelected = null;
        List<String> filderIds = null;
        if (SELECT_START_TAG.equals(type)) {
            pickPointResultSelected = pickPointResultStart;
//            filderIds = pickPointResultEnd != null ? pickPointResultEnd.getSelectedIds() : null;
            title = Tools.getString(R.string.choose_starting_point);
        } else if (SELECT_END_TAG.equals(type)) {
            pickPointResultSelected = pickPointResultEnd;
//            filderIds = pickPointResultStart != null ? pickPointResultStart.getSelectedIds() : null;
            title = Tools.getString(R.string.select_end_point);
        }
        PickPointFragment.pickPoint(
                getContext(),
                type,
                1,
                title,
                filderIds,//filderIds取消起点和终点的限制
                getProjectId(), pickPointResultSelected, defaultStationLauncher);
    }

    public interface CallBack {
        void result(String routeName, String startPointId, String endPointId);
    }

    private String getProjectId() {
        if (getArguments() != null) {
            return getArguments().getString(KEY_PROJECT_ID);
        }
        return null;
    }

    private String getRouteId() {
        if (getArguments() != null) {
            return getArguments().getString(KEY_ROUTE_ID);
        }
        return null;
    }


}
