package com.bingce.controlnetwork.fragment.heighttraverse.viewmodel

import androidx.fragment.app.FragmentManager
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.bingce.controlnetwork.fragment.heighttraverse.CreateOrEditRouteDialog
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.route.RouteRecord
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class HeightCreateOrEditVm : ViewModel() {

    fun showCreateOrEditDialog(
        projectId: String,
        routeId: String?,
        childFragmentManager: FragmentManager,
        viewLifecycleOwner: LifecycleOwner
    ) {
        CreateOrEditRouteDialog.start(
            projectId, routeId, childFragmentManager, viewLifecycleOwner
        ) { routeName, startPointId, endPointId ->
            saveOrUpdateRecord(projectId, routeName, routeId, startPointId, endPointId)
        }
    }

    /**
     * 存路线的同时存默认项目
     */
    private fun saveOrUpdateRecord(
        projectId: String,
        routeName: String,
        routeId: String?,
        startPointId: String,
        endPointId: String
    ) {
        viewModelScope.launch(Dispatchers.IO) {
            if (routeId != null) {
                val routeRecord =
                    SurveyorDatabaseFactory.instance.getRouteDataSource().getById(routeId)
                routeRecord?.let {
                    it.routeName = routeName
                    it.startPointId = startPointId
                    it.endPointId = endPointId
                    SurveyorDatabaseFactory.instance.getRouteDataSource().update(it)
                }
            } else {
                val routeRecord =
                    RouteRecord.createRouteRecord(projectId, routeName, startPointId, endPointId)
                SurveyorDatabaseFactory.instance.getRouteDataSource().saveRecord(routeRecord)
                //存默认的项目id
                SurveyorDatabaseFactory.instance.configDataSource.setSync(
                    ConfigConstants.KEY_DEFAULT_PROJECT_ID,
                    projectId
                )
            }
        }
    }

}