package com.bingce.controlnetwork.fragment.projectdetail


import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.annotation.WorkerThread
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.ProjectType
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.event.SwitchPageEvent
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity
import com.bingce.controlnetwork.databinding.FragmentProjectDetailListLayoutBinding
import com.bingce.controlnetwork.fragment.PointListFragment
import com.bingce.controlnetwork.fragment.ProjectToleranceSettingFragment
import com.bingce.controlnetwork.fragment.base.BaseFragment
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.greenrobot.eventbus.EventBus


abstract class ProjectDetailFragment : BaseFragment() {

    companion object {
        @JvmStatic
        fun start(context: Context, projectType: String?, projectId: String, projectName: String?) {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId)
            bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType)
            val tag = if (projectType == ProjectType.HEIGHT_TRAVERSE.flag) {
                ProjectDetailHeightTraverseFragment.TAG
            } else {
                ProjectDetailTriangleFragment.TAG
            }
            FragmentContainerActivity.start(context, tag, projectName, bundle)
        }
    }

    private var _binding: FragmentProjectDetailListLayoutBinding? = null
    val mBinding get() = _binding!!

    private val configDataSource = SurveyorDatabaseFactory.instance.configDataSource

    protected val projectId: String by lazy {
        requireArguments().getString(BundleConstants.KEY_PROJECT_ID)!!
    }

    protected val projectType: String? by lazy {
        requireArguments().getString(BundleConstants.KEY_PROJECT_TYPE)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentProjectDetailListLayoutBinding.inflate(inflater, container, false)
        initView()
        setHasOptionsMenu(true)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    protected abstract fun getContentFragment(): Fragment

    @WorkerThread
    protected abstract fun saveDefaultProjectAndSoOn(): String?

    protected abstract fun clickCreateStation()

    private fun initView() {
        //设置开始测量
        mBinding.ilBtn.btnOne.text = getString(R.string.new_surveyor_station)
        mBinding.ilBtn.btnTwo.text = getString(R.string.start_surveyor)

        childFragmentManager
            .beginTransaction()
            .replace(R.id.fragment_container, getContentFragment(), "__station_list")
            .commit()
        //已知点
        initKnownPoint()
        //未知点
        initUnKnownPoint()
        //去测量页面
        initGoMeasurePage()
        //新建测站
        initCreateStation()
    }

    private fun initCreateStation() =
        mBinding.ilBtn.btnOne.setOnClickListener { v -> clickCreateStation() }

    private fun initGoMeasurePage() {
        mBinding.ilBtn.btnTwo.setOnClickListener { view ->
            lifecycleScope.launch(Dispatchers.IO) {
                val msg = saveDefaultProjectAndSoOn()
                if (!msg.isNullOrEmpty()) {
                    ToastUtils.showShort(msg)
                    return@launch
                }
                EventBus.getDefault().post(
                    SwitchPageEvent<Any?>(
                        SwitchPageEvent.TYPE_SWITCH_TO_MEASURE_PAGE_AND_DATA,
                        null
                    )
                )
                val activity = activity
                activity?.finish()
            }
        }
    }

    private fun initUnKnownPoint() {
        mBinding.unknownPointList.setOnClickListener { view ->
            PointListFragment.go2UnknownPointList(
                projectId,
                projectType,
                context
            )
        }
    }

    private fun initKnownPoint() {
        mBinding.knownPointList.setOnClickListener { view ->
            PointListFragment.go2KnownPointList(
                projectId,
                projectType,
                context
            )
        }
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.project_detail_fragment_menu, menu)
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.project_parameter_setting) {
            //设置参数点击事件
            ProjectToleranceSettingFragment.start(context, projectId, projectType)
            return true
        }
        return super.onOptionsItemSelected(item)
    }


}