package com.bingce.controlnetwork.fragment.projectdetail

import android.os.Bundle
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.surveyorcycle.SurveyorPeriodRecord
import com.bingce.controlnetwork.fragment.BottomSheetSurveyorPeriodFragment
import com.bingce.controlnetwork.fragment.SelectSurveyorStationTypeFragment
import com.bingce.controlnetwork.fragment.SelectSurveyorStationTypeWellSubFragment
import com.bingce.controlnetwork.fragment.station.defaultstation.DefaultStationSetupFragment
import com.bingce.controlnetwork.fragment.station.line.attached.LineStationAttachedSetupFragment
import com.bingce.controlnetwork.fragment.station.line.close.LineStationCloseSetupFragment
import com.bingce.controlnetwork.fragment.station.line.random.LineStationRandomSetupFragment
import com.bingce.controlnetwork.fragment.station.line.single.LineStationSingleSetupFragment
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellDoubleGroundSetupFragment
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellDoubleUnderGroundSetupFragment
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellSetupFragment
import com.bingce.controlnetwork.newui.stationlist.StationListFragment
import com.bingce.controlnetwork.newui.stationlist.StationListVm

class ProjectDetailTriangleFragment : ProjectDetailFragment() {

    private var currentPeriodId: String = ""

    private var contentFragment: StationListFragment? = null

    private val stationListVm by viewModels<StationListVm>()

    companion object {
        @JvmField
        val TAG = ProjectDetailTriangleFragment::class.simpleName

        @JvmStatic
        fun newInstance(bundle: Bundle): ProjectDetailTriangleFragment {
            val fragment = ProjectDetailTriangleFragment()
            fragment.arguments = bundle
            return fragment
        }

    }

    override fun getContentFragment(): Fragment {
        //测站列表
        contentFragment = StationListFragment()
        return contentFragment!!
    }

    override fun saveDefaultProjectAndSoOn(): String? {
        SurveyorDataBase.getInstance().runInTransaction {
            SurveyorDatabaseFactory.instance.configDataSource.setSync(
                ConfigConstants.KEY_DEFAULT_PROJECT_ID,
                projectId
            )
            SurveyorDatabaseFactory.instance.configDataSource.setSync(
                ConfigConstants.KEY_DEFAULT_PERIOD_ID,
                currentPeriodId
            )
        }
        return null
    }

    override fun clickCreateStation() {
        SelectSurveyorStationTypeFragment
            .select(
                childFragmentManager,
                viewLifecycleOwner,
                projectType
            ) { type: String ->
                if (SelectSurveyorStationTypeFragment.TYPE_DEFAULT == type) {
                    DefaultStationSetupFragment.start(context, projectId, currentPeriodId)
                } else if (SelectSurveyorStationTypeFragment.TYPE_LINE_SINGLE == type) {
                    //支导线
                    LineStationSingleSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeFragment.TYPE_LINE_ATTACH == type) {
                    //附合导线
                    LineStationAttachedSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeFragment.TYPE_LINE_CLOSE == type) {
                    LineStationCloseSetupFragment.startCreateCloseLine(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeFragment.TYPE_LINE_RANDOM == type) {
                    LineStationRandomSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeFragment.TYPE_WELL_WIRE == type) {
                    //一井定向
                    LineStationWellSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeFragment.TYPE_WELL_WIRE_DOUBLE == type) {
                    //二井定向
                    selectWellDoubleType()
                }
            }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        stationListVm.isCanEditDelete = true
        initListener()
        initData()
    }

    private fun initListener() {
        initSwithCycle()
    }

    private fun initSwithCycle() {
        mBinding.cycleLayout.setOnClickListener {
            //切换测期
            BottomSheetSurveyorPeriodFragment.pick(
                childFragmentManager,
                viewLifecycleOwner,
                projectId,
                currentPeriodId
            ) { id: String, name: String ->
                setCreatedData {
                    currentPeriodId = id
                    mBinding.orderText.text = name
                    updateStationList(projectId, currentPeriodId)
                }
            }
        }
    }

    private fun initData() {
        //获取默认测期,刷新页面
        SurveyorDatabaseFactory.instance
            .surveyorPeriod
            .getDefault(
                projectId
            ) { instance: SurveyorPeriodRecord ->
                setCreatedData {
                    currentPeriodId = instance.value
                    runOnUI {
                        //设置测期名称
                        mBinding.orderText.text = instance.name
                        //设置测站列表
                        updateStationList(projectId, instance.id)
                    }
                }
            }
    }

    private fun updateStationList(projectId: String, periodId: String) {
        if (contentFragment == null) {
            return
        }
        stationListVm.setArgument(projectId, periodId)
        contentFragment!!.refreshUi()
    }

    /**
     * 选择二井定向地上还是地下的弹框
     */
    private fun selectWellDoubleType() {
        SelectSurveyorStationTypeWellSubFragment
            .select(
                childFragmentManager,
                viewLifecycleOwner
            ) { type: String ->
                if (SelectSurveyorStationTypeWellSubFragment.TYPE_GROUND == type) {
                    LineStationWellDoubleGroundSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                } else if (SelectSurveyorStationTypeWellSubFragment.TYPE_UNDERGROUND == type) {
                    LineStationWellDoubleUnderGroundSetupFragment.startCreate(
                        context,
                        projectId,
                        currentPeriodId
                    )
                }
            }
    }


}