package com.bingce.controlnetwork.fragment.projectdetail.content

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.room.withTransaction
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDelete
import com.bingce.controlnetwork.fragment.TipsFragment
import com.bingce.controlnetwork.fragment.base.BaseFragment
import com.bingce.controlnetwork.fragment.heighttraverse.adapter.RouteListAdapter
import com.bingce.controlnetwork.fragment.heighttraverse.viewmodel.HeightCreateOrEditVm
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.route.RouteRecord
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.PointUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentListLayoutBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class RouteFragment : BaseFragment() {

    private val heightCreateOrEditVm: HeightCreateOrEditVm by viewModels()

    private var _binding: FragmentListLayoutBinding? = null
    val mBinding get() = _binding!!

    private val dataList = mutableListOf<RouteRecord>()

    val adapter: RouteListAdapter by lazy {
        RouteListAdapter(dataList,
            {
                //路线item点击
                doLookAndEditRoute(it)
            },
            object : ContextMenuDelete<RouteRecord>() {
                override fun delete(item: RouteRecord) {
                    doDelete(item)
                }
            })
    }

    private fun doLookAndEditRoute(item: RouteRecord) {
        heightCreateOrEditVm.showCreateOrEditDialog(
            getProjectId()!!,
            routeId = item.getId(),
            childFragmentManager,
            viewLifecycleOwner
        )
    }

    private fun doDelete(item: RouteRecord) {
        lifecycleScope.launch(Dispatchers.IO) {

            val stationList = SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRoute(
                item.projectId,
                item.getId()
            )

            var hasData = false
            for (stationItem in stationList) {
                val recordCount =
                    SurveyorDatabaseFactory.instance.surveyorRecordDataSource.recordCount(
                        stationItem.getId()
                    )
                if (recordCount > 0) {
                    hasData = true
                    break
                }
            }

            if (hasData) {
                withContext(Dispatchers.Main) {
                    TipsFragment.cancelAbleTips(
                        childFragmentManager, viewLifecycleOwner, context,
                        getString(R.string.tips_is_sure_to_delete_route_with_record),
                        4
                    ) {
                        deleteDatabase(item)
                    }
                }
            } else {
                deleteDatabase(item)
            }
        }
    }

    private fun deleteDatabase(routeRecord: RouteRecord) {
        lifecycleScope.launch(Dispatchers.IO) {
            SurveyorDataBase.getInstance().withTransaction {
                val stationList =
                    SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRoute(
                        routeRecord.projectId,
                        routeRecord.getId()
                    )

                //删除路线
                SurveyorDatabaseFactory.instance.getRouteDataSource().delete(routeRecord)
                //删除测站
                SurveyorDatabaseFactory.instance.surveyorStation.deleteByRouteId(routeRecord.getId())

                if (PointUtil.isCanDeleteHeightTraverseAutoCreatePoint(routeRecord.startPointId)) {
                    SurveyorDatabaseFactory.instance.pointDataSource.deleteById(routeRecord.startPointId)
                }

                if (PointUtil.isCanDeleteHeightTraverseAutoCreatePoint(routeRecord.endPointId)) {
                    SurveyorDatabaseFactory.instance.pointDataSource.deleteById(routeRecord.endPointId)
                }

                for (stationItem in stationList) {
                    if (stationItem.items != null) {
                        for (surveyorPoint in stationItem.items) {
                            if (PointUtil.isCanDeleteHeightTraverseAutoCreatePoint(surveyorPoint.originalPointId)) {
                                //删除点
                                SurveyorDatabaseFactory.instance.pointDataSource.deleteById(
                                    surveyorPoint.originalPointId
                                )
                            }

                        }
                    }
                }

            }
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentListLayoutBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        mBinding.list.adapter = adapter

        getProjectId()?.let { it ->
            SurveyorDatabaseFactory.instance.getRouteDataSource().getListLiveData(it)
                .observe(viewLifecycleOwner) {
                    adapter.updateData(it)
                }
        }
    }


    fun getProjectId() = arguments?.getString(BundleConstants.KEY_PROJECT_ID)


    companion object {
        fun newInstance(projectId: String): RouteFragment {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId)
            val routeFragment = RouteFragment()
            routeFragment.arguments = bundle
            return routeFragment
        }
    }

    /**
     * 获取即将要测量的路线id
     */
    fun getSurveyorRouteId(): String? {
        if (dataList.isNotEmpty()) {
            return dataList.last().getId()
        }
        return null
    }


}