package com.bingce.controlnetwork.fragment.projectedit

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.EveryWay
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.HeighTraverseConfigRecord
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.SeparateWay
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.StationLayout
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentProjectEditHeightTraverseBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 高程导线新建编辑
 */
class ProjectEditHeightTraverseFragment : AbstractProjectEditFragment() {

    private val stationLayoutList = listOf(
        "隔点设站，设站一次",
        "隔点设站，设站两次",
        "每点设站，单向观测",
        "每点设站，对向观测",
    )

    companion object {
//        const val POSITION_SEPARATION = 0//隔点设站的index

        const val POSITION_SEPARATE_SET_UP_ONCE = 0
        const val POSITION_SEPARATE_SET_UP_TWICE = 1
        const val POSITION_EVERY_SINGLE_MEASURE = 2
        const val POSITION_EVERY_OPPOSITE_MEASURE = 3
    }

    init {
        initSelectToleranceLauncher(this)
    }

    private var _binding: FragmentProjectEditHeightTraverseBinding? = null
    private val mBinding get() = _binding!!

    override fun createContentView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentProjectEditHeightTraverseBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun initView() {
        super.initView()

        mBinding.spinner.setAdapter(
            ArrayAdapter(
                requireContext(),
                R.layout.item_spinner_text,
                stationLayoutList
            )
        )
//        mBinding.dtlStationMode.observeIndexChange { fromIndex, toIndex, reselect, fromUser ->
//            if (toIndex == POSITION_SEPARATION) {
//                //隔点设站
//                mBinding.everyMeasureModeParent.visibility = View.GONE
//                mBinding.separationMeasureModeParent.visibility = View.VISIBLE
//            } else {
//                //每点设站
//                mBinding.everyMeasureModeParent.visibility = View.VISIBLE
//                mBinding.separationMeasureModeParent.visibility = View.GONE
//            }
//        }
    }

    override fun initData() {
        super.initData()
        getProjectId()?.let {
            scope.launch {
                setEditClickEnable(it)
                setStationLayoutData(it)
            }
        }
    }

    /**
     *设置布局方式数据
     */
    private suspend fun setStationLayoutData(projectId: String) {
        val record = withContext(Dispatchers.IO) {
            getRecord(projectId)
        }
        record?.let {
            val stationLayoutSpinnerPos = if (it.stationLayout == StationLayout.SEPARATE) {
                if (it.separateWay == SeparateWay.SET_UP_ONCE) {
                    POSITION_SEPARATE_SET_UP_ONCE
                } else {
                    POSITION_SEPARATE_SET_UP_TWICE
                }
            } else {
                if (it.everyWay == EveryWay.SINGLE_MEASURE) {
                    POSITION_EVERY_SINGLE_MEASURE
                } else {
                    POSITION_EVERY_OPPOSITE_MEASURE
                }
            }
            mBinding.spinner.setSelection(stationLayoutSpinnerPos)
//            mBinding.dtlStationMode.onPageSelected(it.stationLayout.ordinal)
//            it.separateWay?.ordinal?.let {
//                mBinding.dtlSeparationMeasureMode.onPageSelected(it)
//            }
//            it.everyWay?.ordinal?.let {
//                mBinding.dtlEveryMeasureMode.onPageSelected(it)
//            }
        }
    }

    /**
     * 当项目中新建测站了就不能编辑了布置方式了
     */
    private suspend fun setEditClickEnable(projectId: String) {
        withContext(Dispatchers.IO) {
            val countByProjectId =
                SurveyorDatabaseFactory.instance.surveyorStation.getCountByProjectId(projectId)

            if (countByProjectId != 0) {
//                mBinding.dtlStationMode.itemEnableSelector = false
//                mBinding.dtlSeparationMeasureMode.itemEnableSelector = false
//                mBinding.dtlEveryMeasureMode.itemEnableSelector = false
                mBinding.spinner.isEnabled = false
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }


    override fun getCurrentProjectName(): String {
        return mBinding.il.editor.text.toString().trim()
    }

    override fun setToleranceName(name: String) {
        mBinding.il.selectTolerance.text = name
    }

    override fun setProjectName(name: String) {
        mBinding.il.editor.setText(name)
    }

    override fun setShowSlopSwitch(isChecked: Boolean) {
        mBinding.il.showSlopSwitch.isChecked = isChecked
    }

    override fun getToleranceBtn(): View {
        return mBinding.il.selectTolerance
    }

    override suspend fun saveOtherRecord(projectId: String) {
        var record = getRecord(projectId)
        if (record == null) {
            record = HeighTraverseConfigRecord.newInstance(
                projectId, getStationLayout(),
                getSeparateWay(),
                getEveryWay()
            )
        } else {
            record.stationLayout = getStationLayout()
            record.separateWay = getSeparateWay()
            record.everyWay = getEveryWay()
        }
        SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource().saveRecord(record)
    }

    private suspend fun getRecord(projectId: String): HeighTraverseConfigRecord? {
        val heightTraverseConfigDataSource =
            SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource()
        return heightTraverseConfigDataSource.getRecordByProjectId(projectId)
    }

//    private fun getStationLayout() = when (mBinding.dtlStationMode.currentItemIndex) {
//        POSITION_SEPARATION -> {
//            StationLayout.SEPARATE
//        }
//
//        else -> StationLayout.EVERY
//    }
//
//    private fun getSeparateWay() = getSeparateOrEveryWay(mBinding.dtlSeparationMeasureMode) {
//        SeparateWay.values()[it]
//    }
//
//    private fun getEveryWay() = getSeparateOrEveryWay(mBinding.dtlEveryMeasureMode) {
//        EveryWay.values()[it]
//    }

    private fun getStationLayout() = when (mBinding.spinner.selectedItemPosition) {
        POSITION_SEPARATE_SET_UP_ONCE, POSITION_SEPARATE_SET_UP_TWICE -> {
            StationLayout.SEPARATE
        }

        else -> StationLayout.EVERY
    }

    private fun getSeparateWay() = when (mBinding.spinner.selectedItemPosition) {
        POSITION_SEPARATE_SET_UP_ONCE -> {
            SeparateWay.SET_UP_ONCE
        }

        POSITION_SEPARATE_SET_UP_TWICE -> {
            SeparateWay.SET_UP_TWICE
        }

        else -> null
    }

    private fun getEveryWay() = when (mBinding.spinner.selectedItemPosition) {
        POSITION_EVERY_SINGLE_MEASURE -> {
            EveryWay.SINGLE_MEASURE
        }

        POSITION_EVERY_OPPOSITE_MEASURE -> {
            EveryWay.OPPOSITE_MEASURE
        }

        else -> null
    }


    override fun getShowSlopSwitch(): Boolean {
        return mBinding.il.showSlopSwitch.isChecked
    }

}