package com.bingce.controlnetwork.fragment.station;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.SwitchCompat;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.IOnBackFilter;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.newui.widget.switchtext.SwitchWithText;

import org.jetbrains.annotations.NotNull;

import java.util.List;

public abstract class AbstractStationFragment extends BaseFragment implements IOnBackFilter {

    /**
     * 错误提示-已经存在重复测期测站
     *
     * @param duplicateStationName
     */
    protected void tipsError(String duplicateStationName) {
        runOnUI(() -> {
            if (getContext() == null) {
                return;
            }
            TipsFragment.tipsOnly(
                    getChildFragmentManager(),
                    getViewLifecycleOwner(),
                    getContext(),
                    duplicateStationError(duplicateStationName));
        });
    }

    protected String duplicateStationError(String duplicateStationName) {
        return Tools.getString(R.string.already_exist_station_period, duplicateStationName);
    }

    protected abstract int layoutRes();

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(layoutRes(), container, false);
    }

    protected static boolean warningDataLosing(@NonNull Fragment fragment) {
        TipsWith2OptionFragment
                .tips(
                        fragment.getChildFragmentManager(),
                        fragment.getViewLifecycleOwner(),
                        fragment.getString(R.string.exit),
                        fragment.getString(R.string.cancel),
                        fragment.getString(R.string.tips_station_data_will_lose),
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                if (fragment.getActivity() != null) {
                                    fragment.getActivity().finish();
                                }
                            }
                        }
                );
        return true;
    }

    protected static boolean switchIsChecked(View view, int id) {
        if (view == null) {
            return false;
        }
        View view1 = view.findViewById(id);
        if (view1 != null) {
            if (view1 instanceof SwitchWithText) {
                SwitchWithText switchCompat = (SwitchWithText) view1;
                return switchCompat.isChecked();
            }
            if (view1 instanceof SwitchCompat) {
                SwitchCompat switchCompat = (SwitchCompat) view1;
                return switchCompat.isChecked();
            }
        }
        return false;
    }

    protected boolean needLeftRightSurveyor() {
        return switchIsChecked(getView(), R.id.left_right_switch);
    }

    protected void updateLeftRightMark(View view, boolean need) {
        if (view == null) {
            return;
        }
        SwitchWithText switchCompat = view.findViewById(R.id.left_right_switch);
        if (switchCompat != null) {
            switchCompat.setChecked(need);
        }
    }

    protected static boolean isListEmpty(List<?> stationItems) {
        return stationItems != null && !stationItems.isEmpty();
    }
}
