package com.bingce.controlnetwork.fragment.station.defaultstation;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.stationdetail.BaseStationDetailAdapter;
import com.bingce.controlnetwork.adapter.stationdetail.EditableStationDetailAdapter;
import com.bingce.controlnetwork.adapter.stationdetail.NoEditableStationDetailAdapter;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.fragment.station.AbstractStationFragment;
import com.bingce.controlnetwork.ui.widget.recyclerview.SwipeDraggableRecyclerViewUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import blankj.utilcode.util.ToastUtils;

public abstract class AbstractSingleStationPointListFragment extends AbstractStationFragment
        implements BaseStationDetailAdapter.IOnSelectPointDelegate,
        SwipeDraggableRecyclerViewUtil.IStationPointOperatorListener {
    private static final String TYPE_SURVEYOR_STATION = "___station_points";
    private static final String TYPE_SURVEYOR_POINT = "___surveyor_points";
    private BaseStationDetailAdapter defaultStationDetailAdapter;
    private final ActivityResultLauncher<Intent> defaultStationLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
            return;
        }
        if (TYPE_SURVEYOR_STATION.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().get(0) == null ||
                    pickPointResult.getSelectedNames().get(0) == null) {
                showStationPointError();
                return;
            }
            defaultStationDetailAdapter.setStationPoint(
                    pickPointResult.getSelectedIds().get(0),
                    pickPointResult.getSelectedNames().get(0));
        } else if (TYPE_SURVEYOR_POINT.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedNames().size() ||
                    pickPointResult.getSelectedIds().size() < 2) {
                showPointCountError();
                return;
            }
            defaultStationDetailAdapter.setSurveyorPoint(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        }
        onItemChanged();
    });

    protected abstract void prepareAdapterData(IDataCallback callback);

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        if (view == null) {
            return null;
        }
        RecyclerView recyclerView = view.findViewById(R.id.recycler_view);
        if (recyclerView != null) {
            prepareAdapterData((editAble, items) -> {
                if (getContext() == null) {
                    return;
                }
                if (editAble) {
                    defaultStationDetailAdapter = new EditableStationDetailAdapter(
                            AbstractSingleStationPointListFragment.this,
                            items);
                } else {
                    defaultStationDetailAdapter = new NoEditableStationDetailAdapter(items);
                }
                runOnUI(() -> {
                    recyclerView.setAdapter(defaultStationDetailAdapter);
                    SwipeDraggableRecyclerViewUtil.setup(defaultStationDetailAdapter,
                            recyclerView,
                            getChildFragmentManager(),
                            getViewLifecycleOwner(),
                            this,
                            getContext());
                });
            });
        }
        setHasOptionsMenu(true);
        return view;
    }

    protected final List<SurveyorPoint> collectDataItems() {
        if (defaultStationDetailAdapter == null) {
            return null;
        }
        return defaultStationDetailAdapter.collectData();
    }

    /**
     * 将数据库存储的测站点列表和显示的顺序保持一致
     *
     * @param surveyorStationRecord
     */
    protected final void sortStationPoint(SurveyorStationRecord surveyorStationRecord) {
        List<SurveyorPoint> currentItems = collectDataItems();
        if (currentItems == null) {
            return;
        }
        //对现有数据进行排序
        Map<String, SurveyorPoint> map = new HashMap<>();
        for (SurveyorPoint item : surveyorStationRecord.getItems()) {
            map.put(item.getPointId(), item);
        }
        for (int index = 0; index < currentItems.size(); index++) {
            SurveyorPoint item = currentItems.get(index);
            if (item == null) {
                continue;
            }
            SurveyorPoint find = map.get(item.getPointId());
            if (find == null) {
                continue;
            }
            currentItems.set(index, find);
        }
        surveyorStationRecord.setItems(currentItems);
    }

    @Override
    public void onSelectStationPoint(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_SURVEYOR_STATION,
                1,
                getContext().getString(R.string.select_station_point),
                filters,
                projectId, pickPointResult, defaultStationLauncher);
    }

    @Override
    public void onSelectSurveyorPoint(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_SURVEYOR_POINT,
                -1,
                getContext().getString(R.string.select_surveyor_point),
                filters,
                projectId, pickPointResult, defaultStationLauncher);
    }

    @Override
    public void onItemRemoved(int adapterPosition) {
        onItemChanged();
    }

    @Override
    public void onItemDragCompleted() {
        onItemChanged();
    }

    /**
     * 测站数据发生了变化--测点被删除、测点被修改、测站被修改、测点被拖拽
     */
    protected abstract void onItemChanged();

    protected int surveyorPointSize() {
        return defaultStationDetailAdapter.surveyorPointSize();
    }

    private void showPointCountError() {
        ToastUtils.showShort(R.string.tips_surveyor_point_must_more_than_2);
    }

    private void showStationPointError() {
        ToastUtils.showShort(R.string.tips_please_select_station_point);
    }

    public interface IDataCallback {
        void onGetAdapterData(boolean editAble, List<StationDetailItem> items);
    }
}
