package com.bingce.controlnetwork.fragment.station.defaultstation;

import android.os.Bundle;

import androidx.annotation.WorkerThread;
import androidx.lifecycle.ViewModelProvider;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.IPointDataSource;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.stationdetail.StationDetailViewModel;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.List;

public class ComponentSingleStationPointListFragment extends AbstractSingleStationPointListFragment {

    @Override
    protected int layoutRes() {
        return R.layout.fragment_surveyor_station_point_list_layout;
    }

    @Override
    protected void prepareAdapterData(IDataCallback callback) {
        if (getArguments() == null) {
            return;
        }
        String stationId = getArguments().getString(KEY_STATION_ID);
        IPointDataSource pointDataSource = SurveyorDatabaseFactory
                .instance
                .getPointDataSource();
        ThreadPoolUtil.execute(() -> {
            if (getContext() == null) {
                return;
            }
            //1.读取测站数据
            SurveyorStationRecord surveyorStationRecord =
                    SurveyorDatabaseFactory.instance.getSurveyorStation().getRecordSync(stationId);
            List<SurveyorPoint> items = surveyorStationRecord.getItems();
            //2.提取测站点和测点
            List<String> pointIdsWithoutStation = new ArrayList<>();
            List<String> pointNamesWithoutStation = new ArrayList<>();
            String stationPointId = null;
            String stationPointName = null;
            for (SurveyorPoint item : items) {
                if (item == null) {
                    continue;
                }
                if (SurveyorPoint.TYPE_STATION.equals(item.getType())) {
                    stationPointId = item.getPointId();
                    PointRecord pointRecord = pointDataSource.findByIdSync(stationPointId);
                    if (pointRecord != null) {
                        stationPointName = pointRecord.getName();
                    }
                } else if (SurveyorPoint.TYPE_POINT.equals(item.getType())) {
                    pointIdsWithoutStation.add(item.getPointId());
                    PointRecord pointRecord = pointDataSource.findByIdSync(item.getPointId());
                    if (pointRecord != null) {
                        pointNamesWithoutStation.add(pointRecord.getName());
                    }
                }
            }
            //3.为adapter创建数据源
            List<StationDetailItem> itemList = new ArrayList<>();
            //添加测站title item
            itemList.add(
                    new StationDetailItem(
                            getContext().getString(R.string.select_station_point),
                            StationDetailItem.ITEM_TYPE_STATION_GROUP,
                            null));
            if (stationPointId != null && stationPointName != null) {
                //添加测站点
                StationDetailItem item =
                        new StationDetailItem(
                                stationPointName,
                                StationDetailItem.ITEM_TYPE_STATION_ITEM,
                                stationPointId);
                itemList.add(item);
            }
            //添加测点title item
            itemList.add(
                    new StationDetailItem(
                            getContext().getString(R.string.select_surveyor_point),
                            StationDetailItem.ITEM_TYPE_POINT_GROUP,
                            null));
            for (int index = 0; index < pointIdsWithoutStation.size(); index++) {
                String id = pointIdsWithoutStation.get(index);
                String name = pointNamesWithoutStation.get(index);
                if (id == null || name == null) {
                    continue;
                }
                //添加测点
                StationDetailItem item =
                        new StationDetailItem(
                                name,
                                StationDetailItem.ITEM_TYPE_POINT_ITEM,
                                id);
                itemList.add(item);
            }
            //4.回调
            callback.onGetAdapterData(isCanEditSurveyorPoint(surveyorStationRecord), itemList);

            runOnUI(() -> updateLeftRightMark(getView(), surveyorStationRecord.leftAndRightSurveyor));
        });
    }

    @Override
    protected void onItemChanged() {
        if (getArguments() == null) {
            return;
        }
        String stationId = getArguments().getString(KEY_STATION_ID);
        ThreadPoolUtil.execute(() -> {
            if (getContext() == null) {
                return;
            }
            ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory.instance.getSurveyorStation();
            //1.读取测站数据
            SurveyorStationRecord currentRecord = stationDataSource.getRecordSync(stationId);
            if (currentRecord == null) {
                return;
            }
            //2.排序测站列表
            sortStationPoint(currentRecord);
            //3.存储测站record
            stationDataSource.saveSync(currentRecord);
            //4.更新测点数量
            runOnUI(() -> {
                if (getActivity() == null) {
                    return;
                }
            });
        });
    }

    @Override
    public boolean cancelBack() {
        return false;
    }

    private static final String KEY_STATION_ID = "__station_id";
    public static final String TAG = ComponentSingleStationPointListFragment.class.getName();

    public static ComponentSingleStationPointListFragment newInstance(String stationId, String projectId) {
        ComponentSingleStationPointListFragment fragment = new ComponentSingleStationPointListFragment();
        Bundle bundle = new Bundle();
        bundle.putString(KEY_STATION_ID, stationId);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        fragment.setArguments(bundle);
        return fragment;
    }

    @WorkerThread
    private boolean isCanEditSurveyorPoint(SurveyorStationRecord stationRecord) {
        ProjectRecord projectRecord = SurveyorDatabaseFactory.instance.getProjectDataSource().getRecord(stationRecord.getProjectId());
        if (projectRecord.getProjectType() == null || projectRecord.getProjectType().equals(ProjectType.TRIANGLE.getFlag())) {
            if (!SurveyorStationRecord.isLineStation(stationRecord.getLineId())) {
                long surveyorRecordCount =
                        SurveyorDatabaseFactory
                                .instance
                                .getSurveyorRecordDataSource()
                                .recordCount(stationRecord.getId());
                return surveyorRecordCount <= 0;
            }
        }
        return false;
    }
}
