package com.bingce.controlnetwork.fragment.station.defaultstation;

import android.content.Context;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.IPointDataSource;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.newui.widget.switchtext.SwitchWithText;
import com.bingce.utils.ThreadPoolUtil;

import java.util.Arrays;
import java.util.List;

public class DefaultStationSetupFragment extends AbstractSingleStationPointListFragment {
    public static final String TAG = DefaultStationSetupFragment.class.getName();
    private MenuItem confirmItem;

    private boolean lastConfirmEnable = false;

    @Override
    protected int layoutRes() {
        return R.layout.fragment_default_surveyor_station_setup_layout;
    }

    @Override
    protected void prepareAdapterData(IDataCallback callback) {
        if (callback == null || getContext() == null) {
            return;
        }
        List<StationDetailItem> itemList = Arrays.asList(
                new StationDetailItem(
                        getContext().getString(R.string.select_station_point),
                        StationDetailItem.ITEM_TYPE_STATION_GROUP,
                        null),
                new StationDetailItem(
                        getContext().getString(R.string.select_surveyor_point),
                        StationDetailItem.ITEM_TYPE_POINT_GROUP,
                        null));
        callback.onGetAdapterData(true, itemList);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        confirmItem = menu.findItem(R.id.menu_confirm);
        if (confirmItem != null) {
            confirmItem.setEnabled(lastConfirmEnable);
        }
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (R.id.menu_confirm == item.getItemId()) {
            //收集信息，并创建站点记录
            List<SurveyorPoint> stationItems = collectDataItems();
            if (stationItems == null || getArguments() == null) {
                return super.onOptionsItemSelected(item);
            }
            String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            String periodId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
            ThreadPoolUtil.execute(() -> {
                //找到站的点id
                String stationPointId = "";
                for (SurveyorPoint stationItem : stationItems) {
                    if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                        stationPointId = stationItem.getPointId();
                        break;
                    }
                }
                String name = checkValidate(stationPointId, projectId, periodId);
                if (name != null && !name.isEmpty()) {
                    tipsError(name);
                    return;
                }
                //先对stationItems点作处理
                //处理方式就是，如果这里面有点是global_control，将其拷贝到project known_point中
                //并使用拷贝的pointId替换stationItems里面的id
                PointUtil.globalToProjectControlPoint(stationItems, projectId);

                IPointDataSource pointDataSource =
                        SurveyorDatabaseFactory.instance.getPointDataSource();
                PointRecord pointRecord = pointDataSource.findByIdSync(stationPointId);
                if (pointRecord == null) {
                    return;
                }
                ProjectRecord projectRecord = SurveyorDatabaseFactory
                        .instance
                        .getProjectDataSource()
                        .getRecord(projectId);
                boolean showSlopDistance = true;
                if (projectRecord != null) {
                    showSlopDistance = projectRecord.isShowSlopDistance();
                }
                SurveyorStationRecord stationRecord = SurveyorStationRecord.mulAngle(projectId, periodId);
                stationRecord.setItems(stationItems);
                if (stationItems.size() > 4) {
                    stationRecord.setAutoClose(true);
                }
                //检测左右角
                stationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor());

                stationRecord.setShowSlopDistance(showSlopDistance);

                SurveyorDatabaseFactory
                        .instance
                        .getSurveyorStation()
                        .saveSync(stationRecord);


                runOnUI(() -> {
                    if (getActivity() != null) {
                        getActivity().finish();
                    }
                });
            });
        }
        return super.onOptionsItemSelected(item);
    }

    protected static String checkValidate(String stationPointId, String projectId, String periodId) {
        //检测该测期下是否有同名测站
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                .instance
                .getSurveyorStation();
        List<SurveyorStationRecord> stationRecordList =
                stationDataSource.getDataList(projectId, periodId);
        if (stationRecordList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : stationRecordList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            if (stationItem.getPointId().equals(stationPointId)) {
                                PointRecord pointRecord = SurveyorDatabaseFactory
                                        .instance
                                        .getPointDataSource()
                                        .findByIdSync(stationItem.getOriginalPointId());
                                if (pointRecord != null) {
                                    return pointRecord.getName();
                                }
                                return "";
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    protected boolean isCurrentDataValidate() {
        List<SurveyorPoint> stationItems = collectDataItems();
        if (stationItems == null || stationItems.size() < 3) {
            return false;
        } else {
            //检测是否有station item
            for (SurveyorPoint item : stationItems) {
                if (item == null) {
                    continue;
                }
                if (SurveyorPoint.TYPE_STATION.equals(item.getType())) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    protected void onItemChanged() {
        if (confirmItem != null) {//设置menuItem enable
            boolean confirmEnable = isCurrentDataValidate();
            confirmItem.setEnabled(confirmEnable);
            lastConfirmEnable = confirmEnable;
        }
        //检测左右角是否
        SwitchWithText leftAndRightSwitch = findViewById(R.id.left_right_switch);
        if (leftAndRightSwitch != null) {
            int pointSize = surveyorPointSize();
            if (pointSize == 2) {
                leftAndRightSwitch.setVisibility(View.VISIBLE);
            } else {
                leftAndRightSwitch.setChecked(false);
                leftAndRightSwitch.setVisibility(View.GONE);
            }
        }
    }

    @Override
    public boolean cancelBack() {
        List<SurveyorPoint> items = collectDataItems();
        if (isListEmpty(items)) {
            return warningDataLosing(this);
        }
        return false;
    }

    public static final int title = R.string.new_surveyor_station;

    public static void start(Context context, String projectId, String period) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, period);
        FragmentContainerActivity.start(context, TAG, title, bundle);
    }

    public static DefaultStationSetupFragment newInstance(Bundle bundle) {
        DefaultStationSetupFragment fragment = new DefaultStationSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
