package com.bingce.controlnetwork.fragment.station.heighttraverse

import android.os.Bundle
import android.view.View
import androidx.annotation.WorkerThread
import androidx.fragment.app.Fragment
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.point.PointRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlapphelper.util.RegExUtil
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.utils.DateUtils
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import java.util.Date

abstract class AbstructSetupStationHeightTraverseFragment : Fragment() {

    val prefixStation: String = Tools.getString(R.string.transfer)
    val prefixJianXie: String = Tools.getString(R.string.break_point)
    val prefixTransfer: String = Tools.getString(R.string.turn_point)

    private val mainScope = MainScope()

    abstract fun doConfirm()
    abstract fun isCheckedJianXiePoint(): Boolean
    abstract fun isCheckedEndPoint(): Boolean
    abstract fun getFrontPointHeight(): String

    abstract suspend fun getBackPoint(): PointRecord
    abstract suspend fun getFrontPoint(): PointRecord
    abstract suspend fun getStationPoint(): PointRecord?
    abstract fun initView()
    abstract fun initListener()
    abstract fun initData()

    fun launch(block: suspend CoroutineScope.() -> Unit) {
        mainScope.launch(Dispatchers.IO) {
            block.invoke(this)
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
        initListener()
        initData()
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mainScope.cancel()
    }

    fun getProjectId(): String {
        val createStationForHeightTraverseFragment =
            parentFragment as CreateStationForHeightTraverseFragment
        return createStationForHeightTraverseFragment.getProjectId()
    }

    fun getRouteId(): String {
        val createStationForHeightTraverseFragment =
            parentFragment as CreateStationForHeightTraverseFragment
        return createStationForHeightTraverseFragment.getRouteId()
    }

    fun createStationRecordNoSurveyorPoints(
        projectId: String,
        projectType: String,
        relationStationId: String?
    ): SurveyorStationRecord {
        val stationRecord = SurveyorStationRecord()
        stationRecord.id = DateUtils.genRandomTimeId()
        stationRecord.projectId = projectId
        stationRecord.projectType = projectType
        stationRecord.showSlopDistance = true
        stationRecord.createDate = Date()
        stationRecord.routeId = getRouteId()
        stationRecord.items = mutableListOf()
        stationRecord.relationStationId = relationStationId
        return stationRecord
    }

    protected fun finish() {
        requireActivity().finish()
    }

    @WorkerThread
    protected fun savePoint(
        pointName: String,
        pointTypeHeightTraverse: String?,
        projectId: String,
        height: String?
    ): String {
//        var pointRecord =
//            SurveyorDatabaseFactory.instance.pointDataSource.findByNameSync(
//                pointName,
//                projectId
//            )
//        if (pointRecord == null) {
//            pointRecord = PointRecord.point(
//                getProjectId(),
//                PointRecord.TYPE_UNKNOWN_POINT,
//                pointName,
//                "",
//                0.0,
//                0.0,
//                stringToDouble(height),
//                "",
//                -1
//            )
//            pointRecord.pointTypeHeightTraverse = pointTypeHeightTraverse
//            SurveyorDatabaseFactory.instance.pointDataSource.saveSync(pointRecord)
//        } else {
//            pointRecord.setZ(stringToDouble(height))
//            SurveyorDatabaseFactory.instance.pointDataSource.update(pointRecord)
//        }

        val pointRecord = PointRecord.point(
            projectId,
            PointRecord.TYPE_UNKNOWN_POINT,
            pointName,
            "",
            0.0,
            0.0,
            stringToDouble(height),
            "",
            -1
        )
        pointRecord.pointTypeHeightTraverse = pointTypeHeightTraverse
        SurveyorDatabaseFactory.instance.pointDataSource.saveSync(pointRecord)

        return pointRecord.getId()
    }

    @WorkerThread
    protected fun saveFrontPoint(projectId: String, frontName: String): String {
        var frontPointTypeHeightTraverse: String? = null
        if (isCheckedJianXiePoint()) {
            frontPointTypeHeightTraverse =
                PointRecord.TYPE_HEIGHT_TRAVERSE_INTERVAL
        } else if (isCheckedEndPoint()) {
            //终点
        } else {
            frontPointTypeHeightTraverse =
                PointRecord.TYPE_HEIGHT_TRAVERSE_TRANSFER
        }

        val frontHeight = getFrontPointHeight()

        return savePoint(frontName, frontPointTypeHeightTraverse, projectId, frontHeight)
    }

    fun doubleToString(value: Double) = value.toString()
    fun stringToDouble(value: String?): Double {
        if (value.isNullOrEmpty()) {
            return 0.0
        }
        return value.toDouble()
    }

    @WorkerThread
    protected fun getNewIndexFromTwoPointId(pointId1: String, pointId2: String): PointRecord {
        val point1 =
            SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(pointId1)
        val point2 =
            SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(pointId2)
        val index1 = RegExUtil.getIndexFromName(point1.getName())
        val index2 = RegExUtil.getIndexFromName(point2.getName())
        if (index1 >= index2) {
            return point1
        }
        return point2
    }


}