package com.bingce.controlnetwork.fragment.station.heighttraverse

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.EveryWay
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.SeparateWay
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.StationLayout
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity
import com.bingce.controlnetwork.databinding.FragmentCreateStationForHeightTraverseBinding
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch

/**
 * 高程导线 新建测站
 */
class CreateStationForHeightTraverseFragment : Fragment() {

    private lateinit var confirmItem: MenuItem

    private var _binding: FragmentCreateStationForHeightTraverseBinding? = null
    val mBinding get() = _binding!!
    private val mainScope = MainScope()

    private var currentFragment: AbstructSetupStationHeightTraverseFragment? = null


    companion object {
        @JvmField
        val TAG: String = CreateStationForHeightTraverseFragment::class.java.name

        @JvmStatic
        fun newInstance(args: Bundle?): Fragment {
            val fragment: Fragment = CreateStationForHeightTraverseFragment()
            fragment.arguments = args
            return fragment
        }

        @JvmStatic
        fun start(context: Context, projectId: String?, routeId: String?) {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId)
            bundle.putString(BundleConstants.KEY_ROUTE_ID, routeId)
            FragmentContainerActivity.start(
                    context,
                    TAG,
                    R.string.new_surveyor_station,//新建测站
                    bundle
            )
        }

    }


    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        setHasOptionsMenu(true)
        _binding =
                FragmentCreateStationForHeightTraverseBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.menu_confirm, menu)
        confirmItem = menu.findItem(R.id.menu_confirm)
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.menu_confirm == item.itemId) {
            currentFragment?.doConfirm()
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        init()
    }

    private fun init() {
        mainScope.launch {
            val configRecord =
                    SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource()
                            .getRecordByProjectId(getProjectId())
            configRecord?.let {

                val stringBuilder = StringBuilder()
                stringBuilder.append(Tools.getString(R.string.note_current_station_is))

                if (it.stationLayout == StationLayout.SEPARATE) {

                    if (it.separateWay == SeparateWay.SET_UP_ONCE) {
                        stringBuilder.append(Tools.getString(R.string.set_up_station_is_seperate_once))

                        replaceFragment(
                                SeparatSetupStationFragment.newInstance(
                                        SeparatSetupStationFragment.TYPE_ONCE
                                )
                        )
                    } else if (it.separateWay == SeparateWay.SET_UP_TWICE) {
                        stringBuilder.append(Tools.getString(R.string.set_up_station_is_seperate_twice))

                        replaceFragment(
                                SeparatSetupStationFragment.newInstance(
                                        SeparatSetupStationFragment.TYPE_TWICE
                                )
                        )
                    }
                } else if (it.stationLayout == StationLayout.EVERY) {

                    if (it.everyWay == EveryWay.SINGLE_MEASURE) {
                        stringBuilder.append(Tools.getString(R.string.set_up_station_is_each_one_way))

                        replaceFragment(
                                EverySetUpStationFragment.newInstance(
                                        EverySetUpStationFragment.TYPE_ONE_WAY
                                )
                        )
                    } else if (it.everyWay == EveryWay.OPPOSITE_MEASURE) {
                        stringBuilder.append(Tools.getString(R.string.set_up_station_is_each_opposite))

                        replaceFragment(
                                EverySetUpStationFragment.newInstance(
                                        EverySetUpStationFragment.TYPE_TWO_WAY
                                )
                        )
                    }
                }

                setTvHint(stringBuilder.toString())
            }
        }


    }

    private fun setTvHint(text: String) {
        mBinding.tvHint.text = text
    }

    fun getProjectId(): String {
        return requireArguments().getString(BundleConstants.KEY_PROJECT_ID)!!
    }

    fun getRouteId(): String {
        return requireArguments().getString(BundleConstants.KEY_ROUTE_ID)!!
    }

    private fun replaceFragment(fragment: AbstructSetupStationHeightTraverseFragment) {
        currentFragment = fragment

        val beginTransaction = childFragmentManager.beginTransaction()
        beginTransaction.replace(R.id.fl_container, fragment)
        beginTransaction.commit()
    }

}