package com.bingce.controlnetwork.fragment.station.heighttraverse

import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.WorkerThread
import androidx.room.withTransaction
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.EveryWay
import com.bingce.controlapphelper.datasource.database.point.PointRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint
import com.bingce.controlapphelper.util.RegExUtil
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentHeightTraverseEverySetupStationBinding
import com.bingce.utils.DateUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

/**
 * 每点设站 创建测站
 */
class EverySetUpStationFragment : AbstructSetupStationHeightTraverseFragment() {

    private val mBinding get() = _binding!!

    private var _binding: FragmentHeightTraverseEverySetupStationBinding? = null

    private lateinit var backPoint: PointRecord
    private var isFirstStation = false

    companion object {
        private const val KEY_TYPE = "KEY_TYPE"
        const val TYPE_ONE_WAY = 1
        const val TYPE_TWO_WAY = 2

        @JvmStatic
        fun newInstance(type: Int): EverySetUpStationFragment {
            val fragment = EverySetUpStationFragment()
            val bundle = Bundle()
            bundle.putInt(KEY_TYPE, type)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding =
            FragmentHeightTraverseEverySetupStationBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun initView() {
        mBinding.ilBackTitle.tvTitle.text = Tools.getString(R.string.back_station)
        mBinding.ilFrontTitle.tvTitle.text = Tools.getString(R.string.front_station)
        //说明信息
        mBinding.ilDesc.tvDesc.text = getString(R.string.staton_setup_every_height_traverse_hint)
        //前视打开设置间歇点和终点
        mBinding.ilFrontTitle.llCb.visibility = View.VISIBLE
        //设置不可点击
        mBinding.ilBack.etPoint.isFocusable = false
        mBinding.ilBack.etHeight.isFocusable = false
        mBinding.ilFront.etPoint.isFocusable = false
    }

    override fun initListener() {
        var isClickCheckBox = false
        mBinding.ilFrontTitle.cbJianXie.setOnCheckedChangeListener { buttonView, isChecked ->
            if (isClickCheckBox) return@setOnCheckedChangeListener
            isClickCheckBox = true
            mBinding.ilFrontTitle.cbEnd.isChecked = false
            refreshFrontUi()
            isClickCheckBox = false
        }
        mBinding.ilFrontTitle.cbEnd.setOnCheckedChangeListener { buttonView, isChecked ->
            if (isClickCheckBox) return@setOnCheckedChangeListener
            isClickCheckBox = true
            mBinding.ilFrontTitle.cbJianXie.isChecked = false
            refreshFrontUi()
            isClickCheckBox = false
        }
    }

    override fun initData() {
        launch {
            backPoint = getBackPoint()
            val frontPoint = getFrontPoint()
            withContext(Dispatchers.Main) {
                setUi(backPoint, frontPoint)
            }
        }
    }

    private fun setUi(back: PointRecord, front: PointRecord) {
        setUiBack(back)
        setUiFront(front)
    }

    private fun setUiBack(pointRecord: PointRecord) {
        mBinding.ilBack.etPoint.setText(pointRecord.getName())
        mBinding.ilBack.etHeight.setText(doubleToString(pointRecord.getZ()))
    }

    private fun setUiFront(pointRecord: PointRecord) {
        mBinding.ilFront.etPoint.setText(pointRecord.getName())
        mBinding.ilFront.etHeight.setText(doubleToString(pointRecord.getZ()))
    }


    private fun refreshFrontUi() {
        launch {
            val frontPoint = getFrontPoint()
            withContext(Dispatchers.Main) {
                setUiFront(frontPoint)
            }
        }
    }

    override suspend fun getBackPoint(): PointRecord {
        val projectId = getProjectId()
        val routeId = getRouteId()

        val stationLatest =
            SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRouteLatest(
                projectId,
                routeId
            )

        if (stationLatest?.items != null && stationLatest.items.size >= 2) {
            val pointRecord: PointRecord =
                if (!TextUtils.isEmpty(stationLatest.relationStationId)) {
                    //对向观测
                    getNewIndexFromTwoPointId(
                        stationLatest.items[0].originalPointId,
                        stationLatest.items[1].originalPointId
                    )
                } else {
                    val pointId = stationLatest.items[1].originalPointId
                    SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(pointId)
                }
            isFirstStation = false
            return pointRecord
        }

        isFirstStation = true
        val routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getById(routeId)
        return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord?.startPointId)
    }

    override suspend fun getFrontPoint(): PointRecord {
        var frontIndexNew = 1
        if (!isFirstStation) {
            val index = RegExUtil.getIndexFromName(backPoint.getName())
            if (index != -1) {
                frontIndexNew = index + 1
            }
        }

//        val projectId = getProjectId()
        val routeId = getRouteId()

        val routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getById(routeId)

        if (mBinding.ilFrontTitle.cbEnd.isChecked) {
            return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord?.endPointId)
        } else if (mBinding.ilFrontTitle.cbJianXie.isChecked) {
//            val intervalPoint =
//                SurveyorDatabaseFactory.instance.pointDataSource.findByPointTypeForHeightTraverseLatest(
//                    projectId, PointRecord.TYPE_HEIGHT_TRAVERSE_INTERVAL
//                )
//            val intervalRecord = PointRecord()
//            if (intervalPoint == null) {
//                intervalRecord.name = "$prefixJianXie${1}"
//            } else {
//                val index = RegExUtil.getIndexFromName(intervalPoint.getName())
//                intervalRecord.name = "$prefixJianXie${index + 1}"
//            }
//            return intervalRecord
            val intervalRecord = PointRecord()
            intervalRecord.name = "$prefixJianXie${frontIndexNew}"
            return intervalRecord
        } else {
//            val transferPoint =
//                SurveyorDatabaseFactory.instance.pointDataSource.findByPointTypeForHeightTraverseLatest(
//                    projectId, PointRecord.TYPE_HEIGHT_TRAVERSE_TRANSFER
//                )
//            val transferRecord = PointRecord()
//            if (transferPoint == null) {
//                transferRecord.name = "$prefixTransfer${1}"
//            } else {
//                val index = RegExUtil.getIndexFromName(transferPoint.getName())
//                transferRecord.name = "$prefixTransfer${index + 1}"
//            }
//            return transferRecord

            val transferRecord = PointRecord()
            transferRecord.name = "$prefixTransfer${frontIndexNew}"
            return transferRecord
        }
    }

    override suspend fun getStationPoint(): PointRecord? {
        return null
    }


    override fun doConfirm() {
        val backName = mBinding.ilBack.etPoint.text.toString().trim()
        val frontName = mBinding.ilFront.etPoint.text.toString().trim()

        if (TextUtils.isEmpty(backName)) {
            ToastUtils.showShort("backName is empty")
            return
        }
        if (TextUtils.isEmpty(frontName)) {
            ToastUtils.showShort("frontName is empty")
            return
        }

        launch {
            val projectId = getProjectId()
            val projectRecord =
                SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId)

            val heightTraverseConfig =
                SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource()
                    .getRecordByProjectId(projectId)

            val isTwiceSetup = heightTraverseConfig?.everyWay == EveryWay.OPPOSITE_MEASURE

            SurveyorDataBase.getInstance().withTransaction {
                val frontSetupId = saveFrontPoint(projectId, frontName)

                val relationStationId = if (isTwiceSetup) DateUtils.genRandomTimeId() else null

                saveStationRecord(
                    projectId,
                    projectRecord.getProjectType(),
                    backPoint.getId(),
                    frontSetupId,
                    relationStationId
                )

                if (isTwiceSetup) {
                    saveStationRecord(
                        projectId,
                        projectRecord.getProjectType(),
                        frontSetupId,
                        backPoint.getId(),
                        relationStationId
                    )
                }
                withContext(Dispatchers.Main) {
                    finish()
                }
            }

        }

    }

    override fun isCheckedJianXiePoint(): Boolean {
        return mBinding.ilFrontTitle.cbJianXie.isChecked
    }

    override fun isCheckedEndPoint(): Boolean {
        return mBinding.ilFrontTitle.cbEnd.isChecked
    }

    override fun getFrontPointHeight(): String {
        return mBinding.ilFront.etHeight.text.toString().trim()
    }

    @WorkerThread
    private fun saveStationRecord(
        projectId: String,
        projectType: String,
        stationPointId: String,
        surveyorPointId: String,
        relationStationId: String?
    ) {
        val stationRecord =
            createStationRecordNoSurveyorPoints(projectId, projectType, relationStationId)
        stationRecord.items.add(
            SurveyorPoint.newInstance(
                stationPointId,
                SurveyorPoint.TYPE_STATION
            )
        )
        stationRecord.items.add(
            SurveyorPoint.newInstance(
                surveyorPointId,
                SurveyorPoint.TYPE_POINT
            )
        )
        SurveyorDatabaseFactory.instance.surveyorStation.saveSync(stationRecord)
    }


}