package com.bingce.controlnetwork.fragment.station.heighttraverse

import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.WorkerThread
import androidx.room.withTransaction
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.SeparateWay
import com.bingce.controlapphelper.datasource.database.point.PointRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint
import com.bingce.controlapphelper.util.RegExUtil
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentHeightTraverseSeperateSetupStationBinding
import com.bingce.utils.DateUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext

/**
 * 隔点设站 创建测站
 */
class SeparatSetupStationFragment : AbstructSetupStationHeightTraverseFragment() {
    private val mBinding get() = _binding!!
    private var _binding: FragmentHeightTraverseSeperateSetupStationBinding? = null

    private lateinit var backPoint: PointRecord
    private var isFirstStation = false

    companion object {
        private const val KEY_TYPE = "KEY_TYPE"
        const val TYPE_ONCE = 1
        const val TYPE_TWICE = 2

        @JvmStatic
        fun newInstance(type: Int): SeparatSetupStationFragment {
            val separatSetupStationFragment = SeparatSetupStationFragment()
            val bundle = Bundle()
            bundle.putInt(KEY_TYPE, type)
            separatSetupStationFragment.arguments = bundle
            return separatSetupStationFragment
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding =
            FragmentHeightTraverseSeperateSetupStationBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun initView() {
        mBinding.ilBackTitle.tvTitle.text = Tools.getString(R.string.rearview)
        mBinding.ilStationTitle.tvTitle.text = Tools.getString(R.string.measure_station)
        mBinding.ilFrontTitle.tvTitle.text = Tools.getString(R.string.forward_view)
        //说明信息
        mBinding.ilDesc.tvDesc.text = getString(R.string.staton_setup_separate_height_traverse_hint)
        //前视打开设置间歇点和终点
        mBinding.ilFrontTitle.llCb.visibility = View.VISIBLE
        //设置不可点击
        mBinding.ilBack.etPoint.isFocusable = false
        mBinding.ilBack.etHeight.isFocusable = false
        mBinding.ilStation.etPoint.isFocusable = false
        mBinding.ilFront.etPoint.isFocusable = false
    }

    override fun initListener() {
        var isClickCheckBox = false
        mBinding.ilFrontTitle.cbJianXie.setOnCheckedChangeListener { buttonView, isChecked ->
            if (isClickCheckBox) return@setOnCheckedChangeListener
            isClickCheckBox = true
            mBinding.ilFrontTitle.cbEnd.isChecked = false
            refreshFrontUi()
            isClickCheckBox = false
        }
        mBinding.ilFrontTitle.cbEnd.setOnCheckedChangeListener { buttonView, isChecked ->
            if (isClickCheckBox) return@setOnCheckedChangeListener
            isClickCheckBox = true
            mBinding.ilFrontTitle.cbJianXie.isChecked = false
            refreshFrontUi()
            isClickCheckBox = false
        }
    }

    override fun initData() {
        launch {
            backPoint = getBackPoint()
            val frontPoint = getFrontPoint()
            val stationPoint = getStationPoint()

            withContext(Dispatchers.Main) {
                setUi(backPoint, stationPoint, frontPoint)
            }
        }
    }

    private fun setUi(back: PointRecord, station: PointRecord, front: PointRecord) {
        setUiBack(back)
        setUiStation(station)
        setUiFront(front)
    }

    private fun setUiFront(pointRecord: PointRecord) {
        mBinding.ilFront.etPoint.setText(pointRecord.getName())
        mBinding.ilFront.etHeight.setText(doubleToString(pointRecord.getZ()))
    }

    private fun setUiBack(pointRecord: PointRecord) {
        mBinding.ilBack.etPoint.setText(pointRecord.getName())
        mBinding.ilBack.etHeight.setText(doubleToString(pointRecord.getZ()))
    }

    private fun setUiStation(pointRecord: PointRecord) {
        mBinding.ilStation.etPoint.setText(pointRecord.getName())
        mBinding.ilStation.etHeight.setText(doubleToString(pointRecord.getZ()))
    }


    private fun refreshFrontUi() {
        launch {
            val frontPoint = getFrontPoint()
            withContext(Dispatchers.Main) {
                setUiFront(frontPoint)
            }
        }
    }

    override fun doConfirm() {

        val backName = mBinding.ilBack.etPoint.text.toString().trim()
        val stationName = mBinding.ilStation.etPoint.text.toString().trim()
        val frontName = mBinding.ilFront.etPoint.text.toString().trim()

        if (TextUtils.isEmpty(backName)) {
            ToastUtils.showShort("backName is empty")
            return
        }
        if (TextUtils.isEmpty(stationName)) {
            ToastUtils.showShort("stationName is empty")
            return
        }
        if (TextUtils.isEmpty(frontName)) {
            ToastUtils.showShort("frontName is empty")
            return
        }

        launch {
            val projectId = getProjectId()
            val projectRecord =
                SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId)

            val configRecord =
                SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource()
                    .getRecordByProjectId(getProjectId())

            val isTwiceSetup = configRecord?.separateWay == SeparateWay.SET_UP_TWICE

            SurveyorDataBase.getInstance().withTransaction {

                val stationSetupIdList =
                    saveStationPoint(projectId, stationName, isTwiceSetup)

                val frontSetupId = saveFrontPoint(projectId, frontName)

                val relationStationId = if (isTwiceSetup) DateUtils.genRandomTimeId() else null

                for (stationPointId in stationSetupIdList) {
                    val stationRecord =
                        createStationRecordNoSurveyorPoints(
                            projectId,
                            projectRecord.getProjectType(),
                            relationStationId
                        )

                    stationRecord.items.add(
                        SurveyorPoint.newInstance(
                            stationPointId,
                            SurveyorPoint.TYPE_STATION
                        )
                    )
                    stationRecord.items.add(
                        SurveyorPoint.newInstance(
                            backPoint.getId(),
                            SurveyorPoint.TYPE_POINT
                        )
                    )
                    stationRecord.items.add(
                        SurveyorPoint.newInstance(
                            frontSetupId,
                            SurveyorPoint.TYPE_POINT
                        )
                    )
                    SurveyorDatabaseFactory.instance.surveyorStation.saveSync(stationRecord)
                }

                withContext(Dispatchers.Main) {
                    finish()
                }
            }

        }
    }

    override fun isCheckedJianXiePoint(): Boolean {
        return mBinding.ilFrontTitle.cbJianXie.isChecked
    }

    override fun isCheckedEndPoint(): Boolean {
        return mBinding.ilFrontTitle.cbEnd.isChecked
    }

    override fun getFrontPointHeight(): String {
        return mBinding.ilFront.etHeight.text.toString().trim()
    }

    override suspend fun getBackPoint(): PointRecord {
        val projectId = getProjectId()
        val routeId = getRouteId()

//        val stationLatest =
//            SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRouteLatest(
//                projectId,
//                routeId
//            )
//
//        if (stationLatest?.items != null && stationLatest.items.size >= 3) {
//            val surveyorPoint = stationLatest.items[2]
//            return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(surveyorPoint.originalPointId)
//        }

        val stationList =
            SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRouteLastTwo(
                projectId,
                routeId
            )
        val newestPointRecord = getNewestPointRecord(stationList, 2)
        if (newestPointRecord != null) {
            isFirstStation = false
            return newestPointRecord
        }

        isFirstStation = true
        val routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getById(routeId)
        return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord?.startPointId)
    }

    override suspend fun getFrontPoint(): PointRecord {
//        val projectId = getProjectId()
        val routeId = getRouteId()

        var frontIndexNew = 1
        if (!isFirstStation) {
            val index = RegExUtil.getIndexFromName(backPoint.getName())
            if (index != -1) {
                frontIndexNew = index + 1
            }
        }

        val routeRecord = SurveyorDatabaseFactory.instance.getRouteDataSource().getById(routeId)

        if (mBinding.ilFrontTitle.cbEnd.isChecked) {
            return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord?.endPointId)
        } else if (mBinding.ilFrontTitle.cbJianXie.isChecked) {
//            val intervalPoint =
//                SurveyorDatabaseFactory.instance.pointDataSource.findByPointTypeForHeightTraverseLatest(
//                    projectId, PointRecord.TYPE_HEIGHT_TRAVERSE_INTERVAL
//                )
//
//            val intervalRecord = PointRecord()
//            if (intervalPoint == null) {
//                intervalRecord.name = "$prefixJianXie${1}"
//            } else {
//                val index = RegExUtil.getIndexFromName(intervalPoint.getName())
//                intervalRecord.name = "$prefixJianXie${index + 1}"
//            }
//            return intervalRecord
            val intervalRecord = PointRecord()
            intervalRecord.name = "$prefixJianXie${frontIndexNew}"
            return intervalRecord
        } else {
//            val transferPoint =
//                SurveyorDatabaseFactory.instance.pointDataSource.findByPointTypeForHeightTraverseLatest(
//                    projectId, PointRecord.TYPE_HEIGHT_TRAVERSE_TRANSFER
//                )
//
//            val transferRecord = PointRecord()
//            if (transferPoint == null) {
//                transferRecord.name = "$prefixTransfer${1}"
//            } else {
//                val index = RegExUtil.getIndexFromName(transferPoint.getName())
//                transferRecord.name = "$prefixTransfer${index + 1}"
//            }
//            return transferRecord
            val transferRecord = PointRecord()
            transferRecord.name = "$prefixTransfer$frontIndexNew"
            return transferRecord
        }
    }

    override suspend fun getStationPoint(): PointRecord {
        val projectId = getProjectId()
        val routeId = getRouteId()
        val stationList =
            SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRouteLastTwo(
                projectId,
                routeId
            )

        var index = 1
        val newestPointRecord = getNewestPointRecord(stationList, 0)
        if (newestPointRecord != null) {
            val indexFromName = RegExUtil.getIndexFromName(newestPointRecord.name)
            if (indexFromName != -1) {
                index = indexFromName + 1
            }
        }
        val stationRecord = PointRecord()
        stationRecord.name = "$prefixStation${index}"
        return stationRecord
//        val stationPoint =
//            SurveyorDatabaseFactory.instance.pointDataSource.findByPointTypeForHeightTraverseLatest(
//                projectId, PointRecord.TYPE_HEIGHT_TRAVERSE_STATION_TRANSFER
//            )
//        val stationRecord = PointRecord()
//        if (stationPoint == null) {
//            stationRecord.name = "$prefixStation${1}"
//        } else {
//            val index = RegExUtil.getIndexFromName(stationPoint.getName())
//            stationRecord.name = "$prefixStation${index+1}"
//        }
//        return stationRecord
    }


    /**
     * 存储测站点并返回测站id
     *
     * return 新建测站的ids
     */
    @WorkerThread
    private fun saveStationPoint(
        projectId: String,
        stationNameShow: String,
        isTwiceSetup: Boolean
    ): List<String> {

        val stationHeight = mBinding.ilStation.etHeight.text.toString().trim()

        val index = RegExUtil.getIndexFromName(stationNameShow)
        val stationIdList = mutableListOf<String>()

        val pointTypeHeightTraverse = PointRecord.TYPE_HEIGHT_TRAVERSE_STATION_TRANSFER

        if (isTwiceSetup) {
            stationIdList.add(
                savePoint(
                    "$prefixStation${index}${Tools.getString(R.string._left)}",
                    pointTypeHeightTraverse,
                    projectId,
                    stationHeight
                )
            )
            stationIdList.add(
                savePoint(
                    "$prefixStation${index}${Tools.getString(R.string._right)}",
                    pointTypeHeightTraverse,
                    projectId,
                    stationHeight
                )
            )
        } else {
            stationIdList.add(
                savePoint(
                    "$prefixStation${index}",
                    pointTypeHeightTraverse,
                    projectId,
                    stationHeight
                )
            )
        }

        return stationIdList
    }

    /**
     * type 0 站点，1 后视 2 前视
     */
    @WorkerThread
    private fun getNewestPointRecord(
        stationList: List<SurveyorStationRecord>,
        type: Int
    ): PointRecord? {
        var pointRecord: PointRecord? = null
        for (item in stationList) {
            if (item.items != null && item.items.size >= 3) {
                val surveyorPoint = item.items[type]
                val everyPointRecord =
                    SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(surveyorPoint.originalPointId)
                val indexFromName = RegExUtil.getIndexFromName(everyPointRecord?.name ?: "")
                if (indexFromName != -1) {
                    if (pointRecord == null) pointRecord = everyPointRecord
                    if (pointRecord != null) {
                        val indexLast = RegExUtil.getIndexFromName(pointRecord.name)
                        if (indexFromName > indexLast) {
                            pointRecord = everyPointRecord
                        }
                    }
                }
            }
        }
        return pointRecord
    }


}