package com.bingce.controlnetwork.fragment.station.line;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.stationdetail.StationCreateLineKnownUnknownKnownAdapter;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.ui.widget.recyclerview.SwipeDraggableRecyclerViewUtil;
import com.bingce.utils.IOnListGetCallback;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

public abstract class AbstractLineStationKnownUnknownKnownSetupFragment extends AbstractLineStationSetupFragment
        implements StationCreateLineKnownUnknownKnownAdapter.IOnSelectPointDelegate,
        SwipeDraggableRecyclerViewUtil.IStationPointOperatorListener {
    private static final String TYPE_KNOWN_POINT1 = "___known_points1";
    private static final String TYPE_UNKNOWN_POINT = "___unknown_points";
    private static final String TYPE_KNOWN_POINT2 = "___known_points2";
    private StationCreateLineKnownUnknownKnownAdapter stationCreateLineAttachAdapter;
    private final ActivityResultLauncher<Intent> activityResultLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
            return;
        }
        if (TYPE_KNOWN_POINT1.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedIds().size() ||
                    pickPointResult.getSelectedIds().size() != knownItemMinCount()) {
                showKnownPointError();
                return;
            }
            stationCreateLineAttachAdapter.setKnownPoints1(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        } else if (TYPE_UNKNOWN_POINT.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedIds().size() ||
                    pickPointResult.getSelectedIds().isEmpty()) {
                showUnknownPointError();
                return;
            }
            stationCreateLineAttachAdapter.setUnknownPoints(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        } else if (TYPE_KNOWN_POINT2.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedIds().size() ||
                    pickPointResult.getSelectedIds().size() != knownItemMinCount()) {
                showKnownPointError();
                return;
            }
            stationCreateLineAttachAdapter.setKnownPoints2(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        }
        onItemChanged();
    });
    private MenuItem confirmItem;

    protected abstract int selectKnownPointTitle();

    protected abstract int knownItemMinCount();

    protected abstract void defaultItems(IOnListGetCallback<StationDetailItem> callback);

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        if (view == null) {
            return null;
        }
        RecyclerView recyclerView = view.findViewById(R.id.recycler_view);
        if (recyclerView != null) {
            defaultItems(instance -> runOnUI(() -> {
                stationCreateLineAttachAdapter = new StationCreateLineKnownUnknownKnownAdapter(
                        AbstractLineStationKnownUnknownKnownSetupFragment.this, instance);
                recyclerView.setAdapter(stationCreateLineAttachAdapter);
                SwipeDraggableRecyclerViewUtil.setup(stationCreateLineAttachAdapter,
                        recyclerView,
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        AbstractLineStationKnownUnknownKnownSetupFragment.this,
                        getContext());
            }));
        }
        setHasOptionsMenu(true);
        return view;
    }

    @Override
    public void onSelectKnownPoint1(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_KNOWN_POINT1,
                knownItemMinCount(),
                getString(R.string.title_pick_known_point),
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onSelectUnknownPoint(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_UNKNOWN_POINT,
                -1,
                getString(R.string.title_pick_unknown_point),
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onSelectKnownPoint2(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_KNOWN_POINT2,
                knownItemMinCount(),
                getString(R.string.title_pick_known_point),
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onItemRemoved(int adapterPosition) {
        onItemChanged();
    }

    @Override
    public void onItemDragCompleted() {
        onItemChanged();
    }

    /**
     * 测站数据发生了变化--测点被删除、测点被修改、测站被修改、测点被拖拽
     */
    protected void onItemChanged() {
        //检测数据是否合法
        List<StationDetailItem> knownItems1 = stationCreateLineAttachAdapter.knownItems1();
        List<StationDetailItem> unknownItems = stationCreateLineAttachAdapter.unknownItems();
        List<StationDetailItem> knownItems2 = stationCreateLineAttachAdapter.knownItems2();
        if (confirmItem != null) {
            confirmItem.setEnabled(!(knownItems1.size() < knownItemMinCount() ||
                    unknownItems.isEmpty() ||
                    knownItems2.size() < knownItemMinCount()));
        }
    }

    private void showUnknownPointError() {
        ToastUtils.showShort(R.string.tips_unknown_point_must_not_empty);
    }

    protected abstract int knownPointCountError();

    private void showKnownPointError() {
        ToastUtils.showShort(knownPointCountError());
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        confirmItem = menu.findItem(R.id.menu_confirm);
        if (confirmItem != null) {
            confirmItem.setEnabled(false);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        if (R.id.menu_confirm == item.getItemId()) {
            if (getArguments() == null) {
                return super.onOptionsItemSelected(item);
            }
            requestLineName(string -> {
                String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
                String periodId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
                if (getCurrentBatch() != null) {
                    projectId = getCurrentBatch().projectId;
                    periodId = getCurrentBatch().cycleId;
                }
                //1.收集所有点信息
                List<StationDetailItem> knownItems1 = stationCreateLineAttachAdapter.knownItems1();
                List<StationDetailItem> unknownItems = stationCreateLineAttachAdapter.unknownItems();
                List<StationDetailItem> knownItems2 = stationCreateLineAttachAdapter.knownItems2();
                //2.建站
                setup(projectId, periodId, string, knownItems1, unknownItems, knownItems2, () -> {
                    if (getActivity() != null) {
                        getActivity().finish();
                    }
                });
            });
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean cancelBack() {
        List<StationDetailItem> knownItems1 = stationCreateLineAttachAdapter.knownItems1();
        List<StationDetailItem> unknownItems = stationCreateLineAttachAdapter.unknownItems();
        List<StationDetailItem> knownItems2 = stationCreateLineAttachAdapter.knownItems2();
        if (isListEmpty(knownItems1) ||
                isListEmpty(unknownItems) ||
                isListEmpty(knownItems2)) {
            return warningDataLosing(this);
        }
        return false;
    }

    protected abstract void setup(String projectId, String periodId,
                                  String lineName,
                                  List<StationDetailItem> knownItems1,
                                  List<StationDetailItem> unknownItems,
                                  List<StationDetailItem> knownItems2,
                                  Runnable callback);
}
