package com.bingce.controlnetwork.fragment.station.line;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.stationdetail.StationCreateLineSingleAdapter;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.ui.widget.recyclerview.SwipeDraggableRecyclerViewUtil;
import com.bingce.utils.IOnListGetCallback;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

public abstract class AbstractLineStationKnownUnknownSetupFragment extends AbstractLineStationSetupFragment
        implements StationCreateLineSingleAdapter.IOnSelectPointDelegate,
        SwipeDraggableRecyclerViewUtil.IStationPointOperatorListener {
    private static final String TYPE_KNOWN_POINT = "___known_points";
    private static final String TYPE_UNKNOWN_POINT = "___unknown_points";
    private StationCreateLineSingleAdapter stationCreateLineSingleAdapter;
    private final ActivityResultLauncher<Intent> activityResultLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
            return;
        }
        if (TYPE_KNOWN_POINT.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedIds().size() ||
                    pickPointResult.getSelectedIds().size() < knownItemMinCount()) {
                showKnownPointError();
                return;
            }
            stationCreateLineSingleAdapter.setKnownPoints(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        } else if (TYPE_UNKNOWN_POINT.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() != pickPointResult.getSelectedIds().size() ||
                    pickPointResult.getSelectedIds().size() < unknownItemMinCount()) {
                showUnknownPointError();
                return;
            }
            stationCreateLineSingleAdapter.setUnknownPoints(
                    pickPointResult.getSelectedIds(),
                    pickPointResult.getSelectedNames());
        }
        onItemChanged();
    });
    private MenuItem confirmItem;

    protected abstract int knownItemMinCount();

    protected abstract int knownGroupTitle();

    protected abstract int knownPointCountError();

    protected abstract int unknownItemMinCount();

    protected abstract int unknownGroupTitle();

    protected abstract int unknownPointCountError();

    protected abstract void defaultItems(IOnListGetCallback<StationDetailItem> callback);

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        if (view == null) {
            return null;
        }
        RecyclerView recyclerView = view.findViewById(R.id.recycler_view);
        if (recyclerView != null) {
            defaultItems(instance -> runOnUI(() -> {
                stationCreateLineSingleAdapter = new StationCreateLineSingleAdapter(
                        AbstractLineStationKnownUnknownSetupFragment.this, instance);
                recyclerView.setAdapter(stationCreateLineSingleAdapter);
                SwipeDraggableRecyclerViewUtil.setup(stationCreateLineSingleAdapter,
                        recyclerView,
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        AbstractLineStationKnownUnknownSetupFragment.this,
                        getContext());
            }));
        }
        setHasOptionsMenu(true);
        return view;
    }

    @Override
    public void onSelectKnownPoint(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_KNOWN_POINT,
                knownItemMinCount(),
                getString(R.string.title_pick_known_point),
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onSelectUnknownPoint(PickPointResult pickPointResult, ArrayList<String> filters) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                TYPE_UNKNOWN_POINT,
                -1,
                getString(R.string.title_pick_unknown_point),
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onItemRemoved(int adapterPosition) {
        onItemChanged();
    }

    @Override
    public void onItemDragCompleted() {
        onItemChanged();
    }

    /**
     * 测站数据发生了变化--测点被删除、测点被修改、测站被修改、测点被拖拽
     */
    protected void onItemChanged() {
        //检测数据是否合法
        List<StationDetailItem> knownItems = knownItems();
        List<StationDetailItem> unknownItems = unknownItems();
        if (confirmItem != null) {
            confirmItem.setEnabled(!(knownItems.size() < knownItemMinCount() || unknownItems.isEmpty()));
        }
    }

    private void showUnknownPointError() {
        ToastUtils.showShort(unknownPointCountError());
    }

    private void showKnownPointError() {
        ToastUtils.showShort(knownPointCountError());
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        if (confirmItem == null) {
            confirmItem = menu.findItem(R.id.menu_confirm);
            if (confirmItem != null) {
                confirmItem.setEnabled(false);
            }
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        if (R.id.menu_confirm == item.getItemId()) {
            if (getArguments() == null) {
                return super.onOptionsItemSelected(item);
            }
            //1.收集所有点信息
            List<StationDetailItem> knownItems = knownItems();
            List<StationDetailItem> unknownItems = unknownItems();
            //2.循环创建测站
            if (knownItems.size() != knownItemMinCount()) {
                //当已知点不满足最少点的个数时提示
                showKnownPointError();
                return super.onOptionsItemSelected(item);
            }
            if (unknownItems.size() < unknownItemMinCount()) {
                //当未知点不满足最少点的个数时提示
                showUnknownPointError();
                return super.onOptionsItemSelected(item);
            }
            //设置导线的名称（弹框设置名称）
            requestLineName(lineName -> {
                String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
                String periodId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
                if (getCurrentBatch() != null) {
                    projectId = getCurrentBatch().projectId;
                    periodId = getCurrentBatch().cycleId;
                }
                setup(projectId, periodId, lineName, knownItems, unknownItems, () -> {
                    if (getActivity() != null) {
                        getActivity().finish();
                    }
                });
            });
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean cancelBack() {
        List<StationDetailItem> knownItems = knownItems();
        List<StationDetailItem> unknownItems = unknownItems();
        if (isListEmpty(knownItems) ||
                isListEmpty(unknownItems)) {
            return warningDataLosing(this);
        }
        return false;
    }

    /**
     * 设置导线
     *
     * @param projectId
     * @param periodId
     * @param lineName
     * @param knownItems
     * @param unknownItems
     * @param runnable
     */
    protected abstract void setup(String projectId, String periodId,
                                  String lineName,
                                  List<StationDetailItem> knownItems,
                                  List<StationDetailItem> unknownItems,
                                  Runnable runnable);

    protected List<StationDetailItem> knownItems() {
        return stationCreateLineSingleAdapter.knownItems();
    }

    protected List<StationDetailItem> unknownItems() {
        return stationCreateLineSingleAdapter.unknownItems();
    }
}
