package com.bingce.controlnetwork.fragment.station.line;

import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.StringInputFragment;
import com.bingce.controlnetwork.fragment.station.AbstractStationFragment;
import com.bingce.controlnetwork.newui.widget.switchtext.SwitchWithText;
import com.bingce.controlnetwork.util.SurveyorStationUtil;
import com.bingce.utils.IOnListGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

public abstract class AbstractLineStationSetupFragment extends AbstractStationFragment {
    private LineRecord currentBatch;
    private ProjectRecord projectRecord;

    @Override
    protected int layoutRes() {
        return R.layout.fragment_line_surveyor_station_setup_layout;
    }

    @Override
    protected String duplicateStationError(String duplicateStationName) {
        return Tools.getString(R.string.station_already_exist_same_name, duplicateStationName);
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setLeftRightEnable(initLeftRightMeasure());
        //读取projectRecord
        ThreadPoolUtil.execute(() -> {
            if (getArguments() == null) {
                return;
            }
            String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            projectRecord =
                    SurveyorDatabaseFactory
                            .instance
                            .getProjectDataSource()
                            .getRecord(projectId);
        });
    }

    private void setLeftRightEnable(boolean isEnable) {
        //显示左右角开关
        View view = getView();
        if (view != null) {
            SwitchWithText switchCompat = view.findViewById(R.id.left_right_switch);
            if (switchCompat != null) {
                switchCompat.setEnabled(isEnable);
            }
        }
    }


    protected String projectId() {
        return projectRecord == null ? null : projectRecord.getId();
    }

    protected ProjectRecord currentProject() {
        return projectRecord;
    }

    protected void loadBatchRecord(String batchId, IOnListGetCallback<StationDetailItem> callback) {
        SurveyorStationUtil.getBatchItemList(batchId, lineRecord -> {
            if (lineRecord == null) {
                return;
            }
            callback.onGet(lineRecord.items);
            currentBatch = lineRecord;
        });
    }

    private String lineName() {
        if (currentBatch != null) {
            return currentBatch.name;
        }
        return defaultLineName();
    }

    protected LineRecord getCurrentBatch() {
        return currentBatch;
    }

    protected abstract boolean initLeftRightMeasure();

    protected abstract String defaultLineName();

    final protected void requestLineName(StringInputFragment.IInputCallback callback) {
        StringInputFragment.input(getChildFragmentManager(),
                getString(R.string.tips_please_input_line_name),
                lineName(),
                getViewLifecycleOwner(),
                callback);
    }
}
