package com.bingce.controlnetwork.fragment.station.line;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.RegExUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.stationdetail.StationCreateLineWellDoubleUnderGroundAdapter;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.ui.widget.recyclerview.SwipeDraggableRecyclerViewUtil;
import com.bingce.controlnetwork.util.SurveyorStationUtil;
import com.bingce.utils.IOnListGetCallback;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 一井定向的父类
 */
public abstract class AbstractLineStationWellDoubleUnderGroundSetupFragment extends AbstractLineStationSetupFragment
        implements StationCreateLineWellDoubleUnderGroundAdapter.IOnSelectPointDelegate,
        SwipeDraggableRecyclerViewUtil.IStationPointOperatorListener {

    public static final String LINE_NAME = Tools.getString(R.string.second_well_directional_underground);

    private final Handler handler = new Handler(Looper.getMainLooper());

    private MenuItem confirmItem;
    //选择到的点
    private List<StationDetailItem> groundLinkLeftDataList;
    private List<StationDetailItem> groundSideADataList;
    private List<StationDetailItem> steelADataList;
    private List<StationDetailItem> underGroundLinkDataList;
    private List<StationDetailItem> steeBDataList;
    private List<StationDetailItem> groundSideBDataList;
    private List<StationDetailItem> groundLinkRightDataList;

    private StationCreateLineWellDoubleUnderGroundAdapter mAdapter;

    private final ActivityResultLauncher<Intent> activityResultLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
            return;
        }
        if (StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundLinkLeftMinCount()) {
                showErrorMsg(Tools.getString(R.string.well_a_ground_connection_hint));
                return;
            }
            mAdapter.setGroundLinkLeftPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundSideAMinCount()) {
                showErrorMsg(Tools.getString(R.string.well_a_surface_near_well_hint));
                return;
            }
            mAdapter.setGroundSideAPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundSideBMinCount()) {
                showErrorMsg(Tools.getString(R.string.well_b_surface_near_well_hint));
                return;
            }
            mAdapter.setGroundSideBPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT.equals(type)) {
            if (pickPointResult.getSelectedIds().size() < groundLinkRightMinCount()) {
                showErrorMsg(Tools.getString(R.string.well_b_ground_connection_hint));
                return;
            }
            mAdapter.setGroundLinkRightPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundSideAMinCount()) {
                showErrorMsg(Tools.getString(R.string.please_select_well_a_wire));
                return;
            }
            mAdapter.setSteelAPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundSideBMinCount()) {
                showErrorMsg(Tools.getString(R.string.please_select_well_b_wire));
                return;
            }
            mAdapter.setSteelBPoints(pickPointResult);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < underGroundLinkMinCount()) {
                showErrorMsg(Tools.getString(R.string.underground_connection_point_min_hint));
                return;
            }
            mAdapter.setUnderGroundLinkPoints(pickPointResult);
        }
        onItemChanged();
    });

    protected abstract int groundLinkLeftMinCount();

    protected abstract int groundSideAMinCount();

    protected abstract int steelAMinCount();

    protected abstract int underGroundLinkMinCount();

    protected abstract int steelBMinCount();

    protected abstract int groundSideBMinCount();

    protected abstract int groundLinkRightMinCount();

    protected abstract void defaultItems(IOnListGetCallback<StationDetailItem> callback);

    protected List<StationDetailItem> getAdapterDataList() {
        List<StationDetailItem> items = null;
        if (mAdapter != null) {
            items = mAdapter.getItems();
        }
        return items;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        if (view == null) {
            return null;
        }
        RecyclerView recyclerView = view.findViewById(R.id.recycler_view);
        if (recyclerView != null) {
            defaultItems(stationDetailItemList -> runOnUI(() -> {
                mAdapter = new StationCreateLineWellDoubleUnderGroundAdapter(
                        AbstractLineStationWellDoubleUnderGroundSetupFragment.this, stationDetailItemList);

                recyclerView.setAdapter(mAdapter);
                SwipeDraggableRecyclerViewUtil.setup(mAdapter,
                        recyclerView,
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        AbstractLineStationWellDoubleUnderGroundSetupFragment.this,
                        getContext());

                refreshFiveDataList();

            }));
        }
        setHasOptionsMenu(true);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
//        initAutoCloseView();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (handler != null)
            handler.removeCallbacksAndMessages(null);
    }

    protected void goSelectPoint(String type, int maxCount, String title, List<String> filters, PickPointResult pickPointResult) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                type,
                maxCount,
                title,
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }

    @Override
    public void onSelectGroundLinkLeft(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT,
                groundLinkLeftMinCount() + 1, Tools.getString(R.string.well_a_surface_connection_point),
                filters, pickPointResult);
    }

    @Override
    public void onSelectGroundLinkRight(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT,
                groundLinkRightMinCount() + 1, Tools.getString(R.string.well_b_surface_connection_point),
                filters, pickPointResult);
    }

    @Override
    public void onSelectGroundSideA(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A,
                1, Tools.getString(R.string.well_a_surface_near_well_point), filters, pickPointResult);
    }

    @Override
    public void onSelectGroundSideB(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B,
                1, Tools.getString(R.string.well_b_surface_near_well_point), filters, pickPointResult);
    }

    @Override
    public void onSelectSteelA(PickPointResult pickPointResult, List<String> filters) {
//        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A,
//                steelAMinCount() + 1, "A井钢丝", filters, pickPointResult);
        addSteelOnCurrentPage(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A, steelAMinCount() + 1);
    }

    @Override
    public void onSelectSteelB(PickPointResult pickPointResult, List<String> filters) {
//        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B,
//                steelBMinCount() + 1, "B井钢丝", filters, pickPointResult);
        addSteelOnCurrentPage(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B, steelBMinCount() + 1);
    }

    @Override
    public void onSelectUnderGroundLink(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK,
                Integer.MAX_VALUE, Tools.getString(R.string.underground_connection_point), filters, pickPointResult);
    }

    @Override
    public void onSelectOnlyMeasureHorAngle(boolean measureHorAngle) {
        LineRecord currentBatch = getCurrentBatch();
        SurveyorStationUtil.updateLineRecord(currentBatch);
        SurveyorStationUtil.updateStationPointsMeasureHorAag(currentBatch);
    }


    private void addSteelOnCurrentPage(String groupType, int steelMaxCount) {
        List<StationDetailItem> pointsFromType = mAdapter.getPointsFromType(groupType + StationDetailItem._DATA);
        PickPointResult lastSteelPickPoint = PickPointResult.create();
        int newName = 1;
        if (pointsFromType != null && !pointsFromType.isEmpty()) {
            PickPointResult.initPicPointResultData(lastSteelPickPoint, pointsFromType);
            //获取新的钢丝的索引
            StationDetailItem stationDetailItem = pointsFromType.get(pointsFromType.size() - 1);
            String num = RegExUtil.findNum(stationDetailItem.getName());
            assert num != null;
            newName = Integer.parseInt(num) + 1;
        }
        if (lastSteelPickPoint.getSelectedIds().size() >= steelMaxCount) {
            ToastUtils.showShort(R.string.steel_max_hint, steelMaxCount);
            return;
        }
        if (StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A.equals(groupType)) {
            PickPointResult.createPointResultNewData(lastSteelPickPoint, Tools.getString(R.string.wire_a) + newName);
            mAdapter.setSteelAPoints(lastSteelPickPoint);
        } else if (StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B.equals(groupType)) {
            PickPointResult.createPointResultNewData(lastSteelPickPoint, Tools.getString(R.string.wire_b) + newName);
            mAdapter.setSteelBPoints(lastSteelPickPoint);
        }
        onItemChanged();
    }


    @Override
    public void onItemRemoved(int adapterPosition) {
        mAdapter.removeItemForLastSelect(adapterPosition);
        onItemChanged();
    }

    @Override
    public void onItemDragCompleted() {
        onItemChanged();
    }

    /**
     * 测站数据发生了变化--测点被删除、测点被修改、测站被修改、测点被拖拽
     */
    private void onItemChanged() {
        handler.postDelayed(() -> {
            refreshFiveDataList();
            //检测数据是否合法
            if (confirmItem != null) {
                confirmItem.setEnabled(!(groundLinkLeftDataList.size() < groundLinkLeftMinCount() ||
                        groundSideADataList.size() < groundSideAMinCount() ||
                        steelADataList.size() < steelAMinCount() ||
                        underGroundLinkDataList.size() < underGroundLinkMinCount() ||
                        steeBDataList.size() < steelBMinCount() ||
                        groundSideBDataList.size() < groundSideBMinCount() ||
                        groundLinkRightDataList.size() < groundLinkRightMinCount()
                ));
            }
        }, 80);
    }

    private void refreshFiveDataList() {
        String _data = StationDetailItem._DATA;
        groundLinkLeftDataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT + _data);
        groundSideADataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A + _data);
        steelADataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A + _data);
        underGroundLinkDataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK + _data);
        steeBDataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B + _data);
        groundSideBDataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B + _data);
        groundLinkRightDataList = mAdapter.getPointsFromType(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT + _data);
    }

    protected void showErrorMsgForWell(String type) {
        switch (type) {
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT:
                showErrorMsg(Tools.getString(R.string.well_a_ground_connection_hint));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT:
                showErrorMsg(Tools.getString(R.string.well_b_ground_connection_hint));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A:
                showErrorMsg(Tools.getString(R.string.well_a_surface_near_well_hint));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B:
                showErrorMsg(Tools.getString(R.string.well_b_surface_near_well_hint));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A:
                showErrorMsg(Tools.getString(R.string.please_select_well_a_wire));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B:
                showErrorMsg(Tools.getString(R.string.please_select_well_b_wire));
                break;
            case StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK:
                showErrorMsg(Tools.getString(R.string.please_select_an_underground_connection_point));
                break;
            default:
                break;
        }
    }

    private void showErrorMsg(String msg) {
        ToastUtils.showShort(msg);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        confirmItem = menu.findItem(R.id.menu_confirm);
        if (confirmItem != null) {
            confirmItem.setEnabled(false);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        // TODO: 2022/6/15
        if (R.id.menu_confirm == item.getItemId()) {
            if (getArguments() == null) {
                return super.onOptionsItemSelected(item);
            }
//            //设置导线的弹框
//            requestLineName(string -> {
//                //建站
//            });

            String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            String periodId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
            if (getCurrentBatch() != null) {
                projectId = getCurrentBatch().projectId;
                periodId = getCurrentBatch().cycleId;
            }

            setup(projectId, periodId, LINE_NAME,
                    groundLinkLeftDataList,
                    groundSideADataList,
                    steelADataList,
                    underGroundLinkDataList,
                    steeBDataList,
                    groundSideBDataList,
                    groundLinkRightDataList, () -> {
                        if (getActivity() != null) {
                            getActivity().finish();
                        }
                    });
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean cancelBack() {
        // TODO: 2022/6/15
//        List<StationDetailItem> knownItems1 = mAdapter.getGroundLinkPoints();
//        List<StationDetailItem> unknownItems = mAdapter.unknownItems();
//        List<StationDetailItem> knownItems2 = mAdapter.knownItems2();
//        if (isListEmpty(knownItems1) ||
//                isListEmpty(unknownItems) ||
//                isListEmpty(knownItems2)) {
//            return warningDataLosing(this);
//        }
        return false;
    }

    protected abstract void setup(String projectId, String periodId,
                                  String lineName,
                                  List<StationDetailItem> groundLinkLeftDataList,
                                  List<StationDetailItem> groundSideADataList,
                                  List<StationDetailItem> steelADataList,
                                  List<StationDetailItem> underGroundLinkDataList,
                                  List<StationDetailItem> steeBDataList,
                                  List<StationDetailItem> groundSideBDataList,
                                  List<StationDetailItem> groundLinkRightDataList,
                                  Runnable callback);

}
