package com.bingce.controlnetwork.fragment.station.line;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationConstants;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.RegExUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.stationdetail.StationCreateLineWellAdapter;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.ui.widget.recyclerview.SwipeDraggableRecyclerViewUtil;
import com.bingce.utils.IOnListGetCallback;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 一井定向的父类
 */
public abstract class AbstractLineStationWellSetupFragment extends AbstractLineStationSetupFragment
        implements StationCreateLineWellAdapter.IOnSelectPointDelegate,
        SwipeDraggableRecyclerViewUtil.IStationPointOperatorListener {

    public static final String LINE_NAME = Tools.getString(R.string.one_well_orientation);

    private MenuItem confirmItem;
    //选择到的点
    private List<StationDetailItem> groundLinkDataList;
    private List<StationDetailItem> groundSideDataList;
    private List<StationDetailItem> steelWireDataList;
    private List<StationDetailItem> ungroundSideDataList;
    private List<StationDetailItem> ungroundLinkDataList;

    private StationCreateLineWellAdapter mAdapter;


    private final Handler handler = new Handler();

    private final ActivityResultLauncher<Intent> activityResultLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
            return;
        }
        if (StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundLinkMinCount()) {
                showErrorMsg(Tools.getString(R.string.underground_connection_min_hint));
                return;
            }
            mAdapter.setGroundLinkPoints(pickPointResult);
        } else if (StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < groundSideMinCount()) {
                showErrorMsg(Tools.getString(R.string.please_select_the_surface_near_well_point));
                return;
            }
            mAdapter.setGroundSidePoints(pickPointResult);
        } else if (StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE.equals(type)) {
            //检测数据是否正常
            if (pickPointResult.getSelectedIds().size() < steelWireMinCount()) {
                showErrorMsg(Tools.getString(R.string.wire_at_least_two_points));
                return;
            }
            mAdapter.setSteelWirePoints(pickPointResult);
        } else if (StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE.equals(type)) {
            if (pickPointResult.getSelectedIds().size() < undergroundSideMinCount()) {
                showErrorMsg(Tools.getString(R.string.please_select_the_underground_near_well_point));
                return;
            }
            mAdapter.setUndergroudSidePoints(pickPointResult);
        } else if (StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK.equals(type)) {
            if (pickPointResult.getSelectedIds().size() < undergroundLinkMinCount()) {
                showErrorMsg(Tools.getString(R.string.underground_connection_direction_min_hint));
                return;
            }
            mAdapter.setUndergroudLinkPoints(pickPointResult);
        }
        onItemChanged();
    });

    protected abstract int groundLinkMinCount();

    protected abstract int groundSideMinCount();

    protected abstract int steelWireMinCount();

    protected abstract int undergroundSideMinCount();

    protected abstract int undergroundLinkMinCount();

    protected abstract void defaultItems(IOnListGetCallback<StationDetailItem> callback);

    protected List<StationDetailItem> getAdapterDataList() {
        List<StationDetailItem> items = null;
        if (mAdapter != null) {
            items = mAdapter.getItems();
        }
        return items;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        if (view == null) {
            return null;
        }
        RecyclerView recyclerView = view.findViewById(R.id.recycler_view);
        if (recyclerView != null) {
            defaultItems(stationDetailItemList -> runOnUI(() -> {
                mAdapter = new StationCreateLineWellAdapter(
                        AbstractLineStationWellSetupFragment.this, stationDetailItemList);

                recyclerView.setAdapter(mAdapter);
                SwipeDraggableRecyclerViewUtil.setup(mAdapter,
                        recyclerView,
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        AbstractLineStationWellSetupFragment.this,
                        getContext());

                refreshFiveDataList();
//                refreshAutoCloseView();

            }));
        }
        setHasOptionsMenu(true);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
//        initAutoCloseView();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        if (handler != null)
            handler.removeCallbacksAndMessages(null);
    }


    @Override
    public void onSelectGroundLink(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK,
                groundLinkMinCount() + 1, Tools.getString(R.string.select_the_ground_connection_direction_point), filters, pickPointResult);
    }

    protected void goSelectPoint(String type, int maxCount, String title, List<String> filters, PickPointResult pickPointResult) {
        if (getArguments() == null || getContext() == null) {
            return;
        }
        String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        PickPointFragment.pickPoint(
                getContext(),
                type,
                maxCount,
                title,
                filters,
                projectId, pickPointResult, activityResultLauncher);
    }


    @Override
    public void onSelectGroundSide(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE,
                1, Tools.getString(R.string.select_surface_near_well_point), filters, pickPointResult);
    }

    @Override
    public void onSelectSteelWire(PickPointResult pickPointResult, List<String> filters) {
//        goSelectPoint(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE,
//                steelWireMinCount() + 1, "选择钢丝点", filters, pickPointResult);
        PickPointResult lastSteelPickPoint = PickPointResult.create();
        int newName = 1;
        List<StationDetailItem> pointsFromType = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE + StationDetailItem._DATA);
        if (pointsFromType != null && !pointsFromType.isEmpty()) {
            PickPointResult.initPicPointResultData(lastSteelPickPoint, pointsFromType);
            //获取新的钢丝的索引
            StationDetailItem stationDetailItem = pointsFromType.get(pointsFromType.size() - 1);
            String num = RegExUtil.findNum(stationDetailItem.getName());
            assert num != null;
            newName = Integer.parseInt(num) + 1;
        }

        if (lastSteelPickPoint.getSelectedIds().size() >= steelWireMinCount() + 1) {
            ToastUtils.showShort(R.string.steel_max_hint, (steelWireMinCount() + 1));
            return;
        }

        PickPointResult.createPointResultNewData(lastSteelPickPoint, Tools.getString(R.string.wire) + newName);
        mAdapter.setSteelWirePoints(lastSteelPickPoint);

        onItemChanged();
    }

    @Override
    public void onSelectUndergroundLink(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK,
                undergroundLinkMinCount() + 1, Tools.getString(R.string.select_underground_connection_direction_point), filters, pickPointResult);
    }

    @Override
    public void onSelectUndergroundSide(PickPointResult pickPointResult, List<String> filters) {
        goSelectPoint(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE,
                1, Tools.getString(R.string.select_underground_near_well_point), filters, pickPointResult);
    }

//    @Override
//    public void onSelectOnlyMeasureHorAngle(boolean measureHorAngle) {
//        LineRecord currentBatch = getCurrentBatch();
//        SurveyorStationUtil.updateLineRecord(currentBatch);
//        SurveyorStationUtil.updateStationPointsMeasureHorAag(currentBatch);
//    }

    @Override
    public void onItemRemoved(int adapterPosition) {
        mAdapter.removeItemForLastSelect(adapterPosition);
        onItemChanged();
    }

    @Override
    public void onItemDragCompleted() {
        onItemChanged();
    }

    /**
     * 测站数据发生了变化--测点被删除、测点被修改、测站被修改、测点被拖拽
     */
    private void onItemChanged() {
        handler.postDelayed(() -> {
            refreshFiveDataList();
            //检测数据是否合法
            if (confirmItem != null) {
                confirmItem.setEnabled(!(groundLinkDataList.size() < groundLinkMinCount() ||
                        groundSideDataList.size() < groundSideMinCount() ||
                        steelWireDataList.size() < steelWireMinCount() ||
                        ungroundSideDataList.size() < undergroundSideMinCount() ||
                        ungroundLinkDataList.size() < undergroundLinkMinCount()
                ));
            }
//        refreshAutoCloseView();
        }, 80);
    }

    private void refreshFiveDataList() {
        String _data = StationDetailItem._DATA;
        groundLinkDataList = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK + _data);
        groundSideDataList = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE + _data);
        steelWireDataList = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE + _data);
        ungroundSideDataList = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE + _data);
        ungroundLinkDataList = mAdapter.getPointsFromType(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK + _data);
    }

//    private void refreshAutoCloseView() {
//        if (isCanSetAutoCloseCheck()) {
//            setAutoCloseCheckEnable(true, true);
//        } else {
//            setAutoCloseCheckEnable(false, false);
//        }
//    }

    private boolean isCanSetAutoCloseCheck() {
        if (groundLinkDataList == null || steelWireDataList == null || ungroundLinkDataList == null) {
            return false;
        }
        int groundLinkSize = groundLinkDataList.size();
        int steelWireSize = steelWireDataList.size();
        int ungroundLinkSize = ungroundLinkDataList.size();
        return groundLinkSize + steelWireSize >= SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION
                || ungroundLinkSize + steelWireSize >= SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION;
    }


    protected void showErrorMsgForWell(String type) {
        switch (type) {
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK:
                showErrorMsg(Tools.getString(R.string.underground_connection_min_hint));
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE:
                showErrorMsg(Tools.getString(R.string.please_select_the_surface_near_well_point));
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE:
                showErrorMsg(Tools.getString(R.string.wire_at_least_two_points));
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE:
                showErrorMsg(Tools.getString(R.string.please_select_the_underground_near_well_point));
                break;
            case StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK:
                showErrorMsg(Tools.getString(R.string.underground_connection_direction_min_hint));
                break;
            default:
                break;
        }
    }

    private void showErrorMsg(String msg) {
        ToastUtils.showShort(msg);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.menu_confirm, menu);
        confirmItem = menu.findItem(R.id.menu_confirm);
        if (confirmItem != null) {
            confirmItem.setEnabled(false);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        // TODO: 2022/6/15
        if (R.id.menu_confirm == item.getItemId()) {
            if (getArguments() == null) {
                return super.onOptionsItemSelected(item);
            }
//            //设置导线的弹框
//            requestLineName(string -> {
//                //建站
//            });

            String projectId = projectId();//getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            String periodId = getArguments().getString(BundleConstants.KEY_PERIOD_ID);
            if (getCurrentBatch() != null) {
                projectId = getCurrentBatch().projectId;
                periodId = getCurrentBatch().cycleId;
            }
            setup(projectId, periodId, LINE_NAME, groundLinkDataList, groundSideDataList, steelWireDataList, ungroundLinkDataList, ungroundSideDataList, () -> {
                if (getActivity() != null) {
                    getActivity().finish();
                }
            });
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean cancelBack() {
        // TODO: 2022/6/15
//        List<StationDetailItem> knownItems1 = mAdapter.getGroundLinkPoints();
//        List<StationDetailItem> unknownItems = mAdapter.unknownItems();
//        List<StationDetailItem> knownItems2 = mAdapter.knownItems2();
//        if (isListEmpty(knownItems1) ||
//                isListEmpty(unknownItems) ||
//                isListEmpty(knownItems2)) {
//            return warningDataLosing(this);
//        }
        return false;
    }

    protected abstract void setup(String projectId, String cycleId,
                                  String lineName,
                                  List<StationDetailItem> groundLinkDataList,
                                  List<StationDetailItem> groundSideDataList,
                                  List<StationDetailItem> steelWireDataList,
                                  List<StationDetailItem> ungroundLinkDataList,
                                  List<StationDetailItem> ungroundSideDataList,
                                  Runnable callback);


//    private SwitchCompat getAutoCloseView() {
//        if (getView() != null) {
//            return getView().findViewById(R.id.auto_close_switch);
//        }
//        return null;
//    }
//
//    private void setAutoCloseCheckEnable(boolean isCheck, boolean isEnable) {
//        SwitchCompat autoCloseView = getAutoCloseView();
//        if (autoCloseView != null) {
//            autoCloseView.setChecked(isCheck);
//            autoCloseView.setEnabled(isEnable);
//        }
//    }
//
//    /**
//     * 默认是自动闭合的
//     *
//     * @return
//     */
//    protected boolean isAutoClose() {
//        SwitchCompat autoCloseView = getAutoCloseView();
//        if (autoCloseView != null) {
//            return autoCloseView.isChecked();
//        }
//        return true;
//    }
//
//    private void initAutoCloseView() {
//        SwitchCompat autoCloseView = getAutoCloseView();
//        if (autoCloseView != null) {
//            autoCloseView.setOnCheckedChangeListener((buttonView, isChecked) -> {
//                if (isCanSetAutoCloseCheck()) {
//                    setAutoCloseCheckEnable(isChecked, true);
//                }
//            });
//        }
//    }

}
