package com.bingce.controlnetwork.fragment.station.line.attached;

import android.content.Context;
import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.station.line.AbstractLineStationKnownUnknownKnownSetupFragment;
import com.bingce.utils.IOnListGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 新建附合导线
 */
public class LineStationAttachedSetupFragment extends AbstractLineStationKnownUnknownKnownSetupFragment {

    @Override
    protected int knownItemMinCount() {
        return 2;
    }

    @Override
    protected void defaultItems(IOnListGetCallback<StationDetailItem> callback) {
        if (callback == null || getArguments() == null) {
            return;
        }
        String batchId = getArguments().getString(BundleConstants.KEY_LINE_ID);
        if (batchId == null || batchId.isEmpty()) {
            callback.onGet(Arrays.asList(
                    new StationDetailItem(
                            getString(selectKnownPointTitle()),
                            StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1,
                            null),
                    new StationDetailItem(
                            getString(R.string.select_1_unknown_point),
                            StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP,
                            null),
                    new StationDetailItem(
                            getString(selectKnownPointTitle()),
                            StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2,
                            null)));
        } else {
            loadBatchRecord(batchId, callback);
        }
    }

    @Override
    protected int selectKnownPointTitle() {
        return R.string.select_2_known_point;
    }

    @Override
    protected int knownPointCountError() {
        return R.string.tips_known_point_must_more_than_2;
    }

    @Override
    protected boolean initLeftRightMeasure() {
        return true;
    }

    @Override
    protected String defaultLineName() {
        return getString(R.string.wire_fuhe);
    }

    @Override
    protected void setup(String projectId, String periodId,
                         String lineName,
                         List<StationDetailItem> knownItems1,
                         List<StationDetailItem> unknownItems,
                         List<StationDetailItem> knownItems2,
                         Runnable runnable) {
        //判断数据是否准确
        if (knownItems1.size() != knownItemMinCount()) {
            ToastUtils.showShort(knownPointCountError());
            return;
        }
        if (unknownItems.isEmpty()) {
            ToastUtils.showShort(R.string.tips_unknown_point_must_not_empty);
            return;
        }
        if (knownItems2.size() != knownItemMinCount()) {
            ToastUtils.showShort(knownPointCountError());
            return;
        }
        //切换线程
        ThreadPoolUtil.execute(() -> {

            ILineRecordDataSource stationBatchDataSource = SurveyorDatabaseFactory
                    .instance.getLineRecordDatabase();
            ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                    .instance.getSurveyorStation();

            String lineId = "";
            if (getArguments() != null) {
                lineId = getArguments().getString(BundleConstants.KEY_LINE_ID);
            }
            if (lineId != null && !lineId.isEmpty()) {
                //0.删除原有导线和相关测站
                stationBatchDataSource.deleteSync(lineId);
                stationDataSource.deleteByLineIdSync(lineId);
            }

            String error = checkValidate(projectId, periodId,
                    unknownItems, knownItems1, knownItems2);
            if (error != null && !error.isEmpty()) {
                tipsError(error);
                return;
            }
            //0.处理全局控制点
            PointUtil.globalToProjectControlPoint(knownItems1, projectId);
            PointUtil.globalToProjectControlPoint(unknownItems, projectId);
            PointUtil.globalToProjectControlPoint(knownItems2, projectId);


            //1.创建并存储batch
            List<StationDetailItem> records = new ArrayList<>();
            records.add(new StationDetailItem(
                    getString(selectKnownPointTitle()),
                    StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP1,
                    null));
            records.addAll(knownItems1);
            records.add(new StationDetailItem(
                    getString(R.string.select_1_unknown_point),
                    StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP,
                    null));
            records.addAll(unknownItems);
            records.add(new StationDetailItem(
                    getString(selectKnownPointTitle()),
                    StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP2,
                    null));
            records.addAll(knownItems2);
            LineRecord batchRecord = LineRecord.attach(lineName, projectId, periodId, records);
            stationBatchDataSource.save(batchRecord);

            ProjectRecord projectRecord = currentProject();
            boolean showSlopDistance = true;
            if (projectRecord != null) {
                showSlopDistance = projectRecord.showSlopDistance;
            }

            //2.创建测站列表
            StationDetailItem item1 = knownItems1.get(0);
            StationDetailItem item2 = knownItems1.get(1);
            long dateValue = (new Date()).getTime();
            List<SurveyorStationRecord> stationRecordList = new ArrayList<>();
            boolean needLeftRightSurveyor = needLeftRightSurveyor();
            for (int index = 0; index < unknownItems.size(); index++) {
                StationDetailItem item3 = unknownItems.get(index);

                SurveyorStationRecord record =
                        SurveyorStationRecord.lineStation(dateValue++,
                                projectId, periodId, batchRecord.id, item1, item2, item3);
                record.setLeftAndRightSurveyor(needLeftRightSurveyor);
                record.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(record);

                item1 = item2;
                item2 = item3;
            }
            //添加konwn point2
            for (int index = 0; index < knownItems2.size(); index++) {
                StationDetailItem item3 = knownItems2.get(index);

                SurveyorStationRecord record =
                        SurveyorStationRecord.lineStation(dateValue++,
                                projectId, periodId, batchRecord.id, item1, item2, item3);
                record.setLeftAndRightSurveyor(needLeftRightSurveyor);
                record.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(record);

                item1 = item2;
                item2 = item3;
            }

            //3.存储新建的测站列表
            stationDataSource.save(stationRecordList);

            if (runnable != null) {
                runnable.run();
            }
        });
    }

    private static String checkValidate(String projectId, String cycleId,
                                        List<StationDetailItem> unknownItems,
                                        List<StationDetailItem> knownItems1,
                                        List<StationDetailItem> knownItems2) {
        //检测该测期下是否有同名测站
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                .instance
                .getSurveyorStation();
        List<SurveyorStationRecord> currentStationList =
                stationDataSource.getDataList(projectId, cycleId);

        //收集要创建的测站站点id
        List<String> expectedStationIds = new ArrayList<>();
        StationDetailItem tmp = knownItems1.get(1);
        for (int index = 0; index < unknownItems.size(); index++) {
            StationDetailItem item3 = unknownItems.get(index);
            expectedStationIds.add(tmp.getOriginalPointId());
            tmp = item3;
        }
        //添加konwn point2
        for (int index = 0; index < knownItems2.size(); index++) {
            StationDetailItem item3 = knownItems2.get(index);
            expectedStationIds.add(tmp.getOriginalPointId());
            tmp = item3;
        }

        if (currentStationList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : currentStationList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            for (String stationDetailItemId : expectedStationIds) {//遍历即将创建的测站列表，对比是否有相同的
                                if (stationItem.getPointId().equals(stationDetailItemId)) {
                                    PointRecord pointRecord = SurveyorDatabaseFactory
                                            .instance
                                            .getPointDataSource()
                                            .findByIdSync(stationItem.getOriginalPointId());
                                    if (pointRecord != null) {
                                        return pointRecord.getName();
                                    }
                                    return "";
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    public static final int titleCreate = R.string.title_create_attach_line_station;
    public static final int titleEdit = R.string.title_edit_attach_line_station;
    public static final String TAG = LineStationAttachedSetupFragment.class.getName();

    public static void startCreate(Context context, String projectId, String periodId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, periodId);
        FragmentContainerActivity.start(context, TAG, titleCreate, bundle);
    }

    public static void startEdit(Context context, String projectId, String batchId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_LINE_ID, batchId);
        FragmentContainerActivity.start(context, TAG, titleEdit, bundle);
    }

    public static LineStationAttachedSetupFragment newInstance(Bundle bundle) {
        LineStationAttachedSetupFragment fragment = new LineStationAttachedSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
