package com.bingce.controlnetwork.fragment.station.line.close;

import android.content.Context;
import android.os.Bundle;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.station.line.single.LineStationSingleSetupFragment;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 新建闭合导线
 */
public class LineStationCloseSetupFragment extends LineStationSingleSetupFragment {

    @Override
    protected int layoutRes() {
        return R.layout.fragment_close_line_surveyor_station_setup_layout;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        View leftRightSwitch = view.findViewById(R.id.left_right_switch);
        if (leftRightSwitch != null) {
            leftRightSwitch.setVisibility(View.GONE);
        }
    }

    @Override
    protected int unknownItemMinCount() {
        return 2;
    }

    @Override
    protected int unknownGroupTitle() {
        return R.string.select_2_unknown_point;
    }

    @Override
    protected int unknownPointCountError() {
        return R.string.tips_unknown_point_must_more_than_2;
    }

    @Override
    protected String defaultLineName() {
        return getString(R.string.wire_bihe);
    }

    @Override
    protected void setup(String projectId, String periodId,
                         String lineName,
                         List<StationDetailItem> knownItems,
                         List<StationDetailItem> unknownItems,
                         Runnable runnable) {
        ILineRecordDataSource stationBatchDataSource = SurveyorDatabaseFactory
                .instance.getLineRecordDatabase();
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                .instance.getSurveyorStation();

        ThreadPoolUtil.execute(() -> {

            String lineId = "";
            if (getArguments() != null) {
                lineId = getArguments().getString(BundleConstants.KEY_LINE_ID);
            }
            if (lineId != null && !lineId.isEmpty()) {
                //0.删除原有导线和相关测站
                stationBatchDataSource.deleteSync(lineId);
                stationDataSource.deleteByLineIdSync(lineId);
            }

            //验证重复测站
            String name = checkValidate(projectId, periodId,
                    startPointSetupOnce(), knownItems, unknownItems);
            if (name != null && !name.isEmpty()) {
                tipsError(name);
                return;
            }
            //0.处理全局控制点
            PointUtil.globalToProjectControlPoint(knownItems, projectId);
            PointUtil.globalToProjectControlPoint(unknownItems, projectId);

            //1.创建并存储batch
            List<StationDetailItem> records = new ArrayList<>();
            records.add(new StationDetailItem(
                    getString(knownGroupTitle()),
                    StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP,
                    null));
            records.addAll(knownItems);
            records.add(new StationDetailItem(
                    getString(unknownGroupTitle()),
                    StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP,
                    null));
            records.addAll(unknownItems);
            LineRecord lineRecord = LineRecord.close(lineName, projectId, periodId, records);
            stationBatchDataSource.save(lineRecord);

            ProjectRecord projectRecord = currentProject();
            boolean showSlopDistance = true;
            if (projectRecord != null) {
                showSlopDistance = projectRecord.showSlopDistance;
            }
            //2.创建测站列表
            if (startPointSetupOnce()) {
                List<SurveyorStationRecord> stationRecordList = setup2(
                        projectId, periodId, lineRecord.id, showSlopDistance, needLeftRightSurveyor(), knownItems, unknownItems);
                //3.存储
                stationDataSource.save(stationRecordList);
            } else {
                List<SurveyorStationRecord> stationRecordList = setup1(
                        projectId, periodId, lineRecord.id, showSlopDistance, needLeftRightSurveyor(), knownItems, unknownItems);
                //3.存储
                stationDataSource.save(stationRecordList);
            }

            if (runnable != null) {
                runnable.run();
            }
        });
    }

    protected static String checkValidate(String projectId, String cycleId,
                                          boolean startPointSetupOnce,
                                          List<StationDetailItem> unknownItems,
                                          List<StationDetailItem> knownItems) {
        ISurveyorStationDataSource stationDataSource =
                SurveyorDatabaseFactory.instance.getSurveyorStation();
        //检测该测期下是否有同名测站
        List<SurveyorStationRecord> currentStationList =
                stationDataSource.getDataList(projectId, cycleId);
        //收集要创建的测站站点id
        List<String> expectedStationIds = new ArrayList<>();
        if (startPointSetupOnce) {
            StationDetailItem tmp2 = knownItems.get(1);
            StationDetailItem tmp3 = unknownItems.get(0);
            //起算点有3个测点
            expectedStationIds.add(tmp2.getOriginalPointId());
            tmp2 = tmp3;
            for (int index = 1; index < unknownItems.size(); index++) {
                tmp3 = unknownItems.get(index);
                expectedStationIds.add(tmp2.getOriginalPointId());
                tmp2 = tmp3;
            }
            //创建最后一个闭合站
            expectedStationIds.add(tmp2.getOriginalPointId());
        } else {
            StationDetailItem tmp = knownItems.get(1);
            for (int index = 0; index < unknownItems.size(); index++) {
                StationDetailItem item3 = unknownItems.get(index);
                expectedStationIds.add(tmp.getOriginalPointId());
                tmp = item3;
            }
            //--闭合导线添加最后一个闭环
            StationDetailItem item3 = knownItems.get(1);
            expectedStationIds.add(tmp.getOriginalPointId());
            //--最后以known2做站点再加一站
            tmp = item3;
            expectedStationIds.add(tmp.getOriginalPointId());
        }
        if (currentStationList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : currentStationList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            for (String stationDetailItemId : expectedStationIds) {//遍历即将创建的测站列表，对比是否有相同的
                                if (stationItem.getPointId().equals(stationDetailItemId)) {
                                    PointRecord pointRecord = SurveyorDatabaseFactory
                                            .instance
                                            .getPointDataSource()
                                            .findByIdSync(stationItem.getOriginalPointId());
                                    if (pointRecord != null) {
                                        return pointRecord.getName();
                                    }
                                    return "";
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    protected boolean startPointSetupOnce() {
        return switchIsChecked(getView(), R.id.start_point_setup_once);
    }

    private static final int titleCreate = R.string.title_create_close_line_station;
    private static final int titleEdit = R.string.title_edit_close_line_station;
    public static final String TAG = LineStationCloseSetupFragment.class.getName();

    public static void startCreateCloseLine(Context context, String projectId, String periodId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, periodId);
        FragmentContainerActivity.start(context, TAG, titleCreate, bundle);
    }

    public static void startEdit(Context context, String projectId, String batchId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_LINE_ID, batchId);
        FragmentContainerActivity.start(context, TAG, titleEdit, bundle);
    }

    public static LineStationCloseSetupFragment newInstance(Bundle bundle) {
        LineStationCloseSetupFragment fragment = new LineStationCloseSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    private static List<SurveyorStationRecord> setup1(String projectId, String cycleId, String lineId,
                                                      boolean showSlopDistance, boolean needLeftRightSurveyor,
                                                      List<StationDetailItem> knownItems, List<StationDetailItem> unknownItems) {
        StationDetailItem item1 = knownItems.get(0);
        StationDetailItem item2 = knownItems.get(1);
        long dateValue = (new Date()).getTime();
        List<SurveyorStationRecord> stationRecordList = new ArrayList<>();
        for (int index = 0; index < unknownItems.size(); index++) {
            StationDetailItem item3 = unknownItems.get(index);

            SurveyorStationRecord record =
                    SurveyorStationRecord.lineStation(dateValue++,
                            projectId, cycleId, lineId, item1, item2, item3);
            record.setLeftAndRightSurveyor(needLeftRightSurveyor);
            record.setShowSlopDistance(showSlopDistance);
            stationRecordList.add(record);

            item1 = item2;
            item2 = item3;
        }
        //--闭合导线添加最后一个闭环
        StationDetailItem item3 = knownItems.get(1);
        SurveyorStationRecord record1 =
                SurveyorStationRecord.lineStation(dateValue++,
                        projectId, cycleId, lineId, item1, item2, item3);
        record1.setLeftAndRightSurveyor(needLeftRightSurveyor);
        record1.setShowSlopDistance(showSlopDistance);
        stationRecordList.add(record1);

        //--最后以known2做站点再加一站
        item1 = item2;
        item2 = item3;
        item3 = knownItems.get(0);
        SurveyorStationRecord record2 =
                SurveyorStationRecord.lineStation(dateValue++,
                        projectId, cycleId, lineId, item1, item2, item3);
        record2.setLeftAndRightSurveyor(needLeftRightSurveyor);
        record2.setShowSlopDistance(showSlopDistance);
        stationRecordList.add(record2);

        return stationRecordList;
    }

    private static List<SurveyorStationRecord> setup2(String projectId, String cycleId, String lineId,
                                                      boolean showSlopDistance, boolean needLeftRightSurveyor,
                                                      List<StationDetailItem> knownItems, List<StationDetailItem> unknownItems) {
        long dateValue = (new Date()).getTime();
        List<SurveyorStationRecord> stationRecordList = new ArrayList<>();

        StationDetailItem item1 = knownItems.get(0);
        StationDetailItem item2 = knownItems.get(1);
        StationDetailItem item3 = unknownItems.get(0);
        StationDetailItem item4 = unknownItems.get(unknownItems.size() - 1);
        //起算点有3个测点
        SurveyorStationRecord startStation =
                SurveyorStationRecord.closeLineStartPointStation(dateValue++,
                        projectId, cycleId, lineId, item1, item2, item3, item4);
        startStation.setLeftAndRightSurveyor(false);
        startStation.setShowSlopDistance(showSlopDistance);
        stationRecordList.add(startStation);

        item1 = item2;
        item2 = item3;

        for (int index = 1; index < unknownItems.size(); index++) {
            item3 = unknownItems.get(index);

            SurveyorStationRecord record =
                    SurveyorStationRecord.lineStation(dateValue++,
                            projectId, cycleId, lineId, item1, item2, item3);
            record.setLeftAndRightSurveyor(needLeftRightSurveyor);
            record.setShowSlopDistance(showSlopDistance);
            stationRecordList.add(record);

            item1 = item2;
            item2 = item3;
        }

        //创建最后一个闭合站
        item3 = knownItems.get(1);
        SurveyorStationRecord record =
                SurveyorStationRecord.lineStation(dateValue++,
                        projectId, cycleId, lineId, item1, item2, item3);
        record.setLeftAndRightSurveyor(needLeftRightSurveyor);
        record.setShowSlopDistance(showSlopDistance);
        stationRecordList.add(record);

        return stationRecordList;
    }
}
