package com.bingce.controlnetwork.fragment.station.line.single;

import android.content.Context;
import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.station.line.AbstractLineStationKnownUnknownSetupFragment;
import com.bingce.utils.IOnListGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 新建支导线
 */
public class LineStationSingleSetupFragment extends AbstractLineStationKnownUnknownSetupFragment {

    @Override
    protected int knownItemMinCount() {
        return 2;
    }

    @Override
    protected int knownGroupTitle() {
        return R.string.select_2_known_point;
    }

    @Override
    protected int knownPointCountError() {
        return R.string.tips_known_point_must_more_than_2;
    }

    @Override
    protected int unknownItemMinCount() {
        return 1;
    }

    @Override
    protected int unknownGroupTitle() {
        return R.string.select_1_unknown_point;
    }

    @Override
    protected int unknownPointCountError() {
        return R.string.tips_unknown_point_must_not_empty;
    }

    @Override
    protected boolean initLeftRightMeasure() {
        return true;
    }

    @Override
    protected String defaultLineName() {
        return getString(R.string.wire_zhi);
    }

    @Override
    protected void defaultItems(IOnListGetCallback<StationDetailItem> callback) {
        if (callback == null || getArguments() == null) {
            return;
        }
        String batchId = getArguments().getString(BundleConstants.KEY_LINE_ID);
        if (batchId == null || batchId.isEmpty()) {
            callback.onGet(Arrays.asList(
                    new StationDetailItem(
                            getString(knownGroupTitle()),
                            StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP,
                            null),
                    new StationDetailItem(
                            getString(unknownGroupTitle()),
                            StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP,
                            null)));
        } else {
            loadBatchRecord(batchId, callback);
        }
    }

    @Override
    protected void setup(String projectId, String periodId,
                         String lineName,
                         List<StationDetailItem> knownItems,
                         List<StationDetailItem> unknownItems,
                         Runnable runnable) {
        ILineRecordDataSource stationBatchDataSource = SurveyorDatabaseFactory
                .instance.getLineRecordDatabase();
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                .instance.getSurveyorStation();

        ThreadPoolUtil.execute(() -> {
            String lineId = "";
            if (getArguments() != null) {
                lineId = getArguments().getString(BundleConstants.KEY_LINE_ID);
            }
            if (lineId != null && !lineId.isEmpty()) {
                //0.删除原有导线和相关测站
                stationBatchDataSource.deleteSync(lineId);
                stationDataSource.deleteByLineIdSync(lineId);
            }

            String error = checkValidate(projectId, periodId, knownItems, unknownItems);
            if (error != null && !error.isEmpty()) {
                tipsError(error);
                return;
            }
            //0.处理全局控制点
            PointUtil.globalToProjectControlPoint(knownItems, projectId);
            PointUtil.globalToProjectControlPoint(unknownItems, projectId);

            //1.创建并存储batch
            List<StationDetailItem> records = new ArrayList<>();
            records.add(new StationDetailItem(
                    getString(knownGroupTitle()),
                    StationDetailItem.ITEM_TYPE_KNOWN_POINT_GROUP,
                    null));
            records.addAll(knownItems);
            records.add(new StationDetailItem(
                    getString(unknownGroupTitle()),
                    StationDetailItem.ITEM_TYPE_UN_KNOWN_POINT_GROUP,
                    null));
            records.addAll(unknownItems);
            LineRecord batchRecord = LineRecord.single(lineName, projectId, periodId, records);
            stationBatchDataSource.save(batchRecord);

            ProjectRecord projectRecord = currentProject();
            boolean showSlopDistance = true;
            if (projectRecord != null) {
                showSlopDistance = projectRecord.showSlopDistance;
            }
            //2.创建测站列表
            StationDetailItem item1 = knownItems.get(0);
            StationDetailItem item2 = knownItems.get(1);
            long dateValue = (new Date()).getTime();
            List<SurveyorStationRecord> stationRecordList = new ArrayList<>();
            boolean needLeftRightSurveyor = needLeftRightSurveyor();
            for (int index = 0; index < unknownItems.size(); index++) {
                StationDetailItem item3 = unknownItems.get(index);
                SurveyorStationRecord record =
                        SurveyorStationRecord.lineStation(dateValue++,
                                projectId, periodId, batchRecord.id, item1, item2, item3);
                record.setLeftAndRightSurveyor(needLeftRightSurveyor);
                record.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(record);

                item1 = item2;
                item2 = item3;
            }

            stationDataSource.save(stationRecordList);

            if (runnable != null) {
                runnable.run();
            }
        });
    }

    private static String checkValidate(String projectId, String cycleId,
                                        List<StationDetailItem> knownItems,
                                        List<StationDetailItem> unknownItems) {
        ISurveyorStationDataSource stationDataSource =
                SurveyorDatabaseFactory.instance.getSurveyorStation();
        //检测该测期下是否有同名测站
        List<SurveyorStationRecord> currentStationList =
                stationDataSource.getDataList(projectId, cycleId);
        //收集要创建的测站站点id
        List<String> expectedStationIds = new ArrayList<>();
        StationDetailItem tmp = knownItems.get(1);
        for (int index = 0; index < unknownItems.size(); index++) {
            StationDetailItem item3 = unknownItems.get(index);
            expectedStationIds.add(tmp.getOriginalPointId());
            tmp = item3;
        }
        if (currentStationList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : currentStationList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            for (String stationDetailItemId : expectedStationIds) {//遍历即将创建的测站列表，对比是否有相同的
                                if (stationItem.getPointId().equals(stationDetailItemId)) {
//                                    return false;
                                    PointRecord pointRecord = SurveyorDatabaseFactory
                                            .instance
                                            .getPointDataSource()
                                            .findByIdSync(stationItem.getOriginalPointId());
                                    if (pointRecord != null) {
                                        return pointRecord.getName();
                                    }
                                    return "";
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    public static final int titleCreate = R.string.title_create_sinle_line_station;
    public static final int titleEdit = R.string.title_edit_sinle_line_station;

    public static final String TAG = LineStationSingleSetupFragment.class.getName();

    public static void startCreate(Context context, String projectId, String period) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, period);
        FragmentContainerActivity.start(context, TAG, titleCreate, bundle);
    }

    public static void startEdit(Context context, String projectId, String batchId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_LINE_ID, batchId);
        FragmentContainerActivity.start(context, TAG, titleEdit, bundle);
    }

    public static LineStationSingleSetupFragment newInstance(Bundle bundle) {
        LineStationSingleSetupFragment fragment = new LineStationSingleSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
