package com.bingce.controlnetwork.fragment.station.line.well;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecordOperate;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationConstants;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.station.line.AbstractLineStationWellDoubleUnderGroundSetupFragment;
import com.bingce.utils.IOnListGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 新建二井定向-地下
 */
public class LineStationWellDoubleUnderGroundSetupFragment extends AbstractLineStationWellDoubleUnderGroundSetupFragment {

    public static final int titleCreate = R.string.title_create_well_line_station_double_under_ground;
    public static final int titleEdit = R.string.title_edit_well_line_station_double_under_ground;
    public static final String TAG = LineStationWellDoubleUnderGroundSetupFragment.class.getName();
    //区分是初始页面还是编辑页面
    private boolean isDefaultPage;

    public static void startCreate(Context context, String projectId, String periodId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, periodId);
        FragmentContainerActivity.start(context, TAG, titleCreate, bundle);
    }

    public static void startEdit(Context context, String projectId, String batchId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_LINE_ID, batchId);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        FragmentContainerActivity.start(context, TAG, titleEdit, bundle);
    }

    public static LineStationWellDoubleUnderGroundSetupFragment newInstance(Bundle bundle) {
        LineStationWellDoubleUnderGroundSetupFragment fragment = new LineStationWellDoubleUnderGroundSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    protected boolean initLeftRightMeasure() {
        return false;
    }

    @Override
    protected int groundLinkLeftMinCount() {
        return 1;
    }

    @Override
    protected int groundSideAMinCount() {
        return 1;
    }

    @Override
    protected int steelAMinCount() {
        return 1;
    }

    @Override
    protected int underGroundLinkMinCount() {
        return 2;
    }

    @Override
    protected int steelBMinCount() {
        return 1;
    }

    @Override
    protected int groundSideBMinCount() {
        return 1;
    }

    @Override
    protected int groundLinkRightMinCount() {
        return 1;
    }

    @Override
    protected void defaultItems(IOnListGetCallback<StationDetailItem> callback) {
        if (callback == null || getArguments() == null) {
            return;
        }
        String batchId = getArguments().getString(BundleConstants.KEY_LINE_ID);
        isDefaultPage = TextUtils.isEmpty(batchId);
        if (isDefaultPage) {
            callback.onGet(Arrays.asList(
                    new StationDetailItem(
                            Tools.getString(R.string.well_a_surface_connection_direction_limit),
                            StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.well_a_surface_near_well_point),
                            StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.a_well_wire),
                            StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.underground_connection_points_min_hint),
                            StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.b_well_wire),
                            StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.well_b_surface_near_well_point),
                            StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.well_b_surface_connection_direction_limit),
                            StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT,
                            null)
            ));
        } else {
            loadBatchRecord(batchId, callback);
        }
    }

    @Override
    protected void setup(String projectId, String periodId, String lineName,
                         List<StationDetailItem> groundLinkLeftDataList, List<StationDetailItem> groundSideADataList,
                         List<StationDetailItem> steelADataList, List<StationDetailItem> underGroundLinkDataList,
                         List<StationDetailItem> steelBDataList, List<StationDetailItem> groundSideBDataList,
                         List<StationDetailItem> groundLinkRightDataList,
                         Runnable callback) {
        //判断数据是否准确
        if (groundLinkLeftDataList.size() < groundLinkLeftMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_LEFT);
            return;
        }
        if (groundSideADataList.size() < groundSideAMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_A);
            return;
        }
        if (steelADataList.size() < steelAMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_A);
            return;
        }
        if (underGroundLinkDataList.size() < underGroundLinkMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_UNDERGROUND_LINK);
            return;
        }
        if (steelBDataList.size() < steelBMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_STEEL_B);
            return;
        }
        if (groundSideBDataList.size() < groundSideBMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_SIDE_B);
            return;
        }
        if (groundLinkRightDataList.size() < groundLinkRightMinCount()) {
            showErrorMsgForWell(StationDetailItem.TYPE_WELL_DOUBLE_GROUND_LINK_RIGHT);
            return;
        }

        //存储钢丝点到数据库
        List<StationDetailItem> stationDetailItemList = new ArrayList<>();
        stationDetailItemList.addAll(steelADataList);
        stationDetailItemList.addAll(steelBDataList);
        //save里有切换到子线程
        PointRecordOperate.saveStationDetailItemListToPointRecord(stationDetailItemList, projectId());

        //切换线程
        ThreadPoolUtil.execute(() -> {
            ILineRecordDataSource stationBatchDataSource = SurveyorDatabaseFactory
                    .instance.getLineRecordDatabase();
            ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                    .instance.getSurveyorStation();

            String lineId = "";
            if (getArguments() != null) {
                lineId = getArguments().getString(BundleConstants.KEY_LINE_ID);
            }
            if (lineId != null && !lineId.isEmpty()) {
                //0.删除原有导线和相关测站
                stationBatchDataSource.deleteSync(lineId);
                stationDataSource.deleteByLineIdSync(lineId);
            }

            String error = checkValidate(projectId, periodId,
                    groundSideADataList,
                    underGroundLinkDataList,
                    groundSideBDataList);
            if (error != null && !error.isEmpty()) {
                tipsError(error);
                return;
            }
            //0.处理全局控制点
            PointUtil.globalToProjectControlPoint(groundLinkLeftDataList, projectId);
            PointUtil.globalToProjectControlPoint(groundSideADataList, projectId);
//            ProjectPointUtil.dealGlobalControlPoints2(steelADataList, projectId);
            PointUtil.globalToProjectControlPoint(underGroundLinkDataList, projectId);
//            ProjectPointUtil.dealGlobalControlPoints2(steelBDataList, projectId);
            PointUtil.globalToProjectControlPoint(groundSideBDataList, projectId);
            PointUtil.globalToProjectControlPoint(groundLinkRightDataList, projectId);
            //1.创建并存储batch
            List<StationDetailItem> lineRecordPointList = getAdapterDataList();
            LineRecord batchRecord = LineRecord.well(LineRecord.TYPE_WELL_DOUBLE_UNDERGROUND, lineName, projectId, periodId, lineRecordPointList);
            stationBatchDataSource.save(batchRecord);

            ProjectRecord projectRecord = currentProject();
            boolean showSlopDistance = true;
            if (projectRecord != null) {
                showSlopDistance = projectRecord.showSlopDistance;
            }

            //2.创建测站列表
            long dateValue = (new Date()).getTime();
            List<SurveyorStationRecord> stationRecordList = new ArrayList<>();
            boolean needLeftRightSurveyor = needLeftRightSurveyor();

            //A井地面近井点
            for (StationDetailItem itemSideA : groundSideADataList) {
                List<StationDetailItem> pointList = new ArrayList<>();
                pointList.add(itemSideA);
                pointList.addAll(groundLinkLeftDataList);
                pointList.addAll(steelADataList);

                SurveyorStationRecord wellStationRecord = SurveyorStationRecord.createWellStationRecord(dateValue++,
                        projectId, periodId, batchRecord.id, pointList);
//                //设置站是否闭合
                if (pointList.size() > SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION) {
                    wellStationRecord.setAutoClose(true);
                }
                wellStationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor);
                wellStationRecord.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(wellStationRecord);
            }

            //地下连接点至少两个
            if (underGroundLinkDataList != null && !underGroundLinkDataList.isEmpty()) {
                for (int i = 0; i < underGroundLinkDataList.size(); i++) {
                    List<StationDetailItem> pointList = new ArrayList<>();
                    pointList.add(underGroundLinkDataList.get(i));

                    if (i - 1 < 0) {
                        //第一个节点
                        pointList.addAll(steelADataList);
                    } else {
                        pointList.add(underGroundLinkDataList.get(i - 1));
                    }

                    if (i + 1 > underGroundLinkDataList.size() - 1) {
                        pointList.addAll(steelBDataList);
                    } else {
                        pointList.add(underGroundLinkDataList.get(i + 1));
                    }

                    SurveyorStationRecord wellStationRecord = SurveyorStationRecord.createWellStationRecord(dateValue++,
                            projectId, periodId, batchRecord.id, pointList);

                    //设置站是否闭合
                    if (pointList.size() > SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION) {
                        wellStationRecord.setAutoClose(true);
                    }

                    wellStationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor);
                    wellStationRecord.setShowSlopDistance(showSlopDistance);
                    stationRecordList.add(wellStationRecord);
                }
            }

            //B井地面近井点
            for (StationDetailItem itemSideB : groundSideBDataList) {
                List<StationDetailItem> pointList = new ArrayList<>();
                pointList.add(itemSideB);
                pointList.addAll(steelBDataList);
                pointList.addAll(groundLinkRightDataList);

                SurveyorStationRecord wellStationRecord = SurveyorStationRecord.createWellStationRecord(dateValue++,
                        projectId, periodId, batchRecord.id, pointList);
                //设置站是否闭合
                if (pointList.size() > SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION) {
                    wellStationRecord.setAutoClose(true);
                }
                wellStationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor);
                wellStationRecord.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(wellStationRecord);
            }


            //3.存储新建的测站列表
            stationDataSource.save(stationRecordList);

            if (callback != null) {
                callback.run();
            }
        });

    }


    @Override
    protected String defaultLineName() {
        return getString(R.string.station_type_well_wire_double);
    }


    private String checkValidate(String projectId, String cycleId,
                                 List<StationDetailItem> groundSideADataList,
                                 List<StationDetailItem> underGroundLinkDataList,
                                 List<StationDetailItem> groundSideBDataList) {

        //如果是编辑页面就不检查
        if (!isDefaultPage) return null;

        //检测该测期下是否有同名测站
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory.instance.getSurveyorStation();
        List<SurveyorStationRecord> currentStationList =
                stationDataSource.getDataList(projectId, cycleId);

        //收集要创建的测站站点id
        List<String> expectedStationIds = new ArrayList<>();
        collectIdsFromList(expectedStationIds, groundSideADataList);
        collectIdsFromList(expectedStationIds, underGroundLinkDataList);
        collectIdsFromList(expectedStationIds, groundSideBDataList);

        if (currentStationList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : currentStationList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            for (String stationDetailItemId : expectedStationIds) {//遍历即将创建的测站列表，对比是否有相同的
                                if (stationItem.getPointId().equals(stationDetailItemId)) {
                                    PointRecord pointRecord = SurveyorDatabaseFactory
                                            .instance
                                            .getPointDataSource()
                                            .findByIdSync(stationItem.getOriginalPointId());
                                    if (pointRecord != null) {
                                        return pointRecord.getName();
                                    }
                                    return "";
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    private void collectIdsFromList(List<String> collectIdList, List<StationDetailItem> tagetList) {
        for (StationDetailItem stationDetailItem : tagetList) {
            String pointId = stationDetailItem.getOriginalPointId();
            if (!TextUtils.isEmpty(pointId)) {
                collectIdList.add(pointId);
            }
        }
    }


}
