package com.bingce.controlnetwork.fragment.station.line.well;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.ILineRecordDataSource;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecordOperate;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationConstants;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.controlapphelper.util.PointUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.station.line.AbstractLineStationWellSetupFragment;
import com.bingce.utils.IOnListGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;


/**
 * 新建一井定向
 */
public class LineStationWellSetupFragment extends AbstractLineStationWellSetupFragment {

    public static final int titleCreate = R.string.title_create_well_line_station;
    public static final int titleEdit = R.string.title_edit_well_line_station;
    public static final String TAG = LineStationWellSetupFragment.class.getName();
    //区分是初始页面还是编辑页面
    private boolean isDefaultPage;

    public static void startCreate(Context context, String projectId, String periodId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PERIOD_ID, periodId);
        FragmentContainerActivity.start(context, TAG, titleCreate, bundle);
    }

    public static void startEdit(Context context, String projectId, String batchId) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_LINE_ID, batchId);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        FragmentContainerActivity.start(context, TAG, titleEdit, bundle);
    }

    public static LineStationWellSetupFragment newInstance(Bundle bundle) {
        LineStationWellSetupFragment fragment = new LineStationWellSetupFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    protected boolean initLeftRightMeasure() {
        return false;
    }

    @Override
    protected void defaultItems(IOnListGetCallback<StationDetailItem> callback) {
        if (callback == null || getArguments() == null) {
            return;
        }
        String batchId = getArguments().getString(BundleConstants.KEY_LINE_ID);
        isDefaultPage = TextUtils.isEmpty(batchId);
        if (isDefaultPage) {
            callback.onGet(Arrays.asList(
                    new StationDetailItem(
                            Tools.getString(R.string.ground_connection_direction_one_or_two),
                            StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.surface_near_well_point),
                            StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.steel_wire_two_or_three),
                            StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.underground_near_well_point),
                            StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE,
                            null),
                    new StationDetailItem(
                            Tools.getString(R.string.underground_connection_direction_one_or_two),
                            StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK,
                            null)
            ));
        } else {
            loadBatchRecord(batchId, callback);
        }
    }

    @Override
    protected int groundLinkMinCount() {
        return 1;
    }

    @Override
    protected int groundSideMinCount() {
        return 1;
    }

    @Override
    protected int steelWireMinCount() {
        return 2;
    }

    @Override
    protected int undergroundSideMinCount() {
        return 1;
    }

    @Override
    protected int undergroundLinkMinCount() {
        return 1;
    }

    @Override
    protected String defaultLineName() {
        return getString(R.string.station_type_well_wire);
    }

    @Override
    protected void setup(String projectId, String cycleId, String lineName,
                         List<StationDetailItem> groundLinkDataList, List<StationDetailItem> groundSideDataList,
                         List<StationDetailItem> steelWireDataList, List<StationDetailItem> ungroundLinkDataList,
                         List<StationDetailItem> ungroundSideDataList, Runnable callback) {
        //判断数据是否准确
        if (groundLinkDataList.size() < groundLinkMinCount()) {
            showErrorMsgForWell(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_LINK);
            return;
        }
        if (groundSideDataList.size() < groundSideMinCount()) {
            showErrorMsgForWell(StationDetailItem.ITEM_TYPE_WELL_GROUP_GROUND_SIDE);
            return;
        }
        if (steelWireDataList.size() < steelWireMinCount()) {
            showErrorMsgForWell(StationDetailItem.ITEM_TYPE_WELL_GROUP_STEEL_WIRE);
            return;
        }
        if (ungroundLinkDataList.size() < undergroundLinkMinCount()) {
            showErrorMsgForWell(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_LINK);
            return;
        }
        if (ungroundSideDataList.size() < undergroundSideMinCount()) {
            showErrorMsgForWell(StationDetailItem.ITEM_TYPE_WELL_GROUP_UNDERGROUND_SIDE);
            return;
        }

        //存储钢丝点到数据库
        List<StationDetailItem> stationDetailItemList = new ArrayList<>();
        stationDetailItemList.addAll(steelWireDataList);
        //save里有切换到子线程
        PointRecordOperate.saveStationDetailItemListToPointRecord(stationDetailItemList, projectId());

        //切换线程
        ThreadPoolUtil.execute(() -> {
            ILineRecordDataSource stationBatchDataSource = SurveyorDatabaseFactory
                    .instance.getLineRecordDatabase();
            ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory
                    .instance.getSurveyorStation();

            String lineId = "";
            if (getArguments() != null) {
                lineId = getArguments().getString(BundleConstants.KEY_LINE_ID);
            }
            if (lineId != null && !lineId.isEmpty()) {
                //0.删除原有导线和相关测站
                stationBatchDataSource.deleteSync(lineId);
                stationDataSource.deleteByLineIdSync(lineId);
            }

            String error = checkValidate(projectId, cycleId,
                    groundSideDataList, ungroundSideDataList);
            if (error != null && !error.isEmpty()) {
                tipsError(error);
                return;
            }
            //0.处理全局控制点
            PointUtil.globalToProjectControlPoint(groundLinkDataList, projectId);
            PointUtil.globalToProjectControlPoint(groundSideDataList, projectId);
            PointUtil.globalToProjectControlPoint(steelWireDataList, projectId);
            PointUtil.globalToProjectControlPoint(ungroundLinkDataList, projectId);
            PointUtil.globalToProjectControlPoint(ungroundSideDataList, projectId);
            //1.创建并存储batch
            List<StationDetailItem> lineRecordPointList = getAdapterDataList();
            LineRecord batchRecord = LineRecord.well(LineRecord.TYPE_WELL, lineName, projectId, cycleId, lineRecordPointList);
            stationBatchDataSource.save(batchRecord);

            ProjectRecord projectRecord = currentProject();
            boolean showSlopDistance = true;
            if (projectRecord != null) {
                showSlopDistance = projectRecord.showSlopDistance;
            }

            //2.创建测站列表
            long dateValue = (new Date()).getTime();
            List<SurveyorStationRecord> stationRecordList = new ArrayList<>();
            boolean needLeftRightSurveyor = needLeftRightSurveyor();

            for (StationDetailItem groundStationDetailItem : groundSideDataList) {
                List<StationDetailItem> pointList = new ArrayList<>();
                pointList.add(groundStationDetailItem);
                //添加地上观测点
                pointList.addAll(groundLinkDataList);
                //添加钢丝点
                pointList.addAll(steelWireDataList);
                SurveyorStationRecord wellStationRecord = SurveyorStationRecord.createWellStationRecord(dateValue++,
                        projectId, cycleId, batchRecord.id, pointList);
                //设置站是否闭合
                if (pointList.size() > SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION) {
                    wellStationRecord.setAutoClose(true);
                }
                wellStationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor);
                wellStationRecord.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(wellStationRecord);
            }

            for (StationDetailItem ungroundStationDetailItem : ungroundSideDataList) {
                List<StationDetailItem> pointList = new ArrayList<>();
                pointList.add(ungroundStationDetailItem);
                //添加地下观测点
                pointList.addAll(ungroundLinkDataList);
                //添加钢丝点
                pointList.addAll(steelWireDataList);
                SurveyorStationRecord wellStationRecord = SurveyorStationRecord.createWellStationRecord(dateValue++,
                        projectId, cycleId, batchRecord.id, pointList);
                //设置站是否闭合
                if (pointList.size() > SurveyorStationConstants.AUTO_CLOSE_NUM_INCLUDE_STATION) {
                    wellStationRecord.setAutoClose(true);
                }

                wellStationRecord.setLeftAndRightSurveyor(needLeftRightSurveyor);
                wellStationRecord.setShowSlopDistance(showSlopDistance);
                stationRecordList.add(wellStationRecord);
            }


            //3.存储新建的测站列表
            stationDataSource.save(stationRecordList);

            if (callback != null) {
                callback.run();
            }
        });
    }

    private String checkValidate(String projectId, String cycleId,
                                 List<StationDetailItem> groundSideDataList,
                                 List<StationDetailItem> ungroundSideDataList) {
        //如果是编辑页面就不检查
        if (!isDefaultPage) return null;

        //检测该测期下是否有同名测站
        ISurveyorStationDataSource stationDataSource = SurveyorDatabaseFactory.instance.getSurveyorStation();
        List<SurveyorStationRecord> currentStationList =
                stationDataSource.getDataList(projectId, cycleId);

        //收集要创建的测站站点id
        List<String> expectedStationIds = new ArrayList<>();
        collectIdsFromList(expectedStationIds, groundSideDataList);
        collectIdsFromList(expectedStationIds, ungroundSideDataList);

        if (currentStationList != null) {//遍历当前已有测站，检测是否有同名测站
            for (SurveyorStationRecord record : currentStationList) {
                List<SurveyorPoint> items = record.getItems();
                if (items != null) {//检测测站名是否相同
                    for (SurveyorPoint stationItem : items) {
                        if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                            for (String stationDetailItemId : expectedStationIds) {//遍历即将创建的测站列表，对比是否有相同的
                                if (stationItem.getPointId().equals(stationDetailItemId)) {
                                    PointRecord pointRecord = SurveyorDatabaseFactory
                                            .instance
                                            .getPointDataSource()
                                            .findByIdSync(stationItem.getOriginalPointId());
                                    if (pointRecord != null) {
                                        return pointRecord.getName();
                                    }
                                    return "";
                                }
                            }
                            break;
                        }
                    }
                }
            }
        }
        return null;
    }

    private void collectIdsFromList(List<String> collectIdList, List<StationDetailItem> tagetList) {
        for (StationDetailItem stationDetailItem : tagetList) {
            String pointId = stationDetailItem.getOriginalPointId();
            if (!TextUtils.isEmpty(pointId)) {
                collectIdList.add(pointId);
            }
        }
    }


}
