package com.bingce.controlnetwork.fragment.stationdetail;

import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.ViewModelProvider;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.station.defaultstation.SingleStationDetailAndEditContentFragment;
import com.bingce.controlnetwork.newui.widget.switchtext.SwitchWithText;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;


/**
 * 测站编辑--包含编辑测点、站点功能,可能是「多测回测角」，也可能是「导线测站」中某站
 */
public class SingleStationDetailAndEditFragment extends BaseFragment {
    public static final int title = R.string.title_surveyor_station_edit;
    public static final String TAG = SingleStationDetailAndEditFragment.class.getName();

    private StationDetailViewModel stationDetailViewModel;
    private String lineType;
    private SwitchWithText leftRightSwitch;
    private SingleStationDetailAndEditContentFragment contentFragment;

    @Override
    public View onCreateView(@NotNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_surveyor_station_detail_or_edit_layout, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getArguments() == null) {
            return;
        }
        //创建显示点列表的fragment
        String stationId = getArguments().getString(BundleConstants.KEY_SURVEYOR_STATION_ID);
        String projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        lineType = getArguments().getString(BundleConstants.KEY_LINE_TYPE);

        leftRightSwitch = view.findViewById(R.id.left_right_switch);
        leftRightSwitch.setEnabled(true);

        stationDetailViewModel = new ViewModelProvider(requireActivity()).get(StationDetailViewModel.class);
        stationDetailViewModel.getRefreshUiLiveData().observe(getViewLifecycleOwner(), refresh -> {
            if (contentFragment != null) {
                int surveyorPointSize = contentFragment.getSurveyorPointSize();

                boolean leftAndRightSurveyor = stationDetailViewModel.isLeftAndRightSurveyor();
                if (surveyorPointSize == 2) {
                    leftRightSwitch.setVisibility(View.VISIBLE);
                    leftRightSwitch.setChecked(leftAndRightSurveyor);
                    updateStationLeftRightSurveyor(leftAndRightSurveyor, stationId);
                } else {
                    leftRightSwitch.setVisibility(View.GONE);
                    updateStationLeftRightSurveyor(false, stationId);
                }

                updateStationAutoClose(surveyorPointSize > 3, stationId);

            }

        });

        contentFragment = SingleStationDetailAndEditContentFragment.newInstance(stationId, projectId);
        //添加fragment
        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.fragment_container,
                        contentFragment,
                        SingleStationDetailAndEditContentFragment.TAG)
                .commit();

        leftRightSwitch.setOnCheckChangeListener(checked -> {
            stationDetailViewModel.setLeftOrRight(checked);
            updateStationLeftRightSurveyor(checked, stationId);
        });
    }

    private void updateStationAutoClose(boolean autoClose, String stationId) {
        ThreadPoolUtil.execute(() -> {
            ISurveyorStationDataSource surveyorStationDataSource =
                    SurveyorDatabaseFactory
                            .instance
                            .getSurveyorStation();
            SurveyorStationRecord surveyorStationRecord =
                    surveyorStationDataSource.getRecordSync(stationId);
            //修改
            surveyorStationRecord.setAutoClose(autoClose);
            //存储
            surveyorStationDataSource.save(surveyorStationRecord);
        });
    }

    private void updateStationLeftRightSurveyor(boolean leftRightSurveyor, String stationId) {
        Log.d("hwhw", "更新左右角到数据库");
        ThreadPoolUtil.execute(() -> {
            ISurveyorStationDataSource surveyorStationDataSource =
                    SurveyorDatabaseFactory
                            .instance
                            .getSurveyorStation();
            SurveyorStationRecord surveyorStationRecord =
                    surveyorStationDataSource.getRecordSync(stationId);
            //修改
            surveyorStationRecord.setLeftAndRightSurveyor(leftRightSurveyor);
            //存储
            surveyorStationDataSource.save(surveyorStationRecord);
        });
    }

    public static void start(Context context, String surveyorStationId,
                             String surveyorStationName, String projectId, String lineType) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_ID, surveyorStationId);
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_NAME, surveyorStationName);
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_LINE_TYPE, lineType);
        FragmentContainerActivity.start(context, TAG, context.getString(title) + "-" + surveyorStationName, bundle);
    }

    public static SingleStationDetailAndEditFragment newInstance(Bundle bundle) {
        SingleStationDetailAndEditFragment fragment = new SingleStationDetailAndEditFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}
