package com.bingce.controlnetwork.fragment.stationsetting

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import androidx.annotation.WorkerThread
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord
import com.bingce.controlapphelper.datasource.database.heighttraverse.stationsetting.model.StationSettingHeightTraverseRecord
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentStationSettingHeightTraverseBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class StationSettingHeightTraverseFragment : BaseStationSettingFragment() {


    private var _binding: FragmentStationSettingHeightTraverseBinding? = null

    val mBinding get() = _binding!!

    private val mainScope = MainScope()


    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding =
            FragmentStationSettingHeightTraverseBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mainScope.cancel()
        _binding = null

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
        initData()
        initListener()
    }

    private fun initListener() {

        //距离修正参数
        mBinding.ilTitle1.button1.setOnClickListener {
            //默认设置
            setDataFromRecord(null)
        }

        mBinding.ilTitle1.button2.setOnClickListener {
            //使用上次数据
            setDataFromLast()
        }

        //仪器高，棱镜高
        mBinding.ilInstrumentHeight.button1.setOnClickListener {
            mBinding.ilInstrumentHeight.etValueAfter.setText(
                getEtText(mBinding.ilInstrumentHeight.etValueBefore)
            )
        }
        mBinding.ilBackPrismHeight.button1.setOnClickListener {
            mBinding.ilBackPrismHeight.etValueAfter.setText(
                getEtText(mBinding.ilBackPrismHeight.etValueBefore)
            )
        }
        mBinding.ilFrontPrismHeight.button1.setOnClickListener {
            mBinding.ilFrontPrismHeight.etValueAfter.setText(
                getEtText(mBinding.ilFrontPrismHeight.etValueBefore)
            )
        }

    }

    private fun setDataFromLast() {
        mainScope.launch {
            val record = withContext(Dispatchers.IO) {
                val dryTemperature = getValueFromLastKey(ConfigConstants.KEY_LAST_DRY_TEMPERATURE)
                val humidityTemperature =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_HUMIDITY_TEMPERATURE)
                val airPressure = getValueFromLastKey(ConfigConstants.KEY_LAST_AIR_PRESSURE)
                val additionConstant =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_ADDITION_CONSTANT)
                val multiplicationConstant =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_MULTIPLICATION_CONSTANT)
                val instrumentHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_BEFORE)
                val instrumentHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_AFTER)
                val backPrismHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_BEFORE)
                val backPrismHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_AFTER)
                val frontPrismHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_BEFORE)
                val frontPrismHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_AFTER)

                if (dryTemperature == null
                    || humidityTemperature == null
                    || airPressure == null
                    || additionConstant == null
                    || multiplicationConstant == null
                    || instrumentHeightBefore == null
                    || instrumentHeightAfter == null
                    || backPrismHeightBefore == null
                    || backPrismHeightAfter == null
                    || frontPrismHeightBefore == null
                    || frontPrismHeightAfter == null
                ) {
                    return@withContext null
                }

                StationSettingHeightTraverseRecord.newInstance(
                    stationId(),
                    dryTemperature,
                    humidityTemperature,
                    airPressure,
                    additionConstant,
                    multiplicationConstant,
                    instrumentHeightBefore,
                    instrumentHeightAfter,
                    backPrismHeightBefore,
                    backPrismHeightAfter,
                    frontPrismHeightBefore,
                    frontPrismHeightAfter
                )
            }

            setDataFromRecord(record)
        }
    }

    private fun getValueFromLastKey(key: String): String? {
        val record =
            SurveyorDatabaseFactory.instance.configDataSource.getByKeySync(key)
        return record?.configValue
    }

    private fun initData() {

        mainScope.launch {
            val record = withContext(Dispatchers.IO) {
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .getRecordByStationId(stationId())
            }
            setDataFromRecord(record)
        }
    }

    private fun setDataFromRecord(record: StationSettingHeightTraverseRecord?) {
        mBinding.ilDryTemperature.etValue.setText(
            record?.dryTemperature ?: DEFAULT_TEMPERATURE
        )
        mBinding.ilHumidityTemperature.etValue.setText(
            record?.humidityTemperature ?: DEFAULT_TEMPERATURE
        )
        mBinding.ilAirPressure.etValue.setText(record?.airPressure ?: DEFAULT_AIR_PRESSURE)
        mBinding.ilAdditionConstant.etValue.setText(
            record?.additionConstant ?: DEFAULT_CONSTANT
        )
        mBinding.ilmultiplicationConstant.etValue.setText(
            record?.multiplicationConstant ?: DEFAULT_CONSTANT
        )

        mBinding.ilInstrumentHeight.etValueBefore.setText(
            record?.instrumentHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilInstrumentHeight.etValueAfter.setText(
            record?.instrumentHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilBackPrismHeight.etValueBefore.setText(
            record?.backPrismHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilBackPrismHeight.etValueAfter.setText(
            record?.backPrismHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilFrontPrismHeight.etValueBefore.setText(
            record?.frontPrismHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilFrontPrismHeight.etValueAfter.setText(
            record?.frontPrismHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
    }


    private fun initView() {
        mBinding.ilTitle1.title.text = Tools.getString(R.string.distance_correction_parameter)
        mBinding.ilTitle1.button1.text = Tools.getString(R.string.default_setting)
        mBinding.ilTitle1.button2.text = Tools.getString(R.string.button_use_last_station_settings)

        mBinding.ilTitle2.title.text = Tools.getString(R.string.instrument_height_prism_height_m)


        mBinding.ilDryTemperature.tvName.text = getString(R.string.dry_temperature)
        mBinding.ilDryTemperature.etValue.hint = getString(R.string.dry_temperature)
        mBinding.ilDryTemperature.tvUnit.text = "℃"

        mBinding.ilHumidityTemperature.tvName.text = getString(R.string.humidity_temperature)
        mBinding.ilHumidityTemperature.etValue.hint = getString(R.string.humidity_temperature)
        mBinding.ilHumidityTemperature.tvUnit.text = "℃"

        mBinding.ilAirPressure.tvName.text = getString(R.string.hint_air_pressure)
        mBinding.ilAirPressure.etValue.hint = getString(R.string.hint_air_pressure)
        mBinding.ilAirPressure.tvUnit.text = "hPa"

        mBinding.ilAdditionConstant.tvName.text = getString(R.string.addition_constant)
        mBinding.ilAdditionConstant.etValue.hint = getString(R.string.addition_constant)
        mBinding.ilAdditionConstant.tvUnit.text = "mm"

        mBinding.ilmultiplicationConstant.tvName.text = getString(R.string.multiplication_constant)
        mBinding.ilmultiplicationConstant.etValue.hint = getString(R.string.multiplication_constant)
        mBinding.ilmultiplicationConstant.tvUnit.text = "ppm"

        mBinding.ilInstrumentHeight.tvName.text = getString(R.string.instrument_height)
        mBinding.ilBackPrismHeight.tvName.text = getString(R.string.back_prism_height)
        mBinding.ilFrontPrismHeight.tvName.text = getString(R.string.front_prism_height)
    }

    companion object {

        @JvmField
        val TAG: String = StationSettingHeightTraverseFragment::class.java.name

        @JvmStatic
        fun newInstance(bundle: Bundle?): StationSettingHeightTraverseFragment {
            val fragment = StationSettingHeightTraverseFragment()
            fragment.arguments = bundle
            return fragment
        }

        @JvmStatic
        fun start(context: Context, surveyorStationId: String?) {
            val bundle = Bundle()
            bundle.putString(KEY_SURVEYOR_STATION_ID, surveyorStationId)
            FragmentContainerActivity.start(
                context,
                TAG,
                R.string.surveyor_station_edit,
                bundle
            )
        }
    }

    override fun clickConfirm() {

        mainScope.launch(Dispatchers.IO) {
            if (checkConfirmEmpty()) {
                return@launch
            }

            val stationId = stationId()

            val stationSettingHeightTraverseRecord =
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .getRecordByStationId(stationId)


            val dryTemperature = getEtText(mBinding.ilDryTemperature.etValue)
            val humidityTemperature = getEtText(mBinding.ilHumidityTemperature.etValue)
            val airPressure = getEtText(mBinding.ilAirPressure.etValue)
            val additionConstant = getEtText(mBinding.ilAdditionConstant.etValue)
            val multiplicationConstant = getEtText(mBinding.ilmultiplicationConstant.etValue)
            val instrumentHeightBefore = getEtText(mBinding.ilInstrumentHeight.etValueBefore)
            val instrumentHeightAfter = getEtText(mBinding.ilInstrumentHeight.etValueAfter)
            val backPrismHeightBefore = getEtText(mBinding.ilBackPrismHeight.etValueBefore)
            val backPrismHeightAfter = getEtText(mBinding.ilBackPrismHeight.etValueAfter)
            val frontPrismHeightBefore = getEtText(mBinding.ilFrontPrismHeight.etValueBefore)
            val frontPrismHeightAfter = getEtText(mBinding.ilFrontPrismHeight.etValueAfter)

            if (stationSettingHeightTraverseRecord == null) {
                val stationSettingHeightTraverseRecordNew =
                    StationSettingHeightTraverseRecord.newInstance(
                        stationId,
                        dryTemperature,
                        humidityTemperature,
                        airPressure,
                        additionConstant,
                        multiplicationConstant,
                        instrumentHeightBefore,
                        instrumentHeightAfter,
                        backPrismHeightBefore,
                        backPrismHeightAfter,
                        frontPrismHeightBefore,
                        frontPrismHeightAfter
                    )
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .saveRecord(stationSettingHeightTraverseRecordNew)
            } else {
                stationSettingHeightTraverseRecord.dryTemperature = dryTemperature
                stationSettingHeightTraverseRecord.humidityTemperature = humidityTemperature
                stationSettingHeightTraverseRecord.airPressure = airPressure
                stationSettingHeightTraverseRecord.additionConstant = additionConstant
                stationSettingHeightTraverseRecord.multiplicationConstant = multiplicationConstant
                stationSettingHeightTraverseRecord.instrumentHeightBefore = instrumentHeightBefore
                stationSettingHeightTraverseRecord.instrumentHeightAfter = instrumentHeightAfter
                stationSettingHeightTraverseRecord.backPrismHeightBefore = backPrismHeightBefore
                stationSettingHeightTraverseRecord.backPrismHeightAfter = backPrismHeightAfter
                stationSettingHeightTraverseRecord.frontPrismHeightBefore = frontPrismHeightBefore
                stationSettingHeightTraverseRecord.frontPrismHeightAfter = frontPrismHeightAfter
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .updateRecord(stationSettingHeightTraverseRecord)
            }

            val stationRecord =
                SurveyorDatabaseFactory.instance.surveyorStation.getRecordSync(stationId)
            if (stationRecord != null) {
                stationRecord.instrumentHeight = instrumentHeightBefore
                if (!stationRecord.items.isNullOrEmpty()) {
                    stationRecord.items.last().prismHeight = frontPrismHeightBefore
                    if (stationRecord.items.size > 2) {
                        stationRecord.items[1].prismHeight = backPrismHeightBefore
                    }
                }
            }
            SurveyorDatabaseFactory.instance.surveyorStation.update(stationRecord)

            saveLastData(
                dryTemperature,
                humidityTemperature,
                airPressure,
                additionConstant,
                multiplicationConstant,
                instrumentHeightBefore,
                instrumentHeightAfter,
                backPrismHeightBefore,
                backPrismHeightAfter,
                frontPrismHeightBefore,
                frontPrismHeightAfter
            )

            ToastUtils.showShort(R.string.save_success)
            requireActivity().finish()

        }

    }

    @WorkerThread
    private fun saveLastData(
        dryTemperature: String,
        humidityTemperature: String,
        airPressure: String,
        additionConstant: String,
        multiplicationConstant: String,
        instrumentHeightBefore: String,
        instrumentHeightAfter: String,
        backPrismHeightBefore: String,
        backPrismHeightAfter: String,
        frontPrismHeightBefore: String,
        frontPrismHeightAfter: String
    ) {
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_DRY_TEMPERATURE,
                dryTemperature
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_HUMIDITY_TEMPERATURE,
                humidityTemperature
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_AIR_PRESSURE,
                airPressure
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_ADDITION_CONSTANT,
                additionConstant
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_MULTIPLICATION_CONSTANT,
                multiplicationConstant
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_BEFORE,
                instrumentHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_AFTER,
                instrumentHeightAfter
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_BEFORE,
                backPrismHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_AFTER,
                backPrismHeightAfter
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_BEFORE,
                frontPrismHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_AFTER,
                frontPrismHeightAfter
            )
        )
    }


    private fun checkConfirmEmpty(): Boolean {
        if (editTextEmpty(mBinding.ilDryTemperature.etValue)) {
            ToastUtils.showShort(R.string.please_enter_dry_temperature)
            return true
        }
        if (editTextEmpty(mBinding.ilHumidityTemperature.etValue)) {
            ToastUtils.showShort(R.string.please_enter_humidity_temperature)
            return true
        }
        if (editTextEmpty(mBinding.ilAirPressure.etValue)) {
            ToastUtils.showShort(Tools.getString(R.string.please_enter_air_pressure))
            return true
        }
        if (editTextEmpty(mBinding.ilAdditionConstant.etValue)) {
            ToastUtils.showShort(R.string.please_enter_addition_constant)
            return true
        }
        if (editTextEmpty(mBinding.ilmultiplicationConstant.etValue)) {
            ToastUtils.showShort(R.string.please_enter_multiplication_constant)
            return true
        }

        if (editTextEmpty(mBinding.ilInstrumentHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_instrument_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilInstrumentHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_instrument_height_after)
            return true
        }
        if (editTextEmpty(mBinding.ilBackPrismHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_back_prism_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilBackPrismHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_back_prism_height_after)
            return true
        }
        if (editTextEmpty(mBinding.ilFrontPrismHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_front_prism_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilFrontPrismHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_front_prism_height_after)
            return true
        }
        return false
    }


    private fun editTextEmpty(et: EditText): Boolean {
        return TextUtils.isEmpty(getEtText(et))
    }

    private fun getEtText(et: EditText) = et.text.toString().trim()


}