package com.bingce.controlnetwork.fragment.stationsetting;

import android.content.Context;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.ProjectType;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.point.IPointDataSource;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.prism.IPrismManagerDataSource;
import com.bingce.controlapphelper.datasource.database.prism.PrismRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.util.HideKeyUtil;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.BaseAdapter;
import com.bingce.controlnetwork.adapter.SpinnerAdapter;
import com.bingce.utils.ThreadPoolUtil;
import com.rengwuxian.materialedittext.MaterialEditText;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 仪器棱镜设置
 */
public class StationSettingsFragment extends BaseStationSettingFragment {
    public static final String TAG = StationSettingsFragment.class.getName();
    private SurveyorStationRecord surveyorStationRecord;
    private _Adapter adapter = null;
    private List<PrismRecord> prismRecords;

    private StationSettingsFragment() {
        super();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull @NotNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_list_layout, container, false);
        RecyclerView recyclerView = view.findViewById(R.id.list);
        if (recyclerView != null && getArguments() != null) {
            String stationId = getArguments().getString(KEY_SURVEYOR_STATION_ID);
            ThreadPoolUtil.execute(() -> {
                surveyorStationRecord =
                        SurveyorDatabaseFactory.instance.getSurveyorStation()
                                .getRecordSync(stationId);
                if (surveyorStationRecord == null) {
                    return;
                }


                IConfigDataSource configDataSource =
                        SurveyorDatabaseFactory
                                .instance.getConfigDataSource();

                ConfigRecord temperature = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_TEMPERATURE);
                ConfigRecord humidity = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_HUMIDITY);
                ConfigRecord airPress = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_AIR_PRESSURE);

                List<_Item> items = new ArrayList<>();
                //添加环境title
                items.add(new _TitleWithButtonItem(getString(R.string.title_set_environment_info),
                        getString(R.string.button_use_default_station_settings),
                        getString(R.string.button_use_last_station_settings),
                        items));
                //添加温度
                items.add(new _1EditTextItem(getString(R.string.hint_temperature),
                        temperature(surveyorStationRecord, temperature, DEFAULT_TEMPERATURE),
                        getString(R.string.hint_temperature),
                        "℃",
                        _VH.ATTRIBUTE_TEMPERATURE,
                        surveyorStationRecord));
                //添加湿度
                items.add(new _1EditTextItem(getString(R.string.hint_humidity),
                        humidity(surveyorStationRecord, humidity, DEFAULT_HUMIDITY),
                        getString(R.string.hint_humidity),
                        "％",
                        _VH.ATTRIBUTE_HUMIDITY,
                        surveyorStationRecord));
                //添加气压
                items.add(new _1EditTextItem(getString(R.string.hint_air_pressure),
                        airPress(surveyorStationRecord, airPress, DEFAULT_AIR_PRESSURE),
                        getString(R.string.hint_air_pressure),
                        "hPa",
                        _VH.ATTRIBUTE_AIR_PRESSURE,
                        surveyorStationRecord));

                if (surveyorStationRecord.getTemperature() == null) {
                    surveyorStationRecord.setTemperature(DEFAULT_TEMPERATURE);
                }
                if (surveyorStationRecord.getHumidity() == null) {
                    surveyorStationRecord.setHumidity(DEFAULT_HUMIDITY);
                }
                if (surveyorStationRecord.getAirPressure() == null) {
                    surveyorStationRecord.setAirPressure(DEFAULT_AIR_PRESSURE);
                }

                //添加高度title
                items.add(new _TitleItem(getString(R.string.title_set_height_info)));
                //添加仪器高
                items.add(new _1EditTextItem(getString(R.string.instrument_height),
                        valueOf(surveyorStationRecord.getInstrumentHeight()),
                        getString(R.string.instrument_height),
                        "m",
                        _VH.ATTRIBUTE_INSTRUMENT_HEIGHT,
                        surveyorStationRecord));
                if (surveyorStationRecord.getInstrumentHeight() == null) {
                    surveyorStationRecord.setInstrumentHeight("0");
                }
                //是否是联系测量
                boolean isConnection = ProjectType.isConnection(surveyorStationRecord.projectType);

                //添加point title
                items.add(new _PointTitleItem(isConnection));
                //添加点坐标
                IPointDataSource pointDataSource = SurveyorDatabaseFactory.instance.getPointDataSource();
                List<SurveyorPoint> stationItems = surveyorStationRecord.getItems();

                IPrismManagerDataSource prismManagerDataSource = SurveyorDatabaseFactory.instance.getPrismManagerDataSource();
                prismRecords = prismManagerDataSource.getAllRecordSync();

                boolean stationHasRecord = StationUtil.hasRecord(surveyorStationRecord.getId());
                if (stationItems != null) {
                    for (SurveyorPoint surveyorPoint : stationItems) {
                        if (surveyorPoint == null || !SurveyorPoint.TYPE_POINT.equals(surveyorPoint.getType())) {
                            continue;
                        }
                        if (surveyorPoint.isWellSteel()) {
                            continue;
                        }

                        if (surveyorPoint.getPrismHeight() == null) {
                            surveyorPoint.setPrismHeight("0");
                        }

                        //当是测点的时候
                        PointRecord pointRecord = pointDataSource.findByIdSync(surveyorPoint.getOriginalPointId());
                        items.add(new _2EditTextItem(
                                pointRecord.getName(),
                                surveyorPoint,
                                prismRecords,
                                !stationHasRecord,
                                isConnection
                        ));
                    }
                }
                //刷新recycle view
                adapter = new _Adapter(items);
                runOnUI(() -> recyclerView.setAdapter(adapter));
            });
        }
        return view;
    }

    @Override
    protected void clickConfirm() {
        if (adapter != null) {
            String error = adapter.checkSave();
            if (error == null || error.isEmpty()) {//成功
                SurveyorDatabaseFactory
                        .instance
                        .getSurveyorStation()
                        .save(surveyorStationRecord);

                IConfigDataSource configDataSource =
                        SurveyorDatabaseFactory.instance.getConfigDataSource();
                configDataSource.set(ConfigConstants.KEY_LAST_TEMPERATURE, surveyorStationRecord.getTemperature());
                configDataSource.set(ConfigConstants.KEY_LAST_AIR_PRESSURE, surveyorStationRecord.getAirPressure());
                configDataSource.set(ConfigConstants.KEY_LAST_HUMIDITY, surveyorStationRecord.getHumidity());

                HideKeyUtil.hideSoftInput(getActivity());
                if (getActivity() != null) {
                    getActivity().finish();
                }
            } else {
                ToastUtils.showShort(error);
            }
        }
    }

    private abstract static class _Item {
        final int type;
        final String name;
        View itemView = null;

        _Item(int type, String name) {
            this.type = type;
            this.name = name;
        }

        void attach(View itemView) {
            this.itemView = itemView;
        }

        abstract String prepare2Save();
    }

    private static class _TitleItem extends _Item {
        _TitleItem(String name) {
            super(_VH.TYPE_TITLE, name);
        }

        @Override
        String prepare2Save() {
            return null;
        }
    }

    private static class _TitleWithButtonItem extends _Item {
        final String button1;
        final String button2;
        final List<_Item> items;

        _TitleWithButtonItem(String name, String button1, String button2, List<_Item> items) {
            super(_VH.TYPE_TITLE_WITH_2BUTTON, name);
            this.button1 = button1;
            this.button2 = button2;
            this.items = items;
        }

        @Override
        String prepare2Save() {
            return null;
        }
    }

    private static class _PointTitleItem extends _Item {
        private final boolean isConnection;//是否是联系测量

        _PointTitleItem(boolean isConnection) {
            super(_VH.TYPE_POINT_TITLE, null);
            this.isConnection = isConnection;
        }

        @Override
        String prepare2Save() {
            return null;
        }

        @Override
        void attach(View itemView) {
            super.attach(itemView);
            TextView tvAloneDistanceTitle = itemView.findViewById(R.id.tvAloneDistanceTitle);
            if (isConnection) {
                tvAloneDistanceTitle.setVisibility(View.VISIBLE);
            } else {
                tvAloneDistanceTitle.setVisibility(View.INVISIBLE);
            }
        }
    }

    private static class _1EditTextItem extends _Item {
        private final String value;
        private final String hintString;
        private final String unit;
        private final SurveyorStationRecord stationRecord;
        private final int attributeType;

        _1EditTextItem(String name, String value,
                       String hintString, String unit,
                       int attributeType,
                       SurveyorStationRecord stationRecord) {
            super(_VH.TYPE_1_EDIT_TEXT, name);
            this.value = value;
            this.hintString = hintString;
            this.unit = unit;
            this.attributeType = attributeType;
            this.stationRecord = stationRecord;
        }

        @Override
        void attach(View itemView) {
            super.attach(itemView);
            MaterialEditText value = itemView.findViewById(R.id.value);
            if (value != null) {
                Watcher watcher = new Watcher() {
                    @Override
                    void onChanged(String string) {
                        if (_VH.ATTRIBUTE_TEMPERATURE == attributeType) {
                            stationRecord.setTemperature(string);
                        } else if (_VH.ATTRIBUTE_HUMIDITY == attributeType) {
                            stationRecord.setHumidity(string);
                        } else if (_VH.ATTRIBUTE_AIR_PRESSURE == attributeType) {
                            stationRecord.setAirPressure(string);
                        } else if (_VH.ATTRIBUTE_INSTRUMENT_HEIGHT == attributeType) {
                            stationRecord.setInstrumentHeight(string);
                        }
                    }
                };
                value.addTextChangedListener(watcher);
            }
        }

        @Override
        String prepare2Save() {
            if (_VH.ATTRIBUTE_TEMPERATURE == attributeType) {
                if (!isEmptyString(stationRecord.getTemperature())) {
                    return null;
                }
            } else if (_VH.ATTRIBUTE_HUMIDITY == attributeType) {
                if (!isEmptyString(stationRecord.getHumidity())) {
                    return null;
                }
            } else if (_VH.ATTRIBUTE_AIR_PRESSURE == attributeType) {
                if (!isEmptyString(stationRecord.getAirPressure())) {
                    return null;
                }
            } else if (_VH.ATTRIBUTE_INSTRUMENT_HEIGHT == attributeType) {
                if (!isEmptyString(stationRecord.getInstrumentHeight())) {
                    return null;
                }
            }
            return Tools.getString(R.string.please_enter) + name;
        }
    }

    private static class _2EditTextItem extends _Item {
        private final SurveyorPoint surveyorPoint;
        private final List<PrismRecord> prismRecords;
        private final boolean isCanSelectAloneDistance;
        private final boolean isConnection;//是否是联系测量
        private final List<String> prismTypes;

        _2EditTextItem(String pointName,
                       SurveyorPoint surveyorPoint,
                       List<PrismRecord> prismRecords,
                       boolean isCanSelectAloneDistance,
                       boolean isConnection) {
            super(_VH.TYPE_2_EDIT_TEXT, pointName);
            this.surveyorPoint = surveyorPoint;
            this.prismRecords = prismRecords;
            this.isCanSelectAloneDistance = isCanSelectAloneDistance;
            this.isConnection = isConnection;
            prismTypes = new ArrayList<>();
            for (PrismRecord prismRecord : prismRecords) {
                prismTypes.add(prismRecord.getShowPrismTypeName());
            }
        }

        @Override
        void attach(View itemView) {
            super.attach(itemView);
            MaterialEditText value1 = itemView.findViewById(R.id.value1);
            if (value1 != null) {
                value1.addTextChangedListener(new Watcher() {
                    @Override
                    void onChanged(String string) {
                        surveyorPoint.setPrismHeight(string);
                    }
                });
            }

            Spinner spinner = itemView.findViewById(R.id.spinner);
            if (spinner != null) {
                spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                    @Override
                    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                        PrismRecord prismRecord = prismRecords.get(position);
                        surveyorPoint.setPrismType(prismRecord.prismType);
                        surveyorPoint.setPrismTypeConstant(prismRecord.prismTypeConstant);
                        surveyorPoint.setPrismTypeName(prismRecord.getShowPrismTypeName());
                    }

                    @Override
                    public void onNothingSelected(AdapterView<?> parent) {

                    }
                });
            }

            CheckBox checkBox = itemView.findViewById(R.id.checkBox);
            if (checkBox != null) {
                if (isConnection) {
                    checkBox.setVisibility(View.VISIBLE);
                    checkBox.setEnabled(isCanSelectAloneDistance);
                    checkBox.setOnCheckedChangeListener((compoundButton, checked) -> {
                        surveyorPoint.setAloneDistance(checked);
                    });
                } else {
                    checkBox.setVisibility(View.INVISIBLE);
                }
            }

        }

        @Override
        String prepare2Save() {
            if (!isEmptyString(surveyorPoint.getPrismHeight())) {
                return null;
            }
            return Tools.getString(R.string.please_enter) + name + Tools.getString(R.string.prism_height);
        }
    }

    private static abstract class _VH extends RecyclerView.ViewHolder {
        static final int TYPE_TITLE = 0;
        static final int TYPE_TITLE_WITH_2BUTTON = TYPE_TITLE + 1;
        static final int TYPE_POINT_TITLE = TYPE_TITLE_WITH_2BUTTON + 1;
        static final int TYPE_1_EDIT_TEXT = TYPE_POINT_TITLE + 1;
        static final int TYPE_2_EDIT_TEXT = TYPE_1_EDIT_TEXT + 1;

        static final int ATTRIBUTE_TEMPERATURE = 0;//温度
        static final int ATTRIBUTE_HUMIDITY = ATTRIBUTE_TEMPERATURE + 1;//湿度
        static final int ATTRIBUTE_AIR_PRESSURE = ATTRIBUTE_HUMIDITY + 1;//气压
        static final int ATTRIBUTE_INSTRUMENT_HEIGHT = ATTRIBUTE_AIR_PRESSURE + 1;//仪器高

        public _VH(@NonNull @NotNull View itemView) {
            super(itemView);
        }

        abstract void setup(_Item item);
    }

    private static class _TitleVH extends _VH {
        private final TextView titleText;

        public _TitleVH(@NonNull @NotNull View itemView) {
            super(itemView);
            titleText = itemView.findViewById(R.id.title);
        }

        @Override
        void setup(_Item item) {
            if (titleText != null) {
                titleText.setText(item.name);
            }
        }
    }

    private static class _TitleWith2ButtonVH extends _TitleVH {
        final TextView button1;
        final TextView button2;

        public _TitleWith2ButtonVH(@NonNull @NotNull View itemView) {
            super(itemView);
            button1 = itemView.findViewById(R.id.button1);
            button2 = itemView.findViewById(R.id.button2);
        }

        @Override
        void setup(_Item item) {
            super.setup(item);
            if (item instanceof _TitleWithButtonItem) {
                _TitleWithButtonItem titleWithButtonItem = (_TitleWithButtonItem) item;
                if (button1 != null) {
                    button1.setText(titleWithButtonItem.button1);
                    button1.setOnClickListener(v -> update(titleWithButtonItem.items,
                            DEFAULT_TEMPERATURE, DEFAULT_HUMIDITY, DEFAULT_AIR_PRESSURE));
                }
                if (button2 != null) {
                    button2.setText(titleWithButtonItem.button2);
                    //判断能否点击
                    ThreadPoolUtil.execute(() -> {
                        IConfigDataSource configDataSource =
                                SurveyorDatabaseFactory
                                        .instance.getConfigDataSource();

                        ConfigRecord temperature = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_TEMPERATURE);
                        ConfigRecord humidity = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_HUMIDITY);
                        ConfigRecord airPress = configDataSource.getByKeySync(ConfigConstants.KEY_LAST_AIR_PRESSURE);
                        if (temperature != null && humidity != null && airPress != null) {
                            ThreadPoolUtil.executeInMain(() -> {
                                button2.setEnabled(true);
                                button2.setAlpha(1);
                                button2.setOnClickListener(v -> update(titleWithButtonItem.items,
                                        temperature.getConfigValue(),
                                        humidity.getConfigValue(),
                                        airPress.getConfigValue()));
                            });
                        }
                    });
                }
            }
        }

        private void update(List<_Item> items, String temperature, String humidity, String airPress) {
            for (int index = 0; index < items.size(); index++) {
                _Item item = items.get(index);
                if (!(item instanceof _1EditTextItem)) {
                    continue;
                }
                _1EditTextItem editTextItem = (_1EditTextItem) item;
                MaterialEditText value = item.itemView.findViewById(R.id.value);
                if (editTextItem.attributeType == _VH.ATTRIBUTE_TEMPERATURE) {//温度
                    if (value != null) {
                        value.setText(temperature);
                    }
                } else if (editTextItem.attributeType == _VH.ATTRIBUTE_AIR_PRESSURE) {//气压
                    if (value != null) {
                        value.setText(airPress);
                    }
                } else if (editTextItem.attributeType == _VH.ATTRIBUTE_HUMIDITY) {//湿度
                    if (value != null) {
                        value.setText(humidity);
                    }
                }
            }
        }
    }

    private static class _PointTitleVH extends _VH {
        public _PointTitleVH(@NonNull @NotNull View itemView) {
            super(itemView);
        }

        @Override
        void setup(_Item item) {
        }
    }

    private static class _1EditTextVH extends _VH {
        private final TextView name;
        private final MaterialEditText value;
        private final TextView unit;

        public _1EditTextVH(@NonNull @NotNull View itemView) {
            super(itemView);
            name = itemView.findViewById(R.id.name);
            value = itemView.findViewById(R.id.value);
            unit = itemView.findViewById(R.id.unit);
        }

        @Override
        void setup(_Item item) {
            if (item instanceof _1EditTextItem) {
                _1EditTextItem editTextItem = (_1EditTextItem) item;
                if (name != null) {
                    name.setText(editTextItem.name);
                }
                String v1 = null;
                if (editTextItem.attributeType == _VH.ATTRIBUTE_INSTRUMENT_HEIGHT) {
                    v1 = editTextItem.stationRecord.getInstrumentHeight();
                } else if (editTextItem.attributeType == _VH.ATTRIBUTE_TEMPERATURE) {//温度
                    v1 = editTextItem.stationRecord.getTemperature();
                } else if (editTextItem.attributeType == _VH.ATTRIBUTE_AIR_PRESSURE) {//气压
                    v1 = editTextItem.stationRecord.getAirPressure();
                } else if (editTextItem.attributeType == _VH.ATTRIBUTE_HUMIDITY) {//湿度
                    v1 = editTextItem.stationRecord.getHumidity();
                }


                if (value != null) {
                    value.setText(v1);
                    value.setHint(editTextItem.hintString);
                }
                if (unit != null) {
                    unit.setText(editTextItem.unit);
                }
            }
        }
    }

    private static class _2EditTextVH extends _VH {
        private final TextView tvPointName;
        private final MaterialEditText etPrismHeight;
        private final Spinner spinnerPrismType;
        private final CheckBox checkBoxAloneDistance;

        public _2EditTextVH(@NonNull @NotNull View itemView) {
            super(itemView);
            tvPointName = itemView.findViewById(R.id.name);
            etPrismHeight = itemView.findViewById(R.id.value1);
            spinnerPrismType = itemView.findViewById(R.id.spinner);
            checkBoxAloneDistance = itemView.findViewById(R.id.checkBox);
        }

        @Override
        void setup(_Item item) {
            if (item instanceof _2EditTextItem) {
                _2EditTextItem editTextItem = (_2EditTextItem) item;
                if (tvPointName != null) {
                    tvPointName.setText(editTextItem.name);
                }

                if (etPrismHeight != null) {
                    etPrismHeight.setText(editTextItem.surveyorPoint.getPrismHeight());
                }

                if (spinnerPrismType != null) {
                    SpinnerAdapter spinnerAdapter = new SpinnerAdapter(itemView.getContext(), editTextItem.prismTypes);
                    spinnerPrismType.setAdapter(spinnerAdapter);
                    spinnerPrismType.setSelection(editTextItem.prismTypes.indexOf(editTextItem.surveyorPoint.getPrismTypeName()));
                }

                if (checkBoxAloneDistance != null) {
                    checkBoxAloneDistance.setChecked(editTextItem.surveyorPoint.isAloneDistance());
                }

            }
        }
    }

    private static class _Adapter extends BaseAdapter<_Item, _VH> {

        public _Adapter(List<_Item> items) {
            super(items);
        }

        @Override
        public int getItemViewType(int position) {
            _Item item = getItem(position);
            return item.type;
        }

        @Override
        protected _VH createVH(@NotNull ViewGroup parent, int viewType) {
            if (_VH.TYPE_TITLE == viewType) {
                View itemView = LayoutInflater
                        .from(parent.getContext())
                        .inflate(R.layout.item_surveyor_station_settings_title_layout, parent, false);
                return new _TitleVH(itemView);
            } else if (_VH.TYPE_TITLE_WITH_2BUTTON == viewType) {
                View itemView = LayoutInflater
                        .from(parent.getContext())
                        .inflate(R.layout.item_surveyor_station_settings_title_with_2button_layout, parent, false);
                return new _TitleWith2ButtonVH(itemView);
            } else if (_VH.TYPE_POINT_TITLE == viewType) {
                View itemView = LayoutInflater
                        .from(parent.getContext())
                        .inflate(R.layout.item_surveyor_station_settings_point_title_layout, parent, false);
                return new _PointTitleVH(itemView);
            } else if (_VH.TYPE_1_EDIT_TEXT == viewType) {
                View itemView = LayoutInflater
                        .from(parent.getContext())
                        .inflate(R.layout.item_surveyor_station_settings_1_edittext_layout, parent, false);
                return new _1EditTextVH(itemView);
            } else if (_VH.TYPE_2_EDIT_TEXT == viewType) {
                View itemView = LayoutInflater
                        .from(parent.getContext())
                        .inflate(R.layout.item_surveyor_station_settings_2_edittext_layout, parent, false);
                return new _2EditTextVH(itemView);
            }
            return null;
        }

        @Override
        public void onBindViewHolder(@NonNull @NotNull StationSettingsFragment._VH holder, int position) {
            _Item item = getItem(position);
            item.attach(holder.itemView);
            holder.setup(item);
        }

        /**
         * 保存前的检查
         *
         * @return
         */
        String checkSave() {
            for (int index = 0; index < getItemCount(); index++) {
                _Item item = getItem(index);
                String error = item.prepare2Save();
                if (error != null && !error.isEmpty()) {
                    return error;
                }
            }
            return null;
        }
    }

    private static String getString(MaterialEditText editText) {
        if (editText == null) {
            return "";
        }
        if (editText.getText() == null) {
            return "";
        }
        return editText.getText().toString();
    }

    private String valueOf(String value) {
        if (value == null) {
            return "";
        }
        return value;
    }

    public static StationSettingsFragment newInstance(Bundle bundle) {
        StationSettingsFragment fragment = new StationSettingsFragment();
        fragment.setArguments(bundle);
        return fragment;
    }

    public static void start(Context context, String surveyorStationId) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_SURVEYOR_STATION_ID, surveyorStationId);
        FragmentContainerActivity.start(context, TAG, R.string.title_instrument_prism_setting, bundle);
    }

    private String temperature(SurveyorStationRecord surveyorStationRecord,
                               ConfigRecord lastRecord,
                               String defaultRecord) {
        if (surveyorStationRecord != null && !valueOf(surveyorStationRecord.temperature).isEmpty()) {
            return surveyorStationRecord.temperature;
        }

        return configValue(lastRecord, defaultRecord);
    }

    private String humidity(SurveyorStationRecord surveyorStationRecord,
                            ConfigRecord lastRecord,
                            String defaultRecord) {
        if (surveyorStationRecord != null && !valueOf(surveyorStationRecord.humidity).isEmpty()) {
            return surveyorStationRecord.humidity;
        }

        return configValue(lastRecord, defaultRecord);
    }

    private String airPress(SurveyorStationRecord surveyorStationRecord,
                            ConfigRecord lastRecord,
                            String defaultRecord) {
        if (surveyorStationRecord != null && !valueOf(surveyorStationRecord.airPressure).isEmpty()) {
            return surveyorStationRecord.airPressure;
        }

        return configValue(lastRecord, defaultRecord);
    }

    private String configValue(ConfigRecord lastRecord, String defaultRecord) {
        if (lastRecord != null && !valueOf(lastRecord.getConfigValue()).isEmpty()) {
            return lastRecord.getConfigValue();
        }
        return defaultRecord;
    }

    private abstract static class Watcher implements TextWatcher {
        @Override
        final public void beforeTextChanged(CharSequence s, int start, int count, int after) {

        }

        @Override
        final public void onTextChanged(CharSequence s, int start, int before, int count) {

        }

        @Override
        final public void afterTextChanged(Editable s) {
            onChanged(s.toString());
        }

        abstract void onChanged(String string);
    }

    private static boolean isEmptyString(String string) {
        return string == null || string.isEmpty();
    }


}
