package com.bingce.controlnetwork.fragment.stationsurvey;

import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.newui.stationsetting.StationSettingsActivity;

import java.util.List;

public class MethodCheckSetting {
    /**
     * 检测测站是否设置了棱镜相关属性，如果没有则跳转到设置界面
     */
    @WorkerThread
    public static void checkStationSettings(String stationId,
                                            @NonNull Fragment fragment,
                                            @NonNull IUIRunner runner) {
        ISurveyorStationDataSource stationDataSource =
                SurveyorDatabaseFactory.instance.getSurveyorStation();
        //检测温度、湿度、气压、棱镜是否设置
        SurveyorStationRecord stationRecord = stationDataSource.getRecordSync(stationId);
        if (stationRecord == null) {
            return;
        }
        List<SurveyorPoint> stationItemList = stationRecord.getItems();
        if (stationItemList == null || stationItemList.isEmpty()) {
            return;
        }
        boolean failure = false;
        for (SurveyorPoint stationItem : stationItemList) {
            if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                continue;
            }
            if (stationItem.isWellSteel()) {
                continue;
            }
            if (isEmpty(stationItem.getPrismHeight())) {
                failure = true;
                break;
            }
        }
        if (!failure && (isEmpty(stationRecord.getTemperature()) ||
                isEmpty(stationRecord.getHumidity()) ||
                isEmpty(stationRecord.getAirPressure()) ||
                isEmpty(stationRecord.getInstrumentHeight()))) {
            failure = true;
        }

        if (!failure) {
            TipsWith2OptionFragment.dismissNowMe();
            return;
        }

        runner.runOnUI(() -> {
            TipsWith2OptionFragment.Argument argument = new TipsWith2OptionFragment.Argument();
            argument.content(Tools.getString(R.string.please_set_environment));
            argument.cancelable(false);
            argument.autoDismiss(false);

            TipsWith2OptionFragment.tips(
                    fragment.getChildFragmentManager(),
                    fragment.getViewLifecycleOwner(),
                    argument,
                    new TipsWith2OptionFragment.SimpleCallback() {
                        @Override
                        public void onConfirm() {
                            StationSettingsActivity.start(fragment.requireContext(), stationId);
//                            StationSettingsFragment.start(fragment.getContext(), stationId);
                        }

                        @Override
                        public void onCancel() {
                            if (fragment.getActivity() != null) {
                                fragment.getActivity().finish();
                            }
                        }
                    });
        });
    }

    public static boolean isEmpty(String string) {
        return string == null || string.isEmpty();
    }
}
