package com.bingce.controlnetwork.fragment.stationsurvey;

import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.AutoSurveyorButtonVisibleEvent;
import com.bingce.controlapphelper.event.RefreshGoBackButton;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.surveyor.test.Test;
import com.bingce.controlapphelper.util.DigitalUtil;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.BottomSheetSelectListFragment;
import com.bingce.controlnetwork.fragment.ComponentStationSurveyorDataListFragment;
import com.bingce.controlnetwork.fragment.SurveyorSettingsFragment;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.newui.stationsetting.StationSettingsActivity;
import com.bingce.controlnetwork.surveyor.surveyor.TestDataSelectFragment;
import com.bingce.utils.ThreadPoolUtil;
import com.github.razir.progressbutton.DrawableButtonExtensionsKt;
import com.github.razir.progressbutton.ProgressButtonHolderKt;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;

/**
 * 测站测量页面（开始测量）
 */
public class StationSurveyorFragment extends BaseFragment {
    public static final String TAG = StationSurveyorFragment.class.getName();
    private ComponentStationSurveyorDataListFragment fragment;
    private MenuItem goBack = null;//数据回退菜单
    private SurveyorStationRecord surveyorStationRecord;
    private int cycleIndex = 0;

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onRefreshGoBack(RefreshGoBackButton event) {
        if (goBack != null) {
            goBack.setEnabled(event.validate);
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onSurveyorStateChanged(SurveyStateEvent event) {
        if (getView() == null) {
            return;
        }
        Button surveyorButton = getView().findViewById(R.id.surveyor);
        if (surveyorButton == null) {
            return;
        }
        if (event.isIdle() || event.isSuccess() || event.isCompleted()) {
            DrawableButtonExtensionsKt.hideProgress(surveyorButton, R.string.start_surveyor);
        } else if (event.isSurveying()) {
            DrawableButtonExtensionsKt.showProgress(surveyorButton, progressParams -> {
                progressParams.setButtonTextRes(R.string.tips_surveying);
                progressParams.setProgressColor(Color.WHITE);
                return null;
            });
        } else if (event.isFailure()) {
            DrawableButtonExtensionsKt.hideProgress(surveyorButton, R.string.tips_surveyor_failure_and_retry);
        }

        boolean enable = event.isIdle() || event.isFailure() || event.isSuccess();
        setSurveyorEnable(getView(), R.id.surveyor, enable);
        setSurveyorEnable(getView(), R.id.auto_surveyor, enable);
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onAutoSurveyorChanged(AutoSurveyorButtonVisibleEvent event) {
        if (getView() == null) {
            return;
        }
        Button autoSurveyButton = getView().findViewById(R.id.auto_surveyor);
        if (autoSurveyButton != null) {
            autoSurveyButton.setVisibility(event.isReadyAutoSurvey() && !event.isOpenAutoSurvey()
                    ? View.VISIBLE : View.GONE);
        }
    }

    private String stationId() {
        if (getArguments() == null) {
            return null;
        }
        return getArguments().getString(BundleConstants.KEY_SURVEYOR_STATION_ID);
    }

    @Override
    public void onResume() {
        super.onResume();
        //测量界面保持常亮
        if (getActivity() != null) {
            getActivity().getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        //关闭测量界面保持常亮
        if (getActivity() != null) {
            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        EventBus.getDefault().unregister(this);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater,
                             @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        EventBus.getDefault().register(this);

        View view = inflater.inflate(R.layout.fragment_project_surveyor_station_detail_layout,
                container,
                false);

        replaceFragment();

        //2.设置测量按钮
        Button surveyor = view.findViewById(R.id.surveyor);
        if (surveyor != null) {
            ProgressButtonHolderKt.bindProgressButton(getViewLifecycleOwner(), surveyor);
        }
        setSurveyorClick(view, R.id.surveyor, () -> {
            if (fragment != null) {
                fragment.doSurveyor();
            }
        });
        setSurveyorClick(view, R.id.auto_surveyor, () -> {
            if (fragment != null) {
                fragment.autoSurvey();
            }
        });
        //3.查询测站信息
        ThreadPoolUtil.execute(() -> {
            surveyorStationRecord = SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getRecordSync(stationId());
            //监听当前测回切换
            initObserver(view);
            //监听数据变化
            ToleranceDetailRecord toleranceRecord = getTolerance(surveyorStationRecord);
            if (toleranceRecord == null) {
                return;
            }
            //注册UI监听
            View pick = view.findViewById(R.id.cycleLayout);
            if (pick == null) {
                return;
            }
            int cycleCount = toleranceRecord.getCycleCount();
            pick.setOnClickListener(v -> {
                ArrayList<String> cycleList = SurveyorCycleUtil.cycleStrings(cycleCount,
                        surveyorStationRecord != null && surveyorStationRecord.leftAndRightSurveyor);

                TextView circleText = view.findViewById(R.id.circle_text);
                runOnUI(() -> BottomSheetSelectListFragment.showSelect(
                        getChildFragmentManager(),
                        getViewLifecycleOwner(),
                        cycleIndex,
                        cycleList,
                        null,//不再限制测期
                        (index, string) -> {
                            if (circleText != null) {
                                circleText.setText(string);
                            }
                            cycleIndex = index;
                            if (fragment != null) {
                                //检测当前测回是否完成
                                fragment.switchCycle(cycleIndex);
                            }
                        }));
            });
        });


        setHasOptionsMenu(true);

        return view;
    }

    private void replaceFragment() {
        //1.添加测量数据显示fragment
        fragment = ComponentStationSurveyorDataListFragment
                .newInstance(stationId());
        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.fragment_container, fragment, "___station_list")
                .commit();
    }

    private void initObserver(View view) {
        runOnUI(() -> {
            monitorCurrentCycle(view);
            //监听测站变化
            SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getLiveDataRecord(stationId())
                    .observe(getViewLifecycleOwner(),
                            stationRecord -> {
                                replaceFragment();
                                checkStationSettings();
                            });

        });
    }

    private void checkStationSettings() {
        ThreadPoolUtil.execute(() -> MethodCheckSetting.checkStationSettings(stationId(), fragment, this::runOnUI));
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);


    }

    private void monitorCurrentCycle(View view) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .findSchedule(SurveyorScheduleConstants.getCurrentCycleKey(stationId(), null))
                .observe(getViewLifecycleOwner(), record -> {
                    TextView circleText = view.findViewById(R.id.circle_text);
                    //测回发生变化的时候，更新相应ui
                    int cycleIndex = DigitalUtil.valueOfIntString(record == null ? "0" : record.getStatue(), 0);
                    //更新测回名称
                    if (circleText != null) {
                        boolean showLeftOrRight = true;
                        if (surveyorStationRecord != null) {
                            showLeftOrRight = surveyorStationRecord.leftAndRightSurveyor;
                        }
                        circleText.setText(SurveyorCycleUtil.cycleName(cycleIndex, showLeftOrRight));
                    }
                    StationSurveyorFragment.this.cycleIndex = cycleIndex;
                });
    }

    private static void setSurveyorClick(View rootView, int id, Runnable runnable) {
        View view = rootView.findViewById(id);
        if (view != null) {
            view.setOnClickListener(v -> runnable.run());
        }
    }

    private static void setSurveyorEnable(View rootView, int id, boolean enable) {
        if (rootView == null) {
            return;
        }
        View view = rootView.findViewById(id);
        if (view != null) {
            view.setEnabled(enable);
            if (enable) {
                view.setAlpha(1);
            } else {
                view.setAlpha(.4f);
            }
        }
    }

    @Override
    public void onCreateOptionsMenu(@NonNull @NotNull Menu menu, @NonNull @NotNull MenuInflater inflater) {
        if (Test.DEBUG) {
            inflater.inflate(R.menu.menu_surveyor_station_record_data_edit_debug_pla, menu);
        } else {
            inflater.inflate(R.menu.menu_surveyor_station_record_data_edit_release_pla, menu);
        }
//        MenuItem stationCheck = menu.findItem(R.id.menu_item_station_check);
//        MenuItem stationAdjust = menu.findItem(R.id.menu_item_station_adjust);
        goBack = menu.findItem(R.id.menu_item_go_back);
//        SurveyorDatabaseFactory
//                .instance
//                .getScheduleDataSource()
//                .findSchedule(SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), null))
//                .observe(getViewLifecycleOwner(), surveyorScheduleRecord -> {
//                    boolean checkStatue = false;
//                    boolean adjustStatue = false;
//                    if (surveyorScheduleRecord != null) {
//                        String statue = surveyorScheduleRecord.getStatue();
//                        if (SurveyorScheduleConstants.STATUE_VALUE_CHECKED.equals(statue)) {
//                            adjustStatue = true;
//                        } else if (SurveyorScheduleConstants.STATUE_VALUE_COMPLETED.equals(statue)) {
//                            checkStatue = true;
//                        } else if (SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED.equals(statue)) {
//                            //重置提醒标识
//                            String key = ConfigConstants.KEY_CONFIG_STATION_CHECK_REMINDED + stationId();
//                            SurveyorDatabaseFactory
//                                    .instance
//                                    .getConfigDataSource()
//                                    .set(key, ConfigConstants.VALUE_NO);
//                        }
//                    }
//                    //更新菜单enable
//                    if (stationCheck != null) {
//                        stationCheck.setEnabled(checkStatue);
//                    }
//                    if (stationAdjust != null) {
//                        stationAdjust.setEnabled(adjustStatue);
//                    }
//                });
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        if (R.id.menu_item_delete_current_records == item.getItemId()) {
            //删除当前测回记录
            if (surveyorStationRecord != null) {
                //通知fragment
                fragment.deleteByStationAndPeriod(cycleIndex);
            }
        } else if (R.id.menu_item_delete_all_records == item.getItemId()) {
            //删除所有测回记录
            if (surveyorStationRecord != null) {
                //通知fragment
                fragment.deleteCurrentStationSurveyorData();
            }
        } else if (R.id.menu_item_station_check == item.getItemId()) {
            //测站检测，检测所有测回
            fragment.stationCheck();
        } else if (R.id.menu_item_data_source == item.getItemId()) {
            //数据源 debug使用
            TestDataSelectFragment.show(getChildFragmentManager());
        } else if (R.id.menu_item_go_back == item.getItemId()) {
            //回退
            fragment.goBack();
        } else if (R.id.menu_item_station_environment_prism_settings == item.getItemId()) {
            //环境、棱镜设置
            StationSettingsActivity.start(requireContext(), stationId());
//            StationSettingsFragment.start(getContext(), stationId());
        } else if (R.id.menu_item_surveyor_settings == item.getItemId()) {
            //测量设置
            ThreadPoolUtil.execute(() -> {
                String name = StationUtil.getStationName(surveyorStationRecord);
                runOnUI(() -> SurveyorSettingsFragment.start(getContext(), surveyorStationRecord.getId(), name));
            });
        } else if (R.id.menu_item_station_adjust == item.getItemId()) {
            //测站平差，平差检测
            fragment.stationAdjust();
        } else if (R.id.menu_item_check_adjust == item.getItemId()) {
            //检测平差
            fragment.stationCheckAdjust();
        }
        return super.onOptionsItemSelected(item);
    }

    @WorkerThread
    private static ToleranceDetailRecord getTolerance(SurveyorStationRecord surveyorStationRecord) {
        if (surveyorStationRecord == null) {
            return null;
        }
        //获取project信息
        ProjectRecord projectRecord = SurveyorDatabaseFactory
                .instance
                .getProjectDataSource()
                .getRecord(surveyorStationRecord.getProjectId());
        if (projectRecord == null) {
            return null;
        }
        //获取限差数据
        return SurveyorDatabaseFactory
                .instance
                .getToleranceDetailDataSource()
                .getRecordSync(projectRecord.getToleranceId());
    }

    public static void start(Context context, String surveyorStationId, String surveyorStationName) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_ID, surveyorStationId);
        FragmentContainerActivity.start(context, TAG, surveyorStationName, bundle);
    }

    public static Fragment newInstance(Bundle args) {
        Fragment fragment = new StationSurveyorFragment();
        fragment.setArguments(args);
        return fragment;
    }
}
