package com.bingce.controlnetwork.fragment.stationsurvey.angledistance;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.RefreshGoBackButtonAloneDistance;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.IDistanceTypeChangeListener;
import com.bingce.controlapphelper.surveyor.data.IMeasureCache;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.data.SurveyorData;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.util.DigitalUtil;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlnetwork.adapter.MeasureAngleDistanceAdapter;
import com.bingce.controlnetwork.fragment.ComponentStationSurveyorDataListFragment;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.IMeasureOperate;
import com.bingce.controlnetwork.surveyor.state.IScrollDelegate;
import com.bingce.totalstation.TotalStation;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.MeasureAngleDistanceAdapter;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.ComponentStationSurveyorDataListFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.IMeasureOperate;
import com.bingce.controlnetwork.surveyor.state.IScrollDelegate;
import com.bingce.controlnetwork.surveyor.state.alonedistance.StateManagerAloneDistance;
import com.bingce.controlnetwork.surveyor.state.alonedistance._CheckStationState;
import com.bingce.controlnetwork.surveyor.state.alonedistance._DoCalAdjustState;
import com.bingce.controlnetwork.surveyor.state.alonedistance._IsSingleCycleCompletedState;

/**
 * 单独测距
 */
public class MeasureDistanceFragment extends BaseFragment
        implements IDistanceTypeChangeListener, ISurveyState, IMeasureOperate {

    private MeasureAngleDistanceAdapter adapter = null;
    private StateManagerAloneDistance stateManager; //这里的stateManager是克隆StateManager
    private RecyclerView recyclerView;
    private IMeasureCache cachedData = null;
    private ISurveyorData surveyedData = null;

    private String stationId() {
        if (getArguments() == null) {
            return null;
        }
        return getArguments().getString(KEY_STATION_ID);
    }

    private String getWellType() {
        if (getArguments() != null) {
            return getArguments().getString(KEY_TYPE_ANGLE_OR_DISTANCE);
        }
        return null;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater,
                             @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(
                R.layout.fragment_project_surveyor_station_point_list_layout,
                container,
                false);
        recyclerView = view.findViewById(R.id.recycler_view);


        collectData(stationId(),
                getContext(),
                getChildFragmentManager(),
                getViewLifecycleOwner(),
                this::runOnUI,
                index -> {
                    scrollToPosition(index);
                },
                new ICollectCacheCallback() {
                    @Override
                    public void onFailure() {
                    }

                    @Override
                    public void onSuccess(boolean showSlopDistance, IMeasureCache cachedData, SurveyorData surveyorData, StateManagerAloneDistance stateManager) {
                        onCollectStationData(showSlopDistance, cachedData, surveyorData, stateManager);
                    }
                }
        );

        return view;
    }

    @WorkerThread
    private void onCollectStationData(
            boolean showSlopDistance,
            IMeasureCache cachedData,
            SurveyorData surveyorData,
            StateManagerAloneDistance stateManager) {
        this.stateManager = stateManager;
        this.cachedData = cachedData;
        this.surveyedData = surveyorData;

        //测量数据收集完毕后，就可以更新UI了
        runOnUI(() -> {
            //刷新UI
            if (adapter == null) {
                adapter = new MeasureAngleDistanceAdapter(stationId(),
                        cachedData, surveyorData, surveyorData, getWellType());
                adapter.onShowSlopDistance(showSlopDistance);
                if (recyclerView != null) {
                    recyclerView.setAdapter(adapter);
                    //滚动到当前目标点
                    int currentItemIndex = surveyorData.currentIndex();
                    scrollToPosition(currentItemIndex);
                }

                adapter.setOnItemClickListener(position -> {
                    if (isSurveying() || isAutoSurveying()) {
                        ToastUtils.showShort(R.string.measuring_please_waite);
                        return;
                    }
                    stateManager.switchCurrentMeasurePosition(position);
                    scrollToPosition(position);
//                    adapter.notifyDataSetChanged();
                });
            }

            //监听测站数据,更新界面「平距」「斜距」显示
            SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getLiveDataRecord(stationId())
                    .observe(getViewLifecycleOwner(),
                            surveyorStationRecord -> {
                                stateManager.updateStationRecord(surveyorStationRecord);
                                onShowSlopDistance(surveyorStationRecord.isShowSlopDistance());
                            });
        });
    }

    private void scrollToPosition(int position) {
        if (position >= 0 && adapter != null && position < adapter.getItemCount()) {
            recyclerView.scrollToPosition(position);
            adapter.notifyDataSetChanged();
        }
    }

    /**
     * 收集测站信息
     */
    private synchronized void collectData(
            String stationId,
            Context context,
            FragmentManager fragmentManager,
            LifecycleOwner lifecycleOwner,
            IUIRunner runner,
            IScrollDelegate scrollDelegate,
            ICollectCacheCallback callback) {

        boolean isSupportAutoMode = TotalStation.getInstance().isSupportMotor();
        ThreadPoolUtil.execute(() -> {
            //读取当前测站的当前测回--很重要
            SurveyorScheduleRecord scheduleRecord =
                    SurveyorDatabaseFactory
                            .instance
                            .getScheduleDataSource()
                            .findScheduleSync(SurveyorScheduleConstants.getCurrentCycleKey(stationId, SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE));
            int defaultCycle = scheduleRecord == null ? 0 :
                    DigitalUtil.valueOfIntString(scheduleRecord.getStatue(), 0);
            //加载测站信息
            SurveyorStationRecord surveyorStationRecord =
                    SurveyorDatabaseFactory
                            .instance
                            .getSurveyorStation()
                            .getRecordSync(stationId);
            //加载对应项目
            ProjectRecord projectRecord = SurveyorDatabaseFactory
                    .instance
                    .getProjectDataSource()
                    .getRecord(surveyorStationRecord.getProjectId());
            if (projectRecord == null) {
                callback.onFailure();
                return;
            }
            //加载对应限差
            ToleranceDetailRecord toleranceRecord = SurveyorDatabaseFactory
                    .instance
                    .getToleranceDetailDataSource()
                    .getRecordSync(projectRecord.getToleranceId());
            if (toleranceRecord == null) {
                callback.onFailure();
                return;
            }
            //创建缓存数据
            CachedDataAloneDistance cachedData = CachedDataAloneDistance.newInstance(stationId);
            //是否显示斜距？
            cachedData.onShowSlopDistance(surveyorStationRecord.isShowSlopDistance());

            //创建stateManager实例
            SurveyorData surveyorDataSource = new SurveyorData(ToleranceDetailRecord.ALONE_DISTANCE_MEASURE_COUNT);
            surveyorDataSource.onShowSlopDistance(surveyorStationRecord.isShowSlopDistance());

            StateManagerAloneDistance stateManager = StateManagerAloneDistance.newInstance(
                    isSupportAutoMode,
                    surveyorStationRecord,
                    context,
                    fragmentManager,
                    lifecycleOwner,
                    toleranceRecord,
                    surveyorDataSource,
                    surveyorDataSource,
                    cachedData,
                    createMeasureVm(),
                    scrollDelegate,
                    runner);
            //fragment创建后立即计算当前cycle的测量状态
            runner.runOnUI(() ->
                    stateManager.calScheduleWhenEnterSurveyFragment(defaultCycle));
            callback.onSuccess(surveyorStationRecord.isShowSlopDistance(), cachedData, surveyorDataSource, stateManager);
        });
    }

    public interface ICollectCacheCallback {
        void onFailure();

        void onSuccess(boolean showSlopDistance, IMeasureCache cachedData, SurveyorData surveyorData, StateManagerAloneDistance stateManager);
    }

    @Override
    public void doSurveyor() {
        if (stateManager == null) {
            return;
        }
        stateManager.doSurveyor();
    }

    @Override
    public void autoSurvey() {
        if (stateManager == null) {
            return;
        }
        stateManager.doAutoSurveyor();
    }

    @Override
    public void switchCycle(int cycleIndex) {
        if (stateManager == null) {
            return;
        }
        stateManager.switch2Cycle(cycleIndex);
    }

    @Override
    public void deleteStationAllPeroidRecord() {
        if (stateManager == null) {
            return;
        }
        stateManager.deleteCurrentStationSurveyorData();
    }

    @Override
    public void deleteStationOnePeriodRecord(int cycleIndex) {
//        if (stateManager == null) {
//            return;
//        }
//        stateManager.deleteCurrentStationSurveyorData(cycleIndex);
    }

    @Override
    public String stationCheck() {
        if (stateManager == null) {
            return null;
        }
        return _CheckStationState._stationCheckAloneDistance(stationId(), stateManager.getToleranceRecord(), stateManager.getCachedData());
    }

    @Override
    public void stationAdjust() {
        if (stateManager == null) {
            return;
        }
        _DoCalAdjustState.doCalAdjust(stationId(), stateManager.getCachedData(), requireContext(),
                getChildFragmentManager(), getViewLifecycleOwner());
//        stateManager.stationAdjust();
    }

    @Override
    public void goBack() {
        if (stateManager == null) {
            return;
        }
        stateManager.moveBack();
    }

    @Override
    public void onShowSlopDistance(boolean value) {
        if (cachedData != null) {
            cachedData.onShowSlopDistance(value);
        }
        if (surveyedData != null) {
            surveyedData.onShowSlopDistance(value);
        }
        if (adapter != null) {
            adapter.onShowSlopDistance(value);
            adapter.notifyItemRangeChanged(0, adapter.getItemCount());
        }
    }

    @Override
    public boolean isSurveying() {
        if (stateManager != null) {
            return stateManager.isSurveying();
        }
        return false;
    }

    @Override
    public boolean isAutoSurveying() {
        if (stateManager != null) {
            return stateManager.isAutoSurveying();
        }
        return false;
    }

    private static final String KEY_STATION_ID = "__station_id";
    private static final String KEY_TYPE_ANGLE_OR_DISTANCE = "__key_type_angle_or_distance";
    public static final String TAG = ComponentStationSurveyorDataListFragment.class.getName();

    public static MeasureDistanceFragment newInstance(String stationId, String typeAngleOrDistance) {
        MeasureDistanceFragment fragment = new MeasureDistanceFragment();
        Bundle bundle = new Bundle();
        bundle.putString(KEY_STATION_ID, stationId);
        bundle.putString(KEY_TYPE_ANGLE_OR_DISTANCE, typeAngleOrDistance);
        fragment.setArguments(bundle);
        return fragment;
    }

    @Override
    public boolean isCurrentCycleComplete() {
        if (stateManager == null)
            return false;
        boolean completed = _IsSingleCycleCompletedState.isSingleCycleCompleted(
                stationId(),
                StateManagerAloneDistance.CYCLE_INDEX_DEFAULT,
                stateManager.getCachedData(), ToleranceDetailRecord.ALONE_DISTANCE_MEASURE_COUNT);
        return completed;
    }

    @Override
    public void refreshGoBackBtnEnable() {
        if (stateManager == null || stateManager.getCachedData().pointCount() == 0) {
            EventBus.getDefault().post(new RefreshGoBackButtonAloneDistance(false));
            return;
        }
        stateManager.refreshGoBackBtnEnable();
    }

    @Override
    public int measureTypePosition() {
        return MeasureFragment.POSITION_DISTANCE_MEASURE;
    }

    private MeasureVm createMeasureVm() {
        return new ViewModelProvider(requireParentFragment()).get(MeasureVm.class);
    }


}



