package com.bingce.controlnetwork.fragment.stationsurvey.angledistance;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.util.StationUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.adapter.MeasureAngleDistanceAdapter;
import com.bingce.controlnetwork.fragment.SurveyorSettingsFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.MethodCheckSetting;
import com.bingce.controlnetwork.fragment.stationsurvey.base.MeasureBaseFragment;
import com.bingce.controlnetwork.newui.stationsetting.StationSettingsActivity;
import com.bingce.controlnetwork.surveyor.surveyor.TestDataSelectFragment;
import com.bingce.utils.ThreadPoolUtil;

import org.jetbrains.annotations.NotNull;

/**
 * 测站测量页面（开始测量）
 */
public class MeasureFragment extends MeasureBaseFragment {
    public static final String TAG = MeasureFragment.class.getName();

    //方位观测fragment的在adapter的索引
    public static final int POSITION_DIRECTION_MEASURE = 0;
    public static final int POSITION_DISTANCE_MEASURE = 1;

    @Override
    protected boolean isShowSelectCycle(int position) {
        return POSITION_DIRECTION_MEASURE == position;
    }

    @Override
    protected int defaultPageIndex() {
        return POSITION_DIRECTION_MEASURE;
    }

    @Override
    protected void addFragmentsTabs(Runnable runnable) {
        MeasureDefaultFragment fragmentAngle = MeasureDefaultFragment.newInstance(stationId(), MeasureAngleDistanceAdapter.TYPE_ANGLE);
        fragments.add(fragmentAngle);
        MeasureDistanceFragment fragmentDistance = MeasureDistanceFragment.newInstance(stationId(), MeasureAngleDistanceAdapter.TYPE_DISTANCE);
        fragments.add(fragmentDistance);

        tabs.add(Tools.getString(R.string.azimuth_observation));
        tabs.add(Tools.getString(R.string.alone_measure_distance));
        runnable.run();
    }

    @Override
    protected boolean isAloneDistancePage(int pageIndex) {
        return POSITION_DISTANCE_MEASURE == pageIndex;
    }

    @Override
    protected String getCurrentPageCycleKey() {
        if (mBinding.vp.getCurrentItem() == POSITION_DIRECTION_MEASURE) {
            return SurveyorScheduleConstants.getCurrentCycleKey(stationId(), null);
        } else {
            return SurveyorScheduleConstants.getCurrentCycleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE);
        }
    }

    @Override
    protected String getScheduleIdForAllCycle() {
        int currentItem = mBinding.vp.getCurrentItem();
        String scheduleId;
        if (!isAloneDistancePage(currentItem)) {
            scheduleId = SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), null);
        } else {
            scheduleId = SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE);
        }
        return scheduleId;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = super.onCreateView(inflater, container, savedInstanceState);
        initObserver();
        return view;
    }

    private void initObserver() {
        //监听测站变化
        SurveyorDatabaseFactory
                .instance
                .getSurveyorStation()
                .getLiveDataRecord(stationId())
                .observe(getViewLifecycleOwner(),
                        stationRecord -> {
                            checkStationSettings();
                        });

//       SurveyorScheduleConstants.getCurrentCycleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE);
        monitorCurrentCycle(SurveyorScheduleConstants.getCurrentCycleKey(stationId(), null));
    }

    private void checkStationSettings() {
        ThreadPoolUtil.execute(() -> MethodCheckSetting.checkStationSettings(stationId(), this, this::runOnUI));
    }

    @Override
    public void onPrepareOptionsMenu(@NonNull Menu menu) {
        super.onPrepareOptionsMenu(menu);
        //删除当前测回记录
        menu.findItem(R.id.menu_item_delete_current_records).setEnabled(deleteSingleCycleEnable());
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull @NotNull MenuItem item) {
        if (R.id.menu_item_delete_current_records == item.getItemId()) {
            //删除当前测回记录
            if (surveyorStationRecord != null && currentFragment != null) {
                //通知fragment
                currentFragment.deleteStationOnePeriodRecord(cycleIndex);
            }
        } else if (R.id.menu_item_delete_all_records == item.getItemId()) {
            //删除所有测回记录
            if (surveyorStationRecord != null && currentFragment != null) {
                //通知fragment
                currentFragment.deleteStationAllPeroidRecord();
            }
        } else if (R.id.menu_item_station_check == item.getItemId() && currentFragment != null) {
            //测站检测 被检测平差替代
            stationCheckBefore();
        } else if (R.id.menu_item_data_source == item.getItemId()) {
            //数据源 debug使用
            TestDataSelectFragment.show(getChildFragmentManager());
        } else if (R.id.menu_item_go_back == item.getItemId() && currentFragment != null) {
            //回退
            currentFragment.goBack();
        } else if (R.id.menu_item_station_environment_prism_settings == item.getItemId()) {
            //环境、棱镜设置
            StationSettingsActivity.start(requireContext(), stationId());
//            StationSettingsFragment.start(getContext(), stationId());
        } else if (R.id.menu_item_surveyor_settings == item.getItemId()) {
            //测量设置 设置是否显示平距
            ThreadPoolUtil.execute(() -> {
                String name = StationUtil.getStationName(surveyorStationRecord);
                runOnUI(() -> SurveyorSettingsFragment.start(getContext(), surveyorStationRecord.getId(), name));
            });
        } else if (R.id.menu_item_check_adjust == item.getItemId() && currentFragment != null) {
            //检测平差
            stationCheckAdjustBefore();
        }
        return super.onOptionsItemSelected(item);
    }

    private boolean deleteSingleCycleEnable() {
        return currentFragment != null && currentFragment instanceof MeasureDefaultFragment;
    }

    public static void start(Context context, String surveyorStationId, String surveyorStationName) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_ID, surveyorStationId);
        FragmentContainerActivity.start(context, TAG, surveyorStationName, bundle);
    }

    public static Fragment newInstance(Bundle args) {
        Fragment fragment = new MeasureFragment();
        fragment.setArguments(args);
        return fragment;
    }

}
