package com.bingce.controlnetwork.fragment.stationsurvey.angledistance;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.utils.ThreadPoolUtil;

import com.bingce.controlnetwork.R;

public class StationCheckBeforeUtil {

    public static final int STATION_CHECK = 1;
    public static final int STATION_ADJUST = 2;


    private static final String DATA_COMPLETE_NOT = Tools.getString(R.string.station_data_is_incomplete_hint);

    /**
     * @param scheduleId 进度id eg:SurveyorScheduleConstants.stationScheduleKeyForAllCycle(stationId(), null)
     * @param callBack
     */
    public static void stationDetectBefore(String scheduleId, ICallBack callBack) {
        ThreadPoolUtil.execute(() -> {
            SurveyorScheduleRecord surveyorScheduleRecord = SurveyorDatabaseFactory
                    .instance
                    .getScheduleDataSource()
                    .findScheduleSync(scheduleId);
            ThreadPoolUtil.executeInMain(() -> {
                if (surveyorScheduleRecord == null) {
                    callBack.error(DATA_COMPLETE_NOT);
                    return;
                }
                String statue = surveyorScheduleRecord.getStatue();
                if (SurveyorScheduleConstants.STATUE_VALUE_CHECKED.equals(statue)
                        || SurveyorScheduleConstants.STATUE_VALUE_COMPLETED.equals(statue)) {
                    //完成测量或者经过检测后进行测站检测
                    callBack.next(STATION_CHECK);
                } else {
                    callBack.error(DATA_COMPLETE_NOT);
                }
            });
        });
    }

    /**
     * 测站平差
     *
     * @param scheduleId 进度id eg:SurveyorScheduleConstants.stationScheduleKeyForAllCycle(stationId(), null)
     * @param callBack
     */
    public static void stationAdjustBefore(String scheduleId, ICallBack callBack) {
        ThreadPoolUtil.execute(() -> {
            SurveyorScheduleRecord surveyorScheduleRecord = SurveyorDatabaseFactory
                    .instance
                    .getScheduleDataSource()
                    .findScheduleSync(scheduleId);
            ThreadPoolUtil.executeInMain(() -> {
                if (surveyorScheduleRecord == null) {
                    callBack.error(DATA_COMPLETE_NOT);
                    return;
                }
                String statue = surveyorScheduleRecord.getStatue();
                if (SurveyorScheduleConstants.STATUE_VALUE_CHECKED.equals(statue)) {
                    //测站经过检测->测站平差
                    callBack.next(STATION_ADJUST);
                } else if (SurveyorScheduleConstants.STATUE_VALUE_COMPLETED.equals(statue)) {
                    //测站完成测量->测站检测
                    callBack.next(STATION_CHECK);
                } else {
                    callBack.error(DATA_COMPLETE_NOT);
                }
            });
        });
    }

    public interface ICallBack {
        void next(int nextType);

        void error(String error);
    }

}
