package com.bingce.controlnetwork.fragment.stationsurvey.base;

import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.lifecycle.Lifecycle;
import androidx.viewpager.widget.ViewPager;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.AutoSurveyorButtonVisibleEvent;
import com.bingce.controlapphelper.event.RefreshGoBackButton;
import com.bingce.controlapphelper.event.RefreshGoBackButtonAloneDistance;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.surveyor.test.Test;
import com.bingce.controlapphelper.util.DigitalUtil;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.FragmentMeasureBinding;
import com.bingce.controlnetwork.fragment.BottomSheetSelectListFragment;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.IMeasureOperate;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.StationCheckBeforeUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.flyco.tablayout.SlidingTabLayout;
import com.github.razir.progressbutton.DrawableButtonExtensionsKt;
import com.github.razir.progressbutton.ProgressButtonHolderKt;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import blankj.utilcode.util.ToastUtils;

public abstract class MeasureBaseFragment extends BaseFragment {
    private boolean menuItemRollBackEnable;

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onRefreshGoBack(RefreshGoBackButton event) {
        if (mBinding != null) {
            int currentItem = mBinding.vp.getCurrentItem();
            if (!isAloneDistancePage(currentItem)) {
                menuItemRollBackEnable = event.validate;
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onRefreshGoBack(RefreshGoBackButtonAloneDistance event) {
        if (mBinding != null) {
            int currentItem = mBinding.vp.getCurrentItem();
            if (isAloneDistancePage(currentItem)) {
                menuItemRollBackEnable = event.validate;
            }
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onSurveyorStateChanged(SurveyStateEvent event) {
        if (isInValidEventPosition(event.getPosition()))
            return;

        if (event.isIdle() || event.isSuccess() || event.isCompleted()) {
            DrawableButtonExtensionsKt.hideProgress(mBinding.surveyor, R.string.start_surveyor);
        } else if (event.isSurveying()) {
            DrawableButtonExtensionsKt.showProgress(mBinding.surveyor, progressParams -> {
                progressParams.setButtonTextRes(R.string.tips_surveying);
                progressParams.setProgressColor(Color.WHITE);
                return null;
            });
        } else if (event.isFailure()) {
            DrawableButtonExtensionsKt.hideProgress(mBinding.surveyor, R.string.tips_surveyor_failure_and_retry);
        }

        boolean enable = event.isIdle() || event.isFailure() || event.isSuccess();
        setSurveyorEnable(mBinding.surveyor, enable);
        setSurveyorEnable(mBinding.autoSurveyor, enable);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onAutoSurveyorChanged(AutoSurveyorButtonVisibleEvent event) {
        if (isInValidEventPosition(event.getPosition()))
            return;
        mBinding.autoSurveyor.setVisibility(event.isReadyAutoSurvey() ? View.VISIBLE : View.GONE);
    }

    protected FragmentMeasureBinding mBinding;

    protected IMeasureOperate currentFragment;

    protected SurveyorStationRecord surveyorStationRecord;
    protected int cycleIndex = 0;
    protected ArrayList<String> cycleList;

    protected abstract boolean isShowSelectCycle(int position);

    protected abstract int defaultPageIndex();

    protected abstract void addFragmentsTabs(Runnable runnable);

    protected abstract boolean isAloneDistancePage(int pageIndex);

    protected abstract String getCurrentPageCycleKey();

    protected abstract String getScheduleIdForAllCycle();

    protected void setSurveyorEnable(View view, boolean enable) {
        if (view != null) {
            view.setEnabled(enable);
            if (enable) {
                view.setAlpha(1);
            } else {
                view.setAlpha(.4f);
            }
        }
    }

    protected List<IMeasureOperate> fragments = new ArrayList<>();
    protected List<String> tabs = new ArrayList<>();

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        setHasOptionsMenu(true);
        EventBus.getDefault().register(this);
        mBinding = FragmentMeasureBinding.inflate(inflater, container, false);
        //设置测量按钮
        ProgressButtonHolderKt.bindProgressButton(getViewLifecycleOwner(), mBinding.surveyor);
        initClickListener();
        return mBinding.getRoot();
    }

    @Override
    public void onResume() {
        super.onResume();
        //测量界面保持常亮
        if (getActivity() != null) {
            getActivity().getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        //关闭测量界面保持常亮
        if (getActivity() != null) {
            getActivity().getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initStation();
        initView();
        initViewPager(view);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
        EventBus.getDefault().unregister(this);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull @NotNull Menu menu, @NonNull @NotNull MenuInflater inflater) {
        if (Test.DEBUG) {
            inflater.inflate(R.menu.menu_surveyor_station_record_data_edit_debug_pla, menu);
        } else {
            inflater.inflate(R.menu.menu_surveyor_station_record_data_edit_release_pla, menu);
        }
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public void onPrepareOptionsMenu(@NonNull Menu menu) {
        super.onPrepareOptionsMenu(menu);
        menu.findItem(R.id.menu_item_go_back).setEnabled(menuItemRollBackEnable);
    }

    private void initClickListener() {
        mBinding.surveyor.setOnClickListener(view -> {
            goSurvey();
        });
        mBinding.autoSurveyor.setOnClickListener(view -> {
            goAutoSurvey();
        });

        //测回点击
        mBinding.cycleLayout.setOnClickListener(view -> {
                    if (cycleList == null || cycleList.isEmpty()) {
                        ToastUtils.showShort(R.string.cycle_data_abnormal_toast);
                        return;
                    }
                    BottomSheetSelectListFragment.showSelect(
                            getChildFragmentManager(),
                            getViewLifecycleOwner(),
                            cycleIndex,
                            cycleList,
                            null,//不再限制测期
                            (index, string) -> {
                                mBinding.circleText.setText(string);
                                cycleIndex = index;
                                refreshCycleForFragment();
                            });
                }
        );
    }

    private void refreshCycleForFragment() {
        if (currentFragment != null) {
            //检测当前测回是否完成
            currentFragment.switchCycle(cycleIndex);
        }
    }

    protected String stationId() {
        if (getArguments() == null) {
            return null;
        }
        return getArguments().getString(BundleConstants.KEY_SURVEYOR_STATION_ID);
    }

    private void initStation() {
        ThreadPoolUtil.execute(() -> {
            surveyorStationRecord = SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getRecordSync(stationId());
            //监听数据变化
            ToleranceDetailRecord toleranceRecord = getTolerance(surveyorStationRecord);
            if (toleranceRecord == null) {
                return;
            }
            AtomicInteger cycleCount = new AtomicInteger(toleranceRecord.getCycleCount());
            cycleList = SurveyorCycleUtil.cycleStrings(cycleCount.get(),
                    surveyorStationRecord != null && surveyorStationRecord.leftAndRightSurveyor);

        });
    }

    protected void initView() {

    }

    private void initViewPager(View view) {
        fragments = new ArrayList<>();
        addFragmentsTabs(() -> {
            mBinding.tabLayout.setVisibility(fragments.size() == 1 ? View.GONE : View.VISIBLE);

            mBinding.cycleLayout.setVisibility(isShowSelectCycle(mBinding.vp.getCurrentItem()) ? View.VISIBLE : View.GONE);

            SlidingTabLayout tabLayout = view.findViewById(R.id.tabLayout);
            mBinding.vp.setAdapter(new MyPagerAdapter(getChildFragmentManager()));
            tabLayout.setViewPager(mBinding.vp);

            mBinding.vp.setCurrentItem(defaultPageIndex());
            currentFragment = fragments.get(defaultPageIndex());

            mBinding.vp.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
                @Override
                public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

                }

                @Override
                public void onPageSelected(int position) {
                    currentFragment = fragments.get(position);
                    View pick = view.findViewById(R.id.cycleLayout);
//                    //是否显示选择测期
                    pick.setVisibility(isShowSelectCycle(position) ? View.VISIBLE : View.GONE);
                    //刷新测量按钮状态
                    boolean currentCycleComplete = currentFragment.isCurrentCycleComplete();
                    if (currentCycleComplete) {
                        SurveyStateEvent.postCompleted(position);
                    } else {
                        SurveyStateEvent.postIdle(position);
                    }

                    //切换tab测回可能会变刷新当前fragment
                    refreshCycleAndFragment();

                    currentFragment.refreshGoBackBtnEnable();
                }

                @Override
                public void onPageScrollStateChanged(int state) {

                }
            });
        });
    }

    private void refreshCycleAndFragment() {
        ThreadPoolUtil.execute(() -> {
            SurveyorScheduleRecord record = SurveyorDatabaseFactory
                    .instance
                    .getScheduleDataSource()
                    .findScheduleSync(getCurrentPageCycleKey());

            if (record != null) {
                refreshCycleUi(record);
                refreshCycleForFragment();
            }
        });
    }

    protected void monitorCurrentCycle(String currentCycleKey) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .findSchedule(currentCycleKey)
                .observe(getViewLifecycleOwner(), record -> {
                    if (!currentCycleKey.equals(getCurrentPageCycleKey())) {
                        return;
                    }
                    refreshCycleUi(record);
                });
    }

    private void refreshCycleUi(SurveyorScheduleRecord record) {
        int cycleIndex = DigitalUtil.valueOfIntString(record == null ? "0" : record.getStatue(), 0);
        //更新测回名称
        boolean showLeftOrRight = true;
        if (surveyorStationRecord != null) {
            showLeftOrRight = surveyorStationRecord.leftAndRightSurveyor;
        }
        mBinding.circleText.setText(SurveyorCycleUtil.cycleName(cycleIndex, showLeftOrRight));
        this.cycleIndex = cycleIndex;
    }


    @WorkerThread
    private static ToleranceDetailRecord getTolerance(SurveyorStationRecord surveyorStationRecord) {
        if (surveyorStationRecord == null) {
            return null;
        }
        //获取project信息
        ProjectRecord projectRecord = SurveyorDatabaseFactory
                .instance
                .getProjectDataSource()
                .getRecord(surveyorStationRecord.getProjectId());
        if (projectRecord == null) {
            return null;
        }
        //获取限差数据
        return SurveyorDatabaseFactory
                .instance
                .getToleranceDetailDataSource()
                .getRecordSync(projectRecord.getToleranceId());
    }

    private void goSurvey() {
        if (currentFragment != null) {
            currentFragment.doSurveyor();
        }
    }

    private void goAutoSurvey() {
        if (currentFragment != null) {
            currentFragment.autoSurvey();
        }
    }

    private class MyPagerAdapter extends FragmentPagerAdapter {
        public MyPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public int getCount() {
            return fragments.size();
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return tabs.get(position);
        }

        @Override
        public Fragment getItem(int position) {
            return (Fragment) fragments.get(position);
        }
    }

    protected void stationCheckAdjustBefore() {
        StationCheckBeforeUtil.stationAdjustBefore(getScheduleIdForAllCycle(), new StationCheckBeforeUtil.ICallBack() {

            @Override
            public void next(int nextType) {
                if (currentFragment != null) {
                    if (nextType == StationCheckBeforeUtil.STATION_ADJUST) {
                        currentFragment.stationAdjust();
                    } else if (nextType == StationCheckBeforeUtil.STATION_CHECK) {
                        //已经完成测量，开始测站检测
                        String error = currentFragment.stationCheck();
                        if (TextUtils.isEmpty(error)) {
                            currentFragment.stationAdjust();
                        } else {
                            TipsFragment.cancelAbleTips(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(),
                                    error + Tools.getString(R.string.is_adjustment_forced), 0, () -> {
                                        currentFragment.stationAdjust();
                                    });
                        }
                    }
                }
            }

            @Override
            public void error(String error) {
                tipsError(error);
            }
        });

    }

    protected void stationCheckBefore() {
        StationCheckBeforeUtil.stationDetectBefore(getScheduleIdForAllCycle(), new StationCheckBeforeUtil.ICallBack() {
            @Override
            public void next(int nextType) {
                if (currentFragment != null) {
                    String error = currentFragment.stationCheck();
                    if (TextUtils.isEmpty(error)) {
                        if (isVisiable()) {
                            TipsFragment.cancelAbleTips(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(),
                                    Tools.getString(R.string.station_passed_is_adjust), 0,
                                    () -> currentFragment.stationAdjust());
                        }
                    } else {
                        tipsError(error);
                    }
                }
            }

            @Override
            public void error(String error) {
                tipsError(error);
            }
        });
    }

    private void tipsError(String error) {
        if (isVisiable() && !TextUtils.isEmpty(error)) {
            TipsFragment.tipsOnly(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(), error);
        }
    }

    private boolean isVisiable() {
        return getViewLifecycleOwner().getLifecycle().getCurrentState().isAtLeast(Lifecycle.State.STARTED);
    }

    private int getCurrentPageTypePosition() {
        return fragments.get(mBinding.vp.getCurrentItem()).measureTypePosition();
    }

    /**
     * 是否是无效的点
     *
     * @param position
     * @return
     */
    private boolean isInValidEventPosition(int position) {
        return mBinding == null || position != getCurrentPageTypePosition();
    }
}
