package com.bingce.controlnetwork.fragment.stationsurvey.heighttraverse

import android.content.Context
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import androidx.fragment.app.Fragment
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.config.model.StationLayout
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.StationUtil.getStationName
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity
import com.bingce.controlnetwork.fragment.SurveyorSettingsFragment
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment
import com.bingce.controlnetwork.fragment.stationsurvey.base.MeasureBaseFragment
import com.bingce.controlnetwork.newui.stationsettingheighttraverse.StationSettingsHeightTraverseActivity
import com.bingce.controlnetwork.surveyor.state.heighttraverse.__BaseState
import com.bingce.controlnetwork.surveyor.surveyor.TestDataSelectFragment
import com.bingce.utils.ThreadPoolUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MeasureHeightTraverseFragment : MeasureBaseFragment() {

    private val mainScope = MainScope()

    companion object {
        const val POSITION_BACK_PAGE = 0
        const val POSITION_FRONT_PAGE = 1

        @JvmField
        val TAG: String = MeasureHeightTraverseFragment::class.java.name

        @JvmStatic
        fun start(context: Context, surveyorStationId: String, surveyorStationName: String?) {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_SURVEYOR_STATION_ID, surveyorStationId)
            FragmentContainerActivity.start(
                context,
                TAG,
                surveyorStationName,
                bundle
            )
        }

        @JvmStatic
        fun newInstance(args: Bundle?): Fragment {
            val fragment: Fragment = MeasureHeightTraverseFragment()
            fragment.arguments = args
            return fragment
        }
    }

    override fun isShowSelectCycle(position: Int): Boolean {
        return false
    }

    override fun defaultPageIndex(): Int {
        return POSITION_BACK_PAGE
    }

    override fun initView() {
        super.initView()
        setSurveyorEnable(mBinding.surveyor, false)
    }

    override fun addFragmentsTabs(runnable: Runnable) {
        mainScope.launch(Dispatchers.IO) {
            val stationId = stationId()

            val stationRecord =
                SurveyorDatabaseFactory.instance.surveyorStation.getRecordSync(stationId)
            val configRecord =
                SurveyorDatabaseFactory.instance.getHeightTraverseConfigDataSource()
                    .getRecordByProjectId(stationRecord.getProjectId())

            if (configRecord?.stationLayout == StationLayout.SEPARATE) {
                fragments.add(
                    MeasureHeightTraverseSubFragment.newInstance(
                        stationId,
                        POSITION_BACK_PAGE
                    )
                )
                fragments.add(
                    MeasureHeightTraverseSubFragment.newInstance(
                        stationId,
                        POSITION_FRONT_PAGE
                    )
                )

                tabs.add(Tools.getString(R.string.rearview))
                tabs.add(Tools.getString(R.string.forward_view))
            } else if (configRecord?.stationLayout == StationLayout.EVERY) {
                fragments.add(
                    MeasureHeightTraverseSubFragment.newInstance(
                        stationId,
                        POSITION_FRONT_PAGE
                    )
                )
                tabs.add(Tools.getString(R.string.forward_view))
            }
            withContext(Dispatchers.Main) {
                runnable.run()
            }
        }
    }

    override fun isAloneDistancePage(pageIndex: Int): Boolean {
        return false
    }

    override fun getCurrentPageCycleKey(): String {
        return SurveyorScheduleConstants.getCurrentCycleKey(
            stationId(),
            __BaseState.getScheduleKey(mBinding.vp.currentItem)
        )
    }

    override fun getScheduleIdForAllCycle(): String {
        val positionFragment = getCurrentMeasurePos()
        return SurveyorScheduleConstants.getAllCycleScheduleKey(
            stationId(),
            __BaseState.getScheduleKey(positionFragment)
        )
    }

    private fun getCurrentMeasurePos(): Int {
        val measureHeightTraverseSubFragment = currentFragment as MeasureHeightTraverseSubFragment
        return measureHeightTraverseSubFragment.backOrFrontPosition
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        monitorCurrentCycle(
            SurveyorScheduleConstants.getCurrentCycleKey(
                stationId(),
                SurveyorScheduleConstants.STATION_TYPE_HEIGHT_TRAVERSE_BACK
            )
        )
        monitorCurrentCycle(
            SurveyorScheduleConstants.getCurrentCycleKey(
                stationId(),
                SurveyorScheduleConstants.STATION_TYPE_HEIGHT_TRAVERSE_FRONT
            )
        )
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.menu_item_delete_current_records == item.itemId) {
            //删除当前测回记录
            if (surveyorStationRecord != null && currentFragment != null) {
                //通知fragment
                currentFragment.deleteStationOnePeriodRecord(getCurrentMeasurePos())
            }
        } else if (R.id.menu_item_delete_all_records == item.itemId) {
            //删除所有测回记录
            showDeleteStationRecord()
        } else if (R.id.menu_item_station_check == item.itemId && currentFragment != null) {
            //测站检测
            stationCheckBefore()
        } else if (R.id.menu_item_data_source == item.itemId) {
            //数据源 debug使用
            TestDataSelectFragment.show(childFragmentManager)
        } else if (R.id.menu_item_go_back == item.itemId && currentFragment != null) {
            //回退
            currentFragment.goBack()
        } else if (R.id.menu_item_station_environment_prism_settings == item.itemId) {
            //环境、棱镜设置
//            StationSettingHeightTraverseFragment.start(requireContext(), stationId())
            StationSettingsHeightTraverseActivity.start(requireContext(), stationId())
        } else if (R.id.menu_item_surveyor_settings == item.itemId) {
            //测量设置 设置是否显示平距
            ThreadPoolUtil.execute {
                val name = getStationName(surveyorStationRecord)
                runOnUI {
                    SurveyorSettingsFragment.start(
                        context,
                        surveyorStationRecord.getId(),
                        name
                    )
                }
            }
        } else if (R.id.menu_item_check_adjust == item.itemId && currentFragment != null) {
            //检测平差
            stationCheckAdjustBefore()
        }
        return super.onOptionsItemSelected(item)
    }

    /**
     * 删除测站的后视和前视的所有测回记录
     */
    private fun showDeleteStationRecord() {
        if (surveyorStationRecord == null || currentFragment == null) {
            return
        }
        val fragment = currentFragment as MeasureHeightTraverseSubFragment
        if (fragment.isInvalite || fragment.isDead) {
            return
        }

        val msg = if (fragments.size == 1) {
            "是否确定删除该站的所有测回数据？"
        } else {
            "是否确定删除该站的后视和前视的所有测回数据？"
        }

        TipsWith2OptionFragment.tips(
            childFragmentManager,
            this,
            null,
            null,
            null,
            msg,
            object : TipsWith2OptionFragment.SimpleCallback() {
                override fun onConfirm() {
                    for (fragmentI in fragments) {
                        fragmentI.deleteStationAllPeroidRecord()
                    }
                }
            })
    }


}