package com.bingce.controlnetwork.fragment.stationsurvey.heighttraverse;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.RefreshGoBackButton;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataHeightTraverse;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.IDistanceTypeChangeListener;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.data.SurveyorData;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParamAuto;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.adapter.MeasureHeightTraverseAdapter;
import com.bingce.controlnetwork.fragment.ComponentStationSurveyorDataListFragment;
import com.bingce.controlnetwork.fragment.StationDataAdjustDialogFragment;
import com.bingce.controlnetwork.fragment.base.BaseLazyFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.IMeasureOperate;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureVm;
import com.bingce.controlnetwork.surveyor.state.IScrollDelegate;
import com.bingce.controlnetwork.surveyor.state.heighttraverse.StateManagerHeightTraverse;
import com.bingce.totalstation.TotalStation;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;

/**
 * 测角测距
 */
public class MeasureHeightTraverseSubFragment extends BaseLazyFragment
        implements IDistanceTypeChangeListener, ISurveyState, IMeasureOperate {
    private MeasureHeightTraverseAdapter adapter = null;
    private StateManagerHeightTraverse stateManager;
    private RecyclerView recyclerView;
    private ICachedData cachedData = null;
    private ISurveyorData surveyedData = null;
    private ISurveyState surveyState;

    private String stationId() {
        if (getArguments() == null) {
            throw new RuntimeException("getArguments()==null");
        }
        return getArguments().getString(KEY_STATION_ID);
    }

    public int getBackOrFrontPosition() {
        if (getArguments() == null) {
            throw new RuntimeException("getArguments()==null");
        }
        return getArguments().getInt(KEY_TYPE_BACK_OR_FRONT);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater,
                             @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(
                R.layout.fragment_project_surveyor_station_point_list_layout,
                container,
                false);
        recyclerView = view.findViewById(R.id.recycler_view);
        return view;
    }

    @WorkerThread
    private void onCollectStationData(
            boolean showSlopDistance,
            ICachedData cachedData,
            SurveyorData surveyorData,
            StateManagerHeightTraverse stateManager) {
        this.stateManager = stateManager;
        this.cachedData = cachedData;
        surveyedData = surveyorData;

        //测量数据收集完毕后，就可以更新UI了
        runOnUI(() -> {
            //刷新UI
            if (adapter == null) {
                adapter = new MeasureHeightTraverseAdapter(stationId(),
                        cachedData, surveyorData, surveyorData, getBackOrFrontPosition());
                adapter.onShowSlopDistance(showSlopDistance);
                if (recyclerView != null) {
                    recyclerView.setAdapter(adapter);
                    //滚动到当前目标点
                    int currentItemIndex = surveyorData.currentIndex();
                    if (currentItemIndex >= 0 && currentItemIndex < adapter.getItemCount()) {
                        recyclerView.scrollToPosition(currentItemIndex);
                    }
                }
            }

            //监听测站数据
            //监听「平距」「斜距」切换，更新UI显示
            SurveyorDatabaseFactory
                    .instance
                    .getSurveyorStation()
                    .getLiveDataRecord(stationId())
                    .observe(getViewLifecycleOwner(),
                            surveyorStationRecord -> onShowSlopDistance(surveyorStationRecord.isShowSlopDistance()));
        });
    }

    /**
     * 收集测站信息
     */
    private void collectData(
            String stationId,
            Context context,
            FragmentManager fragmentManager,
            LifecycleOwner lifecycleOwner,
            IUIRunner runner,
            IScrollDelegate scrollDelegate,
            ICollectCacheCallback callback) {
        if (callback == null) {
            return;
        }
        boolean isSupportAutoMode = TotalStation.getInstance().isSupportMotor();
        ThreadPoolUtil.execute(() -> {
            //加载测站信息
            SurveyorStationRecord surveyorStationRecord =
                    SurveyorDatabaseFactory
                            .instance
                            .getSurveyorStation()
                            .getRecordSync(stationId);
            //加载对应项目
            ProjectRecord projectRecord = SurveyorDatabaseFactory
                    .instance
                    .getProjectDataSource()
                    .getRecord(surveyorStationRecord.getProjectId());
            if (projectRecord == null) {
                callback.onFailure();
                return;
            }
            //加载对应限差
            ToleranceDetailRecord toleranceRecord = SurveyorDatabaseFactory
                    .instance
                    .getToleranceDetailDataSource()
                    .getRecordSync(projectRecord.getToleranceId());
            if (toleranceRecord == null) {
                callback.onFailure();
                return;
            }
            //创建缓存数据
            CachedDataHeightTraverse cachedData =
                    CachedDataHeightTraverse.newInstance(getBackOrFrontPosition(),
                            stationId, toleranceRecord);
            if (cachedData == null) {
                callback.onFailure();
                return;
            }

            //是否显示斜距？
            cachedData.onShowSlopDistance(surveyorStationRecord.isShowSlopDistance());
            if (cachedData.pointCount() <= 0) {
                callback.onFailure();
                return;
            }

            //创建stateManager实例
            SurveyorData surveyorDataSource = new SurveyorData(toleranceRecord.getSurveyorCount());
            surveyorDataSource.onShowSlopDistance(surveyorStationRecord.isShowSlopDistance());

            //存入cacheData 用于自动测量时使用
            MeasureVm measureVm = new ViewModelProvider(requireParentFragment()).get(MeasureVm.class);
            RequestSurveyorParamAuto requestSurveyorParamAuto = new RequestSurveyorParamAuto(stationId, cachedData);
            measureVm.setRequestSurveyorParamAuto(requestSurveyorParamAuto);

            StateManagerHeightTraverse stateManager = StateManagerHeightTraverse.newInstance(
                    getBackOrFrontPosition(),
                    isSupportAutoMode,
                    requestSurveyorParamAuto,
                    surveyorStationRecord,
                    context,
                    fragmentManager,
                    lifecycleOwner,
                    toleranceRecord,
                    surveyorDataSource,
                    surveyorDataSource,
                    cachedData,
                    scrollDelegate,
                    runner);
            //fragment创建后立即计算当前cycle的测量状态
            runner.runOnUI(() -> stateManager.calScheduleWhenEnterSurveyFragment(0));
            callback.onSuccess(surveyorStationRecord.isShowSlopDistance(), cachedData, surveyorDataSource, stateManager);
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        if (stateManager != null) {
            stateManager.resume();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (stateManager != null) {
            stateManager.quite();
        }
    }

    @Override
    protected void loadData() {
        String stationId = stationId();
        if (stationId != null) {
            //收集信息
            collectData(stationId,
                    getContext(),
                    getChildFragmentManager(),
                    getViewLifecycleOwner(),
                    this::runOnUI,
                    index -> {
                        if (index >= 0 && adapter != null && index < adapter.getItemCount()) {
                            recyclerView.scrollToPosition(index);
                        }
                    },
                    new ICollectCacheCallback() {
                        @Override
                        public void onFailure() {
                        }

                        @Override
                        public void onSuccess(boolean showSlopDistance, ICachedData cachedData, SurveyorData surveyorData, StateManagerHeightTraverse stateManager) {
                            onCollectStationData(showSlopDistance, cachedData, surveyorData, stateManager);
                            surveyState = stateManager;
                        }
                    }
            );
        }
    }

    public interface ICollectCacheCallback {
        void onFailure();

        void onSuccess(boolean showSlopDistance, ICachedData cachedData, SurveyorData surveyorData, StateManagerHeightTraverse stateManager);
    }

    @Override
    public void doSurveyor() {
        if (stateManager == null) {
            return;
        }
        stateManager.doSurveyor();
    }

    @Override
    public void autoSurvey() {
        if (stateManager == null) {
            return;
        }
        stateManager.doAutoSurveyor();
    }

    @Override
    public void switchCycle(int cycleIndex) {
        if (stateManager == null) {
            return;
        }
        stateManager.switch2Cycle(cycleIndex);
    }

    @Override
    public void deleteStationAllPeroidRecord() {
        if (stateManager == null) {
            return;
        }
        stateManager.deleteCurrentStationSurveyorData();
    }

    @Override
    public void deleteStationOnePeriodRecord(int positionFragment) {
        if (stateManager == null) {
            return;
        }
        stateManager.deleteCurrentStationSurveyorData(positionFragment);
    }

    @Override
    public String stationCheck() {
        if (stateManager == null) {
            return null;
        }
        return stateManager.stationCheckOnly();
    }

    @Override
    public void stationAdjust() {
        if (stateManager == null) {
            return;
        }
        stateManager.stationAdjustOnly(StationDataAdjustDialogFragment.TYPE_ANGLE_VER_AND_DISTANCE);
    }

    @Override
    public void goBack() {
        if (stateManager == null) {
            return;
        }
        stateManager.moveBack();
    }

    /**
     * 对于高程导线来说 这个方法实现当前目标的所有测回是否完成
     *
     * @return
     */
    @Override
    public boolean isCurrentCycleComplete() {
        if (stateManager == null || cachedData == null || cachedData.pointCount() == 0)
            return false;
        for (int i = 0; i < cachedData.pointCount(); i++) {
            String pointId = cachedData.pointId(i, i);
            String recordIdLeft = IdUtil.createSurveyorRecordId(stationId(), i, pointId, LeftOrRight.left);
            String recordIdRight = IdUtil.createSurveyorRecordId(stationId(), i, pointId, LeftOrRight.right);
            DataCategory dataCategoryLeft = cachedData.valueState(recordIdLeft);
            DataCategory dataCategoryRight = cachedData.valueState(recordIdRight);
            if (dataCategoryLeft.isEmptyData() || dataCategoryRight.isEmptyData()) {
                //只要有空数据就是未完成
                return false;
            }
        }
        return true;
    }

    @Override
    public void refreshGoBackBtnEnable() {
        if (stateManager == null || stateManager.getCachedData().pointCount() == 0) {
            EventBus.getDefault().post(new RefreshGoBackButton(false));
            return;
        }
        stateManager.refreshGoBackBtnEnable();
    }

    @Override
    public int measureTypePosition() {
        return getBackOrFrontPosition();
    }

    @Override
    public void onShowSlopDistance(boolean value) {
        if (cachedData != null) {
            cachedData.onShowSlopDistance(value);
        }
        if (surveyedData != null) {
            surveyedData.onShowSlopDistance(value);
        }
        if (adapter != null) {
            adapter.onShowSlopDistance(value);
            adapter.notifyItemRangeChanged(0, adapter.getItemCount());
        }
    }

    @Override
    public boolean isSurveying() {
        if (surveyState != null) {
            return surveyState.isSurveying();
        }
        return false;
    }

    @Override
    public boolean isAutoSurveying() {
        if (surveyState != null) {
            return surveyState.isAutoSurveying();
        }
        return false;
    }

    public boolean isInvalite() {
        return stateManager.isInValidate();
    }

    public boolean isDead() {
        return stateManager.isDead();
    }

    private static final String KEY_STATION_ID = "__station_id";
    private static final String KEY_TYPE_BACK_OR_FRONT = "__KEY_TYPE_BACK_OR_FRONT";
    public static final String TAG = ComponentStationSurveyorDataListFragment.class.getName();

    public static MeasureHeightTraverseSubFragment newInstance(String stationId, int frontOrBack) {
        MeasureHeightTraverseSubFragment fragment = new MeasureHeightTraverseSubFragment();
        Bundle bundle = new Bundle();
        bundle.putString(KEY_STATION_ID, stationId);
        bundle.putInt(KEY_TYPE_BACK_OR_FRONT, frontOrBack);
        fragment.setArguments(bundle);
        return fragment;
    }
}



