package com.bingce.controlnetwork.func.level.activity

import android.content.Context
import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import androidx.activity.viewModels
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentStatePagerAdapter
import androidx.viewpager.widget.ViewPager.OnPageChangeListener
import com.bingce.controlnetwork.func.level.fragment.LevelStationSettingConditionFragment
import com.bingce.controlnetwork.func.level.fragment.LevelStationSettingInformationFragment
import com.bingce.controlnetwork.func.level.viewmodel.LevelStationSettingVm
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.ActivityLevelStationSettingBinding
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import com.bingce.controlnetwork.newui.stationsetting.IStationSettingOperate

class LevelStationSettingActivity : BaseBindingActivity<ActivityLevelStationSettingBinding>() {

    private val viewModel by viewModels<LevelStationSettingVm>()
    private val fragments = mutableListOf<Fragment>()
    private val fragmentTitleList = listOf("观测信息", "观测条件")

    companion object {
        private const val KEY_IS_SAVE_END_STATION = "KEY_IS_SAVE_END_STATION"

        @JvmStatic
        fun start(context: Context, routeId: String, stationId: String, isSaveEndStation: Boolean) {
            context.startActivity(Intent(context, LevelStationSettingActivity::class.java).apply {
                putExtra(BundleConstants.KEY_ROUTE_ID, routeId)
                putExtra(BundleConstants.KEY_SURVEYOR_STATION_ID, stationId)
                putExtra(KEY_IS_SAVE_END_STATION, isSaveEndStation)
            })
        }
    }


    override fun getBinding(): ActivityLevelStationSettingBinding {
        return ActivityLevelStationSettingBinding.inflate(layoutInflater)
    }

    override fun initView() {
        viewModel.initRouteStationId(getRouteId(), getStationId())
        initViewPager()
    }

    override fun initData() {
        viewModel.getEditLastData(getStationId())

        viewModel.getInitCurrentTabPos {
            mBinding.tabLayout.currentTab = it
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_confirm, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.menu_confirm == item.itemId) {
            doConfirm()
        }
        return super.onOptionsItemSelected(item)
    }

    private fun initViewPager() {
        fragments.add(LevelStationSettingInformationFragment())
        fragments.add(LevelStationSettingConditionFragment())
        mBinding.vp.offscreenPageLimit = 3
        mBinding.vp.adapter = MyPagerAdapter(supportFragmentManager)
        mBinding.tabLayout.setViewPager(mBinding.vp)
        mBinding.vp.addOnPageChangeListener(object : OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {

            }

            override fun onPageScrollStateChanged(state: Int) {

            }
        })

//        //设置当前tab为仪高镜高
//        mBinding.vp.currentItem = 1
    }


    private inner class MyPagerAdapter(fm: FragmentManager) : FragmentStatePagerAdapter(fm) {
        override fun getCount(): Int {
            return fragments.size
        }

        override fun getPageTitle(position: Int): CharSequence {
            return fragmentTitleList[position]
        }

        override fun getItem(position: Int): Fragment {
            return fragments[position]
        }

        override fun getItemPosition(`object`: Any): Int {
            return POSITION_NONE
        }
    }

    private fun doConfirm() {
        //检查未完成的选项
        for (fragment in fragments) {
            val operate = fragment as IStationSettingOperate
            if (operate.checkSaveNot()) {
                val fragmentNotFinishPos = fragments.indexOf(fragment)
                if (mBinding.vp.currentItem != fragmentNotFinishPos) {
                    mBinding.tabLayout.currentTab = fragmentNotFinishPos
                }
                return
            }
        }

        viewModel.createNeedSaveLevelRouteSave(getRouteId(), getStationId())
        //赋值
        for (fragment in fragments) {
            val operate = fragment as IStationSettingOperate
            operate.setDataToStation()
        }

        viewModel.save(getIsSaveEndStation()) {
            finish()
        }
    }

    private fun getStationId() = intent.getStringExtra(BundleConstants.KEY_SURVEYOR_STATION_ID)!!

    private fun getRouteId() = intent.getStringExtra(BundleConstants.KEY_ROUTE_ID)!!
    private fun getIsSaveEndStation() = intent.getBooleanExtra(KEY_IS_SAVE_END_STATION, false)
}