package com.bingce.controlnetwork.func.level.dialog;

import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_IGNORE;
import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_REMEASURE_POINT;
import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_REMEASURE_STATION;

import android.app.Dialog;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlnetwork.R;

/**
 * 水准限差超限提示框
 */
public class LevelToleranceTipsFragment extends AppCompatDialogFragment {

    private static final String KEY_TITLE = "__TITLE";
    private static final String KEY_CONTENT = "___CONTENT";
    private static final String KEY_RESULT = "KEY_RESULT";
    private static final String KEY_BUNDLE = "KEY_BUNDLE";

    private static final String TAG = LevelToleranceTipsFragment.class.getName();

    public static void tips(@NonNull FragmentManager fragmentManager,
                            @NonNull LifecycleOwner lifecycleOwner,
                            String title,
                            String content,
                            IOperatorCallback callback) {
        Bundle args = new Bundle();
        LevelToleranceTipsFragment fragment = new LevelToleranceTipsFragment();
        fragment.setArguments(args);
        args.putString(KEY_TITLE, title);
        args.putString(KEY_CONTENT, content);
        fragmentManager
                .setFragmentResultListener(
                        KEY_BUNDLE,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            int type = result.getInt(KEY_RESULT, -1);
                            if (VALUE_IGNORE == type) {
                                callback.onIgnore();
                            } else if (VALUE_REMEASURE_STATION == type) {
                                callback.onReMeasureStation();
                            } else if (VALUE_REMEASURE_POINT == type) {
                                callback.onReMeasurePoint();
                            }
                        });
        fragment.show(fragmentManager, TAG);
    }

    public interface IOperatorCallback {
        void onReMeasureStation();

        void onReMeasurePoint();

        void onIgnore();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View contentView = inflater.inflate(R.layout.fragment_level_tolerance_tips, container, false);

        TextView tvTitle = contentView.findViewById(R.id.tvTitle);
        tvTitle.setText(getTitle());
        TextView tvContent = contentView.findViewById(R.id.tvContent);
        tvContent.setText(getContent());

        contentView.findViewById(R.id.btnIgnore).setOnClickListener(v -> {
            setBackResult(VALUE_IGNORE);
        });
        contentView.findViewById(R.id.btnRemeasureStation).setOnClickListener(v -> {
            setBackResult(VALUE_REMEASURE_STATION);
        });
        contentView.findViewById(R.id.btnRemeasureCurrent).setOnClickListener(v -> {
            setBackResult(VALUE_REMEASURE_POINT);
        });
        return contentView;
    }

    private void setBackResult(int clickType) {
        Bundle bundle = new Bundle();
        bundle.putInt(KEY_RESULT, clickType);
        getParentFragmentManager().setFragmentResult(KEY_BUNDLE, bundle);
        //关闭dialog
        dismissAllowingStateLoss();
    }

    @Override
    public void onStart() {
        super.onStart();
        Dialog dialog = getDialog();
        if (dialog == null) {
            return;
        }
        dialog.getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    private String getTitle() {
        Bundle arguments = getArguments();
        if (arguments != null) {
            return arguments.getString(KEY_TITLE);
        }
        return null;
    }

    private String getContent() {
        Bundle arguments = getArguments();
        if (arguments != null) {
            return arguments.getString(KEY_CONTENT);
        }
        return null;
    }


}
