package com.bingce.controlnetwork.func.level.export

import android.util.Log
import androidx.annotation.WorkerThread
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant
import com.bingce.controlapphelper.datasource.database.leveling.idetection.list.LevelIDetectionList
import com.bingce.controlapphelper.datasource.database.leveling.idetection.record.LevelIDetectionRecord
import com.bingce.controlapphelper.leveling.detaction.LevelFormatUtil
import com.bingce.controlapphelper.model.getLength
import com.bingce.controlnetwork.func.level.util.LevelDateUtil
import com.bingce.controlnetwork.func.level.util.LevelDateUtil.getLevelHeadDateHourMin

/**
 * i角检测导出
 */
class LevelExportIDetection {

    private var levelNoIndex = 0

    companion object {
        const val _N = "\r\n"
        const val _S = "\u0020"
        const val LEVEL_NO = "LEVEL|No"
        const val TO = "TO"
        const val INF = "INF"
        const val SHU = "|"
        const val CHECK_RECORD_SIZE = 20
    }


    @WorkerThread
    fun export(iDetection: LevelIDetectionList): String? {
        val recordList =
            LevelingDbFactory.instance.iDetectionRecordDataSource.getRecordsByIDetectionId(
                iDetection.id
            )

        if (recordList.isEmpty() || recordList.size != CHECK_RECORD_SIZE) {
            return null
        }
        if (iDetection.iResult == null) {
            return null
        }
        if (iDetection.shortDistance == null || iDetection.longDistance == null
            || iDetection.measureType == null
        ) {
            return null
        }
        val firstRecordCreateDate = recordList.first().createDate
        val dateYMD =
            LevelDateUtil.getLevelHeadDateYearMonthDay(firstRecordCreateDate)
        val dateHM = getLevelHeadDateHourMin(firstRecordCreateDate)

        val settting =
            LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(iDetection.id)

        val sb = StringBuilder()

        //startI
        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append(TO).append(getSpace2()).append("Start-I").append(getSpace10())
            .append(iDetection.measureType).append(SHU).append(getSpace30()).append(SHU)
            .append(getSpace30()).append(SHU).append(getSpace30()).append(SHU)
            .append(_N)

        levelNoIndexIncrease()

        //i角检测的设置
        appendSetting(sb, "instrumentModel", toString(settting.instrumentModel))
        appendSetting(sb, "deviceNumber", "No.${toString(settting.deviceNumber)}")
        appendSetting(sb, "dateYMD", dateYMD)
        appendSetting(sb, "dateHM", dateHM)
        appendSetting(sb, "rulerNum1", toString(settting.rulerNum1))
        appendSetting(sb, "rulerNum2", toString(settting.rulerNum2))
        appendSetting(sb, "imaging", toString(settting.imaging))
        appendSetting(sb, "watcher", toString(settting.watcher))
        appendSetting(sb, "recorder", toString(settting.recorder))
        appendSetting(sb, "examiner", toString(settting.examiner))
        //D1 D2
        appendD1D2(sb, "D1", iDetection.shortDistance)
        appendD1D2(sb, "D2", iDetection.longDistance)

        //先排个序
        recordList.sortWith(
            Comparator.comparing(LevelIDetectionRecord::stationType)
                .thenComparing(LevelIDetectionRecord::rulerType)
                .thenComparing(LevelIDetectionRecord::measureTime)
        )
        for (record in recordList) {
            Log.d(
                "hwhw",
                "" + record.stationType + "," + record.rulerType + "," + record.measureTime
            )
            appendRecord(sb, record)
        }
        //iResult
        val iResultUnit = if (iDetection.iResult != null) {
            "${iDetection.iResult} s"
        } else {
            ""
        }
        val spaceIResultLeft = getSpace(30 - 4 - iResultUnit.length - 1)
        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append("I").append(getSpace(29)).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(getSpace30()).append(SHU)
            .append("I").append(spaceIResultLeft).append(iResultUnit).append(getSpace4())
            .append(SHU)
            .append(_N)
        levelNoIndexIncrease()
        //endI
        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append(TO).append(getSpace(4)).append("End-I").append(getSpace(19)).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(_N)

        return sb.toString()
    }

    private fun levelNoIndexIncrease() {
        levelNoIndex++
    }

    private fun appendRecord(sb: StringBuilder, record: LevelIDetectionRecord) {
        val hourMinSec = LevelDateUtil.getHourMinSec(record.createDate)

        val height = LevelFormatUtil.formatHeight(record.height)
        val distance = LevelFormatUtil.formatDistance(record.distance)
        val heightUnit = "$height m"
        val distanceUnit = "$distance m"

        val space2 = getSpace2()
        val spaceHeightLeft = getSpace(24 - 2 - heightUnit.length)
        val spaceDistanceLeft = getSpace(28 - 2 - distanceUnit.length)

        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append("KD1").append(getSpace10()).append("${record.measureTime + 1}")
            .append(getSpace8()).append(hourMinSec).append(SHU)
            .append(getRI1A(record)).append(spaceHeightLeft).append(heightUnit).append(space2)
            .append(SHU)
            .append("HD").append(spaceDistanceLeft).append(distanceUnit).append(space2).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(_N)

        levelNoIndexIncrease()
    }

    private fun getRI1A(record: LevelIDetectionRecord): String {
        return if (record.stationType == LevelIDetectionConstant.STATION_TYPE_0) {
            if (record.rulerType == LevelIDetectionConstant.RULER_TYPE_0) {
                "R_I1_A"
            } else {
                "R_I1_B"
            }
        } else {
            if (record.rulerType == LevelIDetectionConstant.RULER_TYPE_0) {
                "R_I2_A"
            } else {
                "R_I2_B"
            }
        }
    }

    private fun appendD1D2(sb: StringBuilder, d1: String, d1Value: String) {
        val d1ValueFormat = LevelFormatUtil.formatHeight(toDouble(d1Value))
        val d1ValueUnit = "$d1ValueFormat m"

        val d1ValueUnitLength = d1ValueUnit.length

        val spaceValueLeft = getSpace(30 - 4 - d1ValueUnitLength - 1)
        val spaceValueRight = getSpace4()

        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append(d1).append(getSpace28()).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(getSpace30()).append(SHU)
            .append("D").append(spaceValueLeft).append(d1ValueUnit).append(spaceValueRight)
            .append(SHU)
            .append(_N)
        levelNoIndexIncrease()
    }

    private fun appendSetting(sb: StringBuilder, key: String, value: String) {
        sb.append(LEVEL_NO).append(getLevelNoRightSpace()).append(getNoIndexName()).append(SHU)
            .append(INF).append(getSpace27()).append(SHU)
            .append(key).append(getSettingKeyRightSpace(key)).append(SHU)
            .append(getSpace30()).append(SHU)
            .append(value).append(getSettingValueRightSpace(value)).append(SHU)
            .append(_N)
        levelNoIndexIncrease()
    }

    private fun getLevelNoRightSpace(): String {
        val length = getNoIndexName().length
        return getSpace(8 - length)
    }

    private fun getSettingKeyRightSpace(key: String): String {
        val length = key.length
        return getSpace(30 - length)
    }

    private fun getSettingValueRightSpace(value: String): String {
        val length = getLength(value)
        return getSpace(30 - length)
    }

    private fun getNoIndexName() = "${levelNoIndex + 1}"


    private fun getSpace2() = getSpace(2)
    private fun getSpace4() = getSpace(4)
    private fun getSpace8() = getSpace(8)
    private fun getSpace10() = getSpace(10)
    private fun getSpace27() = getSpace(27)
    private fun getSpace28() = getSpace(28)

    private fun getSpace30() = getSpace(30)

    private fun getSpace(size: Int) = _S.repeat(size)

    private fun toString(value: String?) = value ?: ""
    private fun toDouble(value: String?): Double {
        return value?.toDouble() ?: 0.0
    }


}