package com.bingce.controlnetwork.func.level.fragment;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlnetwork.R;

import blankj.utilcode.util.ToastUtils;


public class LevelInputFragment extends AppCompatDialogFragment {

    public static String KEY_DISTANCE = "KEY_DISTANCE";
    public static String KEY_HEIGHT = "KEY_HEIGHT";
    public static String KEY_REQUEST = "KEY_REQUEST";

    public static void show(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                            IConfirmCallback callback) {
        Bundle args = new Bundle();
        LevelInputFragment fragment = new LevelInputFragment();
        fragment.setArguments(args);

        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST.equals(requestKey)) {
                                if (callback != null) {
                                    double distance = result.getDouble(KEY_DISTANCE);
                                    double height = result.getDouble(KEY_HEIGHT);
                                    callback.onConfirm(distance, height);
                                }
                            }
                        });

        fragment.show(fragmentManager, null);

    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View contentView = inflater.inflate(R.layout.fragment_level_input, container, false);

        EditText etDisTance = contentView.findViewById(R.id.etDisTance);
        EditText etHeight = contentView.findViewById(R.id.etHeight);

        contentView.findViewById(R.id.btnConfirm).setOnClickListener(v -> {

            String disTance = getEtValue(etDisTance);
            String height = getEtValue(etHeight);
            if (TextUtils.isEmpty(disTance)) {
                ToastUtils.showShort("请输入距离");
                return;
            }
            if (TextUtils.isEmpty(height)) {
                ToastUtils.showShort("请输入高度");
                return;
            }

            Bundle bundle = new Bundle();
            bundle.putDouble(KEY_DISTANCE, Double.parseDouble(disTance));
            bundle.putDouble(KEY_HEIGHT, Double.parseDouble(height));
            getParentFragmentManager()
                    .setFragmentResult(KEY_REQUEST, bundle);
            dismissAllowingStateLoss();
        });

        return contentView;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        if (getDialog() == null) {
            return;
        }
        getDialog().getWindow().setLayout(
                WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    private String getEtValue(EditText et) {
        return et.getText().toString().trim();
    }

    public interface IConfirmCallback {
        void onConfirm(double distance, double height);
    }
}
