package com.bingce.controlnetwork.func.level.fragment

import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.EditText
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.func.level.constant.LevelConstant
import com.bingce.controlnetwork.func.level.constant.LevelStationSettingContant
import com.bingce.controlnetwork.func.level.util.LevelStationSettingUtil.getSpinnerText
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.model.LevelRouteSave
import com.bingce.controlapphelper.model.LevelingMeasureOrder
import com.bingce.controlapphelper.model.LevelingMeasureOrderUtil
import com.bingce.controlapphelper.model.PickPointResult
import com.bingce.controlapphelper.util.CheckLocationUtil
import com.bingce.controlapphelper.util.GetLocationUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity
import com.bingce.controlnetwork.databinding.FragmentLevelCreateRouteBinding
import com.bingce.controlnetwork.fragment.PickPointFragment
import com.bingce.controlnetwork.func.level.viewmodel.LevelCreateOrEditRouteVm
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.newui.projectedit.ProjectLastData
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 新建编辑测段
 */
class LevelRouteCreateEditFragment : BaseFragmentBinding<FragmentLevelCreateRouteBinding>() {

    private val viewModel by viewModels<LevelCreateOrEditRouteVm>()

    private val SELECT_POINT_TYPE_START = "SELECT_POINT_TYPE_START"
    private val SELECT_POINT_TYPE_END = "SELECT_POINT_TYPE_END"

    private var pickPointResultStart: PickPointResult? = null
    private var pickPointResultEnd: PickPointResult? = null

    private lateinit var levelingMeasureOrder: LevelingMeasureOrder

    private val getLocation by lazy {
        GetLocationUtil()
    }


    private val defaultStationLauncher = PickPointFragment.createActivityLauncher(
        this
    ) { type: String, pickPointResult: PickPointResult ->
        if (SELECT_POINT_TYPE_START == type) {
            pickPointResultStart = pickPointResult
        } else if (SELECT_POINT_TYPE_END == type) {
            pickPointResultEnd = pickPointResult
        }
        if (pickPointResult.selectedIds.isNullOrEmpty()
            || pickPointResult.selectedNames.isNullOrEmpty()
            || pickPointResult.selectedNames.size != pickPointResult.selectedIds.size
        ) {
            ToastUtils.showShort(R.string.abnormal_point_selection)
            return@createActivityLauncher
        }
        setSelectPointUi()
    }


    companion object {
        @JvmField
        val TAG = LevelRouteCreateEditFragment::class.java.getName()

        @JvmStatic
        fun newInstance(args: Bundle?): Fragment {
            val fragment: Fragment = LevelRouteCreateEditFragment()
            fragment.setArguments(args)
            return fragment
        }

        @JvmStatic
        fun start(context: Context, projectId: String, routeId: String?, title: String) {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId)
            bundle.putString(BundleConstants.KEY_ROUTE_ID, routeId)
            FragmentContainerActivity.start(context, TAG, title, bundle)
        }
    }

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentLevelCreateRouteBinding {
        return FragmentLevelCreateRouteBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        lifecycle.addObserver(getLocation)
        initLevelOrder()
        initListener()
    }

    private fun initListener() {
        mBinding.btnConfirm.setOnClickListener {
            checkConfirm()
        }
        mBinding.btnDetermineAndMeasure.setOnClickListener {

        }

        mBinding.ivSelectStartPoint.setOnClickListener {
            selectPoint(SELECT_POINT_TYPE_START)
        }
        mBinding.ivSelectEndPoint.setOnClickListener {
            selectPoint(SELECT_POINT_TYPE_END)
        }

        mBinding.ivGetLocation.setOnClickListener {
            getLocation {
                setLocation(mBinding.etLongitude, mBinding.etLatitude, mBinding.etHeight, it)
            }
        }
    }

    override fun initData() {
        initSpinnerObservationDirection()

        initEditData()
    }

    private fun initEditData() {
        val routeId = getRouteId()
        if (routeId != null) {
            viewModel.getEditRouteData(routeId) {
                mBinding.etStartPoint.setText(it.startPointName)
                mBinding.etEndPoint.setText(it.endPointName)
                mBinding.spinnerObservationDirection.setSelection(
                    LevelStationSettingContant.observationDirectionList.indexOf(
                        it.observationDirection
                    )
                )
                val spinnerShowPosByLevelingMeasureOrder =
                    LevelingMeasureOrderUtil.getSpinnerShowPosByLevelingMeasureOrderPla(it.levelingMeasureOrder)

                mBinding.spinner.setSelection(spinnerShowPosByLevelingMeasureOrder)
//                //经纬度
//                if (isEmpty(it.longitude) || isEmpty(it.latitude)) {
//                    setDefaultLocation()
//                } else {
//                    setLocation(
//                        mBinding.etLongitude,
//                        mBinding.etLatitude,
//                        mBinding.etHeight,
//                        it.longitude,
//                        it.latitude,
//                        it.z
//                    )
//                }

            }
        } else {
            setDefaultLocation()
        }
    }

    private fun setSelectPointUi() {
        if (pickPointResultStart == null) {
            mBinding.etStartPoint.setText("")
        } else {
            mBinding.etStartPoint.setText(pickPointResultStart!!.selectedNames[0])
        }
        if (pickPointResultEnd == null) {
            mBinding.etEndPoint.setText("")
        } else {
            mBinding.etEndPoint.setText(pickPointResultEnd!!.selectedNames[0])
        }
    }

    private fun selectPoint(type: String) {
        var title: String? = null
        var pickPointResultSelected: PickPointResult? = null
        var filderIds: List<String>? = null
        if (SELECT_POINT_TYPE_START == type) {
            pickPointResultSelected = pickPointResultStart
            filderIds = pickPointResultEnd?.selectedIds
            title = getString(R.string.choose_starting_point)
        } else if (SELECT_POINT_TYPE_END == type) {
            pickPointResultSelected = pickPointResultStart
            filderIds = pickPointResultStart?.selectedIds
            title = getString(R.string.select_end_point)
        }
        PickPointFragment.pickKnownAndUnknownPoint(
            context,
            type,
            1,
            title,
            filderIds,
            getProjectId(),
            pickPointResultSelected,
            defaultStationLauncher
        )
    }

    /**
     * 照准次序
     */
    private fun initLevelOrder() {
        val showList = LevelingMeasureOrderUtil.getShowListPla()
        mBinding.spinner.setAdapter(
            ArrayAdapter(
                requireContext(),
                android.R.layout.simple_list_item_1,
                showList
            )
        )
        mBinding.spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parent: AdapterView<*>?,
                view: View,
                position: Int,
                id: Long
            ) {
                levelingMeasureOrder = LevelingMeasureOrderUtil.getLevelingMeasureOrderByCreate(
                    showList[position]
                )
            }

            override fun onNothingSelected(parent: AdapterView<*>?) {}
        }
    }

    /**
     * 观测方向
     */
    private fun initSpinnerObservationDirection() {
        mBinding.spinnerObservationDirection.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.observationDirectionList
        )
        mBinding.spinnerObservationDirection.onItemSelectedListener =
            object : AdapterView.OnItemSelectedListener {
                override fun onItemSelected(
                    parent: AdapterView<*>?,
                    view: View,
                    position: Int,
                    id: Long
                ) {

                }

                override fun onNothingSelected(parent: AdapterView<*>?) {

                }
            }
    }

    private fun checkConfirm() {
        val projectId = getProjectId()
        if (projectId == null) {
            ToastUtils.showShort(R.string.project_data_is_empty)
            return
        }

        val startPointName = mBinding.etStartPoint.getText().toString().trim()
        if (TextUtils.isEmpty(startPointName)) {
            ToastUtils.showShort(R.string.please_enter_or_select_a_starting_point)
            return
        }
        val endPointName = mBinding.etEndPoint.getText().toString().trim()
        if (TextUtils.isEmpty(endPointName)) {
            ToastUtils.showShort(R.string.please_enter_or_select_a_destination)
            return
        }

//        val longitude = mBinding.etLongitude.getText().toString().trim()
//        if (TextUtils.isEmpty(longitude)) {
//            ToastUtils.showShort(R.string.please_input_longitude)
//            return
//        }
//        val latitude = mBinding.etLatitude.getText().toString().trim()
//        if (TextUtils.isEmpty(latitude)) {
//            ToastUtils.showShort(R.string.please_input_latitude)
//            return
//        }
//        val height = mBinding.etHeight.getText().toString().trim()
//        if (TextUtils.isEmpty(height)) {
//            ToastUtils.showShort(R.string.please_enter_elevation)
//            return
//        }

        val levelRouteSave = LevelRouteSave()
        levelRouteSave.projectId = projectId
        levelRouteSave.routeId = getRouteId()
        levelRouteSave.routeName = "$startPointName-$endPointName"
        levelRouteSave.startPointName = startPointName
        levelRouteSave.endPointName = endPointName
        levelRouteSave.levelingMeasureOrder = levelingMeasureOrder

        //观测方向
        levelRouteSave.observationDirection = getSpinnerText(mBinding.spinnerObservationDirection)

        viewModel.startSave(requireContext(), levelRouteSave) { isSuccess: Boolean ->
            if (isSuccess) requireActivity().finish()
        }
    }

    private fun getProjectId() = getArgumentString(BundleConstants.KEY_PROJECT_ID)
    private fun getRouteId() = getArgumentString(BundleConstants.KEY_ROUTE_ID)


    private fun getLocation(resultListener: GetLocationUtil.IResultListener) {
        CheckLocationUtil.check(requireContext()) {
            getLocation.getResult(resultListener)
        }
    }

    private fun setLocation(
        etLongitude: EditText,
        etLatitude: EditText,
        etHeight: EditText,
        result: GetLocationUtil.Result?
    ) {
        setLocation(
            etLongitude,
            etLatitude,
            etHeight,
            result?.longitude,
            result?.latitude,
            result?.altitude
        )
    }

    private fun setLocation(
        etLongitude: EditText,
        etLatitude: EditText,
        etHeight: EditText,
        longitude: String?,
        latitude: String?,
        z: String?
    ) {
        etLongitude.setText(longitude)
        etLatitude.setText(latitude)
        etHeight.setText(z)
    }

    private fun setDefaultLocation() {
        lifecycleScope.launch(Dispatchers.IO) {
            val longitude = ProjectLastData.getLongitude()
            val latitude = ProjectLastData.getLatitude()
            withContext(Dispatchers.Main) {
                if (isEmpty(latitude) || isEmpty(latitude)) {
                    setLocation(
                        mBinding.etLongitude,
                        mBinding.etLatitude,
                        mBinding.etHeight,
                        LevelConstant.LONGITUDE_DEFAULT.toString(),
                        LevelConstant.LATITUDE_DEFAULT.toString(),
                        LevelConstant.HEIGHT_DEFAULT.toString()
                    )
                } else {
                    setLocation(
                        mBinding.etLongitude,
                        mBinding.etLatitude,
                        mBinding.etHeight,
                        longitude,
                        latitude,
                        LevelConstant.HEIGHT_DEFAULT.toString()
                    )
                }
            }
        }
    }

    private fun isEmpty(value: String?) = TextUtils.isEmpty(value)

}