package com.bingce.controlnetwork.func.level.fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.lifecycleScope
import androidx.room.withTransaction
import com.bingce.controlapphelper.datasource.database.SurveyorDataBase
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.route.LevelRouteRecord
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.func.level.viewmodel.LevelCreateOrEditRouteVm
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuDeleteEdit
import com.bingce.controlnetwork.databinding.FragmentListLayoutBinding
import com.bingce.controlnetwork.fragment.TipsFragment
import com.bingce.controlnetwork.func.level.adapter.LevelRouteListAdapter
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class LevelRouteFragment : Fragment() {

    private val createOrEditRouteVm by viewModels<LevelCreateOrEditRouteVm>()

    private var _binding: FragmentListLayoutBinding? = null
    val mBinding get() = _binding!!

    private val dataList = mutableListOf<LevelRouteRecord>()

    private val adapter by lazy {
        LevelRouteListAdapter(requireContext())
    }

    private fun doLookAndEditRoute(item: LevelRouteRecord) {
        createOrEditRouteVm.showCreateOrEditDialog(
            requireContext(),
            getProjectId(),
            routeId = item.getId()
        )
    }

    private fun doDelete(item: LevelRouteRecord) {
        lifecycleScope.launch(Dispatchers.IO) {

            val stationList = SurveyorDatabaseFactory.instance.surveyorStation.getByProjectRoute(
                item.projectId,
                item.getId()
            )

            var hasData = false
            for (stationItem in stationList) {
                val recordCount =
                    SurveyorDatabaseFactory.instance.surveyorRecordDataSource.recordCount(
                        stationItem.getId()
                    )
                if (recordCount > 0) {
                    hasData = true
                    break
                }
            }

            if (hasData) {
                withContext(Dispatchers.Main) {
                    TipsFragment.cancelAbleTips(
                        childFragmentManager, viewLifecycleOwner, context,
                        getString(R.string.tips_is_sure_to_delete_route_with_record),
                        4
                    ) {
                        deleteDatabase(item)
                    }
                }
            } else {
                deleteDatabase(item)
            }
        }
    }

    private fun deleteDatabase(routeRecord: LevelRouteRecord) {
        lifecycleScope.launch(Dispatchers.IO) {
            SurveyorDataBase.getInstance().withTransaction {
                //删除路线
                LevelingDbFactory.instance.levelDataSource.delete(routeRecord)
                //删除测站
                LevelingDbFactory.instance.stationDataSource.deleteByRouteId(routeRecord.getId())
            }
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentListLayoutBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initAdapter()

        LevelingDbFactory.instance.levelDataSource.getListLiveData(getProjectId())
            .observe(viewLifecycleOwner) {
                adapter.refreshData(it)
            }
    }

    private fun initAdapter() {
        mBinding.list.adapter = adapter

        adapter.setAdapterContextMenu(object : ContextMenuDeleteEdit<LevelRouteRecord>() {
            override fun delete(item: LevelRouteRecord) {
                doDelete(item)
            }

            override fun edit(item: LevelRouteRecord) {
                doLookAndEditRoute(item)
            }
        })
    }


    fun getProjectId() = requireArguments().getString(BundleConstants.KEY_PROJECT_ID)!!


    companion object {
        @JvmStatic
        fun newInstance(projectId: String): LevelRouteFragment {
            val bundle = Bundle()
            bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId)
            val routeFragment = LevelRouteFragment()
            routeFragment.arguments = bundle
            return routeFragment
        }
    }

    /**
     * 获取即将要测量的路线id
     */
    fun getSurveyorRouteId(): String? {
        if (dataList.isNotEmpty()) {
            return dataList.last().getId()
        }
        return null
    }


}