package com.bingce.controlnetwork.func.level.fragment;

import android.content.Context;
import android.os.Bundle;
import android.os.Environment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;

import com.bingce.controlapphelper.util.CheckStorageUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlapphelper.widget.treeview.TreeNode;
import com.bingce.controlapphelper.widget.treeview.TreeView;
import com.bingce.controlapphelper.widget.treeview.custom.MyNodeViewFactory;
import com.bingce.controlnetwork.BuildConfig;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.databinding.FragmentSelectExportBinding;
import com.bingce.controlnetwork.fragment.FullLoadingFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.func.level.util.LevelExportUtilTj;
import com.bingce.controlnetwork.func.level.viewmodel.LevelSelectExportVm;
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding;
import com.bingce.controlnetwork.surveyor.method.ExportUtilsPla;
import com.bingce.file.FileOperator;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.zip.ZipUtil;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;

import blankj.utilcode.util.FileUtils;
import blankj.utilcode.util.ToastUtils;


/**
 * 水准选择导出
 */
public class LevelSelectExportFragment extends BaseFragmentBinding<FragmentSelectExportBinding> {
    public static final String TAG = LevelSelectExportFragment.class.getName();

    private LevelSelectExportVm viewModel;

    private TreeNode rootNode;

    public static LevelSelectExportFragment newInstance(Bundle args) {
        LevelSelectExportFragment fragment = new LevelSelectExportFragment();
        fragment.setArguments(args);
        return fragment;
    }

    public static void start(Context context) {
        FragmentContainerActivity.start(context, TAG, R.string.level_export, null);
    }

    @Override
    protected FragmentSelectExportBinding getViewBinding(@NonNull LayoutInflater inflater, ViewGroup container) {
        return FragmentSelectExportBinding.inflate(inflater, container, false);
    }

    @Override
    protected void initView() {
        viewModel = new ViewModelProvider(this).get(LevelSelectExportVm.class);
        mBinding.title.setText(R.string.tips_select_expected_export_data_level);
        initListener();
    }

    private void initListener() {
        mBinding.export.setOnClickListener(v -> {
            if (ExportUtilsPla.isCanExport()) {
                CheckStorageUtil.check(requireContext(), this::startExport);
            }
        });
    }

    private void startExport() {
        DialogFragment dialogFragment = FullLoadingFragment.showLoading(getChildFragmentManager());

        doExport(new ICallback() {
            @Override
            public void onFailureInMainThread(String msg) {
                dialogFragment.dismiss();
                ToastUtils.showShort(msg);
            }

            @Override
            public void onSuccessInMainThread(File zip) {
                if (dialogFragment.isVisible()) {
                    dialogFragment.dismiss();
                }
                TipsWith2OptionFragment.tipsCannotCancel(getChildFragmentManager(), getViewLifecycleOwner(),
                        getString(R.string.share),
                        getString(R.string.cancel),
                        getString(R.string.export_successful),
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                FileOperator.shareFile(getContext(), zip, BuildConfig.APPLICATION_ID);
                            }
                        });
            }
        });
    }

    private void doExport(ICallback callback) {
        List<TreeNode> projectList = rootNode.getChildren();

        ThreadPoolUtil.execute(() -> {
            List<ExportRoute> exportRouteList = new ArrayList<>();
            HashSet<String> exportProjectIdSet = new HashSet<>();
            for (TreeNode projectNode : projectList) {
                List<TreeNode> children = projectNode.getChildren();
                for (TreeNode routeNode : children) {
                    if (routeNode.isEnable() && routeNode.isSelected()) {
                        String projectId = projectNode.getValue().getId();
                        String projectName = projectNode.getValue().getValue().toString();
                        exportProjectIdSet.add(projectId);
                        exportRouteList.add(new ExportRoute(projectId, projectName,
                                routeNode.getValue().getId(),
                                routeNode.getValue().getValue().toString()));
                    }
                }
            }
            if (exportRouteList.isEmpty()) {
                backError(callback, "没有要导出的数据");
                return;
            }

            try {
                LevelExportUtilTj levelExportUtilTj = new LevelExportUtilTj();
                //导出路线数据
                for (ExportRoute exportRoute : exportRouteList) {
                    levelExportUtilTj.doExportRouteData(exportRoute.projectId,
                            exportRoute.routeId);
                }

                for (String projectId : exportProjectIdSet) {
                    //导出项目数据
                    levelExportUtilTj.exportProjectTxt(projectId);
                    //导出点信息
                    levelExportUtilTj.exportControlTxt(projectId);
                    //导出i角检测
                    levelExportUtilTj.exportIDetection(projectId);
                    //导出SingalHeight和平差准备文件
                    levelExportUtilTj.exportTjSingalHeight(projectId);
                }

            } catch (Exception e) {
                e.printStackTrace();
                backError(callback, Tools.getString(R.string.export_failure));
                File sourceFileFolder = LevelExportUtilTj.Companion.getExportOutFolder();
                FileUtils.delete(sourceFileFolder);
                return;
            }

            //压缩文件
            File file = startZipFile();
            if (file == null) {
                return;
            }
            ThreadPoolUtil.executeInMain(() -> callback.onSuccessInMainThread(file));
        });
    }

    private void backError(ICallback callback, String msg) {
        ThreadPoolUtil.executeInMain(() -> callback.onFailureInMainThread(msg));
    }

    private File startZipFile() {
        //源数据文件
        File sourceFileFolder = LevelExportUtilTj.Companion.getExportOutFolder();
        if (!sourceFileFolder.exists()) return null;
        File[] files = sourceFileFolder.listFiles();
        if (files == null || files.length == 0) return null;
        List<File> sourceFileList = Arrays.asList(files);

        //将文件存储到一个zip中
        File zipFolder = new File(Environment.getExternalStorageDirectory(), ExportUtilsPla.getExportFirstDirectory());
        if (!zipFolder.exists()) {
            zipFolder.mkdir();
        }

        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss", Locale.getDefault());
        String dateString = format.format(date);
        File zip = new File(zipFolder, "水准数据_" + dateString + ".zip");
        if (!ZipUtil.zip(zip, sourceFileList)) {
            ToastUtils.showShort(R.string.export_failure);
            return null;
        }
        try {
            //存储成功，删除被压缩的源文件
            FileUtils.delete(sourceFileFolder);
        } catch (Exception e) {
            e.printStackTrace();
            ToastUtils.showShort(R.string.export_failure);
            return null;
        }
        return zip;
    }

    @Override
    protected void initData() {
        viewModel.initData(treeNode -> {
            setData(treeNode);
            return null;
        });
    }

    private void setData(TreeNode treeNode) {
        rootNode = treeNode;
        TreeView treeView = new TreeView(treeNode, requireContext(), new MyNodeViewFactory());
        treeView.setup(mBinding.list);
        mBinding.list.setVisibility(View.VISIBLE);
        mBinding.loading.setVisibility(View.GONE);
    }

    private interface ICallback {
        void onFailureInMainThread(String msg);

        void onSuccessInMainThread(File zip);
    }

    private static class ExportRoute {
        public String projectId;
        public String projectName;
        public String routeId;
        public String routeName;

        public ExportRoute(String projectId, String projectName, String routeId, String routeName) {
            this.projectId = projectId;
            this.projectName = projectName;
            this.routeId = routeId;
            this.routeName = routeName;
        }
    }

}
