package com.bingce.controlnetwork.func.level.fragment

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ArrayAdapter
import androidx.fragment.app.activityViewModels
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.func.level.constant.LevelStationSettingContant
import com.bingce.controlnetwork.func.level.util.LevelStationSettingUtil
import com.bingce.controlnetwork.func.level.viewmodel.LevelStationSettingVm
import com.bingce.controlapphelper.model.LevelRouteSave
import com.bingce.controlapphelper.surveyor.method.model.MeasureConstant
import com.bingce.controlnetwork.databinding.FragmentLevelStationSettingConditionBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.newui.stationsetting.IStationSettingOperate

/**
 * 水准测站设置-观测条件
 */
class LevelStationSettingConditionFragment :
    BaseFragmentBinding<FragmentLevelStationSettingConditionBinding>(), IStationSettingOperate {

    private val stationSettingVm by activityViewModels<LevelStationSettingVm>()

    private lateinit var levelRouteSave: LevelRouteSave

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentLevelStationSettingConditionBinding {
        return FragmentLevelStationSettingConditionBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        initEditTextTitle()
    }

    private fun initEditTextTitle() {
        mBinding.ilTemperature.tvTitle.text = "温度（℃）"
    }

    override fun initData() {
        initSpinnerWeather()
        initSpinnerImaging()
        initSpinnerSunDirection()
        initSpinnerWindDirection()
        initSpinnerWindSpeed()
        initSpinnerRoads()
        initSpinnerSoil()

        stationSettingVm.editLastDataLiveData.observe(viewLifecycleOwner) {
            if (it == null) {
                setTemperatureData(MeasureConstant.DEFAULT_TEMPERATURE)
            } else {
                setData(it)
            }
        }
    }

    private fun initSpinnerSoil() {
        mBinding.spinnerSoil.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.soilList
        )
    }

    private fun initSpinnerRoads() {
        mBinding.spinnerRoads.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.roadsList
        )
    }

    private fun initSpinnerWindSpeed() {
        mBinding.spinnerWindSpeed.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.windSpeedList
        )
    }

    private fun initSpinnerWindDirection() {
        mBinding.spinnerWindDirection.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.windDirectionList
        )
    }

    private fun initSpinnerSunDirection() {
        mBinding.spinnerSunDirection.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.sunDirectionList
        )
    }

    private fun initSpinnerImaging() {
        mBinding.spinnerImaging.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.imagingList
        )
    }

    private fun initSpinnerWeather() {
        mBinding.spinnerWeather.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.weatherList
        )
    }

    override fun checkSaveNot(): Boolean {
        if (isEmpty(mBinding.ilTemperature.editText.text.toString().trim())) {
            ToastUtils.showShort("请输入温度")
            return true
        }
        collectData()
        return false
    }

    override fun setDataToStation() {
        stationSettingVm.addConditionDataToLevelRouteSave(levelRouteSave)
    }

    private fun collectData() {
        levelRouteSave = LevelRouteSave()
        levelRouteSave.temperature = mBinding.ilTemperature.editText.text.toString().trim()
        levelRouteSave.weather = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerWeather)
        levelRouteSave.imaging = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerImaging)
        levelRouteSave.sunDirection =
            LevelStationSettingUtil.getSpinnerText(mBinding.spinnerSunDirection)
        levelRouteSave.windDirection =
            LevelStationSettingUtil.getSpinnerText(mBinding.spinnerWindDirection)
        levelRouteSave.windSpeed = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerWindSpeed)
        levelRouteSave.roads = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerRoads)
        levelRouteSave.soil = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerSoil)
    }

    private fun isEmpty(value: String?) = TextUtils.isEmpty(value)

    private fun setData(levelRouteSave: LevelRouteSave) {
        setTemperatureData(levelRouteSave.temperature)

        mBinding.spinnerWeather.setSelection(
            LevelStationSettingContant.weatherList.indexOf(
                levelRouteSave.weather
            )
        )
        mBinding.spinnerImaging.setSelection(
            LevelStationSettingContant.imagingList.indexOf(
                levelRouteSave.imaging
            )
        )
        mBinding.spinnerSunDirection.setSelection(
            LevelStationSettingContant.sunDirectionList.indexOf(
                levelRouteSave.sunDirection
            )
        )
        mBinding.spinnerWindDirection.setSelection(
            LevelStationSettingContant.windDirectionList.indexOf(
                levelRouteSave.windDirection
            )
        )
        mBinding.spinnerWindSpeed.setSelection(
            LevelStationSettingContant.windSpeedList.indexOf(
                levelRouteSave.windSpeed
            )
        )
        mBinding.spinnerRoads.setSelection(
            LevelStationSettingContant.roadsList.indexOf(
                levelRouteSave.roads
            )
        )
        mBinding.spinnerSoil.setSelection(LevelStationSettingContant.soilList.indexOf(levelRouteSave.soil))
    }

    private fun setTemperatureData(value: String?) {
        mBinding.ilTemperature.editText.setText(value)
    }
}