package com.bingce.controlnetwork.func.level.fragment

import android.text.TextUtils
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.TextView
import androidx.fragment.app.activityViewModels
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.func.level.constant.LevelStationSettingContant
import com.bingce.controlnetwork.func.level.util.LevelStationSettingUtil
import com.bingce.controlnetwork.func.level.viewmodel.LevelStationSettingVm
import com.bingce.controlapphelper.model.LevelRouteSave
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.FragmentLevelStationSettingInformationBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.newui.stationsetting.IStationSettingOperate
import com.bingce.controlnetwork.widget.editspinner.EditSpinner
import com.bingce.controlnetwork.widget.editspinner.SimpleAdapter

/**
 * 水准测站设置-观测信息
 */
class LevelStationSettingInformationFragment :
    BaseFragmentBinding<FragmentLevelStationSettingInformationBinding>(), IStationSettingOperate {

    private val viewModel by activityViewModels<LevelStationSettingVm>()

    private lateinit var levelRouteSave: LevelRouteSave

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentLevelStationSettingInformationBinding {
        return FragmentLevelStationSettingInformationBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        initEditTextTitle()
        initSpinnerRulerType()
        initRulerReceptType()
        viewModel.initHasTwoRuler { hasTwoRuler ->
            if (hasTwoRuler) {
                mBinding.ilFirstStationBackFootNumber.tvTitle.setText(R.string.first_station_back_foot_number_l)
                mBinding.ilFirstStationFrontFootNumber.tvTitle.setText(R.string.first_station_front_foot_number_l)
                mBinding.llRuler2.visibility = View.VISIBLE
            } else {
                mBinding.llRuler2.visibility = View.GONE
            }
        }
    }

    /**
     * 初始化editText坐标的title
     */
    private fun initEditTextTitle() {
        mBinding.ilWatcher.tvTitle.text = "观测者"
        mBinding.ilRecorder.tvTitle.text = "记录者"
        mBinding.ilExaminer.tvTitle.text = "检查者"
        mBinding.ilFirstStationBackFootNumber.tvTitle.text =
            Tools.getString(R.string.first_station_back_foot_number)
        mBinding.ilFirstStationFrontFootNumber.tvTitle.text =
            Tools.getString(R.string.first_station_front_foot_number)
        mBinding.ilFirstStationBackFootNumber2.tvTitle.text =
            Tools.getString(R.string.first_station_back_foot_number_r)
        mBinding.ilFirstStationFrontFootNumber2.tvTitle.text =
            Tools.getString(R.string.first_station_front_foot_number_r)
        mBinding.ilInstrumentModel.tvTitle.text = "仪器型号"
        mBinding.ilDeviceNumber.tvTitle.text = "仪器编号"
        mBinding.ilSteelRuleParam.tvTitle.text = "标尺因瓦带膨胀系数"
        mBinding.ilRulerCorrectionFactor.tvTitle.text = "标尺改正系数(mm/m)"
        mBinding.ilRulerLengthProvingTemperature.tvTitle.text = "标尺长度检定温度（℃）"
    }

    private fun initRulerReceptType() {
        initEditSpinner(
            mBinding.ilEditSpinnerRulerRecept.tvName,
            "尺承类型",
            mBinding.ilEditSpinnerRulerRecept.etSpinner,
            "请输入或选择"
        )
        val adapter =
            SimpleAdapter(requireContext(), LevelStationSettingContant.rulerReceptTypeList)
        mBinding.ilEditSpinnerRulerRecept.etSpinner.setAdapter(adapter)
    }

    private fun initSpinnerRulerType() {
        mBinding.spinnerRulerType.adapter = ArrayAdapter(
            requireContext(),
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.rulerTypeList
        )
    }

    override fun initData() {
        viewModel.editLastDataLiveData.observe(viewLifecycleOwner) {
            if (it == null) {
                setRulerReceptTypeData("不足5kg尺台")
            } else {
                setData(it)
            }
        }
    }

    override fun checkSaveNot(): Boolean {
        collectData()
        if (isEmpty(levelRouteSave.frontFootNumber)) {
            ToastUtils.showShort(R.string.first_station_front_foot_number_hint)
            return true
        }
        if (isEmpty(levelRouteSave.backFootNumber)) {
            ToastUtils.showShort(R.string.first_station_back_foot_number_hint)
            return true
        }

        if (mBinding.llRuler2.visibility == View.VISIBLE) {
            if (isEmpty(levelRouteSave.frontFootNumber2)) {
                ToastUtils.showShort(R.string.first_station_front_foot_number_hint)
                return true
            }
            if (isEmpty(levelRouteSave.backFootNumber2)) {
                ToastUtils.showShort(R.string.first_station_back_foot_number_hint)
                return true
            }
        }

        return false
    }

    override fun setDataToStation() {
        viewModel.addInforDataToLevelRouteSave(levelRouteSave)
    }

    private fun collectData() {
        levelRouteSave = LevelRouteSave()
        levelRouteSave.watcher = mBinding.ilWatcher.editText.text.toString().trim()
        levelRouteSave.recorder = mBinding.ilRecorder.editText.text.toString().trim()
        levelRouteSave.examiner = mBinding.ilExaminer.editText.text.toString().trim()
        levelRouteSave.frontFootNumber =
            mBinding.ilFirstStationFrontFootNumber.editText.text.toString().trim()
        levelRouteSave.backFootNumber =
            mBinding.ilFirstStationBackFootNumber.editText.text.toString().trim()
        if (mBinding.llRuler2.visibility == View.VISIBLE) {
            levelRouteSave.frontFootNumber2 =
                mBinding.ilFirstStationFrontFootNumber2.editText.text.toString().trim()
            levelRouteSave.backFootNumber2 =
                mBinding.ilFirstStationBackFootNumber2.editText.text.toString().trim()
        }
        levelRouteSave.instrumentModel = mBinding.ilInstrumentModel.editText.text.toString().trim()
        levelRouteSave.deviceNumber = mBinding.ilDeviceNumber.editText.text.toString().trim()
        levelRouteSave.steelRuleParam = mBinding.ilSteelRuleParam.editText.text.toString().trim()
        levelRouteSave.rulerCorrectionFactor =
            mBinding.ilRulerCorrectionFactor.editText.text.toString().trim()
        levelRouteSave.rulerLengthProvingTemperature =
            mBinding.ilRulerLengthProvingTemperature.editText.text.toString().trim()
        levelRouteSave.rulerType = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerRulerType)
        levelRouteSave.rulerReceptType = mBinding.ilEditSpinnerRulerRecept.etSpinner.text.trim()
    }

    private fun isEmpty(value: String?) = TextUtils.isEmpty(value)

    private fun setData(levelRouteSave: LevelRouteSave) {
        mBinding.ilWatcher.editText.setText(levelRouteSave.watcher)
        mBinding.ilRecorder.editText.setText(levelRouteSave.recorder)
        mBinding.ilExaminer.editText.setText(levelRouteSave.examiner)
        mBinding.ilFirstStationBackFootNumber.editText.setText(levelRouteSave.backFootNumber)
        mBinding.ilFirstStationFrontFootNumber.editText.setText(levelRouteSave.frontFootNumber)
        mBinding.ilFirstStationBackFootNumber2.editText.setText(levelRouteSave.backFootNumber2)
        mBinding.ilFirstStationFrontFootNumber2.editText.setText(levelRouteSave.frontFootNumber2)
        mBinding.ilInstrumentModel.editText.setText(levelRouteSave.instrumentModel)
        mBinding.ilDeviceNumber.editText.setText(levelRouteSave.deviceNumber)
        mBinding.ilSteelRuleParam.editText.setText(levelRouteSave.steelRuleParam)
        mBinding.ilRulerCorrectionFactor.editText.setText(levelRouteSave.rulerCorrectionFactor)
        mBinding.ilRulerLengthProvingTemperature.editText.setText(levelRouteSave.rulerLengthProvingTemperature)

        mBinding.spinnerRulerType.setSelection(
            LevelStationSettingContant.rulerTypeList.indexOf(
                levelRouteSave.rulerType
            )
        )
        setRulerReceptTypeData(levelRouteSave.rulerReceptType)
    }

    private fun setRulerReceptTypeData(value: String?) {
        mBinding.ilEditSpinnerRulerRecept.etSpinner.text = value
    }

    private fun initEditSpinner(
        textView: TextView,
        textViewName: String,
        view: EditSpinner,
        textHint: String
    ) {
        textView.text = textViewName
        view.setHint(textHint)
        view.setEtGravity(Gravity.CENTER_VERTICAL or Gravity.END)
    }
}