package com.bingce.controlnetwork.func.level.fragment;


import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentStatePagerAdapter;
import androidx.lifecycle.ViewModelProvider;
import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;

import com.bingce.controlapphelper.datasource.database.leveling.station.LevelingStationRecord;
import com.bingce.controlapphelper.leveling.detaction.ILevelRefreshBtn;
import com.bingce.controlapphelper.leveling.detaction.INextStation;
import com.bingce.controlapphelper.leveling.detaction.LevelCheckBtnUtil;
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.surveyor.test.Test;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.databinding.FragmentLevelSurveyorBinding;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.func.level.activity.LevelStationSettingActivity;
import com.bingce.controlnetwork.func.level.viewmodel.LevelSuveyorVm;
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding;
import com.bingce.controlnetwork.util.DialogUtil;
import com.bingce.controlnetwork.util.StationSettingCheckUtil;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;
import com.bingce.controlnetwork.func.level.sendreceive.ILevelBackData;
import com.bingce.controlnetwork.func.level.sendreceive.LevelRequest;
import com.bingce.controlnetwork.func.level.sendreceive.LevelRequestRelease;
import com.bingce.controlnetwork.func.level.sendreceive.LevelRequestTest;

/**
 * 水准测量页
 */
public class LevelSurveyFragment extends BaseFragmentBinding<FragmentLevelSurveyorBinding> {
    public static final String TAG = LevelSurveyFragment.class.getName();

    public static final String KEY_STATION_INDEX = "STATION_INDEX";
    private LevelSuveyorVm viewModel;
    private List<LevelSurveySubFragment> fragments;
    private MenuItem rollBackBtn;
    private MenuItem reMeasureStationBtn;

    private StationSettingCheckUtil stationSettingCheckUtil;

    private final INextStation nextStationI = this::showSelectLevelNextStepDialog;

    private LevelRequest levelRequest;
    private final ILevelBackData levelBackData = new ILevelBackData() {
        @Override
        public void receive(@NonNull LevelReceiveData data) {
            receiveData(data);
        }

        @Override
        public void measureError(@NonNull String msg) {
            if (getView() == null) {
                return;
            }
            DialogUtil.showAlertDialog(requireContext(), getChildFragmentManager(), getViewLifecycleOwner(), msg);
        }
    };

    private MyPagerAdapter myPagerAdapter;

    private void receiveData(LevelReceiveData data) {
        if (getView() == null) {
            return;
        }
        LevelSurveySubFragment currentFragment = getCurrentFragment();
        if (currentFragment != null) {
            currentFragment.recieveData(data);
        }
    }

    private final ILevelRefreshBtn levelRefreshBtnI = new ILevelRefreshBtn() {
        @Override
        public void refreshMeasureBtn() {
            boolean measureEnable = LevelCheckBtnUtil.getMeasureEnable(viewModel.getCurrentStationId());
            ThreadPoolUtil.executeInMain(() -> {
                setViewEnable(mBinding.surveyor, measureEnable);
                setViewEnable(mBinding.btnInput, measureEnable);
            });
        }

        @Override
        public void refreshNextStationBtn() {
            boolean measureEnable = LevelCheckBtnUtil.getNextStationEnable(viewModel.getCurrentStationId());
            ThreadPoolUtil.executeInMain(() -> setViewEnable(mBinding.btnNextStation, measureEnable));
        }

        @Override
        public void refreshRollBackBtn() {
            doRefreshRollBackBtn();
        }

        @Override
        public void refreshReMeasureBtn() {
            doRefreshReMeasureBtn();
        }
    };

    private void doRefreshReMeasureBtn() {
        if (reMeasureStationBtn != null) {
            boolean measureEnable = LevelCheckBtnUtil.getReMeasureStationEnable(viewModel.getCurrentStationId());
            ThreadPoolUtil.executeInMain(() -> reMeasureStationBtn.setEnabled(measureEnable));
        }
    }

    private void doRefreshRollBackBtn() {
        if (rollBackBtn != null) {
            boolean measureEnable = LevelCheckBtnUtil.getRollBackEnable(viewModel.getCurrentStationId());
            ThreadPoolUtil.executeInMain(() -> rollBackBtn.setEnabled(measureEnable));
        }
    }

    public static void start(Context context, String projectId, String routeId, String routeName) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_ROUTE_ID, routeId);
        bundle.putString(BundleConstants.KEY_ROUTE_NAME, routeName);
        FragmentContainerActivity.start(context, TAG, "水准测量", bundle);
    }

    public static Fragment newInstance(Bundle args) {
        Fragment fragment = new LevelSurveyFragment();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    protected FragmentLevelSurveyorBinding getViewBinding(@NonNull LayoutInflater inflater, ViewGroup container) {
        return FragmentLevelSurveyorBinding.inflate(inflater, container, false);
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.menu_level_survey, menu);
        rollBackBtn = menu.findItem(R.id.rollBack);
        reMeasureStationBtn = menu.findItem(R.id.reMeasureStation);
        doRefreshRollBackBtn();
        doRefreshReMeasureBtn();
    }

    @Override
    public void onPrepareOptionsMenu(@NonNull Menu menu) {
        if (Test.DEBUG) {
            menu.findItem(R.id.test).setVisible(true);
        }
        super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (R.id.rollBack == item.getItemId()) {
            showConfirmDialog(getString(R.string.level_roll_back_hint), () -> {
                LevelSurveySubFragment currentFragment = getCurrentFragment();
                if (currentFragment != null) {
                    currentFragment.startRollBack();
                }
            });
        } else if (R.id.reMeasureStation == item.getItemId()) {
            showConfirmDialog(getString(R.string.level_remeasure_station_hint), () -> {
                LevelSurveySubFragment currentFragment = getCurrentFragment();
                if (currentFragment != null) {
                    currentFragment.reMeasureStation();
                }
            });
        } else if (R.id.surveyorSettings == item.getItemId()) {
            LevelingStationRecord currentStation = viewModel.getCurrentStation();
            if (currentStation != null) {
                LevelStationSettingActivity.start(requireContext(), getRouteId(), currentStation.getId(), false);
            }
        } else if (R.id.levelRouteResult == item.getItemId()) {
            viewModel.calLevelRouteResult(result -> {
                DialogUtil.showAlertDialog(requireContext(), getChildFragmentManager(), getViewLifecycleOwner(), result);
                return null;
            });
        } else if (R.id.receiveDataType == item.getItemId()) {
            LevelInputAllowFragment.show(getChildFragmentManager(), getViewLifecycleOwner(),
                    mBinding.surveyor.getVisibility() == View.VISIBLE,//是否仪器获取数据
                    auto -> {
                        viewModel.setGetDataAuto(getRouteId(), auto);
                        refreshManuleAutoInput(auto);
                    });
        } else if (R.id.test == item.getItemId()) {
            TestDataSelectLevelFragment.show(getChildFragmentManager(), position -> {
                LevelRequestTest levelReceiveData1 = (LevelRequestTest) levelRequest;
                levelReceiveData1.initTestData(position);
            });
        } else if (R.id.rollBackStation == item.getItemId()) {
            startRollBackStation();
        }
        return super.onOptionsItemSelected(item);
    }

    private void startRollBackStation() {
        if (!viewModel.getMeasureStationList().isEmpty()) {
            String lastStationId = viewModel.getMeasureStationList().get(0).lastStationId;
            if (TextUtils.isEmpty(lastStationId)) {
                ToastUtils.showShort("没有上一站了");
                return;
            }
            showConfirmDialog("是否要回退至上一站，该操作将清空当前站所有数据", () -> {
                viewModel.rollBackStation();
            });
        }
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
//        EventBus.getDefault().unregister(this);
        levelRequest.onDestroy();
        LevelCheckBtnUtil.destroy();
    }

    @Override
    protected void initView() {
        setHasOptionsMenu(true);
        initLevelRequest();
        stationSettingCheckUtil = new StationSettingCheckUtil();
        viewModel = new ViewModelProvider(this).get(LevelSuveyorVm.class);

        viewModel.initData(levelRefreshBtnI, () -> {
            initViewPager();
            return null;
        }, integer -> {
            mBinding.vp.setCurrentItem(integer);
            return null;
        }, nextStationI);

        viewModel.getGetDadaAuto(getRouteId(), auto -> {
            refreshManuleAutoInput(auto);
            return null;
        });

        initTitle();

        initListener();
    }

    private void initLevelRequest() {
        if (Test.DEBUG) {
            levelRequest = new LevelRequestTest(levelBackData);
        } else {
            levelRequest = new LevelRequestRelease(levelBackData);
        }
        levelRequest.onStart();
    }

    private void initTitle() {
        mBinding.tvRouteName.setText("测段：" + getRouteName());
    }

    @Override
    protected void initData() {
        viewModel.initDataStationList(getProjectId(), getRouteId());

        viewModel.checkSunTime(getProjectId(), hint -> {
            if (!TextUtils.isEmpty(hint)) {
                DialogUtil.showAlertDialog(requireContext(), getChildFragmentManager(), getViewLifecycleOwner(), hint);
            }
            return null;
        });
    }

    private void initListener() {
        mBinding.surveyor.setOnClickListener(v -> {
            stationSettingCheckUtil.checkStationSettingsLevel(viewModel.getCurrentStationId(), getChildFragmentManager(), this,
                    requireContext(),
                    this,
                    () -> levelRequest.requeset());
        });

        mBinding.btnNextStation.setOnClickListener(v -> {
            stationSettingCheckUtil.checkStationSettingsLevel(viewModel.getCurrentStationId(), getChildFragmentManager(), this,
                    requireContext(),
                    this,
                    () -> {
                        LevelSurveySubFragment currentFragment = getCurrentFragment();
                        if (currentFragment != null) {
                            currentFragment.nextStation();
                        }
                    });
        });

        mBinding.btnInput.setOnClickListener(v -> {
            stationSettingCheckUtil.checkStationSettingsLevel(viewModel.getCurrentStationId(), getChildFragmentManager(), this,
                    requireContext(),
                    this,
                    () -> {
                        runOnUI(() -> {
                            LevelInputFragment.show(getChildFragmentManager(), getViewLifecycleOwner(),
                                    (distance, height) -> receiveData(new LevelReceiveData(height, distance)));

                        });
                    });
        });

    }

    private String getProjectId() {
        return getArgumentString(BundleConstants.KEY_PROJECT_ID);
    }

    private String getRouteId() {
        return getArgumentString(BundleConstants.KEY_ROUTE_ID);
    }

    private String getRouteName() {
        return getArgumentString(BundleConstants.KEY_ROUTE_NAME);
    }

    private void initViewPager() {
        fragments = new ArrayList<>();
        for (int i = 0; i < viewModel.getMeasureStationList().size(); i++) {
            fragments.add(LevelSurveySubFragment.newInstance(i));
        }

        if (myPagerAdapter == null) {
            mBinding.vp.setOffscreenPageLimit(3);
            myPagerAdapter = new MyPagerAdapter(getChildFragmentManager());
            mBinding.vp.setAdapter(myPagerAdapter);
            mBinding.tabLayout.setViewPager(mBinding.vp);

            mBinding.vp.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
                @Override
                public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {

                }

                @Override
                public void onPageSelected(int position) {
                    viewModel.setCurrentFragmentPos(position);
                    viewModel.switchRefreshBtnEnable();
                }

                @Override
                public void onPageScrollStateChanged(int state) {

                }
            });
        } else {
            myPagerAdapter.notifyDataSetChanged();
            mBinding.tabLayout.notifyDataSetChanged();
            mBinding.tabLayout.setCurrentTab(0);
        }
    }

    private LevelSurveySubFragment getCurrentFragment() {
        int currentFragmentPos = viewModel.getCurrentFragmentPos();
        if (fragments != null && fragments.size() > currentFragmentPos) {
            return fragments.get(currentFragmentPos);
        }
        return null;
    }

    private void setViewEnable(View view, boolean measureEnable) {
        view.setEnabled(measureEnable);
        if (measureEnable) {
            view.setAlpha(1);
        } else {
            view.setAlpha(.4f);
        }
    }

    private class MyPagerAdapter extends FragmentStatePagerAdapter {
        public MyPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public int getCount() {
            return fragments.size();
        }

        @Override
        public CharSequence getPageTitle(int position) {
            return viewModel.getMeasureStationList().get(position).stationName;
        }

        @Override
        public Fragment getItem(int position) {
            return fragments.get(position);
        }

        @Override
        public int getItemPosition(@NonNull Object object) {
            return PagerAdapter.POSITION_NONE;
        }
    }


    private void showConfirmDialog(String content, TipsFragment.IConfirmCallback callback) {
        TipsFragment.cancelAbleTips(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(),
                content, -1, callback);
    }

    /**
     * 下一站的弹框
     */
    private void showSelectLevelNextStepDialog(LevelingStationRecord station) {
        runOnUI(() -> SelectLevelNextStationTypeFragment.select(getChildFragmentManager(), getViewLifecycleOwner(), type -> {
            switch (type) {
                case SelectLevelNextStationTypeFragment.TYPE_CONTINUE_OBSERVE:
                    //继续观测
                    startMeasureNextStation(station);
                    break;
                case SelectLevelNextStationTypeFragment.TYPE_INTERMITTENT:
                    viewModel.intermittent(station);
                    break;
                case SelectLevelNextStationTypeFragment.TYPE_END_STATION:
                    checkEndStation(station);
                    break;
            }
        }));
    }

    /**
     * 点击结束测站时验证
     *
     * @param station
     */
    private void checkEndStation(LevelingStationRecord station) {
        ThreadPoolUtil.execute(() -> {
            //增加确认结束测站的提醒
            if (station.isOddStation()) {
                //当前是奇数站
                ToastUtils.showShort("结束测站必须是偶数站！");
                return;
            }
            //是否设置过结束测站的测站设置
            boolean isSetEndStationSetting = viewModel.isHaveSetEndStationSetting(station.getId());
            if (!isSetEndStationSetting) {
                runOnUI(() -> StationSettingCheckUtil.showLevelSetStationSetting(getChildFragmentManager(),
                        getViewLifecycleOwner(), getContext(), StationSettingCheckUtil.HINT_LEVEL_END,
                        getRouteId(), station.getId(), true));
                return;
            }
            runOnUI(() -> showConfirmEndRouteStartEndStation(station));
        });

    }

    /**
     * 下一站继续观测
     */
    public void startMeasureNextStation(LevelingStationRecord station) {
        viewModel.contineObserve(station);
    }

    /**
     * 结束路线开始下一站的确认提醒
     *
     * @param station
     */
    private void showConfirmEndRouteStartEndStation(LevelingStationRecord station) {
        DialogUtil.showConfirmDialog(getChildFragmentManager(), getViewLifecycleOwner(), Tools.getString(R.string.level_end_route_hint), Tools.getString(R.string.confirm), new DialogUtil.IConfirmBack() {
            @Override
            public void onConfirm() {
                viewModel.endStation(station);
            }
        });
    }

    private void refreshManuleAutoInput(boolean receiveDataTypeAuto) {
        if (receiveDataTypeAuto) {
            mBinding.surveyor.setVisibility(View.VISIBLE);
            mBinding.btnInput.setVisibility(View.GONE);
        } else {
            mBinding.btnInput.setVisibility(View.VISIBLE);
            mBinding.surveyor.setVisibility(View.GONE);
        }
    }

}
