package com.bingce.controlnetwork.func.level.fragment;


import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_IGNORE;
import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_REMEASURE_POINT;
import static com.bingce.controlapphelper.model.LevelToleranceNoticeKt.VALUE_REMEASURE_STATION;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.ViewModelProvider;

import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory;
import com.bingce.controlapphelper.datasource.database.leveling.measure.backfront.LevelingMeasureBackFrontRecord;
import com.bingce.controlapphelper.datasource.database.leveling.measure.station.LevelingMeasureStationRecord;
import com.bingce.controlapphelper.datasource.database.leveling.station.LevelingStationRecord;
import com.bingce.controlapphelper.datasource.database.leveling.stationsetting.LevelingStationSettingRecord;
import com.bingce.controlapphelper.leveling.detaction.IDetectionHint;
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData;
import com.bingce.controlapphelper.leveling.detaction.LevelSpeakMsg;
import com.bingce.controlapphelper.leveling.detaction.surveyorcache.SurveyorCacheData;
import com.bingce.controlapphelper.model.LevelToleranceNotice;
import com.bingce.controlapphelper.model.LevelingMeasureOrderUtil;
import com.bingce.controlapphelper.model.PickPointResult;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.FragmentLevelSurveyorSubBinding;
import com.bingce.controlnetwork.fragment.PickPointFragment;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.func.level.dialog.LevelToleranceTipsFragment;
import com.bingce.controlnetwork.func.level.viewmodel.LevelSuveyorSubVm;
import com.bingce.controlnetwork.func.level.viewmodel.LevelSuveyorVm;
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding;
import com.bingce.controlnetwork.util.DialogUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * 水准测量页
 */
public class LevelSurveySubFragment extends BaseFragmentBinding<FragmentLevelSurveyorSubBinding> {
    public static final String TAG = LevelSurveySubFragment.class.getName();
    private LevelSuveyorSubVm viewModel;
    private LevelSuveyorVm stationListVm;

    private LevelingStationSettingRecord stationSettingRecord;

    private final ActivityResultLauncher<Intent> defaultStationLauncher = PickPointFragment.createActivityLauncher(this, (type, pickPointResult) -> {
//        if (pickPointResult.getSelectedIds().isEmpty() || pickPointResult.getSelectedNames().isEmpty()) {
//            return;
//        }
//        //只有前视才允许选点
//        String pointId = pickPointResult.getSelectedIds().get(0);
//        viewModel.saveMeasureFrontPointId(pointId, () -> {
//            setFrontPointNameUi(pickPointResult.getSelectedNames().get(0));
//            return null;
//        });
    });

//    private void setFrontPointNameUi(String name) {
//        mBinding.ilPoint.tvPoint.setText(name);
//    }

    public static LevelSurveySubFragment newInstance(int stationIndex) {
        Bundle args = new Bundle();
        args.putInt(LevelSurveyFragment.KEY_STATION_INDEX, stationIndex);
        LevelSurveySubFragment fragment = new LevelSurveySubFragment();
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    protected FragmentLevelSurveyorSubBinding getViewBinding(@NonNull LayoutInflater inflater, ViewGroup container) {
        return FragmentLevelSurveyorSubBinding.inflate(inflater, container, false);
    }

    @Override
    protected void initView() {
        viewModel = new ViewModelProvider(this).get(LevelSuveyorSubVm.class);
        stationListVm = new ViewModelProvider(requireParentFragment()).get(LevelSuveyorVm.class);

        mBinding.ilSightDistanceDiffCurrent.tvTitle.setText("视距差：");
        mBinding.ilDistanceTotal.tvTitle.setText("总距离：");
        mBinding.ilSightDistanceDiffTotal.tvTitle.setText("累积视距差：");
        mBinding.ilHeightDiffTotal.tvTitle.setText("总高差：");
        mBinding.ilZ.tvTitle.setText("高程：");
        initListener();
        initObserver();
    }

    private void initObserver() {
        viewModel.getStationLiveData().observe(this, stationRecord -> {
            boolean measureBack = LevelingMeasureOrderUtil.INSTANCE.isMeasureBack(stationRecord);

            String backFrontType = measureBack ? getString(R.string.rearview) : getString(R.string.forward_view);
            mBinding.tvBackFrontType.setText(backFrontType);

            //除了结束测站的第一个前视点可以选择测点
//            setSelectPointShowHide(stationRecord);

            setMeasureOrderAndCurrent(stationRecord);

            setSurveyorCacheUiVisiable(stationRecord);

//            viewModel.getMeasurePointName(stationRecord, pointName -> {
//                setFrontPointNameUi(pointName);
//                return null;
//            });

            refreshRulerNum();

        });

        viewModel.getSurveyorCacheDataLiveData().observe(this, this::setSurveyorCacheUi);

        viewModel.getMeasureStationLiveData().observe(this, this::setMeasureStationUi);

        //监听路线的最后一次设置更新尺子编号
        if (stationListVm.getRouteId() != null) {
            LevelingDbFactory.instance.getStationSettingDataSource().getLatestLiveData(stationListVm.getRouteId())
                    .observe(getViewLifecycleOwner(), levelingStationSettingRecord -> {
                        stationSettingRecord = levelingStationSettingRecord;
                        refreshRulerNum();
                    });
        }

    }

    private void refreshRulerNum() {
        LevelingStationRecord stationRecord = viewModel.getStationLiveData().getValue();
        if (stationRecord != null && stationSettingRecord != null) {
            int stationIndex = getStationIndex();
            if (stationIndex == 0) {
                if (stationRecord.isOddStation()) {
                    mBinding.tvRulerNumBack.setText(getRulerNumShow(stationSettingRecord.firstStationBackFootNumber));
                    mBinding.tvRulerNumFront.setText(getRulerNumShow(stationSettingRecord.firstStationFrontFootNumber));
                } else {
                    mBinding.tvRulerNumBack.setText(getRulerNumShow(stationSettingRecord.firstStationFrontFootNumber));
                    mBinding.tvRulerNumFront.setText(getRulerNumShow(stationSettingRecord.firstStationBackFootNumber));
                }
            } else {
                if (stationRecord.isOddStation()) {
                    mBinding.tvRulerNumBack.setText(getRulerNumShow(stationSettingRecord.firstStationBackFootNumber2));
                    mBinding.tvRulerNumFront.setText(getRulerNumShow(stationSettingRecord.firstStationFrontFootNumber2));
                } else {
                    mBinding.tvRulerNumBack.setText(getRulerNumShow(stationSettingRecord.firstStationFrontFootNumber2));
                    mBinding.tvRulerNumFront.setText(getRulerNumShow(stationSettingRecord.firstStationBackFootNumber2));
                }
            }
        }
    }

    private String getRulerNumShow(String rulerNum) {
        if (rulerNum == null) {
            rulerNum = "";
        }
        return "No." + rulerNum;
    }

    private void setSelectPointShowHide(LevelingStationRecord stationRecord) {
        viewModel.isEndStation(stationRecord, endStation -> {
            //非单程双侧，前视，非终点站可以选择测点
            boolean measureFirstFront = LevelingMeasureOrderUtil.INSTANCE.isMeasureFirstFront(stationRecord);
            boolean canSelectPoint = measureFirstFront && getStationIndex() == 0 && !endStation;
//            mBinding.ilPoint.ivSelectPoint.setVisibility(canSelectPoint ? View.VISIBLE : View.INVISIBLE);
//            mBinding.ilPoint.tvPoint.setClickable(canSelectPoint);
            return null;
        });
    }

    private void setSurveyorCacheUiVisiable(LevelingStationRecord stationRecord) {
        boolean showTwiceRead = LevelingMeasureOrderUtil.INSTANCE.isShowTwiceRead(stationRecord);
        if (showTwiceRead) {
            mBinding.ilB2.getRoot().setVisibility(View.VISIBLE);
            mBinding.ilF2.getRoot().setVisibility(View.VISIBLE);
        } else {
            mBinding.ilB2.getRoot().setVisibility(View.GONE);
            mBinding.ilF2.getRoot().setVisibility(View.GONE);
        }

        viewModel.setMeasureTextColor(stationRecord, mBinding.ilB1, mBinding.ilB2, mBinding.ilF1, mBinding.ilF2);
    }

    private void setMeasureOrderAndCurrent(LevelingStationRecord stationRecord) {
//        String textByLevelingMeasureOrder = LevelingMeasureOrderUtil.INSTANCE.getTextByLevelingMeasureOrder(stationRecord.levelingMeasureOrder);
//        mBinding.tvOrderType.setText(textByLevelingMeasureOrder);

        CharSequence[] currentOrderArray = viewModel.getCurrentOrderArray(stationRecord);
        mBinding.tvCurrentOrderOdd.setText(currentOrderArray[0]);
        mBinding.tvCurrentOrderEven.setText(currentOrderArray[1]);
    }

    private void setSurveyorCacheUi(SurveyorCacheData surveyorCacheData) {
        LevelingMeasureBackFrontRecord b1Data = surveyorCacheData.getB1Data();
        LevelingMeasureBackFrontRecord b2Data = surveyorCacheData.getB2Data();
        LevelingMeasureBackFrontRecord f1Data = surveyorCacheData.getF1Data();
        LevelingMeasureBackFrontRecord f2Data = surveyorCacheData.getF2Data();

        setSurveyorPointData(mBinding.ilB1.tvDisTance, mBinding.ilB1.tvHeight, b1Data);
        setSurveyorPointData(mBinding.ilB2.tvDisTance, mBinding.ilB2.tvHeight, b2Data);
        setSurveyorPointData(mBinding.ilF1.tvDisTance, mBinding.ilF1.tvHeight, f1Data);
        setSurveyorPointData(mBinding.ilF2.tvDisTance, mBinding.ilF2.tvHeight, f2Data);

        //更新视距差
        String currentFrontBackDistancceDiff = surveyorCacheData.getCurrentFrontBackDistancceDiff();
        mBinding.ilSightDistanceDiffCurrent.tvValue.setText(currentFrontBackDistancceDiff);
    }

    private void setSurveyorPointData(TextView tvDisTance, TextView tvHeight, LevelingMeasureBackFrontRecord data) {
        if (data == null) {
            tvDisTance.setText("");
            tvHeight.setText("");
        } else {
            tvDisTance.setText(data.getMeasureDistanceShow());
            tvHeight.setText(data.getMeasureHeightShow());
        }
    }

    private void setMeasureStationUi(LevelingMeasureStationRecord record) {
        if (record == null) {
            mBinding.ilDistanceTotal.tvValue.setText("");
            mBinding.ilSightDistanceDiffTotal.tvValue.setText("");
            mBinding.ilHeightDiffTotal.tvValue.setText("");
            mBinding.ilZ.tvValue.setText("");
            return;
        }

        viewModel.getZ(z -> {
            mBinding.ilZ.tvValue.setText(record.getZShow(z));
            return null;
        });

        mBinding.ilDistanceTotal.tvValue.setText(record.getDistanceTotalShow());
        mBinding.ilSightDistanceDiffTotal.tvValue.setText(record.getCumulativeVisualDistanceDiffShow());
        mBinding.ilHeightDiffTotal.tvValue.setText(record.getHeightDiffTotalShow());
    }

    @Override
    protected void initData() {
        String currentStationId = getCurrentStationId();
        if (currentStationId != null) {
            viewModel.initData(currentStationId, new IDetectionHint() {

                @Override
                public void noticeHint(@NonNull LevelToleranceNotice notice) {
                    LevelToleranceTipsFragment.tips(getChildFragmentManager(), getViewLifecycleOwner(),
                            notice.getTitle(), notice.getContent(), new LevelToleranceTipsFragment.IOperatorCallback() {
                                @Override
                                public void onReMeasureStation() {
                                    notice.getNoticeBack().levelContinue(VALUE_REMEASURE_STATION, notice.getIgnoreToleranceList());
                                }

                                @Override
                                public void onReMeasurePoint() {
                                    notice.getNoticeBack().levelContinue(VALUE_REMEASURE_POINT, notice.getIgnoreToleranceList());
                                }

                                @Override
                                public void onIgnore() {
                                    notice.getNoticeBack().levelContinue(VALUE_IGNORE, notice.getIgnoreToleranceList());
                                }
                            });
                }

                @Override
                public void stationCheckFinish(@NonNull LevelingStationRecord stationRecord) {
                    if (stationRecord.endStation) {
                        LevelSpeakMsg.speakStationFinish("该测段已观测完毕");
                        stationListVm.calLevelRouteResult(result -> {
                            DialogUtil.showAlertDialog(requireContext(), getChildFragmentManager(), getViewLifecycleOwner(),
                                    "该测段已观测完毕\n" + result);
                            return null;
                        });
                    } else {
                        boolean simStation = stationRecord.isSimStation();
                        if (simStation && !stationRecord.twiceStation) {
                            String hint = "请观测单程双转第二测站";
                            LevelSpeakMsg.speakStationFinish(hint);
                            //单程双转的第一个测站提示
                            TipsFragment.cancelAbleTips(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(),
                                    hint, -1,
                                    () -> switchTwiceStation(stationRecord));
                        } else {
                            String hint = getString(R.string.level_check_finish_next_station_hint);
                            LevelSpeakMsg.speakStationFinish(hint);

                            TipsFragment.tipsOnly(getChildFragmentManager(), getViewLifecycleOwner(), requireContext(),
                                    hint);
                        }
                    }
                }
            }, stationListVm.getNextStationI());
        }
    }

    private String getCurrentStationId() {
        int stationIndex = getStationIndex();
        List<LevelingStationRecord> measureStationList = stationListVm.getMeasureStationList();
        if (stationIndex >= 0 && measureStationList.size() > stationIndex) {
            return measureStationList.get(stationIndex).getId();
        }
        return null;
    }

    /**
     * 开始切换到第二测站（第二个tab）
     */
    private void switchTwiceStation(LevelingStationRecord record) {
        LevelSurveyFragment parentFragment = (LevelSurveyFragment) getParentFragment();
        if (parentFragment != null) {
            parentFragment.startMeasureNextStation(record);
        }
    }

    /**
     * 去创建测段
     */
    private void goCreateRoute() {
        String projectId = viewModel.getProjectId();
        if (projectId != null) {
            LevelRouteCreateEditFragment.start(requireContext(), projectId, null, getString(R.string.create_segment_survey));
            requireActivity().finish();
        }
    }

    private void initListener() {
//        mBinding.ilPoint.tvPoint.setOnClickListener((v) -> {
//            StringInputFragment.input(getChildFragmentManager(), "新建点", getViewLifecycleOwner(), string -> {
//                if (TextUtils.isEmpty(string)) return;
//                viewModel.saveMeasureFrontPoint(PointRecordTj.createHidePoint(string), () -> {
//                    setFrontPointNameUi(string);
//                    return null;
//                });
//            });
//        });
//
//        mBinding.ilPoint.ivSelectPoint.setOnClickListener((v) -> {
//            if (viewModel.getProjectId() == null) return;
//            PickPointResult alreadyResult = getAlreadySelectFrontPoint();
//            PickPointFragment.pickKnownAndUnknownPoint(getChildFragmentManager(),
//                    getViewLifecycleOwner(),
//                    getContext(),
//                    "",
//                    1,
//                    getString(R.string.select_surveyor_point),
//                    null,
//                    viewModel.getProjectId(), alreadyResult, defaultStationLauncher);
//        });

    }

    @Nullable
    private PickPointResult getAlreadySelectFrontPoint() {
        PickPointResult alreadyResult = null;
        LevelingStationRecord station = viewModel.getStationLiveData().getValue();
        String measurePointId = null;
        if (station != null) {
            measurePointId = LevelingMeasureOrderUtil.INSTANCE.getMeasurePointId(station);
        }
        if (measurePointId != null) {
            alreadyResult = new PickPointResult();
            List<String> ids = new ArrayList<>();
            List<String> names = new ArrayList<>();
            ids.add(measurePointId);
            alreadyResult.setSelectedIds(ids);
            alreadyResult.setSelectedNames(names);
        }
        return alreadyResult;
    }

    public void startRollBack() {
        viewModel.startRollBack();
    }

    public void reMeasureStation() {
        viewModel.reMeasureStation();
    }

    public void recieveData(LevelReceiveData data) {
        if (viewModel != null) {
            viewModel.recieveData(data);
        }
    }

    public void nextStation() {
        viewModel.nextStation();
    }

    private int getStationIndex() {
        return requireArguments().getInt(LevelSurveyFragment.KEY_STATION_INDEX);
    }

}
