package com.bingce.controlnetwork.func.level.fragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory;
import com.bingce.controlapphelper.datasource.database.leveling.route.LevelRouteRecord;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.databinding.FragmentLevelSurveyorHomeBinding;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.func.level.adapter.LevelRouteListAdapter;

import blankj.utilcode.util.ToastUtils;


/**
 * 水准测量首页
 */
public class LevelSurveyorHomeFragment extends BaseFragment {
    public static final String TAG = LevelSurveyorHomeFragment.class.getName();
    private LevelRouteListAdapter adapter;
    private FragmentLevelSurveyorHomeBinding mBinding;

    private static final String KEY_DETAIL_OR_SURVEYOR = "___editable___";//标识点击item进入测量界面还是测站详情

    public static LevelSurveyorHomeFragment newInstance(String projectId, String routeId) {
        return newInstance(projectId, routeId, true);
    }

    public static LevelSurveyorHomeFragment newInstance(String projectId, String routeId, boolean detailOrSurveyor) {
        LevelSurveyorHomeFragment fragment = new LevelSurveyorHomeFragment();
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_ROUTE_ID, routeId);
        bundle.putBoolean(KEY_DETAIL_OR_SURVEYOR, detailOrSurveyor);
        fragment.setArguments(bundle);
        return fragment;
    }

    /**
     * 去测量
     */
    private void goMeasurePage(LevelRouteRecord item) {
        String projectId = item.getProjectId();
        if (projectId == null) {
            ToastUtils.showShort("项目id异常");
            return;
        }
        LevelSurveyFragment.start(getContext(), projectId, item.getId(), item.getRouteName());
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        mBinding = FragmentLevelSurveyorHomeBinding.inflate(inflater, container, false);
        return mBinding.getRoot();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        mBinding = null;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        initView();
        initData();
    }

    private void initView() {
        adapter = new LevelRouteListAdapter(requireContext());
        mBinding.recyclerView.setAdapter(adapter);
        adapter.setOnItemClickListener(this::goMeasurePage);
    }

    private void initData() {
        String projectId = getProjectId();
        if (projectId != null) {
            LevelingDbFactory.instance.getLevelDataSource().getListLiveData(projectId)
                    .observe(getViewLifecycleOwner(), routeList -> {
                        adapter.refreshData(routeList);
                    });
        }
    }

    private String getProjectId() {
        if (getArguments() != null) {
            return getArguments().getString(BundleConstants.KEY_PROJECT_ID);
        }
        return null;
    }

    private String getRouteId() {
        if (getArguments() != null) {
            return getArguments().getString(BundleConstants.KEY_ROUTE_ID);
        }
        return null;
    }

}
