package com.bingce.controlnetwork.func.level.fragment;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.ViewModelProvider;

import com.bingce.controlapphelper.datasource.database.config.ConfigOperate;
import com.bingce.controlapphelper.event.SwitchPageEvent;
import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.activity.fragmentcontainer.FragmentContainerActivity;
import com.bingce.controlnetwork.fragment.PointListFragment;
import com.bingce.controlnetwork.fragment.ProjectToleranceSettingFragment;
import com.bingce.controlnetwork.fragment.base.BaseFragment;
import com.bingce.controlnetwork.func.level.idetection.LevelIDetectionListActivity;
import com.bingce.controlnetwork.func.level.viewmodel.LevelCreateOrEditRouteVm;
import com.bingce.controlnetwork.ui.ViewUtil;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;
import org.jetbrains.annotations.NotNull;


/**
 * 水准路线列表，新建测段
 */
public class ProjectDetailLevelFragment extends BaseFragment {
    public static final int title = R.string.project_detail;
    public static final String TAG = ProjectDetailLevelFragment.class.getName();
    private String projectId = "";
    private String projectType = "";
    //测站列表
    private LevelRouteFragment fragment;

    private LevelCreateOrEditRouteVm createEditRouteVm;


    @Override
    public View onCreateView(@NotNull LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        if (getArguments() != null) {
            projectId = getArguments().getString(BundleConstants.KEY_PROJECT_ID);
            projectType = getArguments().getString(BundleConstants.KEY_PROJECT_TYPE);
        }
        createEditRouteVm = new ViewModelProvider(this).get(LevelCreateOrEditRouteVm.class);
        View view = inflater.inflate(R.layout.fragment_project_detail_list_level, container, false);
        fragment = LevelRouteFragment.newInstance(projectId);
        getChildFragmentManager()
                .beginTransaction()
                .replace(R.id.fragment_container, fragment, "__station_list")
                .commit();
        //已知点
        View known = view.findViewById(R.id.known_point_list);
        initKnownPoint(known);
        //未知点
        View unknown = view.findViewById(R.id.unknown_point_list);
        initUnKnownPoint(unknown);
        //新建测段
        View ilBtn = view.findViewById(R.id.ilBtn);
        Button createNewRoute = ilBtn.findViewById(R.id.btnOne);
        createNewRoute.setText(R.string.create_segment_survey);
        initCreateNewRoute(createNewRoute);
        //去测量页面
        Button ivGoMeasurePage = ilBtn.findViewById(R.id.btnTwo);
        ivGoMeasurePage.setText(R.string.start_surveyor);
        initGoMeasurePage(ivGoMeasurePage);

        setHasOptionsMenu(true);
        return view;
    }

    private void initGoMeasurePage(View ivGoMeasurePage) {
        ViewUtil.setOnClick(ivGoMeasurePage, v -> {
            FragmentActivity activity = getActivity();
            if (activity != null) {
                activity.finish();
            }

            ThreadPoolUtil.execute(() -> {
                String surveyorRouteId = fragment.getSurveyorRouteId();
                ConfigOperate.changeDefaultProjectAndRoute(projectId, surveyorRouteId);
                EventBus.getDefault().post(new SwitchPageEvent<>(SwitchPageEvent.TYPE_SWITCH_TO_MEASURE_PAGE_AND_DATA, null));
            });
        });
    }

    private void initUnKnownPoint(View unknown) {
        ViewUtil.setOnClick(unknown,
                v -> PointListFragment.go2UnknownPointList(projectId, projectType, getContext()));
    }

    private void initKnownPoint(View known) {
        ViewUtil.setOnClick(
                known,
                v -> PointListFragment.go2KnownPointList(projectId, projectType, getContext()));
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        getData();
    }

    private void getData() {

    }

    /**
     * 新建测段
     */
    private void initCreateNewRoute(View viewCreateRoute) {
        if (viewCreateRoute != null) {
            viewCreateRoute.setOnClickListener((v) -> createEditRouteVm.showCreateOrEditDialog(
                    requireContext(),
                    projectId,
                    null
            ));
        }
    }

    @Override
    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
        inflater.inflate(R.menu.project_detail_fragment_menu_level, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        if (item.getItemId() == R.id.project_parameter_setting) {
            //限差设置
            ProjectToleranceSettingFragment.start(getContext(), projectId, projectType);
            return true;
        } else if (R.id.iAngleDetection == item.getItemId()) {
            //i角检测
            LevelIDetectionListActivity.launch(requireContext(), projectId);
        }
        return super.onOptionsItemSelected(item);
    }

    public static void start(Context context, String projectType, String projectId, String projectName) {
        Bundle bundle = new Bundle();
        bundle.putString(BundleConstants.KEY_PROJECT_ID, projectId);
        bundle.putString(BundleConstants.KEY_PROJECT_TYPE, projectType);
        FragmentContainerActivity.start(context, TAG, projectName, bundle);
    }

    public static ProjectDetailLevelFragment newInstance(Bundle bundle) {
        ProjectDetailLevelFragment fragment = new ProjectDetailLevelFragment();
        fragment.setArguments(bundle);
        return fragment;
    }
}