package com.bingce.controlnetwork.func.level.fragment;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;

import com.bingce.controlapphelper.datasource.database.ProjectTypeValue;
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.group.ToleranceGroupRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.level.ToleranceLevelRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate;
import com.bingce.controlapphelper.model.LevelingMeasureOrder;
import com.bingce.controlapphelper.model.LevelingMeasureOrderUtil;
import com.bingce.controlapphelper.util.ToleranceUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.AbstractFullScreenDialogFragment;
import com.bingce.controlnetwork.fragment.ToleranceSelectFragment;
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil;
import com.bingce.utils.DateUtils;
import com.bingce.utils.ThreadPoolUtil;

import java.util.Date;
import java.util.List;

import blankj.utilcode.util.ToastUtils;


/**
 * 水准新建编辑项目
 */
public class ProjectEditForLevelFragment extends AbstractFullScreenDialogFragment {
    public final String TAG = ProjectEditForLevelFragment.class.getSimpleName();
    private String toleranceId;
    private final ActivityResultLauncher<Intent> activityResultLauncher;

    private LevelingMeasureOrder levelingMeasureOrder;

    public ProjectEditForLevelFragment() {
        super();
        activityResultLauncher = ToleranceSelectFragment
                .createActivityLauncher(
                        this,
                        this::dealSelectTolerance);
    }

    @Override
    protected View createContentView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_project_edit_for_level, container, false);

        Button selectToleranceButton = view.findViewById(R.id.select_tolerance);
        if (selectToleranceButton != null) {
            selectToleranceButton.setOnClickListener(v ->
                    //选择限差方案
                    ToleranceSelectFragment.start(getContext(), toleranceId, ProjectTypeValue.LEVEL, activityResultLauncher)
            );
        }
        //设置项目名称
        EditText projectNameEdit = view.findViewById(R.id.editor);
        if (projectNameEdit != null && getArguments() != null) {
            String projectName = getArguments().getString(KEY_PROJECT_NAME);
            if (projectName == null || projectName.isEmpty()) {
                projectName = DateUtils.ymdWithCharacter(new Date());
            }
            projectNameEdit.setText(projectName);
        }
        if (getArguments() != null) {
            toleranceId = getArguments().getString(KEY_TOLERANCE_ID);
            if (toleranceId != null && !toleranceId.isEmpty()) {
                ThreadPoolUtil.execute(() -> {
                    ToleranceDetailRecord instance =
                            SurveyorDatabaseFactory
                                    .instance
                                    .getToleranceDetailDataSource()
                                    .getRecordSync(toleranceId);
                    updateTolerance(selectToleranceButton, instance);
                });
            }

            String projectId = getArguments().getString(KEY_PROJECT_ID);
            if (projectId != null && !projectId.isEmpty()) {
                ThreadPoolUtil.execute(() -> {
                    ProjectRecord projectRecord =
                            SurveyorDatabaseFactory
                                    .instance
                                    .getProjectDataSource()
                                    .getRecord(projectId);
                    if (projectRecord == null) {
                        return;
                    }
                });
            }
        }
        return view;
    }

    @WorkerThread
    private void updateTolerance(Button button, IToleranceDetail iToleranceDetail) {
        if (iToleranceDetail == null) {
            return;
        }
        toleranceId = iToleranceDetail.getId();

        IToleranceOperate operate = ToleranceOperateUtil.getOperate(toleranceId);
        if (operate == null) return;
        ToleranceGroupRecord group = operate.getGroup(iToleranceDetail);
        ToleranceLevelRecord level = operate.getLevel(iToleranceDetail);
        runOnUI(() -> {
            if (button != null) {
                String toleranceShowName = ToleranceUtil.completedToleranceName(group, level, iToleranceDetail);
                button.setText(toleranceShowName);
            }
        });
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        List<String> showList = LevelingMeasureOrderUtil.INSTANCE.getShowList();
        Spinner spinner = view.findViewById(R.id.spinner);
        spinner.setAdapter(new ArrayAdapter<>(requireContext(), android.R.layout.simple_list_item_1, showList));
//        spinner.setSelection(editTextItem.prismTypes.indexOf(editTextItem.surveyorPoint.getPrismTypeName()));
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                levelingMeasureOrder = LevelingMeasureOrderUtil.INSTANCE.getLevelingMeasureOrderByCreate(showList.get(position));
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void runOnUI(Runnable runnable) {
        if (getActivity() != null) {
            getActivity().runOnUiThread(runnable);
        }
    }

    @Override
    protected boolean quitWhenTouchBg() {
        return true;
    }

    @Override
    protected void onConfirm() {
        View view = getView();
        if (view == null) {
            return;
        }
        //判断项目名称是否为空
        EditText editor = view.findViewById(R.id.editor);
        String projectName = editor.getText().toString();
        if (projectName.isEmpty()) {
            Toast.makeText(getContext(), R.string.tips_need_select_input_project_name, Toast.LENGTH_SHORT).show();
            return;
        }
        //判断限差参数
        if (toleranceId == null || toleranceId.isEmpty()) {
            Toast.makeText(getContext(), R.string.tips_need_select_project_tolerance, Toast.LENGTH_SHORT).show();
            return;
        }
        if (levelingMeasureOrder == null) {
            ToastUtils.showShort(R.string.please_select_measure_order);
            return;
        }
        if (getArguments() == null) {
            return;
        }
        String projectId = getArguments().getString(KEY_PROJECT_ID);
        boolean showSlop = true;
        if (projectId == null || projectId.isEmpty()) {
            ProjectRecord projectRecord = new ProjectRecord(projectName);
            projectRecord.setToleranceId(toleranceId);
            projectRecord.projectType = ProjectTypeValue.LEVEL;
            projectRecord.setShowSlopDistance(showSlop);
            //存储
            SurveyorDatabaseFactory
                    .instance
                    .getProjectDataSource()
                    .saveRecordSync(projectRecord);


            //判断是否需要记录默认项目和测期
//            ThreadPoolUtil.execute(() -> {
//                IConfigDataSource configDataSource =
//                        TjDbFactory.instance.getConfigDataSource();
//                ConfigRecord defaultProject = configDataSource.getByKeySync(ConfigConstantsTj.KEY_DEFAULT_PROJECT_ID);
//                if (defaultProject != null && !defaultProject.getConfigValue().isEmpty()) {
//                    return;
//                }
//            });
        } else {
            ThreadPoolUtil.execute(() -> {
                ProjectRecord projectRecord =
                        SurveyorDatabaseFactory
                                .instance
                                .getProjectDataSource()
                                .getRecord(projectId);
                projectRecord.setProjectName(projectName);
                projectRecord.setToleranceId(toleranceId);
                projectRecord.setShowSlopDistance(showSlop);
                //存储
                SurveyorDatabaseFactory
                        .instance
                        .getProjectDataSource()
                        .saveRecordSync(projectRecord);
            });
        }
        super.onConfirm();
    }

    private static final String KEY_PROJECT_ID = "___project__id";
    private static final String KEY_TOLERANCE_ID = "___tolerance_id";
    private static final String KEY_PROJECT_NAME = "___project___name___";

    private static DialogFragment newInstance(Context context, String projectId, String toleranceId, String projectName) {
        ProjectEditForLevelFragment fragment = new ProjectEditForLevelFragment();
        Bundle args = new Bundle();
        if (projectId == null || projectId.isEmpty()) {
            setArgs(args, context.getString(R.string.title_create_new_project), null);
        } else {
            setArgs(args, context.getString(R.string.title_edit_project), null);
            args.putString(KEY_PROJECT_ID, projectId);
            args.putString(KEY_TOLERANCE_ID, toleranceId);
            args.putString(KEY_PROJECT_NAME, projectName);
        }
        fragment.setArguments(args);
        return fragment;
    }

    public static void editProject(FragmentManager fragmentManager,
                                   Context context,
                                   String projectId, String toleranceId, String projectName) {
        newInstance(context, projectId, toleranceId, projectName).show(fragmentManager, "_edit_project_fragment");
    }

    public static void createProject(FragmentManager fragmentManager, Context context) {
        newInstance(context, null, null, null)
                .show(fragmentManager, "__create_project_fragment__");
    }

    /**
     * 处理选择返回的限差id
     *
     * @param id
     */
    private void dealSelectTolerance(String id) {
        ThreadPoolUtil.execute(() -> {
            IToleranceOperate operate = ToleranceOperateUtil.getOperate(id);
            if (operate == null) return;
            IToleranceDetail toleranceDetail = operate.getIToleranceDetail(id);
            //存限差数据
            operate.saveToleranceDetail(toleranceDetail);
            View view = getView();
            if (view != null) {
                updateTolerance(view.findViewById(R.id.select_tolerance), toleranceDetail);
            }
        });
    }

}
