package com.bingce.controlnetwork.func.level.fragment;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlnetwork.R;

import org.jetbrains.annotations.NotNull;

import java.util.List;

import com.bingce.controlnetwork.func.level.sendreceive.TestData;

public class TestDataSelectLevelFragment extends DialogFragment {
    private static IClickItem clickItem;

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_single_list_layout, container, false);

        // Set the adapter
        if (view instanceof RecyclerView) {
            RecyclerView recyclerView = (RecyclerView) view;
            refreshRecyclerView(recyclerView);
        }

        return view;
    }

    @Override
    public void onViewCreated(@NonNull @NotNull View view, @Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        Dialog dialog = getDialog();
        if (dialog == null) {
            return;
        }
        Window window = dialog.getWindow();
        if (window == null) {
            return;
        }
        window.setLayout(WindowManager.LayoutParams.MATCH_PARENT, WindowManager.LayoutParams.MATCH_PARENT);
        WindowManager.LayoutParams lps = window.getAttributes();
        lps.verticalMargin = 0.4F;
        lps.horizontalMargin = 0.4F;
        window.setAttributes(lps);
    }

    protected void refreshRecyclerView(RecyclerView recyclerView) {
        List<String> list = TestData.Companion.getTextShowList();
        recyclerView.setAdapter(new MyAdapter(list));
    }

    public static void show(FragmentManager fragmentManager, IClickItem clickItem) {
        TestDataSelectLevelFragment.clickItem = clickItem;
        TestDataSelectLevelFragment fragment = new TestDataSelectLevelFragment();
        fragment.show(fragmentManager, TestDataSelectLevelFragment.class.getName());
    }

    private class MyAdapter extends RecyclerView.Adapter<ViewHolder> {

        private final List<String> data;

        public MyAdapter(List<String> data) {
            this.data = data;
        }

        @NonNull
        @Override
        public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            Context context = parent.getContext();
            LayoutInflater inflater = LayoutInflater.from(context);

            // Inflate the custom layout
            View itemView = inflater.inflate(R.layout.list_item_index_text_layout, parent, false);

            // Return a new holder instance
            return new ViewHolder(itemView);
        }

        @Override
        public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
            // Get the data model based on position
            String item = data.get(position);

            // Set item views based on your views and data model
            holder.index.setText(String.valueOf(position + 1));
            holder.text.setText(item);

            holder.itemView.setOnClickListener(v -> {
                if (clickItem != null) {
                    clickItem.click(holder.getBindingAdapterPosition());

                    dismissAllowingStateLoss();
                }

            });
        }

        @Override
        public int getItemCount() {
            return data.size();
        }

        // Provide a reference to the views for each data item

    }

    private static class ViewHolder extends RecyclerView.ViewHolder {
        public TextView index;
        public TextView text;

        public ViewHolder(View itemView) {
            super(itemView);
            index = itemView.findViewById(R.id.index);
            text = itemView.findViewById(R.id.text);
        }
    }

    public interface IClickItem {
        void click(int position);
    }
}
