package com.bingce.controlnetwork.func.level.idetection

import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant
import com.bingce.controlapphelper.datasource.database.leveling.idetection.record.LevelIDetectionRecord
import kotlin.math.pow

object LevelIDetectionCalUtil {
    private val station1 = LevelIDetectionConstant.STATION_TYPE_0
    private val station2 = LevelIDetectionConstant.STATION_TYPE_1
    private val ruler1 = LevelIDetectionConstant.RULER_TYPE_0
    private val ruler2 = LevelIDetectionConstant.RULER_TYPE_1
    private val measureTime1 = LevelIDetectionConstant.MEASURE_TIME_0
    private val measureTime2 = LevelIDetectionConstant.MEASURE_TIME_1
    private val measureTime3 = LevelIDetectionConstant.MEASURE_TIME_2
    private val measureTime4 = LevelIDetectionConstant.MEASURE_TIME_3
    private val measureTime5 = LevelIDetectionConstant.MEASURE_TIME_4

    /**
     * 单位mm
     * d1短距
     * d2长距
     */
    fun getIResult(
        iDetectionId: String,
        measureType: String,
        d1: Double,
        d2: Double,
        stationRecordMap: HashMap<String, LevelIDetectionRecord>
    ): Double {
        val a1 = getA1(iDetectionId, stationRecordMap)
        val b1 = getB1(iDetectionId, stationRecordMap)
        val a2 = getA2(iDetectionId, stationRecordMap)
        val b2 = getB2(iDetectionId, stationRecordMap)
        val deTa = getDeTa(measureType, a1, b1, a2, b2)
        return getI(deTa, d1, d2)
    }

    private fun getI(deTa: Double, d1: Double, d2: Double) =
        deTa * 206265 / (d2 - d1) - 1.61 * 10.0.pow(-5.0) * (d1 + d2)


    private fun getDeTa(
        measureType: String,
        a1: Double,
        b1: Double,
        a2: Double,
        b2: Double
    ): Double {
        return if (LevelIDetectionConstant.A_I_I_B == measureType || LevelIDetectionConstant.I_A_B_I == measureType) {
            ((a2 - b2) - (a1 - b1)) / 2
        } else {
            (a2 - b2) - (a1 - b1)
        }
    }


    /**
     * 一站一尺
     */
    private fun getA1(
        iDetectionId: String,
        stationRecordMap: HashMap<String, LevelIDetectionRecord>
    ): Double {
        val height1 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler1,
                measureTime1
            )]
        )
        val height2 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler1,
                measureTime2
            )]
        )
        val height3 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler1,
                measureTime3
            )]
        )
        val height4 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler1,
                measureTime4
            )]
        )
        val height5 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler1,
                measureTime5
            )]
        )
        return getAverage(height1, height2, height3, height4, height5)
    }

    /**
     * 一站二尺
     */
    private fun getB1(
        iDetectionId: String,
        stationRecordMap: HashMap<String, LevelIDetectionRecord>
    ): Double {
        val height1 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler2,
                measureTime1
            )]
        )
        val height2 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler2,
                measureTime2
            )]
        )
        val height3 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler2,
                measureTime3
            )]
        )
        val height4 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler2,
                measureTime4
            )]
        )
        val height5 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station1, ruler2,
                measureTime5
            )]
        )
        return getAverage(height1, height2, height3, height4, height5)
    }

    /**
     * 二站一尺
     */
    private fun getA2(
        iDetectionId: String,
        stationRecordMap: HashMap<String, LevelIDetectionRecord>
    ): Double {
        val height1 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler1,
                measureTime1
            )]
        )
        val height2 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler1,
                measureTime2
            )]
        )
        val height3 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler1,
                measureTime3
            )]
        )
        val height4 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler1,
                measureTime4
            )]
        )
        val height5 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler1,
                measureTime5
            )]
        )
        return getAverage(height1, height2, height3, height4, height5)
    }

    /**
     * 二站二尺
     */
    private fun getB2(
        iDetectionId: String,
        stationRecordMap: HashMap<String, LevelIDetectionRecord>
    ): Double {
        val height1 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler2,
                measureTime1
            )]
        )
        val height2 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler2,
                measureTime2
            )]
        )
        val height3 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler2,
                measureTime3
            )]
        )
        val height4 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler2,
                measureTime4
            )]
        )
        val height5 = getHeight(
            stationRecordMap[LevelIDetectionRecord.getRecordId(
                iDetectionId, station2, ruler2,
                measureTime5
            )]
        )
        return getAverage(height1, height2, height3, height4, height5)
    }

    private fun getHeight(record: LevelIDetectionRecord?): Double {
        return record?.height ?: 0.0
    }

    /**
     * 返回单位mm
     */
    private fun getAverage(
        height1: Double,
        height2: Double,
        height3: Double,
        height4: Double,
        height5: Double
    ): Double {
        return (height1 + height2 + height3 + height4 + height5) / 5 * 1000
    }

}