package com.bingce.controlnetwork.func.level.idetection

import android.content.Context
import android.content.Intent
import androidx.activity.viewModels
import com.bingce.controlapphelper.datasource.database.leveling.idetection.list.LevelIDetectionList
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlapphelper.widget.swiperecyclerview.widget.DefaultItemDecoration
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.ActivityLevelIDetectionListBinding
import com.bingce.controlnetwork.func.level.fragment.LevelIDetectionCreateFragment
import com.bingce.controlnetwork.func.level.idetection.adapter.LevelIDetectionListAdapter
import com.bingce.controlnetwork.func.level.idetection.viewmodel.LevelIDetectionListVm
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import com.bingce.controlnetwork.newui.widget.swipemenu.SwipeMenuDelete
import com.bingce.controlnetwork.util.DialogUtil
import com.bingce.utils.DateUtils
import java.util.Date

/**
 * 水准i角检测列表
 */
class LevelIDetectionListActivity : BaseBindingActivity<ActivityLevelIDetectionListBinding>() {
    private val viewModel by viewModels<LevelIDetectionListVm>()

    private val adater by lazy {
        LevelIDetectionListAdapter(this)
    }

    companion object {
        @JvmStatic
        fun launch(context: Context, projectId: String) {
            context.startActivity(Intent(context, LevelIDetectionListActivity::class.java).apply {
                putExtra(BundleConstants.KEY_PROJECT_ID, projectId)
            })
        }
    }

    override fun getBinding(): ActivityLevelIDetectionListBinding {
        return ActivityLevelIDetectionListBinding.inflate(layoutInflater)
    }

    override fun initView() {
        initAdapter()
        initListener()
    }

    private fun initListener() {
        mBinding.tvCreateIAngleDetection.setOnClickListener {
            LevelIDetectionCreateFragment.input(
                supportFragmentManager,
                "i角检测名称",
                DateUtils.ymdHMSWithCharacter(Date()),
                this
            ) { name -> viewModel.saveNewRecord(getProjectId(), name) }

        }
    }

    private fun initAdapter() {
        mBinding.ilSwipeList.swipeRecyclerView.addItemDecoration(
            DefaultItemDecoration(
                Tools.getColor(R.color.transparent), 1, 1
            )
        )
        mBinding.ilSwipeList.swipeRecyclerView.setSwipeMenuCreator(
            SwipeMenuDelete(this)
        )
        mBinding.ilSwipeList.swipeRecyclerView.setOnItemMenuClickListener { menuBridge, adapterPosition ->
            menuBridge.closeMenu()
            dealMenuClick(menuBridge.position, adater.getBean(adapterPosition))
        }
        mBinding.ilSwipeList.swipeRecyclerView.setOnItemClickListener { view, adapterPosition ->
            val bean = adater.getBean(adapterPosition)
            LevelIDetectionRecordActivity.launch(this, bean.id)
        }
        mBinding.ilSwipeList.swipeRecyclerView.adapter = adater
    }

    private fun dealMenuClick(position: Int, bean: LevelIDetectionList?) {
        DialogUtil.showConfirmDialog(
            supportFragmentManager, this, "您确定要删除该i角检测吗？",
            Tools.getString(R.string.confirm)
        ) {
            bean?.let {
                viewModel.deleteRecord(it)
            }
        }
    }

    override fun initData() {
        viewModel.getListLiveData(getProjectId()).observe(this) {
            adater.refreshData(it)
        }
    }

    private fun getProjectId() = intent.getStringExtra(BundleConstants.KEY_PROJECT_ID)!!


}