package com.bingce.controlnetwork.func.level.idetection

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.EditText
import androidx.activity.viewModels
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.I_A_B_I
import com.bingce.controlapphelper.datasource.database.leveling.idetection.list.LevelIDetectionList
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlapphelper.widget.swiperecyclerview.widget.DefaultItemDecoration
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.ActivityLevelIDetectionRecordBinding
import com.bingce.controlnetwork.event.RefreshEvent
import com.bingce.controlnetwork.fragment.TipsFragment
import com.bingce.controlnetwork.func.level.fragment.LevelInputAllowFragment
import com.bingce.controlnetwork.func.level.fragment.LevelInputFragment
import com.bingce.controlnetwork.func.level.idetection.adapter.LevelIDetectionRecordAdapter
import com.bingce.controlnetwork.func.level.idetection.viewmodel.LevelIDetectionRecordVm
import com.bingce.controlnetwork.func.level.sendreceive.ILevelBackData
import com.bingce.controlnetwork.func.level.sendreceive.LevelRequest
import com.bingce.controlnetwork.func.level.sendreceive.LevelRequestRelease
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import com.bingce.controlnetwork.newui.widget.selectitem.BaseSelectItemFragment
import com.bingce.controlnetwork.newui.widget.selectitem.SelectIDetectionStationFragment
import com.bingce.controlnetwork.newui.widget.selectitem.SelectIDetectionStationMeasureTypeFragment
import com.bingce.controlnetwork.util.DialogUtil
import com.bingce.controlnetwork.util.StationSettingCheckUtil
import com.bingce.utils.ScreenUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode

/**
 * 水准i角检测-测量
 */
class LevelIDetectionRecordActivity : BaseBindingActivity<ActivityLevelIDetectionRecordBinding>() {
    private val viewModel by viewModels<LevelIDetectionRecordVm>()

    private val stationSettingCheckUtil = StationSettingCheckUtil()

    private val adapter by lazy {
        LevelIDetectionRecordAdapter(this, surveyorData)
    }

    private val surveyorData by lazy {
        LevelIDetectionSurveyorData(getIDetectionId())
    }

    private val levelReceiveData: LevelRequest = LevelRequestRelease(object : ILevelBackData {
        override fun receive(data: LevelReceiveData) {
            receiveData(data)
        }

        override fun measureError(msg: String) {
            if (isDestroyed || isFinishing) {
                return
            }
            DialogUtil.showAlertDialog(
                this@LevelIDetectionRecordActivity,
                supportFragmentManager,
                this@LevelIDetectionRecordActivity,
                msg
            )
        }
    })

    @Subscribe(threadMode = ThreadMode.BACKGROUND)
    fun refresh(event: RefreshEvent<*>) {
        if (event.type == RefreshEvent.REFRESH_BY_LEVEL_I_DETECTION_SETTING_UPDATE) {
            viewModel.updateStationSetting()
            adapter.setStationSetting(viewModel.stationSetting)
            runOnUiThread {
                adapter.notifyDataSetChanged()
            }
        }
    }

    companion object {
        @JvmStatic
        fun launch(context: Context, levelIDetectionId: String) {
            context.startActivity(Intent(context, LevelIDetectionRecordActivity::class.java).apply {
                putExtra(BundleConstants.KEY_I_DETECTION_ID, levelIDetectionId)
            })
        }
    }

    override fun getBinding(): ActivityLevelIDetectionRecordBinding {
        return ActivityLevelIDetectionRecordBinding.inflate(layoutInflater)
    }

    override fun onDestroy() {
        super.onDestroy()
        levelReceiveData.onDestroy()
        EventBus.getDefault().unregister(this)
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_level_survey_i_detection, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.result == item.itemId) {
            startIResult()
        } else if (R.id.stationSetting == item.itemId) {
            goLevelIDetectionSetting()
        } else if (R.id.rollBack == item.itemId) {
            startRollBack()
        } else if (R.id.level_i_detection_sample == item.itemId) {
            LevelIDetectionSampleActivity.launch(this)
        } else if (R.id.receiveDataType == item.itemId) {
            showReceiveDataTypeDialog()
        }
        return super.onOptionsItemSelected(item)
    }

    /**
     * i角检测的测站设置
     */
    private fun goLevelIDetectionSetting() {
        LevelIDetectionStationSettingActivity.launch(this, getIDetectionId())
    }

    private fun showReceiveDataTypeDialog() {
        LevelInputAllowFragment.show(
            supportFragmentManager,
            this,
            mBinding.surveyor.visibility == View.VISIBLE
        ) {
            viewModel.setGetDataAuto(getIDetectionId(), it)
            refreshManuleAutoInput(it)
        }
    }

    private fun startIResult() {
        val d1String = getEtText(mBinding.etShortDistance)
        if (TextUtils.isEmpty(d1String)) {
            ToastUtils.showShort("请输入短距")
            return
        }
        val d2String = getEtText(mBinding.etLongDistance)
        if (TextUtils.isEmpty(d2String)) {
            ToastUtils.showShort("请输入长距")
            return
        }

        if (viewModel.measureType == null) {
            ToastUtils.showShort("请选择测量方式")
            return
        }

        lifecycleScope.launch {
            withContext(Dispatchers.IO) {
                //存短长距和测量方式
                viewModel.saveShortLongDistanceMeasureType(d1String, d2String)
            }

            //计算结果
            val result = viewModel.startResult(getD(d1String), getD(d2String))
            if (!TextUtils.isEmpty(result)) {
                DialogUtil.showAlertDialog(
                    this@LevelIDetectionRecordActivity,
                    supportFragmentManager,
                    this@LevelIDetectionRecordActivity,
                    result
                )
            }
        }

    }

    private fun getEtText(et: EditText): String {
        return et.text.toString().trim()
    }

    /**
     * 返回mm的D
     */
    private fun getD(value: String): Double {
        return value.toDouble() * 1000
    }

    private fun startRollBack() {
        val findRollBackData = viewModel.findRollBackData()
        if (findRollBackData == null) {
            ToastUtils.showShort("没有可以回退的数据")
            return
        }

        val deleteStationShow =
            LevelIDetectionConstant.getStationTypeShow(findRollBackData.currentStationType)
        val deleteRuleShow =
            LevelIDetectionConstant.getRulerTypeShow(findRollBackData.currentRulerType)

        val hint = "回退操作将删除 $deleteStationShow $deleteRuleShow 的数据，您是否还要回退？"

        showConfirmDialog(hint) {
            viewModel.deleteStationRulerData(findRollBackData) {
                refreshAdapterAndButton()
            }
        }
    }

    override fun initView() {
        EventBus.getDefault().register(this)
        levelReceiveData.onStart()
        initAdapter()

        initListener()

        viewModel.getGetDadaAuto(getIDetectionId()) {
            refreshManuleAutoInput(it)
        }
    }

    private fun initListener() {
        mBinding.llStation.setOnClickListener {
            if (adapter.itemCount == 0) {
                ToastUtils.showShort("数据加载中，请稍后尝试")
                return@setOnClickListener
            }
            BaseSelectItemFragment.select(
                SelectIDetectionStationFragment(),
                supportFragmentManager,
                this, object : BaseSelectItemFragment.ISelectItem {
                    override fun onBack(value: String) {
                        //value stationType
                        viewModel.switchStation(value.toInt())
                        setCurrentStation()
                        refreshAdapterAndButton()
                    }
                })
        }

        mBinding.llMeasureType.setOnClickListener {
            BaseSelectItemFragment.select(
                SelectIDetectionStationMeasureTypeFragment(),
                supportFragmentManager,
                this, object : BaseSelectItemFragment.ISelectItem {
                    override fun onBack(value: String) {
                        switchMeasureType(value)
                    }
                })
        }

        mBinding.surveyor.setOnClickListener {
            stationSettingCheckUtil.checkStationSettingsIDetection(
                getIDetectionId(),
                this,
                this::runOnUiThread
            ) {
                levelReceiveData.requeset()
            }
        }

        mBinding.btnInput.setOnClickListener {
            LevelInputFragment.show(
                supportFragmentManager, this
            ) { distance: Double, height: Double ->
                receiveData(LevelReceiveData(height, distance))
            }
        }
    }

    private fun receiveData(data: LevelReceiveData) {
        if (isFinishing || isDestroyed) {
            return
        }
        viewModel.receiveData(data.height, data.distance)
        refreshAdapterAndButton()
    }

    /**
     * 切换测量方法带存储
     */
    private fun switchMeasureType(measureType: String) {
        setCurrentMeasureType(measureType)
        viewModel.saveMeasureType(getIDetectionId(), measureType)
    }

    private fun setCurrentMeasureType(measureType: String?) {
        mBinding.tvMeasureType.text = measureType
        viewModel.measureType = measureType
    }

    private fun setCurrentStation() {
        val stationTypeShow =
            LevelIDetectionConstant.getStationTypeShow(surveyorData.currentStationType)
        mBinding.tvStation.text = stationTypeShow
    }

    private fun initAdapter() {
        mBinding.recyclerView.adapter = adapter
        mBinding.recyclerView.addItemDecoration(
            DefaultItemDecoration(
                Tools.getColor(R.color.transparent),
                1,
                ScreenUtil.dp2px(this, 10f)
            )
        )
    }

    override fun initData() {
        viewModel.initData(surveyorData) {
            adapter.setStationSetting(viewModel.stationSetting)

            setCurrentStation()

            if (it.measureType == null) {
                switchMeasureType(I_A_B_I)
            } else {
                setCurrentMeasureType(it.measureType)
            }

            //设置短长距
            setShortLongDistance(it)

            refreshButton()

            adapter.refreshData(viewModel.dataListAdapter)
        }
    }

    private fun setShortLongDistance(it: LevelIDetectionList) {
        mBinding.etShortDistance.setText(it.shortDistance)
        mBinding.etLongDistance.setText(it.longDistance)
    }

    private fun getIDetectionId() = intent.getStringExtra(BundleConstants.KEY_I_DETECTION_ID)!!


    private fun showConfirmDialog(content: String, callback: TipsFragment.IConfirmCallback) {
        TipsFragment.cancelAbleTips(
            supportFragmentManager, this, this,
            content, -1, callback
        )
    }

    private fun refreshAdapterAndButton() {
        adapter.notifyDataSetChanged()
        refreshButton()
    }

    private fun refreshButton() {
        //刷新测量按钮
        setViewEnable(mBinding.surveyor, surveyorData.isHaveSurveyor())
        setViewEnable(mBinding.btnInput, surveyorData.isHaveSurveyor())
    }

    private fun setViewEnable(view: View, measureEnable: Boolean) {
        view.isEnabled = measureEnable
        if (measureEnable) {
            view.alpha = 1f
        } else {
            view.alpha = .4f
        }
    }

    private fun refreshManuleAutoInput(receiveDataTypeAuto: Boolean) {
        if (receiveDataTypeAuto) {
            mBinding.surveyor.visibility = View.VISIBLE
            mBinding.btnInput.visibility = View.GONE
        } else {
            mBinding.btnInput.visibility = View.VISIBLE
            mBinding.surveyor.visibility = View.GONE
        }
    }

}