package com.bingce.controlnetwork.func.level.idetection

import android.content.Context
import android.content.Intent
import android.text.TextUtils
import android.view.Menu
import android.view.MenuItem
import android.widget.ArrayAdapter
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.func.level.constant.LevelStationSettingContant
import com.bingce.controlnetwork.func.level.util.LevelStationSettingUtil
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.idetection.isetting.LevelIDetectionSetting
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.ActivityLevelIDetectionStationSettingBinding
import com.bingce.controlnetwork.event.RefreshEvent
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.greenrobot.eventbus.EventBus

/**
 * i角检测测站设置
 */
class LevelIDetectionStationSettingActivity :
    BaseBindingActivity<ActivityLevelIDetectionStationSettingBinding>() {

    companion object {
        @JvmStatic
        fun launch(context: Context, iDetectionId: String) {
            context.startActivity(
                Intent(
                    context,
                    LevelIDetectionStationSettingActivity::class.java
                ).apply {
                    putExtra(BundleConstants.KEY_I_DETECTION_ID, iDetectionId)
                })
        }
    }

    override fun getBinding(): ActivityLevelIDetectionStationSettingBinding {
        return ActivityLevelIDetectionStationSettingBinding.inflate(layoutInflater)
    }

    override fun initView() {
        initEditTextTitle()
        initSpinnerImaging()
    }

    override fun initData() {
        getEditData()
    }

    private fun getEditData() {
        lifecycleScope.launch(Dispatchers.IO) {
            val iDetectionId = getIDetectionId()
            val record =
                LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(
                    iDetectionId
                )

            if (record != null) {
                withContext(Dispatchers.Main) {
                    setData(record)
                }
            }
        }
    }

    /**
     * 初始化editText坐标的title
     */
    private fun initEditTextTitle() {
        mBinding.ilWatcher.tvTitle.text = "观测者"
        mBinding.ilRecorder.tvTitle.text = "记录者"
        mBinding.ilExaminer.tvTitle.text = "检查者"
        mBinding.ilRuler1.tvTitle.text = "尺一编号"
        mBinding.ilRuler2.tvTitle.text = "尺二编号"
        mBinding.ilInstrumentModel.tvTitle.text = "仪器型号"
        mBinding.ilDeviceNumber.tvTitle.text = "仪器编号"
    }

    private fun setData(record: LevelIDetectionSetting) {
        mBinding.ilWatcher.editText.setText(record.watcher)
        mBinding.ilRecorder.editText.setText(record.recorder)
        mBinding.ilExaminer.editText.setText(record.examiner)
        mBinding.ilRuler1.editText.setText(record.rulerNum1)
        mBinding.ilRuler2.editText.setText(record.rulerNum2)
        mBinding.ilInstrumentModel.editText.setText(record.instrumentModel)
        mBinding.ilDeviceNumber.editText.setText(record.deviceNumber)
        mBinding.spinnerImaging.setSelection(LevelStationSettingContant.imagingList.indexOf(record.imaging))
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_save, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.menuSave) {
            startSave()
        }
        return super.onOptionsItemSelected(item)
    }

    private fun initSpinnerImaging() {
        mBinding.spinnerImaging.adapter = ArrayAdapter(
            this,
            android.R.layout.simple_list_item_1,
            LevelStationSettingContant.imagingList
        )
    }

    private fun startSave() {
        val watcher = mBinding.ilWatcher.editText.text.toString().trim()
        val recorder = mBinding.ilRecorder.editText.text.toString().trim()
        val examiner = mBinding.ilExaminer.editText.text.toString().trim()
        val ruler1 = mBinding.ilRuler1.editText.text.toString().trim()
        val ruler2 = mBinding.ilRuler2.editText.text.toString().trim()
        val instrumentModel = mBinding.ilInstrumentModel.editText.text.toString().trim()
        val deviceNumber = mBinding.ilDeviceNumber.editText.text.toString().trim()
        val imaging = LevelStationSettingUtil.getSpinnerText(mBinding.spinnerImaging)

        if (isEmpty(ruler1)) {
            ToastUtils.showShort("请输入尺一编号")
            return
        }
        if (isEmpty(ruler2)) {
            ToastUtils.showShort("请输入尺二编号")
            return
        }

        lifecycleScope.launch(Dispatchers.IO) {
            val iDetectionId = getIDetectionId()
            val record =
                LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(
                    iDetectionId
                )

            val saveRecord = record ?: LevelIDetectionSetting(iDetectionId)

            saveRecord.watcher = watcher
            saveRecord.recorder = recorder
            saveRecord.examiner = examiner
            saveRecord.rulerNum1 = ruler1
            saveRecord.rulerNum2 = ruler2
            saveRecord.instrumentModel = instrumentModel
            saveRecord.deviceNumber = deviceNumber
            saveRecord.imaging = imaging

            if (record == null) {
                LevelingDbFactory.instance.iDetectionSettingDataSource.save(saveRecord)
            } else {
                LevelingDbFactory.instance.iDetectionSettingDataSource.update(saveRecord)
            }

            EventBus.getDefault()
                .post(RefreshEvent(RefreshEvent.REFRESH_BY_LEVEL_I_DETECTION_SETTING_UPDATE, null))

            finish()
        }

    }

    private fun isEmpty(value: String?) = TextUtils.isEmpty(value)


    private fun getIDetectionId() = intent.getStringExtra(BundleConstants.KEY_I_DETECTION_ID)!!

}