package com.bingce.controlnetwork.func.level.idetection.adapter;

import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_0;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_1;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_2;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_3;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_4;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.RULER_TYPE_0;
import static com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.RULER_TYPE_1;

import android.content.Context;
import android.view.ViewGroup;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant;
import com.bingce.controlapphelper.datasource.database.leveling.idetection.isetting.LevelIDetectionSetting;
import com.bingce.controlapphelper.datasource.database.leveling.idetection.record.LevelIDetectionRecord;
import com.bingce.controlapphelper.leveling.detaction.LevelFormatUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ItemLevelIDetectionRecordBinding;
import com.bingce.controlnetwork.newui.adapter.BaseAdapterNew;

import java.util.Map;

import com.bingce.controlnetwork.func.level.idetection.LevelIDetectionSurveyorData;


/**
 * i角检测列表
 */
public class LevelIDetectionRecordAdapter extends BaseAdapterNew<LevelIDetectionRecordHolder, Map<String, LevelIDetectionRecord>> {

    private final LevelIDetectionSurveyorData surveyorData;
    private LevelIDetectionSetting stationSetting;

    public LevelIDetectionRecordAdapter(Context context, LevelIDetectionSurveyorData surveyorData) {
        super(context);
        this.surveyorData = surveyorData;
    }

    @NonNull
    @Override
    public LevelIDetectionRecordHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new LevelIDetectionRecordHolder(ItemLevelIDetectionRecordBinding.inflate(getInflater(), parent, false));
    }

    @Override
    protected void onBindViewHolderDo(LevelIDetectionRecordHolder holder, int position) {
        Map<String, LevelIDetectionRecord> recordMap = mDataList.get(0);

        //尺子
        int rulerType;
        if (position == 0) {
            rulerType = RULER_TYPE_0;
        } else {
            rulerType = RULER_TYPE_1;
        }


        //当前测的尺子选中
        boolean isSelect = rulerType == surveyorData.getCurrentRulerType();
        if (isSelect) {
            holder.mBinding.getRoot().setBackgroundDrawable(Tools.getDrawable(R.drawable.select_mask));
        } else {
            holder.mBinding.getRoot().setBackgroundColor(Tools.getColor(R.color.white));
        }

        LevelIDetectionRecord record1 = getRecord(recordMap, rulerType, MEASURE_TIME_0);
        LevelIDetectionRecord record2 = getRecord(recordMap, rulerType, MEASURE_TIME_1);
        LevelIDetectionRecord record3 = getRecord(recordMap, rulerType, MEASURE_TIME_2);
        LevelIDetectionRecord record4 = getRecord(recordMap, rulerType, MEASURE_TIME_3);
        LevelIDetectionRecord record5 = getRecord(recordMap, rulerType, MEASURE_TIME_4);

        //尺几
        holder.mBinding.tvRulerType.setText(LevelIDetectionConstant.getRulerTypeShow(rulerType));
        //尺号
        holder.mBinding.tvRulerNum.setText(LevelIDetectionConstant.getRulerNumShow(rulerType, stationSetting));

        holder.mBinding.tvHeight1.setText(getHeight(record1));
        holder.mBinding.tvDistance1.setText(getDistance(record1));
        holder.mBinding.tvHeight2.setText(getHeight(record2));
        holder.mBinding.tvDistance2.setText(getDistance(record2));
        holder.mBinding.tvHeight3.setText(getHeight(record3));
        holder.mBinding.tvDistance3.setText(getDistance(record3));
        holder.mBinding.tvHeight4.setText(getHeight(record4));
        holder.mBinding.tvDistance4.setText(getDistance(record4));
        holder.mBinding.tvHeight5.setText(getHeight(record5));
        holder.mBinding.tvDistance5.setText(getDistance(record5));
    }

    private LevelIDetectionRecord getRecord(Map<String, LevelIDetectionRecord> map, int rulerType, int measureTime) {
        String recordId = LevelIDetectionRecord.getRecordId(surveyorData.getCurrentIDetectionId(), surveyorData.getCurrentStationType(), rulerType, measureTime);
        return map.get(recordId);
    }

    private String getHeight(LevelIDetectionRecord record) {
        if (record == null) {
            return "";
        }
        return LevelFormatUtil.INSTANCE.formatHeight(record.height);
    }

    private String getDistance(LevelIDetectionRecord record) {
        if (record == null) {
            return "";
        }
        return LevelFormatUtil.INSTANCE.formatDistance(record.distance);
    }

    @Override
    public int getItemCount() {
        return mDataList == null ? 0 : 2;
    }

    public void setStationSetting(LevelIDetectionSetting stationSetting) {
        this.stationSetting = stationSetting;
    }
}
