package com.bingce.controlnetwork.func.level.idetection.viewmodel

import androidx.annotation.WorkerThread
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDb
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_0
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_1
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_2
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_3
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.MEASURE_TIME_4
import com.bingce.controlapphelper.datasource.database.leveling.idetection.constants.LevelIDetectionConstant.RULER_TYPE_1
import com.bingce.controlapphelper.datasource.database.leveling.idetection.isetting.LevelIDetectionSetting
import com.bingce.controlapphelper.datasource.database.leveling.idetection.list.LevelIDetectionList
import com.bingce.controlapphelper.datasource.database.leveling.idetection.record.LevelIDetectionRecord
import com.bingce.controlapphelper.leveling.detaction.LevelFormatUtil
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.func.level.idetection.LevelIDetectionCalUtil
import com.bingce.controlnetwork.func.level.idetection.LevelIDetectionSurveyorData
import com.bingce.controlnetwork.func.level.idetection.util.LevelIDetectionFindUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class LevelIDetectionRecordVm : ViewModel() {

    //key 记录id
    val dataListAdapter = mutableListOf<Map<String, LevelIDetectionRecord>>()

    //key 记录id
    private val stationRecordMap = hashMapOf<String, LevelIDetectionRecord>()

    lateinit var surveyorData: LevelIDetectionSurveyorData

    var measureType: String? = null

    var stationSetting: LevelIDetectionSetting? = null

    val findUtil = LevelIDetectionFindUtil()

    fun initData(
        surveyorData: LevelIDetectionSurveyorData,
        callBack: (LevelIDetectionList) -> Unit
    ) {
        this.surveyorData = surveyorData
        viewModelScope.launch(Dispatchers.IO) {
            dataListAdapter.clear()
            dataListAdapter.add(stationRecordMap)
            val records =
                LevelingDbFactory.instance.iDetectionRecordDataSource.getRecordsByIDetectionId(
                    surveyorData.currentIDetectionId
                )
            //更新测站设置
            updateStationSetting()

            for (record in records) {
                stationRecordMap[record.id] = record
            }

            //查询第一个需要测量的数据
            val findMeasureDataAll = findMeasureDataAll()
            if (findMeasureDataAll != null) {
                changeCurrentSurveyorData(findMeasureDataAll)
            }

            val iDetectionRecord =
                LevelingDbFactory.instance.iDetectionListDataSource.getById(surveyorData.currentIDetectionId)

            withContext(Dispatchers.Main) {
                callBack(iDetectionRecord)
            }
        }
    }

    /**
     * 存数据
     */
    fun receiveData(height: Double, distance: Double) {
        if (surveyorData.currentRulerType == -1) {
            ToastUtils.showShort("未找到观测尺")
            return
        }

        //存当前测量的数据
        val recordId = LevelIDetectionRecord.getRecordId(
            surveyorData.currentIDetectionId,
            surveyorData.currentStationType,
            surveyorData.currentRulerType,
            surveyorData.currentMeasureTime
        )
        val record = LevelIDetectionRecord.create(
            surveyorData.currentIDetectionId, surveyorData.currentStationType,
            surveyorData.currentRulerType, surveyorData.currentMeasureTime,
            height, distance
        )

        stationRecordMap[recordId] = record

        //测完一个数据更新尺子和测量次数
        if (surveyorData.currentMeasureTime == MEASURE_TIME_4) {
            //最后一个测完了
            if (surveyorData.currentRulerType == RULER_TYPE_1) {
                //该站全部测完
                surveyorData.currentRulerType = -1
                surveyorData.currentMeasureTime = MEASURE_TIME_0

                Tools.speak("该测站测量完毕")
            } else {
                //一尺测量完
                surveyorData.currentRulerType = RULER_TYPE_1
                surveyorData.currentMeasureTime = MEASURE_TIME_0
                Tools.speak("尺一测量完毕")
            }
        } else {
            //不是最后一次直接加
            surveyorData.currentMeasureTime++
            if (surveyorData.currentMeasureTime > MEASURE_TIME_4) {
                surveyorData.currentMeasureTime = MEASURE_TIME_4
            }
        }

        //存数据库
        viewModelScope.launch(Dispatchers.IO) {
            LevelingDb.getInstance().iDetectionRecordDao().save(record)
        }
    }

    /**
     * 找第一个没有数据的
     */
    private fun findMeasureDataAll(): LevelIDetectionSurveyorData? {
        val recordList = findUtil.getFindMeasureList(surveyorData.currentIDetectionId)
        return findUtil.find(recordList) {
            stationRecordMap[it.id] == null
        }
    }

    /**
     * 找第一个没有数据的
     */
    fun switchStation(stationType: Int) {
        val recordList = findUtil.getFindMeasureList(surveyorData.currentIDetectionId, stationType)
        val find = findUtil.find(recordList) {
            stationRecordMap[it.id] == null
        }
        if (find != null) {
            changeCurrentSurveyorData(find)
        } else {
            val surveyorData = LevelIDetectionSurveyorData("")
            surveyorData.currentStationType = stationType
            changeCurrentSurveyorData(surveyorData)
        }
    }

    private fun changeCurrentSurveyorData(surveyorData: LevelIDetectionSurveyorData) {
        this.surveyorData.currentStationType = surveyorData.currentStationType
        this.surveyorData.currentRulerType = surveyorData.currentRulerType
        this.surveyorData.currentMeasureTime = surveyorData.currentMeasureTime
    }

    /**
     * 找最后一个有数据的
     */
    fun findRollBackData(): LevelIDetectionSurveyorData? {
        val recordList = findUtil.getFindMeasureList(
            surveyorData.currentIDetectionId,
            surveyorData.currentStationType
        ).reversed()
        return findUtil.find(recordList) {
            stationRecordMap[it.id] != null
        }
    }


    /**
     * 回退删除数据
     * 一次删一个测站的一个尺子的数据
     */
    fun deleteStationRulerData(findRollBackData: LevelIDetectionSurveyorData, success: () -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            val deleteIDetectionId = surveyorData.currentIDetectionId
            val deleteStationType = findRollBackData.currentStationType
            val deleteRulerType = findRollBackData.currentRulerType

            val recordId1 = LevelIDetectionRecord.getRecordId(
                deleteIDetectionId,
                deleteStationType,
                deleteRulerType,
                MEASURE_TIME_0
            )
            val recordId2 = LevelIDetectionRecord.getRecordId(
                deleteIDetectionId,
                deleteStationType,
                deleteRulerType,
                MEASURE_TIME_1
            )
            val recordId3 = LevelIDetectionRecord.getRecordId(
                deleteIDetectionId,
                deleteStationType,
                deleteRulerType,
                MEASURE_TIME_2
            )
            val recordId4 = LevelIDetectionRecord.getRecordId(
                deleteIDetectionId,
                deleteStationType,
                deleteRulerType,
                MEASURE_TIME_3
            )
            val recordId5 = LevelIDetectionRecord.getRecordId(
                deleteIDetectionId,
                deleteStationType,
                deleteRulerType,
                MEASURE_TIME_4
            )
            //删除缓存
            stationRecordMap.remove(recordId1)
            stationRecordMap.remove(recordId2)
            stationRecordMap.remove(recordId3)
            stationRecordMap.remove(recordId4)
            stationRecordMap.remove(recordId5)
            //删除数据
            LevelingDbFactory.instance.iDetectionRecordDataSource.deleteById(recordId1)
            LevelingDbFactory.instance.iDetectionRecordDataSource.deleteById(recordId2)
            LevelingDbFactory.instance.iDetectionRecordDataSource.deleteById(recordId3)
            LevelingDbFactory.instance.iDetectionRecordDataSource.deleteById(recordId4)
            LevelingDbFactory.instance.iDetectionRecordDataSource.deleteById(recordId5)

            //确定要测哪个
            findRollBackData.currentMeasureTime = MEASURE_TIME_0
            changeCurrentSurveyorData(findRollBackData)

            withContext(Dispatchers.Main) {
                success()
            }
        }
    }

    /**
     * 保存测量方式
     */
    fun saveMeasureType(iDetectionId: String, measureType: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val record =
                LevelingDbFactory.instance.iDetectionListDataSource.getById(iDetectionId)
            record.measureType = measureType
            LevelingDbFactory.instance.iDetectionListDataSource.update(record)
        }
    }

    /**
     * 单位mm
     * d1短距
     * d2长距
     */
    fun startResult(d1: Double, d2: Double): String? {
        val findMeasureList = findUtil.getFindMeasureList(surveyorData.currentIDetectionId)
        for (needCheckItem in findMeasureList) {
            val levelIDetectionRecord = stationRecordMap[needCheckItem.id]
            if (levelIDetectionRecord == null) {
                ToastUtils.showShort(LevelIDetectionConstant.getStationTypeShow(needCheckItem.stationType) + "数据不完整")
                return null
            }
        }

        val iResult = LevelIDetectionCalUtil.getIResult(
            surveyorData.currentIDetectionId,
            measureType!!,
            d1,
            d2,
            stationRecordMap
        )

        val hint = if (iResult.isFinite()) {
            val formatAngle = LevelFormatUtil.formatAngle(iResult)
            saveIResult(formatAngle)
            if (iResult > 15) {
                "i角计算结果：$formatAngle\"\n大于15\",请校正仪器"
            } else {
                "i角计算结果：$formatAngle\"\n合限"
            }
        } else {
            saveIResult(null)
            "i角计算结果：数据异常，请检查"
        }
        return hint
    }

    /**
     * 存短长距和测量方法
     * 这些变化快，统一存
     */
    private fun saveIResult(iResult: String?) {
        viewModelScope.launch(Dispatchers.IO) {
            val iDetectionItem =
                LevelingDbFactory.instance.iDetectionListDataSource.getById(surveyorData.currentIDetectionId)
            iDetectionItem.iResult = iResult
            LevelingDbFactory.instance.iDetectionListDataSource.update(iDetectionItem)
        }
    }

    /**
     * 存短长距和测量方法
     * 这些变化快，统一存
     */
    fun saveShortLongDistanceMeasureType(d1Short: String, d2Long: String) {
        val iDetectionItem =
            LevelingDbFactory.instance.iDetectionListDataSource.getById(surveyorData.currentIDetectionId)
        iDetectionItem.shortDistance = d1Short
        iDetectionItem.longDistance = d2Long
        iDetectionItem.measureType = measureType
        LevelingDbFactory.instance.iDetectionListDataSource.update(iDetectionItem)
    }

    fun getGetDadaAuto(iDetectionId: String, callBack: (Boolean) -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            val record =
                LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(
                    iDetectionId
                )
            withContext(Dispatchers.Main) {
                if (record == null) {
                    callBack(true)
                } else {
                    callBack(!record.getDataManually)
                }
            }
        }
    }

    fun setGetDataAuto(iDetectionId: String, auto: Boolean) {
        viewModelScope.launch(Dispatchers.IO) {
            var record =
                LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(
                    iDetectionId
                )
            if (record == null) {
                record = LevelIDetectionSetting(iDetectionId)
                record.getDataManually = !auto
                LevelingDbFactory.instance.iDetectionSettingDataSource.save(record)
            } else {
                record.getDataManually = !auto
                LevelingDbFactory.instance.iDetectionSettingDataSource.update(record)
            }
        }
    }

    @WorkerThread
    fun updateStationSetting() {
        stationSetting =
            LevelingDbFactory.instance.iDetectionSettingDataSource.getByIDetectionId(
                surveyorData.currentIDetectionId
            )
    }


}