package com.bingce.controlnetwork.func.level.sendreceive

import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData
import com.bingce.controlnetwork.App

abstract class LevelRequest(protected val backDataI: ILevelBackData) {

    private var requestTime = 0L
    private var requestState = LevelRequestState.INIT

    private val receiveDataList = mutableListOf<LevelReceiveData>()

    abstract fun doRequest()
    abstract fun onStart()
    abstract fun onDestroy()
    abstract fun sendCommand()


    fun requeset() {
        if (checkRequestNot()) {
            return
        }
        requestState = LevelRequestState.REQUEST_ING
        requestTime = System.currentTimeMillis()

        //获取数据
        doRequest()
    }

    private fun checkRequestNot(): Boolean {
        val intervalTime = System.currentTimeMillis() - requestTime
//        if (intervalTime < 600) {
//            return true
//        }
        if (requestState == LevelRequestState.REQUEST_ING) {
            ToastUtils.showShort("测量中,请稍后...")
            return true
        }

        if (requestState == LevelRequestState.REQUEST_ING) {
            if (intervalTime > 20000) {
                //超过20秒回复默认状态
                requestState = LevelRequestState.INIT
            }
        }
        return false
    }

    protected fun resetInit() {
        requestTime = 0
        requestState = LevelRequestState.INIT
    }

    protected fun addReceiveData(data: LevelReceiveData) {
        receiveDataList.add(data)
    }

    protected fun clearReceiveDataList() {
        receiveDataList.clear()
    }

    protected fun isNeedRequestAgain(): Boolean {
        val levelSurveyorTime = App.getApp().levelSurveyorTime.toInt()
        return receiveDataList.size < levelSurveyorTime
    }

    /**
     * 获取返回的数据
     */
    protected fun getBackData(): LevelReceiveData {
        var height = 0.0
        var distance = 0.0
        for (item in receiveDataList) {
            height += item.height
            distance += item.distance
        }
        val size = receiveDataList.size
        return LevelReceiveData(height / size, distance / size)
    }

    protected fun hasData() = receiveDataList.isNotEmpty()

}