package com.bingce.controlnetwork.func.level.sendreceive

import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData
import com.bingce.device.Device
import com.bingce.level.Level
import com.bingce.level.LevelListener

/**
 * 发布版
 */
class LevelRequestRelease(backDataI: ILevelBackData) : LevelRequest(backDataI),
    LevelListener.LevelReadListener {

    override fun doRequest() {
        if (!Device.getInstance().isDeviceConnected) {
            ToastUtils.showShort("设备未连接，请连接设备")
            resetInit()
            return
        }
        ToastUtils.showShort("开始测量")
        sendCommand()
    }

    override fun onStart() {
        //LevelListener
        LevelListener.getInstance().addLevelReadListener(this)
    }

    override fun onDestroy() {
        LevelListener.getInstance().removeLevelReadListener(this)
    }

    override fun onReadSuccess(height: Double, distance: Double) {
        addReceiveData(LevelReceiveData(height, distance))
        if (isNeedRequestAgain()) {
            sendCommand()
            return
        }
        if (hasData()) {
            backDataI.receive(getBackData())
        }
        clearReceiveDataList()
        resetInit()
    }

    override fun onReadFailed(errorCode: String?, errorMsg: String?, original: String?) {
        val error = errorMsg ?: ""
        backDataI.measureError(error)
        clearReceiveDataList()
        resetInit()
    }

    override fun sendCommand() {
        Level.getInstance().requestMeasure()
    }

}