package com.bingce.controlnetwork.func.level.sendreceive

import android.content.res.AssetManager
import android.os.Handler
import android.os.Looper
import android.os.Message
import android.util.Log
import blankj.utilcode.util.ToastUtils
import blankj.utilcode.util.Utils
import com.bingce.controlapphelper.leveling.detaction.LevelReceiveData
import com.bingce.controlapphelper.util.GsonUtil
import com.bingce.path.PathManager
import com.bingce.utils.FileUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import java.io.BufferedReader
import java.io.File
import java.io.IOException
import java.io.InputStreamReader
import java.nio.charset.StandardCharsets
import java.util.regex.Pattern


class LevelRequestTest(backDataI: ILevelBackData) : LevelRequest(backDataI) {
    private var dataIndex = 0
    private var dataList = emptyList<LevelReceiveData>()

    private val handler = object : Handler(Looper.getMainLooper()) {
        override fun handleMessage(msg: Message) {
            val data = dataList[dataIndex]
            addReceiveData(data)
            if (isNeedRequestAgain()) {
                dataIndex++
                sendCommand()
                return
            }

            if (hasData()) {
                backDataI.receive(getBackData())
            }

            clearReceiveDataList()
            resetInit()
        }
    }

    init {
        GlobalScope.launch(Dispatchers.IO) {
            dataList = TestData().getData()
        }
    }

    override fun doRequest() {
        if (dataList.lastIndex < dataIndex) {
            ToastUtils.showShort("没有数据了")
            return
        }
        ToastUtils.showShort("开始测量")
        sendCommand()
    }

    override fun onStart() {

    }

    override fun onDestroy() {

    }

    override fun sendCommand() {
        handler.sendEmptyMessageDelayed(0, 500)
    }

    fun initTestData(position: Int) {
        dataList = TestData().getData(position)
    }

}

class TestData {
    private val data11 =
        "[{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0},{\"distance\":1.0,\"height\":1.0}]"
    private val dataBFFBNormal =
        "[{\"distance\":4.154,\"height\":2.42073},{\"distance\":4.905,\"height\":0.97681},{\"distance\":4.904,\"height\":0.97685},{\"distance\":4.157,\"height\":2.42074},{\"distance\":5.367,\"height\":0.91105},{\"distance\":6.041,\"height\":2.55628},{\"distance\":6.047,\"height\":2.55615},{\"distance\":5.371,\"height\":0.91101},{\"distance\":5.837,\"height\":2.45984},{\"distance\":5.553,\"height\":0.92191},{\"distance\":5.553,\"height\":0.92189},{\"distance\":5.837,\"height\":2.45978},{\"distance\":6.341,\"height\":0.85347},{\"distance\":6.85,\"height\":2.69778},{\"distance\":6.833,\"height\":2.69768},{\"distance\":6.345,\"height\":0.85337},{\"distance\":7.445,\"height\":2.66834},{\"distance\":7.025,\"height\":0.67221},{\"distance\":7.025,\"height\":0.67225},{\"distance\":7.44,\"height\":2.66833},{\"distance\":6.899,\"height\":0.69217},{\"distance\":7.812,\"height\":2.55316},{\"distance\":7.815,\"height\":2.55313},{\"distance\":6.9,\"height\":0.69219},{\"distance\":6.717,\"height\":2.53018},{\"distance\":6.31,\"height\":0.89984},{\"distance\":6.31,\"height\":0.89981},{\"distance\":6.72,\"height\":2.53014},{\"distance\":6.785,\"height\":0.75673},{\"distance\":7.375,\"height\":2.6826},{\"distance\":7.372,\"height\":2.68262},{\"distance\":6.79,\"height\":0.75672},{\"distance\":6.753,\"height\":2.55411},{\"distance\":7.238,\"height\":0.56473}]"

    private val dataBFFBError1 =
        "[{\"distance\":4.154,\"height\":2.42073},{\"distance\":4.905,\"height\":0.97681},{\"distance\":5.904,\"height\":1.97685},{\"distance\":5.157,\"height\":3.42074},{\"distance\":5.367,\"height\":0.91105},{\"distance\":6.041,\"height\":2.55628},{\"distance\":6.047,\"height\":2.55615},{\"distance\":5.371,\"height\":0.91101},{\"distance\":5.837,\"height\":2.45984},{\"distance\":5.553,\"height\":0.92191},{\"distance\":5.553,\"height\":0.92189},{\"distance\":5.837,\"height\":2.45978},{\"distance\":6.341,\"height\":0.85347},{\"distance\":6.85,\"height\":2.69778},{\"distance\":6.833,\"height\":2.69768},{\"distance\":6.345,\"height\":0.85337},{\"distance\":7.445,\"height\":2.66834},{\"distance\":7.025,\"height\":0.67221},{\"distance\":7.025,\"height\":0.67225},{\"distance\":7.44,\"height\":2.66833},{\"distance\":6.899,\"height\":0.69217},{\"distance\":7.812,\"height\":2.55316},{\"distance\":7.815,\"height\":2.55313},{\"distance\":6.9,\"height\":0.69219},{\"distance\":6.717,\"height\":2.53018},{\"distance\":6.31,\"height\":0.89984},{\"distance\":6.31,\"height\":0.89981},{\"distance\":6.72,\"height\":2.53014},{\"distance\":6.785,\"height\":0.75673},{\"distance\":7.375,\"height\":2.6826},{\"distance\":7.372,\"height\":2.68262},{\"distance\":6.79,\"height\":0.75672},{\"distance\":6.753,\"height\":2.55411},{\"distance\":7.238,\"height\":0.56473}]"

    private val dataBFFBError2 =
        "[{\"distance\":4.154,\"height\":2.42073},{\"distance\":14.905,\"height\":0.97681},{\"distance\":15.904,\"height\":0.97685},{\"distance\":5.157,\"height\":3.42074},{\"distance\":5.367,\"height\":0.91105},{\"distance\":6.041,\"height\":2.55628},{\"distance\":6.047,\"height\":2.55615},{\"distance\":5.371,\"height\":0.91101},{\"distance\":5.837,\"height\":2.45984},{\"distance\":5.553,\"height\":0.92191},{\"distance\":5.553,\"height\":0.92189},{\"distance\":5.837,\"height\":2.45978},{\"distance\":6.341,\"height\":0.85347},{\"distance\":6.85,\"height\":2.69778},{\"distance\":6.833,\"height\":2.69768},{\"distance\":6.345,\"height\":0.85337},{\"distance\":7.445,\"height\":2.66834},{\"distance\":7.025,\"height\":0.67221},{\"distance\":7.025,\"height\":0.67225},{\"distance\":7.44,\"height\":2.66833},{\"distance\":6.899,\"height\":0.69217},{\"distance\":7.812,\"height\":2.55316},{\"distance\":7.815,\"height\":2.55313},{\"distance\":6.9,\"height\":0.69219},{\"distance\":6.717,\"height\":2.53018},{\"distance\":6.31,\"height\":0.89984},{\"distance\":6.31,\"height\":0.89981},{\"distance\":6.72,\"height\":2.53014},{\"distance\":6.785,\"height\":0.75673},{\"distance\":7.375,\"height\":2.6826},{\"distance\":7.372,\"height\":2.68262},{\"distance\":6.79,\"height\":0.75672},{\"distance\":6.753,\"height\":2.55411},{\"distance\":7.238,\"height\":0.56473}]"

    private val dataBFFBError3 =
        "[{\"distance\":4.154,\"height\":2.42073},{\"distance\":4.905,\"height\":0.97681},{\"distance\":4.904,\"height\":0.97685},{\"distance\":4.157,\"height\":2.42074},{\"distance\":4.154,\"height\":2.42073},{\"distance\":4.905,\"height\":0.97681},{\"distance\":4.904,\"height\":0.97685},{\"distance\":4.157,\"height\":2.42074}]"

    companion object {
        val textShowList = listOf(
            "11数据",
            "BFFB",
            "BFFB异常数据1",
            "BFFB异常数据-前后视距差",
            "BFFB异常数据-转角差"
        )
    }

    fun getData(): List<LevelReceiveData> {
        return GsonUtil.toAny<List<LevelReceiveData>>(dataBFFBNormal) ?: emptyList()
    }

    fun getData(position: Int): List<LevelReceiveData> {
        return when (position) {
            0 -> GsonUtil.toAny<List<LevelReceiveData>>(data11) ?: emptyList()
            1 -> GsonUtil.toAny<List<LevelReceiveData>>(dataBFFBNormal) ?: emptyList()
            2 -> GsonUtil.toAny<List<LevelReceiveData>>(dataBFFBError1) ?: emptyList()
            3 -> GsonUtil.toAny<List<LevelReceiveData>>(dataBFFBError2) ?: emptyList()
            4 -> GsonUtil.toAny<List<LevelReceiveData>>(dataBFFBError3) ?: emptyList()
            else -> emptyList()
        }
    }

    fun test() {
        val processFile = processFile("test666.DAT")
        val toJson = GsonUtil.toJson(processFile)
        FileUtil.writeString(toJson, File(PathManager.appDocumentFolder(), "test.json"))
        Log.d("hwhw", "hwhwhw写入成功")
    }

    private fun loadJsonFromAsset(fileName: String): String? {
        val json: String = try {
            val manager: AssetManager = Utils.getApp().assets
            val `is` = manager.open(fileName)
            val size = `is`.available()
            val buffer = ByteArray(size)
            `is`.read(buffer)
            `is`.close()
            String(buffer, StandardCharsets.UTF_8)
        } catch (e: IOException) {
            e.printStackTrace()
            return null
        }
        return json
    }

    private fun processFile(fileName: String): List<LevelReceiveData> {
        val result = mutableListOf<LevelReceiveData>()
        try {
            val assetManager: AssetManager = Utils.getApp().assets
            val inputStream = assetManager.open(fileName)
            val reader = BufferedReader(InputStreamReader(inputStream))
            var line = ""

            // 定义正则表达式
            val regex = "(Rf|Rb)\\s+([\\d.]+)\\s+m"
            val regex2 = "HD\\s+([\\d.]+)\\s+m"
            val pattern1 = Pattern.compile(regex)
            val pattern2 = Pattern.compile(regex2)

            while (reader.readLine()?.also { line = it } != null && !line.contains("#")) {
                val matcher = pattern1.matcher(line)
                val matcher2 = pattern2.matcher(line)

                if (matcher.find() && matcher2.find()) {
                    val height = matcher.group(2)?.toDouble() ?: 0.0
                    val distance = matcher2.group(1)?.toDouble() ?: 0.0
                    result.add(LevelReceiveData(height, distance))
                }
            }
            inputStream.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return result
    }

}