package com.bingce.controlnetwork.func.level.util;

import com.bingce.controlnetwork.func.level.util.suntime.SunTimes;

import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalTime;
import java.time.ZonedDateTime;
import java.util.Date;



public class LevelDateUtil {
    /**
     * 34.757975, 113.665412
     *
     * @param lat
     * @param lng
     */
    public static String getLevelOperateSunTimeHint(double lat, double lng) {
        ZonedDateTime now = ZonedDateTime.now();
        SunTimes times = SunTimes.compute()
                .on(now.with(LocalTime.MIN))   // set a date
                .at(lat, lng)   // set a location
                .execute();     // get the results

        ZonedDateTime rise = times.getRise();
        ZonedDateTime set = times.getSet();
        if (rise == null || set == null) {
            //未计算出日出和日落时间
            return null;
        }
        // 计算日中时间
        ZonedDateTime noon = rise.plus(Duration.between(rise, set).dividedBy(2));

        // 计算前后两个小时的时间窗口
        ZonedDateTime startWindow = noon.minusHours(2);
        ZonedDateTime endWindow = noon.plusHours(2);

        ZonedDateTime riseAfter2 = rise.plusMinutes(30);
        ZonedDateTime setBefore2 = set.minusMinutes(30);

        boolean isWithinTwoHoursOfNoon = !now.isBefore(startWindow) && !now.isAfter(endWindow);

        if (now.isBefore(riseAfter2)) {
            //当前时间在日出前2小时内
            return "请在日出后30分钟进行作业";
        }

        if (now.isAfter(setBefore2)) {
            //当前时间在日落后2小时
            return "请在日落前30分钟进行作业";
        }

        // 输出结果
        if (isWithinTwoHoursOfNoon) {
            //当前时间在日中前后2小时内
            return "请勿在日中前后2小时内进行作业";
        }

        return null;
    }

    public static String dateToYearMonthDayHourMin(Date date) {
        SimpleDateFormat simple = new SimpleDateFormat("yyyy年MM月dd日HH:mm");
        return simple.format(date);
    }

    public static String getLevelHeadDateYearMonthDay(Date date) {
        SimpleDateFormat simple = new SimpleDateFormat("yyyy年MM月dd日");
        return simple.format(date);
    }

    public static String getLevelHeadDateHourMin(Date date) {
        SimpleDateFormat simple = new SimpleDateFormat("HH:mm");
        return simple.format(date);
    }

    public static String getHourMinSec(Date date) {
        SimpleDateFormat simple = new SimpleDateFormat("HH:mm:ss");
        return simple.format(date);
    }

}
