package com.bingce.controlnetwork.func.level.util

import android.text.TextUtils
import androidx.annotation.WorkerThread
import blankj.utilcode.util.Utils

import com.bingce.controlnetwork.func.level.export.LevelExportIDetection
import com.bingce.controlnetwork.func.level.export.LevelExportSingalHeight
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.measure.backfront.LevelingMeasureBackFrontRecord
import com.bingce.controlapphelper.datasource.database.leveling.route.LevelRouteRecord
import com.bingce.controlapphelper.datasource.database.leveling.station.LevelingStationRecord
import com.bingce.controlapphelper.datasource.database.point.PointRecord
import com.bingce.controlapphelper.leveling.detaction.LevelFormatUtil
import com.bingce.controlapphelper.model.LevelExportLine
import com.bingce.controlapphelper.model.LevelingMeasureOrder
import com.bingce.controlapphelper.model.LevelingMeasureOrderUtil
import com.bingce.controlapphelper.model.LevelingStationState
import com.bingce.controlapphelper.model._N
import com.bingce.controlapphelper.surveyor.method.ExportStationMethod
import com.bingce.controlapphelper.surveyor.method.ExportUtils
import com.bingce.controlapphelper.util.DateUtil
import com.bingce.utils.DateUtils
import com.bingce.utils.Util
import java.io.File


class LevelExportUtilTj {

    companion object {
        const val EXPORT_LEVEL_FOLDER = "Leveling"

        //导出的外层目录
        val exportOutFolder = File(Utils.getApp().externalCacheDir, EXPORT_LEVEL_FOLDER)
    }

    private lateinit var exportRouteFolder: File
    private lateinit var exportProjectFolder: File

    private var column1Index = 0

    private var startPointHeight = 0.0
    private var startPointName = ""
    private var endPointName = ""


    private val printSbLev = StringBuilder()

    /**
     * 导出路线文件夹下的数据
     */
    @WorkerThread
    fun doExportRouteData(projectId: String, routeId: String) {
        val routeRecord = LevelingDbFactory.instance.levelDataSource.getByIdSync(routeId)
            ?: throw IllegalArgumentException("routeRecord==null")
        //路线的基础数据
        confirmRouteCommonData(routeRecord)
        //收集路线的具体数据
        collectRouteLevData(projectId, routeRecord)
    }

    private fun confirmRouteCommonData(it: LevelRouteRecord) {
        val startPoint =
            SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(it.startPointId)
        startPointHeight = getPointHeight(startPoint)
        startPointName = getPointName(startPoint)
        val endPoint = SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(it.endPointId)
        endPointName = getPointName(endPoint)
    }

    private fun collectRouteLevData(projectId: String, it: LevelRouteRecord) {
        val routeId = it.getId()
        val routeName = it.routeName
        val levelingMeasureOrder = it.levelingMeasureOrder
        val isHasTwiceStation = when (levelingMeasureOrder) {
            LevelingMeasureOrder.SIM_BF,
            LevelingMeasureOrder.SIM_BFFB -> {
                true
            }

            else -> false
        }

        collectRouteLevDataLeftOrRight(
            projectId,
            routeId,
            false,
            levelingMeasureOrder
        )
        if (isHasTwiceStation) {
            collectRouteLevDataLeftOrRight(
                projectId,
                routeId,
                true,
                levelingMeasureOrder
            )
        }

        if (printSbLev.isNotEmpty()) {
            initExportFile(projectId, routeName)
            if (::exportRouteFolder.isInitialized) {
                saveRouteLevFile(exportRouteFolder, routeName)
                saveRouteTxtFile(exportRouteFolder, routeId, routeName)
                saveRouteInfoFile(exportRouteFolder, projectId, routeId, routeName)
            }
        }
    }

    /**
     * INF 观测记录
     */
    private fun saveRouteInfoFile(
        exportFolder: File,
        projectId: String,
        routeId: String,
        routeName: String
    ) {
        val projectRecord = SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId)
        val routeRecord = LevelingDbFactory.instance.levelDataSource.getByIdSync(routeId)!!
        val toleranceTetail =
            LevelingDbFactory.instance.toleranceDetailDataSource.getRecordSync(projectRecord.toleranceId)

        val startPointNameRoute =
            SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord.startPointId)
                .getName()
        val endPointNameRoute =
            SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(routeRecord.endPointId)
                .getName()

        //作业单位
        val measureUnit = projectRecord.measureUnit ?: ""
        //限差等级
        val exportGrade = toleranceTetail.exportGrade

        //测站数量 单程双转的按照twiceStaion false处理
        val stationCount =
            LevelingDbFactory.instance.stationDataSource.getCountByRoute(projectId, routeId, false)

        val stationListLeft =
            LevelingDbFactory.instance.stationDataSource.getStationList(projectId, routeId, false)
        val stationListRight =
            LevelingDbFactory.instance.stationDataSource.getStationList(projectId, routeId, true)

        val remeasureCountAll =
            getRemeasureCount(stationListLeft) + getRemeasureCount(stationListRight)

        //8,9,10行
        val routeDistanceHeightInfo =
            getRouteDistanceHeightInfo(
                stationListLeft?.lastOrNull(),
                stationListRight?.lastOrNull()
            )

        val record = LevelingDbFactory.instance.stationSettingDataSource.getLatest(routeId)
        record?.let {
            val sb = StringBuilder()
            //第一行
            sb.append(measureUnit).append(",")
            sb.append(exportToString(it.watcher)).append(",")
            sb.append(exportToString(it.recorder)).append(",")
            sb.append(exportToString(it.examiner)).append("\n")
            //第二行
            sb.append(exportGrade).append(",")
            sb.append("").append(",")//观测方法：自动采集法
            sb.append(routeRecord.observationDirection).append(",")
            sb.append(stationCount).append(",")//测站数
            sb.append(remeasureCountAll).append("\n")//重测数
            //第三行
            sb.append(exportToString(it.instrumentModel)).append(",")
            sb.append("No.${exportToString(it.deviceNumber)}").append(",")
            sb.append("").append(",")//测微读数精度
            sb.append("").append(",")//视距乘常数
            sb.append("").append("\n")//视距加常数
            //第四行
            sb.append(exportToString(it.rulerType)).append(",")
            sb.append("").append(",")//标尺分划
            sb.append(exportToString(it.rulerReceptType)).append("\n")//尺承类型
            //第五行
            sb.append(startPointNameRoute).append(",")//起点
            sb.append(endPointNameRoute).append("\n")//终点
            //第六行
            sb.append(exportToString(it.firstStationBackFootNumber)).append(",")
            sb.append(exportToString(it.firstStationFrontFootNumber)).append("\n")
            //第七行
            sb.append(exportToString(it.firstStationBackFootNumber2)).append(",")
            sb.append(exportToString(it.firstStationFrontFootNumber2)).append("\n")
            //第八行
            sb.append(routeDistanceHeightInfo)

            ExportStationMethod.save(sb.toString(), exportFolder, "${routeName}.INF")
        }
    }

    /**
     *  INFO文件里的距离和高差
     */
    private fun getRouteDistanceHeightInfo(
        leftEndStation: LevelingStationRecord?,
        rightEndStation: LevelingStationRecord?
    ): String {
        val sb = StringBuilder()
        if (leftEndStation == null && rightEndStation == null) {
            sb.append("").append(",").append("").append(",").append("").append(",").append("")
                .append("\n")
                .append(",").append("\n")
                .append("").append("\n")
                .append("").append("\n")
            return sb.toString()
        }

        val left =
            LevelingDbFactory.instance.measureStationDataSource.getRecordSync(leftEndStation?.getId())
        val right =
            LevelingDbFactory.instance.measureStationDataSource.getRecordSync(rightEndStation?.getId())

        val distanceTotalLeft = formatDistance(left?.distanceTotal)
        val heightDiffTotalLeft = formatDistance(left?.heightDiffTotal)
        val distanceTotalRight = formatDistance(right?.distanceTotal)
        val heightDiffTotalRight = formatDistance(right?.heightDiffTotal)

        val heightAverage = if (left?.heightDiffTotal != null && right?.heightDiffTotal != null) {
            formatDistance((left.heightDiffTotal + right.heightDiffTotal) / 2)
        } else if (left?.heightDiffTotal != null) {
            formatDistance(left.heightDiffTotal)
        } else {
            ""
        }

        //测段不符值 右-左
        val rightLeftHeightDiff =
            if (left?.heightDiffTotal != null && right?.heightDiffTotal != null) {
                formatDistance(right.heightDiffTotal - left.heightDiffTotal)
            } else {
                "0"
            }

        sb.append(distanceTotalLeft).append(",").append(heightDiffTotalLeft).append(",")
            .append(distanceTotalRight).append(",").append(heightDiffTotalRight).append("\n")
            //第二行
            .append(heightAverage).append("\n")
            //第三行
            .append(rightLeftHeightDiff).append("\n")

        return sb.toString()
    }

    private fun formatDistance(value: Double?): String {
        if (value == null) {
            return ""
        }
        return ExportUtils.formatDistance(value)
    }

    private fun exportToString(value: String?) = value ?: ""

    /**
     * 通过测站列表获取重测数量
     */
    @WorkerThread
    private fun getRemeasureCount(stationList: List<LevelingStationRecord>): Int {
        var count = 0
        for (station in stationList) {
            val remeasureCountByStationId =
                LevelingDbFactory.instance.measureBackFrontDataSource.getRemeasureCountByStationId(
                    station.getId()
                )
            count += remeasureCountByStationId
        }
        return count
    }

    /**
     * TXT 观测条件
     */
    private fun saveRouteTxtFile(exportFolder: File, routeId: String, routeName: String) {
//        //起点，终点，照准次序，测站数量，路线总长度（km），路线总高差
//        val routeRecord = TjDbFactory.instance.routeDataSource.getByIdSync(routeId) ?: return
//        val startPointName =
//            TjDbFactory.instance.pointDataSource.findByIdSync(routeRecord.startPointId).name
//        val endPointName =
//            TjDbFactory.instance.pointDataSource.findByIdSync(routeRecord.endPointId).name
//        val orderText =
//            LevelingMeasureOrderUtil.getTextByLevelingMeasureOrder(routeRecord.levelingMeasureOrder)
//
//        val stationCount = LevelingDbFactory.instance.stationDataSource.getCountByRoute(
//            routeRecord.projectId,
//            routeId,
//            false
//        )
//
//        val stationLatest = LevelingDbFactory.instance.stationDataSource.getStationLatest(
//            routeRecord.projectId,
//            routeId
//        )
//
//        val measureStationList = mutableListOf<LevelingMeasureStationRecord>()
//        val measureStationRecordLast =
//            LevelingDbFactory.instance.measureStationDataSource.getRecordSync(stationLatest.getId())
//        if (stationLatest.twiceStation) {
//            val measureStationRecordLeft =
//                LevelingDbFactory.instance.measureStationDataSource.getRecordSync(stationLatest.lastStationId)
//            measureStationList.add(measureStationRecordLeft)
//            measureStationList.add(measureStationRecordLast)
//        } else {
//            measureStationList.add(measureStationRecordLast)
//        }
//
//        val sb = StringBuilder()
//        for (item in measureStationList) {
//            sb.append(startPointName).append(",")
//            sb.append(endPointName).append(",")
//            sb.append(orderText).append(",")
//            sb.append(stationCount).append(",")
//
//            sb.append(ExportUtilsTj.formatDistance(item.distanceTotal)).append(",")
//            sb.append(ExportUtilsTj.formatDistance(item.heightDiffTotal))
//            sb.append("\n")
//        }

        //观测条件list
        val observationConditionList =
            LevelingDbFactory.instance.stationSettingDataSource.getByRouteId(routeId)

        val sb = StringBuilder()
        for (item in observationConditionList) {
            if (item.stationId==null) {
                continue
            }
            val station =
                LevelingDbFactory.instance.stationDataSource.getRecordSync(item.stationId)
                    ?: continue
            val stationNum = station.stationName.replace("测站", "")

            //第一行
            sb.append(stationNum).append(",")
                .append(LevelDateUtil.dateToYearMonthDayHourMin(item.createDate)).append("\n")
            //第二行
            sb.append(exportToString(item.temperature)).append(",")
                .append(exportToString(item.sunDirection)).append(",")
                .append(exportToString(item.imaging)).append(",")
                .append(exportToString(item.weather)).append("\n")
            //第三行
            sb.append(exportToString(item.windDirection)).append(",")
                .append(exportToString(item.windSpeed)).append(",")
                .append(exportToString(item.roads)).append(",")
                .append(exportToString(item.soil)).append("\n")
        }

        ExportStationMethod.save(sb.toString(), exportFolder, "${routeName}.TXT")
    }

    private fun removeLastBuilder(sb: StringBuilder) {
        if (sb.isNotEmpty()) {
            sb.deleteCharAt(sb.lastIndex)
        }
    }

    private fun saveRouteLevFile(exportFolder: File, routeName: String) {
        ExportStationMethod.save(printSbLev.toString(), exportFolder, "${routeName}.LEV")
    }

    private fun initExportFile(projectId: String, routeName: String) {
        exportProjectFolder = initExportProjectFolder(projectId)
        confirmRouteFolder(exportProjectFolder, routeName)
    }

    private fun initExportProjectFolder(projectId: String): File {
        val exportFolder = File(Utils.getApp().externalCacheDir, EXPORT_LEVEL_FOLDER)
        if (!exportFolder.exists()) {
            exportFolder.mkdirs()
        }
        val projectName =
            SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId).projectName
        val projectFolder = File(exportFolder, projectName)
        if (!projectFolder.exists()) {
            projectFolder.mkdir()
        }
        return projectFolder
    }

    /**
     * 重名项目或路线文件夹增加_1
     */
    private fun confirmRouteFolder(exportProjectFolder: File, name: String) {
        val exportFolderTarget = File(exportProjectFolder, name)
        if (!exportFolderTarget.exists()) {
            exportFolderTarget.mkdir()
            this.exportRouteFolder = exportFolderTarget
            return
        }
        confirmRouteFolder(exportProjectFolder, name + "_1")
    }

    /**
     * 收集左线或者右线的数据
     */
    private fun collectRouteLevDataLeftOrRight(
        projectId: String,
        routeId: String,
        twiceStation: Boolean,
        levelingMeasureOrder: LevelingMeasureOrder
    ) {
        val orderText =
            LevelingMeasureOrderUtil.getTextByLevelingMeasureOrder(levelingMeasureOrder)

        val stationList = LevelingDbFactory.instance.stationDataSource.getStationList(
            projectId,
            routeId,
            twiceStation
        )

        var finishIndex = 0
        for ((index, station) in stationList.withIndex()) {
            if (station.stationState != LevelingStationState.FINISH)
                break

            if (index == 0) {
                appendStartOrEndLine("Start-Line", orderText)
                appendRouteStationHead(routeId, station)
                appendZ(station.backSightPointId, startPointHeight)
            }

            //测量数据
            appendMeasureData(station)
            finishIndex = index
            column1Index++
        }

        //最后完成测站
        val lastFinishStation = stationList[finishIndex]
        val stationHeightDiffTotal = getStationHeightDiffTotal(lastFinishStation)

        //结束点高程
        appendZ(
            lastFinishStation.frontSightPointId,
            getFrontPointZ(stationHeightDiffTotal)
        )
        column1Index++
        //end-line
        appendStartOrEndLine("End-Line", "")
    }

    private fun getStationHeightDiffTotal(stationRecord: LevelingStationRecord): Double {
        //当前站的累积高差
        return LevelingDbFactory.instance.measureStationDataSource.getRecordSync(stationRecord.getId()).heightDiffTotal
    }

    private fun getFrontPointZ(heightDiffTotal: Double) = startPointHeight + heightDiffTotal

    private fun appendStartOrEndLine(startOrEndLine: String, column2Name3: String) {
        val dataFirstLine = LevelExportLine(
            LevelExportLine.Column1(column1Index),
            LevelExportLine.Column2("TO", startOrEndLine, column2Name3),
            LevelExportLine.ColumnHeight("", ""),
            LevelExportLine.ColumnDistance(""),
            LevelExportLine.ColumnEnd(null, null)
        ).getData()
        printSbLev.append(dataFirstLine).append(_N)

        column1Index++
    }

    /**
     * 路线的首站头
     * 起点，终点等
     */
    private fun appendRouteStationHead(routeId: String, station: LevelingStationRecord) {
        val stationSetting = LevelingDbFactory.instance.stationSettingDataSource.getLatest(routeId)

        val measureDataList =
            LevelingDbFactory.instance.measureBackFrontDataSource.getMeasureDataList(station.getId())
        val firstRecordDate = measureDataList.first().createDate
        val endRecordDate = measureDataList.last().createDate

        val levelHeadDateYearMonthDay =
            LevelDateUtil.getLevelHeadDateYearMonthDay(firstRecordDate)
        val startHourMin = LevelDateUtil.getLevelHeadDateHourMin(firstRecordDate)
        val endHourMin = LevelDateUtil.getLevelHeadDateHourMin(endRecordDate)

        val line1 = getRouteStationHeadLine("startName", startPointName)
        printSbLev.append(line1).append(_N)
        val line2 = getRouteStationHeadLine("endName", endPointName)
        printSbLev.append(line2).append(_N)
        val line3 = getRouteStationHeadLine("observationDate", levelHeadDateYearMonthDay)
        printSbLev.append(line3).append(_N)
        val line4 = getRouteStationHeadLine("observationStartTime", startHourMin)
        printSbLev.append(line4).append(_N)
        val line5 = getRouteStationHeadLine("observationEndTime", endHourMin)
        printSbLev.append(line5).append(_N)
        val line6 = getRouteStationHeadLine("weather", stationSetting.weather ?: "")
        printSbLev.append(line6).append(_N)
        val line7 = getRouteStationHeadLine("imaging", stationSetting.imaging ?: "")
        printSbLev.append(line7).append(_N)

        column1Index++
    }

    private fun getRouteStationHeadLine(key: String, value: String): String {
        return LevelExportLine(
            LevelExportLine.Column1(column1Index),
            LevelExportLine.Column2("INF", "", ""),
            LevelExportLine.ColumnHeight(key, ""),
            LevelExportLine.ColumnDistance(""),
            LevelExportLine.ColumnEnd(value, "")
        ).getData()
    }

    /**
     * 测量数据
     */
    private fun appendMeasureData(station: LevelingStationRecord) {
        val measureDataList =
            LevelingDbFactory.instance.measureBackFrontDataSource.getMeasureDataList(station.getId())

        val stationHeightDiffTotal = getStationHeightDiffTotal(station)

        val backDataList = mutableListOf<LevelingMeasureBackFrontRecord>()
        val frontDataList = mutableListOf<LevelingMeasureBackFrontRecord>()

        for ((index, record) in measureDataList.withIndex()) {
            val point = getPoint(record.measurePointId)
            val pointName = getPointName(point)

            var rbOrRf: String
            if (record.measurePointId == station.backSightPointId) {
                rbOrRf = "Rb"
                backDataList.add(record)
            } else {
                rbOrRf = "Rf"
                frontDataList.add(record)
            }

            val time = DateUtil.formatDateHHmmss(record.createDate)

            val line = LevelExportLine(
                LevelExportLine.Column1(column1Index),
                LevelExportLine.Column2("KD1", pointName, time),
                LevelExportLine.ColumnHeight(rbOrRf, "${record.measureHeightShow} m"),
                LevelExportLine.ColumnDistance(record.measureDistanceShow),
                LevelExportLine.ColumnEnd(null, null)
            ).getData()
            printSbLev.append(line).append(_N)

            //最后再增加一行
            if (index == measureDataList.lastIndex) {
                //每一站的结果
                column1Index++
                appendZ(point.getId(), getFrontPointZ(stationHeightDiffTotal))
            }

            column1Index++
        }
    }

    private fun getPointHeight(point: PointRecord?): Double {
        return point?.getZ() ?: 0.0
    }

    private fun getPointName(point: PointRecord?): String {
        return point?.getName() ?: ""
    }

    private fun getPoint(pointId: String): PointRecord {
        return SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(pointId)
    }

    private fun appendZ(pointId: String, height: Double) {
        val startPoint = getPoint(pointId)
        val z = LevelFormatUtil.formatHeight(height)
        val dataSecondLine = LevelExportLine(
            LevelExportLine.Column1(column1Index),
            LevelExportLine.Column2("KD1", getPointName(startPoint), ""),
            LevelExportLine.ColumnHeight("", ""),
            LevelExportLine.ColumnDistance(""),
            LevelExportLine.ColumnEnd("Z", "$z m")
        ).getData()
        printSbLev.append(dataSecondLine).append(_N)
        column1Index++
    }


    data class CalTotal(var heightDiffTotal: Double = 0.0, var distanceTotal: Double = 0.0) {
        fun addHeightDiffTotal(heightDiffTotal: Double) {
            this.heightDiffTotal += heightDiffTotal
        }

        fun addDistanceTotal(distanceTotal: Double) {
            this.distanceTotal += distanceTotal
        }
    }

    @WorkerThread
    fun exportProjectTxt(projectId: String) {
        val projectFolder = initExportProjectFolder(projectId)
        val projectRecord = SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId)

        val text = projectRecord.projectName +
                "," +
                "" +
                "," +
                "," +
                "," +
                "," +
                "," +
                "," +
                "," +
                (if (projectRecord == null) "" else projectRecord.getMeasureUnit()) +
                "," +
                (if (projectRecord == null) "" else DateUtils.toDate(projectRecord.getCreateDate())) +
                "\n"
        ExportStationMethod.save(
            text,
            projectFolder,
            "project.TXT"
        )
    }

    fun exportControlTxt(projectId: String) {
        val projectFolder = initExportProjectFolder(projectId)
        val knownList = SurveyorDatabaseFactory.instance.pointDataSource.getListByProjectId(
            projectId,
            PointRecord.TYPE_KNOWN_POINT
        )
        val unknownList = SurveyorDatabaseFactory.instance.pointDataSource.getListByProjectId(
            projectId,
            PointRecord.TYPE_UNKNOWN_POINT
        )

        val stringBuilder = StringBuilder()
        appendExportPoint(stringBuilder, knownList)
        appendExportPoint(stringBuilder, unknownList)

        ExportStationMethod.save(
            stringBuilder.toString(),
            projectFolder,
            "controlpoint.TXT"
        )
    }

    private fun appendExportPoint(sb: StringBuilder, list: List<PointRecord>) {
        for (pointRecord in list) {
            sb
                .append(pointRecord.getName())
                .append(",")
                .append(pointRecord.getCode())
                .append(",")
                .append(getXyzFormat(pointRecord.getX()))
                .append(",")
                .append(getXyzFormat(pointRecord.getY()))
                .append(",")
                .append(getXyzFormat(pointRecord.getZ()))
                .append(",")
//                .append(pointRecord.getLongitude())
                .append(",")
//                .append(pointRecord.getLatitude())
                .append(",")
                .append("")
                .append("\n")
        }
    }

    private fun getXyzFormat(xyz: Double): String {
        return Util.formatDouble2String(xyz, 4)
    }

    /**
     * 导出i角检测
     */
    @WorkerThread
    fun exportIDetection(projectId: String) {
        val projectName =
            SurveyorDatabaseFactory.instance.projectDataSource.getRecord(projectId).projectName

        val iDetectionList =
            LevelingDbFactory.instance.iDetectionListDataSource.getListByProjectIdDesc(projectId)

        val sb = StringBuilder()
        val levelExportIDetection = LevelExportIDetection()
        for (iDetection in iDetectionList) {
            val export = levelExportIDetection.export(iDetection)
            if (!TextUtils.isEmpty(export)) {
                sb.append(export)
            }
        }

        if (sb.isNotEmpty()) {
            val projectFolder = initExportProjectFolder(projectId)
            ExportStationMethod.save(
                sb.toString(),
                projectFolder,
                "$projectName.IAC"
            )
        }
    }


    /**
     * 导出
     */
    fun exportTjSingalHeight(projectId: String) {
        val levelExportSingalHeight = LevelExportSingalHeight()
        val projectFolder = initExportProjectFolder(projectId)
        levelExportSingalHeight.export(projectFolder, projectId)
    }


}