package com.bingce.controlnetwork.func.level.util

import android.widget.Spinner
import androidx.annotation.WorkerThread
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.stationsetting.LevelingStationSettingRecord
import com.bingce.controlapphelper.model.LevelRouteSave
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R

object LevelStationSettingUtil {

    /**
     * 保存测站设置
     */
    fun saveStationSetting(
        isSaveEndStationToStationSetting: Boolean,
        levelRouteSave: LevelRouteSave
    ) {

        var saveEndStation = isSaveEndStationToStationSetting

        val stationSetting = if (levelRouteSave.stationId != null) {
            val before =
                LevelingDbFactory.instance.stationSettingDataSource.getByStationId(levelRouteSave.stationId)

            if (before != null) {
                saveEndStation = before.setEndStationSetting or saveEndStation
            }

            before ?: LevelingStationSettingRecord()
        } else {
            LevelingStationSettingRecord()
        }

        //初始化保存的endStation
        stationSetting.setEndStationSetting = saveEndStation

        stationSetting.routeId = levelRouteSave.routeId
        stationSetting.stationId = levelRouteSave.stationId
        stationSetting.watcher = levelRouteSave.watcher
        stationSetting.recorder = levelRouteSave.recorder
        stationSetting.examiner = levelRouteSave.examiner
        stationSetting.firstStationBackFootNumber = levelRouteSave.backFootNumber
        stationSetting.firstStationFrontFootNumber = levelRouteSave.frontFootNumber
        stationSetting.firstStationBackFootNumber2 = levelRouteSave.backFootNumber2
        stationSetting.firstStationFrontFootNumber2 = levelRouteSave.frontFootNumber2
        stationSetting.instrumentModel = levelRouteSave.instrumentModel
        stationSetting.deviceNumber = levelRouteSave.deviceNumber
        stationSetting.steelRuleParam = levelRouteSave.steelRuleParam
        stationSetting.rulerLengthProvingTemperature = levelRouteSave.rulerLengthProvingTemperature
        stationSetting.rulerCorrectionFactor = levelRouteSave.rulerCorrectionFactor
        stationSetting.rulerType = levelRouteSave.rulerType
        stationSetting.rulerReceptType = levelRouteSave.rulerReceptType

        stationSetting.temperature = levelRouteSave.temperature
        stationSetting.weather = levelRouteSave.weather
        stationSetting.imaging = levelRouteSave.imaging
        stationSetting.sunDirection = levelRouteSave.sunDirection
        stationSetting.windDirection = levelRouteSave.windDirection
        stationSetting.windSpeed = levelRouteSave.windSpeed
        stationSetting.roads = levelRouteSave.roads
        stationSetting.soil = levelRouteSave.soil

        LevelingDbFactory.instance.stationSettingDataSource.save(stationSetting)
    }

    fun getSpinnerText(spinner: Spinner): String {
        val text = spinner.selectedItem.toString()
        if (text.contains(Tools.getString(R.string.please_choose))) {
            return ""
        }
        return text
    }

    @WorkerThread
    fun saveStationSettingLastData(levelRouteSave: LevelRouteSave) {
        LevelStationSettingLastData.setWatcher(levelRouteSave.watcher)
        LevelStationSettingLastData.setRecorder(levelRouteSave.recorder)
        LevelStationSettingLastData.setExaminer(levelRouteSave.examiner)
        LevelStationSettingLastData.setBackFootNumber(levelRouteSave.backFootNumber)
        LevelStationSettingLastData.setFrontFootNumber(levelRouteSave.frontFootNumber)
        LevelStationSettingLastData.setBackFootNumber2(levelRouteSave.backFootNumber2)
        LevelStationSettingLastData.setFrontFootNumber2(levelRouteSave.frontFootNumber2)
        LevelStationSettingLastData.setInstrumentModel(levelRouteSave.instrumentModel)
        LevelStationSettingLastData.setDeviceNumber(levelRouteSave.deviceNumber)
        LevelStationSettingLastData.setSteelRuleParam(levelRouteSave.steelRuleParam)
        LevelStationSettingLastData.setRulerLengthProvingTemperature(levelRouteSave.rulerLengthProvingTemperature)
        LevelStationSettingLastData.setRulerCorrectionFactor(levelRouteSave.rulerCorrectionFactor)
        LevelStationSettingLastData.setRulerType(levelRouteSave.rulerType)
        LevelStationSettingLastData.setRulerReceptType(levelRouteSave.rulerReceptType)

        LevelStationSettingLastData.setTemperature(levelRouteSave.temperature)
        LevelStationSettingLastData.setWeather(levelRouteSave.weather)
        LevelStationSettingLastData.setImaging(levelRouteSave.imaging)
        LevelStationSettingLastData.setSunDirection(levelRouteSave.sunDirection)
        LevelStationSettingLastData.setWindDirection(levelRouteSave.windDirection)
        LevelStationSettingLastData.setWindSpeed(levelRouteSave.windSpeed)
        LevelStationSettingLastData.setRoads(levelRouteSave.roads)
        LevelStationSettingLastData.setSoil(levelRouteSave.soil)
    }

    @WorkerThread
    fun getStationSettingLastData(): LevelRouteSave {
        return LevelRouteSave.createLastData(
            LevelStationSettingLastData.getWatcher(),
            LevelStationSettingLastData.getRecorder(),
            LevelStationSettingLastData.getExaminer(),
            LevelStationSettingLastData.getBackFootNumber(),
            LevelStationSettingLastData.getFrontFootNumber(),
            LevelStationSettingLastData.getBackFootNumber2(),
            LevelStationSettingLastData.getFrontFootNumber2(),
            LevelStationSettingLastData.getInstrumentModel(),
            LevelStationSettingLastData.getDeviceNumber(),
            LevelStationSettingLastData.getSteelRuleParam(),
            LevelStationSettingLastData.getRulerLengthProvingTemperature(),
            LevelStationSettingLastData.getRulerCorrectionFactor(),
            LevelStationSettingLastData.getRulerType(),
            LevelStationSettingLastData.getRulerReceptType(),

            LevelStationSettingLastData.getTemperature(),
            LevelStationSettingLastData.getWeather(),
            LevelStationSettingLastData.getImaging(),
            LevelStationSettingLastData.getSunDirection(),
            LevelStationSettingLastData.getWindDirection(),
            LevelStationSettingLastData.getWindSpeed(),
            LevelStationSettingLastData.getRoads(),
            LevelStationSettingLastData.getSoil()
        )
    }

}