package com.bingce.controlnetwork.func.level.viewmodel

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.bingce.controlapphelper.datasource.database.ProjectTypeValue
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.model.LevelingMeasureOrder
import com.bingce.controlapphelper.widget.treeview.TreeNode
import com.bingce.controlnetwork.func.level.util.LevelStationUtilTj
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 水准选择导出
 */
class LevelSelectExportVm : ViewModel() {

    fun initData(callBack: (TreeNode) -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            val root = TreeNode.root()
            val projectList =
                SurveyorDatabaseFactory.instance.projectDataSource.getListByProjectType(
                    ProjectTypeValue.LEVEL
                )
            for (project in projectList) {
                val projectNode = TreeNode(TreeNode.Value(project.projectName, project.id), 0)
                root.addChild(projectNode)
                val routeList =
                    LevelingDbFactory.instance.levelDataSource.getByProjectIdSync(project.id)
                //当项目里没有路线的时候禁止选中
                if (routeList.isEmpty()) {
                    projectNode.isEnable = false
                }

                for (route in routeList) {

                    val routeNode = TreeNode(TreeNode.Value(route.routeName, route.id), 1)
                    projectNode.addChild(routeNode)

                    val countFinish = LevelStationUtilTj.getStationFinishCount(
                        project.id,
                        route.id
                    )
                    if (countFinish < 1) {
                        routeNode.isEnable = false
                    } else {
                        val isEnable = when (route.levelingMeasureOrder) {
                            LevelingMeasureOrder.A_BF,
                            LevelingMeasureOrder.A_BFFB,
                            LevelingMeasureOrder.A_FBBF,
                            LevelingMeasureOrder.SIM_BF,
                            LevelingMeasureOrder.SIM_BFFB -> {
                                countFinish % 2 == 0
                            }

                            else -> true
                        }
                        routeNode.isEnable = isEnable
                    }
                }
            }

            withContext(Dispatchers.Main) {
                callBack(root)
            }

        }
    }

}