package com.bingce.controlnetwork.func.level.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.bingce.controlnetwork.func.level.util.LevelStationSettingUtil
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory
import com.bingce.controlapphelper.datasource.database.leveling.stationsetting.LevelingStationSettingRecord
import com.bingce.controlapphelper.model.LevelRouteSave
import com.bingce.controlapphelper.model.LevelingMeasureOrder
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class LevelStationSettingVm : ViewModel() {
    private lateinit var routeId: String
    private lateinit var stationId: String

    private lateinit var needSaveLevelRouteSave: LevelRouteSave

    val editLastDataLiveData = MutableLiveData<LevelRouteSave?>()


    fun getEditLastData(stationId: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val record =
                LevelingDbFactory.instance.stationSettingDataSource.getByStationId(stationId)
            if (record != null) {
                editLastDataLiveData.postValue(recordToLevelSave(record))
                return@launch
            }
            val stationSettingLastData = LevelStationSettingUtil.getStationSettingLastData()
            if (stationSettingLastData.weather != null) {
                editLastDataLiveData.postValue(stationSettingLastData)
                return@launch
            }
            editLastDataLiveData.postValue(null)
        }
    }

    fun getInitCurrentTabPos(callBack: (Int) -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            val record =
                LevelingDbFactory.instance.stationSettingDataSource.getLatest(routeId)
            withContext(Dispatchers.Main) {
                val pos = if (record?.weather != null) {
                    1
                } else {
                    0
                }
                callBack(pos)
            }
        }
    }

    fun initRouteStationId(routeId: String, stationId: String) {
        this.routeId = routeId
        this.stationId = stationId
    }

    fun initHasTwoRuler(callBack: (Boolean) -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            val route = LevelingDbFactory.instance.levelDataSource.getByIdSync(routeId)
                ?: return@launch
            val hasTwoRuler =
                route.levelingMeasureOrder == LevelingMeasureOrder.SIM_BF || route.levelingMeasureOrder == LevelingMeasureOrder.SIM_BFFB
            withContext(Dispatchers.Main) {
                callBack(hasTwoRuler)
            }
        }
    }

    private fun recordToLevelSave(record: LevelingStationSettingRecord): LevelRouteSave {
        return LevelRouteSave.createLastData(
            record.watcher,
            record.recorder,
            record.examiner,
            record.firstStationBackFootNumber,
            record.firstStationFrontFootNumber,
            record.firstStationBackFootNumber2,
            record.firstStationFrontFootNumber2,
            record.instrumentModel,
            record.deviceNumber,
            record.steelRuleParam,
            record.rulerLengthProvingTemperature,
            record.rulerCorrectionFactor,
            record.rulerType,
            record.rulerReceptType,

            record.temperature,
            record.weather,
            record.imaging,
            record.sunDirection,
            record.windDirection,
            record.windSpeed,
            record.roads,
            record.soil
        )
    }

    fun createNeedSaveLevelRouteSave(routeId: String, stationId: String) {
        needSaveLevelRouteSave = LevelRouteSave()
        needSaveLevelRouteSave.routeId = routeId
        needSaveLevelRouteSave.stationId = stationId
    }

    fun addInforDataToLevelRouteSave(needAdd: LevelRouteSave) {
        needSaveLevelRouteSave.watcher = needAdd.watcher
        needSaveLevelRouteSave.recorder = needAdd.recorder
        needSaveLevelRouteSave.examiner = needAdd.examiner
        needSaveLevelRouteSave.frontFootNumber = needAdd.frontFootNumber
        needSaveLevelRouteSave.backFootNumber = needAdd.backFootNumber
        needSaveLevelRouteSave.frontFootNumber2 = needAdd.frontFootNumber2
        needSaveLevelRouteSave.backFootNumber2 = needAdd.backFootNumber2
        needSaveLevelRouteSave.instrumentModel = needAdd.instrumentModel
        needSaveLevelRouteSave.deviceNumber = needAdd.deviceNumber
        needSaveLevelRouteSave.steelRuleParam = needAdd.steelRuleParam
        needSaveLevelRouteSave.rulerLengthProvingTemperature = needAdd.rulerLengthProvingTemperature
        needSaveLevelRouteSave.rulerCorrectionFactor = needAdd.rulerCorrectionFactor
        needSaveLevelRouteSave.rulerType = needAdd.rulerType
        needSaveLevelRouteSave.rulerReceptType = needAdd.rulerReceptType
    }

    fun addConditionDataToLevelRouteSave(needAdd: LevelRouteSave) {
        needSaveLevelRouteSave.temperature = needAdd.temperature
        needSaveLevelRouteSave.weather = needAdd.weather
        needSaveLevelRouteSave.imaging = needAdd.imaging
        needSaveLevelRouteSave.sunDirection = needAdd.sunDirection
        needSaveLevelRouteSave.windDirection = needAdd.windDirection
        needSaveLevelRouteSave.windSpeed = needAdd.windSpeed
        needSaveLevelRouteSave.roads = needAdd.roads
        needSaveLevelRouteSave.soil = needAdd.soil
    }

    fun save(isSaveEndStationToStationSetting: Boolean, callBack: () -> Unit) {
        viewModelScope.launch(Dispatchers.IO) {
            LevelStationSettingUtil.saveStationSetting(
                isSaveEndStationToStationSetting,
                needSaveLevelRouteSave
            )

            LevelStationSettingUtil.saveStationSettingLastData(needSaveLevelRouteSave)

            withContext(Dispatchers.Main) {
                callBack()
            }
        }
    }

}