package com.bingce.controlnetwork.http

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

fun <T> CoroutineScope.retrofit(dsl: RetrofitDSL<T>.() -> Unit) {
    launch(Dispatchers.IO) {
        val retrofitDsl = RetrofitDSL<T>()
        retrofitDsl.dsl()
        try {
            val result = retrofitDsl.api() ?: throw NullPointerException()
            retrofitDsl.onSuccess?.invoke(result)
        } catch (e: NullPointerException) {
            retrofitDsl.onFailed?.invoke(e.message, -1)
        } catch (e: Exception) {
            retrofitDsl.onFailed?.invoke(e.message, -100)
        }
    }
}

class RetrofitDSL<T> {
    internal lateinit var api: (suspend () -> T)
        private set
    internal var onSuccess: ((T) -> Unit)? = null
        private set
    internal var onFailed: ((msg: String?, code: Int) -> Unit)? = null
        private set

    /**
     * 获取数据
     * @param block (T) -> Unit
     */
    fun api(block: suspend () -> T) {
        this.api = block
    }

    /**
     * 获取数据成功
     * @param block (T) -> Unit
     */
    fun onSuccess(block: (T) -> Unit) {
        this.onSuccess = block
    }

    /**
     * 获取数据失败
     * @param block (msg: String, errorCode: Int) -> Unit
     */
    fun onFailed(block: (msg: String?, code: Int) -> Unit) {
        this.onFailed = block
    }

}