package com.bingce.controlnetwork.newui.adapter;

import android.content.Context;
import android.view.ContextMenu;
import android.view.MenuItem;
import android.view.ViewGroup;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.datasource.database.ProjectTypeValue;
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.NewItemProjectListBinding;
import com.bingce.controlnetwork.newui.viewholder.ProjectListViewHolder;


public class ProjectListAdapter extends BaseAdapterNew<ProjectListViewHolder, ProjectRecord> {

    public ProjectListAdapter(Context context) {
        super(context);
    }

    @NonNull
    @Override
    public ProjectListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ProjectListViewHolder(NewItemProjectListBinding.inflate(getInflater(), parent, false));
    }

    @Override
    protected void onBindViewHolderDo(ProjectListViewHolder holder, int position) {
        ProjectRecord bean = mDataList.get(position);
        holder.mBinding.tvIndex.setText(String.valueOf(position + 1));
        holder.mBinding.tvProjectName.setText(bean.getProjectName());

        if (bean.projectType == null || ProjectTypeValue.TRIANGLE.equals(bean.projectType)) {
            holder.mBinding.tvProjectType.setText(Tools.getString(R.string.triangular_wire));
        } else if (ProjectTypeValue.CONNECTION.equals(bean.projectType)) {
            holder.mBinding.tvProjectType.setText(Tools.getString(R.string.contact_measurement));
        } else if (ProjectTypeValue.HEIGHT_TRAVERSE.equals(bean.projectType)) {
            holder.mBinding.tvProjectType.setText(Tools.getString(R.string.elevation_conductor));
        } else if (ProjectTypeValue.LEVEL.equals(bean.projectType)) {
            holder.mBinding.tvProjectType.setText(Tools.getString(R.string.level));
        } else if (ProjectTypeValue.GNSS.equals(bean.projectType)) {
            holder.mBinding.tvProjectType.setText(Tools.getString(R.string.gnss));
        } else {
            holder.mBinding.tvProjectType.setText("");
        }

        holder.mBinding.tvTolerance.setText(bean.getToleranceName());
    }

    @Override
    protected void createMenu(ContextMenu menu, ProjectRecord bean) {
        if (adapterContextMenu == null) return;
        menu.clear();
        int menuSize = 2;
        if (ProjectTypeValue.GNSS.equals(bean.projectType)) {
            menuSize = 3;
        }
        for (int index = 0; index < menuSize; index++) {
            menu.add(0, index, 0, adapterContextMenu.menuItemName(index));
            MenuItem menuItem = menu.getItem(index);
            if (menuItem != null) {
                menuItem.setOnMenuItemClickListener(item ->
                        adapterContextMenu.onMenuItemClick(bean, item));
            }
        }
    }
}
