package com.bingce.controlnetwork.newui.base

import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.LinearLayout
import androidx.annotation.StringRes
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.appcompat.widget.Toolbar
import androidx.core.content.ContextCompat
import androidx.viewbinding.ViewBinding
import blankj.utilcode.util.Utils
import com.bingce.controlnetwork.App
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.ActivityBaseBindingBinding

import org.polaric.colorful.Colorful

abstract class BaseBindingActivity<VB : ViewBinding> : AppCompatActivity() {
    protected val mBinding get() = _binding!!
    private var _binding: VB? = null

    private lateinit var toolBar: Toolbar


    abstract fun getBinding(): VB
    abstract fun initView()
    abstract fun initData()

    override fun onCreate(savedInstanceState: Bundle?) {
        initTheme()
        super.onCreate(savedInstanceState)
        setContentView(getContentView())
        initToolBar()
        initView()
        initData()
    }

    private fun initTheme() {
        setTheme(Colorful.getThemeDelegate().styleResBase)
        getTheme().applyStyle(Colorful.getThemeDelegate().styleResPrimary, true)
        getTheme().applyStyle(Colorful.getThemeDelegate().styleResAccent, true)
        val app = Utils.getApp() as App
        if (app.isThemeDark) {
            delegate.localNightMode = AppCompatDelegate.MODE_NIGHT_YES
        } else {
            delegate.localNightMode = AppCompatDelegate.MODE_NIGHT_NO
        }
        window.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
    }

    private fun getContentView(): View {
        val baseBinding = ActivityBaseBindingBinding.inflate(layoutInflater)
        val layoutParams = LinearLayout.LayoutParams(
            ViewGroup.LayoutParams.MATCH_PARENT,
            ViewGroup.LayoutParams.MATCH_PARENT
        )
        _binding = getBinding()
        baseBinding.llParent.addView(mBinding.root, layoutParams)
        baseBinding.llParent.setBackgroundColor(
            ContextCompat.getColor(
                this,
                R.color.bg_content_new
            )
        )

        toolBar = baseBinding.ilToolBar.toolbar
        return baseBinding.root
    }

    protected open fun initToolBar() {
        setSupportActionBar(toolBar)
        if (supportActionBar != null) supportActionBar!!.setDisplayHomeAsUpEnabled(true)
    }

    protected fun setToolbarTitle(text: String?) {
        toolBar.setTitle(text)
    }

    protected fun setToolbarTitle(@StringRes text: Int) {
        toolBar.setTitle(text)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == android.R.id.home) {
            if (isFinishAtOnce())
                finish()
        }
        return super.onOptionsItemSelected(item)
    }

    protected open fun isFinishAtOnce(): Boolean {
        return true
    }

}