package com.bingce.controlnetwork.newui.configfragment

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.LifecycleOwner
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentConfigBinding
import com.bingce.controlnetwork.fragment.PrismManageFragment
import com.bingce.controlnetwork.fragment.SelectExportHeightTraverseFragment
import com.bingce.controlnetwork.fragment.SelectExportSurveyorRecordFragment
import com.bingce.controlnetwork.fragment.SelectExportSurveyorRecordWireFileFragment
import com.bingce.controlnetwork.fragment.ToleranceManageFragment
import com.bingce.controlnetwork.func.level.fragment.LevelSelectExportFragment
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.util.SelectDeviceTypeUtil


class ConfigFragment : BaseFragmentBinding<NewFragmentConfigBinding>() {
    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentConfigBinding {
        return NewFragmentConfigBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        //管理
        mBinding.ilInstrumentManagement.iv.setImageResource(R.mipmap.icon_ts_connecting)
        mBinding.ilInstrumentManagement.tv.setText(R.string.total_station_online)
        mBinding.ilLevelConnect.iv.setImageResource(R.mipmap.ic_level_connect)
        mBinding.ilLevelConnect.tv.text = "水准仪联机"
        mBinding.ilPrismManagement.iv.setImageResource(R.mipmap.icon_prism_manager)
        mBinding.ilPrismManagement.tv.setText(R.string.prism_manager)
        mBinding.ilCustomTolerance.iv.setImageResource(R.mipmap.ic_tolerance_24)
        mBinding.ilCustomTolerance.tv.text = "自定义限差"
        //三角导线联系测量
        mBinding.ilExportMeasure.iv.setImageResource(R.mipmap.ic_export)
        mBinding.ilExportMeasure.tv.setText(R.string.export)
        mBinding.ilExportWire.iv.setImageResource(R.mipmap.ic_export_wire)
        mBinding.ilExportWire.tv.setText(R.string.export_wire_file)
        //水准
        mBinding.ilExportLevel.iv.setImageResource(R.mipmap.ic_export)
        mBinding.ilExportLevel.tv.setText(R.string.level_export)
        //高程导线
        //高程导线数据导出
        mBinding.ilExportHeightTraverse.iv.setImageResource(R.mipmap.ic_export)
        mBinding.ilExportHeightTraverse.tv.setText(R.string.export_height_traverse)

        initListener()
    }

    private fun initListener() {
        mBinding.ilInstrumentManagement.root.setOnClickListener {
            //仪器管理
//            val intent = Intent(activity, ConnectTSActivity::class.java)
//            intent.putExtra("customChannel", AppChannel.customChannel)
//            startActivity(intent)
            SelectDeviceTypeUtil.connectTsActivity()
        }
        mBinding.ilLevelConnect.root.setOnClickListener {
            //水准仪联机
            SelectDeviceTypeUtil.connectLevelActivity()
        }
        mBinding.ilPrismManagement.root.setOnClickListener {
            //棱镜管理
            PrismManageFragment.start(activity)
        }
        mBinding.ilCustomTolerance.root.setOnClickListener {
            //自定义限差
            ToleranceManageFragment.start(activity, null)
        }
        mBinding.ilExportMeasure.root.setOnClickListener {
            //观测数据导出 先选导出类型
            startExportMeasureData(requireContext(), childFragmentManager, viewLifecycleOwner)
        }
        mBinding.ilExportWire.root.setOnClickListener {
            //导线文件导出
            SelectExportSurveyorRecordWireFileFragment.start(context)
        }
        mBinding.ilExportLevel.root.setOnClickListener {
            //水准数据导出
            LevelSelectExportFragment.start(activity)
        }
        mBinding.ilExportHeightTraverse.root.setOnClickListener {
            //高程导线导出
            SelectExportHeightTraverseFragment.start(requireContext())
        }
    }

    override fun initData() {
    }

    /**
     * 测院导出观测前先选择导出文件类型
     *
     * @param context
     * @param fragmentManager
     * @param lifecycleOwner
     */
    private fun startExportMeasureData(
        context: Context, fragmentManager: FragmentManager,
        lifecycleOwner: LifecycleOwner
    ) {
        SelectMeasureDataExportTypeFragment.start(fragmentManager, lifecycleOwner) { beanList ->
            SelectExportSurveyorRecordFragment.start(
                context, bundleOf(SelectExportSurveyorRecordFragment.KEY_DATA to beanList)
            )
        }
    }

}