package com.bingce.controlnetwork.newui.configfragment;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.databinding.ItemSelectMeasureDataExportTypeBinding;
import com.google.android.material.bottomsheet.BottomSheetDialogFragment;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

/**
 * 选择观测数据导出的文件类型
 */
public class SelectMeasureDataExportTypeFragment extends BottomSheetDialogFragment {

    private static final String KEY_REQUEST_SELECT = "__SELECT__";
    private static final String KEY_TYPE = "__SELECT_TYPE__";

    private final List<SelectExportType> dataList = new ArrayList<>();
    private RecyclerView recyclerView;
    private View btnCancel;
    private View btnConfirm;

    @Override
    public void onStart() {
        super.onStart();
//        Dialog dialog = getDialog();
//        if (dialog != null) {
//            Window window = dialog.getWindow();
//            if (window != null) {
//                window.setLayout(
//                        WindowManager.LayoutParams.MATCH_PARENT,
//                        WindowManager.LayoutParams.WRAP_CONTENT);
//            }
//        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_select_measure_data_export_type, container, false);
    }


    public static void start(FragmentManager fragmentManager,
                             @NonNull LifecycleOwner lifecycleOwner,
                             IOnSelectCallback callback) {
        SelectMeasureDataExportTypeFragment fragment = new SelectMeasureDataExportTypeFragment();
        fragmentManager
                .setFragmentResultListener(
                        KEY_REQUEST_SELECT,
                        lifecycleOwner,
                        (requestKey, result) -> {
                            if (KEY_REQUEST_SELECT.equals(requestKey)) {
                                callback.onSurveyorCycleSelected(result.getParcelableArrayList(KEY_TYPE));
                            }
                        });
        fragment.show(fragmentManager, null);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        recyclerView = view.findViewById(R.id.recyclerView);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        initView();
        initData();
        initListener();
    }

    private void initListener() {
        btnCancel.setOnClickListener((v) -> {
            dismissAllowingStateLoss();
        });
        btnConfirm.setOnClickListener((v) -> {
            startConfirm();
        });
    }

    private void startConfirm() {
        List<SelectExportType> exportList = new ArrayList<>();
        for (SelectExportType bean : dataList) {
            if (bean.isSelect()) {
                exportList.add(bean);
            }
        }
        if (exportList.isEmpty()) {
            ToastUtils.showShort("请选择导出格式");
            return;
        }
        backData(exportList);
    }

    private void initView() {

    }

    private void initData() {
        dataList.add(new SelectExportType("控制测量电子手簿格式", SelectExportType.TYPE_CONTROL));
        dataList.add(new SelectExportType("科傻格式", SelectExportType.TYPE_KE_SHA));
        dataList.add(new SelectExportType("南方平差易格式", SelectExportType.TYPE_NAN_FANG));
        dataList.add(new SelectExportType("启飞格式", SelectExportType.TYPE_QI_FEI));
        dataList.add(new SelectExportType("平差之星格式", SelectExportType.TYPE_PING_CHA_ZHI_XING));
        recyclerView.setAdapter(new Adapter());
    }

    private void backData(List<SelectExportType> selectList) {
        Bundle args = new Bundle();
        args.putParcelableArrayList(KEY_TYPE, new ArrayList<>(selectList));
        getParentFragmentManager().setFragmentResult(KEY_REQUEST_SELECT, args);
        dismiss();
    }

    public interface IOnSelectCallback {
        void onSurveyorCycleSelected(ArrayList<SelectExportType> beanList);
    }

    class Adapter extends RecyclerView.Adapter<ViewHolder> {

        @NonNull
        @Override
        public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            return new ViewHolder(ItemSelectMeasureDataExportTypeBinding.inflate(getLayoutInflater(), parent, false));
        }

        @Override
        public void onBindViewHolder(@NonNull ViewHolder holder, int position) {

            SelectExportType bean = dataList.get(position);

            holder.mBinding.tv.setText(bean.getName());
            holder.mBinding.cb.setChecked(bean.isSelect());

            holder.itemView.setOnClickListener(v -> {
                bean.setSelect(!bean.isSelect());
                notifyItemChanged(holder.getBindingAdapterPosition());
            });

        }

        @Override
        public int getItemCount() {
            return dataList.size();
        }
    }

    private static class ViewHolder extends RecyclerView.ViewHolder {
        @NonNull
        private final ItemSelectMeasureDataExportTypeBinding mBinding;

        public ViewHolder(ItemSelectMeasureDataExportTypeBinding binding) {
            super(binding.getRoot());
            this.mBinding = binding;
        }
    }

}