package com.bingce.controlnetwork.newui.mefragment

import android.content.Intent
import android.os.Process
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.ViewGroup
import androidx.fragment.app.activityViewModels
import blankj.utilcode.util.ToastUtils
import blankj.utilcode.util.Utils
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.activity.AboutActivity
import com.bingce.controlnetwork.activity.SettingsActivity
import com.bingce.controlnetwork.activity.ViewRecentCrashActivity
import com.bingce.controlnetwork.activity.backup.BackupExportActivity
import com.bingce.controlnetwork.activity.backup.launch.BackupImportHelper
import com.bingce.controlnetwork.databinding.NewFragmentMeBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.viewmodel.MainActivityViewModel
import com.bumptech.glide.Glide

/**
 * 我的页面
 */
class MeFragment : BaseFragmentBinding<NewFragmentMeBinding>() {
    private val backupImportUtil = BackupImportHelper(this)

    private val mainActivityVm by activityViewModels<MainActivityViewModel>()

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentMeBinding {
        return NewFragmentMeBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        setHasOptionsMenu(true)
        initListener()
        initObserver()
    }

    private fun initObserver() {
        mainActivityVm.refreshLoginUserLiveData.observe(this) {
            Log.d("hwhw", " mainActivityVm.refreshLoginUserLiveData.=" + it.userName)
            mBinding.tvNickName.text = it?.nickName ?: "请登录"
            mBinding.tvPhoneOrEmail.text = it?.userName
            it?.avator?.let { avator ->
                Glide.with(this).load(avator).into(mBinding.ivAvatar)
            }
        }
    }

    override fun initData() {
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.menu_measure_debug, menu)
        super.onCreateOptionsMenu(menu, inflater)
    }

    private fun initListener() {
        mBinding.llSetting.setOnClickListener {
            //设置
            startActivity(Intent(requireContext(), SettingsActivity::class.java))
        }
        mBinding.llDataBackup.setOnClickListener {
            //数据备份
            BackupExportActivity.start(requireContext(), BackupExportActivity::class.java)
        }
        mBinding.llDataImport.setOnClickListener {
            //数据导入
            backupImportUtil.doImport(requireContext()) { success ->
                if (success) {
                    restartApplication()
                }
            }
        }
        mBinding.llAbout.setOnClickListener {
            //关于
            startActivity(Intent(requireContext(), AboutActivity::class.java))
        }
        mBinding.llUser.setOnClickListener {
//            if (LCUser.getCurrentUser() != null) {
//                startActivity(Intent(requireContext(), UserInfoActivity::class.java))
//            } else {
//                val intent = Intent(Utils.getApp(), LoginActivity::class.java)
//                intent.putExtra("finishTo", UserInfoActivity::class.java.name)
//                startActivity(intent)
//            }
        }

        mBinding.llViewException.setOnClickListener {
            //查看异常
            startActivity(Intent(requireContext(), ViewRecentCrashActivity::class.java))
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.measureDebug) {
            Utils.isTest = !Utils.isTest
            if (Utils.isTest) {
                ToastUtils.showShort(R.string.debug_on)
            } else {
                ToastUtils.showShort(R.string.debug_off)
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun restartApplication() {
        Process.killProcess(Process.myPid())
    }

}
