package com.bingce.controlnetwork.newui.projectedit

import android.content.Context
import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import androidx.activity.viewModels
import androidx.fragment.app.Fragment

import com.bingce.controlapphelper.datasource.database.ProjectTypeValue
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewActivityProjectCreateEidtBinding
import com.bingce.controlnetwork.newui.base.BaseBindingActivity

class ProjectCreateEditActivity : BaseBindingActivity<NewActivityProjectCreateEidtBinding>() {
    private val projectCreateEditVm by viewModels<ProjectCreateEditViewModel>()

    private var currentFragment: Fragment? = null

    companion object {
        @JvmStatic
        fun launch(context: Context, projectType: String?, projectId: String? = null) {
            context.startActivity(Intent(context, ProjectCreateEditActivity::class.java).apply {
                putExtra(BundleConstants.KEY_PROJECT_ID, projectId)
                putExtra(BundleConstants.KEY_PROJECT_TYPE, projectType)
            })
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_save, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.menuSave) {
            val iProjectCreateEditOperate = currentFragment as? IProjectCreateEditOperate
            iProjectCreateEditOperate?.save {
                finish()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    override fun getBinding(): NewActivityProjectCreateEidtBinding {
        return NewActivityProjectCreateEidtBinding.inflate(layoutInflater)
    }

    override fun initView() {
        projectCreateEditVm.projectId = getProjectId()
        projectCreateEditVm.projectType = getProjectType()

        startFragment()
    }

    private fun startFragment() {
        when (val projectType = getProjectType()) {
            ProjectTypeValue.CONNECTION -> {
//                currentFragment = ProjectCreateEditConnectionFragment()
            }

            ProjectTypeValue.LEVEL -> {
                currentFragment = ProjectCreateEditLevelFragment()
            }

            ProjectTypeValue.TRIANGLE -> {
//                currentFragment = ProjectCreateEditTriangleFragment()
            }

            else -> {
                if (projectType == null) {
//                    currentFragment = ProjectCreateEditTriangleFragment()
                }
            }
        }

        if (currentFragment != null) {
            supportFragmentManager.beginTransaction()
                .replace(R.id.container, currentFragment!!)
                .commitNow()
        }
    }

    override fun initData() {
    }

    private fun getProjectType() = intent.getStringExtra(BundleConstants.KEY_PROJECT_TYPE)
    private fun getProjectId() = intent.getStringExtra(BundleConstants.KEY_PROJECT_ID)
}