package com.bingce.controlnetwork.newui.projectedit

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.EditText
import androidx.annotation.WorkerThread
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.func.level.constant.LevelConstant

import com.bingce.controlapphelper.datasource.database.ProjectTypeValue
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigOperate
import com.bingce.controlapphelper.datasource.database.project.ProjectRecord
import com.bingce.controlapphelper.datasource.database.tolerance.detail.IToleranceDetail
import com.bingce.controlapphelper.datasource.database.tolerance.load.IToleranceOperate
import com.bingce.controlapphelper.datasource.database.tolerance.load.ToleranceLoadUtil
import com.bingce.controlapphelper.util.CheckLocationUtil
import com.bingce.controlapphelper.util.GetLocationUtil
import com.bingce.controlapphelper.util.ToleranceUtil.completedToleranceName
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentProjectCreateEditLevelBinding
import com.bingce.controlnetwork.fragment.ToleranceSelectFragment
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.util.tolerance.ToleranceOperateUtil.getOperate
import com.bingce.utils.DateUtils
import com.bingce.utils.ThreadPoolUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.Date

/**
 * 新建编辑项目-水准
 */
class ProjectCreateEditLevelFragment :
    BaseFragmentBinding<NewFragmentProjectCreateEditLevelBinding>(), IProjectCreateEditOperate {

    private val projectCreateEditVm by activityViewModels<ProjectCreateEditViewModel>()

    private var toleranceId: String? = null //回显，选择

    private val getLocation by lazy {
        GetLocationUtil()
    }


    private val activityResultLauncher = ToleranceSelectFragment
        .createActivityLauncher(
            this
        ) { id: String ->
            this.dealSelectTolerance(id)
        }


    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentProjectCreateEditLevelBinding {
        return NewFragmentProjectCreateEditLevelBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        lifecycle.addObserver(getLocation)
        initListener()
    }

    private fun initListener() {
        mBinding.llSelectTolerance.setOnClickListener {
            //选择限差方案
            ToleranceSelectFragment.start(
                context,
                toleranceId,
                ProjectTypeValue.LEVEL,
                activityResultLauncher
            )
        }
        mBinding.ivGetLocation.setOnClickListener {
            getLocation {
                setLocation(mBinding.etLongitude, mBinding.etLatitude, it)
            }
        }
    }

    override fun initData() {
        if (projectCreateEditVm.isEdit()) {
            setAlreadyData()
        } else {
            mBinding.etProjectName.setText(DateUtils.ymdWithCharacter(Date()))
            initCreateMeasureUnitData()
            setDefaultLocation()
            val iToleranceDetail = getIToleranceDetail(null)
            updateTolerance(iToleranceDetail)
        }
    }

    private fun initCreateMeasureUnitData() {
        lifecycleScope.launch(Dispatchers.IO) {
            val measureUnit = ProjectLastData.getMeasureUnit()
            withContext(Dispatchers.Main) {
                mBinding.etMeasurementUnit.setText(measureUnit)
            }
        }
    }

    private fun setAlreadyData() {
        ThreadPoolUtil.execute {
            val projectRecord = SurveyorDatabaseFactory.instance
                .getProjectDataSource()
                .getRecord(projectCreateEditVm.projectId) ?: return@execute
            //设置项目名称
            mBinding.etProjectName.setText(projectRecord.getProjectName())
            //设置测量单位
            mBinding.etMeasurementUnit.setText(projectRecord.getMeasureUnit())
            //经纬度
            if (isEmpty(projectRecord.longitude) || isEmpty(projectRecord.latitude)) {
                setDefaultLocation()
            } else {
                setLocation(
                    mBinding.etLongitude,
                    mBinding.etLatitude,
                    projectRecord.longitude,
                    projectRecord.latitude
                )
            }

            //更新限差
            val iToleranceDetail = getIToleranceDetail(projectRecord.toleranceId)
            updateTolerance(iToleranceDetail)
        }
    }

    private fun getIToleranceDetail(toleranceId: String?) = if (toleranceId != null) {
        //编辑情况
        getOperate(toleranceId)?.getIToleranceDetail(toleranceId)
    } else {
        //新建情况走默认的限差
        ToleranceLoadUtil.getToleranceLoad(ProjectTypeValue.LEVEL)
            .getNewDefaultTolerance(ProjectTypeValue.LEVEL)
    }

    override fun save(success: () -> Unit) {
        view ?: return
        //判断项目名称是否为空
        val projectName = mBinding.etProjectName.text.toString().trim()
        if (TextUtils.isEmpty(projectName)) {
            ToastUtils.showShort(R.string.tips_need_select_input_project_name)
            return
        }
        //判断限差参数
        if (TextUtils.isEmpty(toleranceId)) {
            ToastUtils.showShort(R.string.tips_need_select_project_tolerance)
            return
        }
        //经度
        val longitude = mBinding.etLongitude.getText().toString().trim()
        if (TextUtils.isEmpty(longitude)) {
            ToastUtils.showShort(R.string.please_input_longitude)
            return
        }
        //纬度
        val latitude = mBinding.etLatitude.getText().toString().trim()
        if (TextUtils.isEmpty(latitude)) {
            ToastUtils.showShort(R.string.please_input_latitude)
            return
        }

        val measurementUnit = mBinding.etMeasurementUnit.text.toString().trim()

        ThreadPoolUtil.execute {
            if (projectCreateEditVm.isEdit()) {
                val projectRecord =
                    SurveyorDatabaseFactory.instance
                        .getProjectDataSource()
                        .getRecord(projectCreateEditVm.projectId)
                projectRecord.setProjectName(projectName)
                projectRecord.setToleranceId(toleranceId)
                projectRecord.measureUnit = measurementUnit
                projectRecord.longitude = longitude
                projectRecord.latitude = latitude
                //存储
                SurveyorDatabaseFactory.instance
                    .getProjectDataSource()
                    .saveRecordSync(projectRecord)
            } else {
                val projectRecord = ProjectRecord(projectName)
                projectRecord.measureUnit = measurementUnit
                projectRecord.setToleranceId(toleranceId)
                projectRecord.projectType = ProjectTypeValue.LEVEL
                projectRecord.isShowSlopDistance = true
                projectRecord.longitude = longitude
                projectRecord.latitude = latitude
                //存储
                SurveyorDatabaseFactory.instance
                    .getProjectDataSource()
                    .saveRecordSync(projectRecord)

                ConfigOperate.changeDefaultProjectAndCycle(projectRecord.getId(), "")

                ProjectLastData.setMeasureUnit(measurementUnit)
                //经纬度
                ProjectLastData.setLongitude(longitude)
                ProjectLastData.setLatitude(latitude)
            }
        }

        success()
    }

    /**
     * 处理选择返回的限差id
     *
     * @param id
     */
    private fun dealSelectTolerance(id: String) {
        ThreadPoolUtil.execute {
            val operate: IToleranceOperate = getOperate(id)
                ?: return@execute
            val toleranceDetail: IToleranceDetail? = operate.getIToleranceDetail(id)
            //存限差数据
            operate.saveToleranceDetail(toleranceDetail)
            val view: android.view.View? = view
            if (view != null) {
                updateTolerance(toleranceDetail)
            }
        }
    }

    @WorkerThread
    private fun updateTolerance(iToleranceDetail: IToleranceDetail?) {
        if (iToleranceDetail == null) {
            return
        }
        toleranceId = iToleranceDetail.getId()
        val operate = getOperate(toleranceId) ?: return
        val group = operate.getGroup(iToleranceDetail)
        val level = operate.getLevel(iToleranceDetail)
        runOnUI {
            val toleranceShowName =
                completedToleranceName(group, level, iToleranceDetail)
            mBinding.btnSelectTolerance.text = toleranceShowName
        }
    }

    private fun getLocation(resultListener: GetLocationUtil.IResultListener) {
        CheckLocationUtil.check(requireContext()) {
            getLocation.getResult(resultListener)
        }
    }

    private fun setLocation(
        etLongitude: EditText,
        etLatitude: EditText,
        result: GetLocationUtil.Result?
    ) {
        setLocation(etLongitude, etLatitude, result?.longitude, result?.latitude)
    }

    private fun setLocation(
        etLongitude: EditText,
        etLatitude: EditText,
        longitude: String?,
        latitude: String?
    ) {
        etLongitude.setText(longitude)
        etLatitude.setText(latitude)
    }

    private fun setDefaultLocation() {
        lifecycleScope.launch(Dispatchers.IO) {
            val longitude = ProjectLastData.getLongitude()
            val latitude = ProjectLastData.getLatitude()
            withContext(Dispatchers.Main) {
                if (isEmpty(longitude) || isEmpty(latitude)) {
                    setLocation(
                        mBinding.etLongitude,
                        mBinding.etLatitude,
                        LevelConstant.LONGITUDE_DEFAULT.toString(),
                        LevelConstant.LATITUDE_DEFAULT.toString()
                    )
                } else {
                    setLocation(
                        mBinding.etLongitude,
                        mBinding.etLatitude,
                        longitude,
                        latitude
                    )
                }
            }
        }

    }

    private fun isEmpty(value: String?) = TextUtils.isEmpty(value)

}