package com.bingce.controlnetwork.newui.stationlist

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import androidx.lifecycle.LiveData

import com.bingce.controlapphelper.datasource.database.ProjectType.Companion.isConnection
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.adapter.SurveyorStationListAdapter
import com.bingce.controlnetwork.adapter.contextmenu.ContextMenuEdit
import com.bingce.controlnetwork.databinding.FragmentListLayoutBinding
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment
import com.bingce.controlnetwork.fragment.stationdetail.SingleStationDetailAndEditFragment
import com.bingce.controlnetwork.fragment.stationsurvey.StationSurveyorFragment
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureFragment
import com.bingce.controlnetwork.newui.adapter.StationListAdapter
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.newui.model.StationListBean
import com.bingce.controlnetwork.util.SurveyorStationUtil
import com.bingce.utils.ThreadPoolUtil

/**
 * 测站列表仅仅是列表
 */
class StationListFragment : BaseFragmentBinding<FragmentListLayoutBinding>() {

    private val viewModel by viewModels<StationListVm>(ownerProducer = { requireParentFragment() })

    private var stationListLiveData: LiveData<List<SurveyorStationRecord>>? = null

    private var hasSurvey = false


    private val adapter by lazy {
        StationListAdapter(requireContext())
    }

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentListLayoutBinding {
        return FragmentListLayoutBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        initAdapter()
    }

    private fun initAdapter() {
        mBinding.list.adapter = adapter

        if (viewModel.isCanEditDelete) {
            adapter.setAdapterContextMenu(object : ContextMenuEdit<StationListBean>() {
                override fun edit(item: StationListBean) {
                    startEdit(item)
                }
            })
        } else {
            //不能编辑和删除是在测量页面中
            adapter.setOnItemClickListener {
                clickStationItem(it)
            }
        }

    }

    /**
     * 点击测站
     */
    private fun clickStationItem(bean: StationListBean) {
        //StationSurveyorFragment原始测量页
        if (isConnection(bean.projectType)) {
            MeasureFragment.start(context, bean.stationId, bean.stationName)
        } else {
            StationSurveyorFragment.start(context, bean.stationId, bean.stationName)
        }
    }

    private fun startEdit(item: StationListBean) {
        if (SurveyorStationRecord.isLineWell(item.lineType)) {
            TipsWith2OptionFragment.tips(
                childFragmentManager, viewLifecycleOwner,
                getString(R.string.button_edit_station),
                getString(R.string.button_delete_station),
                getString(R.string.tips_select_default_station_operator_well),
                object : TipsWith2OptionFragment.SimpleCallback() {
                    override fun onConfirm() {
                        SurveyorStationUtil.editLine(context, item.lineId)
                    }

                    override fun onCancel() {
                        SurveyorStationListAdapter.try2DeleteLine(
                            item.lineId,
                            context,
                            childFragmentManager,
                            viewLifecycleOwner,
                            this@StationListFragment
                        )
                    }
                })
            return
        }

        //检测是否属于导线测量
        if (SurveyorStationRecord.isLineStation(item.lineId)) {
            TipsWith2OptionFragment.tips(
                childFragmentManager, viewLifecycleOwner,
                getString(R.string.button_edit_line),
                getString(R.string.button_delete_line),
                getString(R.string.tips_forbid_delete_station_because_batch),
                object : TipsWith2OptionFragment.SimpleCallback() {
                    override fun onConfirm() {
                        SurveyorStationListAdapter.try2EditLine(
                            item.lineId,
                            context,
                            childFragmentManager,
                            viewLifecycleOwner,
                            this@StationListFragment
                        )
                    }

                    override fun onCancel() {
                        SurveyorStationListAdapter.try2DeleteLine(
                            item.lineId,
                            context,
                            childFragmentManager,
                            viewLifecycleOwner,
                            this@StationListFragment
                        )
                    }
                })
        } else {
            TipsWith2OptionFragment.tips(
                childFragmentManager, viewLifecycleOwner,
                getString(R.string.button_edit_station),
                getString(R.string.button_delete_station),
                getString(R.string.tips_select_default_station_operator),
                object : TipsWith2OptionFragment.SimpleCallback() {
                    override fun onConfirm() {
                        ThreadPoolUtil.execute {
                            SingleStationDetailAndEditFragment.start(
                                context,
                                item.stationId,
                                item.stationName,
                                viewModel.projectId,
                                item.lineType
                            )
                        }
                    }

                    override fun onCancel() {
                        SurveyorStationListAdapter.try2DeleteStation(
                            item.stationId,
                            context,
                            childFragmentManager,
                            viewLifecycleOwner,
                            this@StationListFragment
                        )
                    }
                })
        }
    }

    override fun initData() {
        refreshUi()
    }

    fun refreshUi() {
        if (stationListLiveData != null) {
            stationListLiveData!!.removeObservers(this)
        }
        stationListLiveData = SurveyorDatabaseFactory.instance.surveyorStation.getLiveDataList(
            viewModel.projectId,
            viewModel.periodId
        )
        if (stationListLiveData != null) {
            stationListLiveData!!.observe(this) {
                viewModel.getStationListData(it) { stationList ->
                    hasSurvey = stationList.isNotEmpty()
                    adapter.refreshData(stationList)
                }
            }
        }
    }

    fun hasSurvey() = hasSurvey
}