package com.bingce.controlnetwork.newui.stationlist

import android.text.TextUtils
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.line.LineRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint
import com.bingce.controlapphelper.util.StationUtil
import com.bingce.controlnetwork.newui.model.StationListBean
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class StationListVm : ViewModel() {

    var isCanEditDelete = false

    var projectId: String? = null
    var periodId: String? = null

    fun getStationListData(
        stationList: List<SurveyorStationRecord>,
        callBack: (List<StationListBean>) -> Unit
    ) {
        viewModelScope.launch(Dispatchers.IO) {

            val showList = mutableListOf<StationListBean>()
            for (station in stationList) {
                val stationName = StationUtil.getStationPointName(station)

                var lineName = ""
                var lineTypeShow = "多测回测角"
                var lineType: String? = null
                station.lineId?.let {
                    val lineRecord = SurveyorDatabaseFactory
                        .instance
                        .lineRecordDatabase
                        .findById(station.lineId)
                    if (lineRecord != null) {
                        lineName = lineRecord.name ?: ""
                        lineTypeShow = LineRecord.lineTypeToName(lineRecord.type)
                        lineType = lineRecord.type
                    }
                }

                val stationTypeSb = StringBuilder()

                stationTypeSb.append(lineTypeShow)
                if (!TextUtils.isEmpty(lineName)) {
                    stationTypeSb.append("-")
                    stationTypeSb.append(lineName)
                }

                val pointOrder = StringBuilder()
                for (point in station.items) {
                    //排除站点
                    if (point.type == SurveyorPoint.TYPE_STATION) {
                        continue
                    }
                    val pointRecord =
                        SurveyorDatabaseFactory.instance.pointDataSource.findByIdSync(point.originalPointId)

                    pointOrder.append(pointRecord?.getName())
                    pointOrder.append("-")
                }

                if (pointOrder.isNotEmpty()) {
                    pointOrder.deleteCharAt(pointOrder.length - 1)
                }

                val stationListBean =
                    StationListBean(
                        station.projectType,
                        station.id,
                        stationName ?: "",
                        stationTypeSb.toString(),
                        pointOrder.toString(),
                        lineType,
                        station.lineId
                    )

                showList.add(stationListBean)

            }

            withContext(Dispatchers.Main) {
                callBack(showList)
            }

        }

    }

    fun setArgument(projectId: String?, periodId: String?) {
        this.projectId = projectId
        this.periodId = periodId
    }


}