package com.bingce.controlnetwork.newui.stationsetting

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.fragment.app.activityViewModels
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentStationSettingEnvironmentBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding

/**
 * 测站设置-环境信息
 */
class StationSettingEnvironmentFragment :
    BaseFragmentBinding<NewFragmentStationSettingEnvironmentBinding>(), IStationSettingOperate {

    private val viewModel by activityViewModels<StationSettingVm>()

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentStationSettingEnvironmentBinding {
        return NewFragmentStationSettingEnvironmentBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        mBinding.ilTemperature.tvTitle.text = "温度（℃）"
        mBinding.ilHumidity.tvTitle.text = "湿度（%）"
        mBinding.ilAirPressure.tvTitle.text = "气压（hPa）"

        initListener()
    }

    private fun initListener() {
        mBinding.tvLastData.setOnClickListener {
            useLastData()
        }
        mBinding.tvStandardData.setOnClickListener {
            useStandardData()
        }
    }

    private fun useLastData() {
        viewModel.getLastEnvironmentData {
            setEnvironmentData()
        }
    }

    private fun useStandardData() {
        viewModel.getDefaultEnvironmentData()
        setEnvironmentData()
    }

    private fun setEnvironmentData() {
        mBinding.ilTemperature.editText.setText(viewModel.environmentData.temperature)
        mBinding.ilHumidity.editText.setText(viewModel.environmentData.humidity)
        mBinding.ilAirPressure.editText.setText(viewModel.environmentData.airPress)
    }


    override fun initData() {
        viewModel.getEnvironmentData {
            setEnvironmentData()
        }
    }

    override fun checkSaveNot(): Boolean {
        val temperature = mBinding.ilTemperature.editText.text
        if (TextUtils.isEmpty(temperature)) {
            ToastUtils.showShort(R.string.temperature_hint)
            return true
        }
        val humidity = mBinding.ilHumidity.editText.text
        if (TextUtils.isEmpty(humidity)) {
            ToastUtils.showShort(R.string.humidity_hint)
            return true
        }
        val airPressure = mBinding.ilAirPressure.editText.text
        if (TextUtils.isEmpty(airPressure)) {
            ToastUtils.showShort(R.string.air_pressure_hint)
            return true
        }
        return false
    }

    private fun getTemperature() = mBinding.ilTemperature.editText.text.toString().trim()
    private fun getHumidity() = mBinding.ilHumidity.editText.text.toString().trim()
    private fun getAirPressure() = mBinding.ilAirPressure.editText.text.toString().trim()

    override fun setDataToStation() {
        viewModel.setEnvironmentDataToStation(
            getTemperature(),
            getHumidity(),
            getAirPressure()
        )
    }

}