package com.bingce.controlnetwork.newui.stationsetting

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.view.ViewGroup.MarginLayoutParams
import android.widget.EditText
import android.widget.TextView
import androidx.fragment.app.activityViewModels
import blankj.utilcode.util.ToastUtils
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentStationSettingHeightBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding

import com.bingce.utils.ScreenUtil

/**
 * 测站设置-仪高镜高
 */
class StationSettingHeightFragment : BaseFragmentBinding<NewFragmentStationSettingHeightBinding>(),
    IStationSettingOperate {

    private val viewModel by activityViewModels<StationSettingVm>()


    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentStationSettingHeightBinding {
        return NewFragmentStationSettingHeightBinding.inflate(inflater, container, false)
    }

    override fun initView() {
        mBinding.ilInstrumentHeight.tvTitle.text = "仪器高（m）"
    }

    override fun initData() {
        //获取点 加载布局
        viewModel.getStationRecord {
            //设置仪器高
            mBinding.ilInstrumentHeight.editText.setText(viewModel.heightData.getInstrumentHeight())

            for (point in viewModel.heightData.pointList) {
                val itemView =
                    layoutInflater.inflate(R.layout.new_item_station_setting_point_height, null)
                val params = MarginLayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ScreenUtil.dp2px(requireContext(), 48f)
                )

                val tvPoint = itemView.findViewById<TextView>(R.id.tvPoint)
                val etHeight = itemView.findViewById<EditText>(R.id.etHeight)

                setItemEtData(
                    tvPoint, etHeight,
                    viewModel.heightData.getPointName(point.originalPointId),
                    viewModel.heightData.getPointHeight(point.originalPointId)
                )

                mBinding.llPoints.addView(itemView, params)
            }
        }
    }

    private fun setItemEtData(
        tvPoint: TextView,
        etHeight: EditText,
        pointName: String?,
        pointHeight: String
    ) {
        tvPoint.text = pointName
        etHeight.setText(pointHeight)
    }


    override fun checkSaveNot(): Boolean {
        val instrumentHeight = getInstrumentHeight()
        if (TextUtils.isEmpty(instrumentHeight)) {
            ToastUtils.showShort(R.string.instrument_height_hint)
            return true
        }
        for (i in 0 until mBinding.llPoints.childCount) {
            val itemView = mBinding.llPoints.getChildAt(i)
            val pointHeight = itemView.findViewById<EditText>(R.id.etHeight).text.toString().trim()
            val pointName = itemView.findViewById<TextView>(R.id.tvPoint).text.toString().trim()
            if (TextUtils.isEmpty(pointHeight)) {
                ToastUtils.showShort("请输入测点" + pointName + "高度")
                return true
            }
        }
        return false
    }

    private fun getInstrumentHeight() = mBinding.ilInstrumentHeight.editText.text.toString().trim()

    override fun setDataToStation() {
        viewModel.setStationHeightToStation(getInstrumentHeight())
        for (i in 0 until mBinding.llPoints.childCount) {
            val itemView = mBinding.llPoints.getChildAt(i)
            val etPointHeight = itemView.findViewById<EditText>(R.id.etHeight)
            val pointHeight = etPointHeight.text.toString().trim()

            viewModel.setPointDataToStation(i, pointHeight)
        }
    }


}