package com.bingce.controlnetwork.newui.stationsetting

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import android.widget.TextView
import androidx.appcompat.widget.SwitchCompat
import androidx.fragment.app.activityViewModels
import com.bingce.controlapphelper.datasource.database.prism.PrismRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint
import com.bingce.controlapphelper.util.StationUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentStationSettingSettingBinding
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import com.bingce.controlnetwork.util.SurveyorOpenAtrUtil
import com.bingce.utils.ScreenUtil

/**
 * 测站设置-测站设置
 */
class StationSettingSettingFragment :
    BaseFragmentBinding<NewFragmentStationSettingSettingBinding>(), IStationSettingOperate {

    private val viewModel by activityViewModels<StationSettingVm>()

    private var isCanSetAloneDistance = false

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentStationSettingSettingBinding {
        return NewFragmentStationSettingSettingBinding.inflate(inflater, container, false)
    }

    override fun initView() {

    }

    override fun initData() {
        viewModel.getStationRecord {
            val pointSize = viewModel.getStationSurveyorPointSize() - 1
            if (StationUtil.isCanLeftOrRightSurveyor(pointSize)) {
                mBinding.scLeftRight.visibility = View.VISIBLE
            } else {
                mBinding.scLeftRight.visibility = View.GONE
            }
            if (StationUtil.isCanZeroSurveyor(pointSize)) {
                mBinding.scZero.visibility = View.VISIBLE
            } else {
                mBinding.scZero.visibility = View.GONE
            }
            mBinding.scLeftRight.isChecked = viewModel.getStationLeftRight()
            mBinding.scZero.isChecked = viewModel.getStationZero()
            //显示斜距还是平距
            mBinding.showSlopSwitch.isChecked = viewModel.getShowSlop()

            setDisEnableByHasData(mBinding.scLeftRight, mBinding.scZero)

            isCanSetAloneDistance = viewModel.isCanShowAloneDistance()

            refreshEveryPointUi()
        }
    }

    private fun refreshEveryPointUi() {
        mBinding.llPoints.removeAllViews()
        for (point in viewModel.settingData.pointList) {
            val itemView =
                layoutInflater.inflate(R.layout.new_item_station_setting_point_setting, null)
            val params = ViewGroup.MarginLayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT
            )

            val tvPoint = itemView.findViewById<TextView>(R.id.tvPoint)
            val scAloneMeasureDistance =
                itemView.findViewById<SwitchCompat>(R.id.scAloneMeasureDistance)
            val scOpenAtr = itemView.findViewById<SwitchCompat>(R.id.scOpenAtr)
            val spinnerPrismType = itemView.findViewById<Spinner>(R.id.spinnerPrismType)

            if (isCanSetAloneDistance) {
                scAloneMeasureDistance.visibility=View.VISIBLE
            }else{
                scAloneMeasureDistance.visibility=View.GONE
            }


            tvPoint.text = viewModel.settingData.getPointName(point.originalPointId)
            //单独测距
            scAloneMeasureDistance.isChecked = point.isAloneDistance

            setDisEnableByHasData(scAloneMeasureDistance)

            setItemSpinnerData(spinnerPrismType, scOpenAtr, point)

            params.topMargin = ScreenUtil.dp2px(requireContext(), 12f)
            mBinding.llPoints.addView(itemView, params)
        }
    }

    override fun checkSaveNot(): Boolean {
        return false
    }

    private fun setItemSpinnerData(
        spinner: Spinner,
        scOpenAtr: SwitchCompat,
        point: SurveyorPoint
    ) {
        spinner.adapter = ArrayAdapter<String>(
            requireContext(),
            android.R.layout.simple_list_item_1,
            viewModel.settingData.getAllPrismNameList()
        )
        spinner.setSelection(viewModel.settingData.getPointPrismSelection(point.originalPointId))
        spinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parent: AdapterView<*>?,
                view: View?,
                position: Int,
                id: Long
            ) {
                val prismRecord = viewModel.settingData.getPrismRecord(position)
                setItemOpenAtr(scOpenAtr, prismRecord, point)
            }

            override fun onNothingSelected(parent: AdapterView<*>?) {
            }

        }
    }

    private fun setItemOpenAtr(
        scOpenAtr: SwitchCompat,
        selectPrismRecord: PrismRecord,
        point: SurveyorPoint
    ) {
        val openAtrEnable = SurveyorOpenAtrUtil.isOpenAtr(
            selectPrismRecord.prismType,
            selectPrismRecord.prismTypeLarge
        )
        val openAtrSurveyorPoint = SurveyorOpenAtrUtil.isOpenAtr(point)
        val openAtr = openAtrEnable && openAtrSurveyorPoint
        scOpenAtr.isChecked = openAtr
        scOpenAtr.isEnabled = openAtrEnable
    }

    override fun setDataToStation() {
        if (mBinding.scLeftRight.visibility == View.VISIBLE) {
            viewModel.setStationLeftRightToStation(mBinding.scLeftRight.isChecked)
        }
        if (mBinding.scZero.visibility == View.VISIBLE) {
            viewModel.setStationZeroToStation(mBinding.scZero.isChecked)
        }
        //显示斜距还是平距
        viewModel.setStationShowSlop(mBinding.showSlopSwitch.isChecked)

        //更新测点
        for (i in 0 until mBinding.llPoints.childCount) {
            val itemView = mBinding.llPoints.getChildAt(i)
            val scAloneMeasureDistance =
                itemView.findViewById<SwitchCompat>(R.id.scAloneMeasureDistance)
            val spinnerPrismType = itemView.findViewById<Spinner>(R.id.spinnerPrismType)
            val scOpenAtr = itemView.findViewById<SwitchCompat>(R.id.scOpenAtr)

            viewModel.setPointSettingToStation(
                i,
                scAloneMeasureDistance.isChecked,
                spinnerPrismType.selectedItemPosition,
                !scOpenAtr.isChecked
            )
        }

        //更新测站的单独测距状态
//        viewModel.updateSeparetionToStation()

    }

    private fun setDisEnableByHasData(vararg ets: View) {
        if (viewModel.hasSurveyorData) {
            for (item in ets) {
                item.isEnabled = false
            }
        }
    }

}