package com.bingce.controlnetwork.newui.stationsetting

import android.content.Context
import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import androidx.activity.viewModels
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentStatePagerAdapter
import androidx.viewpager.widget.ViewPager.OnPageChangeListener
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.FastClickUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewActivityStationSettingBinding
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import com.bingce.controlnetwork.util.KeyboardUtil

class StationSettingsActivity : BaseBindingActivity<NewActivityStationSettingBinding>() {

    private val viewModel by viewModels<StationSettingVm>()

    private val fragments = mutableListOf<Fragment>()
    private val fragmentTitleList = listOf("仪高镜高", "环境信息", "测站设置")

    companion object {
        @JvmStatic
        fun start(context: Context, stationId: String?) {
            if (stationId == null) {
                return
            }
            context.startActivity(Intent(context, StationSettingsActivity::class.java).apply {
                putExtra(BundleConstants.KEY_SURVEYOR_STATION_ID, stationId)
            })
        }
    }

    override fun getBinding(): NewActivityStationSettingBinding {
        return NewActivityStationSettingBinding.inflate(layoutInflater)
    }

    override fun initView() {
        viewModel.initStationId(getStationId()) {
            initViewPager()
        }
    }

    override fun initData() {

    }

    private fun initViewPager() {
        fragments.add(StationSettingHeightFragment())
        fragments.add(StationSettingEnvironmentFragment())
        fragments.add(StationSettingSettingFragment())

        mBinding.vp.offscreenPageLimit = 3
        mBinding.vp.adapter = MyPagerAdapter(supportFragmentManager)
        mBinding.tabLayout.setViewPager(mBinding.vp)
        mBinding.vp.addOnPageChangeListener(object : OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {

            }

            override fun onPageScrollStateChanged(state: Int) {

            }
        })

//        //设置当前tab为仪高镜高
//        mBinding.vp.currentItem = 1
    }


    private inner class MyPagerAdapter(fm: FragmentManager) : FragmentStatePagerAdapter(fm) {
        override fun getCount(): Int {
            return fragments.size
        }

        override fun getPageTitle(position: Int): CharSequence {
            return fragmentTitleList[position]
        }

        override fun getItem(position: Int): Fragment {
            return fragments[position]
        }

        override fun getItemPosition(`object`: Any): Int {
            return POSITION_NONE
        }
    }

    private fun getStationId(): String {
        return intent.getStringExtra(BundleConstants.KEY_SURVEYOR_STATION_ID)!!
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_save, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.menuSave) {
            FastClickUtil.click {
                KeyboardUtil.hideSoftInput()
                startSave()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun startSave() {
        //检查未完成的选项
        for (fragment in fragments) {
            val operate = fragment as IStationSettingOperate
            if (operate.checkSaveNot()) {
                val fragmentNotFinishPos = fragments.indexOf(fragment)
                if (mBinding.vp.currentItem != fragmentNotFinishPos) {
                    mBinding.tabLayout.currentTab = fragmentNotFinishPos
                }
                return
            }
        }
        //赋值
        for (fragment in fragments) {
            val operate = fragment as IStationSettingOperate
            operate.setDataToStation()
        }

        viewModel.saveToDataBase {
//            EventBus.getDefault().post(ReplaceSurveyorFragmentEvent())
            ToastUtils.showShort(R.string.save_success)
        }

    }

    override fun onBackPressed() {
        if (isFinishAtOnce()) {
            super.onBackPressed()
        }
    }

}