package com.bingce.controlnetwork.newui.stationsetting.model

import androidx.annotation.WorkerThread
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint


/**
 * 高度tab相关数据
 */
class StationSettingHeightData {

    private var instrumentHeight: String? = null
    val pointList = mutableListOf<SurveyorPoint>()

    private val pointNameMap = hashMapOf<String, String>()
    private val pointHeightMap = hashMapOf<String, String?>()

    @WorkerThread
    fun initData(stationRecord: SurveyorStationRecord?) {
        if (stationRecord?.items == null) return
        instrumentHeight = stationRecord.instrumentHeight

        for (point in stationRecord.items) {
            if (point.type == SurveyorPoint.TYPE_STATION || point.isWellSteel) {
                //忽略站点和钢丝点
                continue
            }
            pointList.add(point)
            val pointRecord =
                SurveyorDatabaseFactory.instance.getPointDataSource().findByIdSync(point.originalPointId)
            pointNameMap[point.originalPointId] = pointRecord.name
            pointHeightMap[point.originalPointId] = point.prismHeight
        }
    }

    fun getPointName(pointId: String) = pointNameMap[pointId]
    fun getPointHeight(pointId: String): String {
        return pointHeightMap[pointId] ?: "0"
    }

    fun getInstrumentHeight(): String {
        return instrumentHeight ?: "0"
    }

    /**
     * 保存时
     */
    fun updatePointData(pointIndex: Int, pointHeight: String) {
        val surveyorPoint = pointList[pointIndex]
        //点高
        surveyorPoint.prismHeight = pointHeight
    }

}


