package com.bingce.controlnetwork.newui.stationsetting.model

import androidx.annotation.WorkerThread
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.prism.PrismRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint


class StationSettingSettingData {

    val pointList = mutableListOf<SurveyorPoint>()
    private val pointNameMap = hashMapOf<String, String>()

    private val pointPrismMap = hashMapOf<String, PrismRecord>()
    private lateinit var prismAllList: List<PrismRecord>
    private val prismAllTypeNameList = mutableListOf<String>()


    @WorkerThread
    fun initData(stationRecord: SurveyorStationRecord?) {
        if (stationRecord?.items == null) return

        prismAllList = SurveyorDatabaseFactory.instance.getPrismManagerDataSource().allRecordSync
        for (prism in prismAllList) {
            prismAllTypeNameList.add(prism.prismTypeName)
        }

        for (point in stationRecord.items) {
            if (point.type == SurveyorPoint.TYPE_STATION || point.isWellSteel) {
                //忽略站点
                continue
            }
            pointList.add(point)
            val pointRecord =
                SurveyorDatabaseFactory.instance.getPointDataSource()
                    .findByIdSync(point.originalPointId)
            pointNameMap[point.originalPointId] = pointRecord.name

            pointPrismMap[point.originalPointId] = PrismRecord().apply {
                prismType = point.prismType
                prismTypeName = point.prismTypeName
                prismTypeConstant = point.prismTypeConstant
            }
        }
    }

    fun getPointName(pointId: String) = pointNameMap[pointId]

    fun getPointPrismSelection(pointId: String): Int {
        val prismRecord = pointPrismMap[pointId]
        for ((index, record) in prismAllList.withIndex()) {
            if (record.prismTypeName.equals(prismRecord?.prismTypeName)) {
                return index
            }
        }
        return -1
    }

    fun getAllPrismNameList(): List<String> {
        return prismAllTypeNameList
    }

    /**
     * 保存时
     */
    fun updatePointSettingData(
        pointIndex: Int,
        aloneMeasureDistance: Boolean,
        selectionPosPrismType: Int,
        isOpenAtrNot: Boolean
    ) {
        val surveyorPoint = pointList[pointIndex]
        surveyorPoint.isAloneDistance = aloneMeasureDistance
        //棱镜类型相关
        val prismRecord = getPrismRecord(selectionPosPrismType)
        surveyorPoint.prismType = prismRecord.prismType
        surveyorPoint.prismTypeLarge = prismRecord.prismTypeLarge
        surveyorPoint.prismTypeConstant = prismRecord.prismTypeConstant
        surveyorPoint.prismTypeName = prismRecord.prismTypeName
        //openAtr
        surveyorPoint.isOpenAtrNot = isOpenAtrNot
    }

    fun getPrismRecord(selectionPosPrismType: Int) = prismAllList[selectionPosPrismType]


}