package com.bingce.controlnetwork.newui.stationsettingheighttraverse

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.EditText
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.heighttraverse.stationsetting.model.StationSettingHeightTraverseRecord
import com.bingce.controlapphelper.util.Tools
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentStationSettingEnvironmentHeightTraverseBinding
import com.bingce.controlnetwork.fragment.stationsetting.BaseStationSettingFragment.Companion.DEFAULT_AIR_PRESSURE
import com.bingce.controlnetwork.fragment.stationsetting.BaseStationSettingFragment.Companion.DEFAULT_CONSTANT
import com.bingce.controlnetwork.fragment.stationsetting.BaseStationSettingFragment.Companion.DEFAULT_TEMPERATURE
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 测站设置-环境信息
 */
class StationSettingEnvironmentHeightTraverseFragment :
    BaseFragmentBinding<NewFragmentStationSettingEnvironmentHeightTraverseBinding>(),
    IStationSettingOperate {

    private val viewModel by activityViewModels<StationSettingHeightTraverseVm>()

    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentStationSettingEnvironmentHeightTraverseBinding {
        return NewFragmentStationSettingEnvironmentHeightTraverseBinding.inflate(
            inflater,
            container,
            false
        )
    }

    override fun initView() {
        mBinding.ilDryTemperature.tvName.text = getString(R.string.dry_temperature)
        mBinding.ilDryTemperature.etValue.hint = getString(R.string.dry_temperature)
        mBinding.ilDryTemperature.tvUnit.text = "℃"

        mBinding.ilHumidityTemperature.tvName.text = getString(R.string.humidity_temperature)
        mBinding.ilHumidityTemperature.etValue.hint = getString(R.string.humidity_temperature)
        mBinding.ilHumidityTemperature.tvUnit.text = "℃"

        mBinding.ilAirPressure.tvName.text = getString(R.string.hint_air_pressure)
        mBinding.ilAirPressure.etValue.hint = getString(R.string.hint_air_pressure)
        mBinding.ilAirPressure.tvUnit.text = "hPa"

        mBinding.ilAdditionConstant.tvName.text = getString(R.string.addition_constant)
        mBinding.ilAdditionConstant.etValue.hint = getString(R.string.addition_constant)
        mBinding.ilAdditionConstant.tvUnit.text = "mm"

        mBinding.ilmultiplicationConstant.tvName.text = getString(R.string.multiplication_constant)
        mBinding.ilmultiplicationConstant.etValue.hint = getString(R.string.multiplication_constant)
        mBinding.ilmultiplicationConstant.tvUnit.text = "ppm"

        initListener()
    }

    private fun initListener() {
        mBinding.tvLastData.setOnClickListener {
            useLastData()
        }
        mBinding.tvStandardData.setOnClickListener {
            setDataFromRecord(null)
        }
    }

    private fun useLastData() {
        lifecycleScope.launch {
            val record = withContext(Dispatchers.IO) {
                val dryTemperature = getValueFromLastKey(ConfigConstants.KEY_LAST_DRY_TEMPERATURE)
                val humidityTemperature =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_HUMIDITY_TEMPERATURE)
                val airPressure = getValueFromLastKey(ConfigConstants.KEY_LAST_AIR_PRESSURE)
                val additionConstant =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_ADDITION_CONSTANT)
                val multiplicationConstant =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_MULTIPLICATION_CONSTANT)
                val instrumentHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_BEFORE)
                val instrumentHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_AFTER)
                val backPrismHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_BEFORE)
                val backPrismHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_AFTER)
                val frontPrismHeightBefore =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_BEFORE)
                val frontPrismHeightAfter =
                    getValueFromLastKey(ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_AFTER)

                if (dryTemperature == null
                    || humidityTemperature == null
                    || airPressure == null
                    || additionConstant == null
                    || multiplicationConstant == null
                    || instrumentHeightBefore == null
                    || instrumentHeightAfter == null
                    || backPrismHeightBefore == null
                    || backPrismHeightAfter == null
                    || frontPrismHeightBefore == null
                    || frontPrismHeightAfter == null
                ) {
                    return@withContext null
                }

                StationSettingHeightTraverseRecord.newInstance(
                    viewModel.stationId,
                    dryTemperature,
                    humidityTemperature,
                    airPressure,
                    additionConstant,
                    multiplicationConstant,
                    instrumentHeightBefore,
                    instrumentHeightAfter,
                    backPrismHeightBefore,
                    backPrismHeightAfter,
                    frontPrismHeightBefore,
                    frontPrismHeightAfter
                )
            }

            setDataFromRecord(record)
        }
    }

    private fun getValueFromLastKey(key: String): String? {
        val record =
            SurveyorDatabaseFactory.instance.configDataSource.getByKeySync(key)
        return record?.configValue
    }

    override fun initData() {
        lifecycleScope.launch {
            val record = withContext(Dispatchers.IO) {
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .getRecordByStationId(viewModel.stationId)
            }
            setDataFromRecord(record)
        }
    }

    private fun setDataFromRecord(record: StationSettingHeightTraverseRecord?) {
        mBinding.ilDryTemperature.etValue.setText(
            record?.dryTemperature ?: DEFAULT_TEMPERATURE
        )
        mBinding.ilHumidityTemperature.etValue.setText(
            record?.humidityTemperature ?: DEFAULT_TEMPERATURE
        )
        mBinding.ilAirPressure.etValue.setText(record?.airPressure ?: DEFAULT_AIR_PRESSURE)
        mBinding.ilAdditionConstant.etValue.setText(
            record?.additionConstant ?: DEFAULT_CONSTANT
        )
        mBinding.ilmultiplicationConstant.etValue.setText(
            record?.multiplicationConstant ?: DEFAULT_CONSTANT
        )
    }

    override fun checkSaveNot(): Boolean {
        if (editTextEmpty(mBinding.ilDryTemperature.etValue)) {
            ToastUtils.showShort(R.string.please_enter_dry_temperature)
            return true
        }
        if (editTextEmpty(mBinding.ilHumidityTemperature.etValue)) {
            ToastUtils.showShort(R.string.please_enter_humidity_temperature)
            return true
        }
        if (editTextEmpty(mBinding.ilAirPressure.etValue)) {
            ToastUtils.showShort(Tools.getString(R.string.please_enter_air_pressure))
            return true
        }
        if (editTextEmpty(mBinding.ilAdditionConstant.etValue)) {
            ToastUtils.showShort(R.string.please_enter_addition_constant)
            return true
        }
        if (editTextEmpty(mBinding.ilmultiplicationConstant.etValue)) {
            ToastUtils.showShort(R.string.please_enter_multiplication_constant)
            return true
        }
        setDataToViewModel()
        return false
    }

    private fun setDataToViewModel() {
        val dryTemperature = getEtText(mBinding.ilDryTemperature.etValue)
        val humidityTemperature = getEtText(mBinding.ilHumidityTemperature.etValue)
        val airPressure = getEtText(mBinding.ilAirPressure.etValue)
        val additionConstant = getEtText(mBinding.ilAdditionConstant.etValue)
        val multiplicationConstant = getEtText(mBinding.ilmultiplicationConstant.etValue)

        viewModel.dryTemperature = dryTemperature
        viewModel.humidityTemperature = humidityTemperature
        viewModel.airPressure = airPressure
        viewModel.additionConstant = additionConstant
        viewModel.multiplicationConstant = multiplicationConstant
    }

    private fun editTextEmpty(et: EditText): Boolean {
        return TextUtils.isEmpty(getEtText(et))
    }

    private fun getEtText(et: EditText) = et.text.toString().trim()

}