package com.bingce.controlnetwork.newui.stationsettingheighttraverse

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.EditText
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.lifecycleScope
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.heighttraverse.stationsetting.model.StationSettingHeightTraverseRecord
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentStationSettingHeightHeightTraverseBinding
import com.bingce.controlnetwork.fragment.stationsetting.BaseStationSettingFragment.Companion.DEFAULT_INSTRUMEN_HEIGHT
import com.bingce.controlnetwork.newui.base.BaseFragmentBinding
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

/**
 * 高程导线-仪高镜高
 */
class StationSettingHeightHeightTraverseFragment :
    BaseFragmentBinding<NewFragmentStationSettingHeightHeightTraverseBinding>(),
    IStationSettingOperate {

    private val viewModel by activityViewModels<StationSettingHeightTraverseVm>()


    override fun getViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): NewFragmentStationSettingHeightHeightTraverseBinding {
        return NewFragmentStationSettingHeightHeightTraverseBinding.inflate(
            inflater,
            container,
            false
        )
    }

    override fun initView() {
        mBinding.ilInstrumentHeight.tvName.text = getString(R.string.instrument_height)
        mBinding.ilBackPrismHeight.tvName.text = getString(R.string.back_prism_height)
        mBinding.ilFrontPrismHeight.tvName.text = getString(R.string.front_prism_height)

        //仪器高，棱镜高
        mBinding.ilInstrumentHeight.button1.setOnClickListener {
            mBinding.ilInstrumentHeight.etValueAfter.setText(
                getEtText(mBinding.ilInstrumentHeight.etValueBefore)
            )
        }
        mBinding.ilBackPrismHeight.button1.setOnClickListener {
            mBinding.ilBackPrismHeight.etValueAfter.setText(
                getEtText(mBinding.ilBackPrismHeight.etValueBefore)
            )
        }
        mBinding.ilFrontPrismHeight.button1.setOnClickListener {
            mBinding.ilFrontPrismHeight.etValueAfter.setText(
                getEtText(mBinding.ilFrontPrismHeight.etValueBefore)
            )
        }

    }

    private fun getEtText(et: EditText) = et.text.toString().trim()


    override fun initData() {
        lifecycleScope.launch {
            val record = withContext(Dispatchers.IO) {
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .getRecordByStationId(viewModel.stationId)
            }
            setDataFromRecord(record)
        }
    }

    private fun setDataFromRecord(record: StationSettingHeightTraverseRecord?) {
        mBinding.ilInstrumentHeight.etValueBefore.setText(
            record?.instrumentHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilInstrumentHeight.etValueAfter.setText(
            record?.instrumentHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilBackPrismHeight.etValueBefore.setText(
            record?.backPrismHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilBackPrismHeight.etValueAfter.setText(
            record?.backPrismHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilFrontPrismHeight.etValueBefore.setText(
            record?.frontPrismHeightBefore ?: DEFAULT_INSTRUMEN_HEIGHT
        )
        mBinding.ilFrontPrismHeight.etValueAfter.setText(
            record?.frontPrismHeightAfter ?: DEFAULT_INSTRUMEN_HEIGHT
        )
    }

    override fun checkSaveNot(): Boolean {
        if (editTextEmpty(mBinding.ilInstrumentHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_instrument_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilInstrumentHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_instrument_height_after)
            return true
        }
        if (editTextEmpty(mBinding.ilBackPrismHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_back_prism_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilBackPrismHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_back_prism_height_after)
            return true
        }
        if (editTextEmpty(mBinding.ilFrontPrismHeight.etValueBefore)) {
            ToastUtils.showShort(R.string.please_enter_front_prism_height_before)
            return true
        }
        if (editTextEmpty(mBinding.ilFrontPrismHeight.etValueAfter)) {
            ToastUtils.showShort(R.string.please_enter_front_prism_height_after)
            return true
        }

        setDataToViewModel()

        return false
    }

    private fun setDataToViewModel() {
        val instrumentHeightBefore = getEtText(mBinding.ilInstrumentHeight.etValueBefore)
        val instrumentHeightAfter = getEtText(mBinding.ilInstrumentHeight.etValueAfter)
        val backPrismHeightBefore = getEtText(mBinding.ilBackPrismHeight.etValueBefore)
        val backPrismHeightAfter = getEtText(mBinding.ilBackPrismHeight.etValueAfter)
        val frontPrismHeightBefore = getEtText(mBinding.ilFrontPrismHeight.etValueBefore)
        val frontPrismHeightAfter = getEtText(mBinding.ilFrontPrismHeight.etValueAfter)

        viewModel.instrumentHeightBefore = instrumentHeightBefore
        viewModel.instrumentHeightAfter = instrumentHeightAfter
        viewModel.backPrismHeightBefore = backPrismHeightBefore
        viewModel.backPrismHeightAfter = backPrismHeightAfter
        viewModel.frontPrismHeightBefore = frontPrismHeightBefore
        viewModel.frontPrismHeightAfter = frontPrismHeightAfter
    }

    private fun editTextEmpty(et: EditText): Boolean {
        return TextUtils.isEmpty(getEtText(et))
    }


}