package com.bingce.controlnetwork.newui.stationsettingheighttraverse

import android.content.Context
import android.content.Intent
import android.view.Menu
import android.view.MenuItem
import androidx.activity.viewModels
import androidx.annotation.WorkerThread
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentStatePagerAdapter
import androidx.lifecycle.lifecycleScope
import androidx.viewpager.widget.ViewPager.OnPageChangeListener
import blankj.utilcode.util.ToastUtils
import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord
import com.bingce.controlapphelper.datasource.database.heighttraverse.stationsetting.model.StationSettingHeightTraverseRecord
import com.bingce.controlapphelper.model.BundleConstants
import com.bingce.controlapphelper.util.FastClickUtil
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewActivityStationSettingBinding
import com.bingce.controlnetwork.newui.base.BaseBindingActivity
import com.bingce.controlnetwork.util.KeyboardUtil
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class StationSettingsHeightTraverseActivity :
    BaseBindingActivity<NewActivityStationSettingBinding>() {

    private val viewModel by viewModels<StationSettingHeightTraverseVm>()

    private val fragments = mutableListOf<Fragment>()
    private val fragmentTitleList = listOf("仪高镜高", "环境信息")

    companion object {
        @JvmStatic
        fun start(context: Context, stationId: String?) {
            if (stationId == null) {
                return
            }
            context.startActivity(
                Intent(
                    context,
                    StationSettingsHeightTraverseActivity::class.java
                ).apply {
                    putExtra(BundleConstants.KEY_SURVEYOR_STATION_ID, stationId)
                })
        }
    }

    override fun getBinding(): NewActivityStationSettingBinding {
        return NewActivityStationSettingBinding.inflate(layoutInflater)
    }

    override fun initView() {
        viewModel.stationId = getStationId()
        initViewPager()
    }

    override fun initData() {

    }

    private fun initViewPager() {
        fragments.add(StationSettingHeightHeightTraverseFragment())
        fragments.add(StationSettingEnvironmentHeightTraverseFragment())

        mBinding.vp.offscreenPageLimit = 3
        mBinding.vp.adapter = MyPagerAdapter(supportFragmentManager)
        mBinding.tabLayout.setViewPager(mBinding.vp)
        mBinding.vp.addOnPageChangeListener(object : OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {

            }

            override fun onPageScrollStateChanged(state: Int) {

            }
        })

//        //设置当前tab为仪高镜高
//        mBinding.vp.currentItem = 1
    }


    private inner class MyPagerAdapter(fm: FragmentManager) : FragmentStatePagerAdapter(fm) {
        override fun getCount(): Int {
            return fragments.size
        }

        override fun getPageTitle(position: Int): CharSequence {
            return fragmentTitleList[position]
        }

        override fun getItem(position: Int): Fragment {
            return fragments[position]
        }

        override fun getItemPosition(`object`: Any): Int {
            return POSITION_NONE
        }
    }

    private fun getStationId(): String {
        return intent.getStringExtra(BundleConstants.KEY_SURVEYOR_STATION_ID)!!
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.menu_save, menu)
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.menuSave) {
            FastClickUtil.click {
                KeyboardUtil.hideSoftInput()
                startSave()
            }
        }
        return super.onOptionsItemSelected(item)
    }

    private fun startSave() {
        //检查未完成的选项
        for (fragment in fragments) {
            val operate = fragment as IStationSettingOperate
            if (operate.checkSaveNot()) {
                val fragmentNotFinishPos = fragments.indexOf(fragment)
                if (mBinding.vp.currentItem != fragmentNotFinishPos) {
                    mBinding.tabLayout.currentTab = fragmentNotFinishPos
                }
                return
            }
        }
        //保存到数据库
        saveData()
    }

    private fun saveData() {
        lifecycleScope.launch(Dispatchers.IO) {
            val stationId = getStationId()

            val stationSettingHeightTraverseRecord =
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .getRecordByStationId(stationId)

            if (stationSettingHeightTraverseRecord == null) {
                val stationSettingHeightTraverseRecordNew =
                    StationSettingHeightTraverseRecord.newInstance(
                        stationId,
                        viewModel.dryTemperature!!,
                        viewModel.humidityTemperature!!,
                        viewModel.airPressure!!,
                        viewModel.additionConstant!!,
                        viewModel.multiplicationConstant!!,
                        viewModel.instrumentHeightBefore!!,
                        viewModel.instrumentHeightAfter!!,
                        viewModel.backPrismHeightBefore!!,
                        viewModel.backPrismHeightAfter!!,
                        viewModel.frontPrismHeightBefore!!,
                        viewModel.frontPrismHeightAfter!!
                    )
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .saveRecord(stationSettingHeightTraverseRecordNew)
            } else {
                stationSettingHeightTraverseRecord.dryTemperature = viewModel.dryTemperature!!
                stationSettingHeightTraverseRecord.humidityTemperature =
                    viewModel.humidityTemperature!!
                stationSettingHeightTraverseRecord.airPressure = viewModel.airPressure!!
                stationSettingHeightTraverseRecord.additionConstant = viewModel.additionConstant!!
                stationSettingHeightTraverseRecord.multiplicationConstant =
                    viewModel.multiplicationConstant!!
                stationSettingHeightTraverseRecord.instrumentHeightBefore =
                    viewModel.instrumentHeightBefore!!
                stationSettingHeightTraverseRecord.instrumentHeightAfter =
                    viewModel.instrumentHeightAfter!!
                stationSettingHeightTraverseRecord.backPrismHeightBefore =
                    viewModel.backPrismHeightBefore!!
                stationSettingHeightTraverseRecord.backPrismHeightAfter =
                    viewModel.backPrismHeightAfter!!
                stationSettingHeightTraverseRecord.frontPrismHeightBefore =
                    viewModel.frontPrismHeightBefore!!
                stationSettingHeightTraverseRecord.frontPrismHeightAfter =
                    viewModel.frontPrismHeightAfter!!
                SurveyorDatabaseFactory.instance.getStationSettingHeightTraverseDataSource()
                    .updateRecord(stationSettingHeightTraverseRecord)
            }

            val stationRecord =
                SurveyorDatabaseFactory.instance.surveyorStation.getRecordSync(stationId)
            if (stationRecord != null) {
                stationRecord.instrumentHeight = viewModel.instrumentHeightBefore
                if (!stationRecord.items.isNullOrEmpty()) {
                    stationRecord.items.last().prismHeight = viewModel.frontPrismHeightBefore
                    if (stationRecord.items.size > 2) {
                        stationRecord.items[1].prismHeight = viewModel.backPrismHeightBefore
                    }
                }
            }
            SurveyorDatabaseFactory.instance.surveyorStation.update(stationRecord)

            saveLastData(
                viewModel.dryTemperature!!,
                viewModel.humidityTemperature!!,
                viewModel.airPressure!!,
                viewModel.additionConstant!!,
                viewModel.multiplicationConstant!!,
                viewModel.instrumentHeightBefore!!,
                viewModel.instrumentHeightAfter!!,
                viewModel.backPrismHeightBefore!!,
                viewModel.backPrismHeightAfter!!,
                viewModel.frontPrismHeightBefore!!,
                viewModel.frontPrismHeightAfter!!
            )

            ToastUtils.showShort(R.string.save_success)

        }
    }

    @WorkerThread
    private fun saveLastData(
        dryTemperature: String,
        humidityTemperature: String,
        airPressure: String,
        additionConstant: String,
        multiplicationConstant: String,
        instrumentHeightBefore: String,
        instrumentHeightAfter: String,
        backPrismHeightBefore: String,
        backPrismHeightAfter: String,
        frontPrismHeightBefore: String,
        frontPrismHeightAfter: String
    ) {
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_DRY_TEMPERATURE,
                dryTemperature
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_HUMIDITY_TEMPERATURE,
                humidityTemperature
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_AIR_PRESSURE,
                airPressure
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_ADDITION_CONSTANT,
                additionConstant
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_MULTIPLICATION_CONSTANT,
                multiplicationConstant
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_BEFORE,
                instrumentHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_INSTRUMENT_HEIGHT_AFTER,
                instrumentHeightAfter
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_BEFORE,
                backPrismHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_BACK_PRISM_HEIGHT_AFTER,
                backPrismHeightAfter
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_BEFORE,
                frontPrismHeightBefore
            )
        )
        SurveyorDatabaseFactory.instance.configDataSource.recordSync(
            ConfigRecord(
                ConfigConstants.KEY_LAST_FRONT_PRISM_HEIGHT_AFTER,
                frontPrismHeightAfter
            )
        )
    }

    override fun onBackPressed() {
        if (isFinishAtOnce()) {
            super.onBackPressed()
        }
    }

}