package com.bingce.controlnetwork.newui.widget.selectitem

import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.ViewGroup.MarginLayoutParams
import androidx.fragment.app.DialogFragment
import androidx.fragment.app.FragmentManager
import androidx.lifecycle.LifecycleOwner
import androidx.recyclerview.widget.RecyclerView

import com.bingce.controlapphelper.model.IUniversalListItem
import com.bingce.controlnetwork.R
import com.bingce.controlnetwork.databinding.NewFragmentBaseSelectItemBinding
import com.bingce.controlnetwork.databinding.NewItemSelectItemBinding
import com.bingce.utils.ScreenUtil


/**
 * 类似选择项目类型的底部弹框
 */
abstract class BaseSelectItemFragment : DialogFragment() {

    private var _binding: NewFragmentBaseSelectItemBinding? = null
    private val mBinding get() = _binding!!

    private val dataList = mutableListOf<IUniversalListItem<String>>()


    companion object {
        private const val KEY_REQUEST_SELECT = "__SELECT__"
        private const val KEY_TYPE = "__SELECT_TYPE__"

        @JvmStatic
        fun select(
            fragment: BaseSelectItemFragment,
            fragmentManager: FragmentManager,
            lifecycleOwner: LifecycleOwner,
            callback: ISelectItem
        ) {
            fragmentManager
                .setFragmentResultListener(
                    KEY_REQUEST_SELECT,
                    lifecycleOwner
                ) { requestKey: String, result: Bundle ->
                    if (KEY_REQUEST_SELECT == requestKey) {
                        callback.onBack(result.getString(KEY_TYPE)!!)
                    }
                }
            fragment.show(fragmentManager, null)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = NewFragmentBaseSelectItemBinding.inflate(inflater, container, false)
        return mBinding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
//        addMargin(view)
        mBinding.tvTitle.text = getTitle()
        initAdapter()
        initListener()
    }

    private fun initListener() {
        mBinding.cancel.setOnClickListener {
            dismissDialog()
        }
    }

    private fun dismissDialog() {
        dismissAllowingStateLoss()
    }

    override fun onStart() {
        super.onStart()
        dialog?.window?.run {
            setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
            val params = attributes
            params.gravity = Gravity.BOTTOM
            params.width = ScreenUtil.getScreenWidth(requireContext()) - ScreenUtil.dp2px(
                requireContext(),
                24f
            )
            params.y = ScreenUtil.dp2px(
                requireContext(),
                12f
            )

            setWindowAnimations(R.style.BottomDialogAnimation)
            //最后设置参数
            attributes = params
        }

    }

    private fun addMargin(view: View) {
        val layoutParams = view.layoutParams as MarginLayoutParams
        val margin = ScreenUtil.dp2px(requireContext(), 16f)
        layoutParams.setMargins(margin, margin, margin, margin)
        view.setLayoutParams(layoutParams)
        view.requestLayout()
    }

    private fun initAdapter() {
        dataList.addAll(getDataList())
        mBinding.recyclerView.addItemDecoration(
            HorDividerDecoration(requireContext())
        )
        mBinding.recyclerView.adapter = Adapter()
    }

    abstract fun getDataList(): List<IUniversalListItem<String>>

    abstract fun getTitle(): String


    private inner class Adapter : RecyclerView.Adapter<ViewHolder>() {

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            return ViewHolder(
                NewItemSelectItemBinding.inflate(
                    layoutInflater,
                    parent,
                    false
                )
            )
        }

        override fun getItemCount(): Int {
            return dataList.size
        }

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val item = dataList[position]
            holder.binding.tv.text = item.name

            holder.itemView.setOnClickListener {
                backData(item.value)
            }
        }

    }

    private class ViewHolder(val binding: NewItemSelectItemBinding) :
        RecyclerView.ViewHolder(binding.root)

    private fun backData(value: String) {
        val args = Bundle()
        args.putString(KEY_TYPE, value)
        getParentFragmentManager().setFragmentResult(
            KEY_REQUEST_SELECT,
            args
        )
        dismissDialog()
    }


    interface ISelectItem {
        fun onBack(value: String)
    }


}