package com.bingce.controlnetwork.newui.widget.switchtext;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.SwitchCompat;

import com.bingce.controlnetwork.R;


public class SwitchWithText extends LinearLayout {

    private String name;
    private String onText;
    private String offText;
    private boolean checked;
    private SwitchCompat switchButton;
    private TextView tvDesc;

    public SwitchWithText(Context context) {
        super(context);
        init(context, null);
    }

    public SwitchWithText(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs);
    }

    public SwitchWithText(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs);
    }

    private void init(Context context, AttributeSet attrs) {
        LayoutInflater inflater = LayoutInflater.from(context);
        inflater.inflate(R.layout.new_switch_with_layout, this);

        switchButton = findViewById(R.id.switchButton);
        tvDesc = findViewById(R.id.tvDesc);

        //获取属性
        if (attrs != null) {
            TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.SwitchWithText);
            name = a.getString(R.styleable.SwitchWithText_name);
            onText = a.getString(R.styleable.SwitchWithText_onText);
            offText = a.getString(R.styleable.SwitchWithText_offText);
            checked = a.getBoolean(R.styleable.SwitchWithText_onChecked, false);
            a.recycle();
        }

        switchButton.setChecked(checked);
        switchButton.setText(name);
        refreshDescText();

        switchButton.setOnCheckedChangeListener((buttonView, isChecked) -> {
            checked = isChecked;
            refreshDescText();
            if (checkChangeListener != null) {
                checkChangeListener.onCheck(isChecked);
            }
        });

    }

    public void setChecked(boolean checked) {
        switchButton.setChecked(checked);
    }

    public boolean isChecked() {
        return switchButton.isChecked();
    }

    private void refreshDescText() {
        if (checked) {
            tvDesc.setText(onText);
        } else {
            tvDesc.setText(offText);
        }
    }

    private ICheckChangeListener checkChangeListener;

    public void setOnCheckChangeListener(ICheckChangeListener checkChangeListener) {
        this.checkChangeListener = checkChangeListener;
    }

    public interface ICheckChangeListener {
        void onCheck(boolean checked);
    }

}
