package com.bingce.controlnetwork.surveyor.method;

import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.IPointDataSource;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.surveyor.method.model.IStateDelegate;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.fragment.SelectReSurveyorPointFragment;
import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.List;

import com.bingce.controlnetwork.R;

public class SingleCycleTipsMethod {
    /*
     * todo 如果没有对单测回内错误处理，后面该怎么办？？
     * */
    public static void showTips(String msg, String stationId,
                                int currentCycleIndex, IStateDelegate delegate,
                                FragmentManager fragmentManager, LifecycleOwner lifecycleOwner) {
        //取消
        delegate.onOptionTips(msg,
                Tools.getString(R.string.retest_direction),
                Tools.getString(R.string.retest_cycle),
                () -> {//选择重测方向
                    ThreadPoolUtil.execute(() -> reSurveyorPoint(stationId, delegate,
                            fragmentManager, lifecycleOwner));
                },
                () -> {//选择重测测回
                    reSurveyorCycle(msg, stationId, currentCycleIndex,
                            delegate, fragmentManager, lifecycleOwner);
                },
                delegate::resetSurveyorStateAndKeepPosition);
    }

    @WorkerThread
    private static void reSurveyorPoint(String stationId, IStateDelegate delegate,
                                        FragmentManager fragmentManager, LifecycleOwner lifecycleOwner) {
        //弹框提示选择重测数据
        ISurveyorStationDataSource dataSource = SurveyorDatabaseFactory.instance.getSurveyorStation();
        SurveyorStationRecord instance = dataSource.getRecordSync(stationId);
        IPointDataSource pointDataSource = SurveyorDatabaseFactory.instance.getPointDataSource();
        List<SurveyorPoint> stationItems = instance.getItems();
        ArrayList<String> ids = new ArrayList<>();
        ArrayList<String> names = new ArrayList<>();
        for (SurveyorPoint item : stationItems) {
            if (item.isAbandon()) {
                continue;
            }

            if (!SurveyorPoint.TYPE_POINT.equals(item.getType())) {
                continue;
            }
            PointRecord pointRecord = pointDataSource.findByIdSync(item.getPointId());
            if (pointRecord == null) {
                continue;
            }
            ids.add(pointRecord.id);
            names.add(pointRecord.name);
        }
        ThreadPoolUtil.executeInMain(() -> SelectReSurveyorPointFragment.pick(
                fragmentManager,
                lifecycleOwner,
                instance.leftAndRightSurveyor,
                ids,
                names,
                (cycleIndex, pointIds) -> {
                    if (pointIds == null || pointIds.isEmpty()) {
                        //没有选择，退出
                        delegate.resetSurveyorStateAndKeepPosition();
                    } else {
                        delegate.resurveyPoints(cycleIndex, pointIds);
                    }
                }
        ));
    }

    private static void reSurveyorCycle(String msg, String stationId,
                                        int currentCycleIndex, IStateDelegate delegate,
                                        FragmentManager fragmentManager, LifecycleOwner lifecycleOwner) {
        delegate.onOptionTips(Tools.getString(R.string.tips_delete_current_cycle_surveyor_record),
                null, null,
                () -> {
                    delegate.deleteSingleCycleData(currentCycleIndex);
                },
                () -> {
                    //循环显示
                    showTips(msg, stationId, currentCycleIndex, delegate,
                            fragmentManager, lifecycleOwner);
                },
                delegate::resetSurveyorStateAndKeepPosition);
    }
}
