package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.RefreshGoBackButtonAloneDistance;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.AutoSurveyorMethod;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.IStateType;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyor;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorListener;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorRemindListener;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParamAuto;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorAutoImpl;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorMock;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorMockAuto;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorSingleImpl;
import com.bingce.controlapphelper.surveyor.test.Test;
import com.bingce.controlapphelper.util.DigitalUtil;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureVm;
import com.bingce.controlnetwork.surveyor.state.AutoModeType;
import com.bingce.controlnetwork.surveyor.state.IScrollDelegate;
import com.bingce.controlnetwork.surveyor.state.StateConstants;
import com.bingce.coordlib.model.Coordinate;
import com.bingce.totalstation.ControlSurveyData;
import com.bingce.totalstation.ControlSurveyResult;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.VipManager;

import org.greenrobot.eventbus.EventBus;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import blankj.utilcode.util.ToastUtils;

public class StateManagerAloneDistance implements IAutoModeSwitcher, ISurveyState, IStateType {

    //单独测距代码里的cycleIndex只有一个测回
    public static final int CYCLE_INDEX_DEFAULT = 0;

    private final Map<StateType, __BaseState> stateMap = new HashMap<>();
    private StateType currentStateType = StateType.idle;
    private final FragmentManager fragmentManager;
    private final LifecycleOwner lifecycleOwner;
    private final IUIRunner iuiRunner;
    final IScrollDelegate scrollDelegate;

    @NonNull
    SurveyorStationRecord stationRecord;
    @NonNull
    final CachedDataAloneDistance cachedData;
    final @NonNull
    ToleranceDetailRecord toleranceRecord;
    final @NonNull
    ISurveyorData surveyorData;
    final @NonNull
    MeasureVm measureVm;
    final @NonNull
    ICycleRecorder cycleRecorder;
    final @NonNull
    Context context;
    private AutoModeType autoMode = AutoModeType.unknown;
    @NonNull
    final ISurveyor surveyor;
    final PointRecord stationPointRecord;

    final boolean supportAutoSurvey;


    private StateManagerAloneDistance(final boolean supportAutoSurvey,//是否支持自动测量
                                      final @NonNull SurveyorStationRecord stationRecord,
                                      final @NonNull PointRecord stationPointRecord,
                                      final @NonNull ToleranceDetailRecord toleranceRecord,
                                      final @NonNull Context context,
                                      final @NonNull ISurveyorData surveyorData,
                                      final @NonNull ICycleRecorder cycleRecorder,
                                      final @NonNull CachedDataAloneDistance cachedData,
                                      final @NonNull MeasureVm measureVm,
                                      final @NonNull IScrollDelegate scrollDelegate,
                                      FragmentManager fragmentManager,
                                      LifecycleOwner lifecycleOwner,
                                      IUIRunner iuiRunner) {
        this.supportAutoSurvey = supportAutoSurvey;

        this.stationRecord = stationRecord;
        this.stationPointRecord = stationPointRecord;
        this.fragmentManager = fragmentManager;
        this.lifecycleOwner = lifecycleOwner;
        this.iuiRunner = iuiRunner;
        this.scrollDelegate = scrollDelegate;

        this.context = context;
        this.toleranceRecord = toleranceRecord;
        this.surveyorData = surveyorData;
        this.cachedData = cachedData;
        this.measureVm = measureVm;
        this.cycleRecorder = cycleRecorder;

        if (Test.DEBUG) {
            surveyor = new SurveyorMockAuto(StateConstants.VALUE_MEASURE_WHAT_DISTANCE,
                    this.stationRecord.getId(),
                    this.stationRecord.isAutoClose(),
                    toleranceRecord.getCycleCount(),
                    cachedData, surveyorData, cycleRecorder, this, this);
        } else {
            if (supportAutoSurvey) {
                surveyor = new SurveyorAutoImpl(StateConstants.VALUE_MEASURE_WHAT_DISTANCE, this);
            } else {
                surveyor = new SurveyorSingleImpl(StateConstants.VALUE_MEASURE_WHAT_DISTANCE);
            }
        }
        _DoSurveyorState doSurveyorState = new _DoSurveyorState(this, this);
        surveyor.register((ISurveyorListener) doSurveyorState);
        surveyor.register((ISurveyorRemindListener) doSurveyorState);

        stateMap.put(StateType.doSurveyor, doSurveyorState);
        stateMap.put(StateType.doAutoSurveyor, new _DoAutoSurveyorState(this, this));
        stateMap.put(StateType.SurveyorFailure, new _SurveyorFailureState(this));
        stateMap.put(StateType.doMove2NextValidate, new _DoMove2NextValidateState(this));
        stateMap.put(StateType.doMoveBack, new _DoMoveBackState(this));
        stateMap.put(StateType.checkReadData, new _CheckReadDataState(this, this));
        stateMap.put(StateType.isSingleCycleCompleted, new _IsSingleCycleCompletedState(this));
        stateMap.put(StateType.isAllCycleCompleted, new _IsAllCycleCompletedState(this, this));
        stateMap.put(StateType.checkSingleCycle, new _CheckSingleCycleState(this));
        stateMap.put(StateType.checkStation, new _CheckStationState(this));
        stateMap.put(StateType.stationCheckedSuccess, new _CheckStationSuccessState(this));

        stateMap.put(StateType.doDeleteAllCycle, new _DoDeleteAllCycleState(this));
        stateMap.put(StateType.doDeleteCurrentCycle, new _DoDeleteCurrentCycleState(this));

        stateMap.put(StateType.doCalScheduleSingleCycle, new _DoCalScheduleInSingleCycleState(this, this));
        stateMap.put(StateType.doCalAdjust, new _DoCalAdjustState(this));
        stateMap.put(StateType.idle, new _IdleState(this, this));
        stateMap.put(StateType.doWellMeasureRotationReady, new _DoWellMeasureRotationReadyState(this));

    }

    public void updateStationRecord(SurveyorStationRecord stationRecord) {
        this.stationRecord = stationRecord;
        ThreadPoolUtil.execute(() -> {
            cachedData.updateDataSouce(stationRecord);
            if (Test.DEBUG && surveyor instanceof SurveyorMock) {
                ((SurveyorMock) surveyor).updateData();
            }
            _DoCalScheduleInSingleCycleState.enter(this, CYCLE_INDEX_DEFAULT);

            //最后刷新下页面
            runOnUI(cachedData::notifyDateChange);
        });

//        //更新测点棱镜信息
//        if (stationRecord.getItems() != null) {
//            for (SurveyorPoint item : stationRecord.getItems()) {
//                if (item == null || !SurveyorPoint.TYPE_POINT.equals(item.getType())) {
//                    continue;
//                }
//                cachedData.updateSurveyPointParams(item);
//            }
//        }
    }

    String stationId() {
        return stationRecord.getId();
    }

    @Override
    public void enter(StateType type, Bundle args) {
        __BaseState baseState = stateMap.get(type);
        if (baseState != null) {
            this.currentStateType = type;
            runOnUI(() -> baseState.enter(args));
        }
    }

    @Override
    public void enter(StateType type) {
        enter(type, null);
    }

    void runOnUI(Runnable action) {
        if (iuiRunner != null) {
            iuiRunner.runOnUI(action);
        }
    }

    public void doSurveyor() {
        boolean inValidate = isInValidate();

        if (inValidate) {
            return;
        }

        int pointCount = cachedData.pointCount();
        if (pointCount == 0) {
            ToastUtils.showShort(R.string.no_data_available_yet);
            return;
        }

        Bundle bundle = null;
        if (isRotationReadyState()) {
            //当前是旋转到位状态
            bundle = new Bundle();
            bundle.putBoolean(StateConstants.KEY_IS_ROTATION_READY_STATE, true);
        }

        enter(StateType.doSurveyor, bundle);

    }

    private boolean isRotationReadyState() {
        return currentStateType == StateType.doWellMeasureRotationReady;
    }

    public void doAutoSurveyor() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doAutoSurveyor);
    }

    /**
     * 删除站的所有测回记录
     */
    public void deleteCurrentStationSurveyorData() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doDeleteAllCycle, _DoDeleteAllCycleState.args(stationId()));
    }

    public void deleteCurrentStationSurveyorData(int cycleIndex) {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doDeleteCurrentCycle, _DoDeleteCurrentCycleState.args(stationId(), cycleIndex));
    }

    public void stationCheck() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.checkStation);
    }

    public void stationAdjust() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doCalAdjust);
    }

    /**
     * 切换测回时触发
     *
     * @param cycleIndex
     */
    public void switch2Cycle(int cycleIndex) {
        //由于井定向的距离测量是不用切换测回的，在一个页面展示了两个测回
//        if (isInValidate()) {
//            return;
//        }
//        _DoCalScheduleInSingleCycleState.enter(this, cycleIndex);
    }

    /**
     * 进入开始测量的时候触发
     *
     * @param cycleIndex
     */
    public void calScheduleWhenEnterSurveyFragment(int cycleIndex) {
        if (isInValidate()) {
            return;
        }
        _DoCalScheduleInSingleCycleState.enter(this, cycleIndex);
    }

    public void moveBack() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doMoveBack);
    }

    //检测当前状态是否支持从外界主动改变状态
    //如果是空闲或者是测站通过检测是有效的状态
    private boolean isInValidate() {
        return currentStateType != StateType.idle
                && StateType.stationCheckedSuccess != currentStateType
                && !isRotationReadyState();
    }


    FragmentManager fragmentManager() {
        return this.fragmentManager;
    }

    LifecycleOwner lifecycleOwner() {
        return this.lifecycleOwner;
    }

    final boolean isStationAutoClosed() {
        return stationRecord.isAutoClose();
    }

    @WorkerThread
    public static StateManagerAloneDistance newInstance(
            boolean isSupportAutoMode,
            @NonNull SurveyorStationRecord stationRecord,
            @NonNull Context context,
            @NonNull FragmentManager fragmentManager,
            @NonNull LifecycleOwner lifecycleOwner,
            @NonNull ToleranceDetailRecord toleranceRecord,
            @NonNull ISurveyorData surveyorData,
            @NonNull ICycleRecorder cycleRecorder,
            @NonNull CachedDataAloneDistance cachedData,
            @NonNull MeasureVm measureVm,
            IScrollDelegate scrollDelegate,
            IUIRunner runner) {
        //读取测点record
        String stationPointId = null;
        if (stationRecord.getItems() != null) {
            for (SurveyorPoint stationItem : stationRecord.getItems()) {
                if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                    stationPointId = stationItem.getPointId();
                    break;
                }
            }
        }
        PointRecord stationPointRecord = null;
        if (stationPointId != null) {
            stationPointRecord = SurveyorDatabaseFactory
                    .instance
                    .getPointDataSource()
                    .findByIdSync(stationPointId);
        }
        if (stationPointRecord == null) {
            return null;
        }
        boolean supportAuto = Test.DEBUG;
        //App.isDebug()
        if (isSupportAutoMode &&
                (VipManager.getInstance().checkReg() || VipManager.getInstance().checkControlProLevel())) {
            supportAuto = true;
        }
        StateManagerAloneDistance manager = new StateManagerAloneDistance(supportAuto,
                stationRecord,
                stationPointRecord,
                toleranceRecord,
                context,
                surveyorData,
                cycleRecorder,
                cachedData,
                measureVm,
                scrollDelegate,
                fragmentManager, lifecycleOwner, runner);
        return manager;
    }

    @Override
    public boolean isAutoModeOpen() {
        return autoMode == AutoModeType.start;
    }

    @Override
    public boolean isAutoModeClose() {
        return autoMode == AutoModeType.stop;
    }

    @Override
    public boolean isAutoModeNotSpecial() {
        return autoMode == AutoModeType.unknown;
    }

    @Override
    public void openAutoMode() {
        autoMode = AutoModeType.start;
    }

    @Override
    public void closeAutoMode() {
        autoMode = AutoModeType.stop;
    }

    @Override
    public void resetAutoMode() {
        autoMode = AutoModeType.unknown;
    }

    @Override
    public int isReadyAutoSurvey() {
        if (!supportAutoSurvey) {
            return ControlSurveyResult.CODE_ERROR;
        }
        if (!surveyor.isReady()) {
            return ControlSurveyResult.CODE_ERROR;
        }
        if (getRequestSurveyorParamAuto() == null) {
            return ControlSurveyResult.CODE_ERROR;
        }

        double instrumentHeight = getInsturmentHeight();
        Coordinate stationBasePoint = getStationCoord();

        return getRequestSurveyorParamAuto().checkAutoCondition(getCachedData(), stationBasePoint, instrumentHeight);
    }

    @Override
    public String isMeasuredValueMatchDesign() {
        if (!supportAutoSurvey) return null;

        double instrumentHeight = getInsturmentHeight();
        Coordinate stationBasePoint = getStationCoord();

        List<ControlSurveyData> originMeasureList = getRequestSurveyorParamAuto().getOriginMeasureList(getCachedData());

        return AutoSurveyorMethod.checkAutoSurveyorMeasuredValue(stationBasePoint, instrumentHeight, originMeasureList);
    }

    @Override
    public boolean isSurveying() {
        return currentStateType == StateType.doSurveyor || currentStateType == StateType.doAutoSurveyor;
    }

    @Override
    public boolean isAutoSurveying() {
        return isAutoModeOpen();
    }

    public void switchCurrentMeasurePosition(int position) {
        if (currentStateType == StateType.idle) {
            __BaseState baseState = stateMap.get(currentStateType);
            if (baseState != null) {
                baseState.setSurveyPosition(position, LeftOrRight.left);
            }
        }
    }

    public CachedDataAloneDistance getCachedData() {
        return cachedData;
    }

    @NonNull
    public ToleranceDetailRecord getToleranceRecord() {
        return toleranceRecord;
    }


    private Coordinate getStationCoord() {
        if (PointRecord.TYPE_KNOWN_POINT.equals(stationPointRecord.getPointType())) {
            return new Coordinate(
                    stationPointRecord.getX(),
                    stationPointRecord.getY(),
                    stationPointRecord.getZ());
        }
        return null;
    }

    public double getInsturmentHeight() {
        return DigitalUtil.valueOfDoubleString(stationRecord.getInstrumentHeight(), 0);
    }

    public RequestSurveyorParamAuto getRequestSurveyorParamAuto() {
        return measureVm.getRequestSurveyorParamAuto();
    }


    public void refreshGoBackBtnEnable() {
        CachedDataAloneDistance cachedData = getCachedData();
        int currentCycle = cycleRecorder.currentCycleIndex();
        String pointId = getCachedData().pointId(currentCycle, 0);
        String cacheKey = IdUtil.createSurveyorRecordId(stationId(), currentCycle, pointId, LeftOrRight.left);
        AloneDistanceRecord aloneDistanceRecord = cachedData.getAloneDistanceRecord(cacheKey);
        boolean isRollBackBtnEnable = aloneDistanceRecord != null && !aloneDistanceRecord.distanceValues.isEmpty();
        EventBus.getDefault().post(new RefreshGoBackButtonAloneDistance(isRollBackBtnEnable));
    }
}
