package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;

import net.tatans.tensorflowtts.tts.TtsManager;

/**
 * 检测当前测量数据读数互差
 */
class _CheckReadDataState extends __BaseState {
    private static final int RESULT_SUCCESS = 0;
    private static final int RESULT_FAILURE_HOR = 1;
    private static final int RESULT_FAILURE_VER = 2;
    private static final int RESULT_FAILURE_DISTANCE = 3;
    private final IAutoModeSwitcher autoModeSwitcher;

    public _CheckReadDataState(StateManagerAloneDistance stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.checkReadData, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    @Override
    public void enter(Bundle args) {
        //先检测测量值和设计值是否匹配（仅在自动测量打开下检测）
        String error = autoModeSwitcher.isMeasuredValueMatchDesign();
        if (error != null) {
            failureTipsWithAutoStandStill(error);
            return;
        }

        onCompleted(RESULT_SUCCESS);
    }

    private void onCompleted(int result) {
        if (RESULT_SUCCESS == result) {
            navigation2(StateType.checkSingleCycle);
        } else {
            String msg = "";
            switch (result) {
                case RESULT_FAILURE_HOR:
                    msg = Tools.getString(R.string.hor_angle_read_data_limit);
                    break;
                case RESULT_FAILURE_VER:
                    msg = Tools.getString(R.string.ver_angle_read_data_limit);
                    break;
                case RESULT_FAILURE_DISTANCE:
                    msg = Tools.getString(R.string.distance_read_data_limit);
                    break;
                default:
                    navigation2(StateType.idle);
                    return;
            }
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(msg);
            }
            failureTipsWithAutoStandStill(msg);
        }
    }

    /**
     * 斜距两次读数差
     *
     * @param distance1 读数1
     * @param distance2 读数2
     * @return 读数差(单位毫米)
     */
    public static double distanceReadDiff(double distance1, double distance2) {
        return Math.abs(distance1 - distance2) * 1000;
    }
}
