package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;

import java.util.List;

/**
 * 测回内不同方向2c、方向i互差检测
 */
class _CheckSingleCycleState extends __BaseState {

    public _CheckSingleCycleState(StateManagerAloneDistance stateManager) {
        super(StateType.checkSingleCycle, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        doCheck();
    }

    private void doCheck() {
        ToleranceDetailRecord toleranceRecord = getToleranceRecord();
        CachedDataAloneDistance cachedData = getCachedData();
        int surveyorCount = getSurveyorCount();
        int cycleIndex = getCycleRecorder().currentCycleIndex();
        int currentIndex = getCurrentIndex();
        double distCyclesDifference = toleranceRecord.getSlopeCycleDistanceDifference()/*slopeCycleDistanceDifference*/;
        AloneDistanceRecord aloneDistanceRecord = getAloneDistanceRecord(currentIndex);
        if (aloneDistanceRecord == null || aloneDistanceRecord.distanceValues.size() < surveyorCount * 2) {
            //进入旋转到位状态等待点击继续测量
            navigation2(StateType.doWellMeasureRotationReady);
            return;
        }

        List<Double> valuesFirst = aloneDistanceRecord.distanceValues.subList(0, surveyorCount);
        List<Double> valuesSecond = aloneDistanceRecord.distanceValues.subList(surveyorCount, surveyorCount * 2);


        Double averageFirst = Tools.calculateCollectionDoubleAverage(valuesFirst);
        Double averageSecond = Tools.calculateCollectionDoubleAverage(valuesSecond);

        double compareValue = Math.abs(averageFirst - averageSecond);

        if (compareValue * 1000 > distCyclesDifference) {
            optionTips(Tools.getString(R.string.cycle_between_distance_limit) + distCyclesDifference,
                    Tools.getString(R.string.retest_cycle),
                    Tools.getString(R.string.retest_point),
                    () -> {
                        cachedData.deleteAloneDistanceRecord(stationId(), cycleIndex, currentIndex, LeftOrRight.left, true);
                        //更新进度
                        setCurrentCycle(cycleIndex);
                        setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                        setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                        findTargetFromBeginInCurrentCycle(false);
                    },
                    () -> {
                        //重测目标
                        cachedData.deleteAloneDistanceRecord(stationId(), cycleIndex, currentIndex, LeftOrRight.left, false);
                        //更新进度
                        setCurrentCycle(cycleIndex);
                        setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                        setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                        findTargetFromBeginInCurrentCycle(false);
                    },
                    () -> {
                        enterIdleAndClearSurveyStateAndResetAutoMode(true);
                    });

        } else {
            onSuccess();
        }
    }

    private void onSuccess() {
        //更新测量进度
        setSingleCycleSchedule(getCycleRecorder().currentCycleIndex(),
                SurveyorScheduleConstants.STATUE_VALUE_CHECKED);
        //跳转
        navigation2(StateType.isSingleCycleCompleted);
    }
}
