package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.Util;

import java.util.ArrayList;
import java.util.List;

/**
 * 执行「测站检测」
 * <p>
 * 前置条件：
 * 所有测回都完成测量，并且通过「测回内检测」
 */
public class _CheckStationState extends __BaseState {

    public _CheckStationState(StateManagerAloneDistance stateManager) {
        super(StateType.checkStation, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        //执行测站检测
        String error = _stationCheckAloneDistance(stationId(), getToleranceRecord(), getCachedData());
        if (error == null || error.isEmpty()) {
            //进入finish状态
            navigation2(StateType.stationCheckedSuccess);
        } else {
            failureTipsWithAutoStandStill(error);
        }
    }

    public static String _stationCheckAloneDistance(final String stationId,
                                                    final ToleranceDetailRecord toleranceRecord,
                                                    final CachedDataAloneDistance cachedDataSource) {

        final int cycleCount = ToleranceDetailRecord.ALONE_DISTANCE_CYCLE_COUNT;
        final int surveyorCount = toleranceRecord.getSurveyorCount();


        //测回间斜距互差
        double distCyclesDifference = toleranceRecord.getSlopeCycleDistanceDifference()/*slopeCycleDistanceDifference*/;
        List<_ErrorItem> distError = new ArrayList<>();

        List<_Point> pointArrayList = new ArrayList<>();
        //遍历第一测回，收集所有待测点id
        for (int index = 0; index < cachedDataSource.pointCount(); index++) {
            pointArrayList.add(new _Point(cachedDataSource.pointId(0, index), cachedDataSource.pointName(0, index)));
        }
        //        if (!distError.isEmpty()) {
//            stringBuilder.append("测回间距离较差超限(限差:")
//                    .append(Util.formatDouble2String(distLimit, 1))
//                    .append("'')");
//            for (_ErrorItem item : distError) {
//                stringBuilder
//                        .append("\n   ")
//                        .append(item.pointName)
//                        .append(":\n")
//                        .append("   |")
//                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex1))
//                        .append("-")
//                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex2))
//                        .append("| = ")
//                        .append(Util.formatDouble2String(item.difference, 1))
//                        .append("'')");
//            }
//        }
        return "";
    }

    /**
     * 根据超限信息生成错误描述
     *
     * @param horValueError 水平角同方向值互差
     * @param verValueError 垂直角同方向值互差
     */
    private static String _errorString(double horLimit, double verLimit, double verILimit, double distLimit,
                                       List<_ErrorItem> horValueError, List<_ErrorItem> verValueError, List<_ErrorItem> verIError, List<_ErrorItem> distError) {
        if (horValueError.isEmpty() && verValueError.isEmpty()) {
            return null;
        }
        StringBuilder stringBuilder = new StringBuilder();
        if (!horValueError.isEmpty()) {
            stringBuilder.append(Tools.getString(R.string.cycle_between_same_direction_value_limit))
                    .append(Util.formatDouble2String(horLimit, 1))
                    .append("'')");
            for (_ErrorItem item : horValueError) {
                stringBuilder
                        .append("\n   ")
                        .append(item.pointName)
                        .append(":\n")
                        .append("   |")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex1))
                        .append("-")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex2))
                        .append("| = ")
                        .append(Util.formatDouble2String(item.difference, 1))
                        .append("'')");
            }
        }

        if (!verValueError.isEmpty()) {
            stringBuilder.append("\n").append(Tools.getString(R.string.cycle_between_ver_angle_limit))
                    .append(Util.formatDouble2String(verLimit, 1))
                    .append("'')");
            for (_ErrorItem item : verValueError) {
                stringBuilder
                        .append("\n   ")
                        .append(item.pointName)
                        .append(":\n")
                        .append("   |")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex1))
                        .append("-")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex2))
                        .append("| = ")
                        .append(Util.formatDouble2String(item.difference, 1))
                        .append("'')");
            }
        }

        if (!verIError.isEmpty()) {
            stringBuilder.append("\n").append(Tools.getString(R.string.ver_angle_i_same_direction_each_cycle))
                    .append(Util.formatDouble2String(verILimit, 1))
                    .append("'')");
            for (_ErrorItem item : verIError) {
                stringBuilder
                        .append("\n   ")
                        .append(item.pointName)
                        .append(":\n")
                        .append("   |")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex1))
                        .append("-")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex2))
                        .append("| = ")
                        .append(Util.formatDouble2String(item.difference, 1))
                        .append("'')");
            }
        }

        if (!distError.isEmpty()) {
            stringBuilder.append("\n").append(Tools.getString(R.string.cycle_between_distance_limit_brackets))
                    .append(Util.formatDouble2String(distLimit, 1))
                    .append("'')");
            for (_ErrorItem item : distError) {
                stringBuilder
                        .append("\n   ")
                        .append(item.pointName)
                        .append(":\n")
                        .append("   |")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex1))
                        .append("-")
                        .append(SurveyorCycleUtil.cycleName(item.cycleIndex2))
                        .append("| = ")
                        .append(Util.formatDouble2String(item.difference, 1))
                        .append("'')");
            }
        }

        return stringBuilder.toString();
    }

    /**
     * 存储错误信息的item
     */
    private static class _ErrorItem {
        final String pointName;
        final int cycleIndex1;
        final int cycleIndex2;
        final double difference;

        public _ErrorItem(String pointName, int cycleIndex1, int cycleIndex2, double difference) {
            this.pointName = pointName;
            this.cycleIndex1 = cycleIndex1;
            this.cycleIndex2 = cycleIndex2;
            this.difference = difference;
        }
    }

    private static class _Point {
        final String pointId;
        final String pointName;

        _Point(String pointId, String pointName) {
            this.pointId = pointId;
            this.pointName = pointName;
        }
    }
}
