package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;

import blankj.utilcode.util.ToastUtils;

/**
 * 移动到上一测量结点，伴随删除上次测量数据
 */
public class _DoMoveBackState extends __BaseState {

    public _DoMoveBackState(StateManagerAloneDistance stateManager) {
        super(StateType.doMoveBack, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        CachedDataAloneDistance cachedDataSource = getCachedData();
        ISurveyorData surveyorDataSource = getSurveyorData();
        ICycleRecorder cycleRecorder = getCycleRecorder();

        final int pointCount = cachedDataSource.pointCount();
        final int currentCycle = cycleRecorder.currentCycleIndex();
        final int currentIndex = surveyorDataSource.currentIndex();
        final LeftOrRight leftOrRight = surveyorDataSource.leftOrRight();

        int targetIndex = currentIndex;
        if (targetIndex < 0) {
            //如果测量结束，直接使用0号点盘右
            targetIndex = pointCount - 1;
        }
        warningWithStandStill(cachedDataSource, currentCycle, targetIndex, LeftOrRight.left);
    }

    private void warningWithStandStill(
            CachedDataAloneDistance cachedDataSource,
            int currentCycle, int itemIndex, LeftOrRight leftOrRight) {
        optionTipsWithStandStill(Tools.getString(R.string.roll_back_node_data),
                () -> {
                    //删除当前数据
                    int nextSurveyorPointIndex = cachedDataSource.deleteAloneDistanceRecord(stationId(),
                            currentCycle, itemIndex, leftOrRight, true);

                    if (nextSurveyorPointIndex == -1) {
                        ToastUtils.showShort(R.string.roll_back_failed);
                        navigation2(StateType.idle);
                        return;
                    }

                    AloneDistanceRecord aloneDistanceRecord = getAloneDistanceRecord(nextSurveyorPointIndex);
                    if (aloneDistanceRecord != null && aloneDistanceRecord.distanceValues.size() == getSurveyorCount()) {
                        navigation2(StateType.doWellMeasureRotationReady);
                    } else {
                        navigation2(StateType.idle);
                    }

                    //重置进度
                    setCurrentCycle(nextSurveyorPointIndex);
                    setSingleCycleSchedule(currentCycle, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                    setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);

                    resetSchedule(this, nextSurveyorPointIndex, leftOrRight);
                });
    }

    private void resetSchedule(__BaseState state,
                               int targetIndex,
                               LeftOrRight leftOrRight) {
        //设置测量目标点
        state.setSurveyPosition(targetIndex, leftOrRight);

        state.scroll2(targetIndex);
    }

}
