package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorListener;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorRemindListener;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParam;
import com.bingce.controlapphelper.util.DebugMsgUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.surveyor.state.StateConstants;
import com.bingce.device.Device;
import com.bingce.totalstation.TotalStation;
import com.bingce.totalstation.protocol.south.SouthAndroidBtInterface;
import com.bingce.totalstation.protocol.topcon.TopconInterface;

import net.tatans.tensorflowtts.tts.TtsManager;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

/**
 * 先检测是否有其他未完成测回，继续本次测量将删除其他测回数据
 */
class _DoSurveyorState extends __BaseState implements ISurveyorListener, ISurveyorRemindListener {
    private final ISurveyState surveyState;
    //最多测量次数，如果5次测量都没有获取正常数据，则跳过
    private static final int MAX_SURVEYOR_COUNT = 3;
    private static int failureCount = 0;
    private static final int RESULT_SUCCESS = 0;
    private static final int RESULT_FAILURE_EXCEED_MAX = 1;//测量失败次数超出最大限制
    private static final int RESULT_FAILURE_ERROR = 2;//测量失败，又没有允许重复尝试
    private static final boolean allowTryWhenFailure = true;//在测量失败的时候，是否允许重新测量

    private int measureTimeIndex = 0;
    private boolean isAlreadyRemindUserAimingAtWire = false;

    private final Handler handler;

    public _DoSurveyorState(StateManagerAloneDistance stateManager, ISurveyState surveyState) {
        super(StateType.doSurveyor, stateManager);
        this.surveyState = surveyState;
        handler = new Handler(Looper.getMainLooper());
    }

    @Override
    public void enter(Bundle args) {
        measureTimeIndex = 0;

        isAlreadyRemindUserAimingAtWire = false;
        if (args != null) {
            isAlreadyRemindUserAimingAtWire = args.getBoolean(StateConstants.KEY_IS_ROTATION_READY_STATE);
        }

        //3.其他测回没有正在测量数据，则在当前测回执行测量
        doSurveyor();
    }

    private void doSurveyor() {
        failureCount = 0;
        //判断设备是否连接
        if (surveyorImpl().isReady()) {
            //发送测量信号
            SurveyStateEvent.postSurveying(positionFragment);
            //注册接收测量数据
            surveyorImpl().prepareSurveyor();
            sendSurveyRequest();
        } else {
            enterIdleAndClearSurveyStateAndResetAutoMode(true);
            Device.getInstance().remindConnectDevice();
        }
    }

    @Override
    public void onReceive(double originalHa, double originalVa, double sd, double clockwiseHa, double topVa, double hd, double vd) {
        //重置错误读数
        failureCount = 0;
        AloneDistanceRecord aloneDistanceRecord
                = getCachedData().cacheAloneDistance(stationId(), StateManagerAloneDistance.CYCLE_INDEX_DEFAULT, getCurrentIndex(), sd, hd);

        //刷新页面
        refreshUI();
        int surveyorCount = getCachedData().getSurveyorCount();
        if (aloneDistanceRecord.distanceValues.size() == surveyorCount) {
            ToastUtils.showShort(Utils.getApp().getString(R.string.alone_distance_first_round_measure_finish));
            onSuccess();
        } else if (aloneDistanceRecord.distanceValues.size() >= surveyorCount * 2) {
            onSuccess();
        } else {
            //当数据不够的时候 继续发送指令
            measureTimeIndex++;
            if (TotalStation.getInstance() instanceof TopconInterface) {
                //这个仪器需要延迟500毫秒发送指令
                handler.postDelayed(this::sendSurveyRequest, 500);
            } else if (TotalStation.getInstance() instanceof SouthAndroidBtInterface) {
                handler.postDelayed(this::sendSurveyRequest, 100);
            } else {
                sendSurveyRequest();
            }
        }
    }

    @Override
    public void onError(String errorMsg) {
        TtsManager.getInstance().speak(errorMsg);
        if (allowTryWhenFailure) {//允许重新测量
            //测量出现异常，需要处理
            failureCount++;
            if (failureCount >= MAX_SURVEYOR_COUNT) {
                if (isAutoSurveyOpen()) {
                    TtsManager.getInstance().speak(Tools.getString(R.string.many_measure_failed));
                }
                onCompleted(RESULT_FAILURE_EXCEED_MAX, Tools.getString(R.string.many_measure_failed_1));
            } else {
                //再次尝试
                DebugMsgUtil.canWrite("仪器返回错误信息：" + errorMsg);
                DebugMsgUtil.canWrite("开始再次测量");

                sendSurveyRequest();
                ToastUtils.showShort(R.string.receive_data_time_out);
            }
        } else {
            onCompleted(RESULT_FAILURE_ERROR, errorMsg);
        }
    }

    private void sendSurveyRequest() {
        SurveyorPoint surveyorPoint = getCachedData().getSurveyorPoint(getCycleRecorder().currentCycleIndex(), getCurrentIndex());

        ISurveyorData surveyorData = getSurveyorData();
        int measureTimeIndex = 0;
        surveyorImpl().requestSurveyor(new RequestSurveyorParam(getRequestSurveyorParamAuto(), getInsturmentHeight(), surveyorPoint, measureTimeIndex,
                surveyorData.currentIndex(), surveyorData.leftOrRight(), isAlreadyRemindUserAimingAtWire));
    }

    private void onSuccess() {
        onCompleted(RESULT_SUCCESS, "");
    }

    private void onCompleted(int result, String msg) {
        //退出测量
        surveyorImpl().exitSurveyor();
        switch (result) {
            case RESULT_SUCCESS://进入读数检测状态
                //检测是否处于「自动测量」
                if (!surveyState.isAutoSurveying()) {
                    SurveyStateEvent.postSuccess(positionFragment);
                }
                navigation2(StateType.checkReadData);
                break;
            case RESULT_FAILURE_EXCEED_MAX:
            case RESULT_FAILURE_ERROR: {
                SurveyStateEvent.postFailure(positionFragment);
                //进入失败状态
                Bundle args = _SurveyorFailureState.args(msg);
                navigation2(StateType.SurveyorFailure, args);
            }
            break;
            default:
                throw new RuntimeException("un reached state -- surveyor");
        }
    }

    @Override
    public void remindUserAimingAtWire() {
        //退出测量
        surveyorImpl().exitSurveyor();

        Bundle bundle = new Bundle();
        bundle.putString(StateConstants.KEY_ROTATION_STATE_FROM_WHERE, StateConstants.ROTATION_STATE_FROM_WHERE_MEASURE);
        navigation2(StateType.doWellMeasureRotationReady, bundle);
    }
}
