package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.AutoSurveyorButtonVisibleEvent;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.totalstation.ControlSurveyResult;
import com.bingce.utils.ThreadPoolUtil;

import org.greenrobot.eventbus.EventBus;

/**
 * 空闲状态
 * <p>
 * 将刷新界面
 */
class _IdleState extends __BaseState {
    private final @NonNull
    IAutoModeSwitcher autoModeSwitcher;

    public _IdleState(StateManagerAloneDistance stateManager, @NonNull IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.idle, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    @Override
    public void enter(Bundle args) {

        refreshGoBack();

        ThreadPoolUtil.execute(() -> {
            boolean completed = _IsSingleCycleCompletedState.isSingleCycleCompleted(
                    stationId(),
                    StateManagerAloneDistance.CYCLE_INDEX_DEFAULT,
                    getCachedData(), ToleranceDetailRecord.ALONE_DISTANCE_MEASURE_COUNT);

            //判断当前条件是否满足自动测量
            boolean haveAutoCondition = autoModeSwitcher.isReadyAutoSurvey() == ControlSurveyResult.CODE_SUCCESS;

            //自动测量是否显示
            EventBus.getDefault().post(new AutoSurveyorButtonVisibleEvent(positionFragment, haveAutoCondition, autoModeSwitcher.isAutoModeOpen()));

            if (haveAutoCondition && autoModeSwitcher.isAutoModeOpen()) {
                //继续自动测量
                navigation2(StateType.doSurveyor);
            } else {
                if (autoModeSwitcher.isAutoModeOpen()) {
                    autoModeSwitcher.resetAutoMode();
                }
                if (completed) {
                    SurveyStateEvent.postCompleted(positionFragment);
                } else {
                    SurveyStateEvent.postIdle(positionFragment);
                }
            }
        });
    }

}