package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;

public class _IsSingleCycleCompletedState extends __BaseState {
    private static final int RESULT_YES = 0;//测回结束
    private static final int RESULT_NO = 1;//测回未结束

    public _IsSingleCycleCompletedState(StateManagerAloneDistance stateManager) {
        super(StateType.isSingleCycleCompleted, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        onCompleted(
                isSingleCycleCompleted(stationId(),
                        getCycleRecorder().currentCycleIndex(),
                        getCachedData(), getSurveyorCount()));
    }

    private void onCompleted(boolean completed) {
        if (completed) {
            //这里先刷新下UI，不然弹窗后面的list不显示最后一条数据
            getSurveyorData().clearSurveyStateAndPosition();
            refreshUI();

            int cycleIndex = getCycleRecorder().currentCycleIndex();
            //更新单测回测量进度
            setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
            //状态切换
            navigation2(StateType.isAllCycleCompleted);
        } else {
            findTargetFromBeginInCurrentCycle(false);
        }
    }

    /**
     * 判断单个测回是否测量完成（不要求通过检测）
     */
    public static boolean isSingleCycleCompleted(String stationId, int cycleIndex, CachedDataAloneDistance cachedData, int surveyCount) {
        int result = RESULT_YES;
        for (int index = 0; index < cachedData.pointCount(); index++) {
            String pointId = cachedData.pointId(cycleIndex, index);
            String leftOrRightKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, pointId, LeftOrRight.left);
            AloneDistanceRecord aloneDistanceRecord = cachedData.getAloneDistanceRecord(leftOrRightKey);
            if (aloneDistanceRecord == null || aloneDistanceRecord.distanceValues == null ||
                    aloneDistanceRecord.distanceValues.size() < surveyCount * 2) {
                result = RESULT_NO;
                break;
            }
        }
        return RESULT_YES == result;
    }
}
