package com.bingce.controlnetwork.surveyor.state.alonedistance;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.StringRes;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyor;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParamAuto;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureFragment;

import blankj.utilcode.util.ToastUtils;

import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureFragment;

abstract class __BaseState {
    private final StateType type;
    private final StateManagerAloneDistance stateManager;

    protected final int positionFragment = MeasureFragment.POSITION_DISTANCE_MEASURE;

    /**
     * 判断常量
     */
    private final int DEFAULT_AUTO_MODE = 0;
    private final int RESET_AUTO_MODE = 1;
    private final int CLOSE_AUTO_MODE = 2;

    protected final int cycleIndexDefault = StateManagerAloneDistance.CYCLE_INDEX_DEFAULT;

    public __BaseState(final StateType stateType, final StateManagerAloneDistance stateManager) {
        this.type = stateType;
        this.stateManager = stateManager;
    }

    final protected StateType getType() {
        return type;
    }

    final protected @NonNull
    ToleranceDetailRecord getToleranceRecord() {
        return stateManager.toleranceRecord;
    }

    final protected @NonNull
    ISurveyorData getSurveyorData() {
        return stateManager.surveyorData;
    }

    private @NonNull
    IAutoModeSwitcher autoModeRecorder() {
        return stateManager;
    }

    /**
     * 设置测量位置
     */
    final protected void setSurveyPosition(int index, LeftOrRight leftOrRight) {
        stateManager.surveyorData.setCurrentIndex(index);
        stateManager.surveyorData.setLeftOrRight(leftOrRight);
    }

    protected int getCurrentIndex() {
        return stateManager.surveyorData.currentIndex();
    }

    protected AloneDistanceRecord getAloneDistanceRecord(int surveyorPointIndex) {
        int cycleIndex = getCycleRecorder().currentCycleIndex();
        String pointId = getCachedData().pointId(cycleIndex, surveyorPointIndex);
        String cacheKey = IdUtil.createSurveyorRecordId(stationId(), cycleIndex, pointId, LeftOrRight.left);
        return getCachedData().getAloneDistanceRecord(cacheKey);
    }

    final protected ISurveyor surveyorImpl() {
        return stateManager.surveyor;
    }


    @NonNull
    final protected CachedDataAloneDistance getCachedData() {
        return stateManager.cachedData;
    }

    @NonNull
    final protected ICycleRecorder getCycleRecorder() {
        return stateManager.cycleRecorder;
    }

    @NonNull
    final protected Context getContext() {
        return stateManager.context;
    }

    final protected void scroll2(int itemIndex) {
        stateManager.scrollDelegate.onScrollTo(itemIndex);
    }


    final protected void toast(String warning) {
        ToastUtils.showShort(warning);
    }

    final protected void toast(@StringRes int warning) {
        ToastUtils.showShort(warning);
    }

    public abstract void enter(Bundle args);//进入状态

    final protected void navigation2(StateType stateType) {
        stateManager.enter(stateType);
    }

    final protected void navigation2(StateType stateType, Bundle args) {
        stateManager.enter(stateType, args);
    }

    final protected String stationId() {
        return stateManager.stationId();
    }

    protected int getSurveyorCount() {
        return stateManager.surveyorData.surveyorCount();
    }

    final protected FragmentManager fragmentManager() {
        return stateManager.fragmentManager();
    }

    final protected LifecycleOwner lifecycleOwner() {
        return stateManager.lifecycleOwner();
    }

    final protected void tips(String msg, Runnable runnable) {
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> {
                    if (runnable != null) {
                        runnable.run();
                    }
                });
    }

    /**
     * 错误提示
     * <p>
     * 用户不能忽略，点击确定后自动「停留原地」
     */
    final protected void failureTipsWithAutoStandStill(String msg) {
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true));
    }

    final protected void failureTipsWithAutoStandStill(String msg, Runnable runnable) {
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> {
                    runnable.run();
                    enterIdleAndClearSurveyStateAndResetAutoMode(true);
                });
    }

    final protected void optionTipsWithStandStill(String msg, Runnable confirmCallback) {
        optionTips(msg, null, null,
                confirmCallback,
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true),
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true));
    }

    final protected void optionTips(String msg,
                                    String confirmButton,
                                    String cancelButton,
                                    Runnable confirmCallback,
                                    Runnable cancelCallback,
                                    Runnable ignoreCallback) {
        TipsWith2OptionFragment.tips(
                stateManager.fragmentManager(),
                stateManager.lifecycleOwner(),
                confirmButton,
                cancelButton,
                null,
                msg,
                new TipsWith2OptionFragment.IOperatorCallback() {
                    @Override
                    public void onConfirm() {
                        if (confirmCallback == null) {
                            return;
                        }
                        confirmCallback.run();
                    }

                    @Override
                    public void onCancel() {
                        if (cancelCallback == null) {
                            return;
                        }
                        cancelCallback.run();
                    }

                    @Override
                    public void onIgnore() {
                        if (ignoreCallback == null) {
                            return;
                        }
                        ignoreCallback.run();
                    }

                    @Override
                    public void onOption() {
                    }
                });
    }

    final protected void runOnUI(Runnable runnable) {
        stateManager.runOnUI(runnable);
    }

    final protected void setCurrentCycle(int cycle) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getCurrentCycleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE),
                                String.valueOf(cycle)));
    }

    final protected void setCurrentCycleSync(int cycle) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getCurrentCycleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE),
                                String.valueOf(cycle)));
    }

    final protected void setSingleCycleSchedule(int cycleIndex, String statue) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getSingleCycleScheduleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE, cycleIndex),
                                statue));
    }

    final protected void setSingleCycleScheduleSync(int cycleIndex, String statue) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getSingleCycleScheduleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE, cycleIndex),
                                statue));
    }

    final protected void setAllCycleSchedule(String statue) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE),
                                statue));
    }

    final protected void setAllCycleScheduleSync(String statue) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), SurveyorScheduleConstants.SCHEDULE_OR_CYCLE_TYPE_ALONE_DISTANCE),
                                statue));
    }

    /**
     * 如果resetAutoSurveyState为true，将重置autoSurvey状态为unknown
     */
    final protected void findTargetFromBeginInCurrentCycle(boolean resetAutoSurveyState) {
        getSurveyorData().clearSurveyStateAndPosition();
        if (resetAutoSurveyState) {
            autoModeRecorder().resetAutoMode();
        }
        navigation2(StateType.doMove2NextValidate);
    }

    final protected void enterIdleAndClearSurveyState() {
        clearSurveyStateAndCloseAutoModeAndStandStill(DEFAULT_AUTO_MODE,
                false, StateType.idle);
    }

    final protected void enterIdleAndClearSurveyorStatePosition() {
        clearSurveyStateAndCloseAutoModeAndStandStill(DEFAULT_AUTO_MODE,
                true, StateType.idle);
    }

    /**
     * 取消测量，会停在当前位置（索引+盘左or盘右），并进入idle
     */
    final protected void enterIdleAndClearSurveyStateAndResetAutoMode(boolean resetAutoMode) {
        clearSurveyStateAndCloseAutoModeAndStandStill(resetAutoMode ? RESET_AUTO_MODE : DEFAULT_AUTO_MODE,
                false, StateType.idle);
    }


    final protected void enterStationCheckAndClearSurveyStateAndResetAutoMode() {
        clearSurveyStateAndCloseAutoModeAndStandStill(RESET_AUTO_MODE,
                false, StateType.checkStation);
    }

    /**
     * 原_Check2CState 使用这个方法
     */
    final protected void enterIdleAndClearSurveyStateAndCloseAutoMode() {
        clearSurveyStateAndCloseAutoModeAndStandStill(CLOSE_AUTO_MODE, false, StateType.idle);
    }

    /**
     * @param autoMode
     * @param clearPosition
     * @param stateType
     */
    private void clearSurveyStateAndCloseAutoModeAndStandStill(int autoMode, boolean clearPosition, StateType stateType) {
        //重置或者关闭自动模式
        if (autoMode == RESET_AUTO_MODE) {
            autoModeRecorder().resetAutoMode();
        } else if (autoMode == CLOSE_AUTO_MODE) {
            autoModeRecorder().closeAutoMode();
        }
        //更新测量进度标识
        if (clearPosition) {
            getSurveyorData().clearSurveyStateAndPosition();
        } else {
            getSurveyorData().clearSurveyState();
        }
        //进入某个状态
        navigation2(stateType);
    }


    /**
     * 更新UI
     */
    final protected void refreshUI() {
        runOnUI(() -> getCachedData().notifyDateChange());
    }

    protected boolean isAutoSurveyOpen() {
        return stateManager.isAutoModeOpen();
    }

    protected boolean isAutoSurveyClosed() {
        return stateManager.isAutoModeClose();
    }

    double getInsturmentHeight() {
        return stateManager.getInsturmentHeight();
    }

    RequestSurveyorParamAuto getRequestSurveyorParamAuto() {
        return stateManager.getRequestSurveyorParamAuto();
    }

    protected void refreshGoBack() {
        stateManager.refreshGoBackBtnEnable();
    }

}
