package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.ValueOf2cMethod;
import com.bingce.controlapphelper.surveyor.method.model.CacheResult;
import com.bingce.controlapphelper.surveyor.method.model.SdAngleValueType;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.Util;

import net.tatans.tensorflowtts.tts.TtsManager;

/*
 * 检测2c
 * */
class _Check2CState extends __BaseState {

    public _Check2CState(StateManager stateManager) {
        super(StateType.check2C, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        String error = _check2C(stationId(), getToleranceRecord(),
                getSurveyorData(),
                getCycleRecorder().currentCycleIndex(),
                getCachedData());
        if (error.isEmpty()) {
            navigation2(StateType.doRecordClean);
        } else {
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(Tools.getString(R.string.erxi_is_over_limit_please_check));
            }
            optionTips(error,
                    Tools.getString(R.string.retest_cycle),
                    Tools.getString(R.string.retest_point),
                    () -> optionTipsWithStandStill(Tools.getString(R.string.tips_delete_current_cycle_surveyor_record),
                            () -> {
                                String stationId = stationId();
                                int currentCycleIndex = getCycleRecorder().currentCycleIndex();
                                //删除数据
                                ICachedData cachedData = getCachedData();
                                cachedData.deleteSurveyorRecord(stationId, currentCycleIndex);
                                //更新进度
                                setSingleCycleSchedule(currentCycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                                setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                                //重新查找目标开启测量
                                findTargetFromBeginInCurrentCycle(true);
                            }),
                    this::enterIdleAndClearSurveyState,
                    this::enterIdleAndClearSurveyStateAndCloseAutoMode);
        }
    }

    private static String _check2C(String stationId,
                                   ToleranceDetailRecord toleranceRecord,
                                   ISurveyorData surveyorData,
                                   int cycleIndex,
                                   ICachedData cachedData) {
        StringBuilder stringBuilder = new StringBuilder();

        int currentIndex = surveyorData.currentIndex();

        CacheResult value2C = ValueOf2cMethod.valueOf2C(SdAngleValueType.horAngle, currentIndex, stationId,
                surveyorData, cycleIndex, cachedData);

        if (!value2C.validate || toleranceRecord.getHor2C() < Math.abs(value2C.value)) {
            stringBuilder.append(Tools.getString(R.string.hor_angle_2c_limit))
                    .append(toleranceRecord.getHor2C())
                    .append("'')\n")
                    .append("   ")
                    .append(Tools.getString(R.string.measurements))
                    .append(Util.formatDouble2String(Math.abs(value2C.value), 1))
                    .append("''");
        }

        value2C = ValueOf2cMethod.valueOf2C(SdAngleValueType.verAngle, currentIndex, stationId,
                surveyorData, cycleIndex, cachedData);
        if (!value2C.validate || toleranceRecord.getVerI() < Math.abs(value2C.value)) {
            stringBuilder.append("\n")
                    .append(Tools.getString(R.string.ver_angle_i_limit))
                    .append(toleranceRecord.getVerI())
                    .append("'')\n")
                    .append("   ")
                    .append(Tools.getString(R.string.measurements))
                    .append(Util.formatDouble2String(Math.abs(value2C.value), 1))
                    .append("''");
        }

        if (!cachedData.isAloneDistance(cycleIndex, surveyorData.currentIndex())) {
            value2C = ValueOf2cMethod.valueOf2C(SdAngleValueType.distance, currentIndex, stationId,
                    surveyorData, cycleIndex, cachedData);
            if (!value2C.validate || toleranceRecord.getSlopeLensDistanceDifference() < Math.abs(value2C.value)) {
                stringBuilder.append("\n")
                        .append(Tools.getString(R.string.distance_positive_and_negative_mirror_limit))
                        .append(toleranceRecord.getSlopeLensDistanceDifference())
                        .append("mm)\n")
                        .append("   ")
                        .append(Tools.getString(R.string.measurements))
                        .append(Util.formatDouble2String(Math.abs(value2C.value), 1))
                        .append("mm");
            }
        }

        return stringBuilder.toString();
    }
}
