package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.AverageValueMethod;
import com.bingce.controlapphelper.surveyor.method.DeviationValueOfRadianMethod;
import com.bingce.controlapphelper.surveyor.method.model.CacheResult;
import com.bingce.controlapphelper.surveyor.method.model.SdAngleValueType;
import com.bingce.controlapphelper.surveyor.state.StateType;

import net.tatans.tensorflowtts.tts.TtsManager;

import com.bingce.controlnetwork.R;

/*
 * 半测回归零检测 --- 只检测水平角
 * */
class _CheckHalfCycleZeroState extends __BaseState {
    private final static int RESULT_SUCCESS = 0;
    private final static int RESULT_FAILURE = 1;

    public _CheckHalfCycleZeroState(StateManager stateManager) {
        super(StateType.checkHalfCycleZero, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        if (isStationClosed()) {
            ISurveyorData surveyorDataSource = getSurveyorData();
            int cycleIndex = getCycleRecorder().currentCycleIndex();
            ICachedData stationSurveyorDataSource = getCachedData();
            ToleranceDetailRecord toleranceRecord = getToleranceRecord();
            int pointCount = stationSurveyorDataSource.pointCount();
            LeftOrRight leftOrRight = surveyorDataSource.leftOrRight();

            CacheResult angleHorFirst = AverageValueMethod.leftOrRightValue(SdAngleValueType.horAngle,
                    stationId(),
                    0,
                    leftOrRight,
                    surveyorDataSource,
                    cycleIndex,
                    stationSurveyorDataSource);
            CacheResult angleHorEnd = AverageValueMethod.leftOrRightValue(SdAngleValueType.horAngle,
                    stationId(),
                    pointCount - 1,
                    leftOrRight,
                    surveyorDataSource,
                    cycleIndex,
                    stationSurveyorDataSource);
            if (!angleHorFirst.validate || !angleHorEnd.validate) {
                onCompleted(RESULT_FAILURE);
            } else {
                double diff = DeviationValueOfRadianMethod.similarRadianDiff(angleHorFirst.value, angleHorEnd.value);
                if (toleranceRecord.getHorHalfReturnZeroDifference() < Math.toDegrees(diff) * 3600) {
                    onCompleted(RESULT_FAILURE);
                } else {
                    onCompleted(RESULT_SUCCESS);
                }
            }
        } else {
            onCompleted(RESULT_SUCCESS);
        }
    }

    private void onCompleted(int result) {
        if (RESULT_SUCCESS == result) {
            navigation2(StateType.isReady2CCheck);
        } else if (RESULT_FAILURE == result) {
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(getContext().getString(R.string.tips_error_half_cycle_zero_stop_survey));
            }
            failureTipsWithAutoStandStill(getContext().getString(R.string.tips_error_half_cycle_zero_stop_survey));
        } else {
            throw new RuntimeException("半测回归零差 failure");
        }
    }
}
