package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.DeviationValueOfRadianMethod;
import com.bingce.controlapphelper.surveyor.method.RadianMethod;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.totalstation.TotalStation;

import net.tatans.tensorflowtts.tts.TtsManager;


/*
 * 检测当前测量数据读数互差
 * */
class _CheckReadDataState extends __BaseState {
    private static final int RESULT_SUCCESS = 0;
    private static final int RESULT_FAILURE_HOR = 1;
    private static final int RESULT_FAILURE_VER = 2;
    private static final int RESULT_FAILURE_DISTANCE = 3;
    private final IAutoModeSwitcher autoModeSwitcher;

    public _CheckReadDataState(StateManager stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.checkReadData, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    @Override
    public void enter(Bundle args) {
        //先检测测量值和设计值是否匹配（仅在自动测量打开下检测）
        String error = autoModeSwitcher.isMeasuredValueMatchDesign();
        if (error != null) {
            failureTipsWithAutoStandStill(error);
            return;
        }
        int surveyorCount = getToleranceRecord().getSurveyorCount();
        if (surveyorCount <= 1) {//单次测量，直接通过检测
            onCompleted(RESULT_SUCCESS);
        } else {//多次测量，执行检测
            ISurveyorData surveyorDataSource = getSurveyorData();
            double horDifference = getToleranceRecord().getHorReadDifference();
            double verDifference = getToleranceRecord().getVerReadDifference();
            double distanceDifference = getToleranceRecord().getSlopeReadDifference();
            double horDiffRadian = DeviationValueOfRadianMethod.similarRadianDiff(surveyorDataSource.getAngleHor1(), surveyorDataSource.getAngleHor2());
            double verDiffRadian = DeviationValueOfRadianMethod.similarRadianDiff(surveyorDataSource.getAngleVer1(), surveyorDataSource.getAngleVer2());

            int resultCode = RESULT_SUCCESS;
            if (horDifference < RadianMethod.radian2S(horDiffRadian)) {
                resultCode = RESULT_FAILURE_HOR;
            } else if (verDifference < RadianMethod.radian2S(verDiffRadian)) {
                resultCode = RESULT_FAILURE_VER;
            } else if (!isAloneDistance()//增加单独测距判断
                    && distanceDifference < distanceReadDiff(surveyorDataSource.getDistance1(), surveyorDataSource.getDistance2())) {
                resultCode = RESULT_FAILURE_DISTANCE;
            }
            onCompleted(resultCode);
        }
    }

    private void onCompleted(int result) {
        if (RESULT_SUCCESS == result) {
            checkLeft();
        } else {
            String msg = "";
            switch (result) {
                case RESULT_FAILURE_HOR:
                    msg = Tools.getString(R.string.hor_angle_read_data_limit);
                    break;
                case RESULT_FAILURE_VER:
                    msg = Tools.getString(R.string.ver_angle_read_data_limit);
                    break;
                case RESULT_FAILURE_DISTANCE:
                    msg = Tools.getString(R.string.distance_read_data_limit);
                    break;
                default:
                    navigation2(StateType.idle);
                    return;
            }
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(msg);
            }
            failureTipsWithAutoStandStill(msg);
        }
    }

    /**
     * 如果是测回内首次测量，需要判断是否是盘左
     */
    private void checkLeft() {
        int currentItem = getSurveyorData().currentIndex();
        LeftOrRight leftOrRight = getSurveyorData().leftOrRight();
        if (leftOrRight == LeftOrRight.left) {
            //如果是首次测量，强制要求使用盘左
            double vAngle = getSurveyorData().getAngleVer();
            if (!isFirstValidate(vAngle)) {
                failureTipsWithAutoStandStill(
                        getContext().getString(R.string.error_tips_please_start_from_left),
                        () -> {
                            TotalStation.getInstance().requestChangeFace();
                        });
                return;
            }
        }
        navigation2(StateType.isHalfCycleZeroCheckReady);
    }

    private boolean isFirstValidate(double vAngle) {
        return vAngle >= 0 && vAngle <= Math.PI;
    }

    /**
     * 斜距两次读数差
     *
     * @param distance1 读数1
     * @param distance2 读数2
     * @return 读数差(单位毫米)
     */
    public static double distanceReadDiff(double distance1, double distance2) {
        return Math.abs(distance1 - distance2) * 1000;
    }
}
