package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.surveyorrecord.SurveyorRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.SingleCycleCheckMethod;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.surveyor.method.SingleCycleTipsMethod;

import net.tatans.tensorflowtts.tts.TtsManager;

/**
 * 测回内不同方向2c、方向i互差检测
 */
class _CheckSingleCycleState extends __BaseState {

    public _CheckSingleCycleState(StateManager stateManager) {
        super(StateType.checkSingleCycle, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        //1.先将该测回内所有补测数据设置为正常数据，再进行检测
        ICachedData cachedDataSource = getCachedData();
        ICycleRecorder cycleRecorder = getCycleRecorder();
        int pointCount = cachedDataSource.pointCount();
        int currentCycleIndex = cycleRecorder.currentCycleIndex();
        String stationId = stationId();

        for (int index = 0; index < pointCount; index++) {
            String pointId = cachedDataSource.pointId(currentCycleIndex, index);
            String keyLeft = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.left);
            SurveyorRecord left = cachedDataSource.getSurveyorRecord(keyLeft);
            _changeDirtyRecord2NormalBeforeCheck(left, cachedDataSource);
            String keyRight = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.right);
            SurveyorRecord right = cachedDataSource.getSurveyorRecord(keyRight);
            _changeDirtyRecord2NormalBeforeCheck(right, cachedDataSource);
        }
        //2.执行检测
        doCheck();
    }

    private static void _changeDirtyRecord2NormalBeforeCheck(
            SurveyorRecord record, ICachedData cachedDataSource) {
        if (record == null) {
            return;
        }
        if (record.value1Status.isResurveyed() || record.value2Status.isResurveyed()) {
            if (record.value1Status.isResurveyed()) {
                record.value1Status = DataCategory.normalCategory();
            }
            if (record.value2Status.isResurveyed()) {
                record.value2Status = DataCategory.normalCategory();
            }

            cachedDataSource.updateCache(record);
        }
    }

    private void doCheck() {
        ToleranceDetailRecord toleranceRecord = getToleranceRecord();
        ISurveyorData surveyorDataSource = getSurveyorData();
        ICycleRecorder cycleRecorder = getCycleRecorder();
        ICachedData stationSurveyorDataSource = getCachedData();

        //1.执行单测回内检测
        String error = SingleCycleCheckMethod.check(
                toleranceRecord.getHorCycleDirect2CDifference(), toleranceRecord.getVerCycleDirectIDifference(),
                stationId(), cycleRecorder.currentCycleIndex(), surveyorDataSource, stationSurveyorDataSource);
        if (error != null) {
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(Tools.getString(R.string.check_cycle_inner_failed));
            }
            //TODO 这里可能存在不对测回内错误处理的情况，后面应该兼容处理下
            SingleCycleTipsMethod.showTips(error, stationId(), cycleRecorder.currentCycleIndex(),
                    stateDelegate(),
                    fragmentManager(),
                    lifecycleOwner());
        } else {
            onSuccess();
        }
    }

    private void onSuccess() {
        //更新测量进度
        setSingleCycleSchedule(getCycleRecorder().currentCycleIndex(),
                SurveyorScheduleConstants.STATUE_VALUE_CHECKED);

        //跳转
        navigation2(StateType.isAllCycleCompleted);
    }
}
