package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.schedule.ISurveyorScheduleDataSource;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.method.FindTargetMethod;
import com.bingce.controlapphelper.surveyor.method.SingleCycleCheckMethod;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlnetwork.surveyor.method.SingleCycleTipsMethod;
import com.bingce.utils.ThreadPoolUtil;

/**
 * 在所有测回范围计算测量进度。
 * <p>
 * 1.检测当前测回，如果有没有完成测量，则跳转到相应位置。
 * 2.如果测量完成，检测数据限差。
 * 3.检测其他测回是否有待测数据
 * A.优先考虑dirty数据
 * B.其次考虑empty数据
 * 发现目标，执行跳转
 * 4.如果其他测回也完成测量，检测其限差
 * 发现超限，则立即返回true
 * 5.如果其他测回限差通过检测，设置标识测站可以执行检测
 */
class _DoCalScheduleInAllCycleState extends _DoCalScheduleInSingleCycleState {

    public _DoCalScheduleInAllCycleState(StateManager stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.doCalScheduleAllCycle, stateManager, autoModeSwitcher);
    }

    @Override
    protected boolean isNeedCancelEnterIdle(int currentCycle) {
        ICachedData cachedData = getCachedData();
        int pointCount = cachedData.pointCount();
        ToleranceDetailRecord toleranceDetailRecord = getToleranceRecord();
        //5.当前测回完成且通过检测，则对其他测回执行检测
        FindTargetMethod.Target resurveyTarget = null;
        int emptyCycleIndex = -1;
        for (int cycleIndex = 0; cycleIndex < toleranceDetailRecord.getCycleCount(); cycleIndex++) {
            if (cycleIndex == getCycleRecorder().currentCycleIndex()) {
                continue;
            }

            if (_IsSingleCycleCompletedState.isSingleCycleCompleted(stationId(), cycleIndex, getCachedData())) {
                //更新该测回进度
                setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
                continue;
            }
            //从0号位置盘左开始向后查询，找到需要补测位置
            resurveyTarget = FindTargetMethod.findTargetInSingleCycle(
                    stationId(), cycleIndex, 0, LeftOrRight.left, pointCount, true,
                    DataCategory.needResurvey(), cachedData);
            if (resurveyTarget != null) {
                break;
            }
            //测量没有完成，且没有补测数据，说明有空数据
            if (emptyCycleIndex < 0) {
                emptyCycleIndex = cycleIndex;
            }
        }

        //6.如果有需要补测数据，优先处理
        if (resurveyTarget != null) {
            option4Jump2OtherCycle4Resurvey(resurveyTarget.cycleIndex);
            return true;
        }
        //7.如果其他测回有empty数据，提示
        if (emptyCycleIndex >= 0) {
            option4Jump2OtherEmptyCycle(emptyCycleIndex);
            return true;
        }
        //8.所有测回都完成测量（当前测回完成检测）,执行检测
        double horCycle2CDiff = toleranceDetailRecord.getHorCycleDirect2CDifference();
        double verCycle2CDiff = toleranceDetailRecord.getVerCycleDirectIDifference();
        for (int cycleIndex = 0; cycleIndex < toleranceDetailRecord.getCycleCount(); cycleIndex++) {
            if (cycleIndex == getCycleRecorder().currentCycleIndex()) {
                continue;
            }
            String result = SingleCycleCheckMethod.check(
                    horCycle2CDiff,
                    verCycle2CDiff,
                    stationId(),
                    cycleIndex,
                    getSurveyorData(),
                    getCachedData());
            if (result != null && !result.isEmpty()) {
                SingleCycleTipsMethod.showTips(result,
                        stationId(),
                        cycleIndex,
                        stateDelegate(),
                        fragmentManager(),
                        lifecycleOwner());
                return true;
            }
            //更新该测回进度
            setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_CHECKED);
        }
        //9.所有测回都通过检测，可以执行「测站检测」
        //设置测量进度,但是需要注意，如果station已经通过检测，则不需要设置为completed
        ThreadPoolUtil.execute(() -> {
            ISurveyorScheduleDataSource scheduleDataSource =
                    SurveyorDatabaseFactory.instance.getScheduleDataSource();
            String key = SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), null);
            SurveyorScheduleRecord scheduleRecord = scheduleDataSource.findScheduleSync(key);
            if (scheduleRecord != null) {
                String statue = scheduleRecord.getStatue();
                if (SurveyorScheduleConstants.STATUE_VALUE_CHECKED.equals(statue)) {
                    onCompletedCheck();
                    return;
                }
            }
            scheduleDataSource.record(new SurveyorScheduleRecord(key,
                    SurveyorScheduleConstants.STATUE_VALUE_COMPLETED));

            //如果测量完成，则进入测站检测界面
            navigation2(StateType.checkStation);
        });
        return true;
    }

    static void enter(StateManager stateManager, int cycleIndex) {
        Bundle args = new Bundle();
        args.putInt(KEY_CYCLE_INDEX, cycleIndex);
        stateManager.enter(StateType.doCalScheduleAllCycle, args);
    }
}
