package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.method.SingleCycleCheckMethod;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlnetwork.surveyor.method.SingleCycleTipsMethod;

/**
 * 在单个测回内计算当前测量进度。
 * <p>
 * 1.如果有没有完成测量，则跳转到相应位置。
 * 2.如果测量完成，检测数据限差
 * 3.进入idle
 */
class _DoCalScheduleInSingleCycleState extends __BaseState {
    private final IAutoModeSwitcher autoModeSwitcher;

    public _DoCalScheduleInSingleCycleState(StateManager stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.doCalScheduleSingleCycle, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    protected _DoCalScheduleInSingleCycleState(StateType stateType, final StateManager stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(stateType, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    /**
     * 在其他测回是否包含需要测量的数据
     * <p>
     * 当前类默认返回false，不检测其他测回
     */
    protected boolean isNeedCancelEnterIdle(int currentCycle) {
        return false;
    }

    @Override
    final public void enter(Bundle args) {
        //重置自动测量状态
        if (autoModeSwitcher.isAutoModeClose()) {
            autoModeSwitcher.resetAutoMode();
        }
        //计算状态之前，重置测量状态
        getSurveyorData().clearSurveyStateAndPosition();

        ICycleRecorder cycleRecorder = getCycleRecorder();
        cycleRecorder.changeCycleIndex(args.getInt(KEY_CYCLE_INDEX, 0));
        setCurrentCycle(cycleRecorder.currentCycleIndex());

        //1.检测当前cycle是否完成测量
        boolean completed = _IsSingleCycleCompletedState.isSingleCycleCompleted(
                stationId(),
                getCycleRecorder().currentCycleIndex(),
                getCachedData());
        //2.如果当前测回还有待测数据，则执行测量
        if (!completed) {
            //从当前测回开始处查找目标并执行测量
            findTargetFromBeginInCurrentCycle(false);
            return;
        }
        //3.如果当前测回结束测量，判断是否通过检测
        ToleranceDetailRecord toleranceDetailRecord = getToleranceRecord();
        double horCycle2CDiff = toleranceDetailRecord.getHorCycleDirect2CDifference();
        double verCycle2CDiff = toleranceDetailRecord.getVerCycleDirectIDifference();
        String error = SingleCycleCheckMethod.check(
                horCycle2CDiff,
                verCycle2CDiff,
                stationId(),
                getCycleRecorder().currentCycleIndex(),
                getSurveyorData(),
                getCachedData());
        if (error != null && !error.isEmpty()) {
            SingleCycleTipsMethod.showTips(error,
                    stationId(),
                    getCycleRecorder().currentCycleIndex(),
                    stateDelegate(),
                    fragmentManager(),
                    lifecycleOwner());
            return;
        }
        //4.更新测站状态
        setSingleCycleSchedule(getCycleRecorder().currentCycleIndex(),
                SurveyorScheduleConstants.STATUE_VALUE_CHECKED);

        //5.检测其他测回状态
        if (isNeedCancelEnterIdle(getCycleRecorder().currentCycleIndex())) {
            return;
        }

        //6.清空测量状态
        onCompletedCheck();
    }

    protected void onCompletedCheck() {
        enterIdleAndClearSurveyStateAndResetAutoMode(true);
    }

    protected static final String KEY_CYCLE_INDEX = "___cycle__index__";

    static void enter(StateManager stateManager, int cycleIndex) {
        Bundle args = new Bundle();
        args.putInt(KEY_CYCLE_INDEX, cycleIndex);
        stateManager.enter(StateType.doCalScheduleSingleCycle, args);
    }
}
