package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.utils.ThreadPoolUtil;

import com.bingce.controlnetwork.R;

/*
 * 删除本测站所有测回数据
 * */
class _DoDeleteAllCycleState extends __BaseState {

    private static final String KEY_STATION_ID = "__station_id";

    static Bundle args(String stationId) {
        Bundle args = new Bundle();
        args.putString(KEY_STATION_ID, stationId);
        return args;
    }

    public _DoDeleteAllCycleState(StateManager stateManager) {
        super(StateType.doDeleteAllCycle, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        String stationId = args.getString(KEY_STATION_ID);

        optionTipsWithStandStill(getContext().getString(R.string.tips_delete_station_all_surveyor_records),
                () -> deleteAllCycleData(stationId));
    }

    private void deleteAllCycleData(String stationId) {
        ThreadPoolUtil.execute(() -> {
            //删除数据
            ICachedData cachedData = getCachedData();
            cachedData.deleteSurveyorRecord(stationId);

            //清除已经提示测站可以检测标识
            IConfigDataSource configDataSource =
                    SurveyorDatabaseFactory.instance.getConfigDataSource();
            String key = ConfigConstants.KEY_CONFIG_STATION_CHECK_REMINDED + stationId;
            configDataSource.setSync(key, ConfigConstants.VALUE_NO);

            //重置进度
            setAllCycleScheduleSync(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
            for (int cycleIndex = 0; cycleIndex < getToleranceRecord().getCycleCount(); cycleIndex++) {
                setSingleCycleScheduleSync(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
            }

            //重置学习测量的标志
            resetLearnDataTag();

            //从第一测回开始
            setCurrentCycleSync(0);

            //重新测量，并刷新
            findTargetFromBeginInCurrentCycle(true);
        });
    }


}
