package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.utils.ThreadPoolUtil;

import com.bingce.controlnetwork.R;

class _DoDeleteCurrentCycleState extends __BaseState {

    private static final String KEY_STATION_ID = "__station_id";
    private static final String KEY_PERIOD_INDEX = "__period_index__";

    static Bundle args(String stationId, int periodIndex) {
        Bundle args = new Bundle();
        args.putString(KEY_STATION_ID, stationId);
        args.putInt(KEY_PERIOD_INDEX, periodIndex);
        return args;
    }

    public _DoDeleteCurrentCycleState(StateManager stateManager) {
        super(StateType.doDeleteCurrentCycle, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        String stationId = args.getString(KEY_STATION_ID);
        int cycleIndex = args.getInt(KEY_PERIOD_INDEX);

        optionTipsWithStandStill(getContext().getString(R.string.tips_delete_current_cycle_surveyor_record),
                () -> deleteSingleCycle(stationId, cycleIndex));
    }

    private void deleteSingleCycle(String stationId, int cycleIndex) {
        ThreadPoolUtil.execute(() -> {
            //删除数据
            ICachedData stationSurveyorDataSource = getCachedData();
            stationSurveyorDataSource.deleteSurveyorRecordSync(stationId, cycleIndex);

            //清除已经提示测站可以检测标识
            IConfigDataSource configDataSource = SurveyorDatabaseFactory.instance.getConfigDataSource();
            String key = ConfigConstants.KEY_CONFIG_STATION_CHECK_REMINDED + stationId;
            configDataSource.setSync(key, ConfigConstants.VALUE_NO);

            //设置当前测回
            setCurrentCycleSync(cycleIndex);
            //重置所有测回和当次测回进度
            setAllCycleScheduleSync(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
            setSingleCycleScheduleSync(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);

            if (cycleIndex == 0) {
                //当第一测回
                //重置学习测量的标志
                resetLearnDataTag();
            }

            //重新测量，并刷新
            findTargetFromBeginInCurrentCycle(true);
        });

    }

}
