package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.FindTargetMethod;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;

import net.tatans.tensorflowtts.tts.TtsManager;

/**
 * 移动到一个待测量位置
 * <p>
 * 不会跳转到其他测期
 * <p>
 * 可能涉及转向（盘左->盘右）
 */
class _DoMove2NextValidateState extends __BaseState {

    public _DoMove2NextValidateState(StateManager stateManager) {
        super(StateType.doMove2NextValidate, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        ICachedData cachedData = getCachedData();
        ISurveyorData surveyorData = getSurveyorData();
        ICycleRecorder cycleRecorder = getCycleRecorder();

        int pointCount = cachedData.pointCount();
        int currentIndex = surveyorData.currentIndex();
        if (currentIndex < 0) {
            currentIndex = 0;
        }
        final int currentCycleIndex = cycleRecorder.currentCycleIndex();
        final LeftOrRight currentSurveyorState = surveyorData.leftOrRight();

        //1.先查找补测数据
        FindTargetMethod.Target surveyorTarget = FindTargetMethod.findDirtyTargetInSingleCycle(stationId(), currentCycleIndex,
                currentIndex,
                currentSurveyorState,
                pointCount,
                true,
                cachedData);
        //如果在当前测回找到待被测数据，立刻跳转到相应位置
        if (surveyorTarget != null) {
            navigation2(surveyorTarget.itemIndex, surveyorTarget.leftOrRight);
            return;
        }
        //2.查找empty数据
        FindTargetMethod.Target emptyTarget = FindTargetMethod.findEmptyTargetInSingleCycle(stationId(), currentCycleIndex,
                currentIndex,
                currentSurveyorState,
                pointCount,
                true,
                cachedData);
        if (emptyTarget != null) {
            if (isAutoSurveyOpen()) {
                if (emptyTarget.itemIndex == getCachedData().pointCount() - 1
                        && emptyTarget.leftOrRight == LeftOrRight.right) {
                    TtsManager.getInstance().speak(Tools.getString(R.string.start_face_right));
                }
            }
            //直接跳转到目标点，开启新测量
            navigation2(emptyTarget.itemIndex, emptyTarget.leftOrRight);
            return;
        }
        //3.当前测回完成测量，停留状态
        //重置标识
//        quitSurveyStateAndStandStill(false);
        enterIdleAndClearSurveyorStatePosition();
    }

    /**
     * 跳转到下一目标点
     *
     * @param targetIndex         目标点索引
     * @param targetSurveyorState 测量状态（盘左or盘右）
     */
    private void navigation2(int targetIndex, LeftOrRight targetSurveyorState) {
        //更新测量进度
        setSurveyPosition(targetIndex, targetSurveyorState);

        //清除测量状态，进入idle，但是不停止自动测量
        enterIdleAndClearSurveyStateAndResetAutoMode(false);

        //滚动
        scroll2(targetIndex);
    }
}
