package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;

import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorListener;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorRemindListener;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParam;
import com.bingce.controlapphelper.util.DebugMsgUtil;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.surveyor.state.StateConstants;
import com.bingce.device.Device;
import com.bingce.totalstation.TotalStation;
import com.bingce.totalstation.protocol.south.SouthAndroidBtInterface;
import com.bingce.totalstation.protocol.topcon.TopconInterface;

import net.tatans.tensorflowtts.tts.TtsManager;

import blankj.utilcode.util.ToastUtils;

/**
 * 先检测是否有其他未完成测回，继续本次测量将删除其他测回数据
 */
class _DoSurveyorState extends __BaseState implements ISurveyorListener, ISurveyorRemindListener {
    private final ISurveyState surveyState;
    //最多测量次数，如果3次测量都没有获取正常数据，则跳过
    private static final int MAX_SURVEYOR_COUNT = 3;
    private static int failureCount = 0;
    private static final int RESULT_SUCCESS = 0;
    private static final int RESULT_FAILURE_EXCEED_MAX = 1;//测量失败次数超出最大限制
    private static final int RESULT_FAILURE_ERROR = 2;//测量失败，又没有允许重复尝试
    private static final boolean allowTryWhenFailure = true;//在测量失败的时候，是否允许重新测量

    private int measureTimeIndex = 0;//当前测点是第几次测量
    private boolean isAlreadyRemindUserAimingAtWire = false;

    private final Handler handler;

    public _DoSurveyorState(StateManager stateManager, ISurveyState surveyState) {
        super(StateType.doSurveyor, stateManager);
        this.surveyState = surveyState;
        handler = new Handler(Looper.getMainLooper());
    }

    @Override
    public void enter(Bundle args) {
        measureTimeIndex = 0;

        isAlreadyRemindUserAimingAtWire = false;
        if (args != null) {
            isAlreadyRemindUserAimingAtWire = args.getBoolean(StateConstants.KEY_IS_ROTATION_READY_STATE);
        }

        ToleranceDetailRecord toleranceRecord = getToleranceRecord();
        ICycleRecorder cycleRecorder = getCycleRecorder();
        ICachedData cachedDataSource = getCachedData();

        int currentCycleIndex = cycleRecorder.currentCycleIndex();
        int cycleCount = toleranceRecord.getCycleCount();
        int pointCount = cachedDataSource.pointCount();

        int otherCycle = -1;
        //1.先检测其他测回是否有正在测量数据
        for (int cycleIndex = 0; cycleIndex < cycleCount; cycleIndex++) {
            if (cycleIndex == currentCycleIndex) {
                continue;
            }
            //遍历该测回内所有点，检测是否有测量数据
            for (int itemIndex = 0; itemIndex < pointCount; itemIndex++) {
                String pointId = cachedDataSource.pointId(cycleIndex, 0);
                String key = IdUtil.createSurveyorRecordId(stationId(), cycleIndex, pointId, LeftOrRight.left);
                //检测该测回是否有数据，目前应该只需要检测盘左0就可以了，因为一个测回内数据不能跳过
                DataCategory dataCategory = cachedDataSource.valueState(key);
                if (dataCategory.hasData()) {
                    //说明该测回有数据，则进一步检测是否完成
                    if (!cachedDataSource.isSingleCycleAllDataValidate(cycleIndex)) {
                        //已经测量但是没有完成，则提示
                        otherCycle = cycleIndex;
                        break;
                    }
                }
            }
            //如果当前遍历到的测回有数据，则退出遍历，提醒用户
            if (otherCycle >= 0) {
                break;
            }
        }
        //2.如果其他测回有正在测量数据，且没有完成测量，提示用户继续测量将会删除其数据
        if (otherCycle != -1) {
            final int targetCycleIndex = otherCycle;
            optionTipsWithStandStill(
                    SurveyorCycleUtil.cycleName(otherCycle) + Tools.getString(R.string.unfinished_measurements_starting_this_measurement_will_delete),
                    () -> optionTipsWithStandStill(
                            Tools.getString(R.string.confirm_delete) + SurveyorCycleUtil.cycleName(targetCycleIndex) + Tools.getString(R.string.data_ask),
                            () -> {
                                String stationId = stationId();
                                //删除数据
                                cachedDataSource.deleteSurveyorRecord(stationId, targetCycleIndex);
//                                isAlreadyRemindUserAimingAtWire = false;
                                //执行测量
                                doSurveyor();
                            }));
            return;
        }

        //3.其他测回没有正在测量数据，则在当前测回执行测量
        doSurveyor();
    }

    private void doSurveyor() {
        failureCount = 0;
        //判断设备是否连接
        if (surveyorImpl().isReady()) {
            //发送测量信号
            SurveyStateEvent.postSurveying(positionFragment);
            //注册接收测量数据
            surveyorImpl().prepareSurveyor();
            sendSurveyRequest();
        } else {
            enterIdleAndClearSurveyStateAndResetAutoMode(true);
            Device.getInstance().remindConnectDevice();
        }
    }

    @Override
    public void onReceive(double originalHa, double originalVa, double sd, double clockwiseHa, double topVa, double hd, double vd) {
        //重置错误读数
        failureCount = 0;
        if (getSurveyorData().onSurveyor(originalHa, originalVa, sd, hd)) {
            //测量数据成功，可以退出测量状态
            onSuccess();
        } else {//测量数据不完全，需要继续测量
            measureTimeIndex = 1;//更新测量索引
            refreshUI();

            if (TotalStation.getInstance() instanceof TopconInterface) {
                //这个仪器需要延迟500毫秒发送指令
                handler.postDelayed(this::sendSurveyRequest, 500);
            } else if (TotalStation.getInstance() instanceof SouthAndroidBtInterface) {
                handler.postDelayed(this::sendSurveyRequest, 100);
            } else {
                sendSurveyRequest();
            }
        }
    }

    @Override
    public void onError(String errorMsg) {
        TtsManager.getInstance().speak(errorMsg);
        if (allowTryWhenFailure) {//允许重新测量
            if (!isLive()) {
                return;
            }
            //测量出现异常，需要处理
            failureCount++;
            if (failureCount >= MAX_SURVEYOR_COUNT) {
                if (isAutoSurveyOpen()) {
                    TtsManager.getInstance().speak(Tools.getString(R.string.many_measure_failed));
                }
                onCompleted(RESULT_FAILURE_EXCEED_MAX, Tools.getString(R.string.many_measure_failed_1));
            } else {
                //再次尝试
                DebugMsgUtil.canWrite("仪器返回错误信息：" + errorMsg);
                DebugMsgUtil.canWrite("开始再次测量");

                sendSurveyRequest();
                ToastUtils.showShort(R.string.receive_data_time_out);
            }
        } else {
            onCompleted(RESULT_FAILURE_ERROR, errorMsg);
        }
    }

    private void sendSurveyRequest() {
        int currentIndex = getCurrentIndex();
        if (currentIndex < 0) {
            currentIndex = 0;
        }

        SurveyorPoint surveyorPoint = getCachedData().getSurveyorPoint(getCycleRecorder().currentCycleIndex(), currentIndex);
        ISurveyorData surveyorData = getSurveyorData();

        surveyorImpl().requestSurveyor(new RequestSurveyorParam(getRequestSurveyorParamAuto(), getInstrumentHeight(), surveyorPoint, measureTimeIndex,
                surveyorData.currentIndex(), surveyorData.leftOrRight(), isAlreadyRemindUserAimingAtWire));
    }

    private void onSuccess() {
        onCompleted(RESULT_SUCCESS, "");
    }

    private void onCompleted(int result, String msg) {
        //退出测量
        surveyorImpl().exitSurveyor();
        switch (result) {
            case RESULT_SUCCESS://进入读数检测状态
                //检测是否处于「自动测量」
                if (!surveyState.isAutoSurveying()) {
                    SurveyStateEvent.postSuccess(positionFragment);
                }
                navigation2(StateType.checkReadData);
                break;
            case RESULT_FAILURE_EXCEED_MAX:
//                if (SettingValueUtil.isAbandonPoint() && isAllowAbandonForVarifyIndex()) {
//                    dealErrorDataSkip(msg);
//                } else {
//                    dealErrorDataNormal(msg);
//                }
                dealErrorDataNormal(msg);
                break;
            case RESULT_FAILURE_ERROR:
                dealErrorDataNormal(msg);
                break;
            default:
                throw new RuntimeException("un reached state -- surveyor");
        }
    }

    @Override
    protected void quite() {
        super.quite();
        surveyorImpl().exitSurveyor();
    }

    @Override
    protected void resume() {
        getSurveyorData().clearSurveyState();
    }

    @Override
    public void remindUserAimingAtWire() {
        //退出测量
        surveyorImpl().exitSurveyor();
        navigation2(StateType.doWellMeasureRotationReady);
    }

    private void dealErrorDataNormal(String msg) {
        SurveyStateEvent.postFailure(positionFragment);
        //进入失败状态
        Bundle args = _SurveyorFailureState.args(msg);
        navigation2(StateType.SurveyorFailure, args);
    }

    private void dealErrorDataNormal(String msg, boolean isShowDialog) {
        SurveyStateEvent.postFailure(positionFragment);
        //进入失败状态
        Bundle args = _SurveyorFailureState.args(msg, isShowDialog);
        navigation2(StateType.SurveyorFailure, args);
    }

    private void dealErrorDataSkip(String msg) {
        optionTipsCountDown(msg + Tools.getString(R.string.is_it_abandoned),
                Tools.getString(R.string.switch_abandon),
                Tools.getString(R.string.ignore),
                () -> {
                    changeSkip(getCycleRecorder().currentCycleIndex(), getCurrentIndex());
                    navigation2(StateType.isSingleCycleCompleted);
                },
                () -> dealErrorDataNormal(msg, false),
                () -> dealErrorDataNormal(msg, false));
    }

    /**
     * 针对第一个点和闭合是否废弃验证
     *
     * @return
     */
    private boolean isAllowAbandonForVarifyIndex() {
        boolean autoClose = getStation().isAutoClose();
        int pointCount = getCachedData().pointCount();
        boolean abandon = true;
        int currentIndex = getCurrentIndex();
        if (autoClose) {
            if (currentIndex == 0 || currentIndex == pointCount - 1) {
                abandon = false;
            }
        } else {
            if (currentIndex == 0) {
                abandon = false;
            }
        }
        return abandon;
    }
}
