package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.event.AutoSurveyorButtonVisibleEvent;
import com.bingce.controlapphelper.event.SurveyStateEvent;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.totalstation.ControlSurveyResult;
import com.bingce.utils.ThreadPoolUtil;

import net.tatans.tensorflowtts.tts.TtsManager;

import org.greenrobot.eventbus.EventBus;

import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;

/**
 * 空闲状态
 * <p>
 * 将刷新界面
 */
class _IdleState extends __BaseState {
    private final @NonNull
    IAutoModeSwitcher autoModeSwitcher;

    public _IdleState(StateManager stateManager, @NonNull IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.idle, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    @Override
    public void enter(Bundle args) {
        //1.刷新界面
        refreshUI();

        refreshGoBackBtnEnable();

        //3.当前cycle是否完成测量
        boolean completed = _IsSingleCycleCompletedState.isSingleCycleCompleted(
                stationId(),
                getCycleRecorder().currentCycleIndex(),
                getCachedData());
        //4.检测是否满足自动测量条件
        ThreadPoolUtil.execute(() -> {
            //判断当前条件是否满足自动测量

            boolean haveAutoCondition = haveAutoCondition();

            if (haveAutoCondition && autoModeSwitcher.isAutoModeOpen() && isLive()) {
                navigation2(StateType.doSurveyor);
            } else if (haveAutoCondition && autoModeSwitcher.isAutoModeNotSpecial() && !completed) {
                //用户选择后会发送通知，更新测量按钮UI
                tips4StartAutoSurvey();
            } else {
//                ControlSurveyResult.isAbandon(autoMeasureReadyCode) != null//这个错误
                if (autoModeSwitcher.isAutoModeOpen()) {
                    autoModeSwitcher.resetAutoMode();
                }
                postEvent(haveAutoCondition, autoModeSwitcher.isAutoModeOpen(), completed);
            }

        });
    }

    /**
     * 弹框提示用户是否开启自动测量
     */
    private void tips4StartAutoSurvey() {
        runOnUI(() ->
                TipsWith2OptionFragment.hardTips(
                        fragmentManager(),
                        lifecycleOwner(),
                        getContext().getString(R.string.tips_finish_study_should_auto_survey),
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                TtsManager.getInstance().speak(Tools.getString(R.string.start_auto_measure));
                                autoModeSwitcher.openAutoMode();
                                navigation2(StateType.doSurveyor);

                                postEvent(true, true, false);
                            }

                            @Override
                            public void onCancel() {
                                autoModeSwitcher.closeAutoMode();

                                postEvent(true, false, false);
                            }

                            @Override
                            public void onIgnore() {
                                autoModeSwitcher.closeAutoMode();

                                postEvent(true, false, false);
                            }
                        }
                ));
    }

    private void postEvent(boolean isReadyAutoSurvey, boolean isOpenAutoSurvey, boolean isSingleCompleted) {
        EventBus.getDefault().post(new AutoSurveyorButtonVisibleEvent(positionFragment, isReadyAutoSurvey, isOpenAutoSurvey));
        if (isSingleCompleted) {
            SurveyStateEvent.postCompleted(positionFragment);
        } else if (!isOpenAutoSurvey) {
            SurveyStateEvent.postIdle(positionFragment);
        }
    }

    @Override
    protected void resume() {
        autoModeSwitcher.resetAutoMode();
        //刷新测量按钮
        boolean completed = _IsSingleCycleCompletedState.isSingleCycleCompleted(
                stationId(),
                getCycleRecorder().currentCycleIndex(),
                getCachedData());
        boolean isReady = autoModeSwitcher.isReadyAutoSurvey() == ControlSurveyResult.CODE_SUCCESS;

        postEvent(isReady, false, completed);
    }

    private boolean haveAutoCondition() {
        int autoMeasureReadyCode = autoModeSwitcher.isReadyAutoSurvey();
        return autoMeasureReadyCode == ControlSurveyResult.CODE_SUCCESS;
    }


}