package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;

/**
 * 判断是否满足「2C」检测条件
 */
class _Is2CCheckReadyState extends __BaseState {
    private static final int RESULT_YES = 0;//可以检测「2C」
    private static final int RESULT_NO = 1;//不可以检测「2C」

    public _Is2CCheckReadyState(StateManager stateManager) {
        super(StateType.isReady2CCheck, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        ISurveyorData surveyorDataSource = getSurveyorData();
        ICycleRecorder cycleRecorder = getCycleRecorder();
        ICachedData cachedData = getCachedData();

        int currentIndex = surveyorDataSource.currentIndex();
        int currentCycleIndex = cycleRecorder.currentCycleIndex();
        LeftOrRight currentLeftOrRight = surveyorDataSource.leftOrRight();

        //只需要对面位置有「新鲜数据」，就能执行2c
        //当前位置正处于测量，数据还没有缓存，所以不需要检测数据合法性
        String pointId = cachedData.pointId(currentCycleIndex, currentIndex);
        DataCategory oppositeCategory = cachedData.valueState(
                IdUtil.createSurveyorRecordId(stationId(), currentCycleIndex, pointId, currentLeftOrRight.opposite()));
        if (oppositeCategory.isFreshData()) {
            onCompleted(RESULT_YES);
        } else {
            onCompleted(RESULT_NO);
        }
    }

    private void onCompleted(int result) {
        if (RESULT_YES == result) {
            navigation2(StateType.check2C);
        } else if (RESULT_NO == result) {
            navigation2(StateType.doRecordClean);
        } else {
            throw new RuntimeException("error half cycle complete check");
        }
    }
}
