package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.surveyor.data.SpeakCache;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.ThreadPoolUtil;

import net.tatans.tensorflowtts.tts.TtsManager;

/*
 * 检测是否所有 测期都完成了测量
 *
 * 只有在当前测期完成测量之后，才会进入当前状态
 * */
public class _IsAllCycleCompletedState extends __BaseState {
    private static final int INVALIDATE_CYCLE = -1;
    private final IAutoModeSwitcher autoModeSwitcher;

    public _IsAllCycleCompletedState(@NonNull StateManager stateManager, IAutoModeSwitcher autoModeSwitcher) {
        super(StateType.isAllCycleCompleted, stateManager);
        this.autoModeSwitcher = autoModeSwitcher;
    }

    @Override
    public void enter(Bundle args) {
        int cycleCount = getToleranceRecord().getCycleCount();

        int theCycleNotAllDataIsValidate = INVALIDATE_CYCLE;
        //查找数据不全的测回
        for (int cycleIndex = 0; cycleIndex < cycleCount; cycleIndex++) {
            if (!getCachedData().isSingleCycleAllDataValidate(cycleIndex)) {
                //对不全数据的测回赋值
                theCycleNotAllDataIsValidate = cycleIndex;
                break;
            }
        }

        boolean completed = INVALIDATE_CYCLE == theCycleNotAllDataIsValidate;

        if (completed) {
            if (isAutoSurveyOpen()) {
                //检测是否已经提示
                //检测是否需要提示测站检测
                IConfigDataSource configDataSource =
                        SurveyorDatabaseFactory.instance.getConfigDataSource();
                ThreadPoolUtil.execute(() -> {
                    String key = ConfigConstants.KEY_CONFIG_STATION_CHECK_REMINDED + stationId();
                    ConfigRecord record = configDataSource.getByKeySync(key);
                    if (record == null || ConfigConstants.VALUE_NO.equals(record.getConfigValue())) {
                        runOnUI(() -> TtsManager.getInstance().speak(Tools.getString(R.string.all_cycle_completed)));
                    }
                });
            }
            //标识测站完成测量（数据库存储）
            setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
            //进入idle状态
            //清除测量状态，进入测站检测，停止自动测量
            enterStationCheckAndClearSurveyStateAndResetAutoMode();
        } else {
            if (autoModeSwitcher.isAutoModeOpen()) {
                //直接切换测回
                //切换测回
                getCycleRecorder().changeCycleIndex(theCycleNotAllDataIsValidate);
                //修改当前测回
                setCurrentCycle(theCycleNotAllDataIsValidate);
                //存储一下已经播放切换测回的的节点
                SpeakCache.getInstance().saveSpeakSwitchCycle();
                //查找测量目标
                findTargetFromBeginInCurrentCycle(false);
                //播报
                TtsManager.getInstance().speak(Tools.getString(R.string.current_cycle_complete_start_surveyor) + SurveyorCycleUtil.cycleName(theCycleNotAllDataIsValidate));
            } else {
                //提示是否进入其他未完成测回继续测量
                option4Jump2OtherEmptyCycle(theCycleNotAllDataIsValidate);
            }
        }
    }
}
