package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;

/*
 * 判断是否需要检测「半测回归零差」
 * */
class _IsHalfCycleZeroCheckReadyState extends __BaseState {
    private static final int RESULT_ERROR = -1;//状态错误
    private static final int RESULT_YES = 0;//可以检测「半测回归零差」
    private static final int RESULT_NO = 1;//不可以检测「半测回归零差」

    public _IsHalfCycleZeroCheckReadyState(StateManager stateManager) {
        super(StateType.isHalfCycleZeroCheckReady, stateManager);
    }


    @Override
    public void enter(Bundle args) {
        if (isStationClosed()) {
            ICachedData cachedData = getCachedData();
            ISurveyorData surveyorData = getSurveyorData();
            int pointCount = cachedData.pointCount();
            int currentItemIndex = surveyorData.currentIndex();

            //检测target处数据类型,如果存在dirty或empty数据，则不进行半测回归零
            //当前位置不需要检测，因为下处于测量中
            int result = RESULT_NO;
            if (currentItemIndex == 0 || currentItemIndex == pointCount - 1) {
                ICycleRecorder cycleRecorder = getCycleRecorder();
                int cycleIndex = cycleRecorder.currentCycleIndex();
                String stationId = stationId();
                LeftOrRight leftOrRight = surveyorData.leftOrRight();
                int targetIndex = pointCount - 1 - currentItemIndex;

                String targetPointId = cachedData.pointId(cycleRecorder.currentCycleIndex(), targetIndex);
                String targetKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, targetPointId, leftOrRight);
                DataCategory targetDataCategory = cachedData.valueState(targetKey);
                //对面数据正常，可以执行半测回归零
                if (targetDataCategory.isFreshData()) {
                    result = RESULT_YES;
                }
            }
            onCompleted(result);
        } else {
            onCompleted(RESULT_ERROR);
        }

    }

    private void onCompleted(int result) {
        if (RESULT_YES == result) {//检测半测回归零
            navigation2(StateType.checkHalfCycleZero);
        } else if (RESULT_NO == result || RESULT_ERROR == result) {
            navigation2(StateType.isReady2CCheck);
        } else {
            throw new RuntimeException("error half cycle complete check");
        }
    }
}
