package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;


public class _IsSingleCycleCompletedState extends __BaseState {
    private static final int RESULT_YES = 0;//测回结束
    private static final int RESULT_NO = 1;//测回未结束

    public _IsSingleCycleCompletedState(StateManager stateManager) {
        super(StateType.isSingleCycleCompleted, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        onCompleted(
                isSingleCycleCompleted(stationId(),
                        getCycleRecorder().currentCycleIndex(),
                        getCachedData()));
    }

    private void onCompleted(boolean completed) {
        if (completed) {
            //这里先刷新下UI，不然弹窗后面的list不显示最后一条数据
            getSurveyorData().clearSurveyStateAndPosition();
            refreshUI();

            int cycleIndex = getCycleRecorder().currentCycleIndex();
            //更新测回测量进度
            setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
            //状态切换
            navigation2(StateType.checkSingleCycle);
        } else {
            findTargetFromBeginInCurrentCycle(false);
        }
    }

    /**
     * 判断单个测回是否测量完成（不要求通过检测）
     */
    public static boolean isSingleCycleCompleted(String stationId, int cycleIndex, ICachedData cachedData) {
        int result = RESULT_YES;

        //测回内所有数据都是「新鲜数据」才能执行「测回内检测」
        for (int index = 0; index < cachedData.pointCount(); index++) {

            if (cachedData.isPointAbandon(cycleIndex, index)) {
                continue;
            }

            String pointId = cachedData.pointId(cycleIndex, index);

            String leftKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, pointId, LeftOrRight.left);
            String rightKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, pointId, LeftOrRight.right);

            DataCategory leftDataCategory = cachedData.valueState(leftKey);
            DataCategory rightDataCategory = cachedData.valueState(rightKey);

            if (leftDataCategory.isFreshData() && rightDataCategory.isFreshData()) {
                continue;
            }
            result = RESULT_NO;
            break;
        }

        return RESULT_YES == result;
    }
}
