package com.bingce.controlnetwork.surveyor.state.angledistance;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;

import androidx.annotation.NonNull;
import androidx.annotation.StringRes;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.model.IStateDelegate;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyor;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParamAuto;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;
import com.bingce.controlnetwork.fragment.stationsurvey.angledistance.MeasureFragment;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

import blankj.utilcode.util.ToastUtils;

public abstract class __BaseState {
    private final StateType type;
    private final StateManager stateManager;

    protected final int positionFragment = MeasureFragment.POSITION_DIRECTION_MEASURE;

    /**
     * 判断常量
     */
    private final int DEFAULT_AUTO_MODE = 0;
    private final int RESET_AUTO_MODE = 1;
    private final int CLOSE_AUTO_MODE = 2;

    public __BaseState(final StateType stateType, final StateManager stateManager) {
        this.type = stateType;
        this.stateManager = stateManager;
    }

    final protected StateType getType() {
        return type;
    }

    final protected @NonNull
    ToleranceDetailRecord getToleranceRecord() {
        return stateManager.toleranceRecord;
    }

    final protected @NonNull
    ISurveyorData getSurveyorData() {
        return stateManager.surveyorData;
    }

    private @NonNull
    IAutoModeSwitcher autoModeRecorder() {
        return stateManager;
    }

    /**
     * 设置测量位置
     */
    final protected void setSurveyPosition(int index, LeftOrRight leftOrRight) {
        stateManager.surveyorData.setCurrentIndex(index);
        stateManager.surveyorData.setLeftOrRight(leftOrRight);
        //新增
        if (isAloneDistance())
            ToastUtils.showShort(getContext().getString(R.string.attention_point_is_alone_distance));
    }

    final protected ISurveyor surveyorImpl() {
        return stateManager.surveyor;
    }

    final protected @NonNull
    ICachedData getCachedData() {
        return stateManager.cachedData;
    }

    SurveyorStationRecord getStation() {
        return stateManager.stationRecord;
    }

    RequestSurveyorParamAuto getRequestSurveyorParamAuto() {
        return stateManager.getRequestSurveyorParamAuto();
    }

    void changeSkip(int cycleIndex, int index) {
        SurveyorStationRecord station = getStation();
        //缓存点
        getCachedData().setPointAbandon(cycleIndex, index);
        //数据库点
        String pointId = getCachedData().originalPointId(cycleIndex, index);
        for (SurveyorPoint surveyorPoint : station.items) {
            if (TextUtils.equals(pointId, surveyorPoint.getOriginalPointId())) {
                surveyorPoint.setAbandon(true);
                break;
            }
        }
        ThreadPoolUtil.execute(() -> {
            SurveyorDatabaseFactory.instance.getSurveyorStation().update(station);
        });
    }

    final protected @NonNull
    ICycleRecorder getCycleRecorder() {
        return stateManager.cycleRecorder;
    }

    final protected @NonNull
    Context getContext() {
        return stateManager.context;
    }

    final protected void scroll2(int itemIndex) {
        stateManager.scrollDelegate.onScrollTo(itemIndex);
    }

    final protected void toast(String warning) {
        ToastUtils.showShort(warning);
    }

    final protected void toast(@StringRes int warning) {
        ToastUtils.showShort(warning);
    }

    public abstract void enter(Bundle args);//进入状态

    final protected void navigation2(StateType stateType) {
        stateManager.enter(stateType);
    }

    final protected void navigation2(StateType stateType, Bundle args) {
        stateManager.enter(stateType, args);
    }

    final boolean isStationClosed() {
        return stateManager.isStationAutoClosed();
    }

    final protected String stationId() {
        return stateManager.stationId();
    }

    final protected FragmentManager fragmentManager() {
        return stateManager.fragmentManager();
    }

    final protected LifecycleOwner lifecycleOwner() {
        return stateManager.lifecycleOwner();
    }

    final protected void tips(String msg, Runnable runnable) {
        if (stateManager.isDead()) {
            return;
        }
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> {
                    if (runnable != null) {
                        runnable.run();
                    }
                });
    }

    /**
     * 错误提示
     * <p>
     * 用户不能忽略，点击确定后自动「停留原地」
     */
    final protected void failureTipsWithAutoStandStill(String msg) {
        if (stateManager.isDead()) {
            return;
        }
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true));
    }

    final protected void failureTipsWithAutoStandStill(String msg, Runnable runnable) {
        if (stateManager.isDead()) {
            return;
        }
        TipsFragment.noCancelAbleTips(stateManager.fragmentManager(), stateManager.lifecycleOwner(),
                getContext(),
                msg,
                -1,
                () -> {
                    runnable.run();
                    enterIdleAndClearSurveyStateAndResetAutoMode(true);
                });
    }

    final protected void optionTipsWithStandStill(String msg, Runnable confirmCallback) {
        optionTips(msg, null, null,
                confirmCallback,
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true),
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true));
    }

    final protected void optionTips(String msg,
                                    String confirmButton,
                                    String cancelButton,
                                    Runnable confirmCallback,
                                    Runnable cancelCallback,
                                    Runnable ignoreCallback) {
        if (stateManager.isDead()) {
            return;
        }
        TipsWith2OptionFragment.tips(
                stateManager.fragmentManager(),
                stateManager.lifecycleOwner(),
                confirmButton,
                cancelButton,
                null,
                msg,
                new TipsWith2OptionFragment.IOperatorCallback() {
                    @Override
                    public void onConfirm() {
                        if (confirmCallback == null) {
                            return;
                        }
                        confirmCallback.run();
                    }

                    @Override
                    public void onCancel() {
                        if (cancelCallback == null) {
                            return;
                        }
                        cancelCallback.run();
                    }

                    @Override
                    public void onIgnore() {
                        if (ignoreCallback == null) {
                            return;
                        }
                        ignoreCallback.run();
                    }

                    @Override
                    public void onOption() {
                    }
                });
    }

    final protected void optionTipsCountDown(String msg,
                                             String confirmButton,
                                             String cancelButton,
                                             Runnable confirmCallback,
                                             Runnable cancelCallback,
                                             Runnable ignoreCallback) {
        if (stateManager.isDead()) {
            return;
        }

        TipsWith2OptionFragment.tipsCountDownTime(
                stateManager.fragmentManager(),
                stateManager.lifecycleOwner(),
                confirmButton,
                cancelButton,
                null,
                msg,
                new TipsWith2OptionFragment.IOperatorCallback() {
                    @Override
                    public void onConfirm() {
                        if (confirmCallback == null) {
                            return;
                        }
                        confirmCallback.run();
                    }

                    @Override
                    public void onCancel() {
                        if (cancelCallback == null) {
                            return;
                        }
                        cancelCallback.run();
                    }

                    @Override
                    public void onIgnore() {
                        if (ignoreCallback == null) {
                            return;
                        }
                        ignoreCallback.run();
                    }

                    @Override
                    public void onOption() {
                    }
                });
    }

    protected final void option4Jump2OtherCycle4Resurvey(int cycleIndex) {
        option4Jump2OtherCycle(cycleIndex, true);
    }

    protected final void option4Jump2OtherEmptyCycle(int cycleIndex) {
        option4Jump2OtherCycle(cycleIndex, false);
    }

    private void option4Jump2OtherCycle(int cycleIndex, boolean resurveyOrEmpty) {
        String msg = Tools.getString(R.string.there_is_empty_data_to_be_tested_hint);
        if (resurveyOrEmpty) {
            msg = Tools.getString(R.string.there_is_supplementary_data_to_be_tested_hint);
        }
        optionTips(Tools.getString(R.string.at) + SurveyorCycleUtil.cycleName(cycleIndex) + msg,
                null,
                null,
                () -> {
                    //切换测回
                    getCycleRecorder().changeCycleIndex(cycleIndex);
                    //数据库存储修改当前测回
                    setCurrentCycle(cycleIndex);
                    //查找测量目标
                    findTargetFromBeginInCurrentCycle(false);
                },
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true),
                () -> enterIdleAndClearSurveyStateAndResetAutoMode(true));
    }

    final protected void runOnUI(Runnable runnable) {
        stateManager.runOnUI(runnable);
    }

    final protected void runOnBackground(Runnable runnable) {
        ThreadPoolUtil.execute(runnable);
    }

    /**
     * 设置测站的测回
     *
     * @param cycle
     */
    final protected void setCurrentCycle(int cycle) {
        getCycleRecorder().changeCycleIndex(cycle);
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getCurrentCycleKey(stationId(), null),
                                String.valueOf(cycle)));
    }

    final protected void setCurrentCycleSync(int cycle) {
        getCycleRecorder().changeCycleIndex(cycle);
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getCurrentCycleKey(stationId(), null),
                                String.valueOf(cycle)));
    }

    final protected void setSingleCycleSchedule(int cycleIndex, String schedule) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getSingleCycleScheduleKey(stationId(), null, cycleIndex),
                                schedule));
    }

    final protected void setSingleCycleScheduleSync(int cycleIndex, String schedule) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getSingleCycleScheduleKey(stationId(), null, cycleIndex),
                                schedule));
    }

    final protected void setAllCycleSchedule(String schedule) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .record(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), null),
                                schedule));
    }

    final protected void setAllCycleScheduleSync(String schedule) {
        SurveyorDatabaseFactory
                .instance
                .getScheduleDataSource()
                .recordSync(
                        new SurveyorScheduleRecord(
                                SurveyorScheduleConstants.getAllCycleScheduleKey(stationId(), null),
                                schedule));
    }

    /**
     * 如果resetAutoSurveyState为true，将重置autoSurvey状态为unknown
     */
    final protected void findTargetFromBeginInCurrentCycle(boolean resetAutoSurveyState) {
        getSurveyorData().clearSurveyStateAndPosition();
        if (resetAutoSurveyState) {
            autoModeRecorder().resetAutoMode();
        }
        navigation2(StateType.doMove2NextValidate);
    }

    final protected void enterIdleAndClearSurveyState() {
        clearSurveyStateAndCloseAutoModeAndStandStill(DEFAULT_AUTO_MODE,
                false, StateType.idle);
    }

    /**
     * 取消测量，会停在当前位置（索引+盘左or盘右），并进入idle
     */
    final protected void enterIdleAndClearSurveyStateAndResetAutoMode(boolean resetAutoMode) {
        clearSurveyStateAndCloseAutoModeAndStandStill(resetAutoMode ? RESET_AUTO_MODE : DEFAULT_AUTO_MODE,
                false, StateType.idle);
    }

    final protected void enterIdleAndClearSurveyorStatePosition() {
        clearSurveyStateAndCloseAutoModeAndStandStill(DEFAULT_AUTO_MODE,
                true, StateType.idle);
    }

    final protected void enterStationCheckAndClearSurveyStateAndResetAutoMode() {
        clearSurveyStateAndCloseAutoModeAndStandStill(RESET_AUTO_MODE,
                false, StateType.checkStation);
    }

    final protected void enterIdleAndClearSurveyStateAndCloseAutoMode() {
        clearSurveyStateAndCloseAutoModeAndStandStill(CLOSE_AUTO_MODE, false, StateType.idle);
    }

    /**
     * @param autoMode
     * @param clearPosition
     * @param stateType
     */
    private void clearSurveyStateAndCloseAutoModeAndStandStill(int autoMode, boolean clearPosition, StateType stateType) {
        //重置或者关闭自动模式
        if (autoMode == RESET_AUTO_MODE) {
            autoModeRecorder().resetAutoMode();
        } else if (autoMode == CLOSE_AUTO_MODE) {
            autoModeRecorder().closeAutoMode();
        }
        //更新测量进度标识
        if (clearPosition) {
            getSurveyorData().clearSurveyStateAndPosition();
        } else {
            getSurveyorData().clearSurveyState();
        }
        //进入某个状态
        navigation2(stateType);
    }


    /**
     * 更新UI
     */
    final protected void refreshUI() {
        runOnUI(() -> getCachedData().notifyDateChange());
    }

    protected void quite() {
    }

    protected void resume() {
    }

    protected boolean isLive() {
        return !stateManager.isDead();
    }

    protected boolean isAutoSurveyOpen() {
        return stateManager.isAutoModeOpen();
    }

    protected boolean isAutoSurveyClosed() {
        return stateManager.isAutoModeClose();
    }

    protected IStateDelegate stateDelegate() {
        return new IStateDelegate() {
            @Override
            public void onOptionTips(
                    String msg, String confirmButton, String cancelButton,
                    Runnable confirmCallback, Runnable cancelCallback, Runnable ignoreCallback) {
                optionTips(
                        msg, confirmButton, cancelButton,
                        confirmCallback, cancelCallback, ignoreCallback);
            }

            @Override
            public void resetSurveyorStateAndKeepPosition() {
                //关闭自动测量
                autoModeRecorder().resetAutoMode();
                //更新测量进度标识
                getSurveyorData().clearSurveyState();
                //进入空闲状态
                navigation2(StateType.idle);
            }

            @Override
            public void resurveyPoints(int cycleIndex, List<String> pointIds) {
                if (cycleIndex == -1) {
                    cycleIndex = getCycleRecorder().currentCycleIndex();
                }

                //更新记录数据，包括将数据置为dirty和切换cycle
                getCachedData().markReSurveyor(stationId(), cycleIndex, pointIds);
                getCycleRecorder().changeCycleIndex(cycleIndex);

                //根据当前状态，查找一个目标
                findTargetFromBeginInCurrentCycle(true);

                //更新数据库标识
                setCurrentCycle(cycleIndex);
                setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
            }

            @Override
            public void deleteSingleCycleData(int cycleIndex) {
                //删除数据
                getCachedData().deleteSurveyorRecord(stationId(), cycleIndex);
                //切换测回--一般不用，但是这里为了保险
                getCycleRecorder().changeCycleIndex(cycleIndex);
                //查找一个新目标开启测量
                findTargetFromBeginInCurrentCycle(true);

                //更新数据库标识
                setCurrentCycle(cycleIndex);
                setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                setSingleCycleSchedule(cycleIndex, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
            }
        };
    }

    protected int getCurrentIndex() {
        return stateManager.surveyorData.currentIndex();
    }

    protected int getCurrentCycleIndex() {
        return stateManager.cycleRecorder.currentCycleIndex();
    }

    protected boolean isAloneDistance() {
        return stateManager.getCachedData().isAloneDistance(stateManager.getCurrentCycle(), getCurrentIndex());
    }

    protected double getInstrumentHeight() {
        return stateManager.getInsturmentHeight();
    }

    protected void refreshGoBackBtnEnable() {
        stateManager.refreshGoBackBtnEnable();
    }

    protected void resetLearnDataTag() {
        getRequestSurveyorParamAuto().resetCodeAutoMeasureCondition();
    }

}
