package com.bingce.controlnetwork.surveyor.state.heighttraverse;

import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.point.PointRecord;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.event.RefreshGoBackButton;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.AutoSurveyorMethod;
import com.bingce.controlapphelper.surveyor.method.FindTargetMethod;
import com.bingce.controlapphelper.surveyor.method.SurveyorAdjustMethod;
import com.bingce.controlapphelper.surveyor.state.IAutoModeSwitcher;
import com.bingce.controlapphelper.surveyor.state.IStateType;
import com.bingce.controlapphelper.surveyor.state.ISurveyState;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyor;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorListener;
import com.bingce.controlapphelper.surveyor.surveyor.ISurveyorRemindListener;
import com.bingce.controlapphelper.surveyor.surveyor.RequestSurveyorParamAuto;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorAutoImpl;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorMockAuto;
import com.bingce.controlapphelper.surveyor.surveyor.SurveyorSingleImpl;
import com.bingce.controlapphelper.surveyor.test.Test;
import com.bingce.controlapphelper.util.DigitalUtil;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.StationDataAdjustDialogFragment;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.surveyor.state.AutoModeType;
import com.bingce.controlnetwork.surveyor.state.IScrollDelegate;
import com.bingce.controlnetwork.surveyor.state.StateConstants;
import com.bingce.coordlib.model.Coordinate;
import com.bingce.totalstation.ControlSurveyData;
import com.bingce.totalstation.ControlSurveyResult;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.VipManager;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import blankj.utilcode.util.ToastUtils;

public class StateManagerHeightTraverse implements IAutoModeSwitcher, ISurveyState, IStateType {
    private final Map<StateType, __BaseState> stateMap = new HashMap<>();
    private StateType currentStateType = StateType.idle;
    private final FragmentManager fragmentManager;
    private final LifecycleOwner lifecycleOwner;
    private final IUIRunner iuiRunner;
    final IScrollDelegate scrollDelegate;

    @NonNull
    SurveyorStationRecord stationRecord;
    final @NonNull
    RequestSurveyorParamAuto requestSurveyorParamAuto;
    final @NonNull
    ToleranceDetailRecord toleranceRecord;
    final @NonNull
    ISurveyorData surveyorData;
    final @NonNull
    ICachedData cachedData;
    final @NonNull
    ICycleRecorder cycleRecorder;
    final @NonNull
    Context context;
    private AutoModeType autoMode = AutoModeType.unknown;
    @NonNull
    final ISurveyor surveyor;
    private final PointRecord stationPointRecord;
    private final int positionFragment;
    final boolean supportAutoSurvey;

    private StateManagerHeightTraverse(
            final int positionFragment,
            final boolean supportAutoSurvey,//是否支持自动测量
            final @NonNull RequestSurveyorParamAuto requestSurveyorParamAuto,
            final @NonNull SurveyorStationRecord stationRecord,
            final @NonNull PointRecord stationPointRecord,
            final @NonNull ToleranceDetailRecord toleranceRecord,
            final @NonNull Context context,
            final @NonNull ISurveyorData surveyorData,
            final @NonNull ICycleRecorder cycleRecorder,
            final @NonNull ICachedData cachedData,
            final @NonNull IScrollDelegate scrollDelegate,
            FragmentManager fragmentManager,
            LifecycleOwner lifecycleOwner,
            IUIRunner iuiRunner) {
        this.positionFragment = positionFragment;
        this.supportAutoSurvey = supportAutoSurvey;

        this.requestSurveyorParamAuto = requestSurveyorParamAuto;
        this.stationRecord = stationRecord;
        this.fragmentManager = fragmentManager;
        this.lifecycleOwner = lifecycleOwner;
        this.iuiRunner = iuiRunner;
        this.scrollDelegate = scrollDelegate;

        this.context = context;
        this.toleranceRecord = toleranceRecord;
        this.surveyorData = surveyorData;
        this.cachedData = cachedData;
        this.cycleRecorder = cycleRecorder;

        this.stationPointRecord = stationPointRecord;

        if (Test.DEBUG) {
            surveyor = new SurveyorMockAuto(StateConstants.VALUE_MEASURE_WHAT_ANGLE_DISTANCE, this.stationRecord.getId(),
                    this.stationRecord.isAutoClose(),
                    toleranceRecord.getCycleCount(),
                    cachedData, surveyorData, cycleRecorder, this, this);
        } else {
            if (supportAutoSurvey) {
                surveyor = new SurveyorAutoImpl(StateConstants.VALUE_MEASURE_WHAT_ANGLE_DISTANCE, this);
            } else {
                surveyor = new SurveyorSingleImpl(StateConstants.VALUE_MEASURE_WHAT_ANGLE_DISTANCE);
            }
        }
        _DoSurveyorState doSurveyorState = new _DoSurveyorState(positionFragment, this, this);
        //注册接收数据的状态
        surveyor.register((ISurveyorListener) doSurveyorState);
        surveyor.register((ISurveyorRemindListener) doSurveyorState);

        stateMap.put(StateType.init, new _Init(positionFragment, this));
        stateMap.put(StateType.doSurveyor, doSurveyorState);
        stateMap.put(StateType.doAutoSurveyor, new _DoAutoSurveyorState(positionFragment, this, this));
        stateMap.put(StateType.SurveyorFailure, new _SurveyorFailureState(positionFragment, this));
        stateMap.put(StateType.doMove2NextValidate, new _DoMove2NextValidateState(positionFragment, this));
        stateMap.put(StateType.doMoveBack, new _DoMoveBackState(positionFragment, this));
        stateMap.put(StateType.checkStation, new _CheckStationState(positionFragment, this));

        stateMap.put(StateType.doDeleteAllCycle, new _DoDeleteAllCycleState(positionFragment, this));
        stateMap.put(StateType.doDeleteCurrentCycle, new _DoDeleteCurrentCycleState(positionFragment, this));
        stateMap.put(StateType.doCalAdjust, new _DoCalAdjustState(positionFragment, this));
        stateMap.put(StateType.idle, new _IdleState(positionFragment, this, this));
        stateMap.put(StateType.doWellMeasureRotationReady, new _DoWellMeasureRotationReadyState(positionFragment, this));
        stateMap.put(StateType.checkTolerance, new _CheckToleranceState(positionFragment, this));

        //监听测站数据变化
        runOnUI(() -> SurveyorDatabaseFactory
                .instance
                .getSurveyorStation()
                .getLiveDataRecord(stationRecord.getId())
                .observe(lifecycleOwner, record -> {
                    if (record == null || !stationRecord.getId().equals(record.getId())) {
                        return;
                    }
                    updateSurveyorStationRecord(record);
                }));
    }

    private void updateSurveyorStationRecord(SurveyorStationRecord record) {
        this.stationRecord = record;
        //更新测点信息
        if (stationRecord.getItems() != null) {
            for (SurveyorPoint item : stationRecord.getItems()) {
                if (item == null || !SurveyorPoint.TYPE_POINT.equals(item.getType())) {
                    //排除非测点
                    continue;
                }
                cachedData.updateSurveyPointParams(item);
            }
        }

        cachedData.notifyDateChange();
    }

    String stationId() {
        return stationRecord.getId();
    }


    @Override
    public void enter(StateType type, Bundle args) {
        if (isDead()) {
            return;
        }
        __BaseState baseState = stateMap.get(type);
        if (baseState != null) {
            this.currentStateType = type;
            runOnUI(() -> baseState.enter(args));
        }
    }

    @Override
    public void enter(StateType type) {
        enter(type, null);
    }

    void runOnUI(Runnable action) {
        if (iuiRunner != null) {
            iuiRunner.runOnUI(action);
        }
    }

    public void doSurveyor() {
        boolean inValidate = isInValidate();

        if (inValidate) {
            return;
        }

        int pointCount = cachedData.pointCount();
        if (pointCount == 0) {
            ToastUtils.showShort(Tools.getString(R.string.no_data_available_yet));
            return;
        }

        Bundle bundle = null;
        if (isRotationReadyState()) {
            //当前是旋转到位状态
            bundle = new Bundle();
            bundle.putBoolean(StateConstants.KEY_IS_ROTATION_READY_STATE, true);
        }

        enter(StateType.doSurveyor, bundle);
    }

    public void doAutoSurveyor() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doAutoSurveyor);
    }

    public void deleteCurrentStationSurveyorData() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doDeleteAllCycle, _DoDeleteAllCycleState.args(stationId()));
    }

    /**
     * 删除当前测回记录（删除后视或前视的所有记录）
     *
     * @param cycleIndex
     */
    public void deleteCurrentStationSurveyorData(int positionFragment) {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doDeleteCurrentCycle, _DoDeleteCurrentCycleState.args(stationId(), positionFragment));
    }

    public void stationCheck() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.checkStation);
    }

    public String stationCheckOnly() {
        return _CheckStationState._stationCheck(stationId(), toleranceRecord, cachedData);
    }

    public void stationAdjustOnly(String showType) {
        ThreadPoolUtil.execute(() -> {
            SurveyorAdjustMethod.AdjustResult adjustResult = SurveyorAdjustMethod.calStationAdjust(stationRecord, cachedData);
            if (adjustResult.isError()) {
                runOnUI(() -> TipsFragment.tipsOnly(fragmentManager, lifecycleOwner, context, adjustResult.errorString));
                return;
            }

            ArrayList<SurveyorAdjustMethod.Item> adjustList = new ArrayList<>();
            if (adjustResult.result != null && !adjustResult.result.isEmpty()) {
                adjustList.add(adjustResult.result.get(0));
            }
            StationDataAdjustDialogFragment.showDialog(showType, adjustList, adjustResult.showSlopDistance, fragmentManager());

            //伴随平差 设置进度为通过检测
            SurveyorDatabaseFactory
                    .instance
                    .getScheduleDataSource()
                    .record(
                            new SurveyorScheduleRecord(
                                    __BaseState.getScheduleKey(positionFragment),
                                    SurveyorScheduleConstants.STATUE_VALUE_CHECKED));
        });

    }


    public void stationAdjust(String type) {
        if (isInValidate()) {
            return;
        }
        Bundle bundle = new Bundle();
        bundle.putString(StationDataAdjustDialogFragment.KEY_SHOW_TYPE, type);
        enter(StateType.doCalAdjust, bundle);
    }

    public void switch2Cycle(int cycleIndex) {
        if (isInValidate()) {
        }
//        _DoCalScheduleInSingleCycleState.enter(this, cycleIndex);
    }

    public void calScheduleWhenEnterSurveyFragment(int cycleIndex) {
        if (isInValidate()) {
            return;
        }
        _Init.enter(this, cycleIndex);
    }

    public void moveBack() {
        if (isInValidate()) {
            return;
        }
        enter(StateType.doMoveBack);
    }

    //检测当前状态是否支持从外界主动改变状态
    //如果是空闲或者是测站通过检测是有效的状态
    public boolean isInValidate() {
        return currentStateType != StateType.idle
                && StateType.stationCheckedSuccess != currentStateType
                && !isRotationReadyState();
    }

    private boolean isRotationReadyState() {
        return currentStateType == StateType.doWellMeasureRotationReady;
    }

    public void quite() {
        _live = false;
        for (Map.Entry<StateType, __BaseState> entry : stateMap.entrySet()) {
            entry.getValue().quite();
        }
        //进入idle状态
        this.currentStateType = StateType.idle;
    }

    private boolean _live = true;

    public boolean isDead() {
        return !_live;
    }

    public void resume() {
        _live = true;
        for (Map.Entry<StateType, __BaseState> entry : stateMap.entrySet()) {
            entry.getValue().resume();
        }
    }

    FragmentManager fragmentManager() {
        return this.fragmentManager;
    }

    LifecycleOwner lifecycleOwner() {
        return this.lifecycleOwner;
    }

    final boolean isStationAutoClosed() {
        return stationRecord.isAutoClose();
    }

    @WorkerThread
    public static StateManagerHeightTraverse newInstance(
            int positionFragment,
            boolean isSupportAutoMode,
            @NonNull RequestSurveyorParamAuto requestSurveyorParamAuto,
            @NonNull SurveyorStationRecord stationRecord,
            @NonNull Context context,
            @NonNull FragmentManager fragmentManager,
            @NonNull LifecycleOwner lifecycleOwner,
            @NonNull ToleranceDetailRecord toleranceRecord,
            @NonNull ISurveyorData surveyorData,
            @NonNull ICycleRecorder cycleRecorder,
            @NonNull ICachedData cachedData,
            IScrollDelegate scrollDelegate,
            IUIRunner runner) {
        //读取测点record
        String stationPointId = null;
        if (stationRecord.getItems() != null) {
            for (SurveyorPoint stationItem : stationRecord.getItems()) {
                if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                    stationPointId = stationItem.getPointId();
                    break;
                }
            }
        }
        PointRecord stationPointRecord = null;
        if (stationPointId != null) {
            stationPointRecord = SurveyorDatabaseFactory
                    .instance
                    .getPointDataSource()
                    .findByIdSync(stationPointId);
        }
        if (stationPointRecord == null) {
            return null;
        }
        boolean supportAuto = Test.DEBUG;
        if (isSupportAutoMode &&
                (VipManager.getInstance().checkReg() || VipManager.getInstance().checkControlProLevel())) {
            supportAuto = true;
        }
        return new StateManagerHeightTraverse(
                positionFragment,
                supportAuto,
                requestSurveyorParamAuto,
                stationRecord,
                stationPointRecord,
                toleranceRecord,
                context,
                surveyorData,
                cycleRecorder,
                cachedData,
                scrollDelegate,
                fragmentManager, lifecycleOwner, runner);
    }

    @Override
    public boolean isAutoModeOpen() {
        return autoMode == AutoModeType.start;
    }

    @Override
    public boolean isAutoModeClose() {
        return autoMode == AutoModeType.stop;
    }

    @Override
    public boolean isAutoModeNotSpecial() {
        return autoMode == AutoModeType.unknown;
    }

    @Override
    public void openAutoMode() {
        autoMode = AutoModeType.start;
    }

    @Override
    public void closeAutoMode() {
        autoMode = AutoModeType.stop;
    }

    @Override
    public void resetAutoMode() {
        autoMode = AutoModeType.unknown;
    }


    @Override
    public int isReadyAutoSurvey() {
        if (!supportAutoSurvey) {
            return ControlSurveyResult.CODE_ERROR;
        }
        if (!surveyor.isReady()) {//仪器连接上才算ready
            return ControlSurveyResult.CODE_ERROR;
        }

        Coordinate stationBasePoint = getStationCoord();
        double instrumentHeight = getInsturmentHeight();

        return requestSurveyorParamAuto.checkAutoCondition(getCachedData(), stationBasePoint, instrumentHeight);
    }

    @Override
    public String isMeasuredValueMatchDesign() {
        if (!supportAutoSurvey) {
            return null;
        }
        Coordinate stationBasePoint = getStationCoord();
        double instrumentHeight = getInsturmentHeight();
        List<ControlSurveyData> originMeasureList = requestSurveyorParamAuto.getOriginMeasureList(getCachedData());
        return AutoSurveyorMethod.checkAutoSurveyorMeasuredValue(stationBasePoint, instrumentHeight, originMeasureList);
    }

    @Override
    public boolean isSurveying() {
        return currentStateType == StateType.doSurveyor || currentStateType == StateType.doAutoSurveyor;
    }

    @Override
    public boolean isAutoSurveying() {
        return isAutoModeOpen();
    }


    @NonNull
    public ICachedData getCachedData() {
        return cachedData;
    }

    /**
     * 获取当前测回
     *
     * @return
     */
    public int getCurrentCycle() {
        return cycleRecorder.currentCycleIndex();
    }

    public int getCurrentIndex() {
        return surveyorData.currentIndex();
    }

    private Coordinate getStationCoord() {
        if (PointRecord.TYPE_KNOWN_POINT.equals(stationPointRecord.getPointType())) {
            return new Coordinate(
                    stationPointRecord.getX(),
                    stationPointRecord.getY(),
                    stationPointRecord.getZ());
        }
        return null;
    }

    public double getInsturmentHeight() {
        return DigitalUtil.valueOfDoubleString(stationRecord.getInstrumentHeight(), 0);
    }

    @NonNull
    public RequestSurveyorParamAuto getRequestSurveyorParamAuto() {
        return requestSurveyorParamAuto;
    }

    public void refreshGoBackBtnEnable() {
        FindTargetMethod.Target deleteTartget = _DoMove2NextValidateState.findTarget(stationId(), cachedData.pointCount(), cachedData,
                surveyorData.currentIndex(), false,
                dataCategory -> !dataCategory.isEmptyData());
        EventBus.getDefault().post(new RefreshGoBackButton(deleteTartget != null));
    }

}
