package com.bingce.controlnetwork.surveyor.state.heighttraverse;

import android.os.Bundle;
import android.text.TextUtils;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.config.ConfigConstants;
import com.bingce.controlapphelper.datasource.database.config.ConfigRecord;
import com.bingce.controlapphelper.datasource.database.config.IConfigDataSource;
import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.surveyorrecord.SurveyorRecord;
import com.bingce.controlapphelper.datasource.database.tolerance.detail.ToleranceDetailRecord;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.data.SpeakCache;
import com.bingce.controlapphelper.surveyor.method.DeviationValueOfRadianMethod;
import com.bingce.controlapphelper.surveyor.method.SingleCycleCheckMethod;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.SurveyorCycleUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.surveyor.state.AutoModeType;
import com.bingce.totalstation.TotalStation;
import com.bingce.utils.ThreadPoolUtil;

import net.tatans.tensorflowtts.tts.TtsManager;

/**
 * 检测限差
 */
public class _CheckToleranceState extends __BaseState {

    public _CheckToleranceState(int positionFragment,
                                StateManagerHeightTraverse stateManager) {
        super(StateType.checkTolerance, positionFragment, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        //检测读数
        if (!checkReadData()) {
            return;
        }

        ICachedData cacheData = getCachedData();
        ISurveyorData surveyorData = getSurveyorData();


        //设置数据正常
        surveyorData.setDataCategory(DataCategory.normalCategory());
        //存储（缓存+数据库）
        cacheData.cache(getCurrentCycleIndex(), surveyorData);

        if (isSingleCycleCompleted(stationId(), getCurrentCycleIndex(), cacheData)) {
            refreshUI();
            //更新测回测量进度
            setSingleCycleSchedule(getCurrentCycleIndex(), SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
            boolean singleSuccess = checkSingleCycle();
            if (singleSuccess) {
                checkAllCycle();
            }
        } else {
            findNextSurveyorPoint();
        }

    }

    private void checkAllCycle() {
        int cycleCount = getToleranceRecord().getCycleCount();
        int nextCycleIndex = -1;
        //查找数据不全的测回
        for (int cycleIndex = 0; cycleIndex < cycleCount; cycleIndex++) {
            if (!getCachedData().isSingleCycleAllDataValidate(cycleIndex)) {
                //对不全数据的测回赋值
                nextCycleIndex = cycleIndex;
                break;
            }
        }

        if (nextCycleIndex != -1) {
            //有其他测回数据没有测完
            if (isAutoSurveyOpen()) {
                //修改当前测回
                setCurrentCycle(nextCycleIndex);
                //存储一下已经播放切换测回的的节点
                SpeakCache.getInstance().saveSpeakSwitchCycle();
                //播报
                TtsManager.getInstance().speak(Tools.getString(R.string.current_cycle_complete_start_surveyor) + SurveyorCycleUtil.cycleName(nextCycleIndex));
                //查找测量目标
                findNextSurveyorPoint();
            } else {
//                //提示是否进入其他未完成测回继续测量
//                option4Jump2OtherEmptyCycle(nextCycleIndex);
                //查找测量目标
                findNextSurveyorPoint();
            }
            return;
        }

        if (isAutoSurveyOpen()) {
            //检测是否已经提示
            //检测是否需要提示测站检测
            IConfigDataSource configDataSource =
                    SurveyorDatabaseFactory.instance.getConfigDataSource();
            ThreadPoolUtil.execute(() -> {
                String key = ConfigConstants.KEY_CONFIG_STATION_CHECK_REMINDED + stationId();
                ConfigRecord record = configDataSource.getByKeySync(key);
                if (record == null || ConfigConstants.VALUE_NO.equals(record.getConfigValue())) {
                    runOnUI(() -> TtsManager.getInstance().speak(Tools.getString(R.string.all_cycle_completed)));
                }
            });
        }
        //标识测站完成测量（数据库存储）
        setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_COMPLETED);
        //进入idle状态
        //清除测量状态，进入测站检测，停止自动测量
        setAutoAndSurveyorState(AutoModeType.unknown, false);

        String error = _CheckStationState._stationCheck(stationId(), getToleranceRecord(), getCachedData());
        if (error.isEmpty()) {
            //进入finish状态
            ISurveyorData surveyorData = getSurveyorData();
            surveyorData.clearSurveyStateAndPosition();
            //设置测量进度(数据库存储)
            setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_CHECKED);
            //通知刷新
            tips(getContext().getString(R.string.tips_all_cycle_has_finish_and_checked), null);
            if (isAutoSurveyOpen()) {
                runOnUI(() -> TtsManager.getInstance().speak(getContext().getString(R.string.tips_all_cycle_has_finish_and_checked)));
            }
            //清除测量状态，进入idle，停止自动测量
            enterIdleAndClearSurveyStateAndResetAutoMode(true);
        } else {
            _CheckStationState.showErrorTips(error, stationId(), getToleranceRecord().getCycleCount(),
                    getCachedData(),
                    this,
                    fragmentManager(),
                    lifecycleOwner());
        }

    }

    private boolean checkSingleCycle() {
        ICachedData cachedData = getCachedData();
        String stationId = stationId();
        ISurveyorData surveyorData = getSurveyorData();
        int currentCycleIndex = surveyorData.currentIndex();
        ToleranceDetailRecord toleranceRecord = getToleranceRecord();
        int cycleCount = toleranceRecord.cycleCount;

        //预处理
        for (int index = 0; index < cachedData.pointCount(); index++) {
            String pointId = cachedData.pointId(currentCycleIndex, index);
            String keyLeft = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.left);
            SurveyorRecord left = cachedData.getSurveyorRecord(keyLeft);
            _changeDirtyRecord2NormalBeforeCheck(left, cachedData);
            String keyRight = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.right);
            SurveyorRecord right = cachedData.getSurveyorRecord(keyRight);
            _changeDirtyRecord2NormalBeforeCheck(right, cachedData);
        }

        //执行单测回内检测
        String error = SingleCycleCheckMethod.checkVerI(
                toleranceRecord.getVerCycleDirectIDifference(),
                stationId(), currentCycleIndex, surveyorData, cachedData);
        if (error != null) {
            if (isAutoSurveyOpen()) {
                TtsManager.getInstance().speak(Tools.getString(R.string.check_cycle_inner_failed));
            }
            _CheckStationState.showErrorTips(error, stationId(), cycleCount, cachedData, this, fragmentManager(), lifecycleOwner());
            return false;
        }

        //更新测量进度
        setSingleCycleSchedule(currentCycleIndex,
                SurveyorScheduleConstants.STATUE_VALUE_CHECKED);

        return true;
    }

    private void _changeDirtyRecord2NormalBeforeCheck(
            SurveyorRecord record, ICachedData cachedDataSource) {
        if (record == null) {
            return;
        }
        if (record.value1Status.isResurveyed() || record.value2Status.isResurveyed()) {
            if (record.value1Status.isResurveyed()) {
                record.value1Status = DataCategory.normalCategory();
            }
            if (record.value2Status.isResurveyed()) {
                record.value2Status = DataCategory.normalCategory();
            }

            cachedDataSource.updateCache(record);
        }
    }

    private boolean checkReadData() {
        int surveyorCount = getToleranceRecord().getSurveyorCount();
        if (surveyorCount > 1) {//单次测量，直接通过检测
            //多次测量，执行检测
            ISurveyorData surveyorDataSource = getSurveyorData();
            double horDifference = getToleranceRecord().getHorReadDifference();
            double verDifference = getToleranceRecord().getVerReadDifference();
            double distanceDifference = getToleranceRecord().getSlopeReadDifference();
            double horDiffRadian = DeviationValueOfRadianMethod.similarRadianDiff(surveyorDataSource.getAngleHor1(), surveyorDataSource.getAngleHor2());
            double verDiffRadian = DeviationValueOfRadianMethod.similarRadianDiff(surveyorDataSource.getAngleVer1(), surveyorDataSource.getAngleVer2());

            String msg = null;
//            if (verDifference < RadianMethod.radian2S(verDiffRadian)) {
//                msg = "垂直角读数超限,请重新观测该目标.";
//            } else
            if (!isAloneDistance()//增加单独测距判断
                    && distanceDifference < distanceReadDiff(surveyorDataSource.getDistance1(), surveyorDataSource.getDistance2())) {
                msg = Tools.getString(R.string.distance_read_data_limit);
            }

            if (!TextUtils.isEmpty(msg)) {
                failureTipsWithAutoStandStill(msg);
                return false;
            }
        }

        return checkLeft();
    }

    /**
     * 如果是测回内首次测量，需要判断是否是盘左
     */
    private boolean checkLeft() {
        LeftOrRight leftOrRight = getSurveyorData().leftOrRight();
        if (leftOrRight == LeftOrRight.left) {
            //如果是首次测量，强制要求使用盘左
            double vAngle = getSurveyorData().getAngleVer();
            if (!isFaceLeft(vAngle)) {
                failureTipsWithAutoStandStill(
                        getContext().getString(R.string.error_tips_please_start_from_left),
                        () -> {
                            TotalStation.getInstance().requestChangeFace();
                        });
                return false;
            }
        }
        return true;
    }

    /**
     * 是否是盘左
     *
     * @param vAngle
     * @return
     */
    private boolean isFaceLeft(double vAngle) {
        return vAngle >= 0 && vAngle <= Math.PI;
    }

    /**
     * 斜距两次读数差
     *
     * @param distance1 读数1
     * @param distance2 读数2
     * @return 读数差(单位毫米)
     */
    public double distanceReadDiff(double distance1, double distance2) {
        return Math.abs(distance1 - distance2) * 1000;
    }

    /**
     * 判断单个测回是否测量完成（不要求通过检测）
     */
    public static boolean isSingleCycleCompleted(String stationId, int cycleIndex, ICachedData cachedData) {
        if (cachedData.pointCount() == 0) {
            return false;
        }
        if (cycleIndex == -1) {
            cycleIndex = 0;
        }

        boolean isCompleted = true;

        //测回内所有数据都是「新鲜数据」才能执行「测回内检测」
        for (int index = 0; index < cachedData.pointCountReal(); index++) {

            if (cachedData.isPointAbandon(cycleIndex, index)) {
                continue;
            }

            String pointId = cachedData.pointId(cycleIndex, index);

            String leftKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, pointId, LeftOrRight.left);
            String rightKey = IdUtil.createSurveyorRecordId(stationId, cycleIndex, pointId, LeftOrRight.right);

            DataCategory leftDataCategory = cachedData.valueState(leftKey);
            DataCategory rightDataCategory = cachedData.valueState(rightKey);

            if (leftDataCategory.isFreshData() && rightDataCategory.isFreshData()) {
                continue;
            }
            isCompleted = false;
            break;
        }

        return isCompleted;
    }

    private void findNextSurveyorPoint() {
        findTargetFromBeginInCurrentCycle(false, 0);
    }

}
