package com.bingce.controlnetwork.surveyor.state.heighttraverse;

import android.os.Bundle;

import androidx.annotation.NonNull;

import com.bingce.controlapphelper.model.BundleConstants;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.method.FindTargetMethod;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;

import net.tatans.tensorflowtts.tts.TtsManager;

/**
 * 移动到一个待测量位置
 * <p>
 * 不会跳转到其他测期
 * <p>
 * 可能涉及转向（盘左->盘右）
 */
class _DoMove2NextValidateState extends __BaseState {

    public _DoMove2NextValidateState(int positionFragment, StateManagerHeightTraverse stateManager) {
        super(StateType.doMove2NextValidate, positionFragment, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        ICachedData cachedData = getCachedData();

        int pointCount = cachedData.pointCount();

        int cycle = 0;
        if (args != null) {
            cycle = args.getInt(BundleConstants.KEY_CYCLE_INDEX, 0);
        }

        //1.先查找补测数据
        FindTargetMethod.Target surveyorTarget = findTarget(stationId(), pointCount, cachedData, cycle, true,
                dataCategory -> DataCategory.dirtyAndNeedReSurveyor == dataCategory);


        //如果在当前测回找到待被测数据，立刻跳转到相应位置
        if (surveyorTarget != null) {
            lookTargetAfter(surveyorTarget.itemIndex, surveyorTarget.leftOrRight);
            return;
        }
        //2.查找empty数据
        FindTargetMethod.Target emptyTarget = findTarget(stationId(), pointCount, cachedData, cycle, true,
                dataCategory -> DataCategory.empty == dataCategory);

        if (emptyTarget != null) {
            if (isAutoSurveyOpen()) {
                if (emptyTarget.itemIndex == getCachedData().pointCount() - 1
                        && emptyTarget.leftOrRight == LeftOrRight.right) {
                    TtsManager.getInstance().speak(Tools.getString(R.string.start_face_right));
                }
            }
            //直接跳转到目标点，开启新测量
            lookTargetAfter(emptyTarget.itemIndex, emptyTarget.leftOrRight);
            return;
        }

        enterIdleAndClearSurveyorStatePosition();

    }

    public static FindTargetMethod.Target findTarget(final String stationId,
                                                     final int pointCount,
                                                     final @NonNull ICachedData cachedData,
                                                     int startLookPosition,
                                                     boolean isAscLook,
                                                     ILookCase lookCase) {
        if (isAscLook) {
            for (int i = startLookPosition; i < pointCount; i++) {
                FindTargetMethod.Target targetAscOrDesc = getTargetAscOrDesc(stationId, i, cachedData, true, lookCase);
                if (targetAscOrDesc != null) {
                    return targetAscOrDesc;
                }
            }
        } else {
            for (int j = startLookPosition; j >= 0; j--) {
                FindTargetMethod.Target targetAscOrDesc = getTargetAscOrDesc(stationId, j, cachedData, false, lookCase);
                if (targetAscOrDesc != null) {
                    return targetAscOrDesc;
                }
            }
        }
        return null;
    }

    public static FindTargetMethod.Target getTargetAscOrDesc(String stationId, int pointIndex, ICachedData cachedData, boolean isAscLook, ILookCase lookCase) {
        FindTargetMethod.Target target;
        if (isAscLook) {
            target = getTarget(stationId, pointIndex, cachedData, LeftOrRight.left, lookCase);
            if (target != null) return target;
            target = getTarget(stationId, pointIndex, cachedData, LeftOrRight.right, lookCase);
        } else {
            target = getTarget(stationId, pointIndex, cachedData, LeftOrRight.right, lookCase);
            if (target != null) return target;
            target = getTarget(stationId, pointIndex, cachedData, LeftOrRight.left, lookCase);
        }
        return target;
    }

    private static FindTargetMethod.Target getTarget(String stationId, int pointIndex, ICachedData cachedData, LeftOrRight leftOrRight, ILookCase lookCase) {
        String pointId = cachedData.pointId(pointIndex, pointIndex);
        String leftKey = IdUtil.createSurveyorRecordId(stationId, pointIndex, pointId, leftOrRight);
        DataCategory dataCategory = cachedData.valueState(leftKey);
        if (lookCase.whichCase(dataCategory)) {
            return new FindTargetMethod.Target(pointIndex, pointIndex, leftOrRight);
        }
        return null;
    }


}
