package com.bingce.controlnetwork.surveyor.state.heighttraverse;

import android.os.Bundle;

import com.bingce.controlapphelper.datasource.database.schedule.SurveyorScheduleConstants;
import com.bingce.controlapphelper.datasource.database.surveyorrecord.SurveyorRecord;
import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.method.FindTargetMethod;
import com.bingce.controlapphelper.surveyor.state.StateType;
import com.bingce.controlapphelper.util.IdUtil;

import com.bingce.controlnetwork.R;

/**
 * 回退
 * 移动到上一测量结点，伴随删除上次测量数据
 */
public class _DoMoveBackState extends __BaseState {

    public _DoMoveBackState(int positionFragment, StateManagerHeightTraverse stateManager) {
        super(StateType.doMoveBack, positionFragment, stateManager);
    }

    @Override
    public void enter(Bundle args) {
        ICachedData cachedData = getCachedData();

        final int pointCount = cachedData.pointCount();

//        if (currentIndex < 0) {
//            //如果测量结束，直接使用0号点盘右
//            int targetIndex = 0;
//            LeftOrRight targetState = LeftOrRight.right;
//            warningWithStandStill(cachedDataSource, currentCycle, targetIndex, targetState);
//            return;
//        }

        FindTargetMethod.Target deleteTartget = _DoMove2NextValidateState.findTarget(stationId(), pointCount, cachedData,
                getCurrentIndex(), false, dataCategory -> !dataCategory.isEmptyData());

        if (deleteTartget == null) {
            goIdle();
            return;
        }

        cachedData.deleteSurveyorRecord(stationId(), deleteTartget.itemIndex, deleteTartget.itemIndex, deleteTartget.leftOrRight);

        setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);


        //直接跳转到目标点，开启新测量
        //更新测量进度
        setSurveyPosition(deleteTartget.itemIndex, deleteTartget.leftOrRight);
        //清除测量状态，进入idle，但是不停止自动测量
        goIdle();
        //滚动
        scroll2(deleteTartget.itemIndex);


//        if (dataCategory.isNeedResurvey()) {//如果当前位置是脏数据（说明正在补测）
//            //回退到上一个脏数据位置
//            back2LastDirtyPosition(currentCycle, currentIndex, leftOrRight, pointCount, cachedDataSource);
//        } else if (dataCategory.isEmptyData()) {
//            //如果是空数据，说明测量进行中，查找上一个数据即可
//            //需要提醒用户数据会被删除
//            back2LastVisiblePosition(currentCycle, currentIndex, leftOrRight, pointCount, cachedDataSource);
//        } else {
//            //bug,不应该有这个地方
//            enterIdleAndClearSurveyStateAndResetAutoMode(true);
//        }
    }

    private void goIdle() {
        enterIdleAndClearSurveyStateAndResetAutoMode(false);
    }

    private void back2LastDirtyPosition(int currentCycle, int currentIndex, LeftOrRight leftOrRight,
                                        int pointCount, ICachedData cachedData) {
        //反向查找，但是找到目标后不能删除其数据，否则会造成「数据空洞」
        FindTargetMethod.Target target = FindTargetMethod.findTargetInSingleCycle(
                stationId(), currentCycle, currentIndex, leftOrRight, pointCount, false,
                DataCategory.dirtyAndSurveyed,
                cachedData);
        if (target == null) {
            //在dirtyAndNeedReSurveyor数据没有修复完成之前，不能回退正常数据
            enterIdleAndClearSurveyStateAndResetAutoMode(true);
        } else {
            //找到dirtyAndSurveyed数据，
            //则将其校正为dirtyAndNeedReSurveyor
            String pointId = cachedData.pointId(target.cycleIndex, target.itemIndex);
            String recordKey = IdUtil.createSurveyorRecordId(stationId(), target.cycleIndex, pointId, target.leftOrRight);
            SurveyorRecord surveyorRecord = cachedData.getSurveyorRecord(recordKey);
            if (surveyorRecord != null) {
                if (surveyorRecord.value1Status.isResurveyed()) {
                    surveyorRecord.value1Status = DataCategory.needResurvey();
                }
                if (surveyorRecord.value2Status.isResurveyed()) {
                    surveyorRecord.value2Status = DataCategory.needResurvey();
                }
                cachedData.updateCache(surveyorRecord);
                //重置进度
                resetSchedule(_DoMoveBackState.this,
                        target.itemIndex,
                        target.leftOrRight);
            } else {
                enterIdleAndClearSurveyStateAndResetAutoMode(true);
            }
        }
    }

    private void back2LastVisiblePosition(int currentCycle, int currentIndex, LeftOrRight leftOrRight,
                                          int pointCount,
                                          ICachedData cachedData) {

        //从当前开始，向上查找第一个normal数据
        FindTargetMethod.Target target =
                FindTargetMethod.findTargetInSingleCycle(
                        stationId(), currentCycle, currentIndex, leftOrRight, pointCount, false,
                        DataCategory.normalCategory(),
                        cachedData);
        if (target == null) {
            failureTipsWithAutoStandStill(getContext().getString(R.string.tips_no_more_record_2_rollback));
            return;
        }

        warningWithStandStill(cachedData,
                currentCycle,
                target.itemIndex,
                target.leftOrRight);
    }

    private void warningWithStandStill(
            ICachedData cachedDataSource,
            int currentCycle, int itemIndex, LeftOrRight leftOrRight) {
        optionTipsWithStandStill(getContext().getString(R.string.tips_will_resurvey_last_record),
                () -> {
                    //删除当前数据
                    cachedDataSource.deleteSurveyorRecord(stationId(),
                            currentCycle, itemIndex, leftOrRight);
                    //重置进度
                    setSingleCycleSchedule(currentCycle, SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);
                    setAllCycleSchedule(SurveyorScheduleConstants.STATUE_VALUE_NO_COMPLETED);

                    resetSchedule(this, itemIndex, leftOrRight);
                });
    }

    private static void resetSchedule(__BaseState state,
                                      int targetIndex,
                                      LeftOrRight leftOrRight) {
        //设置测量目标点
        state.setSurveyPosition(targetIndex, leftOrRight);
        //清除测量状态，进入idle
        //如果当前已经关闭自动测量，就不重置自动测量开关（用户已经表示不想自动测量）
        state.enterIdleAndClearSurveyStateAndResetAutoMode(!state.isAutoSurveyClosed());

        state.scroll2(targetIndex);
    }
}
