package com.bingce.controlnetwork.ui.vh;

import android.content.Context;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.datasource.database.alonedistance.AloneDistanceRecord;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.CachedDataAloneDistance;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.Util;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class SurveyorDataAloneDistanceViewHolder extends RecyclerView.ViewHolder {
    private final Context context;
    private final TextView leftLabel;//平距或斜距

    private final TextView pointName;
    private final TextView heightValue;
    private final TextView prismValue;

    private final List<TextView> distanceViews;
    private final TextView tvFirstRound1, tvFirstRound2, tvFirstRound3, tvFirstRound4;
    private final TextView tvSecondRound1, tvSecondRound2, tvSecondRound3, tvSecondRound4;

    private final TextView tvFirstRoundAverageDistance, tvSecondRoundAverageDistance;//第一,第二测回中值
    private final TextView tvFirstRound2D, tvSecondRound2D;//第一,第二测回2D

    private final View selectMask;//黄框

    private final int ROUND_NUM = 2;
    private final int EVERY_ROUND_NUM = 4;

    private final List<Double> realDistanceValues = new ArrayList<>();

    public SurveyorDataAloneDistanceViewHolder(Context context, @NonNull @NotNull View itemView) {
        super(itemView);
        this.context = context;

        pointName = itemView.findViewById(R.id.pointName);
        heightValue = itemView.findViewById(R.id.height_value);
        prismValue = itemView.findViewById(R.id.prism_value);

        leftLabel = itemView.findViewById(R.id.left_label);

        tvFirstRound1 = itemView.findViewById(R.id.tvFirstRound1);
        tvFirstRound2 = itemView.findViewById(R.id.tvFirstRound2);
        tvFirstRound3 = itemView.findViewById(R.id.tvFirstRound3);
        tvFirstRound4 = itemView.findViewById(R.id.tvFirstRound4);
        tvSecondRound1 = itemView.findViewById(R.id.tvSecondRound1);
        tvSecondRound2 = itemView.findViewById(R.id.tvSecondRound2);
        tvSecondRound3 = itemView.findViewById(R.id.tvSecondRound3);
        tvSecondRound4 = itemView.findViewById(R.id.tvSecondRound4);

        distanceViews = new ArrayList<>();
        distanceViews.add(tvFirstRound1);
        distanceViews.add(tvFirstRound2);
        distanceViews.add(tvFirstRound3);
        distanceViews.add(tvFirstRound4);
        distanceViews.add(tvSecondRound1);
        distanceViews.add(tvSecondRound2);
        distanceViews.add(tvSecondRound3);
        distanceViews.add(tvSecondRound4);


        tvFirstRoundAverageDistance = itemView.findViewById(R.id.tvFirstRoundAverageDistance);
        tvSecondRoundAverageDistance = itemView.findViewById(R.id.tvSecondRoundAverageDistance);
        tvFirstRound2D = itemView.findViewById(R.id.tvFirstRound2D);
        tvSecondRound2D = itemView.findViewById(R.id.tvSecondRound2D);

        selectMask = itemView.findViewById(R.id.mask);
    }

    public void setup(int itemIndex,
                      String stationId,
                      boolean showSlopDistance,
                      CachedDataAloneDistance cachedData,
                      ICycleRecorder cycleRecorder,
                      ISurveyorData surveyorData) {
        if (showSlopDistance) {
            leftLabel.setText(R.string.distance_slope);
        } else {
            leftLabel.setText(R.string.distance_horizontal);
        }
        int currentCycleIndex = cycleRecorder.currentCycleIndex();
        int currentSurveyorIndex = surveyorData.currentIndex();
        //设置当前测量状态标识
        if (currentSurveyorIndex == itemIndex) {
            selectMask.setVisibility(View.VISIBLE);
        } else {
            selectMask.setVisibility(View.GONE);
        }

        //初始化
        initDefault();

        //设置值
        pointName.setText(cachedData.pointName(currentCycleIndex, itemIndex));
        heightValue.setText(cachedData.getPrismHeight(currentCycleIndex, itemIndex));
        prismValue.setText(cachedData.getPrismTypeName(currentCycleIndex, itemIndex));

        //先清空下realDistanceValues保证数据不错乱,注意位置在前面
        clearRealDistanceValues();
        List<Double> distanceDatas = getDistanceDataFromCacheData(cachedData, stationId, currentCycleIndex, itemIndex);
        setDistanceViewData(distanceDatas, currentSurveyorIndex == itemIndex);

        //设置中值和2d
        if (!realDistanceValues.isEmpty()) {
            int distanceSize = realDistanceValues.size();
            int hanNum = distanceSize / EVERY_ROUND_NUM;
            if (hanNum >= 1) {
                //有可以显示的中值
                if (hanNum == ROUND_NUM) {
                    //当满足两测回都可以显示中值和2d的时候
                    setFirstRound2DAndAverageDistance();
                    setSecondRound2DAndAverageDistance();
                } else {
                    //当满足第一测回可以显示中值和2d的时候
                    setFirstRound2DAndAverageDistance();
                }
            }
        }

    }

    /**
     * 从缓存数据中读取距离数据
     *
     * @param cachedData
     * @param stationId
     * @param currentCycleIndex
     * @param itemIndex
     * @return
     */
    private List<Double> getDistanceDataFromCacheData(CachedDataAloneDistance cachedData, String stationId, int currentCycleIndex, int itemIndex) {
        List<Double> distanceValues = null;
        String pointId = cachedData.pointId(currentCycleIndex, itemIndex);
        String recordId = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.left);
        AloneDistanceRecord aloneDistanceRecord = cachedData.getAloneDistanceRecord(recordId);
        if (aloneDistanceRecord != null) {
            distanceValues = aloneDistanceRecord.distanceValues;
        }
        return distanceValues;
    }

    private void setFirstRound2DAndAverageDistance() {
        List<Double> doubles1 = realDistanceValues.subList(0, EVERY_ROUND_NUM);
        Double averageFirstRound = Tools.calculateCollectionDoubleAverage(doubles1);
        Double maxSubMinFirstRound = Tools.calculateCollectionMaxSubMin(doubles1);
        setAverageDistanceText(tvFirstRoundAverageDistance, averageFirstRound);
        set2DText(tvFirstRound2D, maxSubMinFirstRound);
    }

    private void setSecondRound2DAndAverageDistance() {
        List<Double> doubles2 = realDistanceValues.subList(EVERY_ROUND_NUM, realDistanceValues.size());
        Double averageSecondRound = Tools.calculateCollectionDoubleAverage(doubles2);
        Double maxSubMinSecondRound = Tools.calculateCollectionMaxSubMin(doubles2);
        setAverageDistanceText(tvSecondRoundAverageDistance, averageSecondRound);
        set2DText(tvSecondRound2D, maxSubMinSecondRound);
    }


    private void setTextColor(TextView tv, int value) {
        tv.setTextColor(context.getResources().getColor(value));
    }

    private void setDistance(TextView tv, double distance) {
        String distanceS;
        if (distance == 0) {
            distanceS = ORIGINAL_VALUE;
        } else {
            distanceS = Util.formatDouble2String(distance, SettingValueUtil.getLengthDecimalPoint());
        }
        tv.setText(distanceS);
    }

    private void set2DText(TextView tv, double value) {
        tv.setText(String.format("%smm", Util.formatDouble2String(value * 1000, 1)));
    }

    private void setAverageDistanceText(TextView tv, double value) {
        tv.setText(Util.formatDouble2String(value, SettingValueUtil.getLengthDecimalPoint()));
    }

    /**
     * 初始化
     */
    private void initDefault() {
        leftLabel.setTextColor(context.getResources().getColor(R.color.value_1_normal));

        initDefaultDistanceViewsData();

        tvFirstRoundAverageDistance.setText(ORIGINAL_VALUE);
        tvSecondRoundAverageDistance.setText(ORIGINAL_VALUE);

        tvFirstRound2D.setText(ORIGINAL_VALUE);
        tvSecondRound2D.setText(ORIGINAL_VALUE);
    }

    private void initDefaultDistanceViewsData() {
        for (TextView textView : distanceViews) {
            setDistance(textView, 0);
            setTextColor(textView, R.color.value_1_normal);
        }
    }

    /**
     * 设置view的数据
     *
     * @param aloneDistanceValues
     * @param isSelected
     */
    private void setDistanceViewData(List<Double> aloneDistanceValues, boolean isSelected) {
        if (aloneDistanceValues == null) return;
        for (int i = 0; i < aloneDistanceValues.size(); i++) {
            Double aDouble = aloneDistanceValues.get(i);
            TextView textView = distanceViews.get(i);
            setDistance(textView, aDouble);
            setTextColor(textView, isSelected ? R.color.value_1_selected : R.color.value_1_normal);
            //存一下
            realDistanceValues.add(aDouble);
        }
    }

    private void clearRealDistanceValues() {
        realDistanceValues.clear();
    }


    private static final String ORIGINAL_VALUE = "-";
}
