package com.bingce.controlnetwork.ui.vh;

import android.content.Context;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlapphelper.model.DataCategory;
import com.bingce.controlapphelper.model.LeftOrRight;
import com.bingce.controlapphelper.surveyor.data.ICachedData;
import com.bingce.controlapphelper.surveyor.data.ICycleRecorder;
import com.bingce.controlapphelper.surveyor.data.ISurveyorData;
import com.bingce.controlapphelper.surveyor.method.AverageValueMethod;
import com.bingce.controlapphelper.surveyor.method.RadianMethod;
import com.bingce.controlapphelper.surveyor.method.ValueOf2cMethod;
import com.bingce.controlapphelper.surveyor.method.model.CacheResult;
import com.bingce.controlapphelper.surveyor.method.model.SdAngleValueType;
import com.bingce.controlapphelper.util.IdUtil;
import com.bingce.controlapphelper.util.SettingValueUtil;
import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.utils.Util;

import org.jetbrains.annotations.NotNull;

public class SurveyorDataViewHolderHeightTraverse extends RecyclerView.ViewHolder {
    private final Context context;
    private final TextView leftLabel;
    private final TextView rightLabel;

    public final TextView pointName;
    public final TextView heightValue;
    public final TextView prismValue;

    public final TextView leftAngleVer1;
    public final TextView leftAngleVer2;
    public final TextView leftDistance1;
    public final TextView leftDistance2;

    public final TextView rightAngleVer1;
    public final TextView rightAngleVer2;
    public final TextView rightDistance1;
    public final TextView rightDistance2;

    public final TextView averageVerAngle;
    public final TextView averageDistance;

    public final View selectMask;

    public final TextView _i;
    public final TextView _2d;

    public final TextView distanceTitle;
    public final TextView tvCycleNum;
    private final View itemView;

    public SurveyorDataViewHolderHeightTraverse(Context context, @NonNull @NotNull View itemView) {
        super(itemView);
        this.context = context;
        leftLabel = itemView.findViewById(R.id.left_label);
        rightLabel = itemView.findViewById(R.id.right_label);

        pointName = itemView.findViewById(R.id.pointName);
        heightValue = itemView.findViewById(R.id.height_value);
        prismValue = itemView.findViewById(R.id.prism_value);

        leftAngleVer1 = itemView.findViewById(R.id.left_angle_ver_1);
        leftAngleVer2 = itemView.findViewById(R.id.left_angle_ver_2);
        leftDistance1 = itemView.findViewById(R.id.left_distance_1);
        leftDistance2 = itemView.findViewById(R.id.left_distance_2);

        rightAngleVer1 = itemView.findViewById(R.id.right_angle_ver_1);
        rightAngleVer2 = itemView.findViewById(R.id.right_angle_ver_2);
        rightDistance1 = itemView.findViewById(R.id.right_distance_1);
        rightDistance2 = itemView.findViewById(R.id.right_distance_2);

        averageVerAngle = itemView.findViewById(R.id.average_angle_ver);
        averageDistance = itemView.findViewById(R.id.average_distance);

        selectMask = itemView.findViewById(R.id.mask);

        _i = itemView.findViewById(R.id.value_i);
        _2d = itemView.findViewById(R.id.value_2d);

        distanceTitle = itemView.findViewById(R.id.distance_title);
        tvCycleNum = itemView.findViewById(R.id.tvCycleNum);

        this.itemView = itemView;
    }

    public void setup(int itemIndex,
                      String stationId,
                      boolean showSlopDistance,
                      ICachedData cachedData,
                      ICycleRecorder cycleRecorder,
                      ISurveyorData surveyorData) {
        if (showSlopDistance) {
            distanceTitle.setText(R.string.distance_slope);
        } else {
            distanceTitle.setText(R.string.distance_horizontal);
        }
//        int currentCycleIndex = cycleRecorder.currentCycleIndex();
        int currentCycleIndex = itemIndex;
        int currentSurveyorIndex = surveyorData.currentIndex();
        //设置当前测量状态标识
        if (currentSurveyorIndex == itemIndex) {
            //当前非跳过点背景黄边白色
            selectMask.setBackground(Tools.getDrawable(R.drawable.select_mask));
        } else {
            if (cachedData.isPointAbandon(currentCycleIndex, itemIndex)) {
                selectMask.setBackground(Tools.getDrawable(R.drawable.bg_gray_3));
            } else {
                selectMask.setBackgroundColor(Tools.getColor(R.color.white));
            }
        }
        tvCycleNum.setText(Tools.getString(R.string.test_n_cycle, (itemIndex + 1)));

        //初始化
        setValue(TYPE_LEFT1, ORIGINAL_VALUE, ORIGINAL_VALUE, ORIGINAL_VALUE);
        setValue(TYPE_LEFT2, ORIGINAL_VALUE, ORIGINAL_VALUE, ORIGINAL_VALUE);
        setValue(TYPE_RIGHT1, ORIGINAL_VALUE, ORIGINAL_VALUE, ORIGINAL_VALUE);
        setValue(TYPE_RIGHT2, ORIGINAL_VALUE, ORIGINAL_VALUE, ORIGINAL_VALUE);
        leftLabel.setTextColor(context.getResources().getColor(R.color.value_1_normal));
        setTextColor(TYPE_LEFT1, R.color.value_1_normal);
        setTextColor(TYPE_LEFT2, R.color.value_2_normal);
        rightLabel.setTextColor(context.getResources().getColor(R.color.value_1_normal));
        setTextColor(TYPE_RIGHT1, R.color.value_1_normal);
        setTextColor(TYPE_RIGHT2, R.color.value_2_normal);

        averageVerAngle.setText(ORIGINAL_VALUE);
        averageDistance.setText(ORIGINAL_VALUE);

        _i.setText(ORIGINAL_VALUE);
        _2d.setText(ORIGINAL_VALUE);

        boolean aloneDistance = cachedData.isAloneDistance(currentCycleIndex, itemIndex);

        //如果当前正在测量，使用缓存数据设置相应信息
        LeftOrRight surveyorState = surveyorData.leftOrRight();
        if (currentSurveyorIndex == itemIndex) {
            if (surveyorState == LeftOrRight.left) {
                if (surveyorData.hasSurveyor1()) {
                    setValue(aloneDistance, TYPE_LEFT1, surveyorData.getAngleHor1(), surveyorData.getAngleVer1(), surveyorData.getDistance1());
                }
                if (surveyorData.hasSurveyor2()) {
                    setValue(aloneDistance, TYPE_LEFT2, surveyorData.getAngleHor2(), surveyorData.getAngleVer2(), surveyorData.getDistance2());
                }
                //设置颜色
                setTextColor(TYPE_LEFT1, R.color.value_1_selected);
                setTextColor(TYPE_LEFT2, R.color.value_2_selected);
                leftLabel.setTextColor(context.getResources().getColor(R.color.value_1_selected));
            } else if (surveyorState == LeftOrRight.right) {
                if (surveyorData.hasSurveyor1()) {
                    setValue(aloneDistance, TYPE_RIGHT1, surveyorData.getAngleHor1(), surveyorData.getAngleVer1(), surveyorData.getDistance1());
                }
                if (surveyorData.hasSurveyor2()) {
                    setValue(aloneDistance, TYPE_RIGHT2, surveyorData.getAngleHor2(), surveyorData.getAngleVer2(), surveyorData.getDistance2());
                }
                //设置颜色
                setTextColor(TYPE_RIGHT1, R.color.value_1_selected);
                setTextColor(TYPE_RIGHT2, R.color.value_2_selected);
                rightLabel.setTextColor(context.getResources().getColor(R.color.value_1_selected));
            }
        }

        //根据测量次数，控制数据显示
        setValue2Visibility(surveyorData.surveyorCount() > 1);

        pointName.setText(cachedData.pointName(currentCycleIndex, itemIndex));
        heightValue.setText(cachedData.getPrismHeight(currentCycleIndex, itemIndex));
        prismValue.setText(cachedData.getPrismTypeName(currentCycleIndex, itemIndex));

        //重新监听observe---但是根据当前测量状态，需要排除一些数据监听
        boolean skipLeft1 = false, skipLeft2 = false, skipRight1 = false, skipRight2 = false;
        //根据内存缓存排除
        if (currentSurveyorIndex == itemIndex) {
            if (surveyorState == LeftOrRight.left) {
                if (surveyorData.hasSurveyor1()) {
                    skipLeft1 = true;
                }
                if (surveyorData.hasSurveyor2() || surveyorData.surveyorCount() == 1) {
                    skipLeft2 = true;
                }
            } else if (surveyorState == LeftOrRight.right) {
                if (surveyorData.hasSurveyor1()) {
                    skipRight1 = true;
                }
                if (surveyorData.hasSurveyor2() || surveyorData.surveyorCount() == 1) {
                    skipRight2 = true;
                }
            }
        }

        String pointId = cachedData.pointId(currentCycleIndex, itemIndex);
        String leftRecordId = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.left);
        if (!skipLeft1) {
            DataCategory dataCategory = cachedData.valueState(leftRecordId, 0);
            if (!dataCategory.isEmptyData()) {
                setValue(aloneDistance, TYPE_LEFT1,
                        cachedData.angleHor1(leftRecordId),
                        cachedData.angleVer1(leftRecordId),
                        cachedData.distance1(leftRecordId));
            }
            if (dataCategory.isNeedResurvey()) {
                setTextColor(TYPE_LEFT1, R.color.value_dirty);
            } else if (dataCategory == DataCategory.dirtyAndSurveyed) {
                setTextColor(TYPE_LEFT1, R.color.value_resurveyed);
            }
        }
        if (!skipLeft2) {
            DataCategory dataCategory = cachedData.valueState(leftRecordId, 1);
            if (!dataCategory.isEmptyData()) {
                setValue(aloneDistance, TYPE_LEFT2,
                        cachedData.angleHor2(leftRecordId),
                        cachedData.angleVer2(leftRecordId),
                        cachedData.distance2(leftRecordId));
            }
            if (dataCategory.isNeedResurvey()) {
                setTextColor(TYPE_LEFT2, R.color.value_dirty);
            } else if (dataCategory == DataCategory.dirtyAndSurveyed) {
                setTextColor(TYPE_LEFT2, R.color.value_resurveyed);
            }
        }
        String rightRecordId = IdUtil.createSurveyorRecordId(stationId, currentCycleIndex, pointId, LeftOrRight.right);
        if (!skipRight1) {
            DataCategory dataCategory = cachedData.valueState(rightRecordId, 0);
            if (!dataCategory.isEmptyData()) {
                setValue(aloneDistance, TYPE_RIGHT1,
                        cachedData.angleHor1(rightRecordId),
                        cachedData.angleVer1(rightRecordId),
                        cachedData.distance1(rightRecordId));
            }
            if (dataCategory.isNeedResurvey()) {
                setTextColor(TYPE_RIGHT1, R.color.value_dirty);
            } else if (dataCategory == DataCategory.dirtyAndSurveyed) {
                setTextColor(TYPE_RIGHT1, R.color.value_resurveyed);
            }
        }
        if (!skipRight2) {
            DataCategory dataCategory = cachedData.valueState(rightRecordId, 1);
            if (!dataCategory.isEmptyData()) {
                setValue(aloneDistance, TYPE_RIGHT2,
                        cachedData.angleHor2(rightRecordId),
                        cachedData.angleVer2(rightRecordId),
                        cachedData.distance2(rightRecordId));
            }
            if (dataCategory.isNeedResurvey()) {
                setTextColor(TYPE_RIGHT2, R.color.value_dirty);
            } else if (dataCategory == DataCategory.dirtyAndSurveyed) {
                setTextColor(TYPE_RIGHT2, R.color.value_resurveyed);
            }
        }


        //均值（中值）
        int surveyorCount = surveyorData.surveyorCount();

        CacheResult verAngleLeftAndRightAverageValue = AverageValueMethod.leftAndRightValue(SdAngleValueType.verAngle,
                leftRecordId,
                rightRecordId, surveyorCount, cachedData);
        if (verAngleLeftAndRightAverageValue.validate) {//显示天顶距
            averageVerAngle.setText(RadianMethod.dmsStringOf(Math.PI / 2 - verAngleLeftAndRightAverageValue.value));
        } else {
            averageVerAngle.setText(ORIGINAL_VALUE);
        }

        CacheResult distanceLeftAndRightAverageValue = AverageValueMethod.leftAndRightValue(SdAngleValueType.distance,
                leftRecordId,
                rightRecordId, surveyorCount, cachedData);
        if (aloneDistance) {
            averageDistance.setText(ORIGINAL_VALUE);
        } else {
            if (distanceLeftAndRightAverageValue.validate) {
                averageDistance.setText(Util.formatDouble2String(distanceLeftAndRightAverageValue.value,
                        SettingValueUtil.getLengthDecimalPoint()));
            } else {
                averageDistance.setText(ORIGINAL_VALUE);
            }
        }

        //设置2c、i、2d
        CacheResult valueI = ValueOf2cMethod.valueOf2C(SdAngleValueType.verAngle,
                itemIndex, stationId, surveyorData, currentCycleIndex, cachedData);
        if (valueI.validate && verAngleLeftAndRightAverageValue.validate) {
            _i.setText(String.format("%s″", Util.formatDouble2String(valueI.value, 1)));
        } else {
            _i.setText(ORIGINAL_VALUE);
        }

        CacheResult value2D = ValueOf2cMethod.valueOf2C(SdAngleValueType.distance,
                itemIndex, stationId, surveyorData, currentCycleIndex, cachedData);
        if (aloneDistance) {
            _2d.setText(ORIGINAL_VALUE);
        } else {
            if (value2D.validate && distanceLeftAndRightAverageValue.validate) {
                _2d.setText(String.format("%smm", Util.formatDouble2String(value2D.value, 1)));
            } else {
                _2d.setText(ORIGINAL_VALUE);
            }
        }

    }

    private void setTextColor(int type, int value) {
        angleVer(type).setTextColor(context.getResources().getColor(value));
        distance(type).setTextColor(context.getResources().getColor(value));
    }

    private void setValue(boolean aloneDistance, int type, double angleHor, double angleVer, double distance) {
        //判断是否测水平角
        String angleVerS = RadianMethod.dmsStringOf(angleVer);
        String distanceS = ORIGINAL_VALUE;
        if (!aloneDistance) {
            distanceS = Util.formatDouble2String(distance,
                    SettingValueUtil.getLengthDecimalPoint());
        }
        setValue(type,
                RadianMethod.dmsStringOf(angleHor),
                angleVerS,
                distanceS);
    }

    private void setValue(int type, String angleHor, String angleVer, String distance) {
        angleVer(type).setText(angleVer);
        distance(type).setText(distance);
    }

    private void setValue2Visibility(boolean visibility) {
        leftAngleVer2.setVisibility(visibility ? View.VISIBLE : View.GONE);
        leftDistance2.setVisibility(visibility ? View.VISIBLE : View.GONE);
        rightAngleVer2.setVisibility(visibility ? View.VISIBLE : View.GONE);
        rightDistance2.setVisibility(visibility ? View.VISIBLE : View.GONE);
    }

    private TextView angleVer(int type) {
        if (TYPE_LEFT1 == type) {
            return leftAngleVer1;
        } else if (TYPE_LEFT2 == type) {
            return leftAngleVer2;
        } else if (TYPE_RIGHT1 == type) {
            return rightAngleVer1;
        }
        return rightAngleVer2;
    }


    private TextView distance(int type) {
        if (TYPE_LEFT1 == type) {
            return leftDistance1;
        } else if (TYPE_LEFT2 == type) {
            return leftDistance2;
        } else if (TYPE_RIGHT1 == type) {
            return rightDistance1;
        }
        return rightDistance2;
    }

    private static final int TYPE_LEFT1 = 1;
    private static final int TYPE_LEFT2 = 2;
    private static final int TYPE_RIGHT1 = 3;
    private static final int TYPE_RIGHT2 = 4;
    private static final String ORIGINAL_VALUE = "-";
    private static final String SKIP_VALUE = "*";
}
