package com.bingce.controlnetwork.ui.widget;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bingce.controlnetwork.R;

public class TextItem extends FrameLayout {
    public final static int GRAVITY_TEXT_START = 0;
    public final static int GRAVITY_TEXT_CENTER = 1;
    public final static int GRAVITY_TEXT_END = 2;

    public final static int DIVIDER_VISIBLE = 0;
    public final static int DIVIDER_INVISIBLE = 1;

    public TextItem(@NonNull Context context) {
        super(context);
        init(context, null);
    }

    public TextItem(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs);
    }

    public TextItem(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs);
    }

    public TextItem(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(context, attrs);
    }

    protected int contentLayout() {
        return R.layout.item_with_text_layout;
    }

    void init(Context context, AttributeSet attrs) {
        String string = "TEXT";
        int color = Color.BLACK;
        int size = -1;
        int gravity = GRAVITY_TEXT_START;
        int divider = DIVIDER_VISIBLE;
        if (attrs != null) {
            final TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TextItem);
            string = a.getString(R.styleable.TextItem_text);
            color = a.getColor(R.styleable.TextItem_text_color, Color.BLACK);
            size = a.getDimensionPixelSize(R.styleable.TextItem_text_size, -1);
            gravity = a.getInt(R.styleable.TextItem_text_gravity, GRAVITY_TEXT_START);
            divider = a.getInt(R.styleable.TextItem_dividerVisible, DIVIDER_VISIBLE);
            a.recycle();
        }
        View content = LayoutInflater.from(context).inflate(contentLayout(), null, false);
        addView(content);
        View text = content.findViewById(R.id.text);
        if (text instanceof TextView) {
            TextView textView = (TextView) text;
            textView.setText(string);
            textView.setTextColor(color);
            if (size > 0) {
                textView.setTextSize(TypedValue.COMPLEX_UNIT_PX, size);
            }
            switch (gravity) {
                case GRAVITY_TEXT_CENTER:
                    textView.setGravity(Gravity.CENTER);
                    break;
                case GRAVITY_TEXT_END:
                    textView.setGravity(Gravity.END | Gravity.CENTER_VERTICAL);
                    break;
                default:
                    textView.setGravity(Gravity.START | Gravity.CENTER_VERTICAL);
                    break;
            }
        }
        View dividerView = content.findViewById(R.id.divider);
        if (dividerView != null) {
            dividerView.setVisibility(divider == DIVIDER_VISIBLE ? VISIBLE : GONE);
        }
    }

    public void setText(String string) {
        View text = findViewById(R.id.text);
        if (text instanceof TextView) {
            ((TextView) text).setText(string);
        }
    }
}
