package com.bingce.controlnetwork.ui.widget.recyclerview;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.RecyclerView;

import org.jetbrains.annotations.NotNull;

public class ItemTouchHelperCallbackImpl extends ItemTouchHelper.Callback {
    private final IDragSwipeActionListener listener;
    private final IDragSwipeChecker dragSwipeChecker;

    public ItemTouchHelperCallbackImpl(IDragSwipeChecker dragSwipeChecker,
                                       IDragSwipeActionListener listener) {
        this.dragSwipeChecker = dragSwipeChecker;
        this.listener = listener;
    }

    @Override
    public int getMovementFlags(@NonNull @NotNull RecyclerView recyclerView, @NonNull @NotNull RecyclerView.ViewHolder viewHolder) {
        int dragFlag;    //拖动标记
        int swipeFlags;    //滑动标记
        //如果是表格布局，则可以上下左右的拖动，但是不能滑动
        if (recyclerView.getLayoutManager() instanceof GridLayoutManager) {
            dragFlag = ItemTouchHelper.UP |
                    ItemTouchHelper.DOWN |
                    ItemTouchHelper.LEFT |
                    ItemTouchHelper.RIGHT;
            swipeFlags = 0;
        } else {//如果是线性布局，那么只能上下拖动，只能左右滑动
            dragFlag = ItemTouchHelper.UP | ItemTouchHelper.DOWN;
            swipeFlags = ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT;
        }

        int position = viewHolder.getAdapterPosition();
        if (dragSwipeChecker != null && !dragSwipeChecker.isItemDragAble(position)) {
            dragFlag = 0;
        }
        if (dragSwipeChecker != null && !dragSwipeChecker.isItemSwipeAble(position)) {
            swipeFlags = 0;
        }

        //通过makeMovementFlags生成最终结果
        return makeMovementFlags(dragFlag, swipeFlags);
    }


    @Override
    public boolean onMove(@NonNull RecyclerView recyclerView, @NonNull RecyclerView.ViewHolder viewHolder, @NonNull RecyclerView.ViewHolder target) {
        //被拖动的item位置
        int fromPosition = viewHolder.getLayoutPosition();
        //他的目标位置
        int targetPosition = target.getLayoutPosition();
        if (!isDragValidate(recyclerView, targetPosition, fromPosition)) {
            return false;
        }
        if (listener != null) {
            listener.onItemDragging(fromPosition, targetPosition);
        }
        return true;
    }

    final protected boolean isDragValidate(@NonNull RecyclerView recyclerView, int targetPosition, int fromPosition) {
        RecyclerView.Adapter<?> adapter = recyclerView.getAdapter();
        if (adapter != null) {
            int targetType = adapter.getItemViewType(targetPosition);
            int fromType = adapter.getItemViewType(fromPosition);
            return targetType == fromType && targetPosition > 1;
        }
        return true;
    }

    @Override
    public void onSwiped(@NonNull @NotNull RecyclerView.ViewHolder viewHolder, int direction) {
        if (listener != null) {
            listener.onItemSwiped(viewHolder.getAdapterPosition());
        }
    }

    @Override
    public boolean isLongPressDragEnabled() {
        return false;
    }

    private boolean listener2DragComplete = false;

    @Override
    public void onSelectedChanged(@Nullable RecyclerView.ViewHolder viewHolder, int actionState) {
        super.onSelectedChanged(viewHolder, actionState);
        if (actionState == ItemTouchHelper.ACTION_STATE_DRAG) {
            listener2DragComplete = true;
        } else if (actionState == ItemTouchHelper.ACTION_STATE_SWIPE) {
            listener2DragComplete = false;
        } else if (actionState == ItemTouchHelper.ACTION_STATE_IDLE) {
            if (listener2DragComplete) {
                if (listener != null) {
                    listener.onItemDragCompleted();
                }
            }
            listener2DragComplete = false;
        }
    }
}