package com.bingce.controlnetwork.ui.widget.recyclerview;

import android.content.Context;

import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;

public class SwipeDraggableRecyclerViewUtil {
    public static void setup(AbstractDraggableRecycleViewAdapter<?> adapter,
                             RecyclerView recyclerView,
                             FragmentManager fragmentManager,
                             LifecycleOwner lifecycleOwner,
                             IStationPointOperatorListener operatorListener,
                             Context context) {
        ItemTouchHelperCallbackImpl callback = new ItemTouchHelperCallbackImpl(adapter, new IDragSwipeActionListener() {
            @Override
            public void onItemSwiped(int adapterPosition) {
                TipsWith2OptionFragment.tips(fragmentManager, lifecycleOwner,
                        null, null,
                        context.getString(R.string.tips_r_u_sure_delete_item),
                        new TipsWith2OptionFragment.IOperatorCallback() {
                            @Override
                            public void onConfirm() {
                                if (adapter != null) {
                                    adapter.onItemRemoved(adapterPosition);
                                }
                                if (operatorListener != null) {
                                    operatorListener.onItemRemoved(adapterPosition);
                                }
                            }

                            @Override
                            public void onOption() {
                            }

                            @Override
                            public void onCancel() {
                                if (adapter != null) {
                                    adapter.notifyItemChanged(adapterPosition);
                                }
                            }

                            @Override
                            public void onIgnore() {
                                if (adapter != null) {
                                    adapter.notifyItemChanged(adapterPosition);
                                }
                            }
                        });
            }

            @Override
            public void onItemDragging(int from, int to) {
                adapter.onItemSwap(from, to);
            }

            @Override
            public void onItemDragCompleted() {
                if (operatorListener != null) {
                    operatorListener.onItemDragCompleted();
                }
            }
        });
        ItemTouchHelper helper = new ItemTouchHelper(callback);
        adapter.setViewHolderDragDelegate(helper::startDrag);
        helper.attachToRecyclerView(recyclerView);
    }

    public interface IStationPointOperatorListener {
        void onItemRemoved(int adapterPosition);

        void onItemDragCompleted();
    }
}
