package com.bingce.controlnetwork.util;

import android.content.Context;

import com.bingce.utils.FileUtil;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.FileUtils;
import blankj.utilcode.util.Utils;

public class CrashCollectUtils {
    public static File crashLogFolder(Context context) {
        File cacheFolder = context.getExternalCacheDir();
        if (!cacheFolder.exists() || cacheFolder.isFile()) {
            cacheFolder.mkdir();
        }
        File dir = new File(cacheFolder, "crash");
        if (!dir.exists() || dir.isFile()) {
            dir.mkdir();
        }
        return dir;
    }

    //收集崩溃日志，在处理完成后根据结果删除
    public static void collectCrashLogInWorkerThread(ICrashLogCollectListener listener) {
        File dir = crashLogFolder(Utils.getApp().getApplicationContext());
        //读取其下面的数据
        if (!dir.exists() || dir.isFile()) {
            return;
        }

        StringBuilder sb = new StringBuilder();

        File[] files = dir.listFiles();

        if (files != null) {
            List<File> needMoveFileList = new ArrayList<>();
            File lastFile = null;
            for (File file : files) {
                if (file == null || !file.isFile()) {
                    continue;
                }

                if (lastFile == null) {
                    lastFile = file;
                } else {
                    if (file.lastModified() > lastFile.lastModified()) {
                        lastFile = file;
                    }
                }

                needMoveFileList.add(file);
            }
            needMoveFileList.remove(lastFile);

            for (File file : needMoveFileList) {
                moveToBackedUpFolder(dir, file);
            }

            if (lastFile != null) {
                FileUtil.ReadFileResult readFileResult = FileUtil.readStringFrom(lastFile);
                if (readFileResult.success) {
                    sb.append(readFileResult.string).append("\n");
                }
            }
        }

        String CRASH_LOG_TABLE = "CloudCrashLog";
        final String contentKey = "crash_content";
        final String fileNameKey = "file_name";
        final String appLabelKey = "app";
        final String uuidKey = "UUID";
        listener.onCollected(CRASH_LOG_TABLE, appLabelKey, fileNameKey, contentKey, uuidKey, sb.toString());
    }

    private static void moveToBackedUpFolder(File dir, File targetLogFile) {
        File repeatedFolder = new File(dir, "BACKED_UP");
        if (!repeatedFolder.exists()) {
            repeatedFolder.mkdirs();
        }
        FileUtils.move(targetLogFile, new File(repeatedFolder, targetLogFile.getName()));
    }

    public interface ICrashLogCollectListener {
        void onCollected(String cloudTable, String appLabelKey, String fileNameKey, String contentKey, String uuidKey,
                         String content);
    }

    public static class CrashItem {
        public final File file;
        public final String content;

        CrashItem(File file, String content) {
            this.file = file;
            this.content = content;
        }
    }
}
