package com.bingce.controlnetwork.util;

import android.content.Context;

import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.util.Tools;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.TipsFragment;
import com.bingce.controlnetwork.fragment.TipsWith2OptionFragment;


public class DialogUtil {
    public static void showDataLoseHintDialog(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner, IConfirmBack callBack) {
        showConfirmDialog(fragmentManager, lifecycleOwner,
                Tools.getString(R.string.tips_station_data_will_lose),
                Tools.getString(R.string.exit), callBack);
    }

    public static void showConfirmDialog(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                         String hint,
                                         String confirmText,
                                         IConfirmBack callBack) {
        TipsWith2OptionFragment
                .tips(
                        fragmentManager,
                        lifecycleOwner,
                        confirmText,
                        Tools.getString(R.string.cancel),
                        hint,
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                if (callBack != null) {
                                    callBack.onConfirm();
                                }
                            }
                        }
                );
    }

    public static void showConfirmCancleDialog(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                               String hint,
                                               String confirmText,
                                               String cancelText,
                                               IConfirmCancelBack callBack) {
        TipsWith2OptionFragment
                .tips(
                        fragmentManager,
                        lifecycleOwner,
                        confirmText,
                        cancelText,
                        hint,
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                if (callBack != null) {
                                    callBack.onConfirm();
                                }
                            }

                            @Override
                            public void onCancel() {
                                if (callBack != null) {
                                    callBack.onCancel();
                                }
                            }
                        }
                );
    }

    public static void showAlertDialog(Context context, FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                       String hint) {
        TipsFragment.tipsOnly(fragmentManager, lifecycleOwner, context, hint);
    }

    public static void showConfirmCountDownDialog(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                                  String hint,
                                                  String confirmText,
                                                  IConfirmBack callBack) {
        TipsWith2OptionFragment
                .tipsCountDownTime(
                        fragmentManager,
                        lifecycleOwner,
                        confirmText,
                        Tools.getString(R.string.cancel),
                        null,
                        hint,
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                callBack.onConfirm();
                            }
                        }
                );
    }

    public static void showHighToNoHighConfirmDialog(FragmentManager fragmentManager, LifecycleOwner lifecycleOwner,
                                                     IConfirmBack callBack) {
        TipsWith2OptionFragment
                .tips(
                        fragmentManager,
                        lifecycleOwner,
                        Tools.getString(R.string.confirm),
                        Tools.getString(R.string.cancel),
                        "当前已有测量数据，切换后高精度气象数据将丢失",
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                callBack.onConfirm();
                            }
                        }
                );
    }


    public interface IConfirmBack {
        void onConfirm();
    }

    public interface IConfirmCancelBack {
        void onConfirm();

        void onCancel();
    }

}
