package com.bingce.controlnetwork.util;

import android.content.Context;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.leveling.LevelingDbFactory;
import com.bingce.controlapphelper.datasource.database.leveling.idetection.isetting.LevelIDetectionSetting;
import com.bingce.controlapphelper.datasource.database.leveling.station.LevelingStationRecord;
import com.bingce.controlapphelper.datasource.database.leveling.stationsetting.LevelingStationSettingRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.ISurveyorStationDataSource;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.ui.fragment.TipsWith2OptionFragment;
import com.bingce.controlapphelper.util.IUIRunner;
import com.bingce.controlnetwork.func.level.activity.LevelStationSettingActivity;
import com.bingce.controlnetwork.newui.stationsetting.StationSettingsActivity;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

import blankj.utilcode.util.ToastUtils;
import com.bingce.controlnetwork.func.level.idetection.LevelIDetectionStationSettingActivity;


public class StationSettingCheckUtil {
    //请设置温度、成像、太阳方向等相关信息（App只对以上信息进行留存，不会依此对数据进行校正）.
    private static final String HINT = "开始测量前请先进行测站设置";
    private static final String HINT_LEVEL = "开始测量前请先进行测段设置";
    public static final String HINT_LEVEL_END = "测段结束前请先进行测段设置";

    private boolean isCheckSuccess;

    /**
     * 检测测站是否设置了棱镜相关属性，如果没有则跳转到设置界面
     */
    public void checkStationSettings(String stationId,
                                     @NonNull Fragment fragment,
                                     @NonNull IUIRunner runner, ICheckSuccess checkSuccess) {
        if (isCheckSuccess) {
            checkSuccess.onSuccess();
            return;
        }

        ThreadPoolUtil.execute(() -> {
            ISurveyorStationDataSource stationDataSource =
                    SurveyorDatabaseFactory.instance.getSurveyorStation();
            //检测温度、湿度、气压、棱镜是否设置
            SurveyorStationRecord stationRecord = stationDataSource.getRecordSync(stationId);
            if (stationRecord == null) {
                ToastUtils.showShort("stationRecord == null");
                return;
            }
            List<SurveyorPoint> stationItemList = stationRecord.getItems();
            if (stationItemList == null || stationItemList.isEmpty()) {
                ToastUtils.showShort("stationItemList == null || stationItemList.isEmpty()");
                return;
            }
            boolean failure = false;
            for (SurveyorPoint stationItem : stationItemList) {
                if (SurveyorPoint.TYPE_STATION.equals(stationItem.getType())) {
                    continue;
                }
                if (stationItem.isWellSteel()) {
                    continue;
                }
                if (isEmpty(stationItem.getPrismHeight())) {
                    failure = true;
                    break;
                }
            }
            if (!failure && (isEmpty(stationRecord.getTemperature()) ||
                    isEmpty(stationRecord.getHumidity()) ||
                    isEmpty(stationRecord.getAirPressure()) ||
                    isEmpty(stationRecord.getInstrumentHeight()))) {
                failure = true;
            }

            if (failure) {
                runner.runOnUI(() -> TipsWith2OptionFragment.hardTips(
                        fragment.getChildFragmentManager(),
                        fragment.getViewLifecycleOwner(),
                        HINT,
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
//                            StationSettingsFragment.start(fragment.getContext(), stationId);
                                StationSettingsActivity.start(fragment.requireContext(), stationId);
                            }

                            @Override
                            public void onCancel() {
//                                if (fragment.getActivity() != null) {
//                                    fragment.getActivity().finish();
//                                }
                            }
                        }));
                return;
            }

            isCheckSuccess = true;
            checkSuccess.onSuccess();

        });


    }

    public static boolean isEmpty(String string) {
        return string == null || string.isEmpty();
    }

    public interface ICheckSuccess {
        void onSuccess();
    }

    /**
     * 水准测站检测
     */
    public void checkStationSettingsLevel(String stationId,
                                          FragmentManager fragmentManager,
                                          LifecycleOwner lifecycleOwner,
                                          Context context,
                                          IUIRunner runner, ICheckSuccess checkSuccessListener) {
        if (isCheckSuccess) {
            checkSuccessListener.onSuccess();
            return;
        }
        ThreadPoolUtil.execute(() -> {
            LevelingStationRecord stationRecord = LevelingDbFactory.instance.getStationDataSource().getRecordSync(stationId);
            if (stationRecord == null) {
                ToastUtils.showShort("stationRecord == null");
                return;
            }
            LevelingStationSettingRecord stationSettingLatest = LevelingDbFactory.instance.getStationSettingDataSource().getLatest(stationRecord.routeId);

            if (stationSettingLatest == null || stationSettingLatest.weather == null) {
                //需要设置环境设置
                runner.runOnUI(() -> showLevelSetStationSetting(fragmentManager,
                        lifecycleOwner, context,
                        HINT_LEVEL,
                        stationRecord.routeId, stationId, false)
                );
                return;
            }

            isCheckSuccess = true;
            checkSuccessListener.onSuccess();
        });

    }

    /**
     * i角检测的测站设置
     */
    public void checkStationSettingsIDetection(String iDetectionId,
                                               @NonNull AppCompatActivity activity,
                                               @NonNull IUIRunner runner, ICheckSuccess checkSuccessListener) {
        if (isCheckSuccess) {
            checkSuccessListener.onSuccess();
            return;
        }
        ThreadPoolUtil.execute(() -> {
            LevelIDetectionSetting setting = LevelingDbFactory.instance.getIDetectionSettingDataSource().getByIDetectionId(iDetectionId);
            if (setting == null) {
                //需要设置环境设置
                runner.runOnUI(() -> TipsWith2OptionFragment.hardTips(
                        activity.getSupportFragmentManager(),
                        activity,
                        HINT,
                        new TipsWith2OptionFragment.SimpleCallback() {
                            @Override
                            public void onConfirm() {
                                LevelIDetectionStationSettingActivity.launch(activity, iDetectionId);
                            }
                        })
                );
                return;
            }
            isCheckSuccess = true;
            checkSuccessListener.onSuccess();
        });

    }

    /**
     * 提示需要进行测站设置
     */
    public static void showLevelSetStationSetting(@NonNull FragmentManager fragmentManager,
                                                  @NonNull LifecycleOwner lifecycleOwner,
                                                  Context context,
                                                  String hint,
                                                  String routeId,
                                                  String stationId,
                                                  boolean isSaveEndStation) {
        TipsWith2OptionFragment.hardTips(
                fragmentManager,
                lifecycleOwner,
                hint,
                new TipsWith2OptionFragment.SimpleCallback() {
                    @Override
                    public void onConfirm() {
                        LevelStationSettingActivity.start(context, routeId, stationId, isSaveEndStation);
                    }
                });
    }

}
