package com.bingce.controlnetwork.util;

import android.content.Context;

import androidx.annotation.MainThread;

import com.bingce.controlapphelper.datasource.database.SurveyorDatabaseFactory;
import com.bingce.controlapphelper.datasource.database.line.LineRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.SurveyorStationRecord;
import com.bingce.controlapphelper.datasource.database.surveyorstation.model.SurveyorPoint;
import com.bingce.controlapphelper.model.StationDetailItem;
import com.bingce.utils.IOnSingleGetCallback;
import com.bingce.utils.ThreadPoolUtil;

import java.util.List;

import com.bingce.controlnetwork.fragment.station.line.attached.LineStationAttachedSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.close.LineStationCloseSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.random.LineStationRandomSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.single.LineStationSingleSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellDoubleGroundSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellDoubleUnderGroundSetupFragment;
import com.bingce.controlnetwork.fragment.station.line.well.LineStationWellSetupFragment;

public class SurveyorStationUtil {
    @MainThread
    public static void editLine(Context context, String lineId) {
        ThreadPoolUtil.execute(() -> {
            LineRecord stationBatchRecord =
                    SurveyorDatabaseFactory.instance.getLineRecordDatabase().findById(lineId);
            if (stationBatchRecord == null) {
                return;
            }
//            //1.查询该线路下所有测站
//            List<SurveyorStationRecord> stationRecordList =
//                    SurveyorDatabaseFactory.instance.getSurveyorStation().getDataListByLine(stationBatchRecord.id);
//            //2.遍历测站，检测是否有测量数据
//            if (stationRecordList != null) {
//                for (SurveyorStationRecord record : stationRecordList) {
//                    if (record == null) {
//                        continue;
//                    }
//                    long count = SurveyorDatabaseFactory
//                            .instance
//                            .getSurveyorRecordDataSource()
//                            .recordCount(record.id);
//                    if (count > 0) {//如果有测量数据，则该导线不能编辑
//                        ThreadPoolUtil.executeInMain(() -> TipsFragment.tipsOnly(fragmentManager,
//                                lifecycleOwner,
//                                context,
//                                context.getString(R.string.tips_forbid_edit_line_because_has_surveyor_record)));
//                        return;
//                    }
//                }
//            }
            if (LineRecord.TYPE_LINE_SINGLE.equals(stationBatchRecord.type)) {
                LineStationSingleSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_LINE_ATTACH.equals(stationBatchRecord.type)) {
                LineStationAttachedSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_LINE_CLOSE.equals(stationBatchRecord.type)) {
                LineStationCloseSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_LINE_RANDOM.equals(stationBatchRecord.type)) {
                LineStationRandomSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_WELL.equals(stationBatchRecord.type)) {
                LineStationWellSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_WELL_DOUBLE_GROUND.equals(stationBatchRecord.type)) {
                LineStationWellDoubleGroundSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            } else if (LineRecord.TYPE_WELL_DOUBLE_UNDERGROUND.equals(stationBatchRecord.type)) {
                LineStationWellDoubleUnderGroundSetupFragment.startEdit(context, stationBatchRecord.projectId, lineId);
            }
        });
    }

    @MainThread
    public static void getBatchItemList(String lineId, IOnSingleGetCallback<LineRecord> callback) {
        ThreadPoolUtil.execute(() -> {
            LineRecord batchRecord =
                    SurveyorDatabaseFactory.instance.getLineRecordDatabase()
                            .findById(lineId);
            if (batchRecord == null) {
                return;
            }
            callback.onGet(batchRecord);
        });
    }

    @MainThread
    public static void updateLineRecord(LineRecord lineRecord) {
        if (lineRecord == null) return;
        SurveyorDatabaseFactory.instance.getLineRecordDatabase().update(lineRecord);
    }

    @MainThread
    public static void updateStationPointsMeasureHorAag(LineRecord lineRecord) {
        if (lineRecord == null) return;
        List<StationDetailItem> stationDetailList = lineRecord.items;
        if (stationDetailList == null) {
            return;
        }

        ThreadPoolUtil.execute(() -> {
            List<SurveyorStationRecord> stationRecordList = SurveyorDatabaseFactory.instance.getSurveyorStation().getDataListByLine(lineRecord.id);
            if (stationRecordList != null && !stationRecordList.isEmpty()) {
                for (SurveyorStationRecord record : stationRecordList) {
                    List<SurveyorPoint> items = record.items;
                    if (items == null) {
                        continue;
                    }
                    for (SurveyorPoint point : items) {
                        for (StationDetailItem stationDetailItem : stationDetailList) {
                            if (point != null && point.getOriginalPointId().equals(stationDetailItem.getOriginalPointId())
                                    && point.isWellSteel() != point.isWellSteel()) {
                                // TODO: 2023/5/22
                                point.setWellSteel(stationDetailItem.isWellSteel());
                                SurveyorDatabaseFactory.instance.getSurveyorStation().update(record);
                            }
                        }
                    }
                }
            }
        });


    }

}
