package com.bingce.controlnetwork.util.config;

import android.content.Context;
import android.content.Intent;

import com.bingce.AppChannel;
import com.bingce.controlnetwork.R;
import com.bingce.controlnetwork.fragment.ConfigFragment;
import com.bingce.controlnetwork.fragment.PointListFragment;
import com.bingce.controlnetwork.fragment.PrismManageFragment;
import com.bingce.controlnetwork.fragment.SelectExportHeightTraverseFragment;
import com.bingce.controlnetwork.fragment.SelectExportSurveyorRecordFragment;
import com.bingce.controlnetwork.fragment.SelectExportSurveyorRecordWireFileFragment;
import com.bingce.controlnetwork.fragment.ToleranceManageFragment;
import com.bingce.device.ui.ConnectTSActivity;
import com.wedcel.dragexpandgrid.model.DragIconInfo;

import java.util.Map;

public class ConfigsSetConfigs extends AbstractSetConfigs {

    private final static int FUC_INSTRUMENTMANAGEMENT = 101; //仪器管理
    private final static int FUC_GLOBAL_CONTROL_POINT = 102;//全局控制点
    private final static int FUC_EXPORT_STATION = 103; //观测数据导出
    private final static int FUC_EXPORT_WIRE_FILE = 104;//导线文件导出
    private final static int FUC_TOLERANCE_MANAGER = 105;//自定义限差管理
    private final static int FUC_PRISM_MANAGER = 106;//棱镜管理
    private final static int FUC_HEIGHT_TRAVERSE_EXPORT = 107;//高程导线数据导出

    @Override
    protected void init(Map<Integer, ConfigItem> list) {
        list.put(FUC_INSTRUMENTMANAGEMENT,
                new ConfigItem(FUC_INSTRUMENTMANAGEMENT, R.string.instrument_management, R.drawable.icon_ts_connecting, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_GLOBAL_CONTROL_POINT,
                new ConfigItem(FUC_GLOBAL_CONTROL_POINT, R.string.global_control_point, R.drawable.icon_parameter_points, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_EXPORT_STATION,
                new ConfigItem(FUC_EXPORT_STATION, R.string.export, R.drawable.ic_export, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_EXPORT_WIRE_FILE,
                new ConfigItem(FUC_EXPORT_WIRE_FILE, R.string.export_wire_file, R.drawable.ic_export_wire, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_TOLERANCE_MANAGER,
                new ConfigItem(FUC_TOLERANCE_MANAGER, R.string.name_custom_telorance_manager, R.drawable.ic_custom_tolerance, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_PRISM_MANAGER,
                new ConfigItem(FUC_PRISM_MANAGER, R.string.prism_manager, R.drawable.icon_prism_manager, DragIconInfo.CATEGORY_ONLY));
        list.put(FUC_HEIGHT_TRAVERSE_EXPORT,
                new ConfigItem(FUC_HEIGHT_TRAVERSE_EXPORT, R.string.export_height_traverse, R.drawable.ic_export_height_traverse, DragIconInfo.CATEGORY_ONLY));
    }

    private static ConfigsSetConfigs instance = null;

    public static ConfigsSetConfigs getInstance() {
        if (instance == null) {
            synchronized (ConfigsSetConfigs.class) {
                if (instance == null) {
                    instance = new ConfigsSetConfigs();
                    instance.init();
                }
            }
        }
        return instance;
    }

    public void onItemClick(int id, ConfigFragment fragment) {
        Context context = fragment.requireContext();
        switch (id) {
            case FUC_INSTRUMENTMANAGEMENT://仪器管理
                Intent intent = new Intent(context, ConnectTSActivity.class);
                intent.putExtra("customChannel", AppChannel.customChannel);
                context.startActivity(intent);
                break;
            case FUC_GLOBAL_CONTROL_POINT://全局控制点
                PointListFragment.go2GlobalPointList(context);
                break;
            case FUC_EXPORT_STATION://观测数据导出
                SelectExportSurveyorRecordFragment.start(context, null);
                break;
            case FUC_EXPORT_WIRE_FILE://导线文件导出
                SelectExportSurveyorRecordWireFileFragment.start(context);
                break;
            case FUC_TOLERANCE_MANAGER://自定义限差管理
                ToleranceManageFragment.start(context, null);
                break;
            case FUC_PRISM_MANAGER://棱镜管理
                PrismManageFragment.start(context);
                break;
            case FUC_HEIGHT_TRAVERSE_EXPORT://高程导线数据导出
                SelectExportHeightTraverseFragment.start(context);
                break;
        }
    }
}