package com.bingce.controlnetwork.wxapi;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.bingce.controlnetwork.R;
import com.bingce.event.WxRespCodeEvent;
import com.tencent.mm.opensdk.constants.ConstantsAPI;
import com.tencent.mm.opensdk.modelbase.BaseReq;
import com.tencent.mm.opensdk.modelbase.BaseResp;
import com.tencent.mm.opensdk.modelmsg.SendAuth;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.IWXAPIEventHandler;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

import org.greenrobot.eventbus.EventBus;

import blankj.utilcode.util.ToastUtils;
import cn.leancloud.LCObject;
import cn.leancloud.LCUser;

public class WXEntryActivity extends Activity implements IWXAPIEventHandler {
    // IWXAPI 是第三方app和微信通信的openapi接口
    private IWXAPI api;
    public static final String APPID = "wx8b41ad369eba83a9";
    public static final String SECRET = "de5faa683f384095d79c6a75305c2bf0";

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 通过WXAPIFactory工厂，获取IWXAPI的实例
        api = WXAPIFactory.createWXAPI(this, APPID, false);
        api.handleIntent(getIntent(), this);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        setIntent(intent);
        api.handleIntent(intent, this);
    }

    // 微信发送请求到第三方应用时，会回调到该方法
    @Override
    public void onReq(BaseReq req) {
    }

    @Override
    public void onResp(BaseResp resp) {
        if (resp.getType() == ConstantsAPI.COMMAND_SENDAUTH) {
            switch (resp.errCode) {
                case BaseResp.ErrCode.ERR_OK:
                    String code = ((SendAuth.Resp) resp).code;
                    EventBus.getDefault().post(new WxRespCodeEvent(code));
                    break;
                case BaseResp.ErrCode.ERR_USER_CANCEL:
                    ToastUtils.showShort("User cancel");
                    break;
                case BaseResp.ErrCode.ERR_AUTH_DENIED:
                    ToastUtils.showShort("Auth denied");
                    break;
                case BaseResp.ErrCode.ERR_COMM:
                    ToastUtils.showShort("Error Comm");
                    break;
                case BaseResp.ErrCode.ERR_SENT_FAILED:
                    ToastUtils.showShort("Send failed");
                    break;
                case BaseResp.ErrCode.ERR_UNSUPPORT:
                    ToastUtils.showShort("Error unsupported");
                    break;
                case BaseResp.ErrCode.ERR_BAN:
                    ToastUtils.showShort("Error ban");
                    break;
            }
        } else if (resp.getType() == ConstantsAPI.COMMAND_SENDMESSAGE_TO_WX) {
            switch (resp.errCode) {
                case BaseResp.ErrCode.ERR_OK:
                    ToastUtils.showShort(R.string.share_success);
                    if (LCUser.getCurrentUser() != null) {
                        LCObject shareLog = new LCObject("ShareLog");
                        shareLog.put("user", LCUser.getCurrentUser());
                        shareLog.put("shareType", "wechat");
                        shareLog.saveInBackground().subscribe();
                    }
                    break;
                case BaseResp.ErrCode.ERR_USER_CANCEL:
                    ToastUtils.showShort(R.string.share_cancel);
                    break;
                case BaseResp.ErrCode.ERR_AUTH_DENIED:
                    ToastUtils.showShort(R.string.share_denied);
                    break;
                case BaseResp.ErrCode.ERR_COMM:
                    ToastUtils.showShort("Error Comm");
                    break;
                case BaseResp.ErrCode.ERR_SENT_FAILED:
                    ToastUtils.showShort("Send failed");
                    break;
                case BaseResp.ErrCode.ERR_UNSUPPORT:
                    ToastUtils.showShort("Error unsupported");
                    break;
                case BaseResp.ErrCode.ERR_BAN:
                    ToastUtils.showShort("Error ban");
                    break;
            }
        }
        finish();
    }
}