package com.bingce

import org.gradle.api.Project
import org.gradle.api.tasks.Copy
import java.io.File

class AarCopyUtilsKt {
    companion object {
        private fun copyToXXX(
            taskName: String,
            project: Project,
            destination: String,
            aarOrJava: Boolean
        ): Copy {
            val projectName = project.name
            return project.tasks.register(taskName, Copy::class.java) {

                outputs.upToDateWhen { false }

                from(if (aarOrJava) "build/outputs/aar/${projectName}-release.aar" else "build/libs/${projectName}.jar") // 源目录，这里假设生成物在 build/libs 目录下
                into(destination) // 目标目录
                enabled = destination.isNotEmpty() && destination.isNotBlank() // 取消任务
            }.get()
        }

        fun configCopyTask(project: Project, aarOrJava: Boolean) {
            val surveyorCopyTaskName = "copy2Surveyor"
            val alphaCopyTaskName = "copy2Alpha"
            val tjCopyTaskName = "copy2Tj"

            val projectName = project.name

            val copy2Surveyor =
                copyToXXX(
                    surveyorCopyTaskName,
                    project,
                    "../../aar/${project.name}/",
                    aarOrJava
                )

            val bcFolder = BcFolder(project)

            val alphaDestination = bcFolder.folderConfig("alpha.outputDir")?.let {
                File(it, "bingceAAR\\${projectName}").absolutePath
            } ?: ""
            println("alphaDestination: $alphaDestination")
            val copy2Alpha = copyToXXX(alphaCopyTaskName, project, alphaDestination, aarOrJava)

            val tjDestination = bcFolder.folderConfig("tj.outputDir")?.let {
                File(it, "aar\\${projectName}").absolutePath
            } ?: ""
            println("tjDestination: $tjDestination")
            val copy2Tj = copyToXXX(tjCopyTaskName, project, tjDestination, aarOrJava)

            // 设置任务依赖，确保在 assemble 任务执行完后执行拷贝任务
            project.tasks.named("assemble").configure {
                copy2Surveyor.enabled = true
                finalizedBy(copy2Surveyor)
            }

            val nextTaskName1 = if (alphaDestination.isEmpty() || alphaDestination.isBlank()) {
                surveyorCopyTaskName
            } else {
                project.tasks.named(surveyorCopyTaskName).configure {
                    copy2Alpha.enabled = true
                    finalizedBy(copy2Alpha)
                }
                alphaCopyTaskName
            }

            if (tjDestination.isNotEmpty() && tjDestination.isNotBlank()) {
                project.tasks.named(nextTaskName1).configure {
                    copy2Tj.enabled = true
                    finalizedBy(copy2Tj)
                }
            }
        }
    }
}