/*
 * Copyright 2020 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.bingce

import org.gradle.api.JavaVersion

object AndroidSdk {
    const val minSdkVersion = 26 //POI库要求最低26
    const val targetSdkVersion = 31
    const val compileSdkVersion = 34
    const val buildToolsVersion = "33.0.0"

    private const val junitVersion = "4.13.2"
    const val junit = "junit:junit:$junitVersion"
    const val runner = "androidx.test:runner:1.4.0"
    const val core = "androidx.test:core:1.4.0"
    const val testJunit = "androidx.test.ext:junit:1.1.5"
    const val testEspresso = "androidx.test.espresso:espresso-core:3.1.0"
    const val androidJUnitRunner = "androidx.test.runner.AndroidJUnitRunner"

    val javaVersion: JavaVersion = JavaVersion.VERSION_1_8
}

object Kotlin {
    private const val kotlinVersion = "1.9.10"
    const val sdk = "org.jetbrains.kotlin:kotlin-stdlib-jdk8:$kotlinVersion"
    const val corountine = "org.jetbrains.kotlinx:kotlinx-coroutines-android:1.3.9"
    const val plugin = "org.jetbrains.kotlin:kotlin-gradle-plugin:$kotlinVersion"
}

object AndroidX {
    const val appcompat = "androidx.appcompat:appcompat:1.6.1"
    const val localBroadcastManager = "androidx.localbroadcastmanager:localbroadcastmanager:1.0.0"
    const val swipeRefreshLayout = "androidx.swiperefreshlayout:swiperefreshlayout:1.0.0"

    private const val legacyVersion = "1.0.0"
    const val legacySupport = "androidx.legacy:legacy-support-v4:$legacyVersion"
    const val legacyPreference = "androidx.legacy:legacy-preference-v14:$legacyVersion"

    const val preference = "androidx.preference:preference:1.1.1"
    const val multidex = "androidx.multidex:multidex:2.0.1"
    const val cardView = "androidx.cardview:cardview:1.0.0"
    const val recyclerview = "androidx.recyclerview:recyclerview:1.2.1"
    const val annotationz = "androidx.annotation:annotation:1.1.0"
    const val documentFile = "androidx.documentfile:documentfile:1.0.1"
    const val constraintLayout = "androidx.constraintlayout:constraintlayout:2.1.4"
    const val material = "com.google.android.material:material:1.8.0"
    const val materialDialogs = "com.afollestad.material-dialogs:core:0.9.6.0"
    const val vectorDrawable = "androidx.vectordrawable:vectordrawable:1.1.0"

    object Core {
        const val coreKtx = "androidx.core:core-ktx:1.9.0"
    }

    object Paging {
        private const val version = "3.2.1"
        const val runtime = "androidx.paging:paging-runtime:$version"
        const val common = "androidx.paging:paging-common:$version"
    }

    object Lifecycle {
        private const val lifecycleVersion = "2.2.0"
        const val extensions = "androidx.lifecycle:lifecycle-extensions:$lifecycleVersion"
        const val viewModel = "androidx.lifecycle:lifecycle-viewmodel:$lifecycleVersion"
        const val runtimeKt = "androidx.lifecycle:lifecycle-runtime-ktx:2.4.0";
        const val viewModelKt = "androidx.lifecycle:lifecycle-viewmodel-ktx:2.4.0";
    }

    object Room {
        private const val version = "2.6.1"
        const val runtime = "androidx.room:room-runtime:$version"
        const val ktx = "androidx.room:room-ktx:$version"
        const val compiler = "androidx.room:room-compiler:$version"
        const val paging = "androidx.room:room-paging:$version"
    }

    object Ktx {
        private const val version = "1.8.0"
        const val activity = "androidx.activity:activity-ktx:$version"
        const val fragment = "androidx.fragment:fragment-ktx:1.2.5"
    }
}

object Libs {
    const val eventBus = "org.greenrobot:eventbus:3.1.1"

    object ButterKnife {
        private const val version = "10.2.3"
        const val core = "com.jakewharton:butterknife:$version"
        const val compiler = "com.jakewharton:butterknife-compiler:$version"
        const val plugin = "com.jakewharton:butterknife-gradle-plugin:$version"
    }

    const val jPinYin = "com.github.stuxuhai:jpinyin:1.0"

    const val switchIcon = "com.github.zagum:Android-SwitchIcon:1.4.0"

    const val zxing = "com.google.zxing:core:3.2.1"

    object Rx {
        const val rxJava = "io.reactivex.rxjava2:rxjava:2.2.8"
        const val rxAndroid = "io.reactivex.rxjava2:rxandroid:2.1.1"
    }

    object POI {
        const val slf4j_api = "org.slf4j:slf4j-api:1.7.30"
        const val jcl_over_slf4j = "org.slf4j:jcl-over-slf4j:1.7.30"
        const val poi_ooxml = "org.apache.poi:poi-ooxml:4.1.2"//4.0.1版本会造成无法识别Poi的类例如Cell，高于4.1.2又会导致读写xls异常，提示provider未找到，所以4.1.2是最好的选着，不好的一点是，4.1.2要求在minsdk>26(Android8)以上才能编译
        const val aalto_xml = "com.fasterxml:aalto-xml:1.3.0"
        const val stax_api = "stax:stax-api:1.0.1"
        const val commons_jexl3 = "org.apache.commons:commons-jexl3:3.2"
        const val commons_beanutils = "commons-beanutils:commons-beanutils:1.9.4"
        const val logback_core = "ch.qos.logback:logback-core:1.2.10"
        const val commons_compress = "org.apache.commons:commons-compress:1.21"
        const val commons_lang3 = "org.apache.commons:commons-lang3:3.7"
    }

    object UMeng {
        const val common = "com.umeng.umsdk:common:9.6.6"
        const val asms = "com.umeng.umsdk:asms:1.8.0"
        const val apm = "com.umeng.umsdk:apm:1.9.4"
    }

    object RxHttp {
        const val core = "com.ljx.rxhttp:rxhttp:2.5.5"
        const val compiler = "com.ljx.rxhttp:rxhttp-compiler:2.5.5"
    }

    object Retrofit2 {
        const val core = "com.squareup.retrofit2:retrofit:2.9.0"
        const val converterFastjson = "org.ligboy.retrofit2:converter-fastjson-android:2.1.0"
    }

    const val scanplus = "com.huawei.hms:scanplus:2.12.0.301"

    object Glide {
        private const val version = "4.9.0"
        const val core = "com.github.bumptech.glide:glide:$version"
        const val compiler = "com.github.bumptech.glide:compiler:$version"
    }

    object RxImagePicker {
        private const val version = "2.5.6"
        const val core = "com.github.qingmei2:rximagepicker:$version"
        const val support = "com.github.qingmei2:rximagepicker_support:$version"
        const val supportWechat = "com.github.qingmei2:rximagepicker_support_wechat:$version"
    }

    object OkHttp {
        private const val version = "4.9.0"
        const val core = "com.squareup.okhttp3:okhttp:$version"
        const val loggingInterceptor = "com.squareup.okhttp3:logging-interceptor:$version"
    }

//    const val fastJson = "com.alibaba:fastjson:1.1.71.android"

    object Iconics {
        private const val version = "3.2.5"
        const val core = "com.mikepenz:iconics-core:$version"
        const val views = "com.mikepenz:iconics-views:$version"
        const val materialIconicTypeface =
            "com.mikepenz:material-design-iconic-typeface:2.2.0.4@aar"
        const val typeiconsTypeface = "com.mikepenz:typeicons-typeface:2.0.7.4@aar"
    }

    object Filament {
        private const val version = "1.49.1"
        const val android = "com.google.android.filament:filament-android:$version"
        const val gltfio = "com.google.android.filament:gltfio-android:$version"
        const val utils = "com.google.android.filament:filament-utils-android:$version"
    }

    const val compressor = "id.zelory:compressor:2.1.1"//图片压缩工具
    const val picasso = "com.squareup.picasso:picasso:2.71828"
    const val rubberStamp = "com.vinaygaba:rubberstamp:1.0.0"//图像处理
    const val photoView = "com.github.chrisbanes:PhotoView:2.1.3"//图片缩放库
    const val circleImage = "de.hdodenhof:circleimageview:3.0.1"//圆角图片

    const val realmPlugin = "io.realm:realm-gradle-plugin:5.14.0"
    const val objectBox = "io.objectbox:objectbox-gradle-plugin:3.5.1"

    const val materialEditText = "com.rengwuxian.materialedittext:library:2.1.4"
    const val swipeLayout = "com.daimajia.swipelayout:library:1.2.0@aar"
    const val recyclerviewAnimator = "jp.wasabeef:recyclerview-animators:2.2.7"
    const val fabButton = "com.github.clans:fab:1.6.4"//悬浮按钮
    const val datePicker = "cn.aigestudio.datepicker:DatePicker:2.2.0"//时间选择器
    const val changeLog = "com.github.gabrielemariotti.changeloglib:changelog:2.1.0"//变更日志库
    const val flycoTabLayout = "com.flyco.tablayout:FlycoTabLayout_Lib:2.1.2@aar"//标签库

    //    const val utilCodex = "com.blankj:utilcodex:1.31.0"//工具类工具\
    const val materialAbout = "com.github.jrvansuita:MaterialAbout:0.2.4"//界面风格工具
    const val banner = "com.youth.banner:banner:2.1.0"//轮播图
    const val webChatSdk = "com.tencent.mm.opensdk:wechat-sdk-android-without-mta:5.4.3"//腾讯opensdk
    const val appUpdate = "com.github.jenly1314.AppUpdater:app-updater:1.1.3"//app版本更新
    const val rootBeer = "com.scottyab:rootbeer-lib:0.0.7"//root检查
    const val tabBar = "devlight.io:navigationtabbar:1.2.5"//tabBar
    const val agentWeb = "com.just.agentweb:agentweb:4.1.2"//网页加载工具
    const val tencentSonic = "com.tencent.sonic:sdk:2.0.0"//腾讯网页加载工具
    const val downloader = "com.download.library:Downloader:4.1.2"//下载工具
    const val filedownloader = "com.liulishuo.filedownloader:library:1.7.7"
    const val keyboardVisibility =
        "net.yslibrary.keyboardvisibilityevent:keyboardvisibilityevent:2.3.0"//软件盘监听工具
    const val progressBar = "com.daimajia.numberprogressbar:library:1.4@aar"//进度条
//    const val fastBle = "clj.fastble:FastBleLib:2.3.4"//蓝牙ble

    object FastAdapter {
        private const val version = "3.3.1"
        const val core = "com.mikepenz:fastadapter:$version"
        const val itemanimators = "com.mikepenz:itemanimators:1.1.0"
    }

    const val pickerView = "com.contrarywind:Android-PickerView:4.1.9"
    const val unCrop = "com.github.yalantis:ucrop:2.2.7"
    const val imageTextButton = "cn.dxjia:imagetextbutton:1.0.0"
    const val threadPoster = "com.techyourchance:threadposter:1.0.1"
    const val fileDownloader = "com.liulishuo.filedownloader:library:1.7.7"
    const val gson = "com.google.code.gson:gson:2.8.9"

    object LeanCloud {
        private const val version = "8.2.16"
        const val storage = "cn.leancloud:storage-android:$version"
        const val realtime = "cn.leancloud:realtime-android:$version"
    }

    const val dragExpandGrid = "com.nineoldandroids:library:2.4.0"//动画效果库
    const val giftDrawable = "pl.droidsonroids.gif:android-gif-drawable:1.2.23"//加载gif图
    const val xxPermissions = "com.github.getActivity:XXPermissions:18.5"

    const val httpLegacy = "org.apache.http.legacy"

    //    const val usbSerial4Android = "com.github.mik3y:usb-serial-for-android:3.4.3"
    const val serialPort = "io.github.xmaihh:serialport:2.1.1"
    const val timber = "com.jakewharton.timber:timber:4.7.1"
    const val loadingView = "com.ldoublem.loadingview:loadingviewlib:1.0"

    const val gjygitEditext="com.github.gjygit:editext:3.0"

}